#ifndef __CSRAM_PARTITION_H__
#define __CSRAM_PARTITION_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CSRAM_BASE_ADDR                 0x00800000
#define CSRAM_MEM_LEN                   (1024*1024)

/* For each PF (pf=0,1,2,3) & VF (vf=0,1,2,3,4,5,6,7), Total: PF x 4 = (28KB x 8VF = 224KB) x 4 = 896KB. */
#define ADDR_CSRAM_SIZE_PER_PF          (0x38000)                                       /* 224KB */
#define PFn_CSRAM_BASE(p)               (CSRAM_BASE_ADDR + (p)*ADDR_CSRAM_SIZE_PER_PF)
#define ADDR_CSRAM_SIZE_PER_VF          (0x7000)                                        /* 28KB */
#define ADDR_SMU_SIZE_PER_VF            (0x1000)                                        /* 4KB  */
#define ADDR_GFX_SIZE_PER_VF            (0x10000)                                       /* 64KB */
#define ADDR_BAR_RSVD_SIZE_PER_VF       (0x8000)                                        /* 32KB */

#define VFn_CSRAM_BASE(p, v)            (PFn_CSRAM_BASE(p) + (v)*ADDR_CSRAM_SIZE_PER_VF)

#define CFG_INFO_PER_VF(p, v)           (VFn_CSRAM_BASE(p, v) + 0)       /* size 1KB */
#define AI_PIPELINE_INFO_PER_VF(p, v)   (VFn_CSRAM_BASE(p, v) + 0x400)   /* size 4KB */
#define SDMA_DESCIPTOR_PER_VF(p, v)     (VFn_CSRAM_BASE(p, v) + 0x1400)  /* size 8KB */
#define MSGQ_PER_VF(p, v)               (VFn_CSRAM_BASE(p, v) + 0x3400)  /* size 2KB */
#define MTO_PER_VF(p, v)                (VFn_CSRAM_BASE(p, v) + 0x3C00)  /* size 2.5KB */
#define MSIX_PER_VF(p, v)               (VFn_CSRAM_BASE(p, v) + 0x4600)  /* size 3.5KB */
#define RESERVED_3KB_PER_VF(p, v)       (VFn_CSRAM_BASE(p, v) + 0x5400)  /* size 3KB */

/* For cedar msg */
#define CEDAR_SVC_INFO                  (CSRAM_BASE_ADDR + 0x000E4000)
#define CEDAR_RING_BUFFER_INFO          (CSRAM_BASE_ADDR + 0x000E4000)
#define CEDAR_SVC_RNG_SYM_INFO          (CSRAM_BASE_ADDR + 0x000E4400)  /* 512BYTE */
#define CEDAR_SVC_TASK_DONE             (CSRAM_BASE_ADDR + 0x000E4400)  /* 512BYTE */
#define CEDAR_SVC_HASH_RES              (CSRAM_BASE_ADDR + 0x000E4600)
#define CEDAR_SVC_STS_RES               (CSRAM_BASE_ADDR + 0x000E47E0)
#define CEDAR_SVC_DONE_FLAG             (CSRAM_BASE_ADDR + 0x000E47E4)
#define CSRAM_CEDAR_SYM_RES             (CSRAM_BASE_ADDR + 0x000E47F0)
#define CSRAM_CEDAR_SYM_DONEFLAG        (CSRAM_BASE_ADDR + 0x000E47F4)
#define CSRAM_CEDAR_ASY_RES             (CSRAM_BASE_ADDR + 0x000E47F8)
#define CSRAM_CEDAR_ASY_DONEFLAG        (CSRAM_BASE_ADDR + 0x000E47Fc)

/* Total 1024 bytes for version */
#define VSERSION_INFO_BASE              (CSRAM_BASE_ADDR + 0x000E1000)     /* total 1024 bytes */
#define FW_VERSION                      (CSRAM_BASE_ADDR + 0x000E1000)     /* 64 bytes */
#define SMCU_VERSION                    (CSRAM_BASE_ADDR + 0x000E1040)     /* 64 bytes */
#define CMCU_VERSION                    (CSRAM_BASE_ADDR + 0x000E1080)     /* 64 bytes */
#define OMCU_VERSION                    (CSRAM_BASE_ADDR + 0x000E10C0)     /* 64 bytes */
#define GMCU_VERSION                    (CSRAM_BASE_ADDR + 0x000E1100)     /* 64 bytes */
#define VEMCU_VERSION                   (CSRAM_BASE_ADDR + 0x000E1140)     /* 64 bytes */
#define ODSP_VERSION                    (CSRAM_BASE_ADDR + 0x000E1180)     /* 64 bytes */
#define VDSP_VERSION                    (CSRAM_BASE_ADDR + 0x000E11C0)     /* 64 bytes */
#define PMCU_VERSION                    (CSRAM_BASE_ADDR + 0x000E1200)     /* 64 bytes */
#define VDMCU_VERSION                   (CSRAM_BASE_ADDR + 0x000E1240)     /* 64 bytes */

/* smi of tools */
#define SMI_INFO_ADDR                   (CSRAM_BASE_ADDR + 0x000E1600) /* 2KB + 512B */

/* profiler of tools; total 2KB */
#define PERFORMANCE_AI_BASE             (CSRAM_BASE_ADDR + 0x000E2000)      // 256 bytes
#define PERFORMANCE_AI_CMCU             (CSRAM_BASE_ADDR + 0x000E2000)      // 8 bytes
#define PERFORMANCE_AI_OMCU0            (CSRAM_BASE_ADDR + 0x000E2008)      // 8 bytes
#define PERFORMANCE_AI_OMCU1            (CSRAM_BASE_ADDR + 0x000E2010)      // 8 bytes
#define PERFORMANCE_AI_GMCU0            (CSRAM_BASE_ADDR + 0x000E2018)      // 8 bytes
#define PERFORMANCE_AI_GMCU1            (CSRAM_BASE_ADDR + 0x000E2020)      // 8 bytes
#define PERFORMANCE_AI_GMCU2            (CSRAM_BASE_ADDR + 0x000E2028)      // 8 bytes
#define PERFORMANCE_AI_GMCU3            (CSRAM_BASE_ADDR + 0x000E2030)      // 8 bytes
#define PERFORMANCE_AI_VEMCU0           (CSRAM_BASE_ADDR + 0x000E2038)      // 8 bytes
#define PERFORMANCE_AI_VEMCU1           (CSRAM_BASE_ADDR + 0x000E2040)      // 8 bytes
#define PERFORMANCE_AI_VEMCU2           (CSRAM_BASE_ADDR + 0x000E2048)      // 8 bytes
#define PERFORMANCE_AI_VEMCU3           (CSRAM_BASE_ADDR + 0x000E2050)      // 8 bytes
#define PERFORMANCE_AI_ODSP0            (CSRAM_BASE_ADDR + 0x000E2058)      // 8 bytes
#define PERFORMANCE_AI_ODSP1            (CSRAM_BASE_ADDR + 0x000E2060)      // 8 bytes
#define PERFORMANCE_AI_VDSP0            (CSRAM_BASE_ADDR + 0x000E2068)      // 12 bytes
#define PERFORMANCE_AI_VDSP1            (CSRAM_BASE_ADDR + 0x000E2074)      // 12 bytes
#define PERFORMANCE_AI_PMCU             (CSRAM_BASE_ADDR + 0x000E2080)      // 8 bytes
#define PERFORMANCE_AI_SMCU             (CSRAM_BASE_ADDR + 0x000E2088)      // 8 bytes
#define PCIE_BANDWIDTH_UPSTREAM         (CSRAM_BASE_ADDR + 0x000E2090)      // 8 bytes
#define PCIE_BANDWIDTH_DOWNSTREAM       (CSRAM_BASE_ADDR + 0x000E2098)      // 8 bytes
// fw mcu cycle count, for host compute time diff
#define PERFORMANCE_AI_CMCU_CYCLE       (CSRAM_BASE_ADDR + 0x000E20A0)      // 4 bytes
#define PERFORMANCE_AI_VDSP0_CYCLE      (CSRAM_BASE_ADDR + 0x000E20A4)      // 4 bytes
#define PERFORMANCE_AI_VDSP1_CYCLE      (CSRAM_BASE_ADDR + 0x000E20A8)      // 4 bytes
#define DDR_BANDWIDTH_UPSTREAM          (CSRAM_BASE_ADDR + 0x000E20AC)      // 8 bytes
#define DDR_BANDWIDTH_DOWNSTREAM        (CSRAM_BASE_ADDR + 0x000E20B4)      // 8 bytes

/* debugger of tools; total 4KB */
#define DEBUGGER_INFO_ADDR              (CSRAM_BASE_ADDR + 0x000E2800)      /* 4KB */

#define CSRAM_RESEREVE2                 (CSRAM_BASE_ADDR + 0x000F4000)      /* 12K */

/* msix table VF0 -- VF7, 32k bytes */
#define PCIE_MSIX_VF_TABLE              (CSRAM_BASE_ADDR + 0x000F7000)      /* 32K */

/* PF, msix table */
#define PCIE_MSIX_PF_PENDING_TABLE      (CSRAM_BASE_ADDR + 0x000FF000)      /* 2K */
#define PCIE_MSIX_PF_TABLE              (CSRAM_BASE_ADDR + 0x000FF800)      /* 2K */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __CSRAM_PARTITION_H__ */
