#ifndef __SG100_CMD_H__
#define __SG100_CMD_H__

// Len of req data per smcu read
#define MCU_REQ_UNIT_LEN_4B 		 4
#define MCU_REQ_UNIT_LEN_8B 		 8

/* host send cmd to smcu */
enum HOST_2_SMCU_CMD {
	SMCU_CHECK_STS = 0,
	SMCU_TEST_VEMCU_RESET = 1,
	SDMA_ERR_CMD = 0x1E,
	SMMU_EVENT_CMD = 0x1F,
	SMCU_READ_CMD = 0x20,
	SMCU_WRITE_CMD = 0x21,
	HOST_READ_CSR_BY_SMCU,
	HOST_WRITE_CSR_BY_SMCU,
	DUMP_GLOBAL_HW_CFG,
	DUMP_CURRENT_FN_HW_CFG,
	SMCU_DUMP_PC,
	SMCU_VSYNC_TIMER_EN,
	SMCU_VSYNC_TIMER_DIS,
	SMCU_CONFIG_ENTRY,
	SMCU_SYNC_FUNC_STATE,
	SMCU_MODIFY_SRIOV_CAP,
	DIAG_SLIDE_WINDOW,
	SMCU_TRIGER_MSI_IRQ,
	SMCU_SMI_CMD_REQ,
	SMCU_XSPI_FW_FLASH,
	SMCU_XSPI_FW_READ,
	SMCU_XSPI_FW_FLAG,
	SMCU_SWITCH_LOG_LEVEL,
	SMCU_MONITOR_PCIE_PERF,
	SMCU_CII_SET,
	SMCU_CMD_REINIT_RST_PKG,
	SMCU_CMD_P2P_RETRAIN,
	SMCU_MONITOR_DDR_PERF,
	SMCU_DMI_FLASH,
	SMCU_SMMU_STE_ENTRY,
	SMCU_BMCU_FW_FLASH = 0x5E,
	SMCU_BMCU_FW_SWITCH = 0x5F,
	SMCU_PCIE_SUB_CORE_RESET,
	SMCU_CEDAR_CMD_REQ,
	VASTAI_PCIE_SUB_CORE_RESET,
	PMCU_BOOT_MODE_REQ,
	SMCU_CHECK_FLR_CNT,
	SMCU_FLR_START,
	SMCU_MONITOR_SDMA_PERF,
	SMCU_CONFIG_VF_NUM_CHANGE,
	SMCU_RESET_GFX,
	HOST_NOTIFY_SMCU_STATUS,
	HOST_READ_SMCU_COMMON_INFO,         // Main type: read common text info or state from SMCU
	SMCU_MONITOR_VENC_PERF,
	SMCU_MONITOR_VDEC_PERF,
	SMCU_MONITOR_GFX_PERF,
	HOST_ACK_RESET_GFX,
	HOST_ACK_UNLOAD_META_FW,
	HOST_ACK_LOAD_META_FW,
	SRIOV_MODE_PF_FLR,
	HOST_ACK_PF_FLR,
	HOST_2_SMCU_CMD_MAX,
};


/* smcu send msix to host */
enum SMCU_2_HOST_CMD {
	SWITCH_ATU_DONE=0x50,
	SMCU_TASK_RET,
	SMCU_CFG_ERR,
	SMCU_VSYNC_TIMER_INT,
	SMCU_SMI_CMD_ACK,
	SMCU_XSPI_RW_ACK,
	/* pmcu --> smcu --> host S*/
	FREQ_REDUCE_BY_TEMP,
	FREQ_REDUCE_BY_VOLT,
	FREQ_RECOVER_NORMAL_BY_TEMP,
	FREQ_RECOVER_NORMAL_BY_VOLT,
	FREQ_POWER_DOEN_REQ,
	/* pmcu --> smcu --> host E*/
	SMCU_TO_HOST_LOG_READ,
	SMCU_TO_HOST_MPU_OUT_OF_BOUND,
	SMCU_BMCU_CMD_ACK,
	VASTAI_PCIE_MSG1_VIDEO_EXCEPTION,
	VASTAI_PCIE_MSG1_EXCEPTION,
	VASTAI_PCIE_MSG1_CORE_EXCEPTION,
	VASTAI_PCIE_MSG1_CORE_RESET_ACK,
	VASTAI_PCIE_NOTIFY_CORE_EXCEPTION,
	VASTAI_PCIE_NOTIFY_CORE_RESET_ACK,
	VASTAI_PCIE_SYNC_FUNC_STATE,
	SMCU_RET_FLR_DONE,
	PMCU_BOOT_MODE_ACK,
	SMCU_CMD_REINIT_RST_PKG_ACK,
	SMCU_WILL_RESET_GFX,
	SMCU_RESET_GFX_DONE,
	SMCU_NOTIFY_PF_UNLOAD_META_FW,
	SMCU_NOTIFY_PF_LOAD_META_FW,
	SMCU_NOTIFY_HOST_PF_WILL_FLR,
	SMCU_NOTIFY_HOST_PF_FLR_DONE,
	SMCU_NOTIFY_HOST_PF_FLR,
	SMCU_2_HOST_MAX,
};

/*Common info sub cmd*/
enum HOST_2_MCU_SUB_CMD {
	HOST_READ_SMCU_INVENTORY_CFG = 0x100,       // sub type: Inventory info code stored in EEPROM
	HOST_2_MCU_CMD_MAX,
};

/*mpu sub cmd*/
enum SMCU_2_HOST_MPU_SUB_CMD {
	SMMU_MPU_OUT_OF_BOUND = 0,
	GFX_MPU_OUT_OF_BOUND,
	ENCODER_MPU_OUT_OF_BOUND,
	DECODER_MPU_OUT_OF_BOUND,
	SMCU_2_HOST_MPU_SUB_CMD_MAX,
};

enum MPU_MODE {
	MPU_READ = 0,
	MPU_WRITE,
	MPU_MODE_MAX,
};



/* host send cmd to gmcu */
enum HOST_2_GMCU_CMD {
	GMCU_READ_CMD = 0x20,
	GMCU_WRITE_CMD = 0x21,
	GMCU_DUMP_PC = 0x26,
	GMCU_OPEN_DDR_BW_TEST = 0x27,
	GMCU_CLOSE_DDR_BW_TEST = 0x28,
	GMCU_OPEN_PCIE_BW_TEST = 0x29,
	GMCU_CLOSE_PCIE_BW_TEST	= 0x2A,
	GMCU_SWITCH_LOG_LEVEL = 0x36,
	GMCU_STOP_SDMA_CHANNEL = 0x37,
	GMCU_FLR_START,
	HOST_2_GMCU_CMD_MAX,
};



enum enum_core_2_core_reset_subcmd {
	CORE_2_CORE_RESET_AI_SYS_HANG_SUBCMD = 0,
	CORE_2_CORE_RESET_AI_ACK_SUBCMD,
	CORE_2_CORE_RESET_AI_RESET_SUBCMD,
	CORE_2_CORE_RESET_AI_FINISH_SUBCMD,
	CORE_2_CORE_RESET_CORE_SYS_HANG_SUBCMD,
	CORE_2_CORE_RESET_VDSP_HANG_SUBCMD,
	CORE_2_CORE_RESET_VDSP_RESET_SUBCMD,
	CORE_2_CORE_RESET_VDSP_ACK_SUBCMD,
	CORE_2_CORE_RESET_VDSP_FINISH_SUBCMD,
	CORE_2_CORE_RESET_AI_FINISH_ACK_SUBCMD,
	CORE_2_CORE_RESET_VDSP_FINISH_ACK_SUBCMD,
	CORE_2_CORE_UTILELATION_SUBCMD_ModifyWindow,
	CORE_2_CORE_UTILELATION_SUBCMD_ModifyWindowAck,
	CORE_2_CORE_CEDAR_SUBCMD_switch,
	CORE_2_CORE_CEDAR_SUBCMD_switchAck,
};

enum enum_core_2_core_exception_subcmd {
	CORE_2_CORE_EXCEPTION_SUBCMD = 1,
};

enum enum_core_2_core_cmd {
	VATOOLS_INTERRUPT_TOOLS_CATEGARY_CMD = 1,
	CORE_2_CORE_RESET_CMD = 2,
    EM_SMCU_CMD_DLC_Idle2Run =3,
	EM_SMCU_CMD_DLC_Idle2RunDone = 4,
	EM_SMCU_CMD_VID_Idle2Run =5,
	EM_SMCU_CMD_Utilization =6,
	CORE_2_CORE_EXCEPTION_CMD = 7,
	EM_SMCU_CMD_Cedar = 8,
    DYNAMIC_VF_NUM_CHANGE = 9,
    DYNAMIC_VF_NUM_CHANGE_ACK = 10,
};

#define DYNAMIC_SRIOV_CHECK_NUM				0x5a5a5a01
#define SMCU_RESET_GFX_CHECK_NUM			0x5a5a5a02
#define HOST_NOTIFY_SMCU_STATUS_CHECK_NUM	0x5a5a5a03
#define SRIOV_MODE_PF_FLR_CHECK_NUM			0x5a5a5a04
#endif
