#ifndef __SG100_PCIE_H__
#define __SG100_PCIE_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* For simulation speedup. If not enable, simulation environment may link up fail. */
#define SIMULATION_SPEED_UP 0

#define PCIE_EP_CSR_CTRL_CDM            (0x01000000)
#define PCIE_EP_CSR_CTRL_IATU           (0x02800000)
#define PCIE_EP_CSR_CTRL_DMA            (0x03000000)
#define PCIE_EP_CSR_CTRL_MSIX_TABLE     (0x03800000)
#define PCIE_EP_CSR_CTRL_MSIX_PBA       (0x04000000)
#define PCIE_EP_CSR_VF_CFG_SPACE        (0x01200000)

/* RC */
#define PCIE_RP_CSR_CTRL_IATU           (0x600000)
#define PCIE_RP_CSR_CTRL_CDM            (0x500000)

#define PCIe_EP_CSR_VASTAI_WRAPPER      (0x940000)
#define PCIe_RC_CSR_VASTAI_WRAPPER      (0x960000)

#define EFUSE_VENDOR_DEVICE_ID          (EFUSE_SRAM_BASE_ADDR + 0*4)
#define EFUSE_SUB_VENDOR_DEVICE_ID      (EFUSE_SRAM_BASE_ADDR + 1*4)




////////////////////////RC/////////////////////////////////////////
// Group: PF0_RAS_DES_CAP Registers, P0RDC_
#define CAP_RC_RAS_DES_PTR              0x2CC
#define RC_P0RDC_SD_CONTROL1_REG        (PCIE_RP_CSR_CTRL_CDM + CAP_RC_RAS_DES_PTR + 0xA0)    // 0x500000
#define RC_P0RDC_SD_CONTROL2_REG        (PCIE_RP_CSR_CTRL_CDM + CAP_RC_RAS_DES_PTR + 0xA4)    // 0x500000

////////////////////////EP/////////////////////////////////////////
// Group: PF_RAS_DES_CAPi Registers, (PRDC_), per PF
#define CAP_EP_RAS_DES_PTR(p)           (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x38C)
#define EP_PRDC_SD_CONTROL1_REG(p)      (CAP_EP_RAS_DES_PTR(p) + 0xA0)

/*
 * PF_AER_CAPi Registers, (PAERC_), per PF
 *  - AER: 0x100
 */
#define PAERC_AER_CAP_REG_BASE(p)        (PCIE_EP_CSR_CTRL_CDM +(p)*0x10000 + 0x100)
#define PAERC_AER_EXT_CAP_HDR_OFF(p)     ((PAERC_AER_CAP_REG_BASE(p)) + 0x0)
#define PAERC_UNCORR_ERR_STATUS_OFF(p)   ((PAERC_AER_CAP_REG_BASE(p)) + 0x4)
#define PAERC_UNCORR_ERR_MASK_OFF(p)     ((PAERC_AER_CAP_REG_BASE(p)) + 0x8)
#define PAERC_UNCORR_ERR_SEV_OFF(p)      ((PAERC_AER_CAP_REG_BASE(p)) + 0xC)
#define PAERC_CORR_ERR_STATUS_OFF(p)     ((PAERC_AER_CAP_REG_BASE(p)) + 0x10)
#define PAERC_CORR_ERR_MASK_OFF(p)       ((PAERC_AER_CAP_REG_BASE(p)) + 0x14)
#define PAERC_ADV_ERR_CAP_CTRL_OFF(p)    ((PAERC_AER_CAP_REG_BASE(p)) + 0x18)
#define PAERC_HDR_LOG_0_OFF(p)           ((PAERC_AER_CAP_REG_BASE(p)) + 0x1C)
#define PAERC_HDR_LOG_1_OFF(p)           ((PAERC_AER_CAP_REG_BASE(p)) + 0x20)
#define PAERC_HDR_LOG_2_OFF(p)           ((PAERC_AER_CAP_REG_BASE(p)) + 0x24)
#define PAERC_HDR_LOG_3_OFF(p)           ((PAERC_AER_CAP_REG_BASE(p)) + 0x28)

/*
 * pcie CDM reg offset
 *  - port_logic: 0x700
 */
#define PCIE2_CDM_MSIX_ADDRESS_MATCH_LOW    (0x0940)
#define PCIE2_CDM_MSIX_ADDRESS_MATCH_HIGH   (0x0944)
#define PCIE2_CDM_MSIX_DOORBELL             (0x0948)

/*
 * pcie ATU reg offset
 */
#define PCIE2_EP_ATU_REG_OUTBOUND(num) (PCIE_EP_CSR_CTRL_IATU + (num) * 0x200)
#define PCIE2_EP_ATU_REG_INBOUND(num)  (PCIE_EP_CSR_CTRL_IATU + (num) * 0x200 + 0x100)

#define PCIE2_ATU_REG_CTRL1             (0x00)
#define PCIE2_ATU_REG_CTRL2             (0x04)
#define PCIE2_ATU_REG_LWR_BASE          (0x08)
#define PCIE2_ATU_REG_UPPER_BASE        (0x0c)
#define PCIE2_ATU_REG_LIMIT             (0x10)
#define PCIE2_ATU_REG_LWR_TARGET        (0x14)
#define PCIE2_ATU_REG_UPPER_TARGET      (0x18)
#define PCIE2_ATU_REG_CTRL3             (0x1c)
#define PCIE2_ATU_REG_UPPER_LIMIT       (0x20)

/*
 * pcie dma reg offset
 */
#define PCIE2_EP_DMA_REG_WR_CH(num)         (PCIE_EP_CSR_CTRL_DMA + (num) * 0x2000)
#define PCIE2_EP_DMA_REG_RD_CH(num)         (PCIE_EP_CSR_CTRL_DMA + (num) * 0x2000 + 0x1000)

#define PICE2_DMA_REG_EN                    (0x00)
#define PICE2_DMA_REG_DOORBELL              (0x04)
#define PICE2_DMA_REG_ELEM_PF               (0x08)
#define PICE2_DMA_REG_HANDSHAKE             (0x0c)
#define PCIE2_DMA_REG_LLP_LOW               (0x10)
#define PCIE2_DMA_REG_LLP_HIGH              (0x14)
#define PCIE2_DMA_REG_CYCLE                 (0x18)
#define PCIE2_DMA_REG_XFERSIZE              (0x1c)
#define PCIE2_DMA_REG_SAR_LOW               (0x20)
#define PCIE2_DMA_REG_SAR_HIGH              (0x24)
#define PCIE2_DMA_REG_DAR_LOW               (0x28)
#define PCIE2_DMA_REG_DAR_HIGH              (0x2c)
#define PCIE2_DMA_REG_WATERMARK_EN          (0x30)
#define PCIE2_DMA_REG_CONTROL1              (0x34)
#define PCIE2_DMA_REG_FUNC_NUM              (0x38)
#define PCIE2_DMA_REG_QOS                   (0x3c)
#define PCIE2_DMA_REG_STATUS                (0x80)
#define PCIE2_DMA_REG_INT_STATUS            (0x84)
#define PCIE2_DMA_REG_INT_SETUP             (0x88)
#define PCIE2_DMA_REG_INT_CLEAR             (0x8c)
#define PCIE2_DMA_REG_MSI_STOP_LOW          (0x90)
#define PCIE2_DMA_REG_MSI_STOP_HIGH         (0x94)
#define PCIE2_DMA_REG_MSI_WATERMARK_LOW     (0x98)
#define PCIE2_DMA_REG_MSI_WATERMARK_HIGH    (0x9c)
#define PCIE2_DMA_REG_MSI_ABORT_LOW         (0xa0)
#define PCIE2_DMA_REG_MSI_ABORT_HIGH        (0xa4)
#define PCIE2_DMA_REG_MSI_MSGD              (0xa8)


#define BIT_LINK_CAPABLE                    16    /* 21:16 */
#define BIT_DIRECT_SPEED_CHANGE             17    /* 0x1 (SET): Writing '1' to this field instructs the LTSSM to initiate a speed change to Gen2 or Gen3 after the link is initialized at Gen1 speed. */

/* group: PF0_PORT_LOGIC Registers (PF0PLR_) */
#define PF0PLR_MISC_CONTROL_1_OFF                   (PCIE_EP_CSR_CTRL_CDM + 0x8bc)
#define PF0PLR_TIMER_CTRL_MAX_FUNC_NUM_OFF_REG      (PCIE_EP_CSR_CTRL_CDM + 0x718)
#define PF0PLR_MULTI_LANE_CONTROL_OFF               (PCIE_EP_CSR_CTRL_CDM + 0x8c0)
#define PF0PLR_GEN2_CTRL_OFF                        (PCIE_EP_CSR_CTRL_CDM + 0x80c)
#define PF0PLR_PORT_LINK_CTRL_OFF                   (PCIE_EP_CSR_CTRL_CDM + 0x710)
#define PF0PLR_ACK_LATENCY_TIMER_OFF                (PCIE_EP_CSR_CTRL_CDM + 0x700)
#define PF0PLR_TIMER_CTRL_MAX_FUNC_NUM_OFF          (PCIE_EP_CSR_CTRL_CDM + 0x718)

/* Extended Capabilities in Configuration Space always begin at offset 100h with a PCI Express Extended Capability header */
#define PCIe_EXT_CAP_CONFIG_SPACE_OFFSET            0x100

// group: PF_PCIE_CAPi (PPC_), per PF
#define PF_PCIE_CAP_BASE_ADDR(p)                    (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x70)
#define PPC_DEVICE_CAPABILITIES_REG(p)		    (PF_PCIE_CAP_BASE_ADDR(p) + 0x04)
#define PPC_DEVICE_CONTROL_DEVICE_STATUS_REG(p)	    (PF_PCIE_CAP_BASE_ADDR(p) + 0x08)
#define PPC_LINK_CONTROL2_LINK_STATUS2_REG(p)       (PF_PCIE_CAP_BASE_ADDR(p) + 0x30)

/* group: PF0_MARGIN_CAP Registers(PMC_), per PF */
#define MARGIN_EXT_CAP_OFFSET                       (0x200)
#define PMC_MARGIN_EXT_CAP_HDR_REG(p)               (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + MARGIN_EXT_CAP_OFFSET + 0x00)  /* Next pointer is SRIOV */

/* group: PF0_PL16G_CAP Registers (PPLC_), per PF */
#define PPLC_PL16G_EXT_CAP_HDR_REG(p)               (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x1D0 + 0x00)  /* Next pointer is ? */

/* group: PF_TPH_CAPi Registers (PTC_), per PF */
#define TPH_EXT_CAP_OFFSET                          (0x288) 
#define PTC_TPH_EXT_CAP_HDR_REG(p)                  (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + TPH_EXT_CAP_OFFSET + 0x00)  /* Next pointer is ATS */

/* group: PF_ATS_CAPi Registers (PATC_), per PF */
#define PATC_ATS_CAP_HDR_REG(p)                     (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x314 + 0x00)

/* group: PF_ACS_CAPi Registers (PAC_),  per PF */
#define PAC_ACS_CAP_HDR_REG(p)                      (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x324 + 0x00)  /* Next pointer is PRS */

/* group: PF_PRS_EXT_CAPi Registers (PPEC_),  per PF */
#define PPEC_PRS_EXT_CAP_HDR_REG(p)                 (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x330 + 0x00)

/* group: PF0_LTR_CAP Registers (PLC_),  per PF */
#define LTR_EXT_CAP_OFFSET                          (0x340) 
#define PLC_LTR_CAP_HDR_REG(p)                      (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + LTR_EXT_CAP_OFFSET + 0x00)  // Next pointer is PASID

/* group: PF_PASID_CAPi Registers (PPAC_),  per PF */
#define PPAC_PASID_EXT_CAP_HDR_REG(p)               (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x348 + 0x00)

//
// PCI Capability List IDs and records
//
#define PCI_CAPABILITY_ID_PMI                       0x01
#define PCI_CAPABILITY_ID_AGP                       0x02
#define PCI_CAPABILITY_ID_VPD                       0x03
#define PCI_CAPABILITY_ID_SLOTID                    0x04
#define PCI_CAPABILITY_ID_MSI                       0x05
#define PCI_CAPABILITY_ID_HOTPLUG                   0x06
#define PCI_CAPABILITY_ID_PCIX                      0x07
#define EFI_PCI_CAPABILITY_ID_SHPC                  0x0C
#define CAPABILITY_ID_PCIEXP                        0x10

#define PCI_VENDOR_ID_OFFSET                        0x00
#define PCI_DEVICE_ID_OFFSET                        0x02
#define PCI_COMMAND_OFFSET                          0x04
#define PCI_PRIMARY_STATUS_OFFSET                   0x06
#define PCI_REVISION_ID_OFFSET                      0x08
#define PCI_CLASSCODE_OFFSET                        0x09
#define PCI_CACHELINE_SIZE_OFFSET                   0x0C
#define PCI_LATENCY_TIMER_OFFSET                    0x0D
#define PCI_HEADER_TYPE_OFFSET                      0x0E
#define PCI_BIST_OFFSET                             0x0F
#define PCI_BASE_ADDRESSREG_OFFSET                  0x10
#define PCI_CARDBUS_CIS_OFFSET                      0x28
#define PCI_SVID_OFFSET                             0x2C ///< SubSystem Vendor id
#define PCI_SUBSYSTEM_VENDOR_ID_OFFSET              0x2C
#define PCI_SID_OFFSET                              0x2E ///< SubSystem ID
#define PCI_SUBSYSTEM_ID_OFFSET                     0x2E
#define PCI_EXPANSION_ROM_BASE                      0x30
#define PCI_CAPBILITY_POINTER_OFFSET                0x34
#define PCI_INT_LINE_OFFSET                         0x3C ///< Interrupt Line Register
#define PCI_INT_PIN_OFFSET                          0x3D ///< Interrupt Pin Register
#define PCI_MAXGNT_OFFSET                           0x3E ///< Max Grant Register
#define PCI_MAXLAT_OFFSET                           0x3F ///< Max Latency Register
#define PCI_MSIX_CAP_OFFSET                         0xB0 
/* group: PF_TYPE0_HDRi (PT0H_), per PF */
#define PT0H_CONFIG_SPACE_BASE(p)                       (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000)
#define PT0H_DEVICE_ID_VENDOR_ID_REG(p)                 (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x00)
#define PT0H_STATUS_COMMAND_REG(p)                      (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x04)       // BME
#define PT0H_CLASS_CODE_REVISION_ID_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x08)
#define PT0H_SUBSYSTEM_ID_SUBSYSTEM_VENDOR_ID_REG(p)    (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x2c)
#define PT0H_BAR0_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x10)
#define PT0H_BAR1_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x14)
#define PT0H_BAR2_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x18)
#define PT0H_BAR3_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x1c)
#define PT0H_BAR4_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x20)
#define PT0H_BAR5_REG(p)                                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x24)
#define PT0H_EXP_ROM_BASE_ADDR_REG(p)                   (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x30)
#define PT0H_CAPBILITY_POINTER_OFFSET(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x34)

#define VF_CONFIG_SPACE_BASE(pfn,vfn,vf_active)         (PCIE_EP_CSR_CTRL_CDM + ((pfn)<<16) + ((vfn) <<20) + ((vf_active) << 19))
#define VF_MSIX_TABLE(pfn,vfn,vf_active)                (PCIE_EP_CSR_CTRL_MSIX_TABLE + ((pfn)<<16) + ((vfn) <<20) + ((vf_active) << 19))



/* group: PF_TYPE0_HDR_DBI2i (PT0HDBI2_), per PF */
#define PT0HDBI2_BAR0_MASK_REG(p)               (PT0H_BAR0_REG(p) + 0x800000)
#define PT0HDBI2_BAR1_MASK_REG(p)               (PT0H_BAR1_REG(p) + 0x800000)
#define PT0HDBI2_BAR2_MASK_REG(p)               (PT0H_BAR2_REG(p) + 0x800000)
#define PT0HDBI2_BAR3_MASK_REG(p)               (PT0H_BAR3_REG(p) + 0x800000)
#define PT0HDBI2_BAR4_MASK_REG(p)               (PT0H_BAR4_REG(p) + 0x800000)
#define PT0HDBI2_BAR5_MASK_REG(p)               (PT0H_BAR5_REG(p) + 0x800000)
#define PT0HDBI2_EXP_ROM_BAR_MASK_REG(p)        (PT0H_EXP_ROM_BASE_ADDR_REG(p) + 0x800000)

/* group: PF_SRIOV_CAPi (PSRIOC_), per PF */
#define SRIOV_CAP_BASE_OFFSET                   (0x248) 
#define PSRIOC_SRIOV_BASE_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x00)
#define PSRIOC_STATUS_CONTROL_REG(p)            (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x08)
#define PSRIOC_TOTAL_VFS_INITIAL_VFS_REG(p)     (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x0c)
#define PSRIOC_SRIOV_VF_OFFSET_POSITION(p)      (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x14)
#define PSRIOC_VF_DEVICE_ID_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x18)
#define PSRIOC_SUP_PAGE_SIZES_REG(p)            (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x1C)

#define PSRIOC_SRIOV_BAR0_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x24)
#define PSRIOC_SRIOV_BAR1_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x28)
#define PSRIOC_SRIOV_BAR2_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x2c)
#define PSRIOC_SRIOV_BAR3_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x30)
#define PSRIOC_SRIOV_BAR4_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x34)
#define PSRIOC_SRIOV_BAR5_REG(p)                (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x38)

#define VF_ACTIVE_OFFSET                        0x400000 /* (1 << 22)  active by default */
/* Bar address for each VF of each PF */
#define PSRIOC_SRIOV_BAR0_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x24)
#define PSRIOC_SRIOV_BAR1_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x28)
#define PSRIOC_SRIOV_BAR2_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x2c)
#define PSRIOC_SRIOV_BAR3_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x30)
#define PSRIOC_SRIOV_BAR4_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x34)
#define PSRIOC_SRIOV_BAR5_REG_VF(p, v)          (PCIE_EP_CSR_CTRL_CDM + VF_ACTIVE_OFFSET + (v)*0x80000 + (p)*0x10000 + SRIOV_CAP_BASE_OFFSET + 0x38)


/* group: PF_SRIOV_CAP_DBI2i (PSRIOC_DBI2_), per PF */
#define PSRIOC_DBI2_SHADOW_STATUS_CONTROL_REG(p)        (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x08)
#define PSRIOC_DBI2_SHADOW_SRIOV_VF_OFFSET_POS(p)       (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x14)
#define PSRIOC_DBI2_SRIOV_BAR0_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x24)
#define PSRIOC_DBI2_SRIOV_BAR1_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x28)
#define PSRIOC_DBI2_SRIOV_BAR2_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x2c)
#define PSRIOC_DBI2_SRIOV_BAR3_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x30)
#define PSRIOC_DBI2_SRIOV_BAR4_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x34)
#define PSRIOC_DBI2_SRIOV_BAR5_MASK_REG(p)              (PCIE_EP_CSR_CTRL_CDM + (p)*0x10000 + 0x248 + 0x800000 + 0x38)

/* group: PF0_ATU_CAP Registers (P0ATUC_) */
#define P0ATUC_IATU_REGION_CTRL_1_OFF_OUTBOUND_i(or_idx)        (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x00)
#define P0ATUC_IATU_REGION_CTRL_2_OFF_OUTBOUND_i(or_idx)        (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x04)
#define P0ATUC_IATU_LWR_BASE_ADDR_OFF_OUTBOUND_i(or_idx)        (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x08)
#define P0ATUC_IATU_UPPER_BASE_ADDR_OFF_OUTBOUND_i(or_idx)      (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x0C)
#define P0ATUC_IATU_LIMIT_ADDR_OFF_OUTBOUND_i(or_idx)           (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x10)
#define P0ATUC_IATU_LWR_TARGET_ADDR_OFF_OUTBOUND_i(or_idx)      (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x14)
#define P0ATUC_IATU_UPPER_TARGET_ADDR_OFF_OUTBOUND_i(or_idx)    (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x18)
#define P0ATUC_IATU_REGION_CTRL_3_OFF_OUTBOUND_i(or_idx)        (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x1C)
#define P0ATUC_IATU_UPPR_LIMIT_ADDR_OFF_OUTBOUND_i(or_idx)      (PCIE_EP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x20)
/* inbound */
#define P0ATUC_IATU_REGION_CTRL_1_OFF_INBOUND_i(ir_idx)         (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x100)
#define P0ATUC_IATU_REGION_CTRL_2_OFF_INBOUND_i(ir_idx)         (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x104)
#define P0ATUC_IATU_LWR_BASE_ADDR_OFF_INBOUND_i(ir_idx)         (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x108)
#define P0ATUC_IATU_UPPER_BASE_ADDR_OFF_INBOUND_i(ir_idx)       (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x10C)
#define P0ATUC_IATU_LIMIT_ADDR_OFF_INBOUND_i(ir_idx)            (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x110)
#define P0ATUC_IATU_LWR_TARGET_ADDR_OFF_INBOUND_i(ir_idx)       (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x114)
#define P0ATUC_IATU_UPPER_TARGET_ADDR_OFF_INBOUND_i(ir_idx)     (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x118)
#define P0ATUC_IATU_REGION_CTRL_3_OFF_INBOUND_i(ir_idx)         (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x11C)
#define P0ATUC_IATU_UPPR_LIMIT_ADDR_OFF_INBOUND_i(ir_idx)       (PCIE_EP_CSR_CTRL_IATU + (ir_idx) * 2 * 0x100 + 0x120)

/* RP outbound */
#define RP_ATUC_IATU_REGION_CTRL_1_OFF_OUTBOUND_i(or_idx)       (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x00)
#define RP_ATUC_IATU_REGION_CTRL_2_OFF_OUTBOUND_i(or_idx)       (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x04)
#define RP_ATUC_IATU_LWR_BASE_ADDR_OFF_OUTBOUND_i(or_idx)       (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x08)
#define RP_ATUC_IATU_UPPER_BASE_ADDR_OFF_OUTBOUND_i(or_idx)     (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x0C)
#define RP_ATUC_IATU_LIMIT_ADDR_OFF_OUTBOUND_i(or_idx)          (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x10)
#define RP_ATUC_IATU_LWR_TARGET_ADDR_OFF_OUTBOUND_i(or_idx)     (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x14)
#define RP_ATUC_IATU_UPPER_TARGET_ADDR_OFF_OUTBOUND_i(or_idx)   (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x18)
#define RP_ATUC_IATU_REGION_CTRL_3_OFF_OUTBOUND_i(or_idx)       (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x1C)
#define RP_ATUC_IATU_UPPR_LIMIT_ADDR_OFF_OUTBOUND_i(or_idx)     (PCIE_RP_CSR_CTRL_IATU + (or_idx) * 2 * 0x100 + 0x20)

#define INCREASE_REGION_SIZE_BIT                                (13)

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __SG100_PCIE_H__ */
