#include <linux/ctype.h>
#include <linux/delay.h>
#include <linux/pci.h>
#include <linux/platform_device.h>
#include <linux/uaccess.h>
#include <linux/version.h>
#include <linux/random.h>
#include <linux/dma-buf.h>

#include "vastai_export_enum_define.h"
#include "vastai_pci.h"
#include "vastai_pci_api.h"
#include "vastai_pci_test.h"
#include "vastai_cfg.h"
#include "hw_config.h"

static void bubble_sort_inbound(struct vastai_inbound_atu_region_info *arry, int len) {
	int i, j;
	struct vastai_inbound_atu_region_info temp;
	for (i = 0; i < len - 1; i++)
		for (j = 0; j < len - 1 - i; j++)
			if (arry[j].noc_base_addr > arry[j + 1].noc_base_addr) {
				//1:save arry[j] element to temp 
				temp.region_num 	  = arry[j].region_num;
				temp.pf_num     	  = arry[j].pf_num;
				temp.vf_num           = arry[j].vf_num;
				temp.mode       	  = arry[j].mode;
				temp.bar_num          = arry[j].bar_num;
				temp.noc_base_addr    = arry[j].noc_base_addr;
				temp.host_base_addr   = arry[j].host_base_addr;
				temp.host_limit_addr  = arry[j].host_limit_addr;
				temp.size             = arry[j].size;

				//2:save arry[j+1] element to arry[j] 
				arry[j].region_num 	     = arry[j+1].region_num;
				arry[j].pf_num     	     = arry[j+1].pf_num;
				arry[j].vf_num           = arry[j+1].vf_num;
				arry[j].mode       	     = arry[j+1].mode;
				arry[j].bar_num          = arry[j+1].bar_num;
				arry[j].noc_base_addr    = arry[j+1].noc_base_addr;
				arry[j].host_base_addr   = arry[j+1].host_base_addr;
				arry[j].host_limit_addr  = arry[j+1].host_limit_addr;
				arry[j].size             = arry[j+1].size;

				//3:save temp element to arry[j+1] 
				arry[j+1].region_num 	   = temp.region_num;
				arry[j+1].pf_num     	   = temp.pf_num;
				arry[j+1].vf_num           = temp.vf_num;
				arry[j+1].mode       	   = temp.mode;
				arry[j+1].bar_num          = temp.bar_num;
				arry[j+1].noc_base_addr    = temp.noc_base_addr;
				arry[j+1].host_base_addr   = temp.host_base_addr;
				arry[j+1].host_limit_addr  = temp.host_limit_addr;
				arry[j+1].size             = temp.size;
			}
	return;
}

static void bubble_sort_outbound(struct vastai_outbound_atu_region_info *arry, int len) {
	int i, j;
	struct vastai_outbound_atu_region_info temp;
	for (i = 0; i < len - 1; i++)
		for (j = 0; j < len - 1 - i; j++)
			if (arry[j].noc_base_addr > arry[j + 1].noc_base_addr) {
				//1:save arry[j] element to temp 
				temp.region_num 	     = arry[j].region_num;
				temp.noc_base_addr       = arry[j].noc_base_addr;
				temp.host_base_addr      = arry[j].host_base_addr;
				temp.noc_limit_addr      = arry[j].noc_limit_addr;
				temp.size                = arry[j].size;

				//2:save arry[j+1] element to arry[j] 
				arry[j].region_num 	     = arry[j+1].region_num;
				arry[j].noc_base_addr    = arry[j+1].noc_base_addr;
				arry[j].host_base_addr   = arry[j+1].host_base_addr;
				arry[j].noc_limit_addr   = arry[j+1].noc_limit_addr;
				arry[j].size             = arry[j+1].size;

				//3:save temp element to arry[j+1] 
				arry[j+1].region_num 	   = temp.region_num;
				arry[j+1].noc_base_addr    = temp.noc_base_addr;
				arry[j+1].host_base_addr   = temp.host_base_addr;
				arry[j+1].noc_limit_addr   = temp.noc_limit_addr;
				arry[j+1].size             = temp.size;
			}
	return;
}

void dump_atu(struct vastai_pci_info *priv)
{
	int region_num;
	int reg_addr;
	int valid_atu_num=0;
	int ret;
	struct vastai_inbound_atu_region_info   *valid_inbound_atu_info_array;
    struct vastai_outbound_atu_region_info  *valid_outbound_atu_info_array;
	struct inbound_atu_regs    temp_inbound_regs;
    struct outbound_atu_regs   temp_outbound_regs;
	u64 temp_noc_base_addr;
	u64 temp_host_base_addr;
	u64 temp_host_limit_addr;
    u64 temp_noc_limit_addr;
	u64 temp_size;
	u8  mode;
	int i;

	valid_inbound_atu_info_array = kzalloc(sizeof(struct vastai_inbound_atu_region_info) * MAX_REGION_NUM, GFP_KERNEL);

	for(region_num = 0;region_num < MAX_REGION_NUM; region_num++) {
		reg_addr=0x02800000 + region_num * 2 * 0x100 + 0x100;
		//to do dump slave bar atu
		ret = vastai_read_by_smcu(priv,  reg_addr, sizeof(struct inbound_atu_regs) ,&temp_inbound_regs);
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_inbound_regs), ret);
			goto end_free;
		}

		temp_host_base_addr  = (u64) temp_inbound_regs.hi32_host_addr << 32 | temp_inbound_regs.lo32_host_addr;
		temp_noc_base_addr   = (u64) temp_inbound_regs.hi32_noc_addr << 32 | temp_inbound_regs.lo32_noc_addr;
		temp_host_limit_addr = (u64) temp_inbound_regs.hi32_limit << 32 | temp_inbound_regs.lo32_limit;

		if((0 == ((temp_inbound_regs.ctrl2_reg>>31)&0x1))) {
			continue;
		}

		temp_size = temp_host_limit_addr - temp_host_base_addr + 1;

		//0:address match mode  1:bar match mode
		mode = temp_inbound_regs.ctrl2_reg>>30 & 0x1;

		valid_inbound_atu_info_array[valid_atu_num].region_num			= region_num;
		valid_inbound_atu_info_array[valid_atu_num].mode				= mode;
		valid_inbound_atu_info_array[valid_atu_num].pf_num				= temp_inbound_regs.ctrl1_reg>>20 & 0xFF;
		valid_inbound_atu_info_array[valid_atu_num].noc_base_addr		= temp_noc_base_addr;
		//0:address match mode 
		if(0 == mode) {
			valid_inbound_atu_info_array[valid_atu_num].host_base_addr	= temp_host_base_addr;
			valid_inbound_atu_info_array[valid_atu_num].host_limit_addr	= temp_host_limit_addr;
			valid_inbound_atu_info_array[valid_atu_num].size			= temp_size;
			valid_inbound_atu_info_array[valid_atu_num].bar_num			= 0xFF;
		} else {
			valid_inbound_atu_info_array[valid_atu_num].bar_num			= temp_inbound_regs.ctrl2_reg>>8  & 0x7;
			
		}

		//check vf enable
		if(1 == (temp_inbound_regs.ctrl2_reg>>20 &0x1)) {
			valid_inbound_atu_info_array[valid_atu_num].vf_num = temp_inbound_regs.ctrl3_reg&0x1F;
		} else {
			valid_inbound_atu_info_array[valid_atu_num].vf_num = 0xFF; 
		}

		valid_atu_num++;

	}

	bubble_sort_inbound(valid_inbound_atu_info_array,valid_atu_num);
	VASTAI_PCI_INFO(priv, priv->pkg_id,"-----start to dump %d valid inbound atu info-----\n",valid_atu_num);
	for(i = 0;i < valid_atu_num; i++) {
		if(0==valid_inbound_atu_info_array[i].mode){
			VASTAI_PCI_INFO(priv, priv->pkg_id,"rgn[%03d] pf:%d vf:%02x noc[0x%010llx ~ 0x%010llx] map to host[%016llx ~ %016llx] size[0x%llx]\n",
								valid_inbound_atu_info_array[i].region_num,
								valid_inbound_atu_info_array[i].pf_num,
								valid_inbound_atu_info_array[i].vf_num,
								valid_inbound_atu_info_array[i].noc_base_addr,
								valid_inbound_atu_info_array[i].noc_base_addr+valid_inbound_atu_info_array[i].size-1,
								valid_inbound_atu_info_array[i].host_base_addr,
								valid_inbound_atu_info_array[i].host_limit_addr,
                                valid_inbound_atu_info_array[i].size);
		} else if (1==valid_inbound_atu_info_array[i].mode){
			VASTAI_PCI_INFO(priv, priv->pkg_id,"rgn[%03d] pf:%d vf:%02x noc[0x%010llx] map to host bar[%x]\n",
								valid_inbound_atu_info_array[i].region_num,
								valid_inbound_atu_info_array[i].pf_num,
								valid_inbound_atu_info_array[i].vf_num,
								valid_inbound_atu_info_array[i].noc_base_addr,
								valid_inbound_atu_info_array[i].bar_num);
		}
		
	}
	
    //start dump outbound atu
    valid_atu_num = 0;
    valid_outbound_atu_info_array = kzalloc(sizeof(struct vastai_outbound_atu_region_info) * MAX_REGION_NUM, GFP_KERNEL);
    for(region_num = 0;region_num < MAX_REGION_NUM; region_num++) {
        reg_addr=0x02800000 + region_num * 2 * 0x100;
		//to do dump slave bar atu
		ret = vastai_read_by_smcu(priv,  reg_addr, sizeof(struct outbound_atu_regs) ,&temp_outbound_regs);
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_outbound_regs), ret);
			continue;
		}

        if((0 == ((temp_outbound_regs.ctrl2_reg>>31)&0x1))) {
			continue;
		}

        temp_host_base_addr  = (u64) temp_outbound_regs.hi32_cpu_addr << 32 | temp_outbound_regs.lo32_cpu_addr;
		temp_noc_base_addr   = (u64) temp_outbound_regs.hi32_noc_addr << 32 | temp_outbound_regs.lo32_noc_addr;
		temp_noc_limit_addr  = (u64) temp_outbound_regs.hi32_limit << 32 | temp_outbound_regs.lo32_limit;
        temp_size            = temp_noc_limit_addr - temp_noc_base_addr + 1;
        valid_outbound_atu_info_array[valid_atu_num].region_num             = region_num;
        valid_outbound_atu_info_array[valid_atu_num].host_base_addr         = temp_host_base_addr;
        valid_outbound_atu_info_array[valid_atu_num].noc_base_addr          = temp_noc_base_addr;
        valid_outbound_atu_info_array[valid_atu_num].noc_limit_addr         = temp_noc_limit_addr;
        valid_outbound_atu_info_array[valid_atu_num].size                   = temp_size;
        valid_atu_num++;
    }

    bubble_sort_outbound(valid_outbound_atu_info_array,valid_atu_num);
    VASTAI_PCI_INFO(priv, priv->pkg_id,"-----start to dump %d valid outbound atu info-----\n",valid_atu_num);
    for(i = 0;i < valid_atu_num; i++) {
        VASTAI_PCI_INFO(priv, priv->pkg_id,"rgn[%03d] noc[0x%010llx ~ 0x%010llx] map to host[%016llx ~ %016llx] size[0x%llx]\n",
								valid_outbound_atu_info_array[i].region_num,
								valid_outbound_atu_info_array[i].noc_base_addr,
								valid_outbound_atu_info_array[i].noc_limit_addr ,
								valid_outbound_atu_info_array[i].host_base_addr,
								valid_outbound_atu_info_array[i].host_base_addr + valid_outbound_atu_info_array[i].size-1,
                                valid_outbound_atu_info_array[i].size);
    }

    kfree(valid_outbound_atu_info_array);
end_free:
    kfree(valid_inbound_atu_info_array);
	return;
}


void smmu_mpu_cfg(struct vastai_pci_info *priv)
{
    u32 stream_id;
    u32 region_num;
    u32 smmu_mpu_reg_base = 0;
    struct smmu_mpu_info *smmu_mpu_info_array;
    struct smmu_mpu_regs temp_smmu_mpu_regs;
    int ret;
    smmu_mpu_info_array = kzalloc(sizeof(struct smmu_mpu_info) * 32, GFP_KERNEL);

    for(stream_id=0;stream_id<32;stream_id++){
        region_num = 0;
        smmu_mpu_reg_base = (0xB00000+0xC0500) + region_num*256*4+ stream_id*(4*4);
        ret = vastai_read_by_smcu(priv,  smmu_mpu_reg_base, sizeof(struct smmu_mpu_regs) ,&temp_smmu_mpu_regs);           
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_smmu_mpu_regs), ret);
			goto end_free;
		}
        smmu_mpu_info_array[stream_id].input_base_ddr  = ((u64)temp_smmu_mpu_regs.inputH_base<<32|temp_smmu_mpu_regs.inputL_base)<<12;
        smmu_mpu_info_array[stream_id].output_base_ddr = (u64)temp_smmu_mpu_regs.output_base<<12;
        smmu_mpu_info_array[stream_id].output_size_ddr = (u64)temp_smmu_mpu_regs.output_size<<12;

        region_num = 1;
        smmu_mpu_reg_base = (0xB00000+0xC0500) + region_num*256*4+ stream_id*(4*4);
        ret = vastai_read_by_smcu(priv,  smmu_mpu_reg_base, sizeof(struct smmu_mpu_regs) ,&temp_smmu_mpu_regs);           
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_smmu_mpu_regs), ret);
			goto end_free;
		}
        smmu_mpu_info_array[stream_id].input_base_csram  = ((u64)temp_smmu_mpu_regs.inputH_base<<32|temp_smmu_mpu_regs.inputL_base)<<12;
        smmu_mpu_info_array[stream_id].output_base_csram = (u64)temp_smmu_mpu_regs.output_base<<12;
        smmu_mpu_info_array[stream_id].output_size_csram = (u64)temp_smmu_mpu_regs.output_size<<12;

        region_num = 2;
        smmu_mpu_reg_base = (0xB00000+0xC0500) + region_num*256*4+ stream_id*(4*4);
        ret = vastai_read_by_smcu(priv,  smmu_mpu_reg_base, sizeof(struct smmu_mpu_regs) ,&temp_smmu_mpu_regs);           
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_smmu_mpu_regs), ret);
			goto end_free;
		}
        smmu_mpu_info_array[stream_id].input_base_rgn2  = ((u64)temp_smmu_mpu_regs.inputH_base<<32|temp_smmu_mpu_regs.inputL_base)<<12;
        smmu_mpu_info_array[stream_id].output_base_rgn2 = (u64)temp_smmu_mpu_regs.output_base<<12;
        smmu_mpu_info_array[stream_id].output_size_rgn2 = (u64)temp_smmu_mpu_regs.output_size<<12;
    }
    VASTAI_PCI_INFO(priv, priv->pkg_id,"-----start to dump valid smmu mpu info-----\n");

    for(stream_id=0;stream_id<32;stream_id++){
        if((0==smmu_mpu_info_array[stream_id].input_base_ddr) && (0==smmu_mpu_info_array[stream_id].output_base_ddr)) {
            continue;   
        }
        VASTAI_PCI_INFO(priv, priv->pkg_id,"stream_id[%02d] rgn0 fn_addr[0x%010llx ~ 0x%010llx] map to noc [0x%010llx ~ 0x%010llx] size=[0x%010llx=%06lldMB]\n",
                                stream_id,smmu_mpu_info_array[stream_id].input_base_ddr,
                                smmu_mpu_info_array[stream_id].input_base_ddr+smmu_mpu_info_array[stream_id].output_size_ddr-1,
                                smmu_mpu_info_array[stream_id].output_base_ddr,
                                smmu_mpu_info_array[stream_id].output_base_ddr+smmu_mpu_info_array[stream_id].output_size_ddr-1,
                                smmu_mpu_info_array[stream_id].output_size_ddr,
                                smmu_mpu_info_array[stream_id].output_size_ddr/1024/1024);
    }

    for(stream_id=0;stream_id<32;stream_id++) {
        if((0==smmu_mpu_info_array[stream_id].output_base_csram) && (0==smmu_mpu_info_array[stream_id].input_base_csram)) {
            continue;   
        }
        VASTAI_PCI_INFO(priv, priv->pkg_id,"stream_id[%02d] rgn1 fn_addr[0x%010llx ~ 0x%010llx] map to noc [0x%010llx ~ 0x%010llx] size=[0x%010llx=%06lldKB]\n",
                                stream_id,smmu_mpu_info_array[stream_id].input_base_csram,
                                smmu_mpu_info_array[stream_id].input_base_csram+smmu_mpu_info_array[stream_id].output_size_csram-1,
                                smmu_mpu_info_array[stream_id].output_base_csram,
                                smmu_mpu_info_array[stream_id].output_base_csram+smmu_mpu_info_array[stream_id].output_size_csram-1,
                                smmu_mpu_info_array[stream_id].output_size_csram,
                                smmu_mpu_info_array[stream_id].output_size_csram/1024);
    }

    for(stream_id=0;stream_id<32;stream_id++) {
        if((0==smmu_mpu_info_array[stream_id].output_base_rgn2) && (0==smmu_mpu_info_array[stream_id].input_base_rgn2)) {
            continue;   
        }
        VASTAI_PCI_INFO(priv, priv->pkg_id,"stream_id[%02d] rgn2 fn_addr[0x%010llx ~ 0x%010llx] map to noc [0x%010llx ~ 0x%10llx] size=[0x%010llx=%06lldMB]\n",
                                stream_id,smmu_mpu_info_array[stream_id].input_base_rgn2,
                                smmu_mpu_info_array[stream_id].input_base_rgn2+smmu_mpu_info_array[stream_id].output_size_rgn2-1,
                                smmu_mpu_info_array[stream_id].output_base_rgn2,
                                smmu_mpu_info_array[stream_id].output_base_rgn2+smmu_mpu_info_array[stream_id].output_size_rgn2-1,
                                smmu_mpu_info_array[stream_id].output_size_rgn2,
                                smmu_mpu_info_array[stream_id].output_size_rgn2/1024/1024);
    }
end_free:        
    kfree(smmu_mpu_info_array);
    return;
}



void gfx_mpu_cfg(struct vastai_pci_info *priv)
{
    u32 osid = 0;
    u32 region_num = 0;
    u32 gfx_mpu_reg_base = 0;
    struct gfx_mpu_info *gfx_mpu_info_array;
    struct gfx_mpu_regs temp_gfx_mpu_regs;
    int ret;
    gfx_mpu_info_array = kzalloc(sizeof(struct gfx_mpu_info) * 32, GFP_KERNEL);
    for(osid = 0; osid < 32; osid++){
        gfx_mpu_reg_base = 0xaf8000 + osid/8 * 0x2000 + 0x300 + 4 + (osid%8*12*4);
        ret = vastai_read_by_smcu(priv,  gfx_mpu_reg_base, sizeof(struct gfx_mpu_regs) ,&temp_gfx_mpu_regs);           
		if (ret < 0) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s region_num = 0x%x, len=%ld, error:%d\n",
				       __func__, region_num, sizeof(temp_gfx_mpu_regs), ret);
			continue;
		}

        gfx_mpu_info_array[osid].input_base_tgt0  = (u64)temp_gfx_mpu_regs.tgt0_inputH_base<<32|temp_gfx_mpu_regs.tgt0_inputL_base;
        gfx_mpu_info_array[osid].output_size_tgt0 = (u64)temp_gfx_mpu_regs.tgt0_output_size<<12;
        gfx_mpu_info_array[osid].output_base_tgt0 = (u64)temp_gfx_mpu_regs.tgt0_output_base<<12;

        gfx_mpu_info_array[osid].input_base_tgt1  = (u64)temp_gfx_mpu_regs.tgt1_inputH_base<<32|temp_gfx_mpu_regs.tgt1_inputL_base;
        gfx_mpu_info_array[osid].output_size_tgt1 = (u64)temp_gfx_mpu_regs.tgt1_output_size<<12;
        gfx_mpu_info_array[osid].output_base_tgt1 = (u64)temp_gfx_mpu_regs.tgt1_output_base<<12;

        gfx_mpu_info_array[osid].input_base_tgt2  = (u64)temp_gfx_mpu_regs.tgt2_inputH_base<<32|temp_gfx_mpu_regs.tgt2_inputL_base;
        gfx_mpu_info_array[osid].output_size_tgt2 = (u64)temp_gfx_mpu_regs.tgt2_output_size<<12;
        gfx_mpu_info_array[osid].output_base_tgt2 = (u64)temp_gfx_mpu_regs.tgt2_output_base<<12;
    }
    
    VASTAI_PCI_INFO(priv, priv->pkg_id,"-----start to dump valid gfx mpu info-----\n");
    for(osid = 0; osid < 32; osid++){
        if(0==gfx_mpu_info_array[osid].output_size_tgt0) {
            continue;
        }

        VASTAI_PCI_INFO(priv, priv->pkg_id,"[GPU%d] osid[%d] rgn[0] fn_addr[0x%010llx ~ 0x%010llx] map to noc[0x%010llx ~ 0x%010llx] size=[0x%010llx=%06lldMB]\n",
                                            osid/8, osid % 8,gfx_mpu_info_array[osid].input_base_tgt0,
                                            gfx_mpu_info_array[osid].input_base_tgt0+gfx_mpu_info_array[osid].output_size_tgt0-1,
                                            gfx_mpu_info_array[osid].output_base_tgt0,
                                            gfx_mpu_info_array[osid].output_base_tgt0+gfx_mpu_info_array[osid].output_size_tgt0-1,
                                            gfx_mpu_info_array[osid].output_size_tgt0,
                                            gfx_mpu_info_array[osid].output_size_tgt0/1024/1024);
    }

    for(osid = 0; osid < 32; osid++){
        if(0==gfx_mpu_info_array[osid].output_size_tgt1) {
            continue;
        }
        
        VASTAI_PCI_INFO(priv, priv->pkg_id,"[GPU%d] osid[%d] rgn[1] fn_addr[0x%010llx ~ 0x%010llx] map to noc[0x%010llx ~ 0x%010llx] size=[0x%010llx=%06lldMB]\n",
                                            osid/8, osid % 8,gfx_mpu_info_array[osid].input_base_tgt1,
                                            gfx_mpu_info_array[osid].input_base_tgt1+gfx_mpu_info_array[osid].output_size_tgt1-1,
                                            gfx_mpu_info_array[osid].output_base_tgt1,
                                            gfx_mpu_info_array[osid].output_base_tgt1+gfx_mpu_info_array[osid].output_size_tgt1-1,
                                            gfx_mpu_info_array[osid].output_size_tgt1,
                                            gfx_mpu_info_array[osid].output_size_tgt1/1024/1024);
    }

    for(osid = 0; osid < 32; osid++){
        if(0==gfx_mpu_info_array[osid].output_size_tgt2) {
            continue;
        }
        
        VASTAI_PCI_INFO(priv, priv->pkg_id,"[GPU%d] osid[%d] rgn[2] fn_addr[0x%010llx ~ 0x%010llx] map to noc[0x%010llx ~ 0x%010llx] size=[0x%010llx=%06lldMB]\n",
                                            osid/8, osid % 8,gfx_mpu_info_array[osid].input_base_tgt2,
                                            gfx_mpu_info_array[osid].input_base_tgt2+gfx_mpu_info_array[osid].output_size_tgt2-1,
                                            gfx_mpu_info_array[osid].output_base_tgt2,
                                            gfx_mpu_info_array[osid].output_base_tgt2+gfx_mpu_info_array[osid].output_size_tgt2-1,
                                            gfx_mpu_info_array[osid].output_size_tgt2,
                                            gfx_mpu_info_array[osid].output_size_tgt2/1024/1024);
    }

    kfree(gfx_mpu_info_array);
    return;
}

static char* ddr_entry_name(struct vastai_pci_info *priv,u8 entry_num)
{
	switch(entry_num) {
		case VF_MANAGE: 
			return "[_vf_management__]";

		case META_FW: 
			return "[__meta_firmware_]";  

		case ENC_DEC_CMD_BUF:
			return "[enc_dec_cmd_buf_]";		 
		case AI_MODULE_OPCODE: 
			return "[ai_module_opcode]";

		case SMMU_CD_TABLE:
			return "[_smmu_cd_table__]";
 
		case RESERVED_DDR: 
			return "[__reserved_ddr__]";

		case RUNTIME: 
			return "[____runtime_____]";

		case GART: 
			return "[______gart______]";
		case GPU_HEAP: 
			return "[___gpu_heap_____]";

		case GPU_DISPLAY: 
            return "[___gpu_display__]";
		default:
		return "[___un_know______]";
	}
}

static void bubble_sort_ddr_entry(struct temp_ddr_entry *arry, int len) {
	int i, j;
	struct temp_ddr_entry temp;
	for (i = 0; i < len - 1; i++)
		for (j = 0; j < len - 1 - i; j++)
			if (arry[j].noc_addr > arry[j + 1].noc_addr) {
				//1:save arry[j] element to temp
                temp.entry_num      = arry[j].entry_num; 
				temp.noc_addr       = arry[j].noc_addr;
				temp.fn_addr        = arry[j].fn_addr;
				temp.cpu_pa         = arry[j].cpu_pa;
				temp.bar_offset     = arry[j].bar_offset;
				temp.size           = arry[j].size;

				//2:save arry[j+1] element to arry[j]
                arry[j].entry_num      = arry[j+1].entry_num; 
				arry[j].noc_addr       = arry[j+1].noc_addr;
				arry[j].fn_addr        = arry[j+1].fn_addr;
				arry[j].cpu_pa         = arry[j+1].cpu_pa;
				arry[j].bar_offset     = arry[j+1].bar_offset;
				arry[j].size           = arry[j+1].size;

				//3:save temp element to arry[j+1] 
                arry[j+1].entry_num      = temp.entry_num; 
				arry[j+1].noc_addr       = temp.noc_addr;
				arry[j+1].fn_addr        = temp.fn_addr;
				arry[j+1].cpu_pa         = temp.cpu_pa;
				arry[j+1].bar_offset     = temp.bar_offset;
				arry[j+1].size           = temp.size;
			}
	return;
}

void dump_ddr_info(struct vastai_pci_info *priv)
{
	int i = 0;
    struct temp_ddr_entry *temp_entry_arry =  kzalloc(sizeof(struct temp_ddr_entry) * DDR_ENTRY_MAX, GFP_KERNEL);

    for(i = 0; i < DDR_ENTRY_MAX;i++){
        temp_entry_arry[i].entry_num = i;
        temp_entry_arry[i].noc_addr  = priv->ddr_info.ddr_entry[i].noc_addr;
        temp_entry_arry[i].fn_addr   = priv->ddr_info.ddr_entry[i].fn_addr;
        temp_entry_arry[i].cpu_pa    = priv->ddr_info.ddr_entry[i].cpu_pa;
        temp_entry_arry[i].bar_offset= priv->ddr_info.ddr_entry[i].bar_offset;
        temp_entry_arry[i].size      = priv->ddr_info.ddr_entry[i].size;
    }

    bubble_sort_ddr_entry(temp_entry_arry,DDR_ENTRY_MAX);

	VASTAI_PCI_INFO(priv, priv->pkg_id,"=============================ddr_map==============================\n");
	for(i = 0; i < DDR_ENTRY_MAX;i++){
		if(0 == temp_entry_arry[i].size)
			continue;
		
		VASTAI_PCI_INFO(priv, priv->pkg_id,"ddr_rgn[%02d]%s fn_addr[0x%010llx~0x%010llx] map to noc [0x%010llx~0x%010llx] size[0x%010llx=%0lldMB]\n",
				temp_entry_arry[i].entry_num,
                ddr_entry_name(priv,temp_entry_arry[i].entry_num),
                temp_entry_arry[i].fn_addr,
				temp_entry_arry[i].fn_addr + temp_entry_arry[i].size -1,
				temp_entry_arry[i].noc_addr,
				temp_entry_arry[i].noc_addr + temp_entry_arry[i].size -1,
				temp_entry_arry[i].size,
				temp_entry_arry[i].size/1024/1024);
	}

	VASTAI_PCI_INFO(priv, priv->pkg_id,"=============================ddr_info==============================\n");
	VASTAI_PCI_INFO(priv, priv->pkg_id,"card_ddr_size = %lldGB \n",priv->ddr_info.card_ddr_size/1024/1024/1024);
	VASTAI_PCI_INFO(priv, priv->pkg_id,"fn_ddr_size   = %lldMB \n",priv->ddr_info.fn_ddr_size/1024/1024);
	VASTAI_PCI_INFO(priv, priv->pkg_id,"vf_ddr_size   = %lldMB \n",priv->ddr_info.vf_ddr_size/1024/1024);
	VASTAI_PCI_INFO(priv, priv->pkg_id,"cur_pf_vf_num = %lld \n",priv->ddr_info.cur_pf_vf_num);
	for(i = 0; i < DDR_ENTRY_MAX;i++){
		VASTAI_PCI_INFO(priv, priv->pkg_id,"ddr_rgn[%02d]fn_addr[0x%010llx] noc_addr[0x%010llx] bar_off[%010llx] size[0x%010llx] cpu_pa[0x%llx] \n",
				i,priv->ddr_info.ddr_entry[i].fn_addr,	
				priv->ddr_info.ddr_entry[i].noc_addr,
				priv->ddr_info.ddr_entry[i].bar_offset,
				priv->ddr_info.ddr_entry[i].size,
				priv->ddr_info.ddr_entry[i].cpu_pa);
	}

    kfree(temp_entry_arry);
	return;
}

void dump_msix_info(struct vastai_pci_info *priv)
{	
	int i;
	u32 len = 256;
	u64 msix_table_addr = 0;
	u64 msix_pba_addr = 0;
	void  *dump_addr;
	int ret=0;
	unsigned char print_str[64];

	dump_addr = kzalloc(len, GFP_KERNEL);

	if(0 == priv->priv_hw_cfg->sys_cfg.pfn){
		msix_table_addr = 0x1000;
		msix_pba_addr	= 0x1100;
	} else {
		msix_table_addr = 0;
		msix_pba_addr	= 0x100;
	}
	
	
	ret = vastai_pci_mmio_read(priv, VASTAI_PCI_BAR0, msix_table_addr, dump_addr,len );
	if(ret){
		VASTAI_PCI_INFO(priv, 0, "%s fail, %d,beacause mmio read fail,ret=%d\n",__func__,__LINE__,ret);
		return;
	}
	VASTAI_PCI_INFO(priv, 0, "%s, %d, dump msix table pfn=%d\n",__func__,__LINE__,priv->priv_hw_cfg->sys_cfg.pfn);
	for (i = 0; i < len; i += 16) {
		sprintf(print_str,
			"%s [pf%d msix_table] 0x%08llx: ",
			VASTAI_PCI_DEBUG_PREFIX,priv->priv_hw_cfg->sys_cfg.pfn,
			 (msix_table_addr + i));
			VASTAI_PCI_HEX_DUMP(KERN_WARNING, print_str,
					DUMP_PREFIX_NONE, 16, 4,
					dump_addr + i, 16, true);
	}

	ret = vastai_pci_mmio_read(priv, VASTAI_PCI_BAR0, msix_pba_addr, dump_addr,len );
	if(ret){
		VASTAI_PCI_INFO(priv, 0, "%s fail, %d,beacause mmio read fail,ret=%d\n",__func__,__LINE__,ret);
		return;
	}
	VASTAI_PCI_INFO(priv, 0, "%s, %d, dump msix pba pfn=%d\n",__func__,__LINE__,priv->priv_hw_cfg->sys_cfg.pfn);
	for (i = 0; i < len; i += 16) {
		sprintf(print_str,
			"%s [pf%d msix_pba] 0x%08llx: ",
			VASTAI_PCI_DEBUG_PREFIX,priv->priv_hw_cfg->sys_cfg.pfn,
			(msix_pba_addr + i));
			VASTAI_PCI_HEX_DUMP(KERN_WARNING, print_str,
					DUMP_PREFIX_NONE, 16, 4,
					dump_addr + i, 16, true);
	}

	kfree(dump_addr);
	return;
}

static char* ddr_partition_name(struct vastai_pci_info *priv,u8 partition_name)
{
	switch(partition_name) {
		/*firmware*/
		case SMCU_FOR_FW: 
			return "[_____smcu_for_fw______]";

		case CMCU_FOR_FW: 
			return "[_____cmcu_for_fw______]";  

		case OMCU0_FOR_FW:
			return "[_____omcu0_for_fw_____]";

		case OMCU1_FOR_FW: 
			return "[_____omcu1_for_fw_____]";

		case GMCU0_FOR_FW:
			return "[_____gmcu0_for_fw_____]";
 
		case GMCU1_FOR_FW: 
			return "[_____gmcu1_for_fw_____]";

		case GMCU2_FOR_FW: 
			return "[_____gmcu2_for_fw_____]";

		case GMCU3_FOR_FW: 
			return "[_____gmcu3_for_fw_____]";

		case VEMCU0_FOR_FW: 
            return "[____vemcu0_for_fw_____]";
		
		case VEMCU1_FOR_FW: 
            return "[____vemcu1_for_fw_____]";

		case VEMCU2_FOR_FW: 
            return "[____vemcu2_for_fw_____]";

		case VEMCU3_FOR_FW: 
            return "[____vemcu3_for_fw_____]";

		case ODSP0_FOR_FW: 
			return "[_____odsp0_for_fw_____]";

		case ODSP1_FOR_FW: 
			return "[_____odsp1_for_fw_____]";

		case VDSP0_FOR_FW: 
			return "[_____vdsp0_for_fw_____]";
		
		case VDSP1_FOR_FW: 
			return "[_____vdsp1_for_fw_____]";
		
		case ODSP_SHARE_DATA: 
			return "[___odsp_share_data____]";
		
		case PMCU_FOR_FW: 
			return "[_____pmcu_for_fw______]";
		
		case REVERSED: 
			return "[_______reversed_______]";
		
		/*encoder*/
		case VEMCU0_HEAP1_FOR_ENC: 
			return "[_vemcu0_heap1_for_enc_]";

		case VEMCU1_HEAP1_FOR_ENC: 
			return "[_vemcu1_heap1_for_enc_]";
		
		case VEMCU2_HEAP1_FOR_ENC: 
			return "[_vemcu2_heap1_for_enc_]";

		case VEMCU3_HEAP1_FOR_ENC: 
			return "[_vemcu3_heap1_for_enc_]";

		case VEMCU0_HEAP2_FOR_ENC: 
			return "[_vemcu0_heap2_for_enc_]";

		case VEMCU1_HEAP2_FOR_ENC: 
			return "[_vemcu1_heap2_for_enc_]";
		
		case VEMCU2_HEAP2_FOR_ENC: 
			return "[_vemcu2_heap2_for_enc_]";

		case VEMCU3_HEAP2_FOR_ENC: 
			return "[_vemcu3_heap2_for_enc_]";
		
		/*device local share*/
		case SMCU_FOR_LOCAL_SHARE: 
			return "[_smcu_for_local_share_]";

		case CMCU_FOR_LOCAL_SHARE: 
			return "[_cmcu_for_local_share_]";
		
		case OMCU0_FOR_LOCAL_SHARE: 
			return "[omcu0_for_local_share_]";

		case OMCU1_FOR_LOCAL_SHARE: 
			return "[omcu1_for_local_share_]";

		case GMCU0_FOR_LOCAL_SHARE: 
			return "[gmcu0_for_local_share_]";

		case GMCU1_FOR_LOCAL_SHARE: 
			return "[gmcu1_for_local_share_]";

		case GMCU2_FOR_LOCAL_SHARE: 
			return "[gmcu2_for_local_share_]";

		case GMCU3_FOR_LOCAL_SHARE: 
			return "[gmcu3_for_local_share_]";

		case VEMCU0_FOR_LOCAL_SHARE: 
			return "[vemcu0_for_local_share]";

		case VEMCU1_FOR_LOCAL_SHARE: 
			return "[vemcu1_for_local_share]";

		case VEMCU2_FOR_LOCAL_SHARE: 
			return "[vemcu2_for_local_share]";

		case VEMCU3_FOR_LOCAL_SHARE: 
			return "[vemcu3_for_local_share]";

		case ODSP0_FOR_LOCAL_SHARE: 
			return "[odsp0_for_local_share_]";

		case ODSP1_FOR_LOCAL_SHARE: 
			return "[odsp1_for_local_share_]";

		case VDSP0_FOR_LOCAL_SHARE: 
			return "[vdsp0_for_local_share_]";

		case VDSP1_FOR_LOCAL_SHARE: 
			return "[vdsp1_for_local_share_]";

		case PMCU_FOR_LOCAL_SHARE: 
			return "[_pmcu_for_local_share_]";

		case AI_OPCODE: 
			return "[______ai_opcode_______]";
		
		/*log buf*/
		case SMCU_LOG_BUF: 
			return "[_____smcu_log_buf_____]";

		case CMCU_LOG_BUF: 
			return "[_____cmcu_log_buf_____]";

		case OMCU0_LOG_BUF: 
			return "[____omcu0_log_buf_____]";

		case OMCU1_LOG_BUF: 
			return "[____omcu1_log_buf_____]";

		case GMCU0_LOG_BUF: 
			return "[____gmcu0_log_buf_____]";

		case GMCU1_LOG_BUF: 
			return "[____gmcu1_log_buf_____]";

		case GMCU2_LOG_BUF: 
			return "[____gmcu2_log_buf_____]";

		case GMCU3_LOG_BUF: 
			return "[____gmcu3_log_buf_____]";

		case VEMCU0_LOG_BUF: 
			return "[____vemcu0_log_buf____]";

		case VEMCU1_LOG_BUF: 
			return "[____vemcu1_log_buf____]";

		case VEMCU2_LOG_BUF: 
			return "[____vemcu2_log_buf____]";

		case VEMCU3_LOG_BUF: 
			return "[____vemcu3_log_buf____]";
		
		case ODSP0_LOG_BUF: 
			return "[____odsp0_log_buf_____]";

		case ODSP1_LOG_BUF: 
			return "[____odsp1_log_buf_____]";

		case VDSP0_LOG_BUF: 
			return "[____vdsp0_log_buf_____]";

		case VDSP1_LOG_BUF: 
			return "[____vdsp1_log_buf_____]";

		case PMCU_LOG_BUF: 
			return "[_____pmcu_log_buf_____]";
		
		case PROFILER_DATA: 
			return "[____profiler_data_____]";

		case DEC_VCMD_BUF: 
			return "[_____dec_vcmd_buf_____]";

		case RAW_FW_IMG: 
			return "[______raw_fw_img______]";

		case VDSP_OPERATOR_LOADER_BUF: 
			return "[_vdsp_operator_loader_]";

		case ENCRYPTED_FW_ON_BOOT: 
			return "[_encrypted_fw_on_boot_]";

		default:
			return "[_______un_know________]";
	}
}

static void init_ddr_partition(struct temp_ddr_partition ddr_partition[]){

	ddr_partition[SMCU_FOR_FW].noc_addr = SMCU_FOR_FW_ADDR;
	ddr_partition[SMCU_FOR_FW].size = DDR_PARTITION_SIZE_2MB;

	ddr_partition[CMCU_FOR_FW].noc_addr = CMCU_FOR_FW_ADDR;
	ddr_partition[CMCU_FOR_FW].size = DDR_PARTITION_SIZE_2MB;

	ddr_partition[OMCU0_FOR_FW].noc_addr = OMCU0_FOR_FW_ADDR;
	ddr_partition[OMCU0_FOR_FW].size = DDR_PARTITION_SIZE_2MB;

	ddr_partition[OMCU1_FOR_FW].noc_addr = OMCU1_FOR_FW_ADDR;
	ddr_partition[OMCU1_FOR_FW].size = DDR_PARTITION_SIZE_2MB;

	ddr_partition[GMCU0_FOR_FW].noc_addr = GMCU0_FOR_FW_ADDR;
	ddr_partition[GMCU0_FOR_FW].size = DDR_PARTITION_SIZE_4MB;

	ddr_partition[GMCU1_FOR_FW].noc_addr = GMCU1_FOR_FW_ADDR;
	ddr_partition[GMCU1_FOR_FW].size = DDR_PARTITION_SIZE_4MB;

	ddr_partition[GMCU2_FOR_FW].noc_addr = GMCU2_FOR_FW_ADDR;
	ddr_partition[GMCU2_FOR_FW].size = DDR_PARTITION_SIZE_4MB;

	ddr_partition[GMCU3_FOR_FW].noc_addr = GMCU3_FOR_FW_ADDR;
	ddr_partition[GMCU3_FOR_FW].size = DDR_PARTITION_SIZE_4MB;

	ddr_partition[VEMCU0_FOR_FW].noc_addr = VEMCU0_FOR_FW_ADDR;
	ddr_partition[VEMCU0_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[VEMCU1_FOR_FW].noc_addr = VEMCU1_FOR_FW_ADDR;
	ddr_partition[VEMCU1_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[VEMCU2_FOR_FW].noc_addr = VEMCU2_FOR_FW_ADDR;
	ddr_partition[VEMCU2_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[VEMCU3_FOR_FW].noc_addr = VEMCU3_FOR_FW_ADDR;
	ddr_partition[VEMCU3_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[ODSP0_FOR_FW].noc_addr = ODSP0_FOR_FW_ADDR;
	ddr_partition[ODSP0_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[ODSP1_FOR_FW].noc_addr = ODSP1_FOR_FW_ADDR;
	ddr_partition[ODSP1_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[VDSP0_FOR_FW].noc_addr = VDSP0_FOR_FW_ADDR;
	ddr_partition[VDSP0_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[VDSP1_FOR_FW].noc_addr = VDSP1_FOR_FW_ADDR;
	ddr_partition[VDSP1_FOR_FW].size = DDR_PARTITION_SIZE_8MB;

	ddr_partition[ODSP_SHARE_DATA].noc_addr = ODSP_SHARE_DATA_ADDR;
	ddr_partition[ODSP_SHARE_DATA].size = DDR_PARTITION_SIZE_16MB;

	ddr_partition[PMCU_FOR_FW].noc_addr = PMCU_FOR_FW_ADDR;
	ddr_partition[PMCU_FOR_FW].size = DDR_PARTITION_SIZE_2MB;

	ddr_partition[REVERSED].noc_addr = REVERSED_ADDR;
	ddr_partition[REVERSED].size = DDR_PARTITION_SIZE_22MB;

	ddr_partition[VEMCU0_HEAP1_FOR_ENC].noc_addr = VEMCU0_HEAP1_FOR_ENC_ADDR;
	ddr_partition[VEMCU0_HEAP1_FOR_ENC].size = DDR_PARTITION_SIZE_128MB;

	ddr_partition[VEMCU1_HEAP1_FOR_ENC].noc_addr = VEMCU1_HEAP1_FOR_ENC_ADDR;
	ddr_partition[VEMCU1_HEAP1_FOR_ENC].size = DDR_PARTITION_SIZE_128MB;

	ddr_partition[VEMCU2_HEAP1_FOR_ENC].noc_addr = VEMCU2_HEAP1_FOR_ENC_ADDR;
	ddr_partition[VEMCU2_HEAP1_FOR_ENC].size = DDR_PARTITION_SIZE_128MB;

	ddr_partition[VEMCU3_HEAP1_FOR_ENC].noc_addr = VEMCU3_HEAP1_FOR_ENC_ADDR;
	ddr_partition[VEMCU3_HEAP1_FOR_ENC].size = DDR_PARTITION_SIZE_128MB;

	ddr_partition[VEMCU0_HEAP2_FOR_ENC].noc_addr = VEMCU0_HEAP2_FOR_ENC_ADDR;
	ddr_partition[VEMCU0_HEAP2_FOR_ENC].size = DDR_PARTITION_SIZE_32MB;

	ddr_partition[VEMCU1_HEAP2_FOR_ENC].noc_addr = VEMCU1_HEAP2_FOR_ENC_ADDR;
	ddr_partition[VEMCU1_HEAP2_FOR_ENC].size = DDR_PARTITION_SIZE_32MB;

	ddr_partition[VEMCU2_HEAP2_FOR_ENC].noc_addr = VEMCU2_HEAP2_FOR_ENC_ADDR;
	ddr_partition[VEMCU2_HEAP2_FOR_ENC].size = DDR_PARTITION_SIZE_32MB;

	ddr_partition[VEMCU3_HEAP2_FOR_ENC].noc_addr = VEMCU3_HEAP2_FOR_ENC_ADDR;
	ddr_partition[VEMCU3_HEAP2_FOR_ENC].size = DDR_PARTITION_SIZE_32MB;

	ddr_partition[SMCU_FOR_LOCAL_SHARE].noc_addr = SMCU_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[SMCU_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[CMCU_FOR_LOCAL_SHARE].noc_addr = CMCU_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[CMCU_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[OMCU0_FOR_LOCAL_SHARE].noc_addr = OMCU0_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[OMCU0_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[OMCU1_FOR_LOCAL_SHARE].noc_addr = OMCU1_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[OMCU1_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU0_FOR_LOCAL_SHARE].noc_addr = GMCU0_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[GMCU0_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU1_FOR_LOCAL_SHARE].noc_addr = GMCU1_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[GMCU1_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU2_FOR_LOCAL_SHARE].noc_addr = GMCU2_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[GMCU2_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU3_FOR_LOCAL_SHARE].noc_addr = GMCU3_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[GMCU3_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU0_FOR_LOCAL_SHARE].noc_addr = VEMCU0_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VEMCU0_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU1_FOR_LOCAL_SHARE].noc_addr = VEMCU1_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VEMCU1_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU2_FOR_LOCAL_SHARE].noc_addr = VEMCU2_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VEMCU2_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU3_FOR_LOCAL_SHARE].noc_addr = VEMCU3_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VEMCU3_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[ODSP0_FOR_LOCAL_SHARE].noc_addr = ODSP0_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[ODSP0_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[ODSP1_FOR_LOCAL_SHARE].noc_addr = ODSP1_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[ODSP1_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VDSP0_FOR_LOCAL_SHARE].noc_addr = VDSP0_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VDSP0_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VDSP1_FOR_LOCAL_SHARE].noc_addr = VDSP1_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[VDSP1_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[PMCU_FOR_LOCAL_SHARE].noc_addr = PMCU_FOR_LOCAL_SHARE_ADDR;
	ddr_partition[PMCU_FOR_LOCAL_SHARE].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[AI_OPCODE].noc_addr = AI_OPCODE_ADDR;
	ddr_partition[AI_OPCODE].size = DDR_PARTITION_SIZE_15MB;

	ddr_partition[SMCU_LOG_BUF].noc_addr = SMCU_LOG_BUF_ADDR;
	ddr_partition[SMCU_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[CMCU_LOG_BUF].noc_addr = CMCU_LOG_BUF_ADDR;
	ddr_partition[CMCU_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[OMCU0_LOG_BUF].noc_addr = OMCU0_LOG_BUF_ADDR;
	ddr_partition[OMCU0_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[OMCU1_LOG_BUF].noc_addr = OMCU1_LOG_BUF_ADDR;
	ddr_partition[OMCU1_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU0_LOG_BUF].noc_addr = GMCU0_LOG_BUF_ADDR;
	ddr_partition[GMCU0_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU1_LOG_BUF].noc_addr = GMCU1_LOG_BUF_ADDR;
	ddr_partition[GMCU1_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU2_LOG_BUF].noc_addr = GMCU2_LOG_BUF_ADDR;
	ddr_partition[GMCU2_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[GMCU3_LOG_BUF].noc_addr = GMCU3_LOG_BUF_ADDR;
	ddr_partition[GMCU3_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU0_LOG_BUF].noc_addr = VEMCU0_LOG_BUF_ADDR;
	ddr_partition[VEMCU0_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU1_LOG_BUF].noc_addr = VEMCU1_LOG_BUF_ADDR;
	ddr_partition[VEMCU1_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU2_LOG_BUF].noc_addr = VEMCU2_LOG_BUF_ADDR;
	ddr_partition[VEMCU2_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VEMCU3_LOG_BUF].noc_addr = VEMCU3_LOG_BUF_ADDR;
	ddr_partition[VEMCU3_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[ODSP0_LOG_BUF].noc_addr = ODSP0_LOG_BUF_ADDR;
	ddr_partition[ODSP0_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[ODSP1_LOG_BUF].noc_addr = ODSP1_LOG_BUF_ADDR;
	ddr_partition[ODSP1_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VDSP0_LOG_BUF].noc_addr = VDSP0_LOG_BUF_ADDR;
	ddr_partition[VDSP0_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[VDSP1_LOG_BUF].noc_addr = VDSP1_LOG_BUF_ADDR;
	ddr_partition[VDSP1_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[PMCU_LOG_BUF].noc_addr = PMCU_LOG_BUF_ADDR;
	ddr_partition[PMCU_LOG_BUF].size = DDR_PARTITION_SIZE_1MB;

	ddr_partition[PROFILER_DATA].noc_addr = PROFILER_DATA_ADDR;
	ddr_partition[PROFILER_DATA].size = DDR_PARTITION_SIZE_34MB;

	ddr_partition[DEC_VCMD_BUF].noc_addr = DEC_VCMD_BUF_ADDR;
	ddr_partition[DEC_VCMD_BUF].size = DDR_PARTITION_SIZE_29MB;

	ddr_partition[RAW_FW_IMG].noc_addr = RAW_FW_IMG_ADDR;
	ddr_partition[RAW_FW_IMG].size = DDR_PARTITION_SIZE_16MB;

	ddr_partition[VDSP_OPERATOR_LOADER_BUF].noc_addr = VDSP_OPERATOR_LOADER_BUF_ADDR;
	ddr_partition[VDSP_OPERATOR_LOADER_BUF].size = DDR_PARTITION_SIZE_112MB;

	ddr_partition[ENCRYPTED_FW_ON_BOOT].noc_addr = ENCRYPTED_FW_ON_BOOT_ADDR;
	ddr_partition[ENCRYPTED_FW_ON_BOOT].size = DDR_PARTITION_SIZE_16MB;

	return;
}

static void bubble_sort_ddr_partition(struct temp_ddr_partition *arry, int len) {
	int i, j;
	struct temp_ddr_partition temp;
	for (i = 0; i < len - 1; i++)
		for (j = 0; j < len - 1 - i; j++)
			if (arry[j].noc_addr > arry[j + 1].noc_addr) {
				//1:save arry[j] element to temp
                temp.partition_num  = arry[j].partition_num; 
				temp.noc_addr       = arry[j].noc_addr;
				temp.size           = arry[j].size;

				//2:save arry[j+1] element to arry[j]
                arry[j].partition_num  = arry[j+1].partition_num; 
				arry[j].noc_addr       = arry[j+1].noc_addr;
				arry[j].size           = arry[j+1].size;

				//3:save temp element to arry[j+1] 
                arry[j+1].partition_num  = temp.partition_num; 
				arry[j+1].noc_addr       = temp.noc_addr;
				arry[j+1].size           = temp.size;
			}
	return;
}

void ddr_partition_info(struct vastai_pci_info *priv)
{
	int i = 0;

	struct temp_ddr_partition *temp_partition_arry =  kzalloc(sizeof(struct temp_ddr_partition) * DDR_PARTITION_MAX, GFP_KERNEL);
	
	init_ddr_partition(temp_partition_arry);

	for(i = 0; i < DDR_PARTITION_MAX;i++){
        temp_partition_arry[i].partition_num = i;
    }

	bubble_sort_ddr_partition(temp_partition_arry,DDR_PARTITION_MAX);

	VASTAI_PCI_INFO(priv, priv->pkg_id,"==============================ddr_partition==============================\n");
	for(i=0;i<DDR_PARTITION_MAX;i++){
		VASTAI_PCI_INFO(priv, priv->pkg_id,"ddr_rgn[%02d]%s noc_addr[0x%010llx~0x%010llx] size[0x%010llx=%0lldMB] \n",
		temp_partition_arry[i].partition_num,
        ddr_partition_name(priv,temp_partition_arry[i].partition_num),
		temp_partition_arry[i].noc_addr,
		temp_partition_arry[i].noc_addr + temp_partition_arry[i].size -1,
		temp_partition_arry[i].size,
		temp_partition_arry[i].size/1024/1024);
	}

	kfree(temp_partition_arry);
	return;
}