#include "vastai_pci.h"
#include "../va_irq.h"
#include "hw_config.h"
#include "vastai_pcie_public.h"
#include "hw_queue.h"
#include "vastai_pci.h"
#include "vastai_cmd.h"
#include <linux/stddef.h>
#include <linux/string.h>
#include "vastai_ai_info.h"


void init_ai_info(struct vastai_pci_info *priv)
{
    struct  ring_buf_entry *hw_entry;	
    struct  ai_info      *ai_entry;
    int i;

    u64 ddr_size = ((u64)priv->priv_hw_cfg->header.hi32_ddr_size<<32)|
                            priv->priv_hw_cfg->header.lo32_ddr_size;

    u8 config_bar4_5_sz = priv->priv_hw_cfg->sys_cfg.bar4_5_cfg_size;

	hw_entry = find_hw_cfg_acord_name(priv,COMMON_AI_PIPE_LINE_INFO_NAME);
    if( NULL == hw_entry ) {
		VASTAI_PCI_INFO(priv, priv->pkg_id, "COMMON_AI_PIPE_LINE_INFO_NAME buf init fail\n");
        return;
	} 
    
    ai_entry = kzalloc(sizeof(struct ai_info), GFP_KERNEL);
    ai_entry->ai_csram_buf_base         = hw_entry->ring_buf_base;
    ai_entry->ai_csram_buf_size         = hw_entry->ring_buf_size;
    ai_entry->fn_ddr_total_sz           = ddr_size;
    ai_entry->ddr_total_sz              = (u64)priv->priv_hw_cfg->sys_cfg.sum_ddr_sz*1024*1024*1024;
    if(priv->is_pf) {
        ai_entry->decoder_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_AI_DECODER_OFFSET;
        ai_entry->decoder_buf_host_phy_addr = priv->bar[VASTAI_PCI_BAR4].mmio_start+PF_AI_DECODER_OFFSET;
        ai_entry->decoder_buf_size          = AI_DECODER_SIZE_16M;

        ai_entry->ai_mod_buf_fn_addr        = VA_DDR_BASE_ADDR+PF_AI_MODULE_OFFSET;
        ai_entry->ai_mod_buf_host_phy_addr  = priv->bar[VASTAI_PCI_BAR4].mmio_start+PF_AI_MODULE_OFFSET;
        ai_entry->ai_mod_buf_size           = AI_MODULE_SIZE_16M;

        if(ZHAOGE_VG600 == priv->priv_hw_cfg->sys_cfg.card_type) {
            ai_entry->ai_dyn_buf_fn_addr        = VA_DDR_BASE_ADDR + PF_USED_SIZE;
            ai_entry->ai_dyn_buf_size           = ddr_size-PF_USED_SIZE;
            goto end;
        }

        switch (config_bar4_5_sz) {
            case HWCFG_BAR4_5_2GB:                
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_DYNAMIC_BUF_OFFSET;
                ai_entry->ai_dyn_buf_size          = ddr_size - BAR_SIZE_2G;
                break;
            case HWCFG_BAR4_5_4GB:
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_USED_SIZE;
                ai_entry->ai_dyn_buf_size          = LARGE_BAR_4GB_PER_PF_RUN_TIME_SIZE;
                break;
            case HWCFG_BAR4_5_8GB:
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_USED_SIZE;
                ai_entry->ai_dyn_buf_size          = LARGE_BAR_8GB_PER_PF_RUN_TIME_SIZE;
                break;
            case HWCFG_BAR4_5_16GB:
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_USED_SIZE;
                ai_entry->ai_dyn_buf_size          = LARGE_BAR_16GB_PER_PF_RUN_TIME_SIZE;
                break;
            case HWCFG_BAR4_5_32GB:
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_USED_SIZE;
                ai_entry->ai_dyn_buf_size          = LARGE_BAR_32GB_PER_PF_RUN_TIME_SIZE;
                break;
            case HWCFG_BAR4_5_64GB:
                ai_entry->ai_dyn_buf_fn_addr       = VA_DDR_BASE_ADDR+PF_USED_SIZE;
                ai_entry->ai_dyn_buf_size          = LARGE_BAR_64GB_PER_PF_RUN_TIME_SIZE;
                break;
            
            default:
                break;
        }

	} else if(priv->is_vf) {
        if(DECODER_SRIOV_ENABLE == priv->priv_hw_cfg->sys_cfg.decoder_sriov_en){
            //ai_entry->decoder_buf_fn_addr       = VA_DDR_BASE_ADDR+VF_AI_DECODER_OFFSET;
            ai_entry->decoder_buf_fn_addr       = ((u64)priv->priv_hw_cfg->header.hi32_video_buf_base<<32)|priv->priv_hw_cfg->header.lo32_video_buf_base;
            ai_entry->decoder_buf_host_phy_addr = priv->bar[VASTAI_PCI_BAR4].mmio_start+VF_AI_DECODER_OFFSET;
            ai_entry->decoder_buf_size          = VEDIO_CMD_BUF_SIZE;
        } else {
            ai_entry->decoder_buf_fn_addr       = VA_DDR_BASE_ADDR+VF_AI_DECODER_OFFSET;
            ai_entry->decoder_buf_host_phy_addr = priv->bar[VASTAI_PCI_BAR4].mmio_start+VF_AI_DECODER_OFFSET;
            ai_entry->decoder_buf_size          = AI_DECODER_SIZE_16M;
        }

        ai_entry->ai_mod_buf_fn_addr        = VA_DDR_BASE_ADDR+VF_AI_MODULE_OFFSET;
        ai_entry->ai_mod_buf_host_phy_addr  = priv->bar[VASTAI_PCI_BAR4].mmio_start+VF_AI_MODULE_OFFSET;
        ai_entry->ai_mod_buf_size           = AI_MODULE_SIZE_16M;

        /*VG600 is not support GPU*/
        if(ZHAOGE_VG600 == priv->priv_hw_cfg->sys_cfg.card_type){
            ai_entry->ai_dyn_buf_fn_addr        = VA_DDR_BASE_ADDR + VF_USED_SIZE;
            ai_entry->ai_dyn_buf_size           = ddr_size-VF_USED_SIZE;
            goto end;
        }

        switch (config_bar4_5_sz) {
            case HWCFG_BAR4_5_256MB:
                ai_entry->ai_dyn_buf_fn_addr        = VA_DDR_BASE_ADDR+VF_DYNAMIC_BUF_OFFSET;
                ai_entry->ai_dyn_buf_size           = ddr_size-VF_DDR_MMIO_SIZE;
                break;

            case HWCFG_BAR4_5_512MB:
                ai_entry->ai_dyn_buf_fn_addr        = VA_DDR_BASE_ADDR+VF_LARGE_BAR_512MB_DDR_SIZE;
                ai_entry->ai_dyn_buf_size           = ddr_size-VF_LARGE_BAR_512MB_DDR_SIZE;
                break;

            case HWCFG_BAR4_5_1GB:
                ai_entry->ai_dyn_buf_fn_addr        = VA_DDR_BASE_ADDR + (VF_USED_SIZE +VF_LARGE_BAR_1GB_DISP_HEAP_SIZE + VF_LARGE_BAR_1GB_GFX_HEAP_SIZE);
                ai_entry->ai_dyn_buf_size           = ddr_size-(VF_USED_SIZE +VF_LARGE_BAR_1GB_DISP_HEAP_SIZE + VF_LARGE_BAR_1GB_GFX_HEAP_SIZE);
                break;

            default:
                VASTAI_PCI_ERR(priv, priv->pkg_id,"[runtime] VF not support bar size now!!!\n");
                break;
        }

        
	}

end:

    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] decoder_buf_fn_addr      = 0x%llx \n",ai_entry->decoder_buf_fn_addr);
    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] decoder_buf_size         = 0x%x(%dMB) \n",ai_entry->decoder_buf_size,
                                                                ai_entry->decoder_buf_size/1024/1024);

    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] ai_mod_buf_fn_addr       = 0x%llx \n",ai_entry->ai_mod_buf_fn_addr);
    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] ai_mod_buf_size          = 0x%x(%dMB)\n",ai_entry->ai_mod_buf_size,
                                                                ai_entry->ai_mod_buf_size/1024/1024);

    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] ai_dynamic_buf_fn_addr = 0x%llx \n",ai_entry->ai_dyn_buf_fn_addr);
    VASTAI_PCI_INFO(priv, priv->pkg_id,"[runtime] ai_dynamic_buf_size    = 0x%llx(%lldMB) \n",ai_entry->ai_dyn_buf_size,
                                                            ai_entry->ai_dyn_buf_size/1024/1024);

    memcpy(&ai_entry->dsp_cfg, &priv->priv_hw_cfg->dsp_cfg, sizeof(struct  dsp_cfg));
    memcpy(&ai_entry->ai_cfg,   &priv->priv_hw_cfg->ai_cfg,   sizeof(struct  ai_cfg));

    VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime] vdsp_num   =%d\n",ai_entry->dsp_cfg.vdsp_num);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime] dsp_entry =%d\n",ai_entry->dsp_cfg.valid_vdsp_entry_n);

    for(i=0;i<ai_entry->dsp_cfg.valid_vdsp_entry_n;i++) {
        VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime] dsp_entry[%d]  dsp_addr[0x%08x] noc_addr[0x%010llx] size[%02dMB]\n",
                                                    i, ai_entry->dsp_cfg.dsp_entry[i].dsp_addr,
                                                    ((u64)ai_entry->dsp_cfg.dsp_entry[i].hi32_soc_addr<<32)|
                                                    ai_entry->dsp_cfg.dsp_entry[i].lo32_soc_addr,
                                                    ai_entry->dsp_cfg.dsp_entry[i].size);
    }

    VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime] odsp_entry =%d\n",ai_entry->dsp_cfg.valid_odsp_entry_n);
    for(i=ai_entry->dsp_cfg.valid_1st_odsp_entry;
        i<(ai_entry->dsp_cfg.valid_1st_odsp_entry + ai_entry->dsp_cfg.valid_odsp_entry_n);i++) {
        VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime] dsp_entry[%d]  dsp_addr[0x%08x] noc_addr[0x%010llx] size[%02dMB]\n",
                                                    i, ai_entry->dsp_cfg.dsp_entry[i].dsp_addr,
                                                    ((u64)ai_entry->dsp_cfg.dsp_entry[i].hi32_soc_addr<<32)|
                                                    ai_entry->dsp_cfg.dsp_entry[i].lo32_soc_addr,
                                                    ai_entry->dsp_cfg.dsp_entry[i].size);
    }

    VASTAI_PCI_INFO(priv, priv->pkg_id, "[runtime][AI] mcu_addr[0x%x] map to noc_addr[0x%010llx]\n",
                                ai_entry->ai_cfg.mcu_addr,
                                ((u64)ai_entry->ai_cfg.hi32_soc_addr<<32)|ai_entry->ai_cfg.lo32_soc_addr);

    priv->vastai_ai_info = ai_entry;
    
    return;
}

void free_ai_info(struct vastai_pci_info *priv)
{
    if(priv->vastai_ai_info != NULL){
        kfree(priv->vastai_ai_info);
    }
    return;
}
