#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/list.h>
#include <linux/printk.h>
#include <linux/slab.h>
#include <linux/completion.h>
#include <linux/sched/signal.h>
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,11,0)
#include <linux/signal.h>
#else
#include <linux/sched/signal.h>
#endif
#include <linux/uaccess.h>
#include <linux/workqueue.h>
#include <linux/pid_namespace.h>
#include <linux/compat.h>
#include "vastai_common.h"
#include "hw_config.h"


static void vastai_common_vdsp_call_back(void *arg);
static void vastai_1pf_vdsp0_call_back(void *arg);
static void vastai_1pf_vdsp1_call_back(void *arg);
static void vastai_1pf_odsp0_call_back(void *arg);
static void vastai_1pf_odsp1_call_back(void *arg);
static void vastai_2pf_1st_vdsp_call_back(void *arg);
static void vastai_2pf_2nd_vdsp_call_back(void *arg);


void vastai_cmcu_call_back(void *arg)
{
	int ret = 0;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)arg;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)priv->vacc_dev;
	struct out_desc out;
	int index = 0;

	switch (priv->fn_mode) {
		case ONLY_1_PF:
			index = ONLY_1PF_CMCU_TO_HOST_INFO_BUF;
			break;
		
		case ONLY_2_PF:
			index = ONLY_2PF_CMCU_TO_HOST_INFO_BUF;
			break;

		case ONLY_4_PF:
		case PF_4_WITH_SRIOV:
			index = COMMON_CMCU_TO_HOST_INFO_BUF;
            break;
		default:
			VASTAI_PCI_ERR(va_dev->pci_info, va_dev->die_id,
				"cmcu_call_back: invalid fn_mode = %d.\n",
				priv->fn_mode);
	    break;
	}
	
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}

	return;
}

static void vastai_common_vdsp_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = COMMON_VDSP_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;
}

static void vastai_1pf_vdsp0_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_1PF_VDSP0_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;
}

static void vastai_1pf_vdsp1_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_1PF_VDSP1_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;
}

static void vastai_1pf_odsp0_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_1PF_ODSP0_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;
}

static void vastai_1pf_odsp1_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_1PF_ODSP1_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;
}

static void vastai_2pf_1st_vdsp_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;    
}

static void vastai_2pf_2nd_vdsp_call_back(void *arg)
{
	int ret;
	struct out_desc out;
	struct vastai_cdev *va_dev = (struct vastai_cdev *)arg;
	struct vastai_pci_info *priv = (struct vastai_pci_info *)va_dev->pci_info;
	int index = ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF;
	while ((ret = vastai_pci_receive_msg(priv, 0, index, &out)) == 0) {
		vaccrt_ai_trans_call(priv, 0, index, (void *)va_dev, (void *)&out);
	}
	return;     
}

int vaccrt_ai_callback_reg(void *pcie_dev, struct vastai_cdev *va_dev)
{
	int ret;
    u8 fn_mode = va_dev->pci_info->fn_mode;
    switch (fn_mode) {
        case ONLY_1_PF:
            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_VDSP_0_VECTOR,
                     vastai_1pf_vdsp0_call_back,va_dev);
            
            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_VDSP_1_VECTOR,
                     vastai_1pf_vdsp1_call_back,va_dev);

            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_ODSP_0_VECTOR,
                     vastai_1pf_odsp0_call_back,va_dev);
            
            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_ODSP_1_VECTOR,
                     vastai_1pf_odsp1_call_back,va_dev);
            break;
        
        case ONLY_2_PF:
            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_2_PF_MSIX_VDSP_0_VECTOR,
                     vastai_2pf_1st_vdsp_call_back,va_dev);
            
            ret = va_register_pcie_interrupt(va_dev->pci_info, ONLY_2_PF_MSIX_VDSP_1_VECTOR,
                     vastai_2pf_2nd_vdsp_call_back,va_dev);
            break;
        
        case ONLY_4_PF:        
        case PF_4_WITH_SRIOV:
            ret = va_register_pcie_interrupt(va_dev->pci_info, COMMON_MSIX_VDSP_VECTOR,
                     vastai_common_vdsp_call_back,va_dev);
            break;
        
        default:
            ret = -1;
            break;
    }
	
    return ret;
}

int vaccrt_ai_callback_unreg(void *pcie_dev, struct vastai_cdev *va_dev)
{
	int ret = 0;
	u8 fn_mode = va_dev->pci_info->fn_mode;

    switch (fn_mode) {
        case ONLY_1_PF:
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_VDSP_0_VECTOR);
            
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_VDSP_1_VECTOR);

            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_ODSP_0_VECTOR);
            
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_1_PF_MSIX_ODSP_1_VECTOR);
            break;
        
        case ONLY_2_PF:
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_2_PF_MSIX_VDSP_0_VECTOR);
            
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, ONLY_2_PF_MSIX_VDSP_1_VECTOR);
            break;
        
        case ONLY_4_PF:        
        case PF_4_WITH_SRIOV:
            ret = va_unregister_pcie_interrupt(va_dev->pci_info, COMMON_MSIX_VDSP_VECTOR);
            break;
        
        default:
            ret = -1;
            break;
    }
	
    return ret;
}