#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <linux/timer.h>
#include "vastai_pci.h"
#include "sg100_cmd.h"
#include "hw_config.h"
#include "vastai_state.h"

int vastai_flr_gmcu_task(struct vastai_pci_info *priv)
{
#if 0
	int ret;
	struct pcie_transfer_cmd trans = {0};

	trans.w0.s_data0.optcode = GMCU_FLR_START;
	if (ONLY_1_PF == priv->fn_mode) {
		ret = vastai_pci_send_msg(priv, 0, ONLY_1PF_HOST_TO_GMCU1_CMD_BUF,  &trans, 0);
	} else if (ONLY_2_PF == priv->fn_mode){
		ret = vastai_pci_send_msg(priv, 0, ONLY_2PF_HOST_TO_GMCU_CMD_BUF,  &trans, 0);
	} else {
		ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_GMCU_CMD_BUF,  &trans, 0);
	}
	if (ret) 
		VASTAI_PCI_ERR(priv, priv->pkg_id, "flr_gmcu_task:push element fail\n");

    /*wait gmcu flr done*/
#endif
	return 0;

}

static int vastai_flr_smcu_prepare(struct vastai_pci_info *priv)
{
    int ret = 0;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};
	u32 smcu_check_val = (SMCU_FLR_START <<16) | (priv->priv_hw_cfg->sys_cfg.pfn << 8) | SMCU_TASK_RET;

	smcu_task_desc.w0_data0 = SMCU_FLR_START;
	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret)
		return ret;

	/*check smcu return data whether correcct*/
	if(smcu_check_val != smcu_ret.w0_data0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"smcu_check_val[0x%x] != smcu_return_addr[0x%x]\n",
					smcu_check_val,smcu_ret.w0_data0);
		return -EIO;
	}

	return ret;
}

static int vastai_flr_smcu_done(struct vastai_pci_info *priv)
{
    /* wait completion int from smcu */
	if (wait_for_completion_timeout(&(priv->smcu_flr_done),
        msecs_to_jiffies(VASTAI_PCIE_WAIT_SMCU_MSG1_TIME)) <= 0) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "flr_smcu_done wait for completion timeout, failed\n");
		return -ETIMEDOUT;
	}

	reinit_completion(&(priv->smcu_flr_done));

    return 0;
}

// FLR: initiate a PCIe function level reset
int vastai_pci_flr(void *pci_info)
{
    int ret = 0;
    struct vastai_pci_info *priv = (struct vastai_pci_info *)pci_info;
    VASTAI_PCI_INFO(priv, priv->pkg_id, "pci_flr\n");
    vastai_flr_smcu_prepare(priv);
    vastai_flr_gmcu_task(priv);
    vastai_pci_state_reset_start(priv);
#if KERNEL_VERSION(4, 20, 0) <= LINUX_VERSION_CODE
    // ret = pcie_has_flr(priv->dev);
    // if(!ret) {
    //     VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "WARNING: pci_flr() devfn=%d no support for FLR. imm_ready=%d\n", priv->dev->devfn, priv->dev->imm_ready);
    //     return -1;
    // }
    // VASTAI_PCI_DBG(priv, DUMMY_DIE_ID, "pci_flr() Current devfn=%d supports FLR.\n", priv->dev->devfn);
    
    // Set it to be not ready state. TBD
    priv->dev->imm_ready = 0;
    
    pci_save_state(priv->dev);
	pci_cfg_access_lock(priv->dev);
    
    VASTAI_PCI_INFO(priv, priv->pkg_id, "pci_flr() to save state, then to do flr\n");
    ret = pcie_flr(priv->dev);
    if(ret < 0) {
        VASTAI_PCI_ERR(priv, priv->pkg_id, "pcie_flr fail, ret=%d", ret);
    }
    
	pci_cfg_access_unlock(priv->dev);
    
    VASTAI_PCI_INFO(priv, priv->pkg_id, "pci_flr() to pci_restore_state\n");
	pci_restore_state(priv->dev);
#else
    // TBD, need detect whether the dev support FLR.
    pcie_flr(priv->dev);
#endif
    vastai_pci_state_boot_done(priv);
    vastai_flr_smcu_done(priv);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "pci_flr() End of pci_restore_state\n");
    
    return ret;
}
EXPORT_SYMBOL(vastai_pci_flr);
