#include <linux/aer.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include "vastai_state.h"
#include "vastai_pci.h"
#include "config.h"
#include "sg100_pcie_wrap.h"
#include "vastai_ras.h"

#if VASTAI_HEARTBEAT_ENABLE
static void vastai_heartbeat_work(struct work_struct *work)
{
	struct heartbeat_monitor *heartbeat = NULL;
	struct vastai_pci_info *priv = NULL;
	u32 heart_beat_cnt[HEARTBEAT_CNT_REG_NUM] = { 0 };
	u32 *last_count = NULL;
	u32 tmp_count = 0;
	int ret = 0, core_id = 0;
	u32 pcie_read_test = 0;

	priv = container_of(work, struct vastai_pci_info, heartbeat.heartbeat_work.work);
	heartbeat = &priv->heartbeat;

	last_count = heartbeat->last_counts;

	/* case of soc reset */
	if (VASTAI_RESET_STATE == vastai_get_pci_state(priv)) {
		heartbeat->cores_status = 0;
	}
	if (VASTAI_NORMAL_STATE != vastai_get_pci_state(priv)) {
		goto skip;
	}
	
	ret = vastai_pci_mem_read(priv, 0, HEART_BEAT_BASE, heart_beat_cnt, sizeof(heart_beat_cnt));
	if (ret) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "heartbeat_work: read heartbeat reg error, %d\n", ret);
		//TODO: this moment pcie error, how to reset dev
	}

	for (core_id = 0; core_id < HEARTBEAT_CNT_REG_NUM; core_id++) {
		if (priv->heartbeat_mask & BIT_MASK(core_id)) {
			continue;
		}

#if 0
		VASTAI_PCI_DBG(priv, DUMMY_DIE_ID,
			"heartbeat_work: core[%d] heartbeat cnt, last=%u cur=%u\n",
			core_id, last_count[core_id], heart_beat_cnt[core_id]);
#endif

		if (heart_beat_cnt[core_id] != last_count[core_id]) {
			continue;
		}

		/* heartbeat counter don't change, read again */
		msleep(3000);
		if (VASTAI_NORMAL_STATE != vastai_get_pci_state(priv)) {
			goto skip;
		}
		ret = vastai_pci_mem_read(priv, 0, HEART_BEAT_BASE + core_id * sizeof(u32), &tmp_count, sizeof(u32));
		if (ret) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "heartbeat_work: read heartbeat reg error, %d\n", ret);
			//TODO: this moment pcie error, how to reset dev
		}
		if (tmp_count != last_count[core_id]) {
			continue;
		}

		/* check pcie link */
		ret = vastai_pci_mem_fread(priv, CSRAM_BASE_ADDR, &pcie_read_test, sizeof(u32));
		if (ret && (VASTAI_NORMAL_STATE == vastai_get_pci_state(priv))) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "heartbeat_work: read CSRAM_BASE_ADDR error, %d\n", ret);
			//TODO: this moment pcie error, how to reset dev
			break;
		}
		if (PCIE_FAULT_CODE == pcie_read_test) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "heartbeat_work: pcie link err.[0x%x]:0x%x\n", CSRAM_BASE_ADDR, pcie_read_test);
			break;
		}

		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "[VA-RAS]%s core[%d] heartbeat stopped!!!!!\n",
				core_name(priv,core_id),core_id);
		/* core is hanged, set status of die and core */
		set_bit(core_id, &heartbeat->cores_status);

		//mask related core acording core id
		priv->heartbeat_mask |= 1<< core_id;
		priv->core_status_bitmap |= 1<< core_id;
	}

	memcpy(last_count, heart_beat_cnt, sizeof(heartbeat->last_counts));

	if ((~HEARTBEAT_CNT_REG_MASK) == heartbeat->cores_status) {
		/* TODO: reset device */
		heartbeat->cores_status = 0;
		memset(heartbeat->last_counts, 0, sizeof(heartbeat->last_counts));
		goto stop;
	}

skip:
	schedule_delayed_work(&heartbeat->heartbeat_work, heartbeat->timeout_threshold);
	return;

stop:
	VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "heartbeat_work stop\n");
	VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "cores_heartbeat_status=0x%lx\n", heartbeat->cores_status);
	return;
}


void vastai_heartbeat_monitor_init(struct vastai_pci_info *priv)
{
	struct heartbeat_monitor *heartbeat = &priv->heartbeat;

	memset(heartbeat, 0, sizeof(*heartbeat));
	priv->heartbeat_mask = HEARTBEAT_CNT_REG_MASK;
	heartbeat->timeout_threshold = msecs_to_jiffies(3000); /* 3s */
	INIT_DELAYED_WORK(&heartbeat->heartbeat_work, vastai_heartbeat_work);
	schedule_delayed_work(&heartbeat->heartbeat_work, heartbeat->timeout_threshold);
}

void vastai_heartbeart_monitor_deinit(struct vastai_pci_info *priv)
{
	cancel_delayed_work_sync(&priv->heartbeat.heartbeat_work);
}

/* bit map: 0 - alive, 1 - hang */
static char* mcu_status(int status)
{
	if(0 == status){
		return "alive";
	} else if(1 == status){
		return "hang!";
	} else {
		return "unknw";
	}
}

void check_soc(struct vastai_pci_info *priv)
{
	int ret = 0;
	u32 core_id = 0;
	u32 heart_beat_cnt[HEARTBEAT_CNT_REG_NUM] = { 0 };
	ret = vastai_pci_mem_read(priv, 0, HEART_BEAT_BASE, heart_beat_cnt, sizeof(heart_beat_cnt));
	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "S<================check soc status==============>S\n");
	for(core_id = 0;core_id < HEARTBEAT_CNT_REG_NUM; core_id++ ) {
		VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "core[%02d]%s HeartBeat[%08x] [Status-%s] Reset[%08x]\n",
			core_id,core_name(priv,core_id),heart_beat_cnt[core_id],
			mcu_status((priv->heartbeat.cores_status>>core_id)&1),
			priv->heartbeat.reset_cnt[core_id]);
	}
	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "E<================check soc status==============>E\n");
	return;
}
#endif
