#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <linux/timer.h>
#include "vastai_pci.h"
#include "sg100_cmd.h"
#include "hw_config.h"
#include "vastai_ras.h"
/*when current vf happen gfx hw error,call this api*/
int vastai_gfx_reset(void *pci_info)
{
    int ret = 0;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};
    struct vastai_pci_info *priv = (struct vastai_pci_info *)pci_info;

	smcu_task_desc.w0_data0 = SMCU_RESET_GFX;
	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret)
		return ret;

	/*check smcu return data whether correcct*/
	if(SMCU_RESET_GFX_CHECK_NUM != smcu_ret.w1_data1) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"smcu_check_val[0x%x] != smcu_return_addr[0x%x]\n",
					SMCU_RESET_GFX_CHECK_NUM,smcu_ret.w0_data0);
		return -EIO;
	}

	return ret;
}
EXPORT_SYMBOL(vastai_gfx_reset);

void pf_unload_meta_fw(struct vastai_pci_info *priv)
{
	int ret;
	struct pcie_transfer_cmd trans = {0};
	VASTAI_PCI_INFO(priv, priv->pkg_id,"pf_unload_meta_fw\n");

	/*to do, gpu unload meta fw */

	trans.w0.s_data0.optcode = HOST_ACK_UNLOAD_META_FW;
	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF, &trans, 0);
	if (ret != 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"pf_unload_meta_fw host ack pf unload meta fw fail ,ret=%d \n", ret);
	}
	return;
}

void pf_load_meta_fw(struct vastai_pci_info *priv)
{
	int ret;
	struct pcie_transfer_cmd trans = {0};
	
	VASTAI_PCI_INFO(priv, priv->pkg_id,"pf_load_meta_fw\n");
	/*to do, gpu load meta fw*/

	trans.w0.s_data0.optcode = HOST_ACK_LOAD_META_FW;
	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF, &trans, 0);
	if (ret != 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"pf_load_meta_fw host ack pf load meta fw fail ,ret=%d \n", ret);
	}
	return;
}

void vastai_gfx_will_reset(struct vastai_pci_info *priv,struct pcie_transfer_cmd *reset_info)
{
	struct pcie_transfer_cmd trans = {0};
	u8 pfn = reset_info->w0.s_data0.rev0;
	u8 vfn = reset_info->w0.s_data0.rev1;
	u8 is_vf = reset_info->w0.s_data0.rev2;
	int ret = 0;
	/*01:to do,notify ddk stop gfx task*/
	if(is_vf)
		VASTAI_PCI_ERR(priv, priv->pkg_id,"gfx_will_reset beacause  pf[%d]vf[%d] driver happened gfx hw error\n",pfn,vfn);
	else
		VASTAI_PCI_ERR(priv, priv->pkg_id,"gfx_will_reset beacause  pf[%d] driver happened gfx hw error\n",pfn);
	/*02:send cmd to smcu notify driver has ready reset gfx */
	trans.w0.s_data0.optcode = HOST_ACK_RESET_GFX;
	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF, &trans, 0);
	if (ret != 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"gfx_will_reset host ack reset gfx fail ,ret=%d \n", ret);
	}
	return;
}

void vastai_gfx_reset_done(struct vastai_pci_info *priv)
{
	/*01:to do,notify ddk start to work gfx task*/

	VASTAI_PCI_INFO(priv, priv->pkg_id,"vastai_gfx_reset_done the gpu can work normally\n");
	return;
}