#include "vastai_pci.h"
#include "vastai_pcie_public.h"
#include "vastai_cmd.h"
#include "vastai_fifo.h"
#include "vastai_state.h"
#include "vastai_common.h"
#include "vastai_ras.h"
#include "sg100_cmd.h"
#include "hw_config.h"
#include "hw_queue.h"
#include "../smmu.h"
#include "../sdma.h"
#include "../va_irq.h"
#include "../vsync/va_vsync.h"
#include <linux/kfifo.h>
#include "../ras/smcu_cfg_err.h"


static void mpu_handler(struct vastai_pci_info *priv, struct pcie_transfer_cmd *msix_info)
{
    u64 out_of_bound_addr = (u64)((u64)msix_info->w2.data2<<32)|msix_info->w1.data1;
    u8  read_or_write     = msix_info->w0.s_data0.rev2;
    switch (msix_info->w0.s_data0.rev0){
        case SMMU_MPU_OUT_OF_BOUND:{
            u8 stream_id = msix_info->w0.s_data0.rev1;
            read_or_write = msix_info->w3.s_data3.rev0;
            if(MPU_READ == read_or_write){
                VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[smmu_mpu]the stream_id[%d] read addr[0x%llx] happen out of bound \n",
                           stream_id,out_of_bound_addr);
            } else if(MPU_WRITE == read_or_write) {
                VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[smmu_mpu]the stream_id[%d] write addr[0x%llx] happen out of bound \n",
                           stream_id,out_of_bound_addr);
            }           
            break;
        }
            
        case GFX_MPU_OUT_OF_BOUND:{
            u8 gpuid = msix_info->w0.s_data0.rev1;
            u8 osid  = msix_info->w0.s_data0.rev2;
            read_or_write = msix_info->w3.s_data3.rev0;
            if(MPU_READ == read_or_write){
                 VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[gfx_mpu]the gpu[%d] osid[%d] read addr[0x%llx] happen out of bound \n",
                           gpuid, osid, out_of_bound_addr);
            } else if(MPU_WRITE == read_or_write) {
                 VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[gfx_mpu]the gpu[%d] osid[%d] write addr[0x%llx] happen out of bound \n",
                           gpuid, osid, out_of_bound_addr);
            }           
            break;
        }

        case ENCODER_MPU_OUT_OF_BOUND:{
            u8 encoder_num = msix_info->w0.s_data0.rev1;
            if(MPU_READ == read_or_write){
                 VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[encoder_mpu%d]read addr[0x%llx] happen out of bound \n",
                           encoder_num, out_of_bound_addr);
            } else if(MPU_WRITE == read_or_write) {
                 VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[encoder_mpu%d]write addr[0x%llx] happen out of bound \n",
                           encoder_num, out_of_bound_addr);
            }            
            break;
        }
            
        case DECODER_MPU_OUT_OF_BOUND:{
            u8 decoder_num = msix_info->w0.s_data0.rev1;
            if(MPU_READ == read_or_write){
                VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[decoder_mpu%d]read addr[0x%llx] happen out of bound \n",
                           decoder_num, out_of_bound_addr);
            } else if(MPU_WRITE == read_or_write) {
                VASTAI_PCI_ERR(priv, priv->pkg_id,
                           "[decoder_mpu%d]write addr[0x%llx] happen out of bound \n",
                           decoder_num, out_of_bound_addr);
            }            
            break;
        }
                
        default:
            break;
    }
    return;
}

static void smcu_task_ret_hander(struct vastai_pci_info *priv,
                          struct pcie_transfer_cmd *msix_info)
{
    int ret = 0;
    struct smcu_task_info smcu_task_desc;
    ret = kfifo_out(&priv->smcu_task.smcu_task_fifo, &smcu_task_desc, 1);
    if (1 != ret) {
        VASTAI_PCI_ERR(priv, priv->pkg_id, "%s, %d, kfifo out smcu_task_desc fail,ret=%d\n",__func__,__LINE__,ret);
        VASTAI_PCI_ERR(priv, priv->pkg_id, "smcu to host w0_data0=0x%x\n",msix_info->w0.data0);
		VASTAI_PCI_ERR(priv, priv->pkg_id, "smcu to host w1_data1=0x%x\n",msix_info->w1.data1);
        VASTAI_PCI_ERR(priv, priv->pkg_id, "smcu to host w2_data2=0x%x\n",msix_info->w2.data2);
		VASTAI_PCI_ERR(priv, priv->pkg_id, "smcu to host w3_data3=0x%x\n",msix_info->w3.data3);
        goto end;
    }
    smcu_task_desc.callback(smcu_task_desc.arg1,msix_info);

end:    
    return;
}

static void vastai_pci_func_state_handler(struct vastai_pci_info *priv, u32 states)
{
	switch (states){
		case VASTAI_NORMAL_STATE:
			/* to reset sdma */
			sdma_reset(priv);
			/* to reset smmu*/
			smmu_handle_for_hotr(priv);
			break;

		default:
			break;
	}

	vastai_pci_set_state(priv, states);

	return;
}

static int smcu_msix_handler(struct vastai_pci_info *priv, struct pcie_transfer_cmd *msix_info)
{
	int ret = 0;

	switch (msix_info->w0.s_data0.optcode){
		case SMMU_EVENT_CMD:
			va_get_smmu_err(priv, msix_info);
			break;

		case SWITCH_ATU_DONE:
			complete(&priv->swtich_atu_done);
			break;

		case SMCU_TASK_RET:{
			smcu_task_ret_hander(priv,msix_info);
			break;
		}

		case SMCU_VSYNC_TIMER_INT:{
			va_vsync_timer_callback(priv, msix_info->w1.data1);
			break;
		}

        case SMCU_CFG_ERR:{
            smcu_cfg_err(priv, msix_info->w1.data1, msix_info->w2.data2);
            break;
        }

#ifdef CONFIG_VASTAI_TOOLS_SUITE
		case SMCU_SMI_CMD_ACK:{
			u32 smi_msg = msix_info->w1.data1;
			u8 bc = (smi_msg >> 8) & 0xFF;
			VASTAI_PCI_DBG(priv, DUMMY_DIE_ID, "recv SMCU_SMI_CMD_ACK block_id=%d", bc);

			if (bc >= 2) {
				VASTAI_PCI_ERR(priv, DUMMY_DIE_ID, "[ERROR] SMI block count > 2 ");
				priv->tools_info.smi_ack_msg[0] = 0;
				priv->tools_info.smi_ack_msg[1] = 0;
				complete(&priv->tools_info.smi_ack[0]);
				complete(&priv->tools_info.smi_ack[1]);
			}else{
			priv->tools_info.smi_ack_msg[bc] = smi_msg;
				complete(&priv->tools_info.smi_ack[bc]);
			}
			break;
		}
#endif

		case SMCU_XSPI_RW_ACK:{
			priv->rw_xspi_msg = msix_info->w2.data2;
			complete(&priv->rw_xspi);
			break;
		}

		case PMCU_BOOT_MODE_ACK:{
			priv->pmcu_boot_mode_msg = msix_info->w1.data1;
			complete(&priv->pmcu_boot_mode_done);
			break;
		}

#ifdef CONFIG_VASTAI_TOOLS_SUITE
		case SMCU_TO_HOST_LOG_READ:{
			complete(&(priv->tools_info.log_comp));
			break;
		}
#endif

		case SMCU_CMD_REINIT_RST_PKG_ACK:{
			complete(&priv->send_reinit_rst_done);
			break;
		}

		case SMCU_TO_HOST_MPU_OUT_OF_BOUND:{
			mpu_handler(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_MSG1_VIDEO_EXCEPTION:{
			vastai_pci_video_exception(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_MSG1_CORE_EXCEPTION:{
			vastai_pci_vemcu_exception(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_MSG1_CORE_RESET_ACK:{
			vastai_pci_vemcu_reset_ack(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_NOTIFY_CORE_EXCEPTION:{
			vastai_pci_mcux_exception(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_NOTIFY_CORE_RESET_ACK:{
			vastai_pci_mcux_reset_ack(priv,msix_info);
			break;
		}

		case VASTAI_PCIE_MSG1_EXCEPTION:{
			vastai_pci_msg1_exception(priv,msix_info);
			break;
		}

		case FREQ_REDUCE_BY_TEMP:{
			VASTAI_PCI_INFO(priv, priv->pkg_id,
					"[vawarn] Frequency reduced to  %d%% due to over-temperature!\n",
					msix_info->w1.s_data1.rev0);
			break;
		}

		case FREQ_REDUCE_BY_VOLT:{
			VASTAI_PCI_INFO(priv, priv->pkg_id,
					"[vawarn] Frequency reduced to  %d%% due to over-current!\n",
					msix_info->w1.s_data1.rev0);
			break;
		}

		case FREQ_RECOVER_NORMAL_BY_TEMP:
		{
			VASTAI_PCI_INFO(priv, priv->pkg_id,
					"[vawarn] Frequency increased to  %d%% as temperature returned to normal.\n",
					msix_info->w1.s_data1.rev0);
			break;
		}

		case FREQ_RECOVER_NORMAL_BY_VOLT:
		{
			VASTAI_PCI_INFO(priv, priv->pkg_id,
					"[vawarn] Frequency increased to  %d%% as current returned to normal.\n",
					msix_info->w1.s_data1.rev0);
			break;
		}

		case FREQ_POWER_DOEN_REQ:{
			/* set smcu machine state for each die */
			vastai_pci_sync_state(priv, VASTAI_EXIT_STATE);
			vastai_pci_state_reset_start(priv);

			VASTAI_PCI_ERR(priv, priv->pkg_id,
					"bmcu power down alert occurs, ret=%d.\n",
					msix_info->w1.s_data1.rev0);
			break;
		}

		case VASTAI_PCIE_SYNC_FUNC_STATE:{
			/* into device state handler */
			vastai_pci_func_state_handler(priv, msix_info->w1.data1);
			break;
		}

		case SDMA_ERR_CMD:{
			/* into sdma err handler */
			sdma_err_handler(priv, msix_info);
			break;
		}

		case SMCU_RET_FLR_DONE:{
			complete(&priv->smcu_flr_done);
			break;
		}

		case SMCU_WILL_RESET_GFX:{
			vastai_gfx_will_reset(priv,msix_info);
			break;
		}

		case SMCU_RESET_GFX_DONE:{
			vastai_gfx_reset_done(priv);
			break;
		}

		case SMCU_NOTIFY_PF_UNLOAD_META_FW:{
			pf_unload_meta_fw(priv);
			break;
		}

		case SMCU_NOTIFY_PF_LOAD_META_FW:{
			pf_load_meta_fw(priv);
			break;
		}

		default:
			VASTAI_PCI_ERR(priv, priv->pkg_id, "smcu to host opcode[0x%x] not handled,\n", msix_info->w0.s_data0.optcode);
			dump_smcu_ring_buf(priv);
			ret = -1 ;
			break;
	}

	return ret;
}


static void smcu_and_cmcu_msix_callback(void *arg)
{
	int ret;
	struct vastai_pci_info *priv = (struct vastai_pci_info *) arg;
	struct pcie_transfer_cmd trans;

	while (1) {
        if( ONLY_1_PF == priv->fn_mode)	{
            ret = vastai_pci_receive_msg(priv,0,ONLY_1PF_SMCU_TO_HOST_INFO_BUF,&trans);
        } else if (ONLY_2_PF == priv->fn_mode) {
            ret = vastai_pci_receive_msg(priv,0,ONLY_2PF_SMCU_TO_HOST_INFO_BUF,&trans);
        } else {
            ret = vastai_pci_receive_msg(priv,0,COMMON_SMCU_TO_HOST_INFO_BUF,&trans);
        }

        if (ret){
            break;
        }
        
        ret = smcu_msix_handler(priv,&trans);
        if (ret) {
             VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "smcu_msix_handler failed\n");
        }
		
	}

#ifdef CONFIG_VASTAI_AI
    if(priv->waiting_cmcu_comback){
        vastai_cmcu_call_back(priv);
    }
#endif    
    return;

}

int smcu_to_host_msix_init(struct vastai_pci_info *priv)
{
    int ret;
    
    ret = va_register_pcie_interrupt(priv, COMMON_MSIX_SMCU_AND_CMCU_VECTOR,
                     smcu_and_cmcu_msix_callback,
                     priv);
    if (ret) {
        VASTAI_PCI_ERR(priv, 0,
                "%s: va_register_pcie_interrupt failed!, %d\n",
                __func__, ret);
        ret = -EIO;

    }

    return ret;
}
