/** 
 * Read & Parse Inventory Info for each function. Update the parsing logic if the structure of Inventory info is changed. 
 *
**/

#include "vastai_pci.h"
#include "sg100_base.h"
#include "sg100_cmd.h"
#include "va_inventory.h"
#include "hw_config.h"

/*
[37][7925][SMCU] Model Name = VG1000-32G
[38][7929][SMCU] HW Version = A0
[39][7931][SMCU] Product SN = ATC136E00023
[40][7935][SMCU] pSN len: 12, pSN:ATC136E00023, card_id:0xb9542737
[41][7944][SMCU] MFG PN = PAG1FFMC8400
[42][7946][SMCU] PCB Supplier = Fastprint
[43][7949][SMCU] Mech_Supplier = Xunshun
[44][7952][SMCU] Manufacturer = Austor
[45][7955][SMCU] MFG_Date = 06/14/2023
[46][7958][SMCU] Main_Chip_PN = SG100
[47][7961][SMCU] Main_Chip_Version = A0
[48][7963][SMCU] Max_Power = 75W
[49][7966][SMCU] Main_Chip_Quantity = 1
[50][7969][SMCU] Single_Memory_Chip_Capacity = 8GB
[51][7972][SMCU] Memory_Chip_Quantity_Per_Main_Chip = 4
[52][7976][SMCU] Input_Clock_Type = OSC
[53][7979][SMCU] Input_Clock_Frequency = 100M
[54][7982][SMCU] PCIe_Max_Lane_Num = 16
[55][7985][SMCU] PCIe_Max_Speed = GEN4
[56][7987][SMCU] Core_OT_Throttling_Threshold = 85
[57][7990][SMCU] Core_OT_Shutdown_Threshold = 105
[58][7993][SMCU] Inlet_OT_Throttling_Threshold = 60
[59][7997][SMCU] Inlet_OT_Shutdown_Threshold = 70
[60][8000][SMCU] Outlet_OT_Throttling_Threshold = 80
[61][8003][SMCU] Outlet_OT_Shutdown_Threshold = 95
[62][8006][SMCU] GF_OC_Throttling_Threshold = 5500
[63][8009][SMCU] GF_OC_Shutdown_Threshold = 5500
[64][8013][SMCU] DP_Interface_Support = No
*/
void show_inventory_info(struct vastai_pci_info *priv)
{
    if(priv->is_pf)
            VASTAI_PCI_INFO(priv, priv->pkg_id,  "----------------------Inventory Info pfn[%u]------------------\n", priv->priv_hw_cfg->sys_cfg.pfn);
    else 
            VASTAI_PCI_INFO(priv, priv->pkg_id,  "----------------------Inventory Info vfn[%u]------------------\n", priv->priv_hw_cfg->sys_cfg.vfn);

    VASTAI_PCI_INFO(priv, priv->pkg_id, "Model Name = %s\n",   priv->inventory_info.model_name);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "HW Version = %s\n", priv->inventory_info.hw_version);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Product SN = %s\n", priv->inventory_info.product_sn);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "MFG PN = %s\n",   priv->inventory_info.mfg_pn);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "PCB Supplier = %s\n", priv->inventory_info.pcb_supplier);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Mech_Supplier = %s\n",   priv->inventory_info.mech_supplier);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Manufacturer = %s\n", priv->inventory_info.manufacturer);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "MFG_Date = %s\n", priv->inventory_info.mfg_date);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Main_Chip_PN = %s\n",   priv->inventory_info.main_chip_pn);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Main_Chip_Version = %s\n", priv->inventory_info.main_chip_version);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Max_Power = %s\n", priv->inventory_info.max_power);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Main_Chip_Quantity = %s\n", priv->inventory_info.main_chip_quantity);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Single_Memory_Chip_Capacity = %s\n", priv->inventory_info.single_memory_chip_sz);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Memory_Chip_Quantity_Per_Main_Chip = %s\n",   priv->inventory_info.mem_chip_num_in_main_chip);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Main_Chip_Version = %s\n", priv->inventory_info.main_chip_version);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Input_Clock_Type = %s\n", priv->inventory_info.input_clock_type);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Input_Clock_Frequency = %s\n", priv->inventory_info.input_clock_frequency);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "PCIe_Max_Lane_Num = %s\n", priv->inventory_info.pcie_max_lane_num);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "PCIe_Max_Speed = %s\n",   priv->inventory_info.pcie_max_speed);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Core_OT_Throttling_Threshold = %s\n", priv->inventory_info.core_ot_throttle_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Core_OT_Shutdown_Threshold = %s\n", priv->inventory_info.core_ot_shutdown_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Inlet_OT_Throttling_Threshold = %s\n", priv->inventory_info.inlet_ot_throttle_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Inlet_OT_Shutdown_Threshold = %s\n", priv->inventory_info.inlet_ot_shutdown_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Outlet_OT_Throttling_Threshold = %s\n",   priv->inventory_info.outlet_ot_throttle_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "Outlet_OT_Shutdown_Threshold = %s\n", priv->inventory_info.outlet_ot_shutdown_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "GF_OC_Throttling_Threshold = %s\n", priv->inventory_info.gf_oc_throttle_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "GF_OC_Shutdown_Threshold = %s\n", priv->inventory_info.gf_oc_shutdown_threshold);
    VASTAI_PCI_INFO(priv, priv->pkg_id, "DP_Interface_Support = %s\n", priv->inventory_info.dp_interface_support);

    VASTAI_PCI_INFO(priv, priv->pkg_id,  "--------------------------------------------------------------\n");
}

static u8 check_tlv_info(struct vastai_pci_info *priv, u8 info[], u8 type, u8 len, u32 cur_pos)
{
	u8 tmp[128] = {0};
      u8 ret = 0;

	if ((len > MAX_INVENTORY_ITEM_LEN) || (type > 0x1f))
	{
		VASTAI_PCI_DBG(priv, priv->pkg_id,  "WARNING: illegal data format, type = %#x, len = %d\n", type, len);
		return 1;
	}

	memcpy(tmp, info, len);
	tmp[len] = '\0';

	switch(type)
	{
		case Nothing:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "%s\n", tmp);
            break;

		case Model_Name:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Model Name = %s\n", tmp);
            strcpy(priv->inventory_info.model_name, tmp);
		break;

		case HW_Version:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "HW Version = %s\n", tmp);
            strcpy(priv->inventory_info.hw_version, tmp);
            break;

		case Product_SN:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Product SN = %s\n", tmp);
            strcpy(priv->inventory_info.product_sn, tmp);
            break;

		case MFG_PN:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "MFG PN = %s\n", tmp);
            strcpy(priv->inventory_info.mfg_pn, tmp);
            break;

		case PCB_Supplier:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "PCB Supplier = %s\n", tmp);
            strcpy(priv->inventory_info.pcb_supplier, tmp);
            break;

		case Mech_Supplier:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Mech_Supplier = %s\n", tmp);
            strcpy(priv->inventory_info.mech_supplier, tmp);
            break;

		case Manufacturer:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Manufacturer = %s\n", tmp);
            strcpy(priv->inventory_info.manufacturer, tmp);
            break;

		case MFG_Date:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "MFG_Date = %s\n", tmp);
            strcpy(priv->inventory_info.mfg_date, tmp);
            break;

		case Main_Chip_PN:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Main_Chip_PN = %s\n", tmp);
            strcpy(priv->inventory_info.main_chip_pn, tmp);
            break;

		case Main_Chip_Version:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Main_Chip_Version = %s\n", tmp);
            strcpy(priv->inventory_info.main_chip_version, tmp);
            break;

		case Max_Power:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Max_Power = %s\n", tmp);
            strcpy(priv->inventory_info.max_power, tmp);
            break;

		case Main_Chip_Quantity:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Main_Chip_Quantity = %s\n", tmp);
            strcpy(priv->inventory_info.main_chip_quantity, tmp);
            break;

		case Single_Memory_Chip_Capacity:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Single_Memory_Chip_Capacity = %s\n", tmp);
            strcpy(priv->inventory_info.single_memory_chip_sz, tmp);
            break;

		case Memory_Chip_Quantity_Per_Main_Chip:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Memory_Chip_Quantity_Per_Main_Chip = %s\n", tmp);
            strcpy(priv->inventory_info.mem_chip_num_in_main_chip, tmp);
            break;

		case Input_Clock_Type:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Input_Clock_Type = %s\n", tmp);
            strcpy(priv->inventory_info.input_clock_type, tmp);
            break;

		case Input_Clock_Frequency:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Input_Clock_Frequency = %s\n", tmp);
            strcpy(priv->inventory_info.input_clock_frequency, tmp);
            break;

		case PCIe_Max_Lane_Num:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "PCIe_Max_Lane_Nu= %s\n", tmp);
            strcpy(priv->inventory_info.pcie_max_lane_num, tmp);
            break;

		case PCIe_Max_Speed:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "PCIe_Max_Speed = %s\n", tmp);
            strcpy(priv->inventory_info.pcie_max_speed, tmp);
            break;

		case Core_OT_Throttling_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Core_OT_Throttling_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.core_ot_throttle_threshold, tmp);
            break;

		case Core_OT_Shutdown_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Core_OT_Shutdown_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.core_ot_shutdown_threshold, tmp);
            break;

		case Inlet_OT_Throttling_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Inlet_OT_Throttling_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.inlet_ot_throttle_threshold, tmp);
            break;

		case Inlet_OT_Shutdown_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Inlet_OT_Shutdown_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.inlet_ot_shutdown_threshold, tmp);
            break;

		case Outlet_OT_Throttling_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Outlet_OT_Throttling_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.outlet_ot_throttle_threshold, tmp);
            break;

		case Outlet_OT_Shutdown_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "Outlet_OT_Shutdown_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.outlet_ot_shutdown_threshold, tmp);
            break;

		case GF_OC_Throttling_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "GF_OC_Throttling_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.gf_oc_throttle_threshold, tmp);
            break;

		case GF_OC_Shutdown_Threshold:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "GF_OC_Shutdown_Threshold = %s\n", tmp);
            strcpy(priv->inventory_info.gf_oc_shutdown_threshold, tmp);
            break;

		case DP_Interface_Support:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "DP_Interface_Support = %s\n", tmp);
            strcpy(priv->inventory_info.dp_interface_support, tmp);
            break;

		default:
            VASTAI_PCI_DBG(priv, priv->pkg_id, "[Warning] Undefined type code! \n");
            ret = 1;
		break;
	}

    return ret;
}


static u8 parse_inventory_info(struct vastai_pci_info *priv, u8 info[])
{
	u32 len = 0;
	u32 tlv_addr = TLV_START_ADDR;
	u8  idx = MAX_INVENTORY_ITEM_NUM;

	if(info[0] == 0xff && info[1] == 0xff && info[10] == 0xff && info[11] == 0xff)
	{
		VASTAI_PCI_DBG(priv, priv->pkg_id, "EEPROM no Data. Set card id to default code\n");
		return 1;
	}

	len = ((info[11] << 8) | info[10]) - 6;     //total length
	if ((len > 250) || (len == 0))
	{
		VASTAI_PCI_DBG(priv, priv->pkg_id,  "illegal inventory info code length, %d!\n", len);
		return 1;
	}

	VASTAI_PCI_DBG(priv, priv->pkg_id,  " -----------------Inventory Info-------------------------\n");

	while(idx--){
		if(check_tlv_info(priv, &info[tlv_addr+2], info[tlv_addr], info[tlv_addr+1], tlv_addr))
            break;
		len = len - info[tlv_addr+1] - 2;
		tlv_addr += info[tlv_addr+1] + 2;
	}

	VASTAI_PCI_DBG(priv, priv->pkg_id,  " ----------------------------------------------------------\n");

	return 0;
}

// Parse Inventory Code into struct
u8 va_read_parse_inventory(struct vastai_pci_info* priv)
{
      u8 inventory_code[INVENTORY_CODE_MAX_SZ] = {0};
      int ret = 0;

      VASTAI_PCI_DBG(priv, priv->pkg_id,  "%s\n", __func__);

      ret = vastai_read_info_from_smcu(priv, HOST_READ_SMCU_INVENTORY_CFG, INVENTORY_CODE_MAX_SZ, inventory_code);
      if(ret == 0) {
            // parse inventory code & store in priv for each PF/VF function
            parse_inventory_info(priv, inventory_code);
      }
      else {
            strcpy(priv->inventory_info.product_sn, "VA1EC6EFEF");
      }

      return 0;
}

// Return Product_SN for current function
u8* va_get_product_sn(void *pci_info)
{
      struct vastai_pci_info *priv = (struct vastai_pci_info *)pci_info;

	return priv->inventory_info.product_sn;
}
EXPORT_SYMBOL(va_get_product_sn);

// Return Model_Name for current function
u8* va_get_model_name(void *pci_info)
{
      struct vastai_pci_info *priv = (struct vastai_pci_info *)pci_info;

	return priv->inventory_info.model_name;
}
EXPORT_SYMBOL(va_get_model_name);
