#include <linux/dmi.h>
#include "vastai_dmi_table.h"
#include "sg100_cmd.h"

#define DMI_STATE 0x8BC0748

struct dmi_info host_dmi_info;

#define INSPUR_SA5212M4				0x010101
#define INSPUR_NF5280M5				0x010201
#define INSPUR_NF5280M6				0x010202
#define INSPUR_NF5468M5				0x010301
#define INSPUR_NF5468M6				0x010302
#define INSPUR_NE5260M5				0x010401
#define LTHPC_SR222					0x020101
#define HUAQIN_P6410				0x020101
#define SUPERMICRO_AS_4124GS_TNR	0x040101
#define SUPERMICRO_SYS_6949GP_TRT	0x040101
#define XFUSION_2288H_V5			0x030101
#define XFUSION_2288H_V6			0x030102
#define HUAWEI_TAISHAN_200			0x050101
#define LENOVO_THINKSYSTEM_SR665	0x060101
#define NEW_H3C_UNISERVER_R4900_G5	0x040101
#define NEW_H3C_UNISERVER_R5300_G5	0x070201
#define H3C_R4900_G3				0x070301
#define SYS_MANU_SYS_PRO_NAME		0x0f0101
#define DEFAULT_STR_DEFAULT_STR		0x0f0201

struct dmi_info dmi_table[] = {
	{
		.sys_vendor = "Inspur",
		.product_name = "SA5212M4",
		.id = INSPUR_SA5212M4,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5280M5",
		.id = INSPUR_NF5280M5,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5280M6",
		.id = INSPUR_NF5280M6,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5468M5",
		.id = INSPUR_NF5468M5,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NF5468M6",
		.id = INSPUR_NF5468M6,
	}, {
		.sys_vendor = "Inspur",
		.product_name = "NE5260M5",
		.id = INSPUR_NE5260M5,
	},{
		.sys_vendor = "LTHPC",
		.product_name = "SR222-Server-System",
		.id = LTHPC_SR222,
	}, {
		.sys_vendor = "HUAQIN",
		.product_name = "P6410",
		.id = HUAQIN_P6410,
	}, {
		.sys_vendor = "Supermicro",
		.product_name = "AS -4124GS-TNR",
		.id = SUPERMICRO_AS_4124GS_TNR,
	}, {
		.sys_vendor = "Supermicro",
		.product_name = "SYS-6949GP-TRT",
		.id = SUPERMICRO_SYS_6949GP_TRT,
	}, {
		.sys_vendor = "XFUSION",
		.product_name = "2288H V5",
		.id = XFUSION_2288H_V5,
	}, {
		.sys_vendor = "XFUSION",
		.product_name = "2288H V6",
		.id = XFUSION_2288H_V6,
	}, {
		.sys_vendor = "Huawei",
		.product_name = "TaiShan 200 (Model 2280)",
		.id = HUAWEI_TAISHAN_200,
	}, {
		.sys_vendor = "Lenovo",
		.product_name = "ThinkSystem SR665",
		.id = LENOVO_THINKSYSTEM_SR665,
	}, {
		.sys_vendor = "New H3C Technologies",
		.product_name = "H3C UniServer R4900 G5",
		.id = NEW_H3C_UNISERVER_R4900_G5,
	}, {
		.sys_vendor = "New H3C Technologies",
		.product_name = "H3C UniServer R5300 G5",
		.id = NEW_H3C_UNISERVER_R5300_G5,
	}, {
		.sys_vendor = "H3C",
		.product_name = "R4900 G3",
		.id = H3C_R4900_G3,
	}, {
		.sys_vendor = "System manufacturer",
		.product_name = "System Product Name",
		.id = SYS_MANU_SYS_PRO_NAME,
	}, {
		.sys_vendor = "Default string",
		.product_name = "Default string",
		.id = DEFAULT_STR_DEFAULT_STR,
	}
};

static const char *bad_index = "<BAD INDEX>";

/* Replace non-ASCII characters with dots */
static void ascii_filter(char *bp, size_t len)
{
	size_t i;

	for (i = 0; i < len; i++)
		if (bp[i] < 32 || bp[i] >= 127)
			bp[i] = '.';
}

static char *_dmi_string(const struct dmi_header *dm, u8 s, int filter)
{
	char *bp = (char *)dm;

	bp += dm->length;
	while (s > 1 && *bp)
	{
		bp += strlen(bp);
		bp++;
		s--;
	}

	if (!*bp)
		return NULL;

	if (filter)
		ascii_filter(bp, strlen(bp));

	return bp;
}

static const char *dmi_string(const struct dmi_header *dm, u8 s)
{
	char *bp;

	if (s == 0)
		return "Not Specified";

	bp = _dmi_string(dm, s, 1);
	if (bp == NULL)
		return bad_index;

	return bp;
}

/* Callback function used to find processor manufacturer name */
static void dmi_get_processor_name(const struct dmi_header *dm, void *private)
{
	const u8 *dmi_data = (const u8 *)dm;
	const char *str;

	if (dm->type == DMI_ENTRY_PROCESSOR && dm->length >= 0x7) {
		str = dmi_string(dm, (*(volatile u8 *) (dmi_data + 0x7)));
		strcpy(private, str);
	}
}

static inline int dmicmp(const struct dmi_info *dmi1, const struct dmi_info *dmi2)
{
	int ret;

	ret = strncmp(dmi1->sys_vendor, dmi2->sys_vendor, strlen(dmi1->sys_vendor));
	if (ret)
		return ret;

	ret = strncmp(dmi1->product_name, dmi2->product_name, strlen(dmi1->product_name));
	return ret;
}

static u32 dmi_info_get_id(struct dmi_info *dmi_info)
{
	u32 id = UINT_MAX;
	int i = 0;

	for (i = 0; i < ARRAY_SIZE(dmi_table); i++) {
		if (!dmicmp(&dmi_table[i], dmi_info)) {
			id = dmi_table[i].id;
			break;
		}
	}
	return id;
}

static void dmi_info_print(struct vastai_pci_info *pci_info)
{
	VASTAI_PCI_INFO(pci_info, DUMMY_DIE_ID, "dmi info: host vendor %s:%s\n",
		host_dmi_info.sys_vendor,
		host_dmi_info.product_name);

	return;
}

static int dmi_trigger_smcu_write_flash(struct vastai_pci_info *pci_info)
{
	int ret;
	struct pcie_transfer_cmd trans;

	trans.w0.s_data0.optcode	= SMCU_DMI_FLASH;
	trans.w1.data1				= host_dmi_info.id;
	ret = vastai_pci_send_msg(pci_info, 0, COMMON_HOST_TO_SMCU_CMD_BUF, &trans, 0);
	
	return ret;
}

int dmi_set_device(struct vastai_pci_info *pci_info)
{
	int ret;

	ret = dmi_trigger_smcu_write_flash(pci_info);
	if (ret) {
		VASTAI_PCI_ERR(pci_info, DUMMY_DIE_ID, "%s send dmi failed\n", __FUNCTION__);
	}

	dmi_info_print(pci_info);
	return ret;
}

void dmi_info_init(void)
{
	host_dmi_info.sys_vendor = dmi_get_system_info(DMI_SYS_VENDOR);
	host_dmi_info.product_name = dmi_get_system_info(DMI_PRODUCT_NAME);
	if((host_dmi_info.sys_vendor != NULL) && (host_dmi_info.product_name != NULL)) {
		host_dmi_info.id = dmi_info_get_id(&host_dmi_info);
		printk("va host vendor:%s product:%s id:%d\n",
			host_dmi_info.sys_vendor,
			host_dmi_info.product_name,
			host_dmi_info.id);
	}
	
	dmi_walk(dmi_get_processor_name, (void *)host_dmi_info.processor_name);
	printk("va host processor vendor:%s\n", host_dmi_info.processor_name);
}

