#include <linux/aer.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <linux/compat.h>

#include "vastai_dmabuf.h"
#include "vastai_pci.h"
#include "vastai_pci_api.h"
#include "sg100_base.h"
#include "vastai_cmd.h"
#include "sg100_cmd.h"
#include "sg100_smcu.h"
#include "hw_config.h"
#include "vastai_ai_info.h"
#include "vastai_pcie_public.h"
#include <linux/kfifo.h>
#include "va_inventory.h"

extern struct mutex vastai_pci_modify_lock;
extern struct mutex vastai_smcu_modify_lock;
#define FN_CSRAM_BASE_ADDR               0x800000
#define FN_GFX_CSR_BASE_ADDR             0xC00000
#define FN_SMU_CSR_BASE_ADDR             0x920000
#define FN_PCIE_WRAPER_BASE_ADDR         0x942000
#define FN_PCIE_WRAPER_BASE_LIMIT        0x944000
#define FN_DDR_BASE_ADDR                 0x1040000000ULL
#define DDR_BASE_ADDR                    0x1000000000ULL

#define BAR2_CSRAM_01                    0
#define BAR2_SMU_CSR                     1
#define BAR2_GFX_CSR                     2
#define BAR2_CSRAM_02                    3

#define ACCESS_PCIE_WRAPER               0
#define ACCESS_CSRAM                     1
#define ACCESS_SMU_CSR                   2
#define ACCESS_GFX_CSR                   3
#define ACCESS_DDR                       4

#define PCIE_ACCESS_LOG                  0
#define SMCU_ACCESS_LOG                  0
u32 test_PCIe_Perf = 0;
u32 test_smcu_RW_Perf = 0;
extern u32 ring_buf_debug; //0XF print debug info


static unsigned char *vastai_pci_bar_vmem(struct vastai_pci_info *priv, int bar,u64 offset)
{
	if (!priv) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pcie_dev is NULL\n");
		return NULL;
	}
#ifdef DIAG_TEST_WC
	if(VASTAI_PCI_BAR4 == bar){
		if((offset >= 0) && (offset < (priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M))){
			return priv->diag_vddr;
		} else {
			return priv->diag_vcsr;
		}
	} else {
		return priv->bar[bar].vmem;
	}
#else
	(void)offset;//not used offset ,fixed build waring
	return priv->bar[bar].vmem;
#endif
	
}


int vastai_pci_mmio_read(struct vastai_pci_info *priv, int bar,
			       u64 offset, void *buf, unsigned int len)
{
	unsigned char *mem = NULL;
	unsigned int l = 0;
#ifdef VASTAI_MMIO_PERF_TEST
	unsigned long old_ts, now_ts;
	unsigned int info_len = len;
#endif
    mem = vastai_pci_bar_vmem(priv, bar,offset);

	if (!mem) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_mmio_read: bar%d vmem is NULL, return\n",
			       bar);
		return -ENXIO;
	}

#ifdef DIAG_TEST_WC
	if(VASTAI_PCI_BAR4 == bar){
		if((offset >= 0) && (offset < (priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M))){
			mem += offset;
		} else {
			mem += offset - (priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M);
		}
	} else {
		mem += offset;
	}
#else
	mem += offset;
#endif
	
#ifdef VASTAI_MMIO_PERF_TEST
	if(test_PCIe_Perf){
		old_ts= ktime_get();
	}
#endif     
	if (len < 4) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_mmio_read: len < 4, return\n");
		return -EINVAL;
	} else {
		if ((long)mem & 0x3) {
			l = 0x4 - ((long)mem & 0x3);
			memcpy_fromio(buf, mem, l);
			buf += l;
			mem += l;
			len -= l;
		}
		while (len >= 4) {
			*((unsigned int *)buf) = readl(mem);
			mem += 4;
			buf += 4;
			len -= 4;
		}
		if (len)
			memcpy_fromio(buf, mem, len);
	}
#ifdef VASTAI_MMIO_PERF_TEST
	if(test_PCIe_Perf) {
		now_ts= ktime_get();
    	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"[mmio-read]%d byte: %09lu(ns)=%06lu(us)\n", info_len ,(now_ts-old_ts),(now_ts-old_ts)/1000 );
	}	
#endif 
	return 0;
}

int vastai_pci_mmio_write(struct vastai_pci_info *priv, int bar,
			       u64 offset, void *buf,  unsigned int len)
{
	unsigned char *mem = NULL;
	unsigned int l = 0;
#ifdef VASTAI_MMIO_PERF_TEST
	unsigned long old_ts, now_ts;
	unsigned int info_len = len;
#endif
    mem = vastai_pci_bar_vmem(priv, bar , offset);

	if (!mem) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_mmio_read: bar%d vmem is NULL, return\n",
			       bar);
		return -ENXIO;
	}
#ifdef DIAG_TEST_WC
	if(VASTAI_PCI_BAR4 == bar){
		if((offset >= 0) && (offset < (priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M))){
			mem += offset;
		} else {
			mem += offset - (priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M);
		}
	} else {
		mem += offset;
	}
#else
	mem += offset;
#endif	
#ifdef VASTAI_MMIO_PERF_TEST
	if(test_PCIe_Perf){
		old_ts= ktime_get();
	}
#endif 
	if (len < 4) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_mmio_read: len < 4, return\n");
		return -EINVAL;
	} else {
		if ((long)mem & 0x3) {
			l = 0x4 - ((long)mem & 0x3);
			memcpy_toio(mem, buf, l);
			buf += l;
			mem += l;
			len -= l;
		}
		while (len >= 4) {
			writel(*((unsigned int *)buf), mem);
			buf += 4;
			mem += 4;
			len -= 4;
		}
		if (len) {
			memcpy_toio(mem, buf, len);
		}
	}

#ifdef VASTAI_MMIO_PERF_TEST
	if(test_PCIe_Perf){
		now_ts= ktime_get();
    	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"[mmio-write]%d byte: %09lu(ns)=%06lu(us)\n", info_len ,(now_ts-old_ts),(now_ts-old_ts)/1000 );
	}
	
#endif 
	return 0;
}

int vastai_pci_config_read(struct vastai_pci_info *priv, int offset, char *buf,
			   int len)
{
	int i, ret;

	if ((!priv) || (!priv->dev)) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_config_read priv 0x%p or dev 0x%p is NULL\n",
			       priv, priv->dev);
		return -ENXIO;
	}
	for (i = 0; i < len; i++) {
		ret = pci_read_config_byte(priv->dev, i + offset, &(buf[i]));
		if (ret) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
				       "pci_config_read err=%d\n", ret);
			return -1;
		}
	}
	return 0;
}

int vastai_pci_config_write(struct vastai_pci_info *priv, int offset, char *buf,
			    int len)
{
	int i, ret;

	if ((!priv) || (!priv->dev)) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
			       "pci_config_write priv 0x%p or dev 0x%p is NULL\n",
			       priv, priv->dev);
		return -ENXIO;
	}
	for (i = 0; i < len; i++) {
		ret = pci_write_config_byte(priv->dev, i + offset, buf[i]);
		if (ret) {
			VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
				       "pci_config_write err=%d\n", ret);
			return -1;
		}
	}
	return 0;
}

#ifdef DIAG_TEST_MODE
static int swtich_atu(struct vastai_pci_info *priv, u64 addr, int *bar, u64 *bar_offset ,int *region_num)
{
	int ret;
	u64 target_addr  = 0;
	unsigned char *vm_base = NULL;
	u64 target_addr_64b = 0;
	u32 devfn = priv->dev->devfn;
	u64 temp_addr    = addr;
	struct pcie_transfer_cmd trans;

	//first switch atu
	if( addr>(SPACE_SIZE_64G-SPACE_SIZE_256M) && addr<SPACE_SIZE_64G) {
		addr = (u64)((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base;
	}

#ifdef DIAG_TEST_DDK
	target_addr  = (u64)((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base;
#else
	{
		u64 ddr_len      = priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M;
		target_addr  = addr / ddr_len * ddr_len;
		if(target_addr < SPACE_SIZE_64G){
			target_addr = SPACE_SIZE_64G;
		}
	}
	
#endif	
	trans.w0.s_data0.optcode	= DIAG_SLIDE_WINDOW;
	trans.w0.s_data0.rev0		= 0;
	trans.w0.s_data0.rev1		= 0;
	trans.w0.s_data0.rev2		= 0;
	trans.w1.data1				= target_addr&0xFFFFFFFF;
	trans.w2.data2				= (target_addr>>32) & 0xFFFFFFFF;
	trans.w3.data3				= devfn;

	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF,  &trans, 0);
	if(ret!=0){
		VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"switch_atu cmd to smcu error\n");
	}		

	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"switch_atu start--->S\n");
	/* wait completion int from smcu */
	if (wait_for_completion_timeout(
			&(priv->swtich_atu_done),
			msecs_to_jiffies(
				VASTAI_PCIE_WAIT_SMCU_MSG1_TIME)) <= 0) {
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"switch_atu addr %#llx failed\n", target_addr);
		return -ETIMEDOUT;
	}
	reinit_completion(&(priv->swtich_atu_done));
	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"switch_atu start--->E\n");

	vm_base = priv->bar[VASTAI_PCI_BAR4].vmem;
	// bar4 region 0
	target_addr_64b  = target_addr;
	priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr    = target_addr_64b;
	priv->bar[VASTAI_PCI_BAR4].regions[0].region_size = priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M;
	priv->bar[VASTAI_PCI_BAR4].regions[0].bar_offset  = 0;
	// bar4 region 1
	target_addr_64b = SPACE_SIZE_64G - SPACE_SIZE_256M;
	priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr    = target_addr_64b;
	priv->bar[VASTAI_PCI_BAR4].regions[1].region_size = SPACE_SIZE_256M;
	priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset  = priv->bar[VASTAI_PCI_BAR4].regions[0].region_size;

	if((temp_addr >= priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr) && 
	(temp_addr < (priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[0].region_size))) {
		*bar        = VASTAI_PCI_BAR4;
		*region_num = 0;
		*bar_offset = temp_addr - priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr + 
					priv->bar[VASTAI_PCI_BAR4].regions[0].bar_offset;

	} else if ((temp_addr >= priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr) && 
	(temp_addr < (priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].region_size))) {
		*bar        = VASTAI_PCI_BAR4;
		*region_num = 1;
		*bar_offset = temp_addr - priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + 
					priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset;
	} else {
		VASTAI_PCI_ERR(priv, 0,"%s addr = 0x%llx is invalid addr \n", __func__, temp_addr);
		return -1;
	}
			
	return 0;
}
#endif

int translate_fn_addr(struct vastai_pci_info *priv,
								u64 fn_addr,
								int *bar, 
								u64 *bar_offset,
								int *region_num)
{
	u64 csram_size    = priv->priv_hw_cfg->header.csram_size;
	u64 csram_limit   = FN_CSRAM_BASE_ADDR + csram_size;

	u64 smu_csr_size  = priv->priv_hw_cfg->header.smu_csr_size;
	u64 smu_csr_limit = FN_SMU_CSR_BASE_ADDR + smu_csr_size;

	u64 gfx_csr_size  = priv->priv_hw_cfg->header.gfx_csr_size;
	u64 gfx_csr_limit = FN_GFX_CSR_BASE_ADDR + gfx_csr_size;

	u64 ddr_limit     = 0;
	u64 ddr_base      = 0;

	u64 video_buf_fbase   = 0;
	u64 video_buf_flimit  = 0;

	if(priv->priv_hw_cfg->sys_cfg.decoder_sriov_en && priv->priv_hw_cfg->sys_cfg.vf_active){
		video_buf_fbase  = priv->bar[VASTAI_PCI_BAR4].regions[0].fn_base_addr;
		video_buf_flimit = video_buf_fbase + priv->bar[VASTAI_PCI_BAR4].regions[0].region_size;

		ddr_base      = priv->bar[VASTAI_PCI_BAR4].regions[1].fn_base_addr;
		ddr_limit     = ddr_base + priv->bar[VASTAI_PCI_BAR4].regions[1].region_size;
	} else {
		ddr_base      = FN_DDR_BASE_ADDR;
		ddr_limit     = ddr_base + priv->bar[4].mmio_len;
	}

	if ((priv->is_pf) && ((fn_addr >= FN_PCIE_WRAPER_BASE_ADDR) && (fn_addr < FN_PCIE_WRAPER_BASE_LIMIT))) {
		*bar 		 = VASTAI_PCI_BAR1;
		*bar_offset  = fn_addr - FN_PCIE_WRAPER_BASE_ADDR;
	} else if((fn_addr >= FN_CSRAM_BASE_ADDR) && (fn_addr <= csram_limit)) {
		*bar 		 = VASTAI_PCI_BAR2;
		*bar_offset  = fn_addr - FN_CSRAM_BASE_ADDR + priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].bar_offset;	
		*region_num  = BAR2_CSRAM_01;

	} else if ((fn_addr >= FN_SMU_CSR_BASE_ADDR) && (fn_addr < smu_csr_limit)) {
		*bar 		 = VASTAI_PCI_BAR2;
		*bar_offset  = fn_addr - FN_SMU_CSR_BASE_ADDR + priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].bar_offset;
		*region_num  = BAR2_SMU_CSR;

	} else if ((fn_addr >= FN_GFX_CSR_BASE_ADDR) && (fn_addr < gfx_csr_limit)) {
		*bar 		 = VASTAI_PCI_BAR2;
		*bar_offset  = fn_addr - FN_GFX_CSR_BASE_ADDR + priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].bar_offset;
		*region_num  = BAR2_GFX_CSR;	

	} else if (priv->is_pf && ( (fn_addr >= 0x8E0000) && (fn_addr < 0x8E8000))) {
		*bar 		 = VASTAI_PCI_BAR2;
		*bar_offset  = fn_addr - 0x8E0000 + priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].bar_offset;
		*region_num  = BAR2_CSRAM_02;

	} 
#ifdef DIAG_TEST_MODE
	/*this fn_addr is really noc addr */		
	else if ((fn_addr >= priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr) &&
			 (fn_addr < (priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[0].region_size))) {
		*bar 		 = VASTAI_PCI_BAR4;
		*bar_offset  = fn_addr - priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr;
		*region_num  = 0;
	} else if ((fn_addr >= priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr) &&
			 (fn_addr < (priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].region_size))) {
		*bar 		 = VASTAI_PCI_BAR4;
		*bar_offset  = fn_addr - priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset;
		*region_num  = 1;
	} else if (fn_addr >= (FN_DDR_BASE_ADDR - SPACE_SIZE_256M)) {
		/*start to swtich bar*/
		int ret;
		unsigned long flags;
		spin_lock_irqsave(&priv->swtich_atu_lock, flags);

		ret = swtich_atu(priv,fn_addr,bar,bar_offset,region_num);
		if (ret < 0) {
			spin_unlock_irqrestore(&priv->swtich_atu_lock, flags);
			return ret;
		}
		spin_unlock_irqrestore(&priv->swtich_atu_lock, flags);		
	} else {
		VASTAI_PCI_ERR(priv, 0,"%s fn_addr = 0x%llx is invalid addr \n", __func__, fn_addr);
	}
#else
	else if ((fn_addr >= ddr_base) && (fn_addr < ddr_limit)) {
		if(priv->priv_hw_cfg->sys_cfg.decoder_sriov_en && priv->priv_hw_cfg->sys_cfg.vf_active){
			*bar 		 = VASTAI_PCI_BAR4;
			*bar_offset  = fn_addr - ddr_base - priv->bar[VASTAI_PCI_BAR4].regions[0].region_size;
			*region_num  = 1;
		} else {
			*bar 		 = VASTAI_PCI_BAR4;
			*bar_offset  = fn_addr - ddr_base;
		}
		
	} else if((fn_addr >= video_buf_fbase) && (fn_addr < video_buf_flimit)) {
		*bar 		 = VASTAI_PCI_BAR4;
		*bar_offset  = fn_addr - video_buf_fbase;
		*region_num  = 0;
	} else {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"%s fn_addr = 0x%llx is invalid addr \n", __func__, fn_addr);
		dump_stack();
		return -1;
	}
#endif	
	return 0;
}

int translate_noc_addr(struct vastai_pci_info *priv,
								int pkg_id, u64 noc_addr,
								int *bar, u64 *bar_offset)
{	
	u64 csram_base    = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].noc_addr;
	u64 csram_size    = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].region_size;
	u64 csram_limit   = csram_base + csram_size;

	u64 smu_csr_base  = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].noc_addr;
	u64 smu_csr_size  = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].region_size;
	u64 smu_csr_limit = smu_csr_base + smu_csr_size;

	u64 gfx_csr_base  = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].noc_addr;
	u64 gfx_csr_size  = priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].region_size;
	u64 gfx_csr_limit = gfx_csr_base + gfx_csr_size;

	u64 ddr_limit     = 0;
	u64 ddr_base      = 0;

	u64 video_buf_base  = 0;
	u64 video_buf_limit = 0;

	if(priv->priv_hw_cfg->sys_cfg.decoder_sriov_en && priv->priv_hw_cfg->sys_cfg.vf_active) {
		video_buf_base  = priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr;
		video_buf_limit = priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[0].region_size;

		ddr_base      = priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr;
		ddr_limit     = priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].region_size;
	} else {
		ddr_base      = priv->bar[VASTAI_PCI_BAR4].noc_addr;
		ddr_limit     = ddr_base + priv->bar[4].mmio_len;
	}

	if( priv->is_pf && (noc_addr >= 0x942000) && (noc_addr < 0x944000)) {
		*bar 		 = VASTAI_PCI_BAR1;
		*bar_offset  = noc_addr - priv->bar[VASTAI_PCI_BAR1].noc_addr;
		
		return 0;

	} else if ((noc_addr >= csram_base) && (noc_addr < csram_limit)) {
		*bar 		  = VASTAI_PCI_BAR2;
		*bar_offset   = noc_addr - priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].noc_addr 
						+ priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].bar_offset;
		return 0;

	} else if ((noc_addr >= smu_csr_base) && (noc_addr < smu_csr_limit)) {
		*bar 		  = VASTAI_PCI_BAR2;
		*bar_offset   = noc_addr - priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].noc_addr 
							+ priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].bar_offset;
		return 0;

	} else if ((noc_addr >= gfx_csr_base) && (noc_addr < gfx_csr_limit)) {
		*bar 		  = VASTAI_PCI_BAR2;
		*bar_offset   = noc_addr - priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].noc_addr 
						+ priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].bar_offset;
		return 0;

	} else if (priv->is_pf && (noc_addr >= 0x8E0000) && (noc_addr < 0x8E8000)) {
		//slave-pkg special csram noc addr
		*bar 		  = VASTAI_PCI_BAR2;
		*bar_offset   = noc_addr - priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].noc_addr 
						+ priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].bar_offset;
		return 0;
	} 
#ifdef DIAG_TEST_MODE	
	else if ((noc_addr >= priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr) &&
			 (noc_addr < (priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[0].region_size))) {
		*bar 		 = VASTAI_PCI_BAR4;
		*bar_offset  = noc_addr - priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr;
	} else if ((noc_addr >= priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr) &&
			 (noc_addr < (priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].region_size))) {
		*bar 		 = VASTAI_PCI_BAR4;
		*bar_offset  = noc_addr - priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr + priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset;
	} else if (noc_addr >= (DDR_BASE_ADDR - SPACE_SIZE_256M)) {
		/*start to swtich bar*/
		int ret;
		unsigned long flags;
		int region_num;
		spin_lock_irqsave(&priv->swtich_atu_lock, flags);

		ret = swtich_atu(priv,noc_addr,bar,bar_offset,&region_num);
		if (ret < 0) {
			spin_unlock_irqrestore(&priv->swtich_atu_lock, flags);
			return ret;
		}
		spin_unlock_irqrestore(&priv->swtich_atu_lock, flags);		
	} else {
		VASTAI_PCI_ERR(priv, pkg_id,"%s diag noc_addr = 0x%llx is invalid addr \n", __func__, noc_addr);
		return -1;
	}
	
#else
	else if ((noc_addr >= ddr_base) && (noc_addr < ddr_limit)) {
		if(priv->priv_hw_cfg->sys_cfg.decoder_sriov_en && priv->priv_hw_cfg->sys_cfg.vf_active){
			*bar 		  = VASTAI_PCI_BAR4;
			*bar_offset   = noc_addr - ddr_base + priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset;
		} else {
			*bar 		  = VASTAI_PCI_BAR4;
			*bar_offset   = noc_addr - priv->bar[VASTAI_PCI_BAR4].noc_addr; 
		}	
		return 0;

	} else if((noc_addr >= video_buf_base) && (noc_addr < video_buf_limit)) {
		*bar 		  = VASTAI_PCI_BAR4;
		*bar_offset   = noc_addr - video_buf_base; 
		return 0;
	} else {
		VASTAI_PCI_ERR(priv, pkg_id,"%s noc_addr = 0x%llx is invalid addr \n", __func__, noc_addr);
		dump_stack();
		return -1;
	}
#endif
		
	return 0;
}


// Func: calculate the absolute virtual address of relative_addr, not the base.
void *vastai_pci_mmio_kva_get(struct vastai_pci_info *priv,
				      u32 die_index,
				      u64 relative_addr,
				      u32 len)
{
    unsigned char *vaddr = NULL;
	int ret;
	u64 offset        = 0;
	int bar           = 0;
    
	ret   = translate_noc_addr(priv, die_index, relative_addr, &bar, &offset);	
	vaddr = offset + priv->bar[bar].vmem;
    
    return vaddr;
}          

int vastai_pci_mem_fread(struct vastai_pci_info *priv,
							u64 fn_addr, 
							void *buf,
							unsigned int len)
{
	int ret;
	u64 offset        = 0;
	int bar_num       = 0;
	int region_num    = 0;
	
	ret = translate_fn_addr(priv,fn_addr, &bar_num, &offset, &region_num);
	if (ret)
		return -ENXIO;

	ret = vastai_pci_mmio_read(priv, bar_num, offset , buf, len);
	if (ret < 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"pci_mem_fread offset = 0x%llx, len=%d, error:%d\n",
				offset, len, ret);
		return ret;
	}
#if PCIE_ACCESS_LOG
	/*this is for debug */
	{
		u64 noc_addr  = 0;
		if (priv->bar[bar_num].region_cnt > 0) {
			noc_addr 	  = fn_addr - priv->bar[bar_num].regions[region_num].fn_base_addr
							+ priv->bar[bar_num].regions[region_num].noc_addr;
		} else {
			noc_addr 	  = fn_addr - priv->bar[bar_num].fn_base_addr + priv->bar[bar_num].noc_addr;			
		}

		VASTAI_PCI_INFO(priv, priv->pkg_id,"[fread]fn_addr[0x%llx] noc_addr[0x%llx] bar[%x] offset[%llx] region[%x] len=0x%d val=%#x\n", 
											fn_addr, noc_addr, bar_num, offset, region_num, len, *((int *)buf));	
	}
#endif	
	return ret;
}

int vastai_pci_mem_fwrite(struct vastai_pci_info *priv,
						u64 fn_addr, 
						void *buf,
						unsigned int len)
{
	int ret;
	u64 offset        = 0;
	int bar_num       = 0;
	int region_num    = 0;
	
	ret = translate_fn_addr(priv,fn_addr, &bar_num, &offset, &region_num);
	if (ret)
		return -ENXIO;
	ret = vastai_pci_mmio_write(priv, bar_num, offset , buf, len);
	if (ret < 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"pci_mem_fwrite offset = 0x%llx, len=%d, error:%d\n",
				offset, len, ret);
		return ret;
	}

#if PCIE_ACCESS_LOG
/*this is for debug */
{
	u64 noc_addr  = 0;
	if (priv->bar[bar_num].region_cnt > 0) {
		noc_addr 	  = fn_addr - priv->bar[bar_num].regions[region_num].fn_base_addr
			 			+ priv->bar[bar_num].regions[region_num].noc_addr;
	} else {
		noc_addr 	  = fn_addr - priv->bar[bar_num].fn_base_addr + priv->bar[bar_num].noc_addr;			
	}

	VASTAI_PCI_INFO(priv, priv->pkg_id,"[fwrite]fn_addr[0x%llx] noc_addr[0x%llx] bar[%x] offset[%llx] region[%x] len=0x%d val=%#x\n", 
										fn_addr, noc_addr, bar_num, offset, region_num, len,*((int *)buf));		
}
#endif
	
	return ret;
}

int vastai_pci_mem_read(struct vastai_pci_info *priv,
							int pkg_id,
							u64 noc_addr, 
							void *buf,
							unsigned int len)
{
	int ret;
	u64 offset        = 0;
	int bar_num       = 0;
	
	ret = translate_noc_addr(priv,pkg_id,noc_addr, &bar_num, &offset);
	if (ret)
		return -ENXIO;
	
	ret = vastai_pci_mmio_read(priv, bar_num, offset , buf, len);
	if (ret < 0) {
		VASTAI_PCI_ERR(priv, pkg_id,"pci_mem_read offset = 0x%llx, len=%d, error:%d\n",
				offset, len, ret);
		return ret;
	}
#if PCIE_ACCESS_LOG
	/*this is for debug */
	{
		VASTAI_PCI_INFO(priv, pkg_id,"[read]noc_addr[0x%llx] bar[%d] offset[0x%llx] len=0x%d val=%#x\n",
										noc_addr, bar_num, offset, len, *((int *)buf));
	}

#endif	
	return ret;
}

int vastai_pci_mem_write(struct vastai_pci_info *priv,
							int pkg_id,
							u64 noc_addr, 
							void *buf,
							unsigned int len)
{
	int ret;
	u64 offset        = 0;
	int bar_num       = 0;
	
	ret = translate_noc_addr(priv,pkg_id,noc_addr, &bar_num, &offset);
	if (ret) 
		return -ENXIO;
		
	ret = vastai_pci_mmio_write(priv, bar_num, offset , buf, len);
	if (ret < 0) {
		VASTAI_PCI_ERR(priv, pkg_id,"pci_mem_write offset = 0x%llx, len=%d, error:%d\n",
				offset, len, ret);
		return ret;
	}
#if PCIE_ACCESS_LOG
	/*this is for debug */
{
	VASTAI_PCI_INFO(priv, pkg_id,"[write]noc_addr[0x%llx] bar[%d] offset[0x%llx] len=0x%d val=%#x\n",
								noc_addr, bar_num, offset, len, *((int *)buf));
}
#endif	
	return ret;
}

int vastai_slave_config_read(struct vastai_pci_info *priv,
			       			u64 offset, void *buf,  unsigned int len)
{
	int ret;
	u64 cfg_base = priv->bar[VASTAI_PCI_BAR2].mmio_len - 0x1000;
	if(MASTER_PKG == priv->pkg_id) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "this api only for access slave pkg config space\n");
		return -1;
	}
	ret = vastai_pci_mmio_read(priv,VASTAI_PCI_BAR2, cfg_base + offset,buf,len);
	if (ret) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "access slave pkg config space error\n");
		return ret;
	}
	return 0 ;
}

int vastai_slave_config_write(struct vastai_pci_info *priv,
			       			u64 offset, void *buf,  unsigned int len)
{
	int ret;
	u64 cfg_base = priv->bar[VASTAI_PCI_BAR2].mmio_len - 0x1000;
	if(MASTER_PKG == priv->pkg_id) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "this api only for access slave pkg config space\n");
		return -1;
	}
	ret = vastai_pci_mmio_write(priv,VASTAI_PCI_BAR2, cfg_base + offset,buf,len);
	if (ret) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "write slave pkg config space error\n");
		return ret;
	}
	return 0;
}

int vastai_pcie_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0     = SMCU_MONITOR_PCIE_PERF;
	smcu_task_desc.w1_data1	    = us_tick;

	if (is_read)
		smcu_task_desc.w2_data2 = 1;
	else
		smcu_task_desc.w2_data2 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"pcie_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_ddr_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 ddr_id, u32 port_id, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0     = SMCU_MONITOR_DDR_PERF;
	smcu_task_desc.w1_data1		= us_tick;
	smcu_task_desc.w2_data2		= (ddr_id & 0xFF) | ((port_id & 0xFF) << 8);

	if (is_read)
		smcu_task_desc.w3_data3 = 1;
	else
		smcu_task_desc.w3_data3 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"ddr_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_sdma_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 sdma_id, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0 	= SMCU_MONITOR_SDMA_PERF;
	smcu_task_desc.w1_data1		= us_tick;
	smcu_task_desc.w2_data2		= sdma_id;

	if (is_read)
		smcu_task_desc.w3_data3 = 1;
	else
		smcu_task_desc.w3_data3 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"sdma_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_venc_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 venc_id, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0 	= SMCU_MONITOR_VENC_PERF;
	smcu_task_desc.w1_data1		= us_tick;
	smcu_task_desc.w2_data2		= venc_id;

	if (is_read)
		smcu_task_desc.w3_data3 = 1;
	else
		smcu_task_desc.w3_data3 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"venc_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_vdec_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 vdec_id, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0 	= SMCU_MONITOR_VDEC_PERF;
	smcu_task_desc.w1_data1		= us_tick;
	smcu_task_desc.w2_data2		= vdec_id;

	if (is_read)
		smcu_task_desc.w3_data3 = 1;
	else
		smcu_task_desc.w3_data3 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"vdec_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_gfx_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 gfx_id, u32 port_id, u64 *bw_cont)
{
	int ret;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};

	smcu_task_desc.w0_data0 	= SMCU_MONITOR_GFX_PERF;
	smcu_task_desc.w1_data1		= us_tick;
	smcu_task_desc.w2_data2		= (gfx_id & 0xFF) | ((port_id & 0xFF) << 8);

	if (is_read)
		smcu_task_desc.w3_data3 = 1;
	else
		smcu_task_desc.w3_data3 = 0;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret){
		VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"gfx_monitor failed\n");
		return ret;
	}
	*bw_cont = ((u64)smcu_ret.w1_data1 << 32) | smcu_ret.w2_data2;

	return 0;
}

int vastai_cii_set(struct vastai_pci_info *priv, u32 ch, u32 addr, u32 pf)
{
	int ret;
	struct pcie_transfer_cmd trans;

	VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID, "cii_set: ch[%d], addr[0x%x], pf[%d]\n", ch, addr, pf);

	if (ch > 8) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "cii_set: channel must be 0~7\n");
		return -1;
	}

	if (addr & 0xFFFFF003) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "cii_set: error addr[0x%x]\n", addr);
		return -1;
	}
	
	trans.w0.s_data0.optcode	= SMCU_CII_SET;
	trans.w0.s_data0.rev0		= ch;
	trans.w0.s_data0.rev1		= pf;
	trans.w1.data1				= addr;
	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF,  &trans, 0);
	if (ret) {
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "cii_set: push element fail\n");
		return -1;
	}

	return 0;
}

void dump_smcu_ring_buf(struct vastai_pci_info *priv)
{
	int i;
	u32 len = 0;
	u64 addr = 0;
	void  *dump_addr;
	unsigned char print_str[64];
    u8 smcu_to_host_index = 0;
	u8 host_to_smcu_index = 0;
     
	//open debug ring_buf info level
	ring_buf_debug = 0xF;

	switch (priv->fn_mode)
	{
		case ONLY_1_PF:
			smcu_to_host_index = ONLY_1PF_SMCU_TO_HOST_INFO_BUF;
			host_to_smcu_index = ONLY_1PF_HOST_TO_SMCU_CMD_BUF;
			break;
		
		case ONLY_2_PF:
			smcu_to_host_index = ONLY_2PF_SMCU_TO_HOST_INFO_BUF;
			host_to_smcu_index = ONLY_2PF_HOST_TO_SMCU_CMD_BUF;
			break;
		
		case ONLY_4_PF:
		case PF_4_WITH_SRIOV:
			smcu_to_host_index = COMMON_SMCU_TO_HOST_INFO_BUF;
			host_to_smcu_index = COMMON_HOST_TO_SMCU_CMD_BUF;
			break;
		
		default:
			break;
	}
		
	len = priv->ring_buf[smcu_to_host_index]->ring_buf_size;
	addr = priv->ring_buf[smcu_to_host_index]->ring_buf_base;

	dump_addr = kzalloc(len, GFP_KERNEL);
	vastai_pci_mem_read(priv,priv->pkg_id,addr,dump_addr,len);
	VASTAI_PCI_INFO(priv, priv->pkg_id, "%s, %d, dump smcu_to_host_ring_buf\n",__func__,__LINE__);
	for (i = 0; i < len; i += 16) {
		sprintf(print_str,
			"%s [dump_smcu_ring_buf] 0x%010llx: ",
			VASTAI_PCI_DEBUG_PREFIX, (addr + i));
			VASTAI_PCI_HEX_DUMP(KERN_WARNING, print_str,
					DUMP_PREFIX_NONE, 16, 4,
					dump_addr + i, 16, true);
	}

	len = priv->ring_buf[host_to_smcu_index]->ring_buf_size;
	addr = priv->ring_buf[host_to_smcu_index]->ring_buf_base;

	vastai_pci_mem_read(priv,priv->pkg_id,addr,dump_addr,len);
	VASTAI_PCI_INFO(priv, priv->pkg_id, "%s, %d, dump host_to_smcu_ring_buf\n",__func__,__LINE__);
	for (i = 0; i < len; i += 16) {
		sprintf(print_str,
			"%s [dump_smcu_ring_buf] 0x%010llx: ",
			VASTAI_PCI_DEBUG_PREFIX, (addr + i));
			VASTAI_PCI_HEX_DUMP(KERN_WARNING, print_str,
					DUMP_PREFIX_NONE, 16, 4,
					dump_addr+i, 16, true);
	}

	kfree(dump_addr);
	return;
}

static int vastai_submit_smcu_task(struct vastai_pci_info *priv, 
							struct smcu_task_info *smcu_task_desc)
{
	struct pcie_transfer_cmd trans;
	int ret = 0;
	mutex_lock(&priv->smcu_task.smcu_task_mutex);

	if (kfifo_is_full(&priv->smcu_task.smcu_task_fifo)) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "submit_smcu_task, %d, smcu read/write kfifo is full\n",
						__LINE__);
		VASTAI_PCI_ERR(priv, priv->pkg_id, "submit_smcu_task, %d, smcu read/write kfifo wr[%d]\n",
						__LINE__,smcu_rw_kfifo_in(&priv->smcu_task.smcu_task_fifo));
		VASTAI_PCI_ERR(priv, priv->pkg_id, "submit_smcu_task, %d, smcu read/write kfifo rd[%d]\n",
						__LINE__,smcu_rw_kfifo_out(&priv->smcu_task.smcu_task_fifo));
		goto err_free_lock;
	}

	ret = kfifo_in(&priv->smcu_task.smcu_task_fifo, smcu_task_desc, 1);
	if (ret != 1) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "kfifo in smcu_task_desc fail\n");
		goto err_free_lock;
	}

	trans.w0.data0	= smcu_task_desc->w0_data0;
	trans.w1.data1	= smcu_task_desc->w1_data1;
	trans.w2.data2	= smcu_task_desc->w2_data2;
	trans.w3.data3	= smcu_task_desc->w3_data3;

	ret = vastai_pci_send_msg(priv, 0, COMMON_HOST_TO_SMCU_CMD_BUF,  &trans, 0);
	if (ret) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "submit_smcu_task, %d, push element to [host_to_smcu buf] fail, ret=%d\n",
		                __LINE__,ret);
		decrease_kfifo_in(&priv->smcu_task.smcu_task_fifo,1);
		dump_smcu_ring_buf(priv);
	}
	 
err_free_lock:
	mutex_unlock(&priv->smcu_task.smcu_task_mutex);
	return 0;
}

static void vastai_smcu_task_complete(void *arg1, void* arg2)
{
	struct smcu_rw_ret_info *smcu_ret = arg1;
	struct pcie_transfer_cmd *smcu_to_host_info = arg2;
	if (smcu_ret) {
		smcu_ret->w0_data0 = smcu_to_host_info->w0.data0;
		smcu_ret->w1_data1 = smcu_to_host_info->w1.data1;
		smcu_ret->w2_data2 = smcu_to_host_info->w2.data2;
		smcu_ret->w3_data3 = smcu_to_host_info->w3.data3;
		complete(&(smcu_ret->smcu_task_done));
	}
	return;
}

int vastai_submit_smcu_task_and_done(struct vastai_pci_info *priv,
								struct smcu_task_info *desc,
								struct smcu_rw_ret_info *smcu_ret)
{
	int ret = 0;
	desc->callback = vastai_smcu_task_complete;
	desc->arg1     = smcu_ret;

	init_completion(&(smcu_ret->smcu_task_done));

	ret = vastai_submit_smcu_task(priv,desc);
	if(ret) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"submit smcu submit task opcode[%d] failed\n",
						desc->w0_data0 & 0xFF);
		return ret;
	}

	/* wait completion int from smcu */
	if (wait_for_completion_timeout(
			&(smcu_ret->smcu_task_done),
			msecs_to_jiffies(VASTAI_PCIE_WAIT_SMCU_MSG1_TIME)) <= 0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"submit_smcu_task_and_done, %d,smcu task opcode[%d] failed because timeout\n",
									__LINE__,desc->w0_data0 & 0xFF);
		dump_smcu_ring_buf(priv);
		return -ETIMEDOUT;
	}

	return ret;
}

static int vastai_read_32bit_by_smcu(struct vastai_pci_info *priv, 
							  u64 noc_addr,void *buf)
{
	int ret = 0;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};
	u64 smcu_ret_noc_addr = 0;
	u32 smcu_ret_val = 0;

	/*opcode: HOST_READ_CSR_BY_SMCU len=4*/
	smcu_task_desc.w0_data0 = (4<<8)|HOST_READ_CSR_BY_SMCU;
	smcu_task_desc.w1_data1 = noc_addr & 0xFFFFFFFF;
	smcu_task_desc.w2_data2 = noc_addr >> 32;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret)
		return ret;

	smcu_ret_noc_addr 	= (u64)smcu_ret.w2_data2<<32|smcu_ret.w1_data1;
	smcu_ret_val		= smcu_ret.w3_data3;

	if(noc_addr != smcu_ret_noc_addr) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"smcu_read_addr[0x%llx] != smcu_return_addr[0x%llx]\n",
					noc_addr,smcu_ret_noc_addr);
		return -EIO;
	}

	*((unsigned int *)buf) = smcu_ret_val;

#if SMCU_ACCESS_LOG
/*this is for debug */
{
	VASTAI_PCI_INFO(priv, 0,"[smcu-read] addr[0x%llx] val[%#x]\n",noc_addr, *((unsigned int *)buf));
}
#endif
	return 0;
}

static int vastai_write_32bit_by_smcu(struct vastai_pci_info *priv, 
							  u64 noc_addr, u32 val)
{
	int ret = 0;
	struct smcu_task_info     smcu_task_desc = {0};
	struct smcu_rw_ret_info smcu_ret = {0};

	u64 smcu_ret_noc_addr = 0;

	/*opcode:HOST_WRITE_CSR_BY_SMCU */
	smcu_task_desc.w0_data0 = HOST_WRITE_CSR_BY_SMCU;
	smcu_task_desc.w1_data1 = noc_addr & 0xFFFFFFFF;
	smcu_task_desc.w2_data2 = noc_addr >> 32;
	smcu_task_desc.w3_data3 = val;

	ret = vastai_submit_smcu_task_and_done(priv,&smcu_task_desc,&smcu_ret);
	if(ret)
		return ret;

	smcu_ret_noc_addr 	= (u64)smcu_ret.w2_data2<<32|smcu_ret.w1_data1;

	if(noc_addr != smcu_ret_noc_addr) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"smcu_read_addr[0x%llx] != smcu_return_addr[0x%llx]\n",
					noc_addr,smcu_ret_noc_addr);
		return -EIO;
	}

#if SMCU_ACCESS_LOG
/*this is for debug */
{
	VASTAI_PCI_INFO(priv, 0,"[smcu-write] addr[0x%llx] val[0x%x]\n",noc_addr, val);
}
#endif
	return 0;
}

int vastai_read_by_smcu(struct vastai_pci_info *priv, u64 noc_addr, unsigned int len ,void *buf)
{
	int ret;
	unsigned long old_ts = 0, now_ts = 0;
	unsigned int info_len = len;

	if (noc_addr % 4 != 0){
		VASTAI_PCI_ERR(priv, priv->pkg_id, "the csr addr=0x%llx is not align 4 byte\n", noc_addr);
		return -1;
	}
	if (len % 4 !=0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "read csr len=0x%x is not align 4 byte\n", len);
		return -1;
	}

	if(test_smcu_RW_Perf){
		old_ts= ktime_get();
	}
	while(len > 0) {
		ret = vastai_read_32bit_by_smcu(priv,noc_addr,buf);
		if(ret){
			VASTAI_PCI_ERR(priv, priv->pkg_id,"read_by_smcu %d,smcu_read addr[0x%llx] failed,ret=%d\n",
						__LINE__,noc_addr,ret);
			return -1;
		}
		len      -= 4;
		noc_addr += 4;
		buf      += 4;
	}
	
	if(test_smcu_RW_Perf){
		now_ts= ktime_get();
    	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"[smcu-read]addr[0x%llx] %d byte: %09lu(ns)=%06lu(us)\n",
		noc_addr, info_len ,(now_ts-old_ts),(now_ts-old_ts)/1000 );
	}
	return 0;
}
/* for dus test,need to remove */
EXPORT_SYMBOL(vastai_read_by_smcu);

// Func: read atomic text info from smcu
static int vastai_read_info_unit_from_smcu(struct vastai_pci_info *priv, u32 sub_info_type, u32 idx, u8 *buf, u32 req_unit_len)
{
	int ret = 0;
	struct smcu_task_info		smcu_task_desc = {0};
	struct smcu_rw_ret_info 	smcu_ret = {0};
	u32 smcu_ret_data_idx = 0;
	u32 smcu_ret_val = 0;

	VASTAI_PCI_DBG(priv, priv->pkg_id, "read_info_unit_from_smcu sub_info_type=%d, idx=%d, req_unit_len=%d\n", sub_info_type, idx, req_unit_len);

	/*opcode: HOST_READ_INVENTORY_CFG etc. */
	smcu_task_desc.w0_data0 = HOST_READ_SMCU_COMMON_INFO;   // not Reg/CSR info
	smcu_task_desc.w1_data1 = sub_info_type;
	smcu_task_desc.w2_data2 = idx;
    smcu_task_desc.w3_data3 = req_unit_len;

	ret = vastai_submit_smcu_task_and_done(priv, &smcu_task_desc, &smcu_ret);
	if(ret)
		return ret;

	smcu_ret_data_idx 	= smcu_ret.w1_data1;   // index used to check data sanity
	smcu_ret_val		= smcu_ret.w2_data2;   // 1st 4B reponse

	if(idx != smcu_ret_data_idx) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"idx read from smcu[%u] != smcu_return_idx[%u]\n", idx, smcu_ret_data_idx);
		return -EIO;
	}

	if(smcu_ret_val == CARD_ID_ERROR_DEFAULT_CODE) {
		VASTAI_PCI_ERR(priv, priv->pkg_id,"smcu ret idx=%u return with Error Code=%#X, %#X\n", smcu_ret_data_idx, smcu_ret_val, smcu_ret.w3_data3);
		return -ENXIO;
	}

	if(req_unit_len == MCU_REQ_UNIT_LEN_4B)
		memcpy(buf, (u8*)&smcu_ret_val, MCU_REQ_UNIT_LEN_4B);
	else {
		memcpy(buf, (u8*)&smcu_ret_val, MCU_REQ_UNIT_LEN_4B);
		smcu_ret_val      = smcu_ret.w3_data3;   // 2nd 4B response
		memcpy(buf + MCU_REQ_UNIT_LEN_4B, (u8*)&smcu_ret_val, MCU_REQ_UNIT_LEN_4B);
	}

	return 0;
}

// Func: read text common info collected by SMCU, which is not stored in CSR or DDR, like inventory code, dynamical status info etc. 
// Args: total_len, should be aligned to 4B, even through has some dummy data.
int vastai_read_info_from_smcu(struct vastai_pci_info *priv, u32 info_type, u32 total_len, u8 *buf)
{
	int ret = 0;
	u32 idx = 0;
	u32 req_unit_len = 0;  // only accept 4B/8B, total_len should be n * req_unit_len.

	VASTAI_PCI_DBG(priv, priv->pkg_id,  "read_info_from_smcu\n");

	if (total_len % MCU_REQ_UNIT_LEN_4B !=0 ) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "read info total_len=0x%x is not 4 byte aligned\n", total_len);
		return -1;
	}

	while(total_len > 0) {
		if(total_len % MCU_REQ_UNIT_LEN_8B != 0) {
			req_unit_len = MCU_REQ_UNIT_LEN_4B;
		} else {
			req_unit_len = MCU_REQ_UNIT_LEN_8B;
		}

		ret = vastai_read_info_unit_from_smcu(priv, info_type, idx, buf, req_unit_len);
		if(ret){
			VASTAI_PCI_ERR(priv, priv->pkg_id, "read_info_from_smcu type[%d] failed with Error=%d\n", info_type, ret);
			return ret;
		}

		total_len      -= req_unit_len;
        idx        	   += req_unit_len;
		buf            += req_unit_len;
	}

	return 0;
}
EXPORT_SYMBOL(vastai_read_info_from_smcu);

int vastai_write_by_smcu(struct vastai_pci_info *priv, u64 noc_addr, u32 val, u32 len)
{
	int ret;
	unsigned long old_ts = 0, now_ts = 0;
	unsigned int info_len = 4;

	if (noc_addr % 4 != 0){
		VASTAI_PCI_ERR(priv, priv->pkg_id, "the csr addr %#llx is not align 4 byte\n",noc_addr);
		return -1;
	}

	if (len % 4 !=0) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "write csr len=0x%x is not align 4 byte\n", len);
		return -1;
	}

	if(test_smcu_RW_Perf){
		old_ts= ktime_get();
	}
	
	while(len > 0) {
		ret = vastai_write_32bit_by_smcu(priv,noc_addr,val);
		if(ret) {
			VASTAI_PCI_ERR(priv, priv->pkg_id,"write_by_smcu, %d, smcu_write addr[0x%llx] failed,ret=%d\n",
							__LINE__,noc_addr,ret);
			return ret;
		}
			
		len      -= 4;
		noc_addr += 4;
	}

	if(test_smcu_RW_Perf){
		now_ts= ktime_get();
		VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"[smcu-write]%d byte: %09lu(ns)=%06lu(us)\n",
		info_len ,(now_ts-old_ts),(now_ts-old_ts)/1000 );
	}

	return 0;
}
/* for dus test,need to remove */
EXPORT_SYMBOL(vastai_write_by_smcu);

static void init_smcu_task(struct vastai_pci_info *priv)
{
	INIT_KFIFO(priv->smcu_task.smcu_task_fifo);
	mutex_init(&priv->smcu_task.smcu_task_mutex);
}

void vastai_get_pcie_attr(struct vastai_pci_info *priv)
{
	int ret;
	union link_control_status_reg reg;
	
	ret = vastai_pci_config_read(priv,0x80,(char *)&(reg.val), 4);
	if (ret < 0)
		VASTAI_PCI_ERR(NULL, DUMMY_DIE_ID, "read pcie link speed and width error \n");

	priv->pcie_nls = reg.bit.negotiated_link_speed;
	priv->pcie_nlw = reg.bit.negotiated_link_width;
	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "[PCIe Attr]Link Speed Gen[%d]\n",priv->pcie_nls);
	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "[PCIe Attr]Link Width[x%d]\n",priv->pcie_nlw);
	return;
}

#ifdef DIAG_TEST_MODE
int diag_vastai_pci_mem_read(struct vastai_pci_info *priv,
			int die_index,
			u64 relative_addr,
			void *buf,
			unsigned int len)
{
	if (relative_addr>0 && relative_addr<SPACE_SIZE_256M) {
		relative_addr += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}

	return vastai_pci_mem_read(priv,die_index,relative_addr,buf,len);
}

int diag_vastai_pci_mem_write(struct vastai_pci_info *priv,
			 int die_index,
			 u64 relative_addr,
			 void *buf,
			 unsigned int len)
{
	if (relative_addr>0 && relative_addr<SPACE_SIZE_256M) {
		relative_addr += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
	return vastai_pci_mem_write(priv,die_index,relative_addr,buf,len);
}
#endif

//mode:1 bit set 0:bit clear
int vastai_pci_modify_reg32(struct vastai_pci_info *priv,u64 addr,u32 mask_bit,u32 mode)
{
    u32 tmp_val = 0;
	int ret = 0;
	mutex_lock(&vastai_pci_modify_lock);

	ret = vastai_pci_mem_read(priv,0,addr,&tmp_val,4);
    tmp_val = (mode)?(tmp_val|(mask_bit)):(tmp_val&(~mask_bit));
	ret = vastai_pci_mem_write(priv,0,addr,&tmp_val,4);

	mutex_unlock(&vastai_pci_modify_lock);
	return ret;
}

//mode:1 bit set 0:bit clear
int vastai_smcu_modify_reg32(struct vastai_pci_info *priv,u64 addr,u32 mask_bit,u32 mode)
{
    u32 tmp_val = 0;
	int ret = 0;
	mutex_lock(&vastai_smcu_modify_lock);

	ret = vastai_read_by_smcu(priv,addr,4,&tmp_val);
    tmp_val = (mode)?(tmp_val|(mask_bit)):(tmp_val&(~mask_bit));
	ret = vastai_write_by_smcu(priv,addr,tmp_val,4);

	mutex_unlock(&vastai_smcu_modify_lock);
	return ret;
}

static void update_bar45_info(struct vastai_pci_info *priv)
{
	u8 config_bar4_5_sz = priv->priv_hw_cfg->sys_cfg.bar4_5_cfg_size;

	switch (config_bar4_5_sz) {
		case HWCFG_BAR4_5_2GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = BAR_SIZE_2G;
			break;
		case HWCFG_BAR4_5_4GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = LARGE_BAR_4GB_DDR_SIZE;
			break;
		case HWCFG_BAR4_5_8GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = LARGE_BAR_8GB_DDR_SIZE;
			break;
		case HWCFG_BAR4_5_16GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = LARGE_BAR_16GB_DDR_SIZE;
			break;
		case HWCFG_BAR4_5_32GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = LARGE_BAR_32GB_DDR_SIZE;
			break;
		case HWCFG_BAR4_5_64GB:
			priv->bar[VASTAI_PCI_BAR4].mmio_len = LARGE_BAR_64GB_DDR_SIZE;
			break;
		
		default:
			break;
	}

	VASTAI_PCI_INFO(priv, DUMMY_DIE_ID,"[BAR4]current pf ddr size = 0x%llx(%lldMB)\n",
									priv->bar[VASTAI_PCI_BAR4].mmio_len,
									priv->bar[VASTAI_PCI_BAR4].mmio_len/1024/1024);
	
#ifdef DIAG_TEST_WC
		if (priv->bar[VASTAI_PCI_BAR4].vmem != NULL) {
			iounmap(priv->bar[VASTAI_PCI_BAR4].vmem);
			priv->bar[VASTAI_PCI_BAR4].vmem = NULL;
		}
		priv->diag_vddr  = ioremap_wc(priv->bar[VASTAI_PCI_BAR4].mmio_start, 
				priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M);
		if(NULL == priv->diag_vddr) {
				VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
					"cannot remap diag ddr bar%d mmio, aborting\n",VASTAI_PCI_BAR4);
			
		}
		priv->diag_vcsr  = ioremap(priv->bar[VASTAI_PCI_BAR4].mmio_start + 
						priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M, 
						SPACE_SIZE_256M);
		if(NULL == priv->diag_vcsr) {
				VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
					"cannot remap diag ddr bar%d mmio, aborting\n", VASTAI_PCI_BAR4);
		}
		
		VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "BAR[%d] [diag_ddr_wc]len=0x%llx pa[0x%llx] va[0x%p]", VASTAI_PCI_BAR4, 
							priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M,
							priv->bar[VASTAI_PCI_BAR4].mmio_start , priv->diag_vddr);
		VASTAI_PCI_INFO(priv, DUMMY_DIE_ID, "BAR[%d] [diag_csr]len=0x%llx pa[0x%llx] va[0x%p]", VASTAI_PCI_BAR4, 
							SPACE_SIZE_256M,
							priv->bar[VASTAI_PCI_BAR4].mmio_start + priv->bar[VASTAI_PCI_BAR4].mmio_len - SPACE_SIZE_256M, 
							priv->diag_vcsr);
#endif
	return;
}

static int init_ddr_info(struct vastai_pci_info *priv)
{
	u8 config_bar4_5_sz		= priv->priv_hw_cfg->sys_cfg.bar4_5_cfg_size;
	u64 ddr_size 			= ((u64)priv->priv_hw_cfg->header.hi32_ddr_size<<32)|
                            	priv->priv_hw_cfg->header.lo32_ddr_size;
	u64 runtime_ddr_size 	 = 0;
	u64 gfx_display_ddr_size = 0;
	u64 gfx_heap_ddr_size	 = 0;
	u64	temp_noc_addr = 0;
	u64 temp_fn_addr = 0;
	u64 temp_cpu_pa = 0;
	u64 temp_bar_offset = 0;	
	u64 temp_size = 0;	
	int ret = 0;
	struct pcie_to_ddk_info *trans;

	priv->ddr_info.card_ddr_size = (u64)priv->priv_hw_cfg->sys_cfg.sum_ddr_sz * 0x40000000;
	priv->ddr_info.fn_ddr_size   = ddr_size;
	priv->ddr_info.cur_pf_vf_num = priv->priv_hw_cfg->sys_cfg.cur_pf_vf_num;
	if(priv->is_pf) {
		if(PF_4_WITH_SRIOV == priv->fn_mode){
			priv->ddr_info.vf_ddr_size   =(priv->ddr_info.fn_ddr_size - META_FW_SIZE_32M - VF_MANAGEMENT_SIZE_64M)
				/priv->ddr_info.cur_pf_vf_num/DDR_SIZE_16M_ALIGN*DDR_SIZE_16M_ALIGN;
		} else {
			priv->ddr_info.vf_ddr_size   = 0;
		}
		
	} else if(priv->is_vf) {
		priv->ddr_info.vf_ddr_size   = ddr_size;
	}

	priv->ddr_info.pcie_to_ddk_info_addr = kzalloc(sizeof(struct pcie_to_ddk_info), GFP_KERNEL);
	trans = (struct pcie_to_ddk_info*)priv->ddr_info.pcie_to_ddk_info_addr;

	trans->cur_pf_vf_num = priv->ddr_info.cur_pf_vf_num;
	trans->vf_ddr_size	 = priv->ddr_info.vf_ddr_size;
	trans->fn_ddr_size	 = priv->ddr_info.fn_ddr_size;
	
	if(priv->is_pf) {
		priv->ddr_info.ddr_entry[VF_MANAGE].fn_addr 	= VA_DDR_BASE_ADDR;
		priv->ddr_info.ddr_entry[VF_MANAGE].noc_addr 	= priv->bar[VASTAI_PCI_BAR4].noc_addr;
		priv->ddr_info.ddr_entry[VF_MANAGE].cpu_pa 	 	= priv->bar[VASTAI_PCI_BAR4].mmio_start;
		priv->ddr_info.ddr_entry[VF_MANAGE].bar_offset 	= 0;
		priv->ddr_info.ddr_entry[VF_MANAGE].size 		= VF_MANAGEMENT_SIZE_64M;

		priv->ddr_info.ddr_entry[META_FW].fn_addr 		= priv->ddr_info.ddr_entry[VF_MANAGE].fn_addr   + VF_MANAGEMENT_SIZE_64M;
		priv->ddr_info.ddr_entry[META_FW].noc_addr 		= priv->ddr_info.ddr_entry[VF_MANAGE].noc_addr  + VF_MANAGEMENT_SIZE_64M;
		priv->ddr_info.ddr_entry[META_FW].cpu_pa 	 	= priv->ddr_info.ddr_entry[VF_MANAGE].cpu_pa    + VF_MANAGEMENT_SIZE_64M;
		priv->ddr_info.ddr_entry[META_FW].bar_offset 	= priv->ddr_info.ddr_entry[VF_MANAGE].bar_offset+ VF_MANAGEMENT_SIZE_64M;
		priv->ddr_info.ddr_entry[META_FW].size 			= META_FW_SIZE_32M;

		/*fn_mode == PF_4_WITH_SRIOV, the pf is not run task, other pf is normal*/
		if(PF_4_WITH_SRIOV != priv->fn_mode){
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr 		= priv->ddr_info.ddr_entry[META_FW].fn_addr  + META_FW_SIZE_32M;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr 		= priv->ddr_info.ddr_entry[META_FW].noc_addr + META_FW_SIZE_32M;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa 	 	= priv->ddr_info.ddr_entry[META_FW].cpu_pa + META_FW_SIZE_32M;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset 	= priv->ddr_info.ddr_entry[META_FW].bar_offset + META_FW_SIZE_32M;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].size 			= ENC_AND_DEC_BUF_SIZE_16M;
		} else {
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr 		= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].size 			= 0;

			priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].noc_addr 	= 0;
			priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].size 		= 0;

			priv->ddr_info.ddr_entry[SMMU_CD_TABLE].fn_addr 	= 0;
			priv->ddr_info.ddr_entry[SMMU_CD_TABLE].noc_addr 	= 0;
			priv->ddr_info.ddr_entry[SMMU_CD_TABLE].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[SMMU_CD_TABLE].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[SMMU_CD_TABLE].size 		= 0;

			priv->ddr_info.ddr_entry[RESERVED_DDR].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr 	= 0;
			priv->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[RESERVED_DDR].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[RESERVED_DDR].size 		= 0;

			priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr 	= 0;
			priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[GPU_HEAP].size 		= 0;

			priv->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr 		= 0;
			priv->ddr_info.ddr_entry[GPU_DISPLAY].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[GPU_DISPLAY].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[GPU_DISPLAY].size 			= 0;

			priv->ddr_info.ddr_entry[RUNTIME].fn_addr 		= 0;
			priv->ddr_info.ddr_entry[RUNTIME].noc_addr 		= 0;
			priv->ddr_info.ddr_entry[RUNTIME].cpu_pa 	 	= 0;
			priv->ddr_info.ddr_entry[RUNTIME].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[RUNTIME].size 			= 0;
			goto end;
		}
		
	} else if(priv->is_vf){
		priv->ddr_info.ddr_entry[VF_MANAGE].fn_addr 	= 0;
		priv->ddr_info.ddr_entry[VF_MANAGE].noc_addr 	= 0;
		priv->ddr_info.ddr_entry[VF_MANAGE].cpu_pa 	 	= 0;
		priv->ddr_info.ddr_entry[VF_MANAGE].bar_offset 	= 0;
		priv->ddr_info.ddr_entry[VF_MANAGE].size 		= 0;

		priv->ddr_info.ddr_entry[META_FW].fn_addr 		= 0;
		priv->ddr_info.ddr_entry[META_FW].noc_addr 		= 0;
		priv->ddr_info.ddr_entry[META_FW].cpu_pa 	 	= 0;
		priv->ddr_info.ddr_entry[META_FW].bar_offset 	= 0;
		priv->ddr_info.ddr_entry[META_FW].size 			= 0;
		
		if(DECODER_SRIOV_ENABLE == priv->priv_hw_cfg->sys_cfg.decoder_sriov_en) {
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr 		= ((u64)priv->priv_hw_cfg->header.hi32_video_buf_base<<32)|priv->priv_hw_cfg->header.lo32_video_buf_base;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr 		= ((u64)priv->priv_hw_cfg->header.hi32_video_buf_base<<32)|priv->priv_hw_cfg->header.lo32_video_buf_base;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa 	 	= priv->bar[VASTAI_PCI_BAR4].mmio_start;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].size 			= VEDIO_CMD_BUF_SIZE;

			temp_noc_addr = priv->bar[VASTAI_PCI_BAR4].noc_addr;
			temp_fn_addr = VA_DDR_BASE_ADDR;
			temp_cpu_pa = priv->bar[VASTAI_PCI_BAR4].mmio_start;
			temp_bar_offset = 0;	
			temp_size = ENC_AND_DEC_BUF_SIZE_16M;

		} else {
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr 		= VA_DDR_BASE_ADDR;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr 		= priv->bar[VASTAI_PCI_BAR4].noc_addr;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa 	 	= priv->bar[VASTAI_PCI_BAR4].mmio_start;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset 	= 0;
			priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].size 			= ENC_AND_DEC_BUF_SIZE_16M;
		}	
	}

	 
	if(DECODER_SRIOV_ENABLE == priv->priv_hw_cfg->sys_cfg.decoder_sriov_en) {
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].fn_addr 		= temp_fn_addr  + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].noc_addr 	= temp_noc_addr + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].cpu_pa 	 	= temp_cpu_pa + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].bar_offset 	= temp_bar_offset + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].size 		= AI_MODLE_AND_OPCODE_SIZE_16M;
	} else {
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].fn_addr 		= priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr  + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].noc_addr 	= priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].cpu_pa 	 	= priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].bar_offset 	= priv->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset + ENC_AND_DEC_BUF_SIZE_16M;
		priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].size 		= AI_MODLE_AND_OPCODE_SIZE_16M;
	}
	

	priv->ddr_info.ddr_entry[SMMU_CD_TABLE].fn_addr 	= priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].fn_addr  + AI_MODLE_AND_OPCODE_SIZE_16M;
	priv->ddr_info.ddr_entry[SMMU_CD_TABLE].noc_addr 	= priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].noc_addr + AI_MODLE_AND_OPCODE_SIZE_16M;
	priv->ddr_info.ddr_entry[SMMU_CD_TABLE].cpu_pa 	 	= priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].cpu_pa + AI_MODLE_AND_OPCODE_SIZE_16M;
	priv->ddr_info.ddr_entry[SMMU_CD_TABLE].bar_offset 	= priv->ddr_info.ddr_entry[AI_MODULE_OPCODE].bar_offset + AI_MODLE_AND_OPCODE_SIZE_16M;
	priv->ddr_info.ddr_entry[SMMU_CD_TABLE].size 		= SMMU_CD_TABLE_SIZE_1M;

	priv->ddr_info.ddr_entry[RESERVED_DDR].fn_addr 		= priv->ddr_info.ddr_entry[SMMU_CD_TABLE].fn_addr  + SMMU_CD_TABLE_SIZE_1M;
	priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr 	= priv->ddr_info.ddr_entry[SMMU_CD_TABLE].noc_addr + SMMU_CD_TABLE_SIZE_1M;
	priv->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa 	 	= priv->ddr_info.ddr_entry[SMMU_CD_TABLE].cpu_pa + SMMU_CD_TABLE_SIZE_1M;
	priv->ddr_info.ddr_entry[RESERVED_DDR].bar_offset 	= priv->ddr_info.ddr_entry[SMMU_CD_TABLE].bar_offset + SMMU_CD_TABLE_SIZE_1M;
	priv->ddr_info.ddr_entry[RESERVED_DDR].size 		= BAR_RESERVED_SIZE_31M;
	

	if(ZHAOGE_VG600 == priv->priv_hw_cfg->sys_cfg.card_type) {
		if(priv->is_vf){
			gfx_display_ddr_size	= 0;
			gfx_heap_ddr_size		= 0;
			runtime_ddr_size        = ddr_size - VF_USED_SIZE;
		} else if(priv->is_pf) {
			gfx_display_ddr_size	= 0;
			gfx_heap_ddr_size		= 0;
			runtime_ddr_size        = ddr_size - PF_USED_SIZE;
		}
		
		priv->ddr_info.ddr_entry[RUNTIME].fn_addr 		= priv->ddr_info.ddr_entry[RESERVED_DDR].fn_addr  + BAR_RESERVED_SIZE_31M;
		priv->ddr_info.ddr_entry[RUNTIME].noc_addr 		= priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr + BAR_RESERVED_SIZE_31M;
		priv->ddr_info.ddr_entry[RUNTIME].cpu_pa 	 	= priv->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa + BAR_RESERVED_SIZE_31M;
		priv->ddr_info.ddr_entry[RUNTIME].bar_offset 	= priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr + BAR_RESERVED_SIZE_31M;
		priv->ddr_info.ddr_entry[RUNTIME].size 			= runtime_ddr_size;

		priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr 		= 0;
		priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr 	= 0;
		priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa 	 	= 0;
		priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset 	= 0;
		priv->ddr_info.ddr_entry[GPU_HEAP].size 		= 0;

		priv->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr 		= 0;
		priv->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr 		= 0;
		priv->ddr_info.ddr_entry[GPU_DISPLAY].cpu_pa 	 	= 0;
		priv->ddr_info.ddr_entry[GPU_DISPLAY].bar_offset 	= 0;
		priv->ddr_info.ddr_entry[GPU_DISPLAY].size 			= 0;

		goto end;
	}

		switch (config_bar4_5_sz) {
            case HWCFG_BAR4_5_256MB:		
				gfx_display_ddr_size	= VF_DISP_HEAP_SIZE;
				gfx_heap_ddr_size		= VF_GFX_HEAP_SIZE;
				runtime_ddr_size        = ddr_size - VF_DDR_MMIO_SIZE;
				break;

			case HWCFG_BAR4_5_512MB:		
				gfx_display_ddr_size	= VF_LARGE_BAR_512MB_DISP_HEAP_SIZE;
				gfx_heap_ddr_size		= VF_LARGE_BAR_512MB_GFX_HEAP_SIZE;
				runtime_ddr_size        = ddr_size - VF_LARGE_BAR_512MB_DDR_SIZE;
				break;
			
			case HWCFG_BAR4_5_1GB:		
				gfx_display_ddr_size	= VF_LARGE_BAR_1GB_DISP_HEAP_SIZE;
				gfx_heap_ddr_size		= VF_LARGE_BAR_1GB_GFX_HEAP_SIZE;
				runtime_ddr_size        = ddr_size - VF_LARGE_BAR_1GB_DISP_HEAP_SIZE - VF_LARGE_BAR_1GB_GFX_HEAP_SIZE - VF_USED_SIZE;
				break;

            case HWCFG_BAR4_5_2GB:
                runtime_ddr_size        = ddr_size - BAR_SIZE_2G;
				gfx_heap_ddr_size 		= SG100_GFX_HEAP_SIZE;
				gfx_display_ddr_size 	= SG100_DISP_HEAP_SIZE;
                break;

            case HWCFG_BAR4_5_4GB:
                runtime_ddr_size          = LARGE_BAR_4GB_PER_PF_RUN_TIME_SIZE;
				gfx_heap_ddr_size 		  = LARGE_BAR_4GB_GFX_HEAP_SIZE;
#ifdef DIAG_TEST_DDK
				gfx_display_ddr_size 	  = LARGE_BAR_4GB_DISP_HEAP_SIZE - DEV_DDR_SIZE_256MB;
#else
				gfx_display_ddr_size 	  = LARGE_BAR_4GB_DISP_HEAP_SIZE;
#endif
                break;

            case HWCFG_BAR4_5_8GB:
                runtime_ddr_size          = LARGE_BAR_8GB_PER_PF_RUN_TIME_SIZE;
				gfx_heap_ddr_size 		  = LARGE_BAR_8GB_GFX_HEAP_SIZE;
#ifdef DIAG_TEST_DDK
				gfx_display_ddr_size 	  = LARGE_BAR_8GB_DISP_HEAP_SIZE - DEV_DDR_SIZE_256MB;
#else
				gfx_display_ddr_size 	  = LARGE_BAR_8GB_DISP_HEAP_SIZE;
#endif
                break;
            case HWCFG_BAR4_5_16GB:
                runtime_ddr_size          = LARGE_BAR_16GB_PER_PF_RUN_TIME_SIZE;
				gfx_heap_ddr_size 		  = LARGE_BAR_16GB_GFX_HEAP_SIZE;
#ifdef DIAG_TEST_DDK
				gfx_display_ddr_size 	  = LARGE_BAR_16GB_DISP_HEAP_SIZE - DEV_DDR_SIZE_256MB;
#else
				gfx_display_ddr_size 	  = LARGE_BAR_16GB_DISP_HEAP_SIZE;
#endif
                break;

            case HWCFG_BAR4_5_32GB:
                runtime_ddr_size          = LARGE_BAR_32GB_PER_PF_RUN_TIME_SIZE;
				gfx_heap_ddr_size 		  = LARGE_BAR_32GB_GFX_HEAP_SIZE;
#ifdef DIAG_TEST_DDK
				gfx_display_ddr_size 	  = LARGE_BAR_32GB_DISP_HEAP_SIZE - DEV_DDR_SIZE_256MB;
#else
				gfx_display_ddr_size 	  = LARGE_BAR_32GB_DISP_HEAP_SIZE;
#endif
                break;

            case HWCFG_BAR4_5_64GB:
                runtime_ddr_size          = LARGE_BAR_64GB_PER_PF_RUN_TIME_SIZE;
				gfx_heap_ddr_size 		  = LARGE_BAR_32GB_GFX_HEAP_SIZE;
#ifdef DIAG_TEST_DDK
			gfx_display_ddr_size 	  = LARGE_BAR_64GB_DISP_HEAP_SIZE - DEV_DDR_SIZE_256MB;
#else
			gfx_display_ddr_size 	  = LARGE_BAR_64GB_DISP_HEAP_SIZE;
#endif
                break;            
            default:
				ret = -1;
				VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s config_bar4_5_sz error\n", __func__);
				goto end;
                break;
        }

		switch (config_bar4_5_sz) {
			case HWCFG_BAR4_5_256MB:
			case HWCFG_BAR4_5_512MB:
			case HWCFG_BAR4_5_1GB:
			case HWCFG_BAR4_5_2GB:
				priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr 		= priv->ddr_info.ddr_entry[RESERVED_DDR].fn_addr  + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr 	= priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa 	 	= priv->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset 	= priv->ddr_info.ddr_entry[RESERVED_DDR].bar_offset + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[GPU_HEAP].size 		= gfx_heap_ddr_size;

				priv->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr 		= priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr  + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr 		= priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].cpu_pa 	 	= priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].bar_offset 	= priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].size 			= gfx_display_ddr_size;

				priv->ddr_info.ddr_entry[RUNTIME].fn_addr 		= priv->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr  + gfx_display_ddr_size;
				priv->ddr_info.ddr_entry[RUNTIME].noc_addr 		= priv->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr + gfx_display_ddr_size;
				priv->ddr_info.ddr_entry[RUNTIME].cpu_pa 	 	= 0;
				priv->ddr_info.ddr_entry[RUNTIME].bar_offset 	= 0;
				priv->ddr_info.ddr_entry[RUNTIME].size 			= runtime_ddr_size;
				break;

			case HWCFG_BAR4_5_4GB:
			case HWCFG_BAR4_5_8GB:
			case HWCFG_BAR4_5_16GB:
			case HWCFG_BAR4_5_32GB:
			case HWCFG_BAR4_5_64GB:
				priv->ddr_info.ddr_entry[RUNTIME].fn_addr 		= priv->ddr_info.ddr_entry[RESERVED_DDR].fn_addr  + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[RUNTIME].noc_addr 		= priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[RUNTIME].cpu_pa 	 	= priv->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[RUNTIME].bar_offset 	= priv->ddr_info.ddr_entry[RESERVED_DDR].noc_addr + BAR_RESERVED_SIZE_31M;
				priv->ddr_info.ddr_entry[RUNTIME].size 			= runtime_ddr_size;

				priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr 		= priv->ddr_info.ddr_entry[RUNTIME].fn_addr  + runtime_ddr_size;
				priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr 	= priv->ddr_info.ddr_entry[RUNTIME].noc_addr + runtime_ddr_size;
				priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa 	 	= priv->ddr_info.ddr_entry[RUNTIME].cpu_pa + runtime_ddr_size;
				priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset 	= priv->ddr_info.ddr_entry[RUNTIME].bar_offset + runtime_ddr_size;
				priv->ddr_info.ddr_entry[GPU_HEAP].size 		= gfx_heap_ddr_size;

				priv->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr 		= priv->ddr_info.ddr_entry[GPU_HEAP].fn_addr  + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr 		= priv->ddr_info.ddr_entry[GPU_HEAP].noc_addr + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].cpu_pa 	 	= priv->ddr_info.ddr_entry[GPU_HEAP].cpu_pa + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].bar_offset 	= priv->ddr_info.ddr_entry[GPU_HEAP].bar_offset + gfx_heap_ddr_size;
				priv->ddr_info.ddr_entry[GPU_DISPLAY].size 			= gfx_display_ddr_size;
				break;			
			default:
				ret = -1;
				VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,"%s config_bar4_5_sz error\n", __func__);
				break;
		}
end:
	return ret;
}



int init_vastai_pci_mem(struct vastai_pci_info *priv)
{
    unsigned long target_addr_64b = 0;
	int ret = 0;
	u64 csram_size    = priv->priv_hw_cfg->header.csram_size;
	u64 smu_csr_size  = priv->priv_hw_cfg->header.smu_csr_size;
	u64 gfx_csr_size  = priv->priv_hw_cfg->header.gfx_csr_size; 
    if ((!priv) || (!priv->dev)) {
        VASTAI_PCI_ERR(priv, DUMMY_DIE_ID,
                   "init_pci_mem priv 0x%p or dev 0x%p is NULL\n",
                   priv, priv->dev);
        return -ENXIO;
    }
    
    update_bar45_info(priv);
    
	init_smcu_task(priv);
	
	priv->bar[VASTAI_PCI_BAR1].noc_addr     = PCIE_WRAPPER_ADDR;
	priv->bar[VASTAI_PCI_BAR1].fn_base_addr = PCIE_WRAPPER_ADDR;

	target_addr_64b = priv->priv_hw_cfg->header.csram_base;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].noc_addr 		= target_addr_64b;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].region_size 	= csram_size;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].bar_offset  	= 0;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_01].fn_base_addr  = FN_CSRAM_BASE_ADDR;

	target_addr_64b = priv->priv_hw_cfg->header.smu_csr_base;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].noc_addr 		= target_addr_64b;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].region_size 	= smu_csr_size;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].bar_offset  	= csram_size;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_SMU_CSR].fn_base_addr   = FN_SMU_CSR_BASE_ADDR;

	target_addr_64b = priv->priv_hw_cfg->header.gfx_csr_base;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].noc_addr 		= target_addr_64b;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].region_size 	= gfx_csr_size;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].bar_offset  	= csram_size + smu_csr_size;
	priv->bar[VASTAI_PCI_BAR2].regions[BAR2_GFX_CSR].fn_base_addr   = FN_GFX_CSR_BASE_ADDR;
	priv->bar[VASTAI_PCI_BAR2].region_cnt                           = BAR2_GFX_CSR+1;

	if (priv->is_pf) {
		target_addr_64b = 0x8E0000;
		priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].noc_addr 		= target_addr_64b;
		priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].region_size 	= smu_csr_size;
		priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].bar_offset  	= csram_size + smu_csr_size + gfx_csr_size;
		priv->bar[VASTAI_PCI_BAR2].regions[BAR2_CSRAM_02].fn_base_addr  = 0x8E0000;
		priv->bar[VASTAI_PCI_BAR2].region_cnt++;
	}

#ifdef DIAG_TEST_MODE
	if (priv->is_pf) {
		priv->bar[VASTAI_PCI_BAR4].noc_addr = (u64)((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base;
		target_addr_64b = priv->bar[VASTAI_PCI_BAR4].noc_addr;
		priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr 		= target_addr_64b;
		priv->bar[VASTAI_PCI_BAR4].regions[0].region_size 	= priv->bar[VASTAI_PCI_BAR4].mmio_len;
		priv->bar[VASTAI_PCI_BAR4].regions[0].bar_offset  	= 0;
		priv->bar[VASTAI_PCI_BAR4].regions[0].fn_base_addr  = FN_DDR_BASE_ADDR;
		priv->bar[VASTAI_PCI_BAR4].region_cnt++;

		target_addr_64b = 0;
		priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr 		= 0;
		priv->bar[VASTAI_PCI_BAR4].regions[1].region_size 	= 0;
		priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset  	= 0;
		priv->bar[VASTAI_PCI_BAR4].regions[1].fn_base_addr  = 0;
		priv->bar[VASTAI_PCI_BAR4].region_cnt++;
	}	
	
#else
	if(priv->priv_hw_cfg->sys_cfg.decoder_sriov_en && priv->priv_hw_cfg->sys_cfg.vf_active){
		priv->bar[VASTAI_PCI_BAR4].noc_addr     = ((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base;
		priv->bar[VASTAI_PCI_BAR4].fn_base_addr = FN_DDR_BASE_ADDR;
		priv->bar[VASTAI_PCI_BAR4].regions[0].noc_addr 		= ((u64)priv->priv_hw_cfg->header.hi32_video_buf_base<<32)|priv->priv_hw_cfg->header.lo32_video_buf_base;
		priv->bar[VASTAI_PCI_BAR4].regions[0].region_size 	= priv->priv_hw_cfg->header.video_buf_size;
		priv->bar[VASTAI_PCI_BAR4].regions[0].bar_offset  	= 0;
		priv->bar[VASTAI_PCI_BAR4].regions[0].fn_base_addr  = FN_DDR_BASE_ADDR;
		priv->bar[VASTAI_PCI_BAR4].region_cnt++;

		priv->bar[VASTAI_PCI_BAR4].regions[1].noc_addr 		= (((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base)
															 + priv->priv_hw_cfg->header.video_buf_size;
		priv->bar[VASTAI_PCI_BAR4].regions[1].region_size 	= priv->bar[VASTAI_PCI_BAR4].mmio_len - priv->priv_hw_cfg->header.video_buf_size;
		priv->bar[VASTAI_PCI_BAR4].regions[1].bar_offset  	= priv->priv_hw_cfg->header.video_buf_size;
		priv->bar[VASTAI_PCI_BAR4].regions[1].fn_base_addr  = FN_DDR_BASE_ADDR + priv->priv_hw_cfg->header.video_buf_size;
		priv->bar[VASTAI_PCI_BAR4].region_cnt++;	

    } else {
		priv->bar[VASTAI_PCI_BAR4].noc_addr     = ((u64)priv->priv_hw_cfg->header.hi32_ddr_base<<32)|priv->priv_hw_cfg->header.lo32_ddr_base;
		priv->bar[VASTAI_PCI_BAR4].fn_base_addr = FN_DDR_BASE_ADDR;
	}
	
#endif

	ret = init_ddr_info(priv);
	return ret;
}

static inline int vastai_pci_is_32_bit_thread(void)
{
	int is_32_bit = 0;
#if (defined(__aarch64__) || defined(__arm__))
	is_32_bit = test_thread_flag(TIF_32BIT);
#else
	is_32_bit = test_thread_flag(TIF_ADDR32);
#endif

	return is_32_bit;
}

unsigned long vastai_pci_copy_to_user(void __user *to, const void *from, unsigned long n)
{
	unsigned long ret = 0;
#ifdef CONFIG_COMPAT
	if (vastai_pci_is_32_bit_thread()) {
		ret = copy_to_user(
			(void __user *)compat_ptr((compat_uptr_t)(((u64)to) & 0xffffffff)),
			from, n);
		if (ret) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"pci_copy_to_user: fail: 32bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	} else
#endif
	{
		ret = copy_to_user(to, from, n);
		if (ret) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"pci_copy_to_user: fail: 64bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	}
	return 0;
}

unsigned long vastai_pci_copy_from_user(void *to, const void __user *from, unsigned long n)
{
	unsigned long ret = 0;
#ifdef CONFIG_COMPAT
	if (vastai_pci_is_32_bit_thread()) {
		ret = copy_from_user(
			to,
			(void __user *)compat_ptr((compat_uptr_t)(((u64)from) & 0xffffffff)),
			n);
		if (ret) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"pci_copy_from_user: fail: 32bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	} else
#endif
	{
		ret = copy_from_user(to, from, n);
		if (ret) {
			VASTAI_PCI_ERR(
				NULL, DUMMY_DIE_ID,
				"pci_copy_from_user: fail: 64bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	}
	return 0;
}

