########################################################################### ###
#@Copyright     Copyright (c) Imagination Technologies Ltd. All Rights Reserved
#@License       Dual MIT/GPLv2
#
# The contents of this file are subject to the MIT license as set out below.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# Alternatively, the contents of this file may be used under the terms of
# the GNU General Public License Version 2 ("GPL") in which case the provisions
# of GPL are applicable instead of those above.
#
# If you wish to allow use of your version of this file only under the terms of
# GPL, and not to allow others to use your version of this file under the terms
# of the MIT license, indicate your decision by deleting the provisions above
# and replace them with the notice and other provisions required by GPL as set
# out in the file called "GPL-COPYING" included in this distribution. If you do
# not delete the provisions above, a recipient may use your version of this file
# under the terms of either the MIT license or GPL.
#
# This License is also included in this distribution in the file called
# "MIT-COPYING".
#
# EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
# PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
### ###########################################################################

GIT_BRANCH_NAME=$(shell git rev-parse --abbrev-ref HEAD)
GIT_BRANCH_NAME_FORMAT=$(shell echo "$(GIT_BRANCH_NAME)" | sed 's/[//]/-/g')
GIT_COMMIT_HASH=$(shell git log -1 --format="%h")
GIT_COMMIT_MSG=$(shell git log -1 --format="%s")
GIT_COMMIT_MSG_FORMAT=$(shell echo "$(GIT_COMMIT_MSG)" | sed 's/["]//g')
MKPACKAGE_TIME=$(shell date +"%Y%m%d_%H%M%S")
GIT_COMMON_INCLUDE_HASH=$(shell cd common && git log -1 --format="%h")

BIN2HEX_TOOL := $(TOP)/fw/bin2hex
BIN2HEX_SRC := $(TOP)/fw/bin2hex.c
bl1_file := $(TOP)/fw/va_bl1.bin
bl1_target := $(TOP)/fw/va_bl1.bin.hex
fw_file := $(TOP)/fw/va_fw.bin
fw_target := $(TOP)/fw/va_fw.bin.hex
bl0_base_file := $(TOP)/fw/va_bl0_base.bin
bl0_base_target := $(TOP)/fw/va_bl0_base.bin.hex
bl0_vbios_file := $(TOP)/fw/va_bl0_vbios.bin
bl0_vbios_target := $(TOP)/fw/va_bl0_vbios.bin.hex
bl0_file := $(TOP)/fw/va_bl0.bin
bl0_target := $(TOP)/fw/va_bl0.bin.hex
bl0_full_file := $(TOP)/fw/va_bl0_full.bin
bl0_full_target := $(TOP)/fw/va_bl0_full.bin.hex
bl0_all_file := $(TOP)/fw/va_bl0_all.bin
bl0_all_target := $(TOP)/fw/va_bl0_all.bin.hex
bl0_pmcu_file := $(TOP)/fw/va_bl0_pmcu.bin
bl0_pmcu_target := $(TOP)/fw/va_bl0_pmcu.bin.hex
bmcu_file := $(TOP)/fw/va_bmcu.bin
bmcu_target := $(TOP)/fw/va_bmcu.bin.hex
mcu_bin_target = $(wildcard $(TOP)/fw/bin_merge/*.bin)
GIT_DIR := $(TOP)/../.git

$(TARGET_PRIMARY_OUT)/kbuild/Makefile: $(MAKE_TOP)/kbuild/Makefile.template
	@[ ! -e $(dir $@) ] && mkdir -p $(dir $@) || true
	$(CP) -f $< $@

# We need to make INTERNAL_KBUILD_MAKEFILES absolute because the files will be
# read while chdir'd into $(KERNELDIR)
ifeq ($(SUPPORT_DDK_COMPILE),1)
INTERNAL_KBUILD_MAKEFILES := $(abspath $(foreach _m,$(KERNEL_COMPONENTS) $(EXTRA_PVRSRVKM_COMPONENTS),$(if $(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)),$(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)),$(error Unknown kbuild module "$(_m)"))))
INTERNAL_KBUILD_OBJECTS := $(foreach _m,$(KERNEL_COMPONENTS),$(if $(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(error BUG: Unknown kbuild module "$(_m)" should have been caught earlier)))
INTERNAL_EXTRA_KBUILD_OBJECTS := $(foreach _m,$(EXTRA_PVRSRVKM_COMPONENTS),$(if $(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(error BUG: Unknown kbuild module "$(_m)" should have been caught earlier)))
else
INTERNAL_KBUILD_MAKEFILES := $(abspath $(foreach _m,$(KERNEL_COMPONENTS) $(EXTRA_PVRSRVKM_COMPONENTS),$(if $(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)),$(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)),$(warning Unknown kbuild module "$(_m)"))))
INTERNAL_KBUILD_OBJECTS := $(foreach _m,$(KERNEL_COMPONENTS),$(if $(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(warning BUG: Unknown kbuild module "$(_m)" should have been caught earlier)))
INTERNAL_EXTRA_KBUILD_OBJECTS := $(foreach _m,$(EXTRA_PVRSRVKM_COMPONENTS),$(if $(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(INTERNAL_KBUILD_OBJECTS_FOR_$(_m)),$(warning BUG: Unknown kbuild module "$(_m)" should have been caught earlier)))
endif
.PHONY: kbuild kbuild_clean kbuild_check

kbuild_check:
	@: $(if $(strip $(KERNELDIR)),,$(error KERNELDIR must be set))
	@: $(call directory-must-exist,$(KERNELDIR))
	@: $(foreach _m,$(ALL_KBUILD_MODULES),$(if $(wildcard $(abspath $(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)))),,$(error In makefile $(INTERNAL_MAKEFILE_FOR_MODULE_$(_m)): Module $(_m) requires kbuild makefile $(INTERNAL_KBUILD_MAKEFILE_FOR_$(_m)), which is missing)))
	@: $(if $(filter-out command line override,$(origin build)),,$(error Overriding $$(build) (with "make build=...") will break kbuild))

driver_version:
ifeq ($(wildcard $(GIT_DIR)),$(GIT_DIR))
	$(info $(GIT_BRANCH_NAME),$(GIT_COMMIT_HASH),"$(GIT_COMMIT_MSG_FORMAT)",$(MKPACKAGE_TIME),$(GIT_COMMON_INCLUDE_HASH))
	$(shell sed -i 's/^.*VASTAI_PCI_GIT_BRANCH_NAME.*/#define VASTAI_PCI_GIT_BRANCH_NAME "$(GIT_BRANCH_NAME_FORMAT)"/g' $(TOP)/include/vastai_version.h)
	$(shell sed -i 's/^.*VASTAI_PCI_GIT_COMMIT_HASH.*/#define VASTAI_PCI_GIT_COMMIT_HASH "$(GIT_COMMIT_HASH)"/g' $(TOP)/include/vastai_version.h)
#	$(shell sed -i 's/^.*VASTAI_PCI_GIT_COMMIT_MSG.*/#define VASTAI_PCI_GIT_COMMIT_MSG "$(GIT_COMMIT_MSG_FORMAT)"/g' $(TOP)/include/vastai_version.h))
	$(shell sed -i 's/^.*VASTAI_PCI_MKPACKAGE_TIME.*/#define VASTAI_PCI_MKPACKAGE_TIME  "$(MKPACKAGE_TIME)"/g' $(TOP)/include/vastai_version.h)
	$(shell sed -i 's/^.*VASTAI_PCI_COMMON_INCLUDE_COMMIT_HASH.*/#define VASTAI_PCI_COMMON_INCLUDE_COMMIT_HASH  "$(GIT_COMMON_INCLUDE_HASH)"/g' $(TOP)/include/vastai_version.h)
else
	$(info No $(GIT_DIR) directory, no setting driver version.)
endif

$(fw_file):$(mcu_bin_target) FORCE
	@echo build bin_merge
	@make -C ./fw/bin_merge bin_merge

$(BIN2HEX_TOOL):$(BIN2HEX_SRC) $(fw_file) FORCE
	@echo build bin2hex
	@rm -rf $(BIN2HEX_TOOL)
	@gcc -o $(BIN2HEX_TOOL) $(BIN2HEX_SRC)

$(bl1_target):$(bl1_file) $(BIN2HEX_TOOL) FORCE
	@echo convert $< to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $<
 
$(fw_target):$(BIN2HEX_TOOL) FORCE
	@echo convert $(fw_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(fw_file)

$(bl0_base_target):$(BIN2HEX_TOOL) $(bl0_base_file) FORCE
	@echo convert $(bl0_base_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_base_file)

$(bl0_vbios_target):$(BIN2HEX_TOOL) $(bl0_vbios_file) FORCE
	@echo convert $(bl0_vbios_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_vbios_file)

$(bl0_target):$(BIN2HEX_TOOL) $(bl0_file) FORCE
	@echo convert $(bl0_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_file)

$(bl0_full_target):$(BIN2HEX_TOOL) $(bl0_full_file) FORCE
	@echo convert $(bl0_full_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_full_file)

$(bl0_all_target):$(BIN2HEX_TOOL) $(bl0_all_file) FORCE
	@echo convert $(bl0_all_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_all_file)

$(bl0_pmcu_target):$(BIN2HEX_TOOL) $(bl0_pmcu_file) FORCE
	@echo convert $(bl0_pmcu_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bl0_pmcu_file)

$(bmcu_target):$(BIN2HEX_TOOL) $(bmcu_file) FORCE
	@echo convert $(bmcu_file) to $@
	@chmod +x $(BIN2HEX_TOOL)
	@$(BIN2HEX_TOOL) $(bmcu_file)

FORCE:;


# Services server headers are generated as part of running the bridge
# generator, which might be included in KM code. So as well as depending on
# the kbuild Makefile, we need to make kbuild also depend on each bridge
# module (including direct bridges), so that 'make kbuild' in a clean tree
# works.
kbuild: kbuild_check $(TARGET_PRIMARY_OUT)/kbuild/Makefile bridges driver_version $(bin2hex_target) $(bl1_target) $(fw_target) \
	$(bl0_base_target) $(bl0_vbios_target) $(bl0_target) $(bl0_full_target) \
	$(bl0_all_target) $(bl0_pmcu_target) $(bmcu_target) FORCE
	$(if $(V),,@)$(MAKE) -Rr --no-print-directory -C $(KERNELDIR) \
		M=$(abspath $(TARGET_PRIMARY_OUT)/kbuild) \
		INTERNAL_KBUILD_MAKEFILES="$(INTERNAL_KBUILD_MAKEFILES)" \
		INTERNAL_KBUILD_OBJECTS="$(INTERNAL_KBUILD_OBJECTS)" \
		INTERNAL_EXTRA_KBUILD_OBJECTS="$(INTERNAL_EXTRA_KBUILD_OBJECTS)" \
		BRIDGE_SOURCE_ROOT=$(abspath $(BRIDGE_SOURCE_ROOT)) \
		TARGET_PRIMARY_ARCH=$(TARGET_PRIMARY_ARCH) \
		PVR_ARCH=$(PVR_ARCH) \
		HWDEFS_DIR=$(HWDEFS_DIR) \
		CLANG_TRIPLE=$(if $(filter %-androideabi,$(CROSS_TRIPLE)),$(patsubst \
		%-androideabi,%-gnueabi,$(CROSS_TRIPLE)),$(patsubst \
		%-android,%-gnu,$(CROSS_TRIPLE)))- \
		CROSS_COMPILE="$(CCACHE) $(KERNEL_CROSS_COMPILE)" \
		EXTRA_CFLAGS="$(ALL_KBUILD_CFLAGS)" \
		CC=$(if $(KERNEL_CC),$(KERNEL_CC),$(KERNEL_CROSS_COMPILE)gcc) \
		AR=$(if $(KERNEL_AR),$(KERNEL_AR),$(KERNEL_CROSS_COMPILE)ar) \
		LD=$(if $(KERNEL_LD),$(KERNEL_LD),$(KERNEL_CROSS_COMPILE)ld) \
		NM=$(if $(KERNEL_NM),$(KERNEL_NM),$(KERNEL_CROSS_COMPILE)nm) \
		OBJCOPY=$(if $(KERNEL_OBJCOPY),$(KERNEL_OBJCOPY),$(KERNEL_CROSS_COMPILE)objcopy) \
		OBJDUMP=$(if $(KERNEL_OBJDUMP),$(KERNEL_OBJDUMP),$(KERNEL_CROSS_COMPILE)objdump) \
		CHECK="$(patsubst @%,%,$(CHECK))" $(if $(CHECK),C=1,) \
		V=$(V) W=$(W) TOP=$(TOP)
	@for kernel_module in $(addprefix $(TARGET_PRIMARY_OUT)/kbuild/,$(INTERNAL_KBUILD_OBJECTS:.o=.ko)); do \
		cp $$kernel_module $(TARGET_PRIMARY_OUT); \
	done
ifeq ($(KERNEL_DEBUGLINK),1)
	@for kernel_module in $(addprefix $(TARGET_PRIMARY_OUT)/,$(INTERNAL_KBUILD_OBJECTS:.o=.ko)); do \
		$(CROSS_COMPILE)objcopy --only-keep-debug $$kernel_module $(basename $$kernel_module).dbg; \
		$(CROSS_COMPILE)strip --strip-debug $$kernel_module; \
		$(if $(V),,echo "  DBGLINK " $(call relative-to-top,$(basename $$kernel_module).dbg)); \
		$(CROSS_COMPILE)objcopy --add-gnu-debuglink=$(basename $$kernel_module).dbg $$kernel_module; \
	done
endif


kbuild_clean: kbuild_check $(TARGET_PRIMARY_OUT)/kbuild/Makefile
	$(if $(V),,@)$(MAKE) -Rr --no-print-directory -C $(KERNELDIR) \
		M=$(abspath $(TARGET_PRIMARY_OUT)/kbuild) \
		INTERNAL_KBUILD_MAKEFILES="$(INTERNAL_KBUILD_MAKEFILES)" \
		INTERNAL_KBUILD_OBJECTS="$(INTERNAL_KBUILD_OBJECTS)" \
		INTERNAL_EXTRA_KBUILD_OBJECTS="$(INTERNAL_EXTRA_KBUILD_OBJECTS)" \
		BRIDGE_SOURCE_ROOT=$(abspath $(BRIDGE_SOURCE_ROOT)) \
		TARGET_PRIMARY_ARCH=$(TARGET_PRIMARY_ARCH) \
		CLANG_TRIPLE=$(if $(filter %-androideabi,$(CROSS_TRIPLE)),$(patsubst \
		%-androideabi,%-gnueabi,$(CROSS_TRIPLE)),$(patsubst \
		%-android,%-gnu,$(CROSS_TRIPLE)))- \
		CROSS_COMPILE="$(CCACHE) $(KERNEL_CROSS_COMPILE)" \
		EXTRA_CFLAGS="$(ALL_KBUILD_CFLAGS)" \
		CC=$(if $(KERNEL_CC),$(KERNEL_CC),$(KERNEL_CROSS_COMPILE)gcc) \
		LD=$(if $(KERNEL_LD),$(KERNEL_LD),$(KERNEL_CROSS_COMPILE)ld) \
		NM=$(if $(KERNEL_NM),$(KERNEL_NM),$(KERNEL_CROSS_COMPILE)nm) \
		OBJCOPY=$(if $(KERNEL_OBJCOPY),$(KERNEL_OBJCOPY),$(KERNEL_CROSS_COMPILE)objcopy) \
		V=$(V) W=$(W) TOP=$(TOP) clean

kbuild_install: installkm
kbuild: install_script_km
