/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_CFG_H__
#define __VASTAI_CFG_H__

#ifdef __KERNEL__
#include <linux/cdev.h>
#include <linux/kernel.h>
#endif

#include "vastai_pci.h"



#define MAX_REGION_NUM		256

#define DDR_PARTITION_SIZE_1MB        0x100000
#define DDR_PARTITION_SIZE_2MB        0x200000
#define DDR_PARTITION_SIZE_4MB        0x400000
#define DDR_PARTITION_SIZE_8MB        0x800000
#define DDR_PARTITION_SIZE_15MB       0xF00000
#define DDR_PARTITION_SIZE_16MB       0x1000000
#define DDR_PARTITION_SIZE_17MB       0x1100000
#define DDR_PARTITION_SIZE_22MB       0x1600000
#define DDR_PARTITION_SIZE_29MB       0x1D00000
#define DDR_PARTITION_SIZE_32MB       0x2000000
#define DDR_PARTITION_SIZE_34MB       0x2200000
#define DDR_PARTITION_SIZE_112MB      0x7000000
#define DDR_PARTITION_SIZE_128MB      0x8000000

#define SMCU_FOR_FW_ADDR                 0x1000000000ULL
#define CMCU_FOR_FW_ADDR                 0x1000200000ULL
#define OMCU0_FOR_FW_ADDR                0x1000400000ULL
#define OMCU1_FOR_FW_ADDR                0x1000600000ULL
#define GMCU0_FOR_FW_ADDR                0x1000800000ULL
#define GMCU1_FOR_FW_ADDR                0x1000C00000ULL
#define GMCU2_FOR_FW_ADDR                0x1001000000ULL
#define GMCU3_FOR_FW_ADDR                0x1001400000ULL
#define VEMCU0_FOR_FW_ADDR               0x1001800000ULL
#define VEMCU1_FOR_FW_ADDR               0x1002000000ULL
#define VEMCU2_FOR_FW_ADDR               0x1002800000ULL
#define VEMCU3_FOR_FW_ADDR               0x1003000000ULL
#define ODSP0_FOR_FW_ADDR                0x1003800000ULL
#define ODSP1_FOR_FW_ADDR                0x1004000000ULL
#define VDSP0_FOR_FW_ADDR                0x1004800000ULL
#define VDSP1_FOR_FW_ADDR                0x1005000000ULL
#define ODSP_SHARE_DATA_ADDR             0x1005800000ULL
#define PMCU_FOR_FW_ADDR                 0x1006800000ULL
#define REVERSED_ADDR                    0x1006A00000ULL
#define VEMCU0_HEAP1_FOR_ENC_ADDR        0x1008000000ULL
#define VEMCU1_HEAP1_FOR_ENC_ADDR        0x1010000000ULL
#define VEMCU2_HEAP1_FOR_ENC_ADDR        0x1018000000ULL
#define VEMCU3_HEAP1_FOR_ENC_ADDR        0x1020000000ULL
#define VEMCU0_HEAP2_FOR_ENC_ADDR        0x1028000000ULL
#define VEMCU1_HEAP2_FOR_ENC_ADDR        0x102A000000ULL
#define VEMCU2_HEAP2_FOR_ENC_ADDR        0x102C000000ULL
#define VEMCU3_HEAP2_FOR_ENC_ADDR        0x102E000000ULL
#define SMCU_FOR_LOCAL_SHARE_ADDR        0x1030000000ULL
#define CMCU_FOR_LOCAL_SHARE_ADDR        0x1030100000ULL
#define OMCU0_FOR_LOCAL_SHARE_ADDR       0x1030200000ULL
#define OMCU1_FOR_LOCAL_SHARE_ADDR       0x1030300000ULL
#define GMCU0_FOR_LOCAL_SHARE_ADDR       0x1030400000ULL
#define GMCU1_FOR_LOCAL_SHARE_ADDR       0x1030500000ULL
#define GMCU2_FOR_LOCAL_SHARE_ADDR       0x1030600000ULL
#define GMCU3_FOR_LOCAL_SHARE_ADDR       0x1030700000ULL
#define VEMCU0_FOR_LOCAL_SHARE_ADDR      0x1030800000ULL
#define VEMCU1_FOR_LOCAL_SHARE_ADDR      0x1030900000ULL
#define VEMCU2_FOR_LOCAL_SHARE_ADDR      0x1030A00000ULL
#define VEMCU3_FOR_LOCAL_SHARE_ADDR      0x1030B00000ULL
#define ODSP0_FOR_LOCAL_SHARE_ADDR       0x1030C00000ULL
#define ODSP1_FOR_LOCAL_SHARE_ADDR       0x1030D00000ULL
#define VDSP0_FOR_LOCAL_SHARE_ADDR       0x1030E00000ULL
#define VDSP1_FOR_LOCAL_SHARE_ADDR       0x1030F00000ULL
#define PMCU_FOR_LOCAL_SHARE_ADDR        0x1031000000ULL
#define AI_OPCODE_ADDR                   0x1031100000ULL
#define SMCU_LOG_BUF_ADDR                0x1032000000ULL
#define CMCU_LOG_BUF_ADDR                0x1032100000ULL
#define OMCU0_LOG_BUF_ADDR               0x1032200000ULL
#define OMCU1_LOG_BUF_ADDR               0x1032300000ULL
#define GMCU0_LOG_BUF_ADDR               0x1032400000ULL
#define GMCU1_LOG_BUF_ADDR               0x1032500000ULL
#define GMCU2_LOG_BUF_ADDR               0x1032600000ULL
#define GMCU3_LOG_BUF_ADDR               0x1032700000ULL
#define VEMCU0_LOG_BUF_ADDR              0x1032800000ULL
#define VEMCU1_LOG_BUF_ADDR              0x1032900000ULL
#define VEMCU2_LOG_BUF_ADDR              0x1032A00000ULL
#define VEMCU3_LOG_BUF_ADDR              0x1032B00000ULL
#define ODSP0_LOG_BUF_ADDR               0x1032C00000ULL
#define ODSP1_LOG_BUF_ADDR               0x1032D00000ULL
#define VDSP0_LOG_BUF_ADDR               0x1032E00000ULL
#define VDSP1_LOG_BUF_ADDR               0x1032F00000ULL
#define PMCU_LOG_BUF_ADDR                0x1033000000ULL
#define PROFILER_DATA_ADDR               0x1033100000ULL
#define DEC_VCMD_BUF_ADDR                0x1035300000ULL
#define RAW_FW_IMG_ADDR                  0x1037000000ULL
#define VDSP_OPERATOR_LOADER_BUF_ADDR    0x1038000000ULL
#define ENCRYPTED_FW_ON_BOOT_ADDR        0x103F000000ULL

struct vastai_inbound_atu_region_info{
	u32 region_num;
	u8  pf_num;
	u8  vf_num;
	//0:address match mode , 1:bar match mode
	u8  mode;
	u8  bar_num;  
	u64 noc_base_addr;
	u64 host_base_addr;
	u64 host_limit_addr;
	u64 size;
};

struct inbound_atu_regs{
	u32 ctrl1_reg;
	u32 ctrl2_reg;
	u32 lo32_host_addr;
	u32 hi32_host_addr;
	u32 lo32_limit;
	u32 lo32_noc_addr;
	u32 hi32_noc_addr;
	u32 ctrl3_reg;
	u32 hi32_limit;
};

struct vastai_outbound_atu_region_info{
    u32 region_num;  
	u64 noc_base_addr;
	u64 host_base_addr;
	u64 noc_limit_addr;
	u64 size;
};
struct outbound_atu_regs{
	u32 ctrl1_reg;
	u32 ctrl2_reg;
	u32 lo32_noc_addr;
	u32 hi32_noc_addr;
	u32 lo32_limit;
	u32 lo32_cpu_addr;
	u32 hi32_cpu_addr;
	u32 ctrl3_reg;
	u32 hi32_limit;
};

struct smmu_mpu_regs{
	u32 output_base;
	u32 output_size;
	u32 inputL_base;
	u32 inputH_base;
};

struct smmu_mpu_info{
	u64 output_base_ddr;
	u64 output_size_ddr;
	u64 input_base_ddr;
    u64 output_base_csram;
	u64 output_size_csram;
	u64 input_base_csram;
    u64 output_base_rgn2;
	u64 output_size_rgn2;
	u64 input_base_rgn2;
};

struct gfx_mpu_regs{
	u32 tgt0_output_base;
	u32 tgt0_output_size;
	u32 tgt0_inputL_base;
	u32 tgt0_inputH_base;

    u32 tgt1_output_base;
	u32 tgt1_output_size;
	u32 tgt1_inputL_base;
	u32 tgt1_inputH_base;

    u32 tgt2_output_base;
	u32 tgt2_output_size;
	u32 tgt2_inputL_base;
	u32 tgt2_inputH_base;
};

struct gfx_mpu_info{
	u64 output_base_tgt0;
	u64 output_size_tgt0;
	u64 input_base_tgt0;
    u64 output_base_tgt1;
	u64 output_size_tgt1;
	u64 input_base_tgt1;
    u64 output_base_tgt2;
	u64 output_size_tgt2;
	u64 input_base_tgt2;
};

struct temp_ddr_entry {
	u32		entry_num;
	u64		noc_addr;
	u64 	fn_addr;
	u64 	cpu_pa;
	u64 	bar_offset;	
	u64 	size;	
};

struct temp_ddr_partition{
	u32		partition_num;
	u64		noc_addr;
	u64 	size;
};

typedef enum DDR_PARTITION_OBJ{	
	SMCU_FOR_FW,
	CMCU_FOR_FW,
	OMCU0_FOR_FW,
	OMCU1_FOR_FW,
	GMCU0_FOR_FW,
	GMCU1_FOR_FW,
	GMCU2_FOR_FW,
	GMCU3_FOR_FW,
	VEMCU0_FOR_FW,
	VEMCU1_FOR_FW,
	VEMCU2_FOR_FW,
	VEMCU3_FOR_FW,
	ODSP0_FOR_FW,
	ODSP1_FOR_FW,
	VDSP0_FOR_FW,
	VDSP1_FOR_FW,
	ODSP_SHARE_DATA,
	PMCU_FOR_FW,
	REVERSED,
	VEMCU0_HEAP1_FOR_ENC,
	VEMCU1_HEAP1_FOR_ENC,
	VEMCU2_HEAP1_FOR_ENC,
	VEMCU3_HEAP1_FOR_ENC,
	VEMCU0_HEAP2_FOR_ENC,
	VEMCU1_HEAP2_FOR_ENC,
	VEMCU2_HEAP2_FOR_ENC,
	VEMCU3_HEAP2_FOR_ENC,
	SMCU_FOR_LOCAL_SHARE,
	CMCU_FOR_LOCAL_SHARE,
	OMCU0_FOR_LOCAL_SHARE,
	OMCU1_FOR_LOCAL_SHARE,
	GMCU0_FOR_LOCAL_SHARE,
	GMCU1_FOR_LOCAL_SHARE,
	GMCU2_FOR_LOCAL_SHARE,
	GMCU3_FOR_LOCAL_SHARE,
	VEMCU0_FOR_LOCAL_SHARE,
	VEMCU1_FOR_LOCAL_SHARE,
	VEMCU2_FOR_LOCAL_SHARE,
	VEMCU3_FOR_LOCAL_SHARE,
	ODSP0_FOR_LOCAL_SHARE,
	ODSP1_FOR_LOCAL_SHARE,
	VDSP0_FOR_LOCAL_SHARE,
	VDSP1_FOR_LOCAL_SHARE,
	PMCU_FOR_LOCAL_SHARE,
	AI_OPCODE,
	SMCU_LOG_BUF,
	CMCU_LOG_BUF,
	OMCU0_LOG_BUF,
	OMCU1_LOG_BUF,
	GMCU0_LOG_BUF,
	GMCU1_LOG_BUF,
	GMCU2_LOG_BUF,
	GMCU3_LOG_BUF,
	VEMCU0_LOG_BUF,
	VEMCU1_LOG_BUF,
	VEMCU2_LOG_BUF,
	VEMCU3_LOG_BUF,
	ODSP0_LOG_BUF,
	ODSP1_LOG_BUF,
	VDSP0_LOG_BUF,
	VDSP1_LOG_BUF,
	PMCU_LOG_BUF,
	PROFILER_DATA,
	DEC_VCMD_BUF,
	RAW_FW_IMG,
	VDSP_OPERATOR_LOADER_BUF,
	ENCRYPTED_FW_ON_BOOT,
	DDR_PARTITION_MAX,
}ddr_partition_index;

void gfx_mpu_cfg(struct vastai_pci_info *priv);
void smmu_mpu_cfg(struct vastai_pci_info *priv);
void dump_atu(struct vastai_pci_info *priv);
void dump_ddr_info(struct vastai_pci_info *priv);
void dump_msix_info(struct vastai_pci_info *priv);
void ddr_partition_info(struct vastai_pci_info *priv);
#endif /* end of __VASTAI_PCI_IOCTL_H__ */
