/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_PCI_H__
#define __VASTAI_PCI_H__

#include <linux/list.h>
#include <linux/pci.h>
#include <linux/semaphore.h>
#include <linux/cdev.h>

#include "vastai_pci_api.h"
#include "vastai_pci_dbg.h"
#include "sg100_pcie.h"
#include "sg100_pcie_resource.h"
#include <linux/kfifo.h>

#if 1 // by keye, only support for debug sg100
#include "vastai_sv100_reg.h"
#endif

#include "../common/msgq_struct_share.h"


#define VASTAI_LOG_MAX_CORE					17
#define VASTAI_PCI_DRIVER_PREFIX			"VA "
#define VASTAI_VER_LEN						(64)
#define VASTAI_VER_BMCU_XSPI_LEN			(32)


#define PCI_VENDOR_ID_VASTAI				(0x1EC6)
#define PCI_VENDOR_ID_DAOCLOUD	 			(0x1F4F)

#define VASTAI_SV100_MAX_DIE_NUM			(4)

#define VASTAI_VAL_DW_MASK					(0xFFFFFFFF)

#define VASTAI_DIR_IN						(0)
#define VASTAI_DIR_OUT						(1)

#define VASTAI_PCIE_MSI_NUM					(32)
/* Support up to 128 msi-x interrupts. */
#define VASTAI_PCIE_MSIX_NUM				(16)
#define VASTAI_PCIE_MSIX_NUM_PER_DIE		(VASTAI_PCIE_MSIX_NUM / VASTAI_SV100_MAX_DIE_NUM)

#define VASTAI_PCIE_WAIT_SMCU_MSG1_TIME		(30000)

#define VASTAI_PCIE_TRANS_NUM				(256)

#define VASTAI_FILE_MODE					0666

#define VASTAI_DMA_SEM_CNT					10

#define VASTAI_CHIP_SYNC					0
#define VASTAI_CHIP_ASYN					1

#define PCIE_FAULT_CODE						0xFFFFFFFF

// ARM platform
//#if (defined(__aarch64__) || defined(__arm__))
//#define VASTAI_MAX_DMA_BUF ((1 << (MAX_ORDER - 3)) * PAGE_SIZE)
//#else
// X86 platform
#define VASTAI_MAX_DMA_BUF ((1 << (MAX_ORDER - 1)) * PAGE_SIZE)
//#endif

#define VASTAI_MAX_DMA_PAYLOAD SZ_16M
#define VASTAI_MAX_MEMSET_BUF SZ_4M
#define VASTAI_DMA_CHN 4
enum VASTAI_DIE_NUM {
	VASTAI_SINGLE_DIE = 1,
	VASTAI_TWO_DIE = 2,
	VASTAI_FOUR_DIE = 4
};

enum VASTAI_DIE_ID {
	VASTAI_DIE0,
	VASTAI_DIE1,
	VASTAI_DIE2,
	VASTAI_DIE3,
	VASTAI_DIE_ID_MAX
};

enum VASTAI_PKG_ID {
	VASTAI_PKG0,
	VASTAI_PKG1,
	VASTAI_PKG_ID_MAX
};

enum VASTAI_FN_ID {
	VASTAI_FN0,
	VASTAI_FN1,
	VASTAI_FN2,
	VASTAI_FN3,
	VASTAI_FN4,
	VASTAI_FN5,
	VASTAI_FN6,
	VASTAI_FN7,
	VASTAI_FN_ID_MAX
};

	
enum CMD_BUF_OBJ{
	CMD_BUF_DMA_RD_CH_LO,
	CMD_BUF_DMA_WR_CH_LO,
	CMD_BUF_DMA_HI,
	CMD_BUF_AI,
	CMD_BUF_SMCU,
	CMD_BUF_VEMCU,
	CMD_BUF_GMCU_MSGQ_TRIGER,
	CMD_BUF_MAX,
};

typedef enum ONLY_1PF_RING_BUF_OBJ{	
	ONLY_1PF_HOST_TO_SMCU_CMD_BUF,
	ONLY_1PF_HOST_TO_CMCU_CMD_BUF,
	ONLY_1PF_HOST_TO_VEMCU0_CMD_BUF,
	ONLY_1PF_HOST_TO_VEMCU1_CMD_BUF,
	ONLY_1PF_HOST_TO_VEMCU2_CMD_BUF,
	ONLY_1PF_HOST_TO_VEMCU3_CMD_BUF,
	ONLY_1PF_HOST_TO_VDSP0_CMD_BUF,
	ONLY_1PF_HOST_TO_VDSP1_CMD_BUF,
	ONLY_1PF_HOST_TO_ODSP0_CMD_BUF,
	ONLY_1PF_HOST_TO_ODSP1_CMD_BUF,
	ONLY_1PF_HOST_TO_VDMCU0_CMD_BUF,
	ONLY_1PF_HOST_TO_VDMCU1_CMD_BUF,
	ONLY_1PF_HOST_TO_GMCU1_CMD_BUF,
	ONLY_1PF_HOST_TO_GMCU3_CMD_BUF,
	ONLY_1PF_SMCU_TO_HOST_INFO_BUF,
	ONLY_1PF_CMCU_TO_HOST_INFO_BUF,
	ONLY_1PF_VEMCU0_TO_HOST_INFO_BUF,
	ONLY_1PF_VEMCU1_TO_HOST_INFO_BUF,
	ONLY_1PF_VEMCU2_TO_HOST_INFO_BUF,
	ONLY_1PF_VEMCU3_TO_HOST_INFO_BUF,
	ONLY_1PF_VDSP0_TO_HOST_INFO_BUF,
	ONLY_1PF_VDSP1_TO_HOST_INFO_BUF,
	ONLY_1PF_ODSP0_TO_HOST_INFO_BUF,
	ONLY_1PF_ODSP1_TO_HOST_INFO_BUF,
	ONLY_1PF_VDMCU0_TO_HOST_INFO_BUF,
	ONLY_1PF_VDMCU1_TO_HOST_INFO_BUF,
	ONLY_1PF_GMCU1_TO_HOST_INFO_BUF,
	ONLY_1PF_GMCU3_TO_HOST_INFO_BUF,
	ONLY_1PF_DISPLAY_H2D_BUF,
	ONLY_1PF_DISPLAY_D2H_BUF,
	ONLY_1PF_RING_BUF_MAX,
}only_1pf_ring_buf_index;

typedef enum ONLY_2PF_RING_BUF_OBJ{	
	ONLY_2PF_HOST_TO_SMCU_CMD_BUF,
	ONLY_2PF_HOST_TO_CMCU_CMD_BUF,
	ONLY_2PF_HOST_TO_1ST_VEMCU_CMD_BUF,
	ONLY_2PF_HOST_TO_2ND_VEMCU_CMD_BUF,
	ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF,
	ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF,
	ONLY_2PF_HOST_TO_VDMCU_CMD_BUF,
	ONLY_2PF_HOST_TO_GMCU_CMD_BUF,
	ONLY_2PF_SMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_CMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_1ST_VEMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_2ND_VEMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF,
	ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF,
	ONLY_2PF_VDMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_GMCU_TO_HOST_INFO_BUF,
	ONLY_2PF_DISPLAY_H2D_BUF,
	ONLY_2PF_DISPLAY_D2H_BUF,
	ONLY_2PF_RING_BUF_MAX,
}only_2pf_ring_buf_index;

typedef enum COMMON_RING_BUF_OBJ{	
	COMMON_HOST_TO_SMCU_CMD_BUF,
	COMMON_HOST_TO_CMCU_CMD_BUF,
	COMMON_HOST_TO_VEMCU_CMD_BUF,
	COMMON_HOST_TO_VDSP_CMD_BUF,
	COMMON_HOST_TO_VDMCU_CMD_BUF,
	COMMON_HOST_TO_GMCU_CMD_BUF,
	COMMON_SMCU_TO_HOST_INFO_BUF,
	COMMON_CMCU_TO_HOST_INFO_BUF,
	COMMON_VEMCU_TO_HOST_INFO_BUF,
	COMMON_VDSP_TO_HOST_INFO_BUF,
	COMMON_VDMCU_TO_HOST_INFO_BUF,
	COMMON_GMCU_TO_HOST_INFO_BUF,
	COMMON_RING_BUF_MAX,
}common_ring_buf_index;

enum VASTAI_PCI_GEN_SEL {
	VASTAI_PCI_GEN1 = 1,
	VASTAI_PCI_GEN2,
	VASTAI_PCI_GEN3,
	VASTAI_PCI_GEN4
};

struct vastai_pci_chn_info {
	struct chn_ops *ops[32];
	// NTD
};

struct __attribute__((packed)) vdmcu_msg_head {
	/* rd msg num */
	u32 rd;
	/* wr msg num */
	u32 wr;
	/* max msg num */
	u32 elem_count;
	/* every msg element size */
	u32 elem_size;
	u8 buf[0]; 
};

struct region_info{
	u64 			fn_base_addr;
    u64 			noc_addr;
    u64 			region_size;
	u64 			bar_offset;
    unsigned char 	*region_vmem;
};

union link_control_status_reg {
	u32 val;
	struct {
		u32 unused0 : 16;
		u32 negotiated_link_speed : 4;
		u32 negotiated_link_width : 6;
		u32 unused1 : 6;
	} bit;
};

struct bar_info {
	resource_size_t 	mmio_start;
	resource_size_t 	mmio_end;
	resource_size_t 	mmio_len;
	unsigned long 		mmio_flags;
	/* inbound pcie to axi address translation */
	u64					noc_addr;
	u64 			    fn_base_addr;
    unsigned long 		region_cnt;
    struct region_info 	regions[6];          // BAR 2/3 supports the most region number.
	unsigned char 		*mem;
	unsigned char 		*vmem;
};

typedef enum DDR_ENTRY_OBJ{	
	VF_MANAGE,
	META_FW,
	ENC_DEC_CMD_BUF,
	AI_MODULE_OPCODE,
	SMMU_CD_TABLE,
	RESERVED_DDR,	
	RUNTIME,
	GART,
	GPU_HEAP,
	GPU_DISPLAY,	
	DDR_ENTRY_MAX,
}ddr_entry_index;
struct va_ddr_entry {
	u64		noc_addr;
	u64 	fn_addr;
	u64 	cpu_pa;
	u64 	bar_offset;	
	u64 	size;	
};

struct va_ddr_info {
	u64					card_ddr_size; 		/*the card ddr sum size*/
	u64					fn_ddr_size;  		/*the function ddr sum size*/
	u64					vf_ddr_size;   		/*the vf of current pf  ddr sum size only windows meta fw use*/
	u64					cur_pf_vf_num;   	/*the vf of current pf vf num for only windows meta fw use*/
	struct pcie_to_ddk_info* pcie_to_ddk_info_addr;   	/*the vf of current pf vf num for only windows meta fw use*/
	struct va_ddr_entry	ddr_entry[DDR_ENTRY_MAX];
};

struct msi_x_table {
	unsigned int msg_addr;
	unsigned int msg_upper_addr;
	unsigned int msg_data;
	unsigned int vector_control;
};

typedef int (*vastai_intr_handle)(void *pci_dev,
	u32 die_index,
	int irq,
	void *ctxt, /* for user keep private data */
	void *out
); /* for out_desc */



enum vastai_reset_type {
	VASTAI_RESET_MSG_TRIGER =  1 << 1, /* host send msg_irq to triger fw reset */
	VASTAI_RESET_BOOT_DEV   =  1 << 2  /* reboot dev, fw download and ... */
};

struct vastai_fw_version {
	u8 ver[VASTAI_VER_LEN];
	u8 smcu_ver[VASTAI_VER_LEN];
	u8 cmcu_ver[VASTAI_VER_LEN];
	u8 omcu_ver[VASTAI_VER_LEN];
	u8 gmcu_ver[VASTAI_VER_LEN];
	u8 vemcu_ver[VASTAI_VER_LEN];
	u8 odsp_ver[VASTAI_VER_LEN];
	u8 vdsp_ver[VASTAI_VER_LEN];
	u8 pmcu_ver[VASTAI_VER_LEN];
	u8 vdmcu_ver[VASTAI_VER_LEN];
	u8 ver_bk[VASTAI_VER_LEN];
	u8 bl0_ver[VASTAI_VER_LEN];
	u8 bl0_ver_bk[VASTAI_VER_LEN];
	u8 driver_ver[VASTAI_VER_LEN];
	u8 phy_ver[VASTAI_VER_LEN];
};

struct vastai_fw_bmcu_xspi_version {
	union {
		u8 ver[VASTAI_VER_LEN];
		struct {
			u8 bmcu_sub_ver[VASTAI_VER_BMCU_XSPI_LEN];
			u8 xspi_sub_ver[VASTAI_VER_BMCU_XSPI_LEN];
		};
	};
};


#define HEARTBEAT_CNT_REG_NUM  17
#define HEARTBEAT_CNT_REG_MASK 0xFFFE0000

struct heartbeat_monitor {
	unsigned long timeout_threshold; /* unit: jiffies */
	unsigned long cores_status; /* bit map: 0 - alive, 1 - hang */
	u32  reset_cnt[HEARTBEAT_CNT_REG_NUM];
	u32  last_counts[HEARTBEAT_CNT_REG_NUM];
	struct delayed_work heartbeat_work;
};

struct bandwidth_test_monitor {
	unsigned long timeout_threshold; /* unit: jiffies */
	u32 is_begin;
	struct delayed_work bandwidth_test_work;
};

#ifdef VIDEO_COMPAT_ENC
#define PACK_EXT __attribute__((packed))
#else
#define PACK_EXT
#endif

union die_index_data{
	u32 val;
	struct {
		u32 die_id:8;
		u32 dev_id:16;
		u32 RESV:8;
	};
} PACK_EXT;

struct msgq {
	u32 addr;
	struct mutex msgq_lock;
};

struct vastai_file_info {
	char file_name[32];
	dev_t dev_num;
	struct cdev dev;
	struct class *class;
	struct device *device;
};

struct vastai_version_file {
	struct vastai_file_info file_info;
	struct vastai_pci_info *pci_info;
};

struct vastai_ctl_file {
	struct vastai_file_info file_info;
	struct vastai_pci_info *pci_info;
};
#ifdef CONFIG_VASTAI_TOOLS_SUITE
struct tools_info_s {
	u32 smi_ack_msg[2]; /*smi ack from smcu*/
	struct completion smi_ack[2]; /*smi ack from smcu*/
	struct completion core_time_sync_ack; /*core time sync ack from smcu*/
	struct mutex tools_dev_mutex; /* tools device mutex */
	struct mutex tools_die_mutex; /* tools die mutex */
	struct completion log_comp;
	u32 pcie_tx_vdsp_count[4];
	u32 pcie_rx_vdsp_count[4];
	u32 ddr_bw_test;
};
#endif

struct vastai_sv100_die {
	u32 die_index;
    u32 pkg_boot_stage;
    u32 dp_enabled;
	/* TODO: refactor msgq */
	u32 tx_cfg_info_addr;
	u32 rx_cfg_info_addr;
	struct intrrupt_info{
		int is_out_as_stream; /* is we need malloc out buf each time ? */
		struct out_desc out;
		void *private_data;   /* for user keep private data */
		vastai_intr_handle intr_handle;
	} ivt[32];
	u64 dma_desc_fifo_address;
	u32 local_desc_cnt;
	spinlock_t msg_ring_buf_lock;
	unsigned char trans_fill_pos;
	unsigned char trans_call_pos;
	struct data_struct *trans_data[VASTAI_PCIE_TRANS_NUM];
	struct msgq msgq[2];
	struct completion log_comp;
	struct vastai_fw_version fw_ver;
};

struct vastai_interrupt_handle {
	void (*handler_function)(void *);
	void *arg;
};

struct va_dma_table {
	spinlock_t lock;
	struct list_head head;
};

struct vastai_dma_buf {
	void 					*vir;
	unsigned long			dma_bus_addr;
	phys_addr_t				phy;
	int						size;
	struct page				**pages;
	u8						*pages_order;
	struct sg_table			sg_table;
	unsigned int			num_pages;
	struct vastai_pci_info	*pci_info;
};

typedef void (*va_host_irq_callback_t)(void *arg, struct host_irq_msg *msg);
struct va_host_irq {
	va_host_irq_callback_t callback;
	void *arg;
};

struct hw_msgq;

#define	decrease_kfifo_in(fifo, n) \
({ \
	typeof((fifo) + 1) __tmp = (fifo); \
	unsigned long __n = (n); \
	struct __kfifo *__kfifo = &__tmp->kfifo; \
	__kfifo->in -= __n;\
})

#define smcu_rw_kfifo_in(fifo) \
({ \
		typeof((fifo) + 1) __tmpl = (fifo); \
		__tmpl->kfifo.in; \
})
				
#define smcu_rw_kfifo_out(fifo) \
({ \
		typeof((fifo) + 1) __tmpl = (fifo); \
		__tmpl->kfifo.out; \
})

struct smcu_rw_ret_info{
	u32 w0_data0;
	u32 w1_data1;
	u32 w2_data2;
	u32 w3_data3;
	struct completion smcu_task_done;
};
struct smcu_task_info {
	void (*callback)(void *arg1, void *arg2);
	void *arg1;
	u32 w0_data0;
	u32 w1_data1;
	u32 w2_data2;
	u32 w3_data3;
};

struct vatsai_smcu_task {
	DECLARE_KFIFO(smcu_task_fifo, struct smcu_task_info, 128);
	struct mutex smcu_task_mutex;
};

typedef uint64_t (*alloc_physmem_by_ddk) (void *dev_node, int ui32MemSize, int pid);
typedef void (*free_physmem_by_ddk) (void *dev_node, uint64_t DevPhysAddr);
typedef int (*tools_get_gfx_status) (u32 cmd, u64 cmd_args, u32 cmd_args_size, u64 info, u32 info_size);


#define MAX_INVENTORY_ITEM_LEN       16

#pragma pack(1)
struct sg_inventory_t {
    u8 model_name[MAX_INVENTORY_ITEM_LEN];
    u8 hw_version[MAX_INVENTORY_ITEM_LEN];
    u8 product_sn[MAX_INVENTORY_ITEM_LEN];
    u8 mfg_pn[MAX_INVENTORY_ITEM_LEN];
    u8 pcb_supplier[MAX_INVENTORY_ITEM_LEN];
    u8 mech_supplier[MAX_INVENTORY_ITEM_LEN];
    u8 manufacturer[MAX_INVENTORY_ITEM_LEN];
    u8 mfg_date[MAX_INVENTORY_ITEM_LEN];
    u8 main_chip_pn[MAX_INVENTORY_ITEM_LEN];
    u8 main_chip_version[MAX_INVENTORY_ITEM_LEN];
    u8 max_power[MAX_INVENTORY_ITEM_LEN];
    u8 main_chip_quantity[MAX_INVENTORY_ITEM_LEN];
    u8 single_memory_chip_sz[MAX_INVENTORY_ITEM_LEN];
    u8 mem_chip_num_in_main_chip[MAX_INVENTORY_ITEM_LEN];
    u8 input_clock_type[MAX_INVENTORY_ITEM_LEN];
    u8 input_clock_frequency[MAX_INVENTORY_ITEM_LEN];
    u8 pcie_max_lane_num[MAX_INVENTORY_ITEM_LEN];
    u8 pcie_max_speed[MAX_INVENTORY_ITEM_LEN]; 
    u8 core_ot_throttle_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 core_ot_shutdown_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 inlet_ot_throttle_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 inlet_ot_shutdown_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 outlet_ot_throttle_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 outlet_ot_shutdown_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 gf_oc_throttle_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 gf_oc_shutdown_threshold[MAX_INVENTORY_ITEM_LEN];
    u8 dp_interface_support[MAX_INVENTORY_ITEM_LEN];
};
#pragma pack()


struct vastai_pci_info {
	struct list_head dev_list;
	struct list_head card_list;
	u32 card_id;   // value mapped from productSN
	char dev_id;
	u8 is_pf:1;
	u8 is_vf:1;
	u8 is_boot_pf:1;
	u8 is_main_dev_in_os:1;
	bool gart_en;
	struct vastai_ctl_file *ctl_file;
	struct pci_dev *dev;
	struct char_drv_info *p_char;
	struct vastai_version_file *version_file;
#ifdef CONFIG_VASTAI_PCI_BOOT
	struct delayed_work boot_wq;
#endif
	unsigned char legacy_en;
	unsigned char msi_en;
	struct sg_inventory_t inventory_info;
	unsigned char msix_en;
	unsigned int  irq;
	unsigned char irq_num;
	unsigned char die_num;
	unsigned char boot_stage; // 0:initial, 1: bl1 done, 2:fw done, 3: all done
	unsigned char bar_num;
	unsigned long long ddr_attr;
	struct mutex trans_mutex;
	spinlock_t mmio_read_spin_lock;
	spinlock_t mmio_write_spin_lock;
	spinlock_t trans_lock;
	spinlock_t swtich_atu_lock;
	spinlock_t ring_buf_lock;
	struct completion swtich_atu_done;
	struct completion rw_xspi;
	struct completion send_reinit_rst_done;
	struct completion pmcu_boot_mode_done;
	struct completion smcu_flr_done;
	u32 rw_xspi_msg;
	u32 pmcu_boot_mode_msg;
	atomic_t is_fw_updating;
	atomic_t pci_state;
	struct msix_entry msix[VASTAI_PCIE_MSIX_NUM];
	struct msi_x_table msix_table[VASTAI_PCIE_MSIX_NUM];
	struct bar_info bar[VASTAI_PCI_BAR_NUM];
	unsigned char 		*diag_vddr;
	unsigned char 		*diag_vcsr;
	struct vastai_sv100_die dies[VASTAI_SV100_MAX_DIE_NUM];
	u32 core_status_bitmap;
	struct work_struct exception_work;
	struct semaphore dma_sem;
	struct bandwidth_test_monitor bandwidth_monitor;
	struct heartbeat_monitor heartbeat;
	u32 heartbeat_mask;
	struct vastai_fw_version fw_ver;
	// the head of vastai_cdev list
	struct list_head vacc_dev_head;
	atomic_t is_reseting;
	u8 disable;
	u8 pkg_id;
	u8 fn_mode;
	u32 waiting_cmcu_comback;
	u32 pcie_nls; /* negotiated_link_speed */
	u32 pcie_nlw; /* negotiated_link_width */
	void *vacc_dev; 
	struct vastai_interrupt_handle irq_handles[VASTAI_PCIE_MSIX_NUM];
	struct delayed_work payload_work;
	struct va_dma_table dma_table;
	struct workqueue_struct *dma_msix_wq;
	struct work_struct dma_msix_handler;
	struct hw_msgq *to_host_msgq;
	struct hw_msgq *smcu_msix_msgq;
	bool   iommu_en;
	struct vastai_mem_pool *vastai_pool;
	struct vastai_cmd_entry *cmd_entrys[CMD_BUF_MAX]; 
	struct ring_buf_info   *ring_buf[ONLY_1PF_RING_BUF_MAX];
	struct priv_hw_cfg *priv_hw_cfg; 
	struct proc_dir_entry *mcu_log_ent[VASTAI_LOG_MAX_CORE];
	struct va_host_irq *host_irqs[HOST_IRQ_MAX];
	struct platform_device *gfx_card_dev;
	struct platform_device *nulldisp_dev;
	struct smmu_info *vastai_smmu_info;
	struct ai_info   *vastai_ai_info;
	struct region_info slave_bar4_region[2];
#ifdef CONFIG_VASTAI_VIDEO
	struct list_head render_head;
#endif
#ifdef CONFIG_VASTAI_TOOLS_SUITE
	struct tools_info_s tools_info;
#endif

	tools_get_gfx_status get_gfx_status; /* get gfx utilization or mem usage info api of ddk */
	alloc_physmem_by_ddk alloc; /*mem alloc api of ddk*/
	free_physmem_by_ddk free;   /*mem free api of ddk*/
	void *ddk_node;   /*ddk pvr dev node*/

	bool hasInitGfxMemAlloc;
	void *heapPtr;
	u64  (*AllocMemFromGfxHeap)(void *heapPtr, int size);
	void (*FreeMemToGfxHeap)(void *heapPtr, u64 addr);

#ifdef CONFIG_PVP_HASH_CMP_EN
	struct mutex cedar_lock;
#endif
	const struct pci_device_id *id;
	struct vastai_dma_buf log_dma_buf;
	struct vastai_dma_buf pcie_bw_buf;
	struct vatsai_smcu_task smcu_task;
	struct va_ddr_info		ddr_info;
	struct list_head fn_node[4];
	bool probe_done;
	struct vastai_die_info *die_info[4]; /* parent */
};

typedef union vastai_pci_bdf_info {
	u64 whole;
	struct {
		u64 fn_id: 4;
		u64 dev_id: 8;
		u64 bus_id: 8;
		u64 domain_id: 32;
		u64 reserved: 12;
	};
} vastai_pci_bdf_info_t;


struct vastai_fn_info {
	struct list_head fn_node;
	struct vastai_die_info *die_info;  /*parent*/
};

struct vastai_die_info {
	struct list_head die_node;
	struct list_head vastai_fn_info_list;
	u8 die_id;
	struct vastai_pkg_info *pkg_info; /*parent*/
};

struct vastai_pkg_info {
	struct list_head pkg_node;
	struct list_head vastai_die_info_list;
	u8 pkg_id;
	struct vastai_card_info *card_info; /*parent*/
};

struct vastai_card_info {
	struct list_head card_node;
	struct list_head vastai_pkg_info_list;
	atomic_t pci_state;
	//board_type_e board_type;
	vastai_pci_bdf_info_t bdf_info;
	u8 sn_info[20];
};


enum DEVICE_STATE {
	VASTAI_NO_RUN_STATE	= 0,
	VASTAI_NORMAL_STATE	= 1,
	VASTAI_UPDATE_STATE	= 2,
	VASTAI_BOOT_STATE	= 3,
	VASTAI_EXIT_STATE	= 4,
	VASTAI_ERROR_STATE	= 5,
	VASTAI_RESET_STATE	= 6,
	VASTAI_DEBUG_STATE	= 7,
	VASTAI_HOTP_STATE	= 8,
	VASTAI_STATE_END,
};

struct inbound_reg {
	unsigned int type; /* region contril 1 ;0:mem, 2:i/o 4:cfg */
	unsigned int en;   /* region contril 2 [10:8]:BAR_NUM, */
	unsigned int lower_base_addr;
	unsigned int upper_base_addr;
	unsigned int limit;
	unsigned int lower_target_addr;
	unsigned int upper_target_addr;
};

struct outbound_reg {
	unsigned int type;
	unsigned int en;
	unsigned int lower_base_addr;
	unsigned int upper_base_addr;
	unsigned int limit;
	unsigned int lower_target_addr;
	unsigned int upper_target_addr;
} __packed;

// private command number
#define CMD_RUN_MODEL	  0x40185602
#define CMD_RUN_STREAM	  0x40185603
#define CMD_LOAD_MODEL	  0x40185604
#define CMD_DESTROY_MODEL 0x40185605
#define CMD_LOAD_VDSP_OPERATOR 0x40185606
#define CMD_LOAD_ODSP_OPERATOR 0x40185607
#define CMD_COPY_VDSP_OPERATOR 0x4018560A
#define CMD_COPY_ODSP_OPERATOR 0x4018560B

struct __attribute__((packed)) pcie_transfer_info {
	/* refer to VASTAI_TRANSFER_TYPE
	 * type: 0:ai model; 1:video
	 */
	u32 type : 8;
	/* dir: refer to VASTAI_TRANSFER_DIR, the direction of data transfer is
	 * defined based on the host side.
	 * 0: host write to device, or device read from host.
	 * 1: host read from device, or device write to host.
	 */
	u32 dir : 8;
	/* transfer number */
	u32 num : 8;
	u32 pad : 8;
	/* channel id, refer to VASTAI_TX_CHN_ID and VASTAI_RX_CHN_ID */
	u32 chn;
	/* data_type: 0:normal process, 1:only copy */
	u32 pcie_config_addr;
	u32 reserved;
};


struct __attribute__((packed)) pcie_transfer_cmd {
	union
	{
		struct
		{
			unsigned int optcode			: 8;
			unsigned int rev0				: 8;
			unsigned int rev1				: 8;
			unsigned int rev2				: 8;
		}s_data0;
		unsigned int data0;
	} w0;

	union
	{
		struct
		{
			unsigned int rev0				: 8;
			unsigned int rev1				: 8;
			unsigned int rev2				: 8;
			unsigned int rev3				: 8;
		}s_data1;
		unsigned int data1;
	} w1;

	union
	{
		struct
		{
			unsigned int rev0				: 8;
			unsigned int rev1				: 8;
			unsigned int rev2				: 8;
			unsigned int rev3				: 8;
		}s_data2;
		unsigned int data2;
	} w2;

	union
	{
		struct
		{
			unsigned int rev0				: 8;
			unsigned int rev1				: 8;
			unsigned int rev2				: 8;
			unsigned int rev3				: 8;
		}s_data3;
		unsigned int data3;
	} w3;
};

struct __attribute__((packed)) pcie_config_info {
	/* refer to VASTAI_TRANSFER_TYPE
	 * type: 0:ai model; 1:video
	 */
	u32 type : 8;
	/* dir: refer to VASTAI_TRANSFER_DIR, the direction of data transfer is
	 * defined based on the host side.
	 * 0: host write to device, or device read from host.
	 * 1: host read from device, or device write to host.
	 */
	u32 dir : 8;
	/* transfer number */
	u32 num : 8;
	u32 pad : 8;
	/* channel id, refer to VASTAI_TX_CHN_ID and VASTAI_RX_CHN_ID */
	u32 chn;
	union buf_data data;
	u32 dma_desc_addr;
};
int vastai_file_create(struct vastai_file_info *file_info,
		struct vastai_pci_info *pci_info,
		const struct file_operations *fop, const char *fmt, ...);
void vastai_file_destroy(struct vastai_file_info *file_info);
struct vastai_pci_info *get_vastai_pci_device_info(char dev_id);
#ifdef CONFIG_VASTAI_TOOLS_SUITE
int get_vastai_pci_device_number( void );
int vatools_fw_config_smi( struct vastai_pci_info* priv, int die_index );
#endif
int vastai_pci_reset_bus(struct vastai_pci_info *pcie_dev);
void vastai_pci_init_sw_source(struct vastai_pci_info *priv);
int vastai_pci_config_read(struct vastai_pci_info *pcie_dev, int offset, char *buf, int len);
int vastai_pci_config_write(struct vastai_pci_info *pcie_dev, int offset, char *buf, int len);
// int vastai_pci_mem_read(struct vastai_pci_info *pcie_dev, int die_index, u64 relative_addr, void *buf, unsigned int len);
// int vastai_pci_mem_write(struct vastai_pci_info *pcie_dev, int die_index, u64 relative_addr, const void *buf, unsigned int len);
int vastai_pci_generation_sel(struct vastai_pci_info *pcie_dev, u8 gen);
int vastai_pci_malloc_udma_buf(struct vastai_pci_info *pcie_dev, struct vastai_dma_buf *dm, u32 len);
int vastai_pci_free_udma_buf(struct vastai_pci_info *pcie_dev, struct vastai_dma_buf *dm);
void * vastai_pci_mmio_kva_get(struct vastai_pci_info *priv, u32 die_index, u64 relative_addr, u32 len);
int init_vastai_pci_mem(struct vastai_pci_info *priv);
void vastai_get_pcie_attr(struct vastai_pci_info *priv);
int vastai_pci_mem_read(struct vastai_pci_info *priv, int pkg_id, u64 noc_addr, void *buf, unsigned int len);
int vastai_pci_mem_write(struct vastai_pci_info *priv, int pkg_id, u64 noc_addr, void *buf, unsigned int len);
int vastai_pci_mmio_read(struct vastai_pci_info *priv, int bar, u64 offset, void *buf, unsigned int len);
int vastai_pci_mmio_write(struct vastai_pci_info *priv, int bar, u64 offset, void *buf, unsigned int len);
int vastai_pcie_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u64 *bw_cont);
int vastai_ddr_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 ddr_id, u32 port_id, u64 *bw_cont);
int vastai_sdma_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 sdma_id, u64 *bw_cont);
int vastai_venc_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 venc_id, u64 *bw_cont);
int vastai_vdec_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 vdec_id, u64 *bw_cont);
int vastai_gfx_monitor(struct vastai_pci_info *priv, u32 us_tick, u32 is_read, u32 gfx_id, u32 port_id, u64 *bw_cont);
int vastai_ddr_bw_test(struct vastai_pci_info *priv);
int vastai_pcie_bw_test(struct vastai_pci_info *priv);
void vastai_pcie_bw_test_cancel(struct vastai_pci_info *priv);
int vastai_cii_set(struct vastai_pci_info *priv, u32 ch, u32 addr, u32 pf);
int vastai_read_by_smcu(struct vastai_pci_info *priv, u64 noc_addr, unsigned int len ,void *buf);
int vastai_write_by_smcu(struct vastai_pci_info *priv, u64 noc_addr, u32 val, u32 len);
int vastai_slave_config_read(struct vastai_pci_info *priv, u64 offset, void *buf,  unsigned int len);
int vastai_slave_config_write(struct vastai_pci_info *priv, u64 offset, void *buf,  unsigned int len);
int vastai_pci_modify_reg32(struct vastai_pci_info *priv,u64 addr,u32 mask_bit,u32 mode);
int vastai_smcu_modify_reg32(struct vastai_pci_info *priv,u64 addr,u32 mask_bit,u32 mode);
int vastai_pci_mem_fread(struct vastai_pci_info *priv, u64 fn_addr, void *buf, unsigned int len);
int vastai_pci_mem_fwrite(struct vastai_pci_info *priv, u64 fn_addr, void *buf, unsigned int len);
int translate_fn_addr(struct vastai_pci_info *priv, u64 fn_addr, int *bar, u64 *bar_offset, int *region_num);
int translate_noc_addr(struct vastai_pci_info *priv, int pkg_id, u64 noc_addr, int *bar, u64 *bar_offset);

/* Func: read text common info collected by SMCU, which is not stored in CSR or DDR, dynamical status info etc. 
 * Args: total_len,     should be aligned to 4B, even through has some dummy data.
 *		 info_type,     specific info type.
*/
int vastai_read_info_from_smcu(struct vastai_pci_info *priv, u32 info_type, u32 total_len, u8 *buf);

#ifdef DIAG_TEST_MODE
int diag_vastai_pci_mem_read(struct vastai_pci_info *priv,
			int die_index,
			u64 relative_addr,
			void *buf,
			unsigned int len);
int diag_vastai_pci_mem_write(struct vastai_pci_info *priv,
			int die_index,
			u64 relative_addr,
			void *buf,
			unsigned int len);
#endif
/* This function will be used intead of vastai_pci_transfer_reg and
 * vastai_pci_transfer_reg_vd (declarate at vastai_pci_api.h).
 * For now if you call vastai_irq_register and
 * vastai_pci_transfer_reg/vastai_pci_transfer_reg_vd at the same time,
 * vastai_pci_transfer_reg/vastai_pci_transfer_reg_vd will take effect.
 */
int vastai_irq_register(struct vastai_pci_info *pcie_dev, /* which pcie_group */
			int die_index,
			int irq, /* reference vastai_sv100_reg.h +272 */
			vastai_intr_handle handle,
			void *private_data); /* for user keep self data */
int vastai_irq_unregister(struct vastai_pci_info *pcie_dev, int die_index, int irq);

int vastai_pci_get_die_id(struct vastai_pci_info *pcie_dev, u32 die_index);
int get_vastai_die_info(struct vastai_pci_info *priv, int die_index[], int *die_num);
int vastai_sdma_transfer_memset(void *pci_info,
			int die_index,
			union core_bitmap core_id,
			dma_addr_t dma_handle,
			u32 is_src_dma_addr,
			u32 is_host_to_dev,
			u32 is_src_not_user_mem,
			u64 axi_addr,
			u32 size);
int vastai_sdma_transfer_sg(void *pci_info, int die_index,
			union core_bitmap core_id,
			struct vastai_dma_buf *dm,
			u32 is_src_dma_addr,
			u32 is_host_to_dev,
			u32 is_src_not_user_mem,
			u64 axi_addr,
			u32 size);
int vastai_sdma_transfer_by_uaddr(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u32 is_host_to_dev,
			u64 axi_addr,
			u64 user_addr,
			u32 size);
int vastai_dma_sg_set_desc(struct vastai_dmadesc* desc,
			struct vastai_sg_gather_info sg_info,
			int i,
			u64 axi_addr,
			u32 done_size,
			u32 is_host_to_dev,
			u32 is_src_not_user_mem,
			u32 is_src_dma_addr);
int vastai_sdma_fill_data(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u64 axi_addr,
			u64 data,
			u32 size);

// func: dump the map of all kchar:xx ~ BDF on current PC
void dump_pf_vf_map_kchar(struct vastai_pci_info *priv);

// Func: get card info API only available for PF
//       Needs put it at the end of the probe.
void va_get_card_info(struct vastai_pci_info *priv);

/* ========================api for dev_id version ============================*/
static inline int vastai_pci_info_get_dev_id(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->dev_id;
}

static inline int vastai_pci_info_get_die_num(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->die_num;
}

/* This function returns the attributes of the DDR used by each DIE in each PCIE device:
 * e.g.
 * return 0x1_0000_0000 means ddr density is 4GB;
 * return 0x2_0000_0000 means ddr density is 8GB;
 * return 0x4_0000_0000 means ddr density is 16GB.
 */
static inline u64 vastai_pci_info_get_ddr_attr(struct vastai_pci_info *pcie_dev)
{
	return pcie_dev->ddr_attr;
}

static inline int vastai_pci_reset_bus_devid(unsigned int dev_id)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_reset_bus(pcie_dev);
	}
	return -ENODEV;
}

static inline int vastai_pci_config_read_devid(unsigned int dev_id, int offset, char *buf, int len)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_config_read(pcie_dev, offset, buf, len);
	}
	return -ENODEV;
}
static inline int vastai_pci_config_write_devid(unsigned int dev_id, int offset, char *buf, int len)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_config_write(pcie_dev, offset, buf, len);
	}
	return -ENODEV;
}
static inline int vastai_pci_mem_read_devid(unsigned dev_id, unsigned int die_index, u64 relative_addr, void *buf, unsigned int len)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_mem_read(pcie_dev, die_index, relative_addr, buf, len);
	}
	return -ENODEV;
}

static inline int vastai_pci_mem_write_devid(unsigned dev_id, unsigned int die_index, u64 relative_addr, void *buf, unsigned int len)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_mem_write(pcie_dev, die_index, relative_addr, buf, len);
	}
	return -ENODEV;
}

static inline int vastai_pci_malloc_udma_buf_devid(unsigned int dev_id, struct vastai_dma_buf *dm, u32 len)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_malloc_udma_buf(pcie_dev, dm, len);
	}
	return -ENODEV;
}

static inline int vastai_pci_free_udma_buf_devid(unsigned int dev_id, struct vastai_dma_buf *dm)
{
	struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(dev_id);
	if (pcie_dev) {
		return vastai_pci_free_udma_buf(pcie_dev, dm);
	}
	return -ENODEV;
}

struct vastai_msgq {
	u32 end;
	u32 start;
	u32 wr;
	u32 rd;
};

int vastai_pci_msgq_wr(struct vastai_pci_info *priv,
		       unsigned int die_index,
		       struct  msgq *msgq,
		       void *data,
		       u32 data_size);

typedef struct PCIE_cbits_s {
	/** Control bits */
	uint8_t control_bits;
}PCIE_cbits;

/** uDMA transfer size and control byte */
typedef struct PCIE_sz_ctrl_s {
	/** Number of bytes to be transferred.  For max bulk transfer size, set
	 * to zero */
	uint32_t size : 24;
	/** Control byte */
	PCIE_cbits ctrl_bits;
}PCIE_sz_ctrl;

/** uDMA status bytes */
typedef struct PCIE_sbytes_s {
	/** System (local) bus status */
	uint8_t sys_status;
	/** External (remote) bus status */
	uint8_t ext_status;
	/** uDMA channel status */
	uint8_t chnl_status;
	/** Reserved */
	uint8_t reserved_0;
}PCIE_sbytes;

/** uDMA Descriptor structure */
struct PCIE_xd_desc_s {
	/** Low 32 bits of system address */
	uint32_t sys_lo_addr;
	/** High 32 bits of system address */
	uint32_t sys_hi_addr;
	/** Access attributes for system bus */
	uint32_t sys_attr;
	/** Low 32 bits of external address */
	uint32_t ext_lo_addr;
	/** High 32 bits of external address */
	uint32_t ext_hi_addr;
	/** Access attributes for external bus */
	uint32_t ext_attr;
	/** High 32 bits of access attributes for external bus */
	uint32_t ext_attr_hi;
	/** Transfer size and control byte */
	PCIE_sz_ctrl size_and_ctrl;
	/** Transfer status.  This word is written by uDMA engine, and can be
	 * read to determine status. */
	PCIE_sbytes status;
	/** Low 32bits of pointer to next descriptor in linked list */
	uint32_t next;
	/** High 32bits of pointer to next descriptor in linked list */
	uint32_t next_hi_addr;
};
typedef struct PCIE_xd_desc_s PCIE_xd_desc;

struct buf_desc_buf {
	struct buf_desc desc;
	struct vastai_dma_buf buf;
};

unsigned long vastai_pci_copy_to_user(void __user *to, const void *from, unsigned long n);
unsigned long vastai_pci_copy_from_user(void *to, const void __user *from, unsigned long n);
struct dma_buf *vastai_alloc_dma_buf(struct vastai_pci_info *pci_dev, int size);
struct dma_buf *vastai_alloc_dma_buf_sg(struct vastai_pci_info *pci_info, int size);
int vastai_device_reset(struct vastai_pci_info *priv, u32 flags);
int vastai_send_pcie_cmd(struct vastai_pci_info *priv, u32 die_index, u32 sub_cmd);
bool vastai_dev_not_ready(struct vastai_pci_info *priv);

int va_register_pcie_interrupt(struct vastai_pci_info *priv,
		u32 interrupt_id,
		void (*handler_function)(void *),
		void *arg);
int va_unregister_pcie_interrupt(struct vastai_pci_info *priv,
		u32 interrupt_id);

int va_config_initial_bars(struct vastai_pci_info *priv);

int vastai_pci_reinitial(struct vastai_pci_info *priv, bool is_conditon);
int vastai_pci_flr(void *pci_info);

u64 vastai_get_host_time_ns(void);
int vastai_pci_send_msg(void *pcie_dev,
			int die_index,
			int index,
			void *msg_buf, /* what's user want pcie send to device */
			u32 msg_len);

int vastai_dev_cedar_send_cmd(void *pci_info,
			void *cedar_info,
			int dev_id,
			int die_id,
			void *hash_output);

uint64_t vastai_cedar_addr_bypass_mpu(void *pci_info, uint64_t dev_addr);

/*==================== DMA transfer && MSG api ====================*/
/* Don't call this function at irq, This only use spin_lock */
int vastai_pci_send_msg_pid(void *pcie_dev, int die_index,
			int index, void *msg_buf, u32 msg_len, pid_t pid);

int vastai_pci_receive_msg(void *pcie_dev, int die_index,
			int index, void *elem);
void dump_bar_addr_map(struct vastai_pci_info *priv);
int vastai_ctl_file_init(struct vastai_pci_info *pci_info);
void vastai_ctl_file_deinit(struct vastai_pci_info *pci_info);
int vastai_version_file_init(struct vastai_pci_info *pci_info);
void vastai_version_file_deinit(struct vastai_pci_info *pci_info);
void vastai_pci_show_fw_ver(struct vastai_pci_info *pci_info, u32 mask);
void vastai_pci_get_fw_ver(struct vastai_pci_info *priv, u32 mask);

void dump_smcu_ring_buf(struct vastai_pci_info *priv);
int vastai_submit_smcu_task_and_done(struct vastai_pci_info *priv,
								struct smcu_task_info *desc,
								struct smcu_rw_ret_info *smcu_ret);
int vastai_flr_gmcu_task(struct vastai_pci_info *priv);
#define to_vastai_file_info(ptr) container_of(ptr, struct vastai_file_info, dev)
int vastai_pci_numa_cpu_info_all(char *buf, int len);

void vastai_die_lock(void *pci_info);
void vastai_die_unlock(void *pci_info);

#ifdef CONFIG_VASTAI_TOOLS_SUITE
typedef enum { RESET_START, RESET_END } reset_event;
#endif
#endif /* end of __VASTAI_PCI_H__ */
