/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_PCI_BOOT_H__
#define __VASTAI_PCI_BOOT_H__

#include "vastai_pci.h"

#define VASTAI_BOOT_WQ_DELAYED_MS           (0)
#define VASTAI_BOOT_POLL_SLEEP_MS           (1)
#define VASTAI_BOOT_POLL_SLEEP_CNT          (100000)  // 15000
#define VASTAI_BOOT_LOCK_POLL_SLEEP_CNT     (200000)  // 200s
#define VASTAI_DIE_CONFIG_TABLE_NUM         (4)


union mcu_dsp_status {
	u32 val;
	struct {
		u32 smcu_img : 1;
		u32 cmcu_img : 1;
		u32 lmcu0_img : 1;
		u32 lmcu1_img : 1;
		u32 lmcu2_img : 1;
		u32 lmcu3_img : 1;
		u32 lmcu4_img : 1;
		u32 lmcu5_img : 1;
		u32 lmcu6_img : 1;
		u32 lmcu7_img : 1;
		u32 odsp0_img : 1;
		u32 odsp1_img : 1;
		u32 odsp2_img : 1;
		u32 odsp3_img : 1;
		u32 odsp4_img : 1;
		u32 odsp5_img : 1;
		u32 odsp6_img : 1;
		u32 odsp7_img : 1;
		u32 vdmcu0_img : 1;
		u32 vdmcu1_img : 1;
		u32 vdmcu2_img : 1;
		u32 vemcu0_img : 1;
		u32 vemcu1_img : 1;
		u32 vemcu2_img : 1;
		u32 vemcu3_img : 1;
		u32 vdsp0_img : 1;
		u32 vdsp1_img : 1;
		u32 vdsp2_img : 1;
		u32 vdsp3_img : 1;
		u32 ddr_attr: 3;
	} bit;
};

struct vastai_die_cfg {
	u8 val;
	u8 die_num;
	u8 die_id;
	u8 pkg_id;
	u8 fn_id[8];
};

struct vastai_fw {
	const u8 *data;
	size_t size;
	u64 addr;
	const void *priv;
};


int vastai_pci_is_boot_done(struct vastai_pci_info *priv);
int vastai_pci_boot(struct vastai_pci_info *priv);
void vastai_pci_boot_wq(struct work_struct *work);
int vastai_pci_reboot(struct vastai_pci_info *priv);
int vastai_pci_boot_fw(struct vastai_pci_info *priv);
int vastai_pci_poll_smcu_stage(struct vastai_pci_info *priv, u8 die_id, u32 smcu_bootstage);
int vastai_pci_set_host_stage(struct vastai_pci_info *priv, u8 die_id, u32 host_bootstage);

int vastai_pci_xspi_get_bin_mode(const u8 *__data, size_t size, u8 partition, u8 *mode);
int vastai_pci_flash_xspi_bin(struct vastai_pci_info *priv, u8 *path_name, const void *bin_buf,
		size_t buf_size, u8 partition, u8 *mode);
int xspi_dma_read(struct vastai_pci_info *priv, u8 *data,
		size_t size, u64 fw_addr);
int xspi_dma_write(struct vastai_pci_info *priv, const u8 *data,
		size_t size, u64 fw_addr);
int vastai_pci_flash_xspi_fw(struct vastai_pci_info *priv, u8 *path_name, const void *bin_buf,
		size_t buf_size, u8 partition);
int vastai_pci_flash_xspi(struct vastai_pci_info *priv, struct char_drv_info *dev, u8 partition);
int vastai_pci_read_xspi(struct vastai_pci_info *priv, unsigned long addr, void *buf,
		u32 len, u8 partition, u8 mode);
int vastai_pci_update_flag_xspi(struct vastai_pci_info *priv, u8 partition, u8 mode);
int vastai_pci_flash_bmcu_bin(struct vastai_pci_info *priv, u8 *path_name, const void *bin_buf, size_t buf_size);
int vastai_pci_flash_bmcu(struct vastai_pci_info *priv, struct char_drv_info *dev);
int vastai_pci_switch_bmcu(struct vastai_pci_info *priv, struct char_drv_info *dev);
int vastai_pci_flash_bmcu_v2(struct vastai_pci_info *priv, const void *bin_buf, size_t buf_size);
int vastai_pci_switch_bmcu_v2(struct vastai_pci_info *priv, const void *bin_buf, size_t buf_size);
int vastai_pci_read_pmcu_boot_mode(struct vastai_pci_info *priv);
#endif /* end of __VASTAI_PCI_BOOT_H__ */
