/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_PCI_DBG_H__
#define __VASTAI_PCI_DBG_H__

#include <linux/printk.h>

#define VASTAI_PCI_DEBUG_PREFIX VASTAI_PCI_DRIVER_PREFIX
#define DUMMY_DIE_ID 0xff

#ifdef pr_fmt
#undef pr_fmt
#endif
#define pr_fmt(fmt) VASTAI_PCI_DEBUG_PREFIX fmt

#define VASTAI_PCI_NORMAL_LEVEL (0x01)
#define VASTAI_PCI_DEBUG_LEVEL	(0x02)
#define VASTAI_PCI_DATA_LEVEL	(0x04)
#define VASTAI_PCI_PERF_LEVEL	(0x08)

#define VASTAI_GET_PCI_NAME(pcie) ({ \
	struct vastai_pci_info *__priv = pcie; \
	(__priv?pci_name((__priv)->dev) : "NULL"); })

#define VASTAI_GET_DIE_INDEX(pcie, die_id) ({\
	struct vastai_pci_info *__priv = pcie; \
	((__priv && ((die_id) < VASTAI_SV100_MAX_DIE_NUM)) ? \
	 (__priv)->dies[die_id].die_index : 0xff); })

extern unsigned char va_pci_log_level;

/* va_pci_log_level = 0, printing pr_err information */
#define VASTAI_PCI_ERR(pcie, die_id, fmt, args...) 				  \
	do {									  \
		pr_err("%s:%02x:%02x: ERROR " fmt,	\
			VASTAI_GET_PCI_NAME(pcie),				  \
			die_id,							  \
			VASTAI_GET_DIE_INDEX(pcie, die_id),			  \
			##args);						  \
	} while (0)

#define VASTAI_PCI_ERR_RATELIMIT(fmt, args...) \
	pr_err_ratelimited(" error: " fmt, ##args)

/* VASTAI_PCI_NORMAL_LEVEL, printing pr_info information */
#define VASTAI_PCI_INFO(pcie, die_id, fmt, args...)							  \
	do {										  \
		if (va_pci_log_level & VASTAI_PCI_NORMAL_LEVEL){			  \
			pr_info("%s:%02x:%02x: INFO " fmt,					  \
				VASTAI_GET_PCI_NAME(pcie),				  \
				die_id,							  \
				VASTAI_GET_DIE_INDEX(pcie, die_id),			  \
				##args);						  \
		}									  \
	} while (0)
#define VASTAI_PCI_INFO_RATELIMIT(fmt, args...)                     \
	do {                                                        \
		if (va_pci_log_level & VASTAI_PCI_NORMAL_LEVEL) \
			pr_info_ratelimited(": " fmt, ##args);      \
	} while (0)

/* VASTAI_PCI_DEBUG_LEVEL, printing debug information */
#define VASTAI_PCI_DBG(pcie, die_id, fmt, args...)		   \
	do {                                                       \
		if (va_pci_log_level & VASTAI_PCI_DEBUG_LEVEL){\
			pr_info("%s:%02x:%02x: DEBUG " fmt,				  \
				VASTAI_GET_PCI_NAME(pcie),				  \
				die_id,							  \
				VASTAI_GET_DIE_INDEX(pcie, die_id),			  \
				##args);						  \
		}									  \
	} while (0)
#define VASTAI_PCI_DBG_RATELIMIT(fmt, args...)                       \
	do {                                                         \
		if (va_pci_log_level & VASTAI_PCI_DEBUG_LEVEL)   \
			pr_info_ratelimited(" debug: " fmt, ##args); \
	} while (0)

/* VASTAI_PCI_DATA_LEVEL, printing data or dump information */
#define VASTAI_PCI_HEX_DUMP(level, prefix_str, prefix_type, rowsize,         \
			    groupsize, buf, len, ascii)                      \
	do {                                                                 \
		if (va_pci_log_level & VASTAI_PCI_NORMAL_LEVEL)            \
			print_hex_dump(level, prefix_str, prefix_type,       \
				       rowsize, groupsize, buf, len, ascii); \
	} while (0)

/* VASTAI_PCI_PERF_LEVEL, printing perf information */
#define VASTAI_PCI_PERF(fmt, args...)                             \
	do {                                                      \
		if (va_pci_log_level & VASTAI_PCI_PERF_LEVEL) \
			trace_printk(" debug: " fmt, ##args);     \
	} while (0)

#if 0
#define VASTAI_PCI_FUNC_ENTERY \
	VASTAI_PCI_INFO_RATELIMIT("%s entry!\n", __func__)
#define VASTAI_PCI_FUNC_EXIT VASTAI_PCI_INFO_RATELIMIT("%s done!\n", __func__)
#endif
#define VASTAI_PCI_FUNC_ENTERY
#define VASTAI_PCI_FUNC_EXIT

#endif /* end of __VASTAI_PCI_DBG_H__ */
