/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_SV100_REG_H__
#define __VASTAI_SV100_REG_H__

#if 1  // by keye, only for debug sg100
#include "csram_partition.h"   
#include "ddr_partition.h" 

#define LOG_MSG_FLAG                (CSRAM_BASE_ADDR + 0x000F3098) // 4 bytes

/* The tx is host to device, rx is device to host */
#define PCIE_HOST_TO_DEV_TRANSF_INFO 	(CSRAM_BASE_ADDR + 0x00000000) // 40K
#define PCIE_DEV_TO_HOST_TRANSF_INFO 	(CSRAM_BASE_ADDR + 0x0000A000) // 40K
#define PCIE_DMA_HOST_TO_DEV_CFG	(CSRAM_BASE_ADDR + 0x00014000)     // 80K
#define PCIE_DMA_HOST_TO_DEV_DESC	(CSRAM_BASE_ADDR + 0x00023000)
#define PCIE_DMA_DEV_TO_HOST_CFG	(CSRAM_BASE_ADDR + 0x00028000)     // 80K
#define PCIE_DMA_DEV_TO_HOST_DESC	(CSRAM_BASE_ADDR + 0x00037000)

#define VASTAI_PKG1_BASE                0x2000000000ULL      // 128G


#endif

//#define VASTAI_PKG1_BASE          (0x1000000000UL)                   // 64G

#define PCIE_LINK_CONTROL_STATUS_REG_OFFSET  (0xD0)
#define PCIE_LINK_CONTROL_STATUS_REG2_OFFSET (0xF0)

/* address map */
#define VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR      (0x08000000UL)
#define VASTAI_PCIE_EP_CONFIG_BASE_ADDR	       (0x08300000UL)
#define VASTAI_AXI_CSR_BASE_ADDR	           (0x08BC0000UL)
#define VASTAI_PCIE_EP_APB_BASE_ADDR	       (0x08E00000UL)
#define VASTAI_PCIE_RC_APB_BASE_ADDR	       (0x09700000UL)

#define VASTAI_CSRAM_ADDR(x)   (x * VASTAI_PKG1_BASE + CSRAM_BASE_ADDR)
#define VASTAI_DDR_SPACE_ADDR(x) \
	(x * VASTAI_PKG1_BASE + DDR_BASE_ADDR)

#define XSPI_FW_DDR_DL_ADDR (0x1030000000UL)
#define BMCU_FW_DDR_DL_ADDR (0x1031000000UL)

/* AXI CSR */
#define VASTAI_MCU_DSP_STATUS_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + 0x6E8)


/* vastai controller wrap registers */
#define VASTAI_PCI_CONFIGURATION_IO_0 (VASTAI_PCIE_EP_APB_BASE_ADDR + 18 * 4)

//#define VASTAI_PCI_BAR0_ADDR (VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR)
//#define VASTAI_PCI_BAR1_ADDR (VASTAI_PCIE_EP_IO_SPACE_BASE_ADDR + VASTAI_PKG1_BASE)
//#define VASTAI_PCI_BAR2_ADDR (DDR_BASE_ADDR)
//#define VASTAI_PCI_BAR4_ADDR (DDR_BASE_ADDR + VASTAI_PKG1_BASE)

////#define VASTAI_PCI_BAR0_ADDR (0x8000000000)   // by keye
////#define VASTAI_PCI_BAR1_ADDR (0x0)
////#define VASTAI_PCI_BAR2_ADDR (0x1200000000)
////#define VASTAI_PCI_BAR4_ADDR (0x1000000000)

/* cadence controller registers */
#define VASTAI_PCI_CDN_CONTROL_BASE_ADDR \
	(VASTAI_PCIE_EP_APB_BASE_ADDR + 0x100000)
#define VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR + 0x100000)
#define VASTAI_PCI_CDN_VENDOR_ID_REG \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x44)
#define VASTAI_PCI_CDN_LINKWIDTH_CONTROL_REG \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x50)
#define VASTAI_PCI_CDN_PHY_FUNC_BAR_CFG_REG0 \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x240)
#define VASTAI_PCI_CDN_PHY_FUNC_BAR_CFG_REG1 \
	(VASTAI_PCI_CDN_LOCAL_MANAGEMENT_REG + 0x244)

#define VASTAI_PCI_CDN_AXI_CONFIGURATION_REG \
	(VASTAI_PCI_CDN_CONTROL_BASE_ADDR + 0x400000)
#define VASTAI_PCI_CDN_FUNC0_BAR0_EP_INBOUND_ADDR_TRANS_REG0 \
	(VASTAI_PCI_CDN_AXI_CONFIGURATION_REG + 0x840)

#define VASTAI_PCIE_EP_CONFIG_VID_DID (VASTAI_PCIE_EP_CONFIG_BASE_ADDR)
#define VASTAI_PCIE_EP_CONFIG_CMD_STATUS \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x4)
#define VASTAI_PCIE_EP_CONFIG_BAR0 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x10)
#define VASTAI_PCIE_EP_CONFIG_BAR1 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x14)
#define VASTAI_PCIE_EP_CONFIG_BAR2 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x18)
#define VASTAI_PCIE_EP_CONFIG_BAR3 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x1C)
#define VASTAI_PCIE_EP_CONFIG_BAR4 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x20)
#define VASTAI_PCIE_EP_CONFIG_BAR5 \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x24)
#define VASTAI_PCIE_EP_CONFIG_MSI_CTRL \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x90)
#define VASTAI_PCIE_EP_CONFIG_MSI_MSG_L \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR+ 0x94)
#define VASTAI_PCIE_EP_CONFIG_MSI_MSG_H \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0x98)
#define VASTAI_PCIE_EP_CONFIG_MSIX_CTRL \
	(VASTAI_PCIE_EP_CONFIG_BASE_ADDR + 0xB0)

#define VASTAI_MULTI_DIE_BAR0_PCIE_ADDR (0x8000000UL)
#define VASTAI_MULTI_DIE_BAR1_PCIE_ADDR (0x10000000UL)
#define VASTAI_MULTI_DIE_BAR2_PCIE_ADDR (0x800000000UL)
#define VASTAI_MULTI_DIE_BAR4_PCIE_ADDR (0UL)
#define VASTAI_MULTI_DIE_MSI_ADDR	   (0x4000000000UL)

#define CMCU_GRP12_WR (96 * 4)// 0x180

#define CMCU_MSGQ_GRP0_AFULL_OFFSET  (110 * 4)// 0x1B8
#define CMCU_MSGQ_GRP0_AEMPTY_OFFSET (111 * 4)// 0x1BC
#define CMCU_MSGQ_GRP0_WR_OFFSET     (112 * 4)// 0x1C0
#define CMCU_MSGQ_GRP0_RD_OFFSET     (113 * 4)// 0x1C4
/*
 * CMCU_MSGQ_GRP0_INT_OFFSET
 * bit0:full_msk
 * bit1:afull_msk
 * bit2:empty_msk
 * bit3:nempty_msk
 * bit4:aempty_msk
 * bit5~7:reserved
 * bit8:full_int
 * bit9:afull_int
 * bit10:empty_int
 * bit11:nempty_int
 * bit12:aempty_int
 */
#define CMCU_MSGQ_GRP0_INT_OFFSET    (114 * 4)// 0x1C8
#define CMCU_MSGQ_GRP1_AFULL_OFFSET  (115 * 4)// 0x1CC
#define CMCU_MSGQ_GRP1_AEMPTY_OFFSET (116 * 4)// 0x1D0
#define CMCU_MSGQ_GRP1_WR_OFFSET     (117 * 4)// 0x1D4
#define CMCU_MSGQ_GRP1_RD_OFFSET     (118 * 4)// 0x1D8
/*
 * CMCU_MSGQ_GRP1_INT_OFFSET
 * bit0:full_msk
 * bit1:afull_msk
 * bit2:empty_msk
 * bit3:nempty_msk
 * bit4:aempty_msk
 * bit5~7:reserved
 * bit8:full_int
 * bit9:afull_int
 * bit10:empty_int
 * bit11:nempty_int
 * bit12:aempty_int
 */ 
#define CMCU_MSGQ_GRP1_INT_OFFSET (119 * 4)// 0x1DC
#define VDSP0_GRP5_WR		  (398 * 4)// 0x638
#define VDSP1_GRP5_WR		  (400 * 4)// 0x640
#define VDSP2_GRP5_WR		  (402 * 4)// 0x648
#define VDSP3_GRP5_WR		  (404 * 4)// 0x650
#define VDMCU0_GRP5_WR		  (414 * 4)// 0x678
#define VDMCU1_GRP4_WR		  (394 * 4)// 0x628
#define VDMCU2_GRP5_WR		  (418 * 4)// 0x688

#define CMCU_GRP12_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + CMCU_GRP12_WR)
#define CMCU_MSGQ_GRP0_AFULL_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_AFULL_OFFSET)
#define CMCU_MSGQ_GRP0_AEMPTY_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_AEMPTY_OFFSET)
#define CMCU_MSGQ_GRP0_WR_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_WR_OFFSET)
#define CMCU_MSGQ_GRP0_RD_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_RD_OFFSET)
#define CMCU_MSGQ_GRP0_INT_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP0_INT_OFFSET)
#define CMCU_MSGQ_GRP1_AFULL_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_AFULL_OFFSET)
#define CMCU_MSGQ_GRP1_AEMPTY_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_AEMPTY_OFFSET)
#define CMCU_MSGQ_GRP1_WR_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_WR_OFFSET)
#define CMCU_MSGQ_GRP1_RD_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_RD_OFFSET)
#define CMCU_MSGQ_GRP1_INT_REG \
	(VASTAI_AXI_CSR_BASE_ADDR + CMCU_MSGQ_GRP1_INT_OFFSET)
#define VDSP0_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP0_GRP5_WR)
#define VDSP1_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP1_GRP5_WR)
#define VDSP2_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP2_GRP5_WR)
#define VDSP3_GRP5_WR_REG  (VASTAI_AXI_CSR_BASE_ADDR + VDSP3_GRP5_WR)
#define VDMCU0_GRP5_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU0_GRP5_WR)
#define VDMCU1_GRP4_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU1_GRP4_WR)
#define VDMCU2_GRP5_WR_REG (VASTAI_AXI_CSR_BASE_ADDR + VDMCU2_GRP5_WR)

#define VASTAI_EACH_CHN_DESC_CNT 64
#define VASTAI_PCIE_2_VDSP_MSG_DEPTH ((MSG_PCIE_TO_VDSP1 - MSG_PCIE_TO_VDSP0 -\
	sizeof(struct vastai_fifo))/ sizeof(union buf_data))

#define SMCU_2_HOST_DMA_DONE (0)
#define SMCU_2_HOST_INT0     (1)
#define SMCU_2_HOST_INT1     (2)
#define VDSP0_2_HOST_INT0    (10)
#define VDSP0_2_HOST_INT1    (11)
#define VDSP1_2_HOST_INT0    (12)
#define VDSP1_2_HOST_INT1    (13)
#define VDSP2_2_HOST_INT0    (14)
#define VDSP2_2_HOST_INT1    (15)
#define VDSP3_2_HOST_INT0    (16)
#define VDSP3_2_HOST_INT1    (17)
#define CMCU_2_HOST_INT0     (20)
#define CMCU_2_HOST_INT1     (21)
#define VEMCU0_2_HOST_INT    (22)
#define VEMCU1_2_HOST_INT    (23)
#define VEMCU2_2_HOST_INT    (24)
#define VEMCU3_2_HOST_INT    (25)
#define VDMCU0_2_HOST_INT    (26)
#define VDMCU1_2_HOST_INT    (27)
#define VDMCU2_2_HOST_INT    (28)

enum heartheart_beat_cnt_reg_core_id {
	HEARTBEAT_SMCU_THREAD	= 0,
	HEARTBEAT_SMCU			= 1,
	HEARTBEAT_PMCU			= 2,
	HEARTBEAT_CMCU			= 3,
	HEARTBEAT_OMCU0			= 4,
	HEARTBEAT_OMCU1			= 5,
	HEARTBEAT_GMCU0			= 6,
	HEARTBEAT_GMCU1			= 7,
	HEARTBEAT_GMCU2			= 8,
	HEARTBEAT_GMCU3			= 9,
	HEARTBEAT_VEMCU0		= 10,
	HEARTBEAT_VEMCU1V		= 11,
	HEARTBEAT_VEMCU2		= 12,
	HEARTBEAT_VEMCU3		= 13,
	HEARTBEAT_ODSP0			= 14,
	HEARTBEAT_ODSP1			= 15,
	HEARTBEAT_VDSP0			= 16,
	HEARTBEAT_VDSP1			= 17
};

#endif /* end of __VASTAI_SV100_REG_H__ */
