/*
 * vastai_pci - driver a Vastai PCI device
 * Copyright (C) 2021 Vastai Technologies All Rights Reserved.
 * This work is licensed under the terms of the GNU GPL, version 2.
 *
 * Create on: 2020/09/25
 * Author: gang.wang
 */

#ifndef __VASTAI_UDMA_ENGINE_H__
#define __VASTAI_UDMA_ENGINE_H__

#include <linux/list.h>
#include <linux/semaphore.h>
#include <linux/spinlock_types.h>
#include "vastai_pci.h"
#include "vastai_dmabuf.h"
#include "vastai_fifo.h"
#include "vastai_pci_test.h"
//#include "vastai_fifo.h"

/* maximum length transfer 2^24 bytes, i.e. 16M bytes. */
#define VASTAI_UDMA_LENGTH_MAX	(0x1000000)
#define VASTAI_UDMA_LENGTH_MASK (0xFFFFFF)
#define VASTAI_UDMA_MALLOC_MAX	(0x400000)

#define CPDREF_TEST_FAILURE 1
#define DEAFULT_VASTAI_PID_NUM 1
//#define CPDREF_FUNC_TIMEDOUT     0x05

#define UDMA_DESC_SIZE (sizeof(PCIE_xd_desc))

#ifdef VIDEO_COMPAT_ENC
struct vastai_channel_buf {
	u64 user_buf;
	u32 width;
	u32 high;
	u32 src_width_offset;
	u32 src_high_offset;
	u32 dst_width_offset;
	u32 dst_high_offset;
}__attribute__((packed));
#else
struct vastai_channel_buf {
	void *user_buf;
	u32 width;
	u32 high;
	u32 src_width_offset;
	u32 src_high_offset;
	u32 dst_width_offset;
	u32 dst_high_offset;
};
#endif

struct vastai_pid_entry {
	struct list_head list;
	int pid;
};

struct vastai_credit {
	struct vastai_pid_entry pid_entry_head;
	spinlock_t lock;
	struct semaphore credit_sem;
};

#define VASTAI_DMA_HIGH_PRIORITY  VASTAI_MEM_POOL_MIN_NR / 2
#define VASTAI_DMA_LOW_PRIORITY	  1
#define VASTAI_DMA_DEFAULT_CREDIT VASTAI_DMA_HIGH_PRIORITY
#define VASTAI_DMA_CREDIT_LIMIT	  10
#define VASTAI_DMA_SUBMIT_LIMIT	  4

typedef int (*set_desc_func)(struct vastai_dmadesc *, struct dma_buf *,
			     struct kchar_cmd *, int, u32, u32,
			     struct vastai_pci_info *);
int vastai_common_cut_dma_buf(u32 size, struct kchar_cmd *kcmd,
			      struct vastai_dmadesc *desc,
			      struct vastai_pci_info *pci_info,
			      struct dma_buf *dmabuf,
			      set_desc_func pSetDescFunc);
int vastai_ioctl_dma_start_set_desc(struct vastai_dmadesc *desc,
				    struct dma_buf *dmabuf,
				    struct kchar_cmd *kcmd, int i,
				    u32 done_size, u32 temp_size,
				    struct vastai_pci_info *pci_info);
int vastai_ioctl_dma_trans_set_desc(struct vastai_dmadesc *desc,
				    struct dma_buf *dmabuf,
				    struct kchar_cmd *kcmd, int i,
				    u32 done_size, u32 temp_size,
				    struct vastai_pci_info *pci_info);

int vastai_sdma_transfer_sync_pid(void *pci_info, int die_index,
				     union core_bitmap core_id,
				     struct vastai_dmadesc desc[], int desc_num,
				     int pid);
int vastai_sdma_transfer_asyn_pid(void *pci_info, int die_index,
				     union core_bitmap core_id,
				     struct vastai_dmadesc desc[], int desc_num,
				     int pid);
int get_vastai_pid(struct vastai_pci_info *pci_info);
int free_vastai_pid(struct vastai_pci_info *pci_info, u32 va_pid);
int vastai_sdma_transfer_by_uaddr_withmmu(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u32 is_host_to_dev,
			u64 dev_addr,
			u64 user_addr,
			u32 size, u32 pid);
int vastai_sdma_transfer_by_sg(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u32 is_host_to_dev,
			u64 dev_addr,
			u64 user_addr,
			u32 size);
int vastai_udma_init(struct vastai_sv100_die *die);
void vastai_udma_deinit(struct vastai_sv100_die *die);
int vastai_dev2dev_sdma_sync_pid(void *priv, int die_index, u64 src_addr, u64 dst_addr,
						u32 length, int pid);
int vastai_dev2dev_sdma_asyn_pid(void *priv, int die_index, u64 src_addr, u64 dst_addr,
						u32 length, int pid);

int vastai_salve2master_sdma_sync_pid(void *priv, int dir,
				u64 src_addr, u64 dst_addr, u32 length);
int vastai_salve2master_sdma_asyn_pid(void *priv, int dir,
				u64 src_addr, u64 dst_addr, u32 length);
int vastai_gfx_video_share_sync_pid(void *priv, int pid, struct dma_buf *dmabuf, int dir, u64 dev_addr);

int vastai_pcie_dma_raw_transfer_by_uaddr(struct vastai_pci_info *pci_info,
			int die_index,
			union core_bitmap core_id,
			u32 is_host_to_dev,
			u64 axi_addr,
			u64 user_addr,
			u32 size);

int vastai_dma_video(void *priv, int die_index, union core_bitmap core_id,
						 struct vastai_channel_buf *channel_buf, 
						 int channel_num,
						 u64 dev_addr,
						 int pid);

#define GET_UDMA_CHN(dir, is_rc) (dir + is_rc * 2)

struct vastai_udma_chn {
	struct vastai_fifo dma_desc_fifo;
	struct vastai_fifo trans_info_fifo;
	struct mutex trans_mutex;
};

struct vastai_udma_config {
	u64 trans_info_fifo_addr;
	u64 dma_desc_fifo_addr;
};

extern const struct vastai_udma_config vastai_udma_config[VASTAI_DMA_CHN];

#endif /* end of __VASTAI_UDMA_ENGINE_H__ */

#define VASTAI_DMA_ALIGN_SIZE  128
