#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include "vastai_pcie_user_api.h"
#include "diag_pcie_user_api.h"

enum CARD {
	DL = 0,
	AIC = 1,
};

#define PF_NUM		4
#define PLATFORM	DL

static char *load_file_name[4] =
{
	"rand_256M.bin",
	"rand_128M.bin",
	"rand_512k.bin",
	"rand_512.bin"
};

static int read_buf_from_file(const char *file_path, void *buf, int buf_size_byte)
{
	int ret = 0;
	int filep = open(file_path, O_RDWR);
	if (filep < 2) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output: open file_path failed!:XXXXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	int lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("lseek src file error, pls check ofst"); //output:lseek src file error, pls check ofst:XXX\n
		ret = ERR_CODE(0);
		goto CLOSE_SRC_FILE;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		perror("read src file error"); //output:read src file error:XXX\n
		ret = ERR_CODE(0);
	}

CLOSE_SRC_FILE:
	close(filep);
RET_ERROR_CODE:
	return ret;
}

static u16 checksum(u16 *packet, int packlen)
{
	unsigned long sum = 0;
	
	while (packlen > 1) {
		sum += *(packet++);
		packlen-=2;
	}

	if (packlen > 0)
		sum += *(unsigned char *)packet;

	while (sum >> 16)
		sum = (sum & 0xffff) + (sum >> 16);
	return (u16) ~sum;
}

static int get_file_checksum(const char *path, u16 *sum_data, int compare_size_byte)
{
	int ret = 0;
	void *buf = malloc(compare_size_byte);
	void *file_buf = buf;
	
	if (!buf) {
		perror("alloc memory failed"); //output：alloc memory failed：XXX\n
		return -1;
	}

	ret = read_buf_from_file(path, file_buf, compare_size_byte);
	if (ret != 0) {
		ret = -1;
		goto FREE_BUF;
	}

	*sum_data = checksum(file_buf, compare_size_byte);

FREE_BUF:
	free(buf);
	return ret;
}

static int compare_file_checksum(const char *path, u16 sum_data, int compare_size_byte)
{
	int ret = 0;
	void *buf = malloc(compare_size_byte);
	void *file_buf = buf;
	
	if (!buf) {
		perror("alloc memory failed"); //output：alloc memory failed：XXX\n
		ret = 0;
		return ret;
	}

	ret = read_buf_from_file(path, file_buf, compare_size_byte);
	if (ret != 0) {
		ret = -1;
		goto FREE_BUF;
	}

	if(sum_data != checksum(file_buf, compare_size_byte)) {
		ret = 0;
		goto FREE_BUF;
	}

	ret = 1;

FREE_BUF:
	free(buf);
	return ret;
}

int vastai_sdma_ddr_test()
{
	u64 file_size;
	int ret = 0;
	u64 addr;
#if ((PF_NUM == 1) && (PLATFORM == DL))
	u64 max_addr = 0x1800000000;
#elif ((PF_NUM == 1) && (PLATFORM == AIC))
	u64 max_addr = 0x1400000000;
#else
	u64 max_addr = 0x11F0000000;
#endif
	u64 i, j, k;
	u32 loop = 0;
	u16 sum_data;

	while(1)
	{
		loop++;
		printf("Test loop : %d.\n", loop);

		for (j = 0; j < 3; j++)
		{
			i = 0;
			file_size = vastai_get_file_size(load_file_name[j]);
			addr = 0x1040000000;
			get_file_checksum(load_file_name[j], &sum_data, file_size);
			while(addr < max_addr)
			{
				for (k = 0; k < PF_NUM; k++)
				{
					vastai_sdma_write_from_file(load_file_name[j], 0, addr, k, 0);
					vastai_sdma_read_to_file("tmp.bin", 0, addr, file_size, k, 0);
					ret = compare_file_checksum("tmp.bin", sum_data, file_size);
					if (ret < 0) {
						printf("ERROR: sdma_ddr_test read data file failed, errcode %d\n", ret);
						return -1;
					}
					if (ret != 1) {
						printf("ERROR: Different Match Compare.\n");
						return -2;
					}
					printf("loop:%d, PF:%lld, addr:0x%llx, %s test succeeded.\n", loop, k, addr, load_file_name[j]);
				}
				i++;
				addr = 0x1040000000 + file_size * i;
			}
		}
	}
	
	return 0;
}

int main(void)
{
	int ret = 0;

	ret = vastai_sdma_ddr_test();
	if (ret){
		printf("va_sdma_ddr_test failed\n");
		return 0;
	}

	return ret;
}
