#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "vastai_pcie_user_api.h"
#include "diag_pcie_user_api.h"

typedef unsigned int u32;
typedef unsigned long long int u64;

#define VASTAI_PCI_IOCTL_DMA_TRANS    3
#define VASTAI_PCI_IOCTL_DMA_TRANS_SG 4
#define ALIGN_SIZE (sysconf(_SC_PAGESIZE))
#define ALIGN(x, a)	(((x) + ((a) - 1)) & ~((a) - 1))
#define ALIGN_ADDR(x, a) (void *)ALIGN((unsigned long)(x), a)

struct va_ioctk_kchar_cmd {
	union {
		struct {
			u32 size;
			int dma_buf_fd;
		} alloc_cmd;

		struct {
			u32 is_dev_to_host;
			u32 dma_buf_fd;
			u64 axi_addr;
			u32 length;
			u32 die_index;
		} dma_start_cmd;

		struct {
			u32 is_dev_to_host;
			void *vir_addr;
			u64 axi_addr;
			u32 length;
			u32 die_index;
			int pid;
		} dma_trans_cmd;
		struct {
			u32 is_dev_to_host;
			u32 die_index;
			u64 axi_addr;
			struct vastai_channel_buf *channel_buf;
			int channel_num;
			int pid;
		} dma_trans_sg_cmd;
		struct {
			u32 die_index;
			u32 timeout_water;
			u32 is_timeout;
			void *log_ring_ctl;
		} log_poll_cmd;
	};
};

struct vastai_channel_buf {
	void *user_buf;
	u32 width;
	u32 high;
	u32 src_width_offset;
	u32 src_high_offset;
	u32 dst_width_offset;
	u32 dst_high_offset;
};

#define IMAGE_WITH     16384
#define IMAGE_HIGH     16384
#define IMAGE_SIZE     (IMAGE_WITH * IMAGE_HIGH)
#define IMAGE_WITH_PAD (0)
#define IMAGE_HIGH_PAD (0)
#define IMAGE_CHN      3

static int open_vastai_ctl(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/va%d_ctl", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!\n",
			 file_path);
		perror(temp_buf);
	}
	return filep;
}

static int vastai_trigger_dma_sg(struct vastai_channel_buf *channel,
				 int channel_num, u64 axi_addr,
				 u32 is_dev_to_host, u32 die_index)
{
	union die_index_data temp = { .val = die_index };
	int vastai_fd = open_vastai_ctl(temp.dev_id);
	struct va_ioctk_kchar_cmd kcmd = {
		.dma_trans_sg_cmd.is_dev_to_host = is_dev_to_host,
		.dma_trans_sg_cmd.axi_addr = axi_addr,
		.dma_trans_sg_cmd.die_index = die_index,
		.dma_trans_sg_cmd.channel_buf = channel,
		.dma_trans_sg_cmd.channel_num = channel_num,
		.dma_trans_sg_cmd.pid = 0,
	};
	if (vastai_fd < 0) {
		printf("open kchar is failed %d\n", vastai_fd);
		return vastai_fd;
	}

	int ret = ioctl(vastai_fd, VASTAI_PCI_IOCTL_DMA_TRANS_SG, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_PCI_MEM_COPY is failed %d\n", ret);
	}
	close(vastai_fd);
	return 0;
}

int dma_copy_sg_test(unsigned int num)
{
	void *mem[IMAGE_CHN];
	void *r_mem;
	int i;
	u32 j;
	u32 k;
	struct vastai_channel_buf *channel;
	int ret;
	u32 dst_off = 0;
	struct timeval begin, end;

	channel = malloc(sizeof(struct vastai_channel_buf) * IMAGE_CHN);
	if (!channel) {
		printf("malloc err \n");
	}

	for (i = 0; i < IMAGE_CHN; i++) {
		mem[i] = malloc(IMAGE_SIZE);
		if (!mem[i]) {
			printf("malloc err \n");
		}
	}

	for (i = 0; i < IMAGE_CHN; i++) {
		memset(mem[i], 0x5a + num + i, IMAGE_SIZE);
		//			memset(mem[i], 0xaa+i, 0x10);
		//			memset(mem[i] + IMAGE_SIZE - 0x10, 0xbb+i, 0x10);
	}

	for (i = 0; i < IMAGE_CHN; i++) {
		channel[i].width = IMAGE_WITH;
		channel[i].high = IMAGE_HIGH;
		channel[i].user_buf = mem[i];
		channel[i].src_width_offset = 0;
		channel[i].src_high_offset = 0;
		channel[i].dst_width_offset = IMAGE_WITH_PAD;
		channel[i].dst_high_offset = IMAGE_HIGH_PAD;
	}
	gettimeofday(&begin, NULL);
	ret = vastai_trigger_dma_sg(channel, IMAGE_CHN, 0x1040000000, 0, 0);
	if (ret) {
		printf("va_trigger_dma_sg err %d\n", ret);
	}
	gettimeofday(&end, NULL);
	printf("cost %f ms\n",
	       1000 * (end.tv_sec - begin.tv_sec) +
		       ((end.tv_usec - begin.tv_usec) / 1000.0));
	/*check data*/
	r_mem = malloc((IMAGE_WITH + IMAGE_WITH_PAD) *
		       (IMAGE_HIGH + IMAGE_HIGH_PAD) * IMAGE_CHN);
	if (!r_mem) {
		printf("malloc err \n");
	}
	
	ret = vastai_sdma_read_buf(0x1040000000, r_mem, (IMAGE_WITH + IMAGE_WITH_PAD) * (IMAGE_HIGH + IMAGE_HIGH_PAD) *
			IMAGE_CHN, 0, 0);

	if (ret) {
		printf("va_trigger_dma_user_addr err %d\n", ret);
	}
	for (i = 0; i < IMAGE_CHN; i++) {
		for (j = 0; j < channel[i].high; j++) {
			ret = memcmp((char *)r_mem + dst_off,
				     mem[i] + (channel[i].width * j),
				     channel[i].width);
			if (ret) {
				printf("not match channel:%d, line:%d, num:%d, dst_off:%u\n",
				       i, j, num, dst_off);
				for (k = 0; k < channel[i].width; k++) {
					#if 0
					printf("real:%x, ori:%x\n",
					       ((char *)r_mem + dst_off)[k],
					       ((char *)(mem[i] +
							 (channel[i].width *
							  j)))[k]);
					#endif
				}
				break;
			}
			dst_off +=
				channel[i].width + channel[i].dst_width_offset;
		}
		dst_off += (channel[i].width + channel[i].dst_width_offset) *
			   channel[i].dst_high_offset;
	}
	if (!ret) {
		printf("image check ok, num:%d\n", num);
	}
	free(r_mem);

	for (i = 0; i < IMAGE_CHN; i++)
		free(mem[i]);

	free(channel);
	return 0;
}

int main(void)
{
	for (int i = 0; i < 10; i++)
		dma_copy_sg_test(i);
	return 0;
}
