#!/bin/bash

# define print log level
LOG_LEVEL=1
# log info
function log_info(){
	content="[INFO] $(date '+%Y-%m-%d %H:%M:%S') $@"
	[ $LOG_LEVEL -le 2  ] && echo -e "\033[32m"  ${content} "\033[0m"
}
#log warn
function log_warn(){
	content="[WARN] $(date '+%Y-%m-%d %H:%M:%S') $@"
	[ $LOG_LEVEL -le 3  ] && echo -e "\033[33m" ${content} "\033[0m"
}
#log err
function log_err(){
	content="[ERROR] $(date '+%Y-%m-%d %H:%M:%S') $@"
	[ $LOG_LEVEL -le 4  ] && echo -e "\033[31m" ${content} "\033[0m"
}

# get the ko path
ko_file_path_arm=binary_sg100_linux_nulldrmws_debug/target_aarch64/va_pci.ko
ko_file_path_x86=binary_sg100_linux_nulldrmws_debug/target_x86_64/va_pci.ko

cur_path=$(readlink -f "$(dirname "$0")")
pcie_root="${cur_path}/../.."

# 0.1 Check script execution permissions ---------------->S
if [ $(id -u) != 0 ]; then
	log_err "You must be root to use the shell script!"
	exit 1
else
	log_info "Check script execution permissions [PASS]"
fi
# ------------------------------------------------------->E

# 0.2 Check the driver status --------------------------->S
output=$(lsmod | grep va_pci | wc -l)
if [[ $? == 0 ]]; then
    if [[ $output == 1 ]]; then
        log_err "Please uninstall the currently loaded driver!"
        exit 1
    fi
fi
# ------------------------------------------------------->E

# 0.3 test loop ----------------------------------------->S
loop_count=0
while true; do
    loop_count=$((loop_count + 1))
    log_info "Begin test: loop[$loop_count]"
    # 0.3.1 Insmod driver and check the status ---------->S
    # insmod ko
    cd $pcie_root
    if uname -m | grep -qi "aarch64"; then
        insmod $ko_file_path_arm
    else
        insmod $ko_file_path_x86
    fi
    # check the status of driver
    output=$(lsmod | grep va_pci | wc -l)
    if [[ $? == 0 ]]; then
        if [[ $output == 1 ]]; then
            log_info "Insmod va_pci successfully!"
        else
            log_err "Insmod va_pci failed!"
            exit 1
        fi
    fi
    # --------------------------------------------------->E

    # 0.3.2 Run pcie self test -------------------------->S
    cd tool/diag_test_simple
    log_info "Run pcie_self_test!"

    kchar_num=$(find /dev/ -type c -name "kchar*" | wc -l)
    log_info "kchar_num=$kchar_num"
    for ((i = 0; i < kchar_num; i++)); do
        ./pcie_self_test $i
        output=$(./pcie_self_test $i)
        if echo "$output" | grep -q "failed"; then
            log_err "pcie_self_test $i failed!"
            exit 1
        fi
    done
    log_info "pcie_self_test [Pass]"
    cd ../..
    # --------------------------------------------------->E

    # 0.3.3 Rmmod driver and check the status ----------->S
    # rmmod ko
    rmmod va_pci
    # check the status of driver
    output=$(lsmod | grep va_pci | wc -l)
    if [[ $? == 0 ]]; then
        if [[ $output == 0 ]]; then
            log_info "Rmmod va_pci successfully!"
        else
            log_err "Rmmod va_pci failed!"
            exit 1
        fi
    fi
    # --------------------------------------------------->E
    echo
    sleep 5
done
# ------------------------------------------------------->E
