#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <sched.h>
#include "vastai_pcie_user_api.h"
#include "diag_pcie_user_api.h"
#define ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))

#define TEST_FN (0)
int write_read_32bit_base_dev0_die0(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x1040000000;

	int ret = diag_vastai_pcie_write_reg_32bit(test_addr, gold_value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned int value;
	ret = diag_vastai_pcie_read_reg_32bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 0. \n",
		       __FUNCTION__);
		printf("gold_value=0x%x value=0x%x\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_64bit_base_dev0_die0(void)
{
	const unsigned long long gold_value = 0x01234567ABCDEF01;
	const unsigned long long test_addr = 0x1040000000;

	int ret = diag_vastai_pcie_write_reg_64bit(test_addr, gold_value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value;
	ret = diag_vastai_pcie_read_reg_64bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 0. \n",
		       __FUNCTION__);
		printf("gold_value=0x%llx value=0x%llx\n", gold_value, value);
		return -2;
	}
	return 0;
}



int write_read_buf_base_dev1_die0(void)
{
	int i, j;
	const unsigned long long gold_value[] = {
		0x101234567ABCDEF0,
		0x0101234567ABCDEF,
		0xF0101234567ABCDE,
		0xEF0101234567ABCD,
	};
	const unsigned long long test_addr = 0x1040000000;
	int ret = diag_vastai_pcie_write_reg_buf(test_addr, (void *)gold_value,
					    sizeof(gold_value), 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value[ARRAY_SIZE(gold_value)];
	ret = diag_vastai_pcie_read_reg_buf(test_addr, (void *)value, sizeof(value),
				       1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	for (i = 0; i < (int)ARRAY_SIZE(value); i++) {
		if (gold_value[i] != value[i]) {
			/* dump two buf */
			printf("ERROR: %s Different Match Compare.\n",
			       __FUNCTION__);
			for (j = 0; j < (int)ARRAY_SIZE(value); j++) {
				printf("gold_value=0x%llx value=0x%llx\n",
				       gold_value[j], value[j]);
			}
			return -2;
		}
	}

	return 0;
}

int test_read_cfg()
{
	int ret;
	unsigned int value;
	int i;
	for(i=0;i<10;i++) {
		ret = vastai_pcie_read_cfg_32bit(i*4, &value,0, 0);
		printf("offset[%d] = 0x%x\n", i*4, value);
		if(ret){
			return -1;
		}
	}
	
	return 0;
}

int test_read_reg_file()
{
	// int ret = vastai_pcie_read_reg_file("./read_src.ttmp", 0x0,
	// 				    0x800000, 0x30, 0, DIE_0);

	int ret = diag_vastai_smcu_read_reg_file("./read.txt", 0x0,
	 				    0x800000, 0x100, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	
	return 0;
}

int test_mmio_read_write()
{
	int ret;
	unsigned int value;

	ret = vastai_pci_mmio_read_32bit(2,0,&value,0);
	if(ret) {
		return -1;
	}
	printf("offset[0] = 0x%x\n",value);

	ret = vastai_pci_mmio_write_32bit(2,0,0x66bb,0);
	if(ret) {
		return -1;
	}

	ret = vastai_pci_mmio_read_32bit(2,0,&value,0);
	if(ret) {
		return -1;
	}

	if(value!=0x66bb){
		return -2;
	} else {
		printf("test_mmio_read/write pass\n");
		return 0;
	}
	
}

int test_smcu_read_write()
{
	int ret;
	unsigned int value;

	ret = vastai_read_32_bit_csr_by_smcu(0x800000,&value,0);
	if(ret) {
		return -1;
	}
	printf("0x800000 = 0x%x\n",value);

	ret = vastai_write_32_bit_csr_by_smcu(0x800000,0x66bb,0);
	if(ret) {
		return -1;
	}

	ret = vastai_read_32_bit_csr_by_smcu(0x800000,&value,0);
	if(ret) {
		return -1;
	}

	if(value!=0x66bb){
		return -2;
	} else {
		printf("test_smcu_read_write pass\n");
		return 0;
	}
}

 int test_modified_reg32()
{
	int ret;
	unsigned int value;

	ret = vastai_read_32_bit_csr_by_smcu(0x800000,&value,0);
	if(ret) {
		return -1;
	}
	printf("addr[0x800000] = 0x%x\n",value);

	ret = diag_vastai_pci_modify_reg32(0x800000,(3<<0),0,0);
	if(ret) {
		return -1;
	}

	ret = vastai_read_32_bit_csr_by_smcu(0x800000,&value,0);
	if(ret) {
		return -1;
	}

	printf("after clear bit 0 addr[0x800000] = 0x%x\n",value);


	ret = diag_vastai_pci_modify_reg32(0x800000,(1<<0),1,0);
	if(ret) {
		return -1;
	}

	ret = vastai_read_32_bit_csr_by_smcu(0x800000,&value,0);
	if(ret) {
		return -1;
	}

	printf("after set bit 0 addr[0x800000] = 0x%x\n",value);
	return 0;
}

int test_fread_fwrite_reg_32bit()
{
	int ret;
	unsigned int value;
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x1040000000;

	ret = vastai_pcie_fwrite_reg_32bit(test_addr, gold_value , 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s fwrite reg failed\n", __FUNCTION__);
		return -1;
	}

	ret = vastai_pcie_fread_reg_32bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s fread reg failed\n", __FUNCTION__);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s failed\n", __FUNCTION__);
		printf("gold_value=0x%x value=0x%x\n", gold_value, value);
		return -2;
	}else{
		printf("%s pass\n", __FUNCTION__);
		return 0;
	}
}

int test_fread_fwrite_reg_64bit()
{
	int ret;
	unsigned long long value;
	const unsigned long long gold_value = 0x01234567ABCDEF01;
	const unsigned long long test_addr = 0x1040000000;

	ret = vastai_pcie_fwrite_reg_64bit(test_addr, gold_value , 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s fwrite reg failed\n", __FUNCTION__);
		return -1;
	}

	ret = vastai_pcie_fread_reg_64bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s fread reg failed\n", __FUNCTION__);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s failed\n", __FUNCTION__);
		printf("gold_value=0x%llx value=0x%llx\n", gold_value, value);
		return -2;
	}else{
		printf("%s pass\n", __FUNCTION__);
		return 0;
	}
}

typedef int (*test_fn)(void);

test_fn test_case_table[] = {
	// write_read_32bit_base_dev0_die0,
	// write_read_64bit_base_dev0_die0,
	//write_read_buf_base_dev1_die0,
	// test_read_cfg,
	// test_modified_reg32,
	// test_mmio_read_write,
	//test_fread_fwrite_reg_32bit,
	//test_fread_fwrite_reg_64bit,
	test_smcu_read_write,
	
};

#ifdef SDMA_PERF_TEST
int vastai_sdma_test0(int fn)
{
	int *dma_test_buf0;
	int *dma_test_buf1;
	int dma_size_byte = 0x10080;
	int ret = 0;
	struct vastai_mem_info mem_info;
	struct timespec time_start = { 0, 0 };
	struct timespec time_end = { 0, 0 };
	unsigned long int ns;
	int va_pid = vastai_get_va_pid(fn);

	/* [1]alloc mem from driver */
	mem_info.size = 0x500;
	mem_info.dma_buf_fd = -1;
	mem_info.dma_phy_addr = 0;

	vastai_alloc_mem(&mem_info, fn);
	vastai_set_mem(&mem_info, 0xb7, fn);
	vastai_free_mem(&mem_info, fn);
	printf("malloc ok, phy_addr:0x%#llx\n", mem_info.dma_phy_addr);

	dma_test_buf0 = malloc(dma_size_byte);
	dma_test_buf1 = malloc(dma_size_byte);
	posix_memalign((void**)&dma_test_buf0, 128, dma_size_byte);
	posix_memalign((void**)&dma_test_buf1, 128, dma_size_byte);

	memset(dma_test_buf0, 0xC5, dma_size_byte);
	memset(dma_test_buf1, 0x81, dma_size_byte);

	/* [2]sdma copy&fill test */
	clock_gettime(CLOCK_REALTIME, &time_start);
	vastai_sdma_write_buf(0x104c000084, dma_test_buf0, dma_size_byte, fn, 0);
	vastai_sdma_read_buf(0x104c000084, dma_test_buf1, dma_size_byte, fn, 0);
	clock_gettime(CLOCK_REALTIME, &time_end);

	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
								(time_end.tv_nsec - time_start.tv_nsec);
	printf("Transfer %dk data, time_cnt:%06lu \n", dma_size_byte, ns);

	ret = memcmp(dma_test_buf0, dma_test_buf1, dma_size_byte);
	if (!ret)
			printf("sdma cp/fence test ok!\n");
	else
			printf("sdmat test failed!\n");

	memset(dma_test_buf0, 0xC6, dma_size_byte);
	memset(dma_test_buf1, 0x87, dma_size_byte);

			/* [4]trigger sdma by va test */
	clock_gettime(CLOCK_REALTIME, &time_start);
	ret = vastai_sdma_write_buf_by_sg(dma_test_buf0, 0x104c000084, dma_size_byte, fn);
	if (ret < 0) {
			printf("trigger sdma by va failed, errcode %d\n", ret);
			return -1;
	}

	ret = vastai_sdma_read_buf_by_sg(dma_test_buf1, 0x104c000084, dma_size_byte, fn);
	if (ret < 0) {
			printf("trigger sdma by va failed, errcode %d\n", ret);
			return -1;
	}
	clock_gettime(CLOCK_REALTIME, &time_end);
	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
								(time_end.tv_nsec - time_start.tv_nsec);
	printf("Transfer %dk data, time_cnt:%06lu \n", dma_size_byte, ns);

	ret = memcmp(dma_test_buf0, dma_test_buf1, dma_size_byte);
	if (!ret)
			printf("sdma sg test ok!\n");
	else
			printf("sdmat sg test failed!\n");

	memset(dma_test_buf0, 0xC7, dma_size_byte);
	memset(dma_test_buf1, 0x89, dma_size_byte);

	/* [4]trigger sdma by va test */
	clock_gettime(CLOCK_REALTIME, &time_start);
	ret = vastai_sdma_write_buf_by_user_addr(dma_test_buf0, 0x104c000084, dma_size_byte, fn, va_pid);
	if (ret < 0) {
			printf("trigger sdma by va failed, errcode %d\n", ret);
			return -1;
	}

	ret = vastai_sdma_read_buf_by_user_addr(dma_test_buf1, 0x104c000084, dma_size_byte, fn, va_pid);
	if (ret < 0) {
			printf("trigger sdma by va failed, errcode %d\n", ret);
			return -1;
	}
	clock_gettime(CLOCK_REALTIME, &time_end);
	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
								(time_end.tv_nsec - time_start.tv_nsec);
	printf("Transfer %dk data, time_cnt:%06lu \n", dma_size_byte, ns);

	ret = memcmp(dma_test_buf0, dma_test_buf1, dma_size_byte);
	if (!ret)
			printf("sdma va test ok!\n");
	else
			printf("sdmat va test failed!\n");

	free(dma_test_buf0);
	free(dma_test_buf1);

	vastai_free_va_pid(va_pid, fn);
	return 0;
}
#endif

int vastai_sdma_test(int fn)
{
	int *dma_test_buf0;
	int *dma_test_buf1;
	int dma_size_byte = 0xa00080;
	int file_size;
	int ret = 0;
	unsigned long long data = 0xC6C6C6C6C6C6C6C6;
	struct vastai_mem_info mem_info;

	/* [1]alloc mem from driver */
	mem_info.size = 0x500;
	mem_info.dma_buf_fd = -1;
	mem_info.dma_phy_addr = 0;

	vastai_alloc_mem(&mem_info, fn);
	vastai_set_mem(&mem_info, 0xb7, fn);
	vastai_free_mem(&mem_info, fn);
	printf("malloc ok, phy_addr:0x%#llx\n", mem_info.dma_phy_addr);
	
	/* [2]sdma copy&fill test */
	dma_test_buf0 = malloc(dma_size_byte);
	memset(dma_test_buf0, 0xC6, dma_size_byte);
	vastai_sdma_write_buf(0x1040000084, dma_test_buf0, dma_size_byte, fn, 0);

	vastai_sdma_fill_data_2_ddr(0x104800001C, data, dma_size_byte, fn, 0);

	vastai_sdma_d2d_transfer(0x104800001C, 0x104c000080, dma_size_byte, 0, fn);
	
	dma_test_buf1 = malloc(dma_size_byte);
	memset(dma_test_buf1, 0x87, dma_size_byte);
	vastai_sdma_read_buf(0x104c000080, dma_test_buf1, dma_size_byte, fn, 0);
	ret = memcmp(dma_test_buf0, dma_test_buf1, dma_size_byte);
	if (!ret)
		printf("sdma cp/fence test ok!\n");
	else
		printf("sdmat test failed!\n");

	/* [3]move file to dev ddr by sdma */
	vastai_sdma_write_from_file("./test1.bin", 0, 0x104700001c, fn, 0);

	file_size = vastai_get_file_size("./test1.bin");
	vastai_sdma_read_to_file("./test2.bin", 0, 0x104700001c, file_size, fn, 0);
	ret = compare_file("./test1.bin", "./test2.bin", 0, file_size);
	if (ret < 0) {
		printf("ERROR: sdma_test read data file failed, errcode %d\n",
		       ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: Different Match Compare.\n");
		return -2;
	}
#if 0
	/* [4]trigger sdma by va test */
	ret = vastai_sdma_write_buf_by_user_addr(dma_test_buf0, 0x104a000000, dma_size_byte, fn);
	if (ret < 0) {
		printf("trigger sdma by va failed, errcode %d\n", ret);
		return -1;
	}

	ret = vastai_sdma_read_buf_by_user_addr(dma_test_buf1, 0x104a000000, dma_size_byte, fn);
	if (ret < 0) {
		printf("trigger sdma by va failed, errcode %d\n", ret);
		return -1;
	}

	ret = memcmp(dma_test_buf0, dma_test_buf1, dma_size_byte);
	if (!ret)
		printf("sdma va test ok!\n");
	else
		printf("sdmat va test failed!\n");

#endif
	free(dma_test_buf0);
	free(dma_test_buf1);
	return 0;
}

int vastai_sdma_performance_test(int dir, int fn, int len, int loop, int va_en)
{
	int *dma_test_buf0;
	struct timespec time_start = { 0, 0 };
	struct timespec time_end = { 0, 0 };
	unsigned long int ns, min_time = 0xFFFFFFFF, max_time = 0, total_time = 0;
	int loop_cnt = loop;
	int va_pid = vastai_get_va_pid(fn);

	dma_test_buf0 = malloc(len);
	memset(dma_test_buf0, 0xC6, len);

	printf("Start to performance test: fn[%d] dir[%d] len=[0x%x] loop[%d] va_en[%d]\n", fn, dir, len, loop, va_en);
	while(loop_cnt)
	{
		clock_gettime(CLOCK_REALTIME, &time_start);

		if (!va_en)
		{
		    if (dir == 0)
			vastai_sdma_write_buf(0x1040000000, dma_test_buf0, len, fn, 0);
		    else if (dir == 1)
		 	vastai_sdma_read_buf(0x104c000000, dma_test_buf0, len, fn, 0);
		}
		else
		{
		    if (dir == 0)
                        vastai_sdma_write_buf_by_user_addr(dma_test_buf0, 0x1040000000, len, fn, va_pid);
                    else if (dir == 1)
			vastai_sdma_read_buf_by_user_addr(dma_test_buf0, 0x1040000000, len, fn, va_pid);
		}
		clock_gettime(CLOCK_REALTIME, &time_end);

		ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
				 (time_end.tv_nsec - time_start.tv_nsec);
		if (ns < min_time)
			min_time = ns;
		if (ns > max_time)
			max_time = ns;

		total_time += ns;
		loop_cnt--;

	}

	printf("Transfer %dk data, max time:%06lu, min time:%06lu, avr time:%06lu\n", len/1024, max_time, min_time, total_time/loop);
	free(dma_test_buf0);
	vastai_free_va_pid(va_pid, fn);
	return 0;
}

void mmio_write_read_perf_test(void)
{
        int len = 0x1000;
        //int loop = 100;
        int loop = 100;
        void *src = malloc(len);
        void *dst = malloc(len);
        //int func_num = 0;
        clock_t start, finish;
        double Total_time;
        cpu_set_t cpuset;
        struct vastai_mem_info mem_info;

        /* [1]alloc mem from driver */
        mem_info.size = len;
        mem_info.dma_buf_fd = -1;
        mem_info.dma_phy_addr = 0;

        vastai_alloc_mem(&mem_info, 0);
        printf("alloc mem fd = %d phy_addr = 0x%llx\n", mem_info.dma_buf_fd, mem_info.dma_phy_addr);
        CPU_ZERO(&cpuset);

        //func_num = atoi(argv[1]);
        CPU_SET(0, &cpuset);

        if(sched_setaffinity( 0, sizeof(cpuset), &cpuset) == -1) {
                printf( "Set CPU affinity failed, error\n");
        }
        start = clock();
        memset(src, 0x5a5a5a5a, len);
        memset(dst, 0x6c6c6c6c, len);
        while(loop--)
        {
            vastai_pci_mmio_write_buf_perf(&mem_info, 0x1040000000, src, len, 0, 0);
            vastai_pci_mmio_read_buf_perf(&mem_info, 0x1040000000, dst, len, 0, 0);
//          printf("dst buf: 0x%llx\n", *(unsigned long long *)dst);
        }
        free(src);
        free(dst);
        close(mem_info.dma_buf_fd);
        finish = clock();
        Total_time = (double)(finish - start) / CLOCKS_PER_SEC; //单位换算成秒
        printf("%f seconds\n", Total_time);
}

int main(int argc, char *argv[])
{
	int ret = 0;
	int fn, len, loop, dir, va_en;

	if (1 == argc)
		fn = 0;
	else if (argc == 2)
		fn = atoi(argv[1]);
	else if (argc == 6)
	{
		fn = atoi(argv[1]);
		dir = atoi(argv[2]);
		len = atoi(argv[3]);
		loop = atoi(argv[4]);
		va_en = atoi(argv[5]);
		vastai_sdma_performance_test(dir, fn, len, loop, va_en);
		return 0;
	}

	printf("sdma_test fuction number:%d\n", fn);
	ret = vastai_sdma_test(fn);
	if (ret){
		printf("sdma_test failed\n");
	}

#ifdef SDMA_PERF_TEST
	ret = vastai_sdma_test0(fn);
	if (ret){
		printf("sdma_test failed\n");
	}
#endif

	for (int i = 0; i < (int)ARRAY_SIZE(test_case_table); i++) {
		ret = test_case_table[i]();
		if (ret) {
			printf("test failed case_id[%d]\n", i);
			break;
		} else {
			printf("test success case_id[%d]\n", i);
		}
	}
	if (!ret) {
		printf("test success!\n");
	}

//    mmio_write_read_perf_test();
//    vastai_do_reset(0, 0, 0);

	return ret;
}
