#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <time.h>
#define _GUN_SOURCE
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "vastai_pcie_user_api.h"
#include "diag_pcie_user_api.h"

struct task {
	int task_id;
	int function_number;
	int direction;
	int length;
	int task_num;
	unsigned long long
		start_address; /* end address = buf_num * 4MByte + start_address */
	unsigned int count;
};

struct task task_config = {
	.function_number = 0,
	.direction = 0,
	.task_num = 10,
	.start_address = 0x104C000000,
};

//#define test_buf_len 0x400000
#define H2D_TEST 0
#define D2H_TEST 1
#define D2D_TEST 2
#define VASTAI_PCI_IOCTL_DMA_START	 1
#define PCIE_WRAPPER_BASE                               0x940000
#define PWRAP_PCIE_MST_MONITOR_PARA             (PCIE_WRAPPER_BASE + 0 * 4)
#define PWRAP_PCIE_MST_MONITOR_TIME1            (PCIE_WRAPPER_BASE + 1 * 4)
#define PWRAP_PCIE_MST_MONITOR_TIME2            (PCIE_WRAPPER_BASE + 2 * 4)
#define PWRAP_PCIE_MST_MONITOR_TIME3            (PCIE_WRAPPER_BASE + 3 * 4)
#define PWRAP_PCIE_MST_MONITOR_TIME4            (PCIE_WRAPPER_BASE + 4 * 4)
#define PWRAP_PCIE_MST_MONITOR_BANDWIDTH_SET    (PCIE_WRAPPER_BASE + 5 * 4)
#define PWRAP_PCIE_MST_MONITOR_OPERAND_SET      (PCIE_WRAPPER_BASE + 6 * 4)
#define PWRAP_PCIE_MST_MONITOR_LATENCY_WID      (PCIE_WRAPPER_BASE + 7 * 4)
#define PWRAP_PCIE_MST_MONITOR_LATENCY_RID      (PCIE_WRAPPER_BASE + 8 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_WRRANG0   (PCIE_WRAPPER_BASE + 9 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_WRRANG1   (PCIE_WRAPPER_BASE + 10 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_WRRANG2   (PCIE_WRAPPER_BASE + 11 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_WRRANG3   (PCIE_WRAPPER_BASE + 12 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_RDRANG0   (PCIE_WRAPPER_BASE + 13 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_RDRANG1   (PCIE_WRAPPER_BASE + 14 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_RDRANG2   (PCIE_WRAPPER_BASE + 15 * 4)
#define PWRAP_PCIE_MST_MONITOR_SPADDR_RDRANG3   (PCIE_WRAPPER_BASE + 16 * 4)
#define PWRAP_PCIE_MST_MONITOR_RESULT_SELSCT    (PCIE_WRAPPER_BASE + 17 * 4)
#define PWRAP_PCIE_MST_MONITOR_RESULT0          (PCIE_WRAPPER_BASE + 18 * 4)
#define PWRAP_PCIE_MST_MONITOR_RESULT1          (PCIE_WRAPPER_BASE + 19 * 4)
#define PWRAP_PCIE_SLV_MONITOR_PARA             (PCIE_WRAPPER_BASE + 20 * 4)
#define PWRAP_PCIE_SLV_MONITOR_TIME1            (PCIE_WRAPPER_BASE + 21 * 4)
#define PWRAP_PCIE_SLV_MONITOR_TIME2            (PCIE_WRAPPER_BASE + 22 * 4)
#define PWRAP_PCIE_SLV_MONITOR_TIME3            (PCIE_WRAPPER_BASE + 23 * 4)
#define PWRAP_PCIE_SLV_MONITOR_TIME4            (PCIE_WRAPPER_BASE + 24 * 4)
#define PWRAP_PCIE_SLV_MONITOR_BANDWIDTH_SET    (PCIE_WRAPPER_BASE + 25 * 4)
#define PWRAP_PCIE_SLV_MONITOR_OPERAND_SET      (PCIE_WRAPPER_BASE + 26 * 4)
#define PWRAP_PCIE_SLV_MONITOR_LATENCY_WID      (PCIE_WRAPPER_BASE + 27 * 4)
#define PWRAP_PCIE_SLV_MONITOR_LATENCY_RID      (PCIE_WRAPPER_BASE + 28 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_WRRANG0   (PCIE_WRAPPER_BASE + 29 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_WRRANG1   (PCIE_WRAPPER_BASE + 30 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_WRRANG2   (PCIE_WRAPPER_BASE + 31 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_WRRANG3   (PCIE_WRAPPER_BASE + 32 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_RDRANG0   (PCIE_WRAPPER_BASE + 33 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_RDRANG1   (PCIE_WRAPPER_BASE + 34 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_RDRANG2   (PCIE_WRAPPER_BASE + 35 * 4)
#define PWRAP_PCIE_SLV_MONITOR_SPADDR_RDRANG3   (PCIE_WRAPPER_BASE + 36 * 4)
#define PWRAP_PCIE_SLV_MONITOR_RESULT_SELSCT    (PCIE_WRAPPER_BASE + 37 * 4)
#define PWRAP_PCIE_SLV_MONITOR_RESULT0          (PCIE_WRAPPER_BASE + 38 * 4)
#define PWRAP_PCIE_SLV_MONITOR_RESULT1          (PCIE_WRAPPER_BASE + 39 * 4)

#define SDMA0_REG_BASE                           	0xAA0000

#define REG_SDMA_MONITOR0_PARA                     (SDMA0_REG_BASE + 260 * 4)
#define REG_SDMA_MONITOR0_TIME1                    (SDMA0_REG_BASE + 261 * 4)
#define REG_SDMA_MONITOR0_TIME2                    (SDMA0_REG_BASE + 262 * 4)
#define REG_SDMA_MONITOR0_TIME3                    (SDMA0_REG_BASE + 263 * 4)
#define REG_SDMA_MONITOR0_TIME4                    (SDMA0_REG_BASE + 264 * 4)
#define REG_SDMA_MONITOR0_BANDWIDTH_SET            (SDMA0_REG_BASE + 265 * 4)
#define REG_SDMA_MONITOR0_OPERAND_SET              (SDMA0_REG_BASE + 266 * 4)
#define REG_SDMA_MONITOR0_LATENCY_WID              (SDMA0_REG_BASE + 267 * 4)
#define REG_SDMA_MONITOR0_LATENCY_RID              (SDMA0_REG_BASE + 268 * 4)
#define REG_SDMA_MONITOR0_SPADDR_WRRANGE0          (SDMA0_REG_BASE + 269 * 4)
#define REG_SDMA_MONITOR0_SPADDR_WRRANGE1          (SDMA0_REG_BASE + 270 * 4)
#define REG_SDMA_MONITOR0_SPADDR_WRRANGE2          (SDMA0_REG_BASE + 271 * 4)
#define REG_SDMA_MONITOR0_SPADDR_WRRANGE3          (SDMA0_REG_BASE + 272 * 4)
#define REG_SDMA_MONITOR0_SPADDR_RDRANGE0          (SDMA0_REG_BASE + 273 * 4)
#define REG_SDMA_MONITOR0_SPADDR_RDRANGE1          (SDMA0_REG_BASE + 274 * 4)
#define REG_SDMA_MONITOR0_SPADDR_RDRANGE2          (SDMA0_REG_BASE + 275 * 4)
#define REG_SDMA_MONITOR0_SPADDR_RDRANGE3          (SDMA0_REG_BASE + 276 * 4)
#define REG_SDMA_MONITOR0_RESULT_SELSCT            (SDMA0_REG_BASE + 277 * 4)
#define REG_SDMA_MONITOR0_RESULT0                  (SDMA0_REG_BASE + 278 * 4)
#define REG_SDMA_MONITOR0_RESULT1                  (SDMA0_REG_BASE + 279 * 4)

typedef unsigned int u32;
typedef unsigned long long int u64;

static int open_pcie_ioctl(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/va%d_ctl", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output:  open /dev/kchar: dev_id failed!:XXXX(error)\n
	}
	return filep;
}

static int vastai_trigger_sdma0(int dma_buf, u64 axi_addr, u32 length,
			      u32 is_dev_to_host, u32 fn)
{
	int ret = 0;
	int filep = open_pcie_ioctl(fn);

	struct kchar_cmd kcmd = { .dma_start_cmd.is_dev_to_host = is_dev_to_host,
				  .dma_start_cmd.dma_buf_fd = dma_buf,
				  .dma_start_cmd.axi_addr = axi_addr,
				  .dma_start_cmd.size = length,
				  .dma_start_cmd.die_index = 0 };
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}
	while(1){
		ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_START, &kcmd);
		if (ret < 0) {
			printf("ioctl is fail %d\n", ret);
			close(filep);
			return ret;
		}
	}
	close(filep);
	return ret;
}

static void vastai_trigger_sdma1(int fn)
{
	u32 irq;

	while(1)
	{
		vastai_write_32_bit_csr_by_smcu(0xaa8000, 1, fn);
		while(1)
		{
			vastai_read_32_bit_csr_by_smcu(0xaa846c, &irq, fn);
			if(irq!=0)
			{
				vastai_write_32_bit_csr_by_smcu(0xaa846c, irq, fn);
				while(1)
				{
					vastai_read_32_bit_csr_by_smcu(0xaa846c, &irq, fn);
					if (irq==0)
						break;
				}
			}
			break;
		}
	}
}

int sub_process_sdma1(struct task *self)
{
	struct vastai_mem_info mem_info;
	int i;
	int phy_low, phy_high;

	/* [1]alloc mem from driver */
	mem_info.size = self->length;
	mem_info.dma_buf_fd = -1;
	mem_info.dma_phy_addr = 0;
	vastai_alloc_mem(&mem_info, self->function_number);
	vastai_set_mem(&mem_info, 0xb7, self->function_number);

	phy_low = mem_info.dma_phy_addr & 0xFFFFFFFF;
	phy_high = (mem_info.dma_phy_addr >> 32) & 0xFFFFFFFF;

	for (i = 0; i < 0x40; i+=4)
		vastai_write_32_bit_csr_by_smcu(0xaa8470 + i, 0xFFFFFFFF, self->function_number);

	vastai_write_32_bit_csr_by_smcu(0x820804, 0x1d30000, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0x82080c, self->length, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0x820810, phy_low, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0x820814, phy_high, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0x820818, 0x40000000, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0x82081C, 0x10, self->function_number);

	vastai_write_32_bit_csr_by_smcu(0xaa8004, 0x820800, self->function_number);
	vastai_write_32_bit_csr_by_smcu(0xaa801c, 0x10000, self->function_number);

	vastai_trigger_sdma1(self->function_number);
	vastai_free_mem(&mem_info, self->function_number);

	return 0;
}

int sub_process_kernel(struct task *self, int i)
{
	struct vastai_mem_info mem_info;
	int ret;

	/* [1]alloc mem from driver */
	mem_info.size = self->length;
	mem_info.dma_buf_fd = -1;
	mem_info.dma_phy_addr = 0;
	vastai_alloc_mem(&mem_info, self->function_number);
	vastai_set_mem(&mem_info, 0xb7, self->function_number);

	ret = vastai_trigger_sdma0(mem_info.dma_buf_fd,
				 self->start_address + (mem_info.size * i),
				 mem_info.size, self->direction,
				 self->function_number);
	if (ret < 0) {
		printf("trigger dma is failed %d\n", ret);
		return -1;
	}
	vastai_free_mem(&mem_info, self->function_number);

	return 0;
}

int sub_process_va(struct task *self, int i)
{
        int ret;
	int va_pid = vastai_get_va_pid(self->function_number);

	while(1){	
	int *dma_test_buf;
		dma_test_buf = malloc(self->length);
	posix_memalign((void**)&dma_test_buf, 128, self->length);
	memset(dma_test_buf, 0xC5, self->length);

	switch(self->direction)
        {
        	case H2D_TEST:
                {
	                ret = vastai_sdma_write_buf_by_user_addr(dma_test_buf, self->start_address + (i * self->length), self->length, self->function_number, va_pid);
                        break;
                }

                case D2H_TEST:
                {
                        ret = vastai_sdma_read_buf_by_user_addr(dma_test_buf, self->start_address + (i * self->length), self->length, self->function_number, va_pid);
                        break;
                }
	
		default:
                {
                        printf("Invalide direction\n");
                                break;
                }

	}
	free(dma_test_buf);
	}
	vastai_free_va_pid(va_pid, self->function_number);
	return ret;
}

int sub_process_sg(struct task *self, int i)
{
	int *dma_test_buf;
        unsigned long long data = 0xC6C6C6C6C6C6C6C6;

	vastai_sdma_fill_data_2_ddr(self->start_address, data, self->length, self->function_number, 0);
	while(1){
		int ret;
		dma_test_buf = malloc(self->length);
        posix_memalign((void**)&dma_test_buf, 128, self->length);
        memset(dma_test_buf, 0xC5, self->length);

        switch(self->direction)
        {
                case H2D_TEST:
                {
                        ret = vastai_sdma_write_buf_by_sg(dma_test_buf, self->start_address + (i * self->length), self->length, self->function_number);
                        break;
                }

                case D2H_TEST:
                {
                        ret = vastai_sdma_read_buf_by_sg(dma_test_buf, self->start_address + (i * self->length),  self->length, self->function_number);
                        break;
                }

                default:
                {
                        printf("Invalide direction\n");
                                break;
                }

        }
	if (ret < 0) {
                        printf("trigger dma is failed %d\n", ret);
                        free(dma_test_buf);
                        return -1;
                }

	free(dma_test_buf);
	}
        return 0;
}


int sub_process_user(struct task *self, int i)
{
	int *dma_test_buf;

	//vastai_sdma_fill_data_2_ddr(self->start_address, data, self->length, self->function_number, 0);
	while (1) {
		int ret=0;
		dma_test_buf = malloc(self->length);
       		posix_memalign((void**)&dma_test_buf, 128, self->length);
		memset(dma_test_buf, 0xC6, self->length);
		switch(self->direction)
		{
			case H2D_TEST:
			{
				ret = vastai_sdma_write_buf(self->start_address + (i * self->length), dma_test_buf, self->length, self->function_number, 0);
				break;
			}

			case D2H_TEST:
			{
				ret = vastai_sdma_read_buf(self->start_address + (i * self->length), dma_test_buf, self->length, self->function_number, 0);
				break;
			}

			case D2D_TEST:
			{
				ret = vastai_sdma_d2d_transfer(self->start_address, self->start_address + (i * self->length), self->length, 0, self->function_number);
				break;
			}

			default:
			{
				printf("Invalide direction\n");
				break;
			}
		}

		if (ret < 0) {
			printf("trigger dma is failed %d %d\n", ret, i);
			free(dma_test_buf);
			return -1;
		}
		free(dma_test_buf);
	}

	return 0;
}

void sdma_enable_axi_monitor(u32 dirction, int fn)
{
	if (dirction > D2D_TEST)
		return;

    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME1, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME2, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME3, 0xFFFFFFFF, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME4, 0xFFFFFFFF, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_BANDWIDTH_SET, 0x1016, fn);
	vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_OPERAND_SET, 0x1016, fn);
	
    if (H2D_TEST == dirction)
        vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_RESULT_SELSCT, 0x1, fn);
    else
        vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_RESULT_SELSCT, 0x0, fn);

    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_PARA, 0x1F, fn);
	return;
}

u64 sdma_get_axi_bandwidth(int fn, int dir)
{
	u64 payload;
	u32 result_h, result_l;
	
    vastai_read_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_RESULT0, &result_h, fn);
    vastai_read_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_RESULT1, &result_l, fn);
	payload = ((u64)result_h << 32) | result_l;

	if (H2D_TEST == dir)
		payload *= 32;

	return payload;
}

void sdma_disable_axi_monitor(int fn)
{
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME1, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME2, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME3, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_TIME4, 0, fn);
    vastai_write_32_bit_csr_by_smcu(REG_SDMA_MONITOR0_PARA,  0, fn);

	return;
}

void pcie_enable_axi_monitor(u32 dirction, int fn)
{
	if (dirction > D2H_TEST)
		return;

    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME1, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME2, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME3, 0xFFFFFFFF, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME4, 0xFFFFFFFF, fn);
    //vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_BANDWIDTH_SET, 0x8868, fn);
	vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_OPERAND_SET, 0+(5<<1)+(0<<9)+(5<<10), fn);

    if (H2D_TEST == dirction)
        vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_RESULT_SELSCT, 0x1, fn);
    else
        vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_RESULT_SELSCT, 0x0, fn);

    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_PARA, 0x1F, fn);

	return;
}

void pcie_disable_axi_monitor(int fn)
{
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME1, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME2, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME3, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_TIME4, 0, fn);
    vastai_write_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_PARA,  0, fn);

	return;
}

u64 pcie_get_axi_bandwidth(int fn, int dir)
{
	u64 payload;
	u32 result_h, result_l;
	
    vastai_read_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_RESULT0, &result_h, fn);
    vastai_read_32_bit_csr_by_smcu(PWRAP_PCIE_SLV_MONITOR_RESULT1, &result_l, fn);
	payload = ((u64)result_h << 32) | result_l;

	if (H2D_TEST == dir)
		payload *= 32;

	return payload;
}

double get_pcie_perf(int fn, int dir)
{
	unsigned long long int start_count = 0;
	unsigned long long int end_count = 0;
	unsigned long long int count;
	struct timespec time_start = { 0, 0 };
	struct timespec time_end = { 0, 0 };
	unsigned long long int ns;
	double perf;

SIMPLE_AGAIN:
	clock_gettime(CLOCK_REALTIME, &time_start);
	start_count = pcie_get_axi_bandwidth(fn, dir);

	usleep(1000000);

	end_count = pcie_get_axi_bandwidth(fn, dir);

	clock_gettime(CLOCK_REALTIME, &time_end);
	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
	     (time_end.tv_nsec - time_start.tv_nsec);

	if (end_count < start_count)
		goto SIMPLE_AGAIN;
	count = end_count - start_count;

	perf = (count/(1024 * 1024 * 1024)) * (1000000000.0 / ns);
	printf("[PCIE]%.04fGB/s 0x%08llxB 0x%08llx\n", perf, count, ns);

	return perf;
}

double get_sdma_perf(int fn, int dir)
{
	unsigned long long int start_count = 0;
	unsigned long long int end_count = 0;
	unsigned long long int count;
	struct timespec time_start = { 0, 0 };
	struct timespec time_end = { 0, 0 };
	unsigned long long int ns;
	double perf;

SIMPLE_AGAIN:
	clock_gettime(CLOCK_REALTIME, &time_start);
	start_count = sdma_get_axi_bandwidth(fn, dir);

	usleep(1000000);

	end_count = sdma_get_axi_bandwidth(fn, dir);

	clock_gettime(CLOCK_REALTIME, &time_end);
	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 +
	     (time_end.tv_nsec - time_start.tv_nsec);

	if (end_count < start_count)
		goto SIMPLE_AGAIN;
	count = end_count - start_count;

	perf = (count/(1024 * 1024 * 1024)) * (1000000000.0 / ns);
	printf("[SDMA]%.04fGB/s 0x%08llxB 0x%08llx\n", perf, count, ns);

	return perf;
}


int main(int argc, char *argv[])
{
	int fn, len, dir, mode;

	if ((argc != 2) && (argc != 5)) {
		printf("usage:\n");
		printf("./sdma_performance <dev_id> <dirction>(0: host2dev 1: dev2host 2: dev2dev) <length>\n");
		return 0;
	}

	if (argc == 2)
	{
		fn = atoi(argv[1]);
			
		/* disable pcie axi monitor */
		pcie_disable_axi_monitor(fn);
		/* disable sdma axi monitor */
		sdma_disable_axi_monitor(fn);
		return 0;
	}

	if (argc == 5)
	{
		fn = atoi(argv[1]);
		mode = atoi(argv[2]);
		dir = atoi(argv[3]);
		len = atoi(argv[4]);
	}

	task_config.function_number = fn;
	task_config.direction = dir;
	if (len) {
		task_config.length = len;
	} else {
		task_config.length = 0x400000;
	}

	printf("perf test begin: fn[%d], mode[%d], dirction[%d], length[0x%x]\n", task_config.function_number, mode, task_config.direction, task_config.length);

	/* enable pcie axi monitor */
	pcie_enable_axi_monitor(task_config.direction, task_config.function_number);
	sdma_enable_axi_monitor(task_config.direction, task_config.function_number);

	for (int i = 0; i < task_config.task_num; i++) {
		pid_t pid = fork();
		if (pid == 0) {
			switch(mode)
			{
				case 0:sub_process_user(&(task_config), i);break;
				case 1:sub_process_sg(&(task_config), i);break;
				case 2:sub_process_va(&(task_config), i);break;
				case 3:sub_process_kernel(&(task_config), i);break;
				default:sub_process_user(&(task_config), i);break;
			}
			return 0;
		}
	}

	while (1) {
		double pcie_perf = 0;
		double sdma_perf = 0;
		int loop = 1;
		for (int i = 0; i < loop; i++) {
			if (D2D_TEST != task_config.direction)
				pcie_perf += get_pcie_perf(fn, dir);
			sdma_perf += get_sdma_perf(fn, dir);
		}
		if (loop > 1)
		{
			if (D2D_TEST != task_config.direction)
				printf("[PCIE]%.04fMB\n", pcie_perf / loop);
			printf("[SDMA]%.04fMB\n", sdma_perf / loop);
		}
	}

	return 0;
}
