#ifndef __DIAG_PCIE_USER_API_H__
#define __DIAG_PCIE_USER_API_H__
#pragma pack(1)

#define SPACE_SIZE_256M         0x10000000ULL       /* 256M */
#define SPACE_SIZE_64G          0x1000000000ULL     /*64G*/

//diag test mode
int open_pcie_kchar(int dev_id);
/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_write_reg_32bit - write one register, width is 32bit.
 * Synopsis:
 * 	int diag_vastai_pcie_write_reg_32bit(unsigned long long address, unsigned int value, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	unsigned int value
 * 		register value, 32bit
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_write_reg_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_write_reg_64bit - write one register, width is 64bit.
 * Synopsis:
 * 	int diag_vastai_pcie_write_reg_64bit(unsigned long long address, unsigned long long value, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	unsigned long long value
 * 		register value, 64bit
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_write_reg_64bit(unsigned long long address,
				unsigned long long value, int dev_id,
				int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_read_reg_32bit - read one register, width is 32bit.
 * Synopsis:
 * 	int diag_vastai_pcie_read_reg_32bit(unsigned long long address, unsigned int *value, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	unsigned int *value
 * 		return value, register value, 32bit, must __NOT__ NULL point
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_read_reg_32bit(unsigned long long address, unsigned int *value,
			       int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_read_reg_64bit - read one register, width is 64bit.
 * Synopsis:
 * 	int diag_vastai_pcie_read_reg_64bit(unsigned long long address, unsigned long long *value, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	unsigned int *value
 * 		return value, register value, 64bit, must __NOT__ NULL point
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_read_reg_64bit(unsigned long long address,
			       unsigned long long *value, int dev_id,
			       int die_id);
/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_write_reg_buf - write range buf into register
 * Synopsis:
 * 	int vastai_pcie_write_reg_buf(unsigned long long address, const void *buf, int buf_size_byte, int dev_id, int
 *die_id); Arguments: unsigned long long address register address, 64bit, have to 4Byte align const void *buf range buf
 *for write int buf_size_byte size of range buf, have to 4Byte align int dev_id select device id int die_id select die
 *id Return Value: 0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_write_reg_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_read_reg_buf - read register, fill into range buf
 * Synopsis:
 * 	int diag_vastai_pcie_read_reg_buf(unsigned long long address, void *buf, int buf_size_byte, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	void *buf
 * 		range buf for read, must __NOT__ NULL point
 * 	int buf_size_byte
 * 		size of range buf, have to 4Byte align
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_read_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_write_reg_file - write binary file into register
 * Synopsis:
 * 	int diag_vastai_pcie_write_reg_file(const char *file_path, int offset, unsigned long long address, int len_byte, int
 *dev_id, int die_id); Arguments: const char *file_path binary file's path, string. int offset offset with binary
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	int len_byte
 * 		size of file, have to 4Byte align
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_write_reg_file(const char *file_path, int offset,
			       unsigned long long address, int len_byte,
			       int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_write_buf_unalign - write range buf into ddr
 * Synopsis:
 * 	int diag_vastai_pcie_write_buf_unalign (unsigned long long address, const void *buf, int buf_size_byte, int dev_id, int
 *die_id); Arguments: unsigned long long address register address, 64bit, have to 4Byte align const void *buf range buf
 *for write int buf_size_byte size of range buf,  int dev_id select device id int die_id select die
 *id Return Value: 0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_write_buf_unalign(unsigned long long address, const void *buf,
				  int buf_size_byte, int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_read_buf_unalign - read ddr or csram, fill into range buf
 * Synopsis:
 * 	int diag_vastai_pcie_read_buf_unalign(unsigned long long address, void *buf, int buf_size_byte, int dev_id, int die_id);
 * Arguments:
 * 	unsigned long long address
 * 		register address, 64bit,
 * 	void *buf
 * 		range buf for read, must __NOT__ NULL point
 * 	int buf_size_byte
 * 		size of range buf
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_read_buf_unalign(unsigned long long address, void *buf,
				 int buf_size_byte, int dev_id, int die_id);

/*****************************************
 * Name-Description
 * 	diag_vastai_pcie_read_reg_file - read register, fill into binary file
 * Synopsis:
 * 	int diag_vastai_pcie_read_reg_file(const char *file_path, int offset, unsigned long long address, int len_byte, int
 *dev_id, int die_id); Arguments: const char *file_path binary file's path, string. int offset offset with binary
 * 	unsigned long long address
 * 		register address, 64bit, have to 4Byte align
 * 	int len_byte
 * 		size of file, have to 4Byte align
 * 	int dev_id
 * 		select device id
 * 	int die_id
 * 		select die id
 * Return Value:
 * 	0 - success. other value is failed and errcode.
 *****************************************/
int diag_vastai_pcie_read_reg_file(const char *file_path, int offset,
			      unsigned long long address, int len_byte,
			      int dev_id, int die_id);
//read config 32 bit 
int vastai_pcie_read_cfg_32bit(unsigned long long cfg_offset, unsigned int *value,
			       int dev_id, int die_id);
//write config 32 bit
int vastai_pcie_write_cfg_32bit(unsigned long long cfg_offset,
				unsigned long long value, int dev_id,int die_id);

int vastai_pci_mmio_read_32bit(int bar_num, unsigned long long offset, 
				unsigned int *value, int dev_id);

int vastai_pci_mmio_write_32bit(int bar_num,unsigned long long offset, 
				unsigned long long value, int dev_id);

int vastai_write_32_bit_csr_by_smcu(unsigned long long address,
				unsigned long long value, int dev_id);

int diag_vastai_smcu_write_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id);

int diag_vastai_smcu_write_64bit(unsigned long long address,
				unsigned long long value, int dev_id,
				int die_id);

int diag_vastai_smcu_write_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id);

int vastai_smcu_write_buf_unalign(unsigned long long address, const void *buf,
				  int buf_size_byte, int dev_id, int die_id);

int vastai_read_32_bit_csr_by_smcu(unsigned long long addr, unsigned int *value, int dev_id);

int diag_vastai_smcu_read_32bit(unsigned long long address, unsigned int *value,
			       int dev_id, int die_id);

int diag_vastai_smcu_read_64bit(unsigned long long address,
			       unsigned long long *value, int dev_id,
			       int die_id);

int vastai_smcu_read_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id);

int diag_vastai_smcu_read_reg_file(const char *file_path, int offset,
			      unsigned long long address, int len_byte,
			      int dev_id, int die_id);

int diag_vastai_pci_modify_reg32(unsigned long long address,unsigned int mask_bit,
								unsigned int mode,int dev_id);

int diag_vastai_smcu_modify_reg32(unsigned long long address,unsigned int mask_bit,
								unsigned int mode,int dev_id);

#pragma pack()
#endif