#ifndef __VASTAI_PCIE_DUMP_API_H__
#define __VASTAI_PCIE_DUMP_API_H__

/*
 * Description:
 *      This api can be used to read noc information.
 * Parameter:
 *      card_id : card id.
 *      address : noc address.
 *      value   : return read value.
 * return value:
 *      0       : OK
 *      -1      : read failed.
 */
int vastai_dump_read(uint32_t card_id, uint64_t address, uint32_t *value);

/*
 * Description:
 *      This api can be used to write noc.
 * Parameter:
 *      card_id : card id.
 *      address : noc address.
 *      value   : the value need to write.
 * return value:
 *      0       : OK
 *      -1      : write failed.
 */
int vastai_dump_write(uint32_t card_id, uint64_t address, uint32_t value);

/*
 * Description:
 *      This api can be used to read noc information.
 * Parameter:
 *      card_id : card id.
 *      address : noc address.
 *      len     : the num of Byte want to read.
 *      buf     : return read data.
 * return value:
 *      0       : OK
 *      -1      : read failed.
 */
int vastai_dump_read_buf(uint32_t card_id, uint64_t address, uint32_t len, uint8_t *buf);

/*
 * Description:
 *      This api can be used to write noc.
 * Parameter:
 *      card_id : card id.
 *      address : noc address.
 *      len     : the num of Byte want to write.
 *      buf     : the data want to write.
 * return value:
 *      0       : OK
 *      -1      : write failed.
 */
int vastai_dump_write_buf(uint32_t card_id, uint64_t address, uint32_t len, uint8_t *buf);

/*
 * Description:
 *      This api can be used to read noc information and save to file.
 * Parameter:
 *      card_id     : card id.
 *      address     : noc address.
 *      len         : the num of Byte want to read.
 *      file_name   : the file will save dump data.
 * return value:
 *      0           : OK
 *      -1          : download failed.
 */
int vastai_dump_download_file(uint32_t card_id, uint64_t address, uint32_t len, char *file_name);

/*
 * Description:
 *      This api can be used to load file to noc.
 * Parameter:
 *      card_id     : card id.
 *      address     : noc address.
 *      file_name   : the file will load to noc.
 * return value:
 *      0           : OK
 *      -1          : load failed.
 */
int vastai_dump_load_file(uint32_t card_id, uint64_t address, char *file_name);

/*
 * Description:
 *      This function make the connection to vastai pcie.
 *  It’s always the first function you have to call to use the vastai dumper.
 * Parameter:
 *      card_id     : card id.
 * return value:
 *      0           : OK
 *      -1          : vastai pcie device open failed.
 */
int vastai_dump_open(uint32_t card_id);

/*
 * Description:
 *      This function closes the connection to vastai pcie.
 * Parameter:
 *      card_id     : card id.
 */
void vastai_dump_close(uint32_t card_id);

/*
 * Description:
 *      This function can convert dbf information to card id information.
 * Parameter:
 *      dev_bdf     : bus, device, function info(such as : 81:00.0).
 * return value:
 *      others      : card id
 *      -1          : not find.
 */
int vastai_dump_map_bdf_to_cardid(char *dev_bdf);

#endif /* __VASTAI_PCIE_DUMP_API_H__ */

