import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as colors
import os,time
import sys


ERR_MAX = 0x3FF
ERR_MIN = 0
X_POINT = 32
Y_POINT = 64
def load_eye_data(file):
    fp = open(file,'r')
    data = fp.readlines()
    eye_data_list = []
    z = [[0 for x in range(X_POINT)] for y in range(Y_POINT)]

    for line in data:
        eye_data_list.append(line.split()) #y,x,z
    '''real data 64x64'''
    for y in range(Y_POINT):
        for x in range(X_POINT):
            z[y][x] = float(eye_data_list[x+y*X_POINT][2])
            #print(eye_data_list[x+y*X_POINT])
            #print("x={:d},y={:.2f},z={:.0f}".format(x*3,1.0*y/127,z[y][x]))

    '''Test data z'''
    return z

def load_eye_data1(file):
    fp = open(file,'r')
    data = fp.readlines()
    eye_data_list = []
    z = [[0 for x in range(X_POINT)] for y in range(Y_POINT)]

    for line in data:
        eye_data_list.append(line.split()) #y,x,z
    '''real data 64x64'''
    for y in range(Y_POINT):
        for x in range(X_POINT):
            z[y][x] = -float(eye_data_list[x+y*X_POINT][2])
            print(eye_data_list[x+y*X_POINT])
            print("x={:d},y={:.2f},z={:.0f}".format(x*3,1.0*y/127,z[y][x]))

    '''Test data z'''
    return z

def pcolor_test(raw_array,sample_bits):
    fig,axes = plt.subplots(3,1)
    '''Err bits raw 0-1023 with 128x128 step=1'''

    x,y = np.mgrid[0:63:64j,0:64:64j]
    c = axes[0].pcolor(x=1.0*x/127,y=y*3,c=raw_array,cmap='coolwarm')
    axes[0].set_title('Raw Err Bit Eye')
    fig.colorbar(c,ax=axes[0])

    # '''Normalize to 0-1 by err_sampled_max'''
    # norm_array = raw_array/raw_array.max()
    # #norm = colors.Normalize(vmin=norm_array.min(), vmax=norm_array.max(), clip=False)
    # c = axes[1].pcolor(norm_array,cmap='coolwarm')
    #
    # '''BER to 10e-12'''
    # ber_array = raw_array/sample_bits
    # axes[1].set_title('Normalization Eye')
    # fig.colorbar(c,ax=axes[1])
    #
    # c = axes[2].pcolor(raw_array, cmap='coolwarm')
    # axes[2].set_title('BER Eye')
    # fig.colorbar(c, ax=axes[2])

    fig.tight_layout()
    plt.show()

def pcolor_mesh_test(raw_array1,raw_array2,raw_array3,raw_array4,lane_read,devstr,mode):
    fig = plt.figure()
    Y_offset=0.2
    
    #1 new
    x= np.arange(-0.1,X_POINT,1)
    y= np.arange(Y_POINT+Y_offset,0,-1)
    ax = fig.add_subplot()
    c = ax.pcolormesh(x,y,raw_array1,cmap=cm.RdBu_r)
    #c = ax.pcolormesh(x,y,raw_array1,cmap=cm.BrBG)
    # plt.plot(11,0,"ro")
    # plt.plot(0,20,"ro")

    #2 new
    x= np.arange(-X_POINT-0.1,0,1)
    y= np.arange(Y_POINT+Y_offset,0,-1)
    #ax = fig.add_subplot()
    c = ax.pcolormesh(x,y,raw_array2,cmap=cm.RdBu_r)
    #c = ax.pcolormesh(x,y,raw_array2,cmap=cm.BrBG)

    #3 old/new
    x= np.arange(-X_POINT-0.1,0,1)
    y= np.arange(0,-Y_POINT-Y_offset,-1)
    #ax = fig.add_subplot()
    c = ax.pcolormesh(x,y,raw_array3,cmap=cm.RdBu_r)
    #c = ax.pcolormesh(x,y,raw_array3,cmap=cm.BrBG)
    # plt.plot(-3,-22,"ro")
    # plt.plot(-16,0,"ro")

    #4 old/new


    x= np.arange(-0.1,X_POINT,1)
    y= np.arange(0,-Y_POINT-Y_offset,-1)
    #ax = fig.add_subplot(224)
    c = ax.pcolormesh(x,y,raw_array4,cmap=cm.RdBu_r)
    #c = ax.pcolormesh(x,y,raw_array4,cmap=cm.BrBG)

    fig.colorbar(c,ax=ax)
    plt.savefig("eye_data_%s_%s/lane%d_eye.jpg"%(devstr,mode,lane_read))
    plt.show()

def pcolor_mesh_test_single(raw_array):
    fig = plt.figure()

    x= np.arange(-0.1,X_POINT,1)
    y= np.arange(-0.1,Y_POINT,1)
    
    ax = fig.add_subplot()

    c = ax.pcolormesh(x,y,raw_array,cmap=cm.RdBu_r)



    fig.colorbar(c,ax=ax)
    plt.show()



if __name__ == '__main__':
    # np.random.seed(19680801)
    # eye_data_raw = np.random.randint(low=ERR_MIN,high=ERR_MAX,size=[16,16])
    
    #len(sys.argv) #number of parameters
    #str(sys.argv) #parameters list
    #str(sys.argv[0]) #script name
    #(die, mode, lane_number):
    devstr = str(sys.argv[1])
    mode = str(sys.argv[2])
    lanenum = int(str(sys.argv[3]))
    for i in range(0, lanenum):
        EYE_DATA_FILE1 = "eye_data_%s_%s/lane%d_a2.txt"%(devstr,mode,i)
        EYE_DATA_FILE2 = "eye_data_%s_%s/lane%d_a1.txt"%(devstr,mode,i)
        EYE_DATA_FILE3 = "eye_data_%s_%s/lane%d_b1.txt"%(devstr,mode,i)
        EYE_DATA_FILE4 = "eye_data_%s_%s/lane%d_b2.txt"%(devstr,mode,i)
        
        eye_data_1 = load_eye_data(EYE_DATA_FILE1)
        eye_data_2 = load_eye_data(EYE_DATA_FILE2)
        eye_data_3 = load_eye_data(EYE_DATA_FILE3)
        eye_data_4 = load_eye_data(EYE_DATA_FILE4)
        pcolor_mesh_test(eye_data_1,eye_data_2,eye_data_3,eye_data_4,i,devstr,mode)
