from vastai_pcie_py import vastai_pcie
import sys
import os
import numpy
import matplotlib
from matplotlib import pyplot
import matplotlib.mlab as mlab
from matplotlib.colors import LogNorm
from time import *
die_id = 0
soPath="../out/libvastai_pcie_user_api.so"
lib = vastai_pcie(soPath,0)

PHY_ep_base_class= [0x08E90000,0x08E90800,0x08E91000,0x08E91800,0x08E92000,0x08E92800,0x08E93000,0x08E93800,
                    0x08E94000,0x08E94800,0x08E95000,0x08E95800,0x08E96000,0x08E96800,0x08E97000,0x08E97800]
PHY_rc_base_class= [0x09790000,0x09790800,0x09791000,0x09791800,0x09792000,0x09792800,0x09793000,0x09793800,
                    0x09794000,0x09794800,0x09795000,0x09795800,0x09796000,0x09796800,0x09797000,0x09797800]
PHY_ep1_base_class= [0x1008E90000,0x1008E90800,0x1008E91000,0x1008E91800,0x1008E92000,0x1008E92800,0x1008E93000,0x1008E93800,
                    0x1008E94000,0x1008E94800,0x1008E95000,0x1008E95800,0x1008E96000,0x1008E96800,0x1008E97000,0x1008E97800]
PHY_rc1_base_class= [0x1009790000,0x1009790800,0x1009791000,0x1009791800,0x1009792000,0x1009792800,0x1009793000,0x1009793800,
                    0x1009794000,0x1009794800,0x1009795000,0x1009795800,0x1009796000,0x1009796800,0x1009797000,0x1009797800]

PIPE_base_class=[0x08EB4000,0x08EB4400,0x08EB4800,0x08EB4C00,0x08EB5000,0x08EB5400,0x08EB5800,0x08EB5C00,
                 0x08EB6000,0x08EB6400,0x08EB6800,0x08EB6C00,0x08EB7000,0x08EB7400,0x08EB7800,0x08EB7C00]

PMA_base_class =[0x08EBC000,0x08EBC400,0x08EBC800,0x08EBCC00,0x08EBD000,0x08EBD400,0x08EBD800,0x08EBDC00,
                 0x08EBE000,0x08EBE400,0x08EBE800,0x08EBEC00,0x08EBF000,0x08EBF400,0x08EBF800,0x08EBFC00]
########################################
def rregaddr(addr):
    if (len("{:x}".format(addr)) > 8): 
        print ("\n****Reg Rd Error: Address length error!\n") 
        return 0
    val = lib.read_4b(addr, die_id) # this lib function takes full 32bits address i.e. 0x08E00000
    return val[0]
########################################
def wregaddr(addr, val):
    if (len("{:x}".format(addr)) > 8): 
        print ("\n****Reg Rd Error: Address length error!\n") 
        return 0
    lib.write_4b(addr, val, die_id) # this lib function takes full 32 bits address i.e. 0x08E00000
############################################ 
def wregBits (addr, bits, value):
##    if (len("{:x}".format(addr)) > 8): 
##        print ("\n****wregBits Error: Max address length Exceeded!\n") 
##        return 0
    reg_val = rregaddr(addr)
    value_bref = 0
    if value > 2**(bits[0]-bits[-1]+1):        # makes sure max value for the bits not exceeded
        value = (2**(bits[0]-bits[-1]+1)) -1
    for i in range(bits[0]-bits[-1]+1):    
        value_bref = value_bref + pow(2,i)
        value_edit = reg_val & (0xffffffff - (value_bref << bits[-1]))
        value_valid = value_edit + ((value & value_bref)<<bits[-1])
    wregaddr(addr, value_valid)
############################################ 
def rregBits(addr, bits):
##    if (len("{:x}".format(addr)) > 8):
##        print ("\n****rregBits Error: Max address length Exceeded!\n") 
##    return 0
    value_all = rregaddr(addr)
    ref = 0
    for i in range(bits[0]-bits[-1]+1):
        ref = ref + pow(2,i)
        value = (value_all >> bits[-1]) & ref
    return value
############################################
def usage():
    print("argv1: die number, argv2: rc or ep, argv3: lane number")

def Eye_Surf(lane_addr, eye_surf_addr, devstr, mode):
    vt = 0
    pi = 0
    vt_step = 2
    pi_step = 2
    g_a1 = open('eye_data_%s_%s/lane%d_a1.txt'%(devstr,mode,lane_addr),'a+')
    g_b1 = open('eye_data_%s_%s/lane%d_b1.txt'%(devstr,mode,lane_addr),'a+')
    g_a2 = open('eye_data_%s_%s/lane%d_a2.txt'%(devstr,mode,lane_addr),'a+')
    g_b2 = open('eye_data_%s_%s/lane%d_b2.txt'%(devstr,mode,lane_addr),'a+')

    wregaddr(eye_surf_addr+4*0x161,0x0023)
    wregaddr(eye_surf_addr+4*0x1A0,0x7A00)

    for vt in range(0x7F,0x0-1,0-vt_step):
        wregaddr(eye_surf_addr+4*0x1A1,vt)
        for pi in range(0x41, 0x7F+1, pi_step):
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
            wregaddr(eye_surf_addr+4*0x1A0,0x7E00|pi)
            flag = rregBits(eye_surf_addr+4*0x1A0,[8,8])
            #sleep(2)
            while(flag!=1):
                #sleep(2)		
                print("1 eyesurf_valid flag = %0x08, waiting for vt and pi %d %d accumulation ready......."%(flag,vt,pi))
                flag = rregBits(eye_surf_addr+4*0x1A0,[8,8]) ## read until bit 8 is high
            a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
            b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            while((a_error == 0xFFFF) or (b_error == 0xFFFF)):
                a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
                b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            g_a1.write('%d    %.4f    %d\n'%(vt*3,1.0*pi/127, rregBits(eye_surf_addr+0x1A2*4,[9,0])))
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
        
        for pi in range(0x00, 0x3F+1, pi_step):
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
            wregaddr(eye_surf_addr+4*0x1A0,0x7E00|pi)
            flag = rregBits(eye_surf_addr+4*0x1A0,[8,8])
            #sleep(2)
            while(flag!=1):
                #sleep(2)		
                print("2 eyesurf_valid flag = %0x08, waiting for vt and pi %d %d accumulation ready......."%(flag,vt,pi))
                flag = rregBits(eye_surf_addr+4*0x1A0,[8,8]) ## read until bit 8 is high
            a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
            b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            while((a_error == 0xFFFF) or (b_error == 0xFFFF)):
                a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
                b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            g_a2.write('%d    %.4f    %d\n'%(vt*3,1.0*pi/127, rregBits(eye_surf_addr+0x1A2*4,[9,0])))
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
    g_a1.close()
    g_a2.close()

    for vt in range(0x0,0x7F+1,vt_step):
        wregaddr(eye_surf_addr+4*0x1A1,vt)
        for pi in range(0x41, 0x7F+1, pi_step):
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
            wregaddr(eye_surf_addr+4*0x1A0,0x7E00|pi)
            flag = rregBits(eye_surf_addr+4*0x1A0,[8,8])
            #sleep(2)
            while(flag!=1):
                #sleep(2)		
                print("3 eyesurf_valid flag = %0x08, waiting for vt and pi %d %d accumulation ready......."%(flag,vt,pi))
                flag = rregBits(eye_surf_addr+4*0x1A0,[8,8]) ## read until bit 8 is high
            a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
            b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            while((a_error == 0xFFFF) or (b_error == 0xFFFF)):
                a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
                b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            g_b1.write('%d    %.4f    %d\n'%(vt*3,1.0*pi/127, rregBits(eye_surf_addr+0x1A3*4,[9,0])))
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
        for pi in range(0x00, 0x3F+1, pi_step):
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
            wregaddr(eye_surf_addr+4*0x1A0,0x7E00|pi)
            flag = rregBits(eye_surf_addr+4*0x1A0,[8,8])
            #sleep(2)
            while(flag!=1):
                #sleep(2)		
                print("4 eyesurf_valid flag = %0x08, waiting for vt and pi %d %d accumulation ready......."%(flag,vt,pi))
                flag = rregBits(eye_surf_addr+4*0x1A0,[8,8]) ## read until bit 8 is high
            a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
            b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            while((a_error == 0xFFFF) or (b_error == 0xFFFF)):
                a_error = rregBits(eye_surf_addr+0x1A2*4,[15,0]) 
                b_error = rregBits(eye_surf_addr+0x1A3*4,[15,0]) 
            g_b2.write('%d    %.4f    %d\n'%(vt*3,1.0*pi/127, rregBits(eye_surf_addr+0x1A3*4,[9,0])))
            wregaddr(eye_surf_addr+4*0x1A0,0x7A00|pi)
    g_b1.close()
    g_b2.close()
    wregaddr(eye_surf_addr+4*0x1A0,0x7800)   

if __name__=="__main__":    
    #print("INFO: ---Begin eyesurf Pattern---.\n")
    #len(sys.argv) #number of parameters
    #str(sys.argv) #parameters list
    #str(sys.argv[0]) #script name
    #(die, mode, lanenum, devstr):
    lib = vastai_pcie(soPath,int(sys.argv[1]))
    die = str(sys.argv[2])
    mode = str(sys.argv[3])
    lanenum = int(str(sys.argv[4]))
    devstr = str(sys.argv[5])
    os.mkdir("eye_data_%s_%s"%(devstr,mode))
    
    if die == "die0":
        if mode == "rc":
            lane_addr = PHY_rc_base_class
            for lane in range(0, lanenum):
                Eye_Surf(lane, lane_addr[lane], devstr, mode)
        elif mode == "ep":
            lane_addr = PHY_ep_base_class
            for lane in range(0, lanenum):
                Eye_Surf(lane, lane_addr[lane], devstr, mode)
    elif die == "die1":
        if mode == "rc":
            lane_addr = PHY_rc1_base_class
            for lane in range(0, lanenum):
                Eye_Surf(lane, lane_addr[lane], devstr, mode)
        elif mode == "ep":
            lane_addr = PHY_ep1_base_class
            for lane in range(0, lanenum):
                Eye_Surf(lane, lane_addr[lane], devstr, mode)
    else:
        usage()
    	
