#ifndef __GEN_MODEL_H
#define __GEN_MODEL_H

#define ns(x) FLATBUFFERS_WRAP_NAMESPACE(vastai_uvp, x)
#define c_vec_len(V) (sizeof(V)/sizeof((V)[0]))

typedef struct layer_csr_cmcu {
    uint32_t layer_id;
    uint32_t opcode_num;
    uint32_t odma_num;
    uint32_t reserved;
    uint32_t opcode_data;
    uint32_t odma_data;
} layer_csr_cmcu_t;

typedef struct layer_csr_lmcu {
    uint32_t layer_id;
    uint32_t opcode_num;
    uint32_t dlc_num;
    uint32_t reserved;
    uint32_t opcode_data;
    uint32_t dlc_data;
} layer_csr_lmcu_t;

typedef struct layer_csr_dsp {
    uint32_t layer_id;
    uint32_t opcode_num;
    uint32_t odsp_op_num;
    uint32_t reserved;
    uint32_t opcode_data;
    uint32_t odsp_op_data;
} layer_csr_dsp_t;

typedef struct wdma_fw {
    uint32_t fw_id;
    uint32_t opcode_num;
    uint32_t csr_num;
    uint32_t reserved;
    uint32_t opcode_data;
    uint32_t csr_data;
} wdma_fw_t;

typedef struct wdma_core {
    uint32_t core_id;
    uint32_t fw_num;
    uint32_t reserved;
    uint32_t fw_data;
} wdma_core_t;

typedef struct model_info {
    uint32_t crc;
    uint32_t reserved0;
    uint32_t version;
    uint32_t model_id;
    uint32_t dlc_num;
    uint32_t odsp_num;
    uint32_t odma_pipe_num;
    uint32_t input_addr_num;
    uint32_t output_addr_num;
    uint32_t layer_num;
    uint32_t reserved1;
    uint32_t input_addr_data;
    uint32_t output_addr_data;
    uint32_t odma_data;
    uint32_t layer_cmcu_data;
    uint32_t wdma_st_data;
    uint32_t wdma_fw_data;
    uint32_t layer_lmcu_data;
    uint32_t layer_odsp_data;
    uint32_t reserved2;
} model_info_t;

typedef enum oak_typ { oak_typ_odma, oak_typ_dlc, oak_typ_odsp, oak_typ_count } oak_typ_t;

void * get_model_info();

uint32_t get_model_info_size();

void * get_model_csr();

uint32_t get_model_csr_size();

uint32_t get_dlc_num();

void model_gen(ns(uvp_package_table_t) uvp);

void model_free();
#endif
