// CSR register base address and offset
#define CSR_ODMA_BASE_ADDR_UVP                0xF00000            //odma0
#define CSR_ODMA1_BASE_ADDR_UVP               0x1000000           //odma1

#define OAK0_CSR_BASE_ADDR_UVP                0x700000            //oak core 0
#define OAK1_CSR_BASE_ADDR_UVP                0x800000            //oak core 1 
#define OAK2_CSR_BASE_ADDR_UVP                0x900000            //oak core 2
#define OAK3_CSR_BASE_ADDR_UVP                0xA00000            //oak core 3
#define OAK4_CSR_BASE_ADDR_UVP                0xB00000            //oak core 4
#define OAK5_CSR_BASE_ADDR_UVP                0xC00000            //oak core 5
#define OAK6_CSR_BASE_ADDR_UVP                0xD00000            //oak core 6
#define OAK7_CSR_BASE_ADDR_UVP                0xE00000            //oak core 7

#define OAK_CTRANS_OFFSET_START_UVP           0x1000              //cdma offset start
#define OAK_CTRANS_OFFSET_END_UVP             0x10f8              //cdma offset end

#define OAK_WDDMA_OFFSET_START_UVP            0x400               //wdma offset start
#define OAK_WDDMA_OFFSET_END_UVP              0x46c               //wdma offset end

#define ODMA_ODMA_OFFSET_START_UVP            0x0                 //odma offset start
#define ODMA_ODMA_OFFSET_END_UVP              0x318               //odma offset end

#define OAK_CSR_OFFSET_END_UVP                0x181c              //oak offset end

#define OAK_WDDMA_SOFT_SET_EN_UVP             0x418               //the registe need to put together with oak csr

// BLOCK name defines
#define SV100_OAK_CORE                        0x00
#define SV100_OAK_WDMA                        0x01
#define SV100_OAK_CDMA                        0x02
#define SV100_ODMA                            0x03

// COMMON data type, 0x0000 - 0x0FFF
// COMMON date type - wdma firmware
#define COMMON_DATA_TYPE_WDMA0_FIRMWARE       0x0000
#define COMMON_DATA_TYPE_WDMA1_FIRMWARE       0x0001
#define COMMON_DATA_TYPE_WDMA2_FIRMWARE       0x0002
#define COMMON_DATA_TYPE_WDMA3_FIRMWARE       0x0003
#define COMMON_DATA_TYPE_WDMA4_FIRMWARE       0x0004
#define COMMON_DATA_TYPE_WDMA5_FIRMWARE       0x0005
#define COMMON_DATA_TYPE_WDMA6_FIRMWARE       0x0006
#define COMMON_DATA_TYPE_WDMA7_FIRMWARE       0x0007

#define COMMON_DATA_TYPE_WDMA0_FW_BIN         0x0000

// COMMON date type - look up table
#define COMMON_DATA_TYPE_LOOK_UP_TABLE0       0x0010
#define COMMON_DATA_TYPE_LOOK_UP_TABLE1       0x0011
#define COMMON_DATA_TYPE_LOOK_UP_TABLE2       0x0012
#define COMMON_DATA_TYPE_LOOK_UP_TABLE3       0x0013
#define COMMON_DATA_TYPE_LOOK_UP_TABLE4       0x0014
#define COMMON_DATA_TYPE_LOOK_UP_TABLE5       0x0015
#define COMMON_DATA_TYPE_LOOK_UP_TABLE6       0x0016
#define COMMON_DATA_TYPE_LOOK_UP_TABLE7       0x0017

// COMMON date type - RESERVE 0
#define COMMON_DATA_TYPE_RESERVE0             0x0F00
// COMMON date type - RESERVE 1
#define COMMON_DATA_TYPE_RESERVE1             0x0F01
// COMMON date type - RESERVE 2
#define COMMON_DATA_TYPE_RESERVE2             0x0F02
// COMMON date type - RESERVE 3
#define COMMON_DATA_TYPE_RESERVE3             0x0F03
// COMMON date type - RESERVE 4
#define COMMON_DATA_TYPE_RESERVE4             0x0F04
// COMMON date type - RESERVE 5
#define COMMON_DATA_TYPE_RESERVE5             0x0F05
// COMMON date type - RESERVE 6
#define COMMON_DATA_TYPE_RESERVE6             0x0F06
// COMMON date type - RESERVE 7
#define COMMON_DATA_TYPE_RESERVE7             0x0F07
// COMMON date type - RESERVE 8
#define COMMON_DATA_TYPE_RESERVE8             0x0F08
// COMMON date type - RESERVE 9
#define COMMON_DATA_TYPE_RESERVE9             0x0F09
// COMMON date type - RESERVE 10
#define COMMON_DATA_TYPE_RESERVE10            0x0F0A
// COMMON date type - RESERVE 11
#define COMMON_DATA_TYPE_RESERVE11            0x0F0B
// COMMON date type - RESERVE 12
#define COMMON_DATA_TYPE_RESERVE12            0x0F0C
// COMMON date type - RESERVE 13
#define COMMON_DATA_TYPE_RESERVE13            0x0F0D
// COMMON date type - RESERVE 14
#define COMMON_DATA_TYPE_RESERVE14            0x0F0E
// COMMON date type - RESERVE 15
#define COMMON_DATA_TYPE_RESERVE15            0x0F0F

// COMMON date type - UNKNOWN
#define COMMON_DATA_TYPE_UNKNOWN              0x0FFF


// USER data type, 0x1000 - 0x1FFF
// USER data type - image
#define USER_DATA_TYPE_IMAGE0                 0x1000
#define USER_DATA_TYPE_IMAGE1                 0x1001
#define USER_DATA_TYPE_IMAGE2                 0x1002
#define USER_DATA_TYPE_IMAGE3                 0x1003
#define USER_DATA_TYPE_IMAGE4                 0x1004
#define USER_DATA_TYPE_IMAGE5                 0x1005
#define USER_DATA_TYPE_IMAGE6                 0x1006
#define USER_DATA_TYPE_IMAGE7                 0x1007

// USER data type - weight
#define USER_DATA_TYPE_WEIGHT0                0x1010
#define USER_DATA_TYPE_WEIGHT1                0x1011
#define USER_DATA_TYPE_WEIGHT2                0x1012
#define USER_DATA_TYPE_WEIGHT3                0x1013
#define USER_DATA_TYPE_WEIGHT4                0x1014
#define USER_DATA_TYPE_WEIGHT5                0x1015
#define USER_DATA_TYPE_WEIGHT6                0x1016
#define USER_DATA_TYPE_WEIGHT7                0x1017
											
// USER data type - oak csr                    
#define USER_DATA_TYPE_OAK0_CSR               0x1020
#define USER_DATA_TYPE_OAK1_CSR               0x1021
#define USER_DATA_TYPE_OAK2_CSR               0x1022
#define USER_DATA_TYPE_OAK3_CSR               0x1023
#define USER_DATA_TYPE_OAK4_CSR               0x1024
#define USER_DATA_TYPE_OAK5_CSR               0x1025
#define USER_DATA_TYPE_OAK6_CSR               0x1026
#define USER_DATA_TYPE_OAK7_CSR               0x1027
									
// USER data type - cdma csr          
#define USER_DATA_TYPE_CDMA0_CSR              0x1030
#define USER_DATA_TYPE_CDMA1_CSR              0x1031
#define USER_DATA_TYPE_CDMA2_CSR              0x1032
#define USER_DATA_TYPE_CDMA3_CSR              0x1033
#define USER_DATA_TYPE_CDMA4_CSR              0x1034
#define USER_DATA_TYPE_CDMA5_CSR              0x1035
#define USER_DATA_TYPE_CDMA6_CSR              0x1036
#define USER_DATA_TYPE_CDMA7_CSR              0x1037
											
// USER data type - wdma csr                
#define USER_DATA_TYPE_WDMA0_CSR              0x1040
#define USER_DATA_TYPE_WDMA1_CSR              0x1041
#define USER_DATA_TYPE_WDMA2_CSR              0x1042
#define USER_DATA_TYPE_WDMA3_CSR              0x1043
#define USER_DATA_TYPE_WDMA4_CSR              0x1044
#define USER_DATA_TYPE_WDMA5_CSR              0x1045
#define USER_DATA_TYPE_WDMA6_CSR              0x1046
#define USER_DATA_TYPE_WDMA7_CSR              0x1047
											
// USER data type - odma csr                
#define USER_DATA_TYPE_ODMA0_CSR              0x1050
#define USER_DATA_TYPE_ODMA1_CSR              0x1051
											
// USER date type - dsp firmware            
#define USER_DATA_TYPE_DSP0_FIRMWARE          0x1060
#define USER_DATA_TYPE_DSP1_FIRMWARE          0x1061
#define USER_DATA_TYPE_DSP2_FIRMWARE          0x1062
#define USER_DATA_TYPE_DSP3_FIRMWARE          0x1063
#define USER_DATA_TYPE_DSP4_FIRMWARE          0x1064
#define USER_DATA_TYPE_DSP5_FIRMWARE          0x1065
#define USER_DATA_TYPE_DSP6_FIRMWARE          0x1066
#define USER_DATA_TYPE_DSP7_FIRMWARE          0x1067
											
// USER date type - lmcu firmware           
#define USER_DATA_TYPE_LMCU0_FIRMWARE         0x1070
#define USER_DATA_TYPE_LMCU1_FIRMWARE         0x1071
#define USER_DATA_TYPE_LMCU2_FIRMWARE         0x1072
#define USER_DATA_TYPE_LMCU3_FIRMWARE         0x1073
#define USER_DATA_TYPE_LMCU4_FIRMWARE         0x1074
#define USER_DATA_TYPE_LMCU5_FIRMWARE         0x1075
#define USER_DATA_TYPE_LMCU6_FIRMWARE         0x1076
#define USER_DATA_TYPE_LMCU7_FIRMWARE         0x1077

// USER date type - cmcu firmware           
#define USER_DATA_TYPE_CMCU0_FIRMWARE         0x1080

// USER data type - reference data
// Notes: the reference data dst_addr need to 
// set with value: 0xFFFFFFFFFFFFFFFF
#define USER_DATA_TYPE_REFERENCE0             0x1090
#define USER_DATA_TYPE_REFERENCE1             0x1091
#define USER_DATA_TYPE_REFERENCE2             0x1092
#define USER_DATA_TYPE_REFERENCE3             0x1093
#define USER_DATA_TYPE_REFERENCE4             0x1094
#define USER_DATA_TYPE_REFERENCE5             0x1095
#define USER_DATA_TYPE_REFERENCE6             0x1096
#define USER_DATA_TYPE_REFERENCE7             0x1097

#define USER_DATA_TYPE_DSP0_OP                0x10a0
#define USER_DATA_TYPE_DSP1_OP                0x10a1
#define USER_DATA_TYPE_DSP2_OP                0x10a2
#define USER_DATA_TYPE_DSP3_OP                0x10a3
#define USER_DATA_TYPE_DSP4_OP                0x10a4
#define USER_DATA_TYPE_DSP5_OP                0x10a5
#define USER_DATA_TYPE_DSP6_OP                0x10a6
#define USER_DATA_TYPE_DSP7_OP                0x10a7

#define USER_DATA_TYPE_WDMA0_FIRMWARE         0x1F00

// USER data reserved type
// USER date type - RESERVE 0
#define USER_DATA_TYPE_RESERVE0               0x1F00
// USER date type - RESERVE 1
#define USER_DATA_TYPE_RESERVE1               0x1F01
// USER date type - RESERVE 2
#define USER_DATA_TYPE_RESERVE2               0x1F02
// USER date type - RESERVE 3
#define USER_DATA_TYPE_RESERVE3               0x1F03
// USER date type - RESERVE 4
#define USER_DATA_TYPE_RESERVE4               0x1F04
// USER date type - RESERVE 5
#define USER_DATA_TYPE_RESERVE5               0x1F05
// USER date type - RESERVE 6
#define USER_DATA_TYPE_RESERVE6               0x1F06
// USER date type - RESERVE 7
#define USER_DATA_TYPE_RESERVE7               0x1F07
// USER date type - RESERVE 8
#define USER_DATA_TYPE_RESERVE8               0x1F08
// USER date type - RESERVE 9
#define USER_DATA_TYPE_RESERVE9               0x1F09
// USER date type - RESERVE 10
#define USER_DATA_TYPE_RESERVE10              0x1F0A
// USER date type - RESERVE 11
#define USER_DATA_TYPE_RESERVE11              0x1F0B
// USER date type - RESERVE 12
#define USER_DATA_TYPE_RESERVE12              0x1F0C
// USER date type - RESERVE 13
#define USER_DATA_TYPE_RESERVE11              0x1F0D
// USER date type - RESERVE 14
#define USER_DATA_TYPE_RESERVE12              0x1F0E
// USER date type - RESERVE 15
#define USER_DATA_TYPE_RESERVE11              0x1F0F
// USER date type - RESERVE 16
#define USER_DATA_TYPE_RESERVE16              0x1F10
// USER date type - RESERVE 17
#define USER_DATA_TYPE_RESERVE17              0x1F11
// USER date type - RESERVE 18
#define USER_DATA_TYPE_RESERVE18              0x1F12
// USER date type - RESERVE 19
#define USER_DATA_TYPE_RESERVE19              0x1F13
// USER date type - RESERVE 20
#define USER_DATA_TYPE_RESERVE20              0x1F14
// USER date type - RESERVE 21
#define USER_DATA_TYPE_RESERVE21              0x1F15
// USER date type - RESERVE 22
#define USER_DATA_TYPE_RESERVE22              0x1F16
// USER date type - RESERVE 23
#define USER_DATA_TYPE_RESERVE23              0x1F17
// USER date type - RESERVE 24
#define USER_DATA_TYPE_RESERVE24              0x1F18
// USER date type - RESERVE 25
#define USER_DATA_TYPE_RESERVE25              0x1F19
// USER date type - RESERVE 26
#define USER_DATA_TYPE_RESERVE26              0x1F1A
// USER date type - RESERVE 27
#define USER_DATA_TYPE_RESERVE27              0x1F1B
// USER date type - RESERVE 28
#define USER_DATA_TYPE_RESERVE28              0x1F1C
// USER date type - RESERVE 29
#define USER_DATA_TYPE_RESERVE29              0xFF1D
// USER date type - RESERVE 30
#define USER_DATA_TYPE_RESERVe30              0x1F1E
// USER date type - RESERVE 31
#define USER_DATA_TYPE_RESERVE31              0x1F1F

// USER date type - elementwise 
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH0	0x2000
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH1	0x2001
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH2	0x2002
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH3	0x2003
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH4	0x2004
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH5	0x2005
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH6	0x2006
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH7	0x2007
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH8	0x2008
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH9	0x2009
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH10	0x200A
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH11	0x200B
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH12	0x200C
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH13	0x200D
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH14	0x200E
#define USER_DATA_TYPE_OAK0_SRC0_BRANCH15	0x200F
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH0	0x2010
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH1	0x2011
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH2	0x2012
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH3	0x2013
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH4	0x2014
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH5	0x2015
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH6	0x2016
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH7	0x2017
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH8	0x2018
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH9	0x2019
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH10	0x201A
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH11	0x201B
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH12	0x201C
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH13	0x201D
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH14	0x201E
#define USER_DATA_TYPE_OAK1_SRC0_BRANCH15	0x201F
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH0	0x2020
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH1	0x2021
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH2	0x2022
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH3	0x2023
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH4	0x2024
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH5	0x2025
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH6	0x2026
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH7	0x2027
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH8	0x2028
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH9	0x2029
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH10	0x202A
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH11	0x202B
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH12	0x202C
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH13	0x202D
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH14	0x202E
#define USER_DATA_TYPE_OAK2_SRC0_BRANCH15	0x202F
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH0	0x2030
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH1	0x2031
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH2	0x2032
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH3	0x2033
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH4	0x2034
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH5	0x2035
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH6	0x2036
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH7	0x2037
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH8	0x2038
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH9	0x2039
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH10	0x203A
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH11	0x203B
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH12	0x203C
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH13	0x203D
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH14	0x203E
#define USER_DATA_TYPE_OAK3_SRC0_BRANCH15	0x203F
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH0	0x2040
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH1	0x2041
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH2	0x2042
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH3	0x2043
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH4	0x2044
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH5	0x2045
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH6	0x2046
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH7	0x2047
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH8	0x2048
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH9	0x2049
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH10	0x204A
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH11	0x204B
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH12	0x204C
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH13	0x204D
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH14	0x204E
#define USER_DATA_TYPE_OAK4_SRC0_BRANCH15	0x204F
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH0	0x2050
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH1	0x2051
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH2	0x2052
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH3	0x2053
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH4	0x2054
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH5	0x2055
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH6	0x2056
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH7	0x2057
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH8	0x2058
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH9	0x2059
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH10	0x205A
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH11	0x205B
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH12	0x205C
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH13	0x205D
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH14	0x205E
#define USER_DATA_TYPE_OAK5_SRC0_BRANCH15	0x205F
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH0	0x2060
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH1	0x2061
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH2	0x2062
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH3	0x2063
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH4	0x2064
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH5	0x2065
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH6	0x2066
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH7	0x2067
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH8	0x2068
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH9	0x2069
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH10	0x206A
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH11	0x206B
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH12	0x206C
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH13	0x206D
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH14	0x206E
#define USER_DATA_TYPE_OAK6_SRC0_BRANCH15	0x206F
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH0	0x2070
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH1	0x2071
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH2	0x2072
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH3	0x2073
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH4	0x2074
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH5	0x2075
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH6	0x2076
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH7	0x2077
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH8	0x2078
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH9	0x2079
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH10	0x207A
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH11	0x207B
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH12	0x207C
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH13	0x207D
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH14	0x207E
#define USER_DATA_TYPE_OAK7_SRC0_BRANCH15	0x207F

#define USER_DATA_TYPE_OAK0_SRC1_BRANCH0	0x2100
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH1	0x2101
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH2	0x2102
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH3	0x2103
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH4	0x2104
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH5	0x2105
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH6	0x2106
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH7	0x2107
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH8	0x2108
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH9	0x2109
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH10	0x210A
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH11	0x210B
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH12	0x210C
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH13	0x210D
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH14	0x210E
#define USER_DATA_TYPE_OAK0_SRC1_BRANCH15	0x210F
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH0	0x2110
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH1	0x2111
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH2	0x2112
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH3	0x2113
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH4	0x2114
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH5	0x2115
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH6	0x2116
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH7	0x2117
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH8	0x2118
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH9	0x2119
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH10	0x211A
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH11	0x211B
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH12	0x211C
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH13	0x211D
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH14	0x211E
#define USER_DATA_TYPE_OAK1_SRC1_BRANCH15	0x211F
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH0	0x2120
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH1	0x2121
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH2	0x2122
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH3	0x2123
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH4	0x2124
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH5	0x2125
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH6	0x2126
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH7	0x2127
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH8	0x2128
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH9	0x2129
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH10	0x212A
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH11	0x212B
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH12	0x212C
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH13	0x212D
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH14	0x212E
#define USER_DATA_TYPE_OAK2_SRC1_BRANCH15	0x212F
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH0	0x2130
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH1	0x2131
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH2	0x2132
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH3	0x2133
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH4	0x2134
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH5	0x2135
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH6	0x2136
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH7	0x2137
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH8	0x2138
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH9	0x2139
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH10	0x213A
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH11	0x213B
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH12	0x213C
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH13	0x213D
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH14	0x213E
#define USER_DATA_TYPE_OAK3_SRC1_BRANCH15	0x213F
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH0	0x2140
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH1	0x2141
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH2	0x2142
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH3	0x2143
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH4	0x2144
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH5	0x2145
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH6	0x2146
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH7	0x2147
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH8	0x2148
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH9	0x2149
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH10	0x214A
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH11	0x214B
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH12	0x214C
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH13	0x214D
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH14	0x214E
#define USER_DATA_TYPE_OAK4_SRC1_BRANCH15	0x214F
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH0	0x2150
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH1	0x2151
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH2	0x2152
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH3	0x2153
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH4	0x2154
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH5	0x2155
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH6	0x2156
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH7	0x2157
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH8	0x2158
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH9	0x2159
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH10	0x215A
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH11	0x215B
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH12	0x215C
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH13	0x215D
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH14	0x215E
#define USER_DATA_TYPE_OAK5_SRC1_BRANCH15	0x215F
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH0	0x2160
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH1	0x2161
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH2	0x2162
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH3	0x2163
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH4	0x2164
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH5	0x2165
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH6	0x2166
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH7	0x2167
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH8	0x2168
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH9	0x2169
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH10	0x216A
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH11	0x216B
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH12	0x216C
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH13	0x216D
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH14	0x216E
#define USER_DATA_TYPE_OAK6_SRC1_BRANCH15	0x216F
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH0	0x2170
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH1	0x2171
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH2	0x2172
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH3	0x2173
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH4	0x2174
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH5	0x2175
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH6	0x2176
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH7	0x2177
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH8	0x2178
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH9	0x2179
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH10	0x217A
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH11	0x217B
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH12	0x217C
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH13	0x217D
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH14	0x217E
#define USER_DATA_TYPE_OAK7_SRC1_BRANCH15	0x217F

// USER date type - LUT
#define USER_DATA_TYPE_OAK0_LUT			0x2200
#define USER_DATA_TYPE_OAK1_LUT			0x2201
#define USER_DATA_TYPE_OAK2_LUT			0x2202
#define USER_DATA_TYPE_OAK3_LUT			0x2203
#define USER_DATA_TYPE_OAK4_LUT			0x2204
#define USER_DATA_TYPE_OAK5_LUT			0x2205
#define USER_DATA_TYPE_OAK6_LUT			0x2206
#define USER_DATA_TYPE_OAK7_LUT			0x2207

// USER date type - SF
#define USER_DATA_TYPE_OAK0_SF			0x2210
#define USER_DATA_TYPE_OAK1_SF			0x2211
#define USER_DATA_TYPE_OAK2_SF			0x2212
#define USER_DATA_TYPE_OAK3_SF			0x2213
#define USER_DATA_TYPE_OAK4_SF			0x2214
#define USER_DATA_TYPE_OAK5_SF			0x2215
#define USER_DATA_TYPE_OAK6_SF			0x2216
#define USER_DATA_TYPE_OAK7_SF			0x2217

// USER date type - UNKNOWN
#define USER_DATA_TYPE_UNKNOWN                0x1FFF
