#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "uvp_builder.h"
#include "uvp_defines.h"
#include "gen_model.h"

uint32_t g_layer_num;
static void *g_info_ptr;
static void *g_csr_ptr;
static uint32_t g_info_size;
static uint32_t g_csr_size;
static uint32_t g_info_offset;
static uint32_t g_csr_offset;
static uint32_t g_info_entry;
static uint32_t g_csr_entry;
static uint32_t g_dlc_flag;
static uint32_t g_odsp_flag;

void * get_model_info()
{
    return g_info_ptr;
}

uint32_t get_model_info_size()
{
    return g_info_offset;
}

void * get_model_csr()
{
    return g_csr_ptr;
}

uint32_t get_model_csr_size()
{
    return g_csr_offset;
}

uint32_t get_dlc_num()
{
    return ((model_info_t *)g_info_ptr)->dlc_num;
}

uint32_t get_odsp_num()
{
    return ((model_info_t *)g_info_ptr)->odsp_num;
}

static ns(user_data_table_t) model_layer_find(ns(user_data_vec_t) data_list, oak_typ_t oak_typ, uint32_t oak_cid, uint8_t csr_op)
{
    int i;
    int data_num;
    int type;
    int data_type;

    switch (oak_typ)
    {
    case oak_typ_odma:
        if (csr_op)
        {
            type = USER_DATA_TYPE_CMCU0_FIRMWARE;
        }
        else
        {
            type = USER_DATA_TYPE_ODMA0_CSR + oak_cid;
        }
        break;
    case oak_typ_dlc:
        if (csr_op)
        {
            type = USER_DATA_TYPE_LMCU0_FIRMWARE + oak_cid;
        }
        else
        {
            type = USER_DATA_TYPE_OAK0_CSR + oak_cid;
        }
        break;
    case oak_typ_odsp:
        if (csr_op)
        {
            type = USER_DATA_TYPE_DSP0_FIRMWARE + oak_cid;
        }
        else
        {
            type = USER_DATA_TYPE_DSP0_OP + oak_cid;
        }
        break;
    }

    data_num = ns(user_data_vec_len(data_list));
    for (i = 0; i < data_num; i++)
    {
        ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, i));
        data_type = ns(user_data_type(user_data));
        if (data_type == type)
        {
            return user_data;
        }
    }
    return NULL;
}

static void model_gen_start(ns(uvp_package_table_t) uvp)
{
    int i, j, k;
    int data_num;
    uint64_t dst_addr;
    uint32_t length;
    uint16_t type;
    uint8_t *data;
    FILE *fp;
    char fileName[256];

    g_layer_num = ns(uvp_package_layer_num(uvp));
    printf("layer num %d.\n", g_layer_num);
    g_csr_size = 0;
    g_info_size = sizeof(model_info_t) +sizeof(uint32_t) * 8 * 4 + \
                      (sizeof(layer_csr_cmcu_t) + (sizeof(layer_csr_lmcu_t) + sizeof(wdma_core_t) * 2 + sizeof(wdma_fw_t) + sizeof(layer_csr_dsp_t)) * 8) * g_layer_num;
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));

    for (i = 0; i < g_layer_num; i++)
    {
        ns(layer_table_t) layer = ns(layer_vec_at(layers, i));
        ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
        data_num = ns(user_data_vec_len(data_list));
        for (j = 0; j < data_num; j++)
        {
            ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
            dst_addr = ns(user_data_dst_addr(user_data));
            length = ns(user_data_length(user_data));
            type = ns(user_data_type(user_data));
            data = ns(user_data_data(user_data));
            switch (type)
            {
                case (USER_DATA_TYPE_IMAGE0)...(USER_DATA_TYPE_IMAGE0 + 7):
                case (USER_DATA_TYPE_WEIGHT0)...(USER_DATA_TYPE_WEIGHT0 + 7):
                case (USER_DATA_TYPE_REFERENCE0)...(USER_DATA_TYPE_REFERENCE0 + 7):
                    break;
                default:
                    g_csr_size += length;
                    break;
            }
        }
    }
}

static void model_info_gen()
{
    g_info_entry = 0x8c58000;
    g_csr_entry = 0x85600000;
    g_info_offset = 0;
    g_csr_offset = 0;
    g_info_ptr = malloc(g_info_size);
    g_csr_ptr = malloc(g_csr_size);

    memset((model_info_t *)g_info_ptr, 0 ,sizeof(model_info_t));
    ((model_info_t *)g_info_ptr)->layer_num = g_layer_num;
    ((model_info_t *)g_info_ptr)->wdma_st_data = g_info_entry + sizeof(model_info_t);
    memset(g_info_ptr + sizeof(model_info_t), 0, 8 * sizeof(uint32_t));
    ((model_info_t *)g_info_ptr)->wdma_fw_data = g_info_entry + sizeof(model_info_t) + 8 * sizeof(uint32_t);
    memset(g_info_ptr + sizeof(model_info_t) + 8 * sizeof(uint32_t), 0, 8 * sizeof(uint32_t));
    ((model_info_t *)g_info_ptr)->layer_lmcu_data = g_info_entry + sizeof(model_info_t) + 8 * sizeof(uint32_t) * 2;
    memset(g_info_ptr + sizeof(model_info_t) + 8 * sizeof(uint32_t) * 2, 0, 8 * sizeof(uint32_t));
    ((model_info_t *)g_info_ptr)->layer_odsp_data = g_info_entry + sizeof(model_info_t) + 8 * sizeof(uint32_t) * 3;
    memset(g_info_ptr + sizeof(model_info_t) + 8 * sizeof(uint32_t) * 3, 0, 8 * sizeof(uint32_t));
    ((model_info_t *)g_info_ptr)->layer_cmcu_data = g_info_entry + sizeof(model_info_t) + 8 * sizeof(uint32_t) * 4;
    g_info_offset = sizeof(model_info_t) + 8 * sizeof(uint32_t) * 4;
}

static void cmcu_info_gen(ns(uvp_package_table_t) uvp)
{
    uint32_t i, j;
    uint32_t data_num;
    uint64_t dst_addr;
    uint32_t length;
    uint16_t type;
    uint8_t *data;
    layer_csr_cmcu_t *layer_csr_cmcu;
    layer_csr_cmcu = g_info_ptr + g_info_offset;
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    for (i = 0; i < g_layer_num; i++)
    {
        ns(layer_table_t) layer = ns(layer_vec_at(layers, i));
        ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
        data_num = ns(user_data_vec_len(data_list));
        memset(layer_csr_cmcu, 0, sizeof(layer_csr_cmcu_t));
        for (j = 0; j < data_num; j++)
        {
            ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
            dst_addr = ns(user_data_dst_addr(user_data));
            length = ns(user_data_length(user_data));
            type = ns(user_data_type(user_data));
            data = ns(user_data_data(user_data));
            if (type == USER_DATA_TYPE_CMCU0_FIRMWARE)
            {
                layer_csr_cmcu->layer_id = i;
                layer_csr_cmcu->opcode_num = (uint32_t)(length/8);
                layer_csr_cmcu->opcode_data = g_csr_entry + g_csr_offset;
                memcpy(g_csr_ptr+g_csr_offset, data, length);
                g_csr_offset += length;
            }
            else if (type == USER_DATA_TYPE_ODMA0_CSR)
            {
                layer_csr_cmcu->odma_num = (uint32_t)(length/8);
                layer_csr_cmcu->odma_data = g_csr_entry + g_csr_offset;
                memcpy(g_csr_ptr+g_csr_offset, data, length);
                g_csr_offset += length;
            }
            else if (type == (USER_DATA_TYPE_ODMA0_CSR + 1))
            {
                layer_csr_cmcu->odma_num += (uint32_t)(length/8);
                memcpy(g_csr_ptr+g_csr_offset, data, length);
                g_csr_offset += length;
            }
        }
        layer_csr_cmcu += 1;
        g_info_offset += sizeof(layer_csr_cmcu_t);
    }
}

static void wdma_info_gen(ns(uvp_package_table_t) uvp)
{
    wdma_core_t *wdma_core;
    wdma_fw_t *wdma_fw;
    uint32_t i, j;
    uint32_t data_num;
    uint64_t dst_addr;
    uint32_t length;
    uint16_t type;
    uint8_t *data;
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    ns(layer_table_t) layer = ns(layer_vec_at(layers, 0));
    ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
    data_num = ns(user_data_vec_len(data_list));
    for (i = 0; i < 8; i++)
    {
        wdma_fw = g_info_ptr + g_info_offset;
        wdma_core = g_info_ptr + g_info_offset + sizeof(wdma_fw_t);
        memset(wdma_fw, 0, sizeof(wdma_fw_t));
        memset(wdma_core, 0, sizeof(wdma_core_t));
        wdma_core->core_id = i;
        wdma_core->fw_num = 1;
        wdma_core->fw_data = g_info_entry + g_info_offset;
        wdma_fw->fw_id = 0;
        *((uint32_t *)(g_info_ptr + sizeof(model_info_t) + (8 + i) * sizeof(uint32_t))) = g_info_entry + g_info_offset + sizeof(wdma_fw_t);
        g_info_offset += sizeof(wdma_fw_t) + sizeof(wdma_core_t);
        for (j = 0; j < data_num; j++)
        {
            ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
            dst_addr = ns(user_data_dst_addr(user_data));
            length = ns(user_data_length(user_data));
            type = ns(user_data_type(user_data));
            data = ns(user_data_data(user_data));
            if (type == (USER_DATA_TYPE_WDMA0_FIRMWARE + i))
            {
                wdma_fw->opcode_num = (uint32_t)(length/8);
                wdma_fw->opcode_data = g_csr_entry + g_csr_offset;
                memcpy(g_csr_ptr + g_csr_offset, data, length);
                g_csr_offset += length;
            }
            else if (type == (USER_DATA_TYPE_WDMA0_CSR + i))
            {
                wdma_fw->csr_num = (uint32_t)(length/8);
                wdma_fw->csr_data = g_csr_entry + g_csr_offset;
                memcpy(g_csr_ptr + g_csr_offset, data, length);
                g_csr_offset += length;
            }
        }
    }
}

static void wdma_st_gen(ns(uvp_package_table_t) uvp)
{
    wdma_core_t *wdma_core;
    uint32_t i;
    for (i = 0; i < 8; i++)
    {
        wdma_core = g_info_ptr + g_info_offset;
        memset(wdma_core, 0, sizeof(wdma_core_t));
        wdma_core->core_id = 0;
        wdma_core->fw_num = 0;
        wdma_core->fw_data = 0;
        *((uint32_t *)(g_info_ptr + sizeof(model_info_t) + i * sizeof(uint32_t))) = g_info_entry + g_info_offset;
        g_info_offset += sizeof(wdma_core_t);
    }
}

static void lmcu_info_gen(ns(uvp_package_table_t) uvp)
{
    uint32_t i, j;
    uint32_t data_num;
    uint64_t dst_addr;
    uint32_t length;
    uint8_t *data;
    layer_csr_lmcu_t *layer_csr_lmcu;
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    g_dlc_flag = 0;
    for (i = 0; i < 8; i++)
    {
        layer_csr_lmcu = g_info_ptr + g_info_offset;
        *((uint32_t *)(g_info_ptr + sizeof(model_info_t) + (8 * 2 + i) * sizeof(uint32_t))) = g_info_entry + g_info_offset;
        for (j = 0; j < g_layer_num; j++)
        {
            ns(layer_table_t) layer = ns(layer_vec_at(layers, j));
            ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
            memset(layer_csr_lmcu, 0, sizeof(layer_csr_lmcu_t));
            layer_csr_lmcu->layer_id = j;
            ns(user_data_table_t) user_data = model_layer_find(data_list, oak_typ_dlc, i, 1);
            if (user_data == NULL)
            {
                goto next_layer;
            }
            g_dlc_flag |= (0x1 << i);
            dst_addr = ns(user_data_dst_addr(user_data));
            length = ns(user_data_length(user_data));
            data = ns(user_data_data(user_data));
            layer_csr_lmcu->opcode_num = (uint32_t)(length/8);
            layer_csr_lmcu->opcode_data = g_csr_entry + g_csr_offset;
            memcpy(g_csr_ptr+g_csr_offset, data, length);
            g_csr_offset += length;
            user_data = model_layer_find(data_list, oak_typ_dlc, i, 0);
            if (user_data == NULL)
            {
                goto next_layer;
            }
            dst_addr = ns(user_data_dst_addr(user_data));
            length = ns(user_data_length(user_data));
            data = ns(user_data_data(user_data));
            layer_csr_lmcu->dlc_num = (uint32_t)(length/8);
            layer_csr_lmcu->dlc_data = g_csr_entry + g_csr_offset;
            memcpy(g_csr_ptr+g_csr_offset, data, length);
            g_csr_offset += length;
        next_layer:
            layer_csr_lmcu += 1;
            g_info_offset += sizeof(layer_csr_lmcu_t);
        }
    }
}

void odsp_info_gen(ns(uvp_package_table_t) uvp)
{
    int i, j;
    uint32_t data_num;
    uint32_t length;
    uint8_t *data;
    layer_csr_dsp_t *layer_csr_dsp;
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    g_odsp_flag = 0;
    for (i = 0; i < 8; i++)
    {
        layer_csr_dsp = g_info_ptr + g_info_offset;
        *((uint32_t *)(g_info_ptr + sizeof(model_info_t) + (8 * 3 + i) * sizeof(uint32_t))) = g_info_entry + g_info_offset;
        for (j = 0; j < g_layer_num; j++)
        {
            ns(layer_table_t) layer = ns(layer_vec_at(layers, j));
            ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
            memset(layer_csr_dsp, 0, sizeof(layer_csr_dsp_t));
            layer_csr_dsp->layer_id = j;
            ns(user_data_table_t) user_data = model_layer_find(data_list, oak_typ_odsp, i, 1);
            if (user_data == NULL)
            {
                goto next_layer;
            }
            g_odsp_flag |= (0x1 << i);
            length = ns(user_data_length(user_data));
            data = ns(user_data_data(user_data));
            layer_csr_dsp->opcode_num = (uint32_t)(length/8);
            layer_csr_dsp->opcode_data = g_csr_entry + g_csr_offset;
            memcpy(g_csr_ptr + g_csr_offset, data, length);
            g_csr_offset += length;
            user_data = model_layer_find(data_list, oak_typ_odsp, i, 0);
            if (user_data == NULL)
            {
                goto next_layer;
            }
            length = ns(user_data_length(user_data));
            data = ns(user_data_data(user_data));
            layer_csr_dsp->odsp_op_num = 1;
            layer_csr_dsp->odsp_op_data = g_csr_entry + g_csr_offset;
            memcpy(g_csr_ptr + g_csr_offset, data, length);
            g_csr_offset += length;
        next_layer:
            layer_csr_dsp += 1;
            g_info_offset += sizeof(layer_csr_dsp_t);
        }
    }
    printf("odsp flag 0x%x.\n", g_odsp_flag);
}

static void model_gen_done()
{
    int i;
    int dlc_num = 0;
    int odsp_num = 0;
    for (i = 0; i < 8; i++)
    {
        if (g_dlc_flag >> i & 0x1)
        {
            dlc_num++;
        }
        if (g_odsp_flag >> i & 0x1)
        {
            odsp_num++;
        }
    }
    ((model_info_t *)g_info_ptr)->dlc_num = dlc_num;
    ((model_info_t *)g_info_ptr)->odsp_num = odsp_num;
}

void model_gen(ns(uvp_package_table_t) uvp)
{
    model_gen_start(uvp);
    model_info_gen();
    cmcu_info_gen(uvp);
    wdma_info_gen(uvp);
    wdma_st_gen(uvp);
    lmcu_info_gen(uvp);
    odsp_info_gen(uvp);
    model_gen_done();
    printf("model gen ok.\n");
}

void model_free()
{
    free(g_info_ptr);
    free(g_csr_ptr);
    g_info_size = 0;
    g_csr_size = 0;
}
