#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "uvp_builder.h"
#include "uvp_defines.h"
#include "gen_model.h"
#include "run_model.h"

extern uint32_t g_layer_num;
extern int fd;

int set_input(ns(uvp_package_table_t) uvp)
{
    int i, j;
    int data_num;
    uint64_t dst_addr;
    uint32_t length;
    uint16_t type;
    uint8_t *data;

    if (fd < 0)
    {
	return -1;
    }
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    for (i = 0; i < g_layer_num; i++)
    {
        ns(layer_table_t) layer = ns(layer_vec_at(layers, i));
	ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
	data_num = ns(user_data_vec_len(data_list));
	for (j = 0; j < data_num; j++)
	{
            ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
	    dst_addr = ns(user_data_dst_addr(user_data));
	    length = ns(user_data_length(user_data));
	    type = ns(user_data_type(user_data));
	    data = ns(user_data_data(user_data));
	    switch (type)
	    {
            case (USER_DATA_TYPE_IMAGE0)...(USER_DATA_TYPE_IMAGE0 + 7):
            case (USER_DATA_TYPE_WEIGHT0)...(USER_DATA_TYPE_WEIGHT0 + 7):
	    case (USER_DATA_TYPE_OAK0_SRC0_BRANCH0)...(USER_DATA_TYPE_OAK0_SRC0_BRANCH0 + 16 * 8 - 1):
	    case (USER_DATA_TYPE_OAK0_SRC1_BRANCH0)...(USER_DATA_TYPE_OAK0_SRC1_BRANCH0 + 16 * 8 - 1):
	    case (USER_DATA_TYPE_OAK0_LUT)...(USER_DATA_TYPE_OAK0_LUT + 7):
	    case (USER_DATA_TYPE_OAK0_SF)...(USER_DATA_TYPE_OAK0_SF + 7):
		pwrite(fd, data, length, dst_addr);
		break;
            default:
		break;
	    }
	}
    }
    ns(user_data_table_t) common_data = ns(uvp_package_common_data(uvp));
    data_num = ns(user_data_vec_len(common_data));
    for (i = 0; i < data_num; i++)
    {
        ns(user_data_table_t) user_data = ns(user_data_vec_at(common_data, i));
	dst_addr = ns(user_data_dst_addr(user_data));
	length = ns(user_data_length(user_data));
	type = ns(user_data_type(user_data));
	data = ns(user_data_data(user_data));
	switch (type)
	{
        case (COMMON_DATA_TYPE_WDMA0_FW_BIN)...(COMMON_DATA_TYPE_WDMA0_FW_BIN + 7):
            pwrite(fd, data, length, dst_addr);
	    break;
	default:
	    break;
	}
    }

    printf("set input ok.\n");
    return 0;
}

int run_model(ns(uvp_package_table_t) uvp)
{
    void *model_info_ptr;
    uint32_t model_info_size;
    void *model_csr_ptr;
    uint32_t model_csr_size;
    run_model_t run_model;
    get_desc_t out_desc;
    int ret = 0;
    memset(&run_model, 0, sizeof(run_model_t));
    memset(&out_desc, 0, sizeof(get_desc_t));
    run_model.model_addr = 0x8c58000;

    if (fd < 0)
    {
        return -1;
    }
    model_info_ptr = get_model_info();
    model_info_size = get_model_info_size();
    model_csr_ptr = get_model_csr();
    model_csr_size = get_model_csr_size();
    pwrite(fd, model_info_ptr, model_info_size, 0x8c58000);
    pwrite(fd, model_csr_ptr, model_csr_size, 0x811600000);
    ret = ioctl(fd, VACC_RUN_MODEL, &run_model);
    if (ret < 0)
    {
        printf("ioctl error.\n");
	goto out;
    }
    ret = ioctl(fd, VACC_GET_OUT_DESC, &out_desc);
    if (ret < 0)
    {
        printf("ioctl error.\n");
	goto out;
    }
    printf("run_model end.\n");
out:
    return ret;
}

int get_output(ns(uvp_package_table_t) uvp, char *resultPath)
{
    int i, j, k;
    int data_num;
    uint32_t length;
    uint16_t type;
    uint8_t *data;
    uint32_t csr_addr;
    uint32_t output_addr_l, output_addr_h;
    uint64_t output_addr[8];
    uint32_t output_length[8];
    uint8_t *output_data;
    char fileName[128];
    FILE *fp;
    int dlc_idx;
    int dlc_num = get_dlc_num();

    if (fd < 0)
    {
	return -1;
    }

    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    i = 0;
    while (i < g_layer_num)
    {
        ns(layer_table_t) layer = ns(layer_vec_at(layers, i));
        ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
        data_num = ns(user_data_vec_len(data_list));
	memset(output_addr, 0, sizeof(uint64_t) * 8);
        for (j = 0; j < data_num; j++)
        {
            ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
	    type = ns(user_data_type(user_data));
            switch (type)
            {
            case (USER_DATA_TYPE_REFERENCE0)...(USER_DATA_TYPE_REFERENCE0 + 7):
                dlc_idx = type - USER_DATA_TYPE_REFERENCE0;
                output_length[dlc_idx] = ns(user_data_length(user_data));
                break;
            case (USER_DATA_TYPE_OAK0_CSR)...(USER_DATA_TYPE_OAK0_CSR + 7):
                dlc_idx = type - USER_DATA_TYPE_OAK0_CSR;
                length = ns(user_data_length(user_data));
                data = ns(user_data_data(user_data));
		output_addr_l = 0 ; 
		output_addr_h = 0 ; 
                for (k = 0; k < (uint32_t)(length / 8); k++)
	        {
                    csr_addr = data[k * 8 + 0] | ((uint32_t)data[k * 8 + 1] << 8) | ((uint32_t)data[k * 8 + 2] << 16) | ((uint32_t)data[k * 8 + 3] << 24);
		    if (csr_addr == 0x701804 + 0x100000 * dlc_idx)
		    {
		        output_addr_h = data[k * 8 + 4] | ((uint32_t)data[k * 8 + 5] << 8) | ((uint32_t)data[k * 8 + 6] << 16) | ((uint32_t)data[k * 8 + 7] << 24);
		    }
		    else if (csr_addr == 0x701808 + 0x100000 * dlc_idx)
		    {
		        output_addr_l = data[k * 8 + 4] | ((uint32_t)data[k * 8 + 5] << 8) | ((uint32_t)data[k * 8 + 6] << 16) | ((uint32_t)data[k * 8 + 7] << 24);
		    }
                }
                //if ((dlc_idx == 0) && (output_addr_h & 0xf) == 0)
                //{
                //    printf("dlc %d layer %d exp data in ssram.\n", dlc_idx, i);
                //    goto nex_layer;
                //}
	        output_addr[dlc_idx] = output_addr_l | ((uint64_t)(output_addr_h & 0xf) << 32);
		break;
            }
        }
        for (j = 0; j < dlc_num; j++)
        {
	    if (((output_addr[j] >> 32) & 0xf) == 0)//(output_addr[j] == 0)
            {
		printf("warning no need to read layer%d dlc%d.\n", i, j);
		continue;
	    }
            output_data = (uint8_t *)malloc(output_length[j]);
	    printf("read addr=0x%llx, size=%d.\n", output_addr[j], output_length[j]);
            pread(fd, output_data, output_length[j], output_addr[j]);
            sprintf(fileName, "%s/output_%d_%d.bin", resultPath, i, j);
            fp = fopen(fileName, "wb+");
            fwrite(output_data, 1, output_length[j], fp);
            fclose(fp);
            free(output_data);
            output_data = NULL;
        }
nex_layer:
        i++;
    }

    printf("get output ok.\n");
    return 0;
}

int compare_result(ns(uvp_package_table_t) uvp, char *resultPath)
{
    int i, j, k;
    int data_num;
    uint32_t length;
    uint16_t type;
    uint8_t *data;
    uint8_t *reference_data;
    FILE *fp = NULL;
    char fileName[128];
    int dlc_num = get_dlc_num();
    ns(layer_vec_t) layers = ns(uvp_package_layers(uvp));
    i = 0;
    while (i < g_layer_num)
    {
        sprintf(fileName, "%s/output_%d_0.bin", resultPath, i);
        //if (access(fileName, F_OK) != 0)
        //{
        //    printf("no need to compare layer %d.\n", i);
        //    goto nex_layer;
        //}
        ns(layer_table_t) layer = ns(layer_vec_at(layers, i));
        ns(user_data_vec_t) data_list = ns(layer_data_list(layer));
        data_num = ns(user_data_vec_len(data_list));
        for (j = 0; j < data_num; j++)
        {
	    ns(user_data_table_t) user_data = ns(user_data_vec_at(data_list, j));
	    type = ns(user_data_type(user_data));
            if ((type >= USER_DATA_TYPE_REFERENCE0) && (type <= (USER_DATA_TYPE_REFERENCE0 + 7)))
	    {
                length = ns(user_data_length(user_data));
	        data = ns(user_data_data(user_data));
	        sprintf(fileName, "%s/output_%d_%d.bin", resultPath, i, type - USER_DATA_TYPE_REFERENCE0);
		if (access(fileName, F_OK) != 0)
		{
		    printf("warning %s not exist.\n", fileName);
		    continue;
		}
                fp = fopen(fileName, "rb");
	        fseek(fp, 0, SEEK_SET);
	        reference_data = (uint8_t *)malloc(length);
	        fread(reference_data, 1, length, fp);
	        for (k = 0; k < length; k++)
	        {
		    if (data[k] != reference_data[k])
		    {
                        printf("layer %d dlc %d compare failed.\n", i, type - USER_DATA_TYPE_REFERENCE0);
		        fclose(fp);
	                free(reference_data);
                        return -1;
		    }
	        }
	        fclose(fp);
	        free(reference_data);
                printf("layer %d dlc %d compared ok.\n", i, type - USER_DATA_TYPE_REFERENCE0);
            }
        }
nex_layer:
    i++;
    }
    printf("compare pass.\n");
    return 0;
}
