#include <string.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "vastai_export_enum_define.h"
#include "vastai_pcie_user_api.h"

/************************************************
test config
SHA2: Support sha2 256,sha2 512
AES: ECB,CBC,CTR,GCM
ECC: 256 enc,256 dec,256 sign,256 verfity
SM4: ECB,CBC,CTR,GCM
DEC: ECB,CBC
************************************************/
#define TEST_SHA2_256_AND_512
#define TEST_AES_ALL
#define TEST_ECC_ALL
#undef TEST_HASH_ALL
#undef TEST_SM4_ALL
#undef TEST_DEC_ALL

int dev_id = 0;

#define CEDAR_ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))

int StringToHex(char *str, unsigned char *out, unsigned int *outlen)
{
	char *p = str;
	char high = 0, low = 0;
	int tmplen = strlen(p), cnt = 0;
	tmplen = strlen(p);
	while (cnt < (tmplen / 2)) {
		high = ((*p > '9') && ((*p <= 'F') || (*p <= 'f'))) ?
			*p - 48 - 7 :
			*p - 48;
		low = (*(++p) > '9' && ((*p <= 'F') || (*p <= 'f'))) ?
			*(p)-48 - 7 :
			*(p)-48;
		out[cnt] = ((high & 0x0f) << 4 | (low & 0x0f));
		p++;
		cnt++;
	}
	if (tmplen % 2 != 0)
		out[cnt] = ((*p > '9') && ((*p <= 'F') || (*p <= 'f'))) ?
			*p - 48 - 7 :
			*p - 48;

	if (outlen != NULL)
		*outlen = tmplen / 2 + tmplen % 2;
	return tmplen / 2 + tmplen % 2;
}

int check_padding_data(int data, int img_type)
{
	int i;
	for (i = 0; i < (1088 + 544); i++) {
		if ((data >= (240 * (i + 1) + i * 16)) &&
		    (data < (240 * (i + 1) + (i + 1) * 16)))
			return 0xff;
		if ((img_type == 0) && ((data >= 276480) && (data < 278528)))
			return 0xff;
	}
	return 0;
}

int sha3_mem_init(int dev_id, int die_id)
{
	const unsigned long long input_addr2 = 0x841500000;
	int ret = 0;
	unsigned char sha3_input[32] = {

		0xFF, 0xFF, 0xFF, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0x72, 0x03, 0xDF, 0x6B, 0x21, 0xC6, 0x05, 0x2B,
		0x53, 0xBB, 0xF4, 0x09, 0x39, 0xD5, 0x41, 0x23
	};

	ret = vastai_pcie_write_reg_buf(input_addr2, (void *)sha3_input,
					sizeof(sha3_input), dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	return ret;
}

int mem_init(int dev_id, int die_id)
{
	const unsigned long long input_addr1 = 0x841000000;
	const unsigned long long input_addr2 = 0x841500000;
	const unsigned long long input_addr3 = 0x841a00000;
	int i = 0;
	int j = 0;
	int k = 0;
	int ret = 0;
	int sts = 0;

#if 0
        for (i = 0; i < ((1920 * (1088 + 544)) >> 3); i++) {
                ret = vastai_pcie_write_reg_buf((input_addr1 + i * 8),
                                                (void *)&i, 8, dev_id, die_id);
                if (ret != 0) {
                        printf("ERROR: %s write reg failed, errcode %d\n",
                               __FUNCTION__, ret);
                        return -1;
                }
        }
#endif
	for (i = 0; i < ((2048 * (1088 + 544)) >> 3); i++) {
		sts = check_padding_data(i, 0);
		if (sts)
			continue;
		ret = vastai_pcie_write_reg_buf((input_addr1 + i * 8),
						(void *)&j, 8, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",
			       __FUNCTION__, ret);
			return -1;
		}
		j++;
	}
	j = 0;
	for (i = 0; i < ((2048 * (1088 + 544)) >> 3); i++) {
		sts = check_padding_data(i, 0);
		if (sts)
			continue;
		ret = vastai_pcie_write_reg_buf((input_addr2 + i * 8),
						(void *)&j, 8, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",
			       __FUNCTION__, ret);
			return -1;
		}
		j++;
	}
	for (i = 0; i < ((2048 * (1088 + 544)) >> 3); i++) {
		sts = check_padding_data(i, 1);
		if (sts)
			continue;
		ret = vastai_pcie_write_reg_buf((input_addr3 + i * 8),
						(void *)&k, 8, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",
			       __FUNCTION__, ret);
			return -1;
		}
		k++;
	}
	return ret;
}

char *s_key_8bytes_sym = "C73C3A08C475C8AD";
char *s_key_16bytes_sym = "BB66B3C4D0A6EB8351420A34464C0E7D";
char *s_key_24bytes_sym = "C73C3A08C475C8AD9B182734536EB92BC6F13304E90A874D";
char *s_key_32bytes_sym = "50B168CC9DD767A64118BAD4E6882C5EE475910197EBB22BDCE8731A2620E39B";
char *s_64bytes_data_sym = "EE87D1305545FFA9CD97230C1BAD24B3898CC9CFBB2A89C16416942868A7528C"
"C9319EB51482DF3E1A4A860CECF627278FEF7861D5BF303464074B06CC1821E5";
char *cbc_iv_in_sym = "9264D24A6AC5013B98077AA241437B4E";
char *ctr_iv_in_sym = "B54A1A3D23EA35501F870DE850F0D967";
char *gcm_iv_in_sym = "B54A1A3D23EA35501F870DE8";
char *des_cbc_iv_sym = "D4F7480B52252EB3";
char *gcm_msg_in_sym = "00112233445566778899AABBCCDDEEFF";


//char *s_128bytes_data_sym = "093182B7EEAEEED416E7B55DA15829B6093182B7EEAEEED416E7B55DA15829B6"
//							"11DED1CF1417CCAAAF1DC1BB782C70BB4A1D82AE7C2CE44F108C93FCBCFEA7CA"
//							"CF1417CCAAAF1DC1BB782C70BB4A1D82AE7C2CF1417CCAAAF1DC1BB782C70BB4"
//							"A1D82AE7C2CF1417CCAAAF1DC1BB782C70BB4A1D82AE7C2CF1417CCAAAF1DC1B";
char *aes_ecb_64bytes_key_16bytes_res = "B763D1109C45558953213061B00484E8807B105743F1D40D1C9D055F67D8B911"
"BCDB72DE6C1688341E3A9BE3E0B3ED4BC94CF21300BDA89F6BFA9E4F57E0467D";
char *aes_ecb_64bytes_key_24bytes_res = "F044E8F72A922C2FEDB00EE505B6C2A89608E28F6F80CA42671BE567985C357D"
"4563B3CB61D1C948C69EE3ACB6F7F5B5740BD39607C82C669DD89AC36CE5FE8A";
char *aes_ecb_64bytes_key_32bytes_res = "50D1872A98171EEB2EC83F7BC4BDC2FF43958CED7B1B52A75C378EA4927ABB65"
"163F5881A9A5C79779D0E800DF824ED7B956F67F7F6809765366D0AA23EC08D4";
char *aes_cbc_64bytes_key_16bytes_res = "E4EB3401F087B06B031B05C6A2672D6F643798532679867E834E3153EB673CD4"
"63E971B8121C1372A964CEF8FBEE1477781A46DFC6D6FD995A81F9A71923C256";
char *aes_cbc_64bytes_key_24bytes_res = "6BF3B8F19909105062D85EB26ED0DF95BB93A76F712706C5BD9E76E4FB3E4B26"
"A116D751AAE43A17D6F5B88B4EBABC4B84FC077FD42E4B1C2A24FE30E8CC7611";
char *aes_cbc_64bytes_key_32bytes_res = "2B8258ED5E7FBD45CB2B5033B6C385808F740990E21C68487815FB96A57A65D6"
"F519318E723E1A9D21DED105D90E366AA67CDD6E8027A3A0F7E91FD1DA72D548";
char *aes_ctr_64bytes_key_16bytes_res =	"A5C796F3714C47F1803083274FDCEB58E4A6E03CA7BEC57FEDAB9FE1588B7120"
"A9871080E0712E5A83ED572F0F87BF8A559FD59B9B9CF4D4BEC5C54B8C7C1390";
char *aes_ctr_64bytes_key_24bytes_res =	"9A9A5E237871A7B4855AA3EDC045862BBEFF5CBFD1C07743575424CFE278E324"
"02F46F4CEBC36B18BB3F430DAC3A4FB275E94ED2D40BBB6F69F2232C39FCE5E3";
char *aes_ctr_64bytes_key_32bytes_res = "7D4D0D405F42BA059DC517A0D4B58265C435D9D1810D2F4479C84925A9A6B7E2"
"B5AB14D1EC5C00D74AC8164F009E648986DB396666F1FAA3CEE26ACB03E41550";
char *aes_gcm_64bytes_key_16bytes_res ="3BE87F7E5DFC273CADFC048BD19809C829C286149F2569F806EFB72F1651CD0F"
"6190D5A45170FBD1E27001B87215648F8DFD5A927429E9982AEEFCA6D86235C7";
char *aes_gcm_64bytes_key_16bytes_tag = "FCAFFE6F3349FFC235BD83C1B3DF69BF";
char *aes_gcm_64bytes_key_24bytes_res ="BCC841E8AC9FBD0A459E5AE374AA94C48F9925D632F63A902D07C4FB02E8818F"
"1BAB03016C1BE158271843EE6F98290ABD1185DDEF0A98E439962BBACF8AC643";
char *aes_gcm_64bytes_key_24bytes_tag = "ECD5969A10A1664C89682A11BC5B1330";
char *aes_gcm_64bytes_key_32bytes_res ="A34BE47EAC5D0040C17398F60C714E0292245CF282D5DBBD694077E4615840AF"
"D747380C2DBBB07B7895EA5CD83250DB3B26516B210DCF1BFB0F96C4E1E34BF8";
char *aes_gcm_64bytes_key_32bytes_tag = "BD047FC30DF07537693572E91BCD6710";

#define AES_NORMAL_MODE                                         (0)//
#define AES_MASK_MODE                                           (1)//
/*
const unsigned long long input_addr 	= 0x1040000000;	//6M
const unsigned long long output_addr	= 0x8E4600;
const unsigned long long input_key_addr = 0x8E4400;
const unsigned long long input_iv_addr = 0x8E4430; 
const unsigned long long input_msg_addr = 0x8E4450;
const unsigned long long input_tag_addr = 0x8E4470;
*/
#define CSRAM_OFFSET (0x400)

//pass
const unsigned long long input_addr0		= 0x1040000000;
const unsigned long long output_addr0	= 0x8E4000;
const unsigned long long input_key_addr0	= 0x8E4200;
const unsigned long long input_iv_addr0	= 0x8E4230;
const unsigned long long input_msg_addr0	= 0x8E4250;
const unsigned long long input_tag_addr0	= 0x8E4270;
const unsigned long long input_prikey_addr0	= 0x8E4290;
const unsigned long long input_rand_addr0	= 0x8E42C0;
const unsigned long long input_pubkey_addr0	= 0x8E42F0;
const unsigned long long input_sign_addr0	= 0x8E4330;
const unsigned long long input_hash_addr0	= 0x8E4370;
const unsigned long long ecc_hash_verify0= 0x8E43F0;

unsigned int sym_mode = AES_NORMAL_MODE ;
unsigned char sym_data[64]= {0};
unsigned char sym_res[64]= {0};
unsigned char sym_std[64]= {0};
unsigned char sym_key[32]= {0};
unsigned char sym_iv[16]= {0};
unsigned char sym_msg[16]= {0};
unsigned char sym_tag_res[16]= {0};

int ret = 0;

u8 sha2_256_fail =0xFF;
u8 sha2_512_fail =0xFF;
u8 aes_ecb_fail =0xFF;
u8 aes_cbc_fail =0xFF;
u8 aes_ctr_fail =0xFF;
u8 aes_gcm_fail =0xFF;
u8 ecc_enc_fail =0xFF;
u8 ecc_dec_fail =0xFF;
u8 ecc_sign_fail =0xFF;
u8 ecc_verify_fail =0xFF;
u8 sm4_ecb_fail =0xFF;
u8 sm4_cbc_fail =0xFF;
u8 sm4_ctr_fail =0xFF;
u8 sm4_gcm_fail =0xFF;
u8 hash_224_fail =0xFF;
u8 hash_384_fail =0xFF;
u8 hash_sm3_fail =0xFF;
char  sha2_256_result_str[256]   = "SHA-256";
char  sha2_512_result_str[256]   = "SHA-512";
char  aes_ecb_result_str[256]   = "AES ECB";
char  aes_cbc_result_str[256]   = "AES CBC";
char  aes_ctr_result_str[256]   = "AES CTR";
char  aes_gcm_result_str[256]   = "AES GCM";
char  ecc_enc_result_str[256]  = "ECC-256 ENC";
char  ecc_dec_result_str[256]   = "ECC-256 DEC";
char  ecc_sign_result_str[256]   = "ECC-256 SIGN";
char  ecc_verify_result_str[256]   = "ECC-256 VERIFY";
char  sm4_ecb_result_str[256]   = "SM4 ECB";
char  sm4_cbc_result_str[256]   = "SM4 CBC";
char  sm4_ctr_result_str[256]   = "SM4 CTR";
char  sm4_gcm_result_str[256]   = "SM4 GCM";
char  hash_224_result_str[256]   = "SHA-224";
char  hash_384_result_str[256]   = "SHA-384";
char  hash_sm3_result_str[256]   = "SM3";

int pvp_test_aes_ecb(void)
{
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int i,j;
	u8 cnt = 0;

	unsigned long long input_key_addr;
	unsigned long long output_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id *CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	char *key_data[] = {s_key_16bytes_sym,s_key_24bytes_sym,s_key_32bytes_sym};
	char *res_std[] = {aes_ecb_64bytes_key_16bytes_res,aes_ecb_64bytes_key_24bytes_res,
		aes_ecb_64bytes_key_32bytes_res};
	char *mode[] = { "NORMAL", "MASK" };
	char *key_type_str[]={"KEY-128","KEY-192","KEY-256"};
	printf("Aes_ecb:\n inputdata: \n");
	for(i =0 ;i < sym_data_len;i++)
	{
		printf( "0x%x  ",sym_data[i]);
	}
	printf("\n");
	for(j = 0 ;j < 3;j++)
	{
		StringToHex(key_data[j], sym_key, &sym_keylen);
		StringToHex(res_std[j], sym_std, &sym_data_len);

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,
				sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
					ret);
			return -1;
		}

		#if 1
		printf("%s:\n" ,key_type_str[j]);
		printf("sym_keylen = %d\n",sym_keylen);
		for(i =0 ;i < sym_keylen;i++)
		{
			printf( "0x%x  ",sym_key[i]);
		}
		printf("\n");
		#endif

		for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
		{
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,
				sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
						ret);
				return -1;
			}

			if (ce_aes_ecb((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, AES_ECB_ENC, dev_id,DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
		#if 1
			printf("sym_res: \n");
			for(i = 0;i<sym_data_len;i++)
			{
				printf("0x%x ",sym_res[i]);
			}
			printf("\n");
		#endif
			if (memcmp(sym_res, sym_std, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_ecb_enc,%s mode fail!\n",mode[sym_mode]);
			} else {
				printf("[Resport]aes_ecb_enc,%s mode succ!\n",mode[sym_mode]);
			}
			//aes ecb dec test
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,
				sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
						ret);
				return -1;
			}

			if (ce_aes_ecb((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, AES_ECB_DEC,dev_id, DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			if (memcmp(sym_res, sym_data, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_ecb_dec,%s mode fail!\n",mode[sym_mode]);

			} else {
				printf("[Resport] aes_ecb_dec,%s mode succ!\n",mode[sym_mode]);
			}
			if(cnt !=0)
				aes_ecb_fail = 1;
			else
				aes_ecb_fail = 0;
		}
	}
	return 0;
}

int pvp_test_aes_cbc(void)
{
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int i,j;
	u8 cnt = 0;
	unsigned long long input_key_addr;
	unsigned long long	input_iv_addr;
	unsigned long long output_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id *CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 +dev_id*CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id *CSRAM_OFFSET;

	char *mode[] = { "NORMAL", "MASK" };

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(cbc_iv_in_sym, sym_iv, &ivlen);
	char *key_data[] = {s_key_16bytes_sym,s_key_24bytes_sym,s_key_32bytes_sym};
	char *res_std[] = {aes_cbc_64bytes_key_16bytes_res,aes_cbc_64bytes_key_24bytes_res,
		aes_cbc_64bytes_key_32bytes_res};
	char *key_type_str[]={"KEY-128","KEY-192","KEY-256"};
	printf("Aes_cbc:\n inputdata: \n");
	for(i =0 ;i < sym_data_len;i++)
	{
		printf( "0x%x  ",sym_data[i]);
	}
	printf("\nIv:\n");
	for(i = 0;i< ivlen;i++)
	{
		printf("0x%x ",sym_iv[i]);
	}
	printf("\n");
	for(j = 0 ;j < 3; j++)
	{
		StringToHex(key_data[j], sym_key, &sym_keylen);
		StringToHex(res_std[j], sym_std, &sym_data_len);

		#if 1
		printf("%s:\n" ,key_type_str[j]);
		printf("sym_keylen = %d\n",sym_keylen);
		for(i =0 ;i < sym_keylen;i++)
		{
			printf( "0x%x  ",sym_key[i]);
		}
		printf("\n");
		#endif
		for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
		{
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
						ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
					ret);
				return -1;
			}

			if (ce_aes_cbc((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, (u64)input_iv_addr, AES_CBC_ENC,dev_id,
						DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			#if 1
			printf("sym_res: \n");
			for(i = 0;i<64;i++)
			{
				printf("0x%x ",sym_res[i]);
			}
			printf("\n");
			#endif
			if (memcmp(sym_res, sym_std, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_cbc_enc,%s mode fail!\n", mode[sym_mode]);
			} else {
				printf("[Resport] aes_cbc_enc,%s  mode succ!\n", mode[sym_mode]);
			}
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
						ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
					ret);
				return -1;
			}
			if (ce_aes_cbc((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, (u64)input_iv_addr, AES_CBC_DEC,dev_id,
						DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			if (memcmp(sym_res, sym_data, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_cbc_dec,%s mode fail!\n", mode[sym_mode]);

			} else {
				printf("[Resport] aes_cbc_dec,%s  mode  succ!\n", mode[sym_mode]);
			}

			if(cnt != 0)
				aes_cbc_fail = 1;
			else
				aes_cbc_fail = 0;
		}
	}
	return 0;
}

int pvp_test_aes_ctr(void)
{
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int i,j;
	u8 cnt = 0;
	char *mode[] = { "NORMAL", "MASK" };

	unsigned long long	output_addr;
	unsigned long long	input_key_addr;
	unsigned long long	input_iv_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 +dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 +dev_id*CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 +dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(ctr_iv_in_sym, sym_iv, &ivlen);
	char *key_data[] = {s_key_16bytes_sym,s_key_24bytes_sym,s_key_32bytes_sym};
	char *res_std[] = {aes_ctr_64bytes_key_16bytes_res,aes_ctr_64bytes_key_24bytes_res,
		aes_ctr_64bytes_key_32bytes_res};
	char *key_type_str[]={"KEY-128","KEY-192","KEY-256"};
	printf("Aes_ctr:\n inputdata: \n");
	for(i =0 ;i < sym_data_len;i++)
	{
		printf( "0x%x  ",sym_data[i]);
	}
	printf("\nIv:\n");
	for(i = 0;i< ivlen;i++)
	{
		printf("0x%x ",sym_iv[i]);
	}
	printf("\n");
	for(j = 0 ;j < 3; j++)
	{
		StringToHex(key_data[j], sym_key, &sym_keylen);
		StringToHex(res_std[j], sym_std, &sym_data_len);

		#if 1
		printf("%s:\n" ,key_type_str[j]);
		printf("sym_keylen = %d\n",sym_keylen);
		for(i =0 ;i < sym_keylen;i++)
		{
			printf( "0x%x  ",sym_key[i]);
		}
		printf("\n");
		#endif
		for(sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
		{
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}
			if (ce_aes_ctr((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, (u64)input_iv_addr, AES_CTR_ENC,dev_id,
						DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			#if 1
			printf("sym_res: \n");
			for(i = 0;i<sym_data_len;i++)
			{
				printf("0x%x ",sym_res[i]);
			}
			printf("\n");
			#endif
			if (memcmp(sym_res, sym_std, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_ctr_enc,%s mode fail!\n", mode[sym_mode]);
			} else {
				printf("[Resport] aes_ctr_enc,%s  mode succ!\n",mode[sym_mode]);
			}
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}
			if (ce_aes_ctr((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, (u64)input_iv_addr, AES_CTR_DEC,dev_id,
						DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}

			if (memcmp(sym_res, sym_data, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_ctr_dec,%s mode fail!\n", mode[sym_mode]);

			} else {
				printf("[Resport] aes_ctr_dec,%s  mode  succ!\n", mode[sym_mode]);
			}
			if(cnt != 0)
				aes_ctr_fail = 1;
			else
				aes_ctr_fail = 0;
		}
	}
	return 0;
}

int pvp_test_aes_gcm(void)
{
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int msglen = 0;
	unsigned int tblen = 0;
	unsigned int i,j;
	u8 cnt  = 0;
	u8 tag[16] = { 0 };
	char *mode[] = { "NORMAL", "MASK" };
	unsigned long long	output_addr;
	unsigned long long	input_key_addr;
	unsigned long long	input_iv_addr;
	unsigned long long	input_msg_addr;
	unsigned long long	input_tag_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 +dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 +dev_id*CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 +dev_id*CSRAM_OFFSET;
	input_msg_addr = input_msg_addr0 +dev_id*CSRAM_OFFSET;
	input_tag_addr = input_tag_addr0 +dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(gcm_iv_in_sym, sym_iv, &ivlen);
	StringToHex(gcm_msg_in_sym, sym_msg, &msglen);
	char *key_data[] = {s_key_16bytes_sym,s_key_24bytes_sym,s_key_32bytes_sym};
	char *res_std[] = {aes_gcm_64bytes_key_16bytes_res,aes_gcm_64bytes_key_24bytes_res,
		aes_gcm_64bytes_key_32bytes_res};
	char *tag_std[] = {aes_gcm_64bytes_key_16bytes_tag,aes_gcm_64bytes_key_24bytes_tag,
		aes_gcm_64bytes_key_32bytes_tag};
	char *key_type_str[]={"KEY-128","KEY-192","KEY-256"};
	printf("Aes_gcm:\ninputdata: \n");
	for(i =0 ;i < sym_data_len;i++)
	{
		printf( "0x%x  ",sym_data[i]);
	}
	printf("\nIv:\n");
	for(i = 0;i < ivlen;i++)
	{
		printf( "0x%x  ",sym_iv[i]);
	}
	printf("\nMsg:\n");
	for(i = 0;i < msglen;i++)
	{
		printf( "0x%x  ",sym_msg[i]);
	}
	printf("\n");
	for(j = 0 ;j < 3; j++)
	{
		StringToHex(key_data[j], sym_key, &sym_keylen);
		StringToHex(res_std[j], sym_std, &sym_data_len);
		StringToHex(tag_std[j], sym_tag_res, &tblen);

		#if 1
		printf("%s:\n" ,key_type_str[j]);
		printf("sym_keylen = %d .\n",sym_keylen);
		for(i =0 ;i < sym_keylen;i++)
		{
			printf( "0x%x  ",sym_key[i]);
		}
		printf("\n");
		#endif
		for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
		{
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d.\n",
						__FUNCTION__, ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_msg_addr, (void *)sym_msg, msglen,dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}			
			if (ce_aes_gcm((u64)input_addr, (u64)output_addr,
						sym_data_len, (u64)input_key_addr, sym_keylen,
						sym_mode, (u64)input_iv_addr,ivlen,
						(u64)input_msg_addr, msglen,(u64)input_tag_addr, tblen,
						AES_GCM_ENC,dev_id,DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			#if 1
			printf("sym_res: \n");
			for(i = 0;i<sym_data_len;i++)
			{
				printf("0x%x ",sym_res[i]);
			}
			printf("\n");
			#endif
			if (memcmp(sym_res, sym_std, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_gcm_enc,%s mode fail!\n", mode[sym_mode]);
			} else {
				printf("[Resport] aes_gcm_enc,%s  mode succ!\n", mode[sym_mode]);
			}
			ret = vastai_pcie_read_reg_buf(input_tag_addr, (void *)tag,
					tblen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			printf("\ngcm_tag : \n");
			for(i =0 ;i < tblen;i++)
			{
				printf( "0x%x  ",tag[i]);
			}
			printf("\n");
			if (memcmp(tag, sym_tag_res, tblen)){
				cnt++;
				printf("[Resport] aes_gcm_enc tag ,%s mode fail!\n",mode[sym_mode]);
			}else{
				printf("[Resport] aes_gcm _enc tag,%s  mode  succ!\n",mode[sym_mode]);
			}
			//dec
			ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}

			ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen, dev_id,DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}
			ret = vastai_pcie_write_reg_buf(input_msg_addr, (void *)sym_msg, msglen,dev_id, DIE_0);
			if (ret != 0) {
				printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
				return -1;
			}			

			if (ce_aes_gcm((u64)input_addr, (u64)output_addr,sym_data_len,
						(u64)input_key_addr, sym_keylen,sym_mode,
						(u64)input_iv_addr,ivlen, (u64)input_msg_addr, msglen,
						(u64)input_tag_addr, tblen, AES_GCM_DEC,dev_id,DIE_0))
				return 0xff;
			ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,
					sym_data_len,dev_id , DIE_0);
			if (ret != 0) {
				printf("ERROR: %s read reg failed, errcode %d\n",
						__FUNCTION__, ret);
				return -1;
			}
			if (memcmp(sym_res, sym_data, sym_data_len)) {
				cnt++;
				printf("[Resport] aes_gcm_dec,%s mode fail!\n", mode[sym_mode]);

			} else {
				printf("[Resport] aes_gcm_dec,%s  mode  succ!\n", mode[sym_mode]);
			}
			if(cnt != 0)
				aes_gcm_fail = 1;
			else
				aes_gcm_fail = 0;
		}
	}
	return 0;
}

int General_security_alg(void)
{
	int ret = 0;
	int i = 0 ,round = 0;
	unsigned char hash_res[128] = { 0 } ;
	//int inputdatalen = 128;
	unsigned long long	output_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 +dev_id*CSRAM_OFFSET;

	const unsigned char input_128bytes[128]=
	{
		0x09, 0x31, 0x82, 0xB7, 0xEE, 0xAE, 0xEE, 0xD4,
		0x16, 0xE7, 0xB5, 0x5D, 0xA1, 0x58, 0x29, 0xB6,
		0x09, 0x31, 0x82, 0xB7, 0xEE, 0xAE, 0xEE, 0xD4,
		0x16, 0xE7, 0xB5, 0x5D, 0xA1, 0x58, 0x29, 0xB6,
		0x11, 0xDE, 0xD1, 0xCF, 0x14, 0x17, 0xCC, 0xAA,
		0xAF, 0x1D, 0xC1, 0xBB, 0x78, 0x2C, 0x70, 0xBB,
		0x4A, 0x1D, 0x82, 0xAE, 0x7C, 0x2C, 0xE4, 0x4F,
		0x10, 0x8C, 0x93, 0xFC, 0xBC, 0xFE, 0xA7, 0xCA,
		0xCF, 0x14, 0x17, 0xCC, 0xAA, 0xAF, 0x1D, 0xC1,
		0xBB, 0x78, 0x2C, 0x70, 0xBB, 0x4A, 0x1D, 0x82,
		0xAE, 0x7C, 0x2C, 0xF1, 0x41, 0x7C, 0xCA, 0xAA,
		0xF1, 0xDC, 0x1B, 0xB7, 0x82, 0xC7, 0x0B, 0xB4,
		0xA1, 0xD8, 0x2A, 0xE7, 0xC2, 0xCF, 0x14, 0x17,
		0xCC, 0xAA, 0xAF, 0x1D, 0xC1, 0xBB, 0x78, 0x2C,
		0x70, 0xBB, 0x4A, 0x1D, 0x82, 0xAE, 0x7C, 0x2C,
		0xF1, 0x41, 0x7C, 0xCA, 0xAA, 0xF1, 0xDC, 0x1B
	};

	const unsigned char sha2_256_128bytes_output_golden[32]=
	{
		0xC4, 0x52, 0x32, 0x14, 0x6B, 0x46, 0x8D, 0x5F,
		0x61, 0x3F, 0xD5, 0x70, 0x74, 0xCC, 0xB1, 0xD6,
		0xFD, 0x7E, 0xE4, 0xA1, 0x9F, 0xB3, 0xB1, 0xCD,
		0x0F, 0x6D, 0xC6, 0xF2, 0x2F, 0x85, 0xC7, 0x81
	};

	const unsigned char  sha2_512_128bytes_output_golden[64]=
	{
		0xBB, 0x66, 0x9D, 0x41, 0x56, 0x61, 0x82, 0x9E,
		0x6C, 0x6E, 0x76, 0xCC, 0x7E, 0x4F, 0xBE, 0x86,
		0x52, 0xF0, 0x7E, 0xD4, 0x3C, 0x3B, 0xC4, 0xA3,
		0xE3, 0x05, 0x4A, 0x90, 0xA4, 0x7A, 0xC4, 0xCE,
		0x2D, 0x4F, 0x96, 0x8F, 0x95, 0x84, 0xD7, 0x9C,
		0x1A, 0x8B, 0x46, 0x9A, 0x34, 0x11, 0x03, 0xC6,
		0xA9, 0x11, 0x77, 0x33, 0x87, 0x9D, 0x51, 0x52,
		0xC6, 0xEC, 0xAD, 0x8F, 0x14, 0x9C, 0xE3, 0x3B
	};

	ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)input_128bytes,
			sizeof(input_128bytes), dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
				ret);
		return -1;
	}

	/* sha2_256*/
	round = 32;
	if(ce_sha2((u64)input_addr,(u64)output_addr,sizeof(input_128bytes),SHA2_256,dev_id,DIE_0)){
		return 0xff;
	}
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)hash_res, round, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	printf("sha2_256:\n  input data:\n");
	for(i = 0; i < 128;i++)
	{
		printf("0x%x ",input_128bytes[i]);
	}
	printf("\n sha2_256 hash value:\n");
	for(i = 0; i < round; i++)
	{
		printf("0x%x ",hash_res[i]);
	}
	if(memcmp(hash_res,sha2_256_128bytes_output_golden,round)){
		sha2_256_fail =1;
		printf("\n [Resport] dev%d die%d sha2_256 fail!\n",dev_id,DIE_0);
	}else{
		sha2_256_fail =0;
		printf("\n [Resport] dev%d die%d sha2_256  pass!\n",dev_id,DIE_0);
	}

	/* sha2_512*/
	round = 64;
	if(ce_sha2((u64)input_addr,(u64)output_addr,sizeof(input_128bytes),SHA2_512,dev_id,DIE_0))
		return 0xff;
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)hash_res, round, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	printf("sha2_512:\n  input data:\n");
	for(i = 0; i < 128;i++)
	{
		printf("0x%x ",input_128bytes[i]);
	}
	printf("\n sha2_512 hash value:\n");
	for(i = 0; i < round; i++)
	{
		printf("0x%x ", hash_res[i]);
	}
	if(memcmp(hash_res,sha2_512_128bytes_output_golden,round)){
		sha2_512_fail =1;
		printf("\n [Resport] dev%d die%d sha2_512 fail!\n",dev_id,DIE_0);
	}else{
		sha2_512_fail =0;
		printf("\n [Resport] dev%d die%d sha2_512  pass!\n",dev_id,DIE_0);
	}
	return ret;
}

int vastai_pvp_aes_test(void)
{
	int res = 0;
	res = pvp_test_aes_ecb();
	if(res != 0)
	{
		return res;
	}
	res = pvp_test_aes_cbc();
	if(res != 0)
	{
		return res;
	}
	res = pvp_test_aes_ctr();
	if(res != 0)
	{
		return res;
	}
	res = pvp_test_aes_gcm();
	if(res != 0)
	{
		return res;
	}
	return 0;
}

int vastai_pvp_ecc_test(void)
{
	char  *inputMsg   = NULL;
	char  *inputRand  = NULL;
	char  *pubkey   = NULL;
	char  *golden_res_char   = NULL;

	unsigned char  Hex_inputMsg[512] = {0};
	unsigned int ByteNum = 0;
	unsigned char  outputMsg[512] = {0};
	unsigned int blen = 32;
	unsigned char Hex_inputRand[48] = {0};
	unsigned char Hex_pubkey[96] = {0};
	unsigned char golden_res[512] = {0};
	unsigned int i;
	unsigned long long input_rand_addr;
	unsigned long long input_pubkey_addr;
	unsigned long long output_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	input_rand_addr = input_rand_addr0 + dev_id*CSRAM_OFFSET;//input_key_addr0
	input_pubkey_addr = input_pubkey_addr0 + dev_id*CSRAM_OFFSET;//input_iv_addr0
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;

	//ecc 256 enc
	blen = 32;
	inputMsg =  "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123";
	inputRand=  "BBDDD05D270A7610F0E43CC5999183F23ECC5EB70623FE3116C52D0864864C01";
	pubkey = "4AEDEECB75171A00AAEED60058046F46E1B7D2A2A0DBF44D5C1BF16E62A2DCFC"
		"D167750F85D38CB69C9A16EA40DC9047979A8A1CC4195FC8B7036034A0CF586F";
	golden_res_char ="3468B0084D607B31F5E928FAAAF1D978B334008D6C2E9FCBAE6329DE4795C32C"
		"960F07BE96B460103662D0616DDAD031723718E11DF914C8E45F897ED0252512"
		"4EF99ADBCAD3FBA4A370DB3827401AA07E47D7C5FC094C359F8EA111F6918D4C"
		"1FB9F2CF4850DFEDC240BF9856393302FBE80C573AD4BB1A274B42D94212CDFF";
	StringToHex(inputMsg,Hex_inputMsg,&ByteNum);

	printf("Ecc 256 enc :\nplaintext data:\n");
	for(i = 0 ;i < ByteNum;i++)
	{
		printf("0x%x ",Hex_inputMsg[i]);
	}
	printf("\n");
	ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)Hex_inputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(inputRand,Hex_inputRand,&ByteNum);
	printf("random data:\n");
	for(i = 0 ;i < ByteNum;i++)
	{
		printf("0x%x ",Hex_inputRand[i]);
	}
	printf("\n");
	ret = vastai_pcie_write_reg_buf(input_rand_addr, (void *)Hex_inputRand, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(pubkey,Hex_pubkey,&ByteNum);
	printf("pubkey:\n");
	for(i = 0 ;i < ByteNum;i++)
	{
		printf("0x%x ",Hex_pubkey[i]);
	}
	printf("\n");
	ret = vastai_pcie_write_reg_buf(input_pubkey_addr, (void *)Hex_pubkey, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	if(ce_ecc_enc((u64)input_addr, (u64)output_addr, blen, (u64)input_rand_addr, (u64)input_pubkey_addr,
				pka256bForEcc, ECC_ENC,dev_id, DIE_0))
		return 0xff;
	StringToHex(golden_res_char,golden_res,&ByteNum);
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)outputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test read reg failed, errcode %d\n", ret);
		return -1;
	}
#if 1
	printf("ciphertext:\n");
	for(i = 0;i < ByteNum;i++)
	{
		printf("0x%x ",outputMsg[i]);
	}
	printf("\n");
#endif
	if(memcmp(outputMsg,golden_res,ByteNum)){
		ecc_enc_fail = 1;
		printf("[Resport] pvp ecc_enc 256b fail !\n");
	}else{
		ecc_enc_fail = 0;
		printf("[Resport] pvp ecc_enc 256b pass !\n");
	}

	//ecc 256 dec
	char  *prikey  = NULL;
	unsigned char Hex_prikey[48] = {0};
	unsigned long long input_hash_addr;
	unsigned long long input_prikey_addr;

	input_hash_addr =input_hash_addr0+dev_id*CSRAM_OFFSET;
	input_prikey_addr = input_prikey_addr0+dev_id*CSRAM_OFFSET;

	blen = 128;
	inputMsg = 	"3468B0084D607B31F5E928FAAAF1D978B334008D6C2E9FCBAE6329DE4795C32C"
		"960F07BE96B460103662D0616DDAD031723718E11DF914C8E45F897ED0252512"
		"4EF99ADBCAD3FBA4A370DB3827401AA07E47D7C5FC094C359F8EA111F6918D4C"
		"1FB9F2CF4850DFEDC240BF9856393302FBE80C573AD4BB1A274B42D94212CDFF";
	prikey = "2C9F3E0B9382C501CC07229F52CBB1C8F0C337B74A3D8FF843DC505AC65B940F";
	golden_res_char = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123";
	StringToHex(inputMsg,Hex_inputMsg,&ByteNum);
	ret = vastai_pcie_write_reg_buf(input_hash_addr, (void *)Hex_inputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0){
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(prikey,Hex_prikey,&ByteNum);
	printf("Ecc 256 dec:\nprikey:\n");
	for(i = 0 ;i < ByteNum;i++)
	{
		printf("0x%x ",Hex_prikey[i]);
	}
	printf("\n");
	ret = vastai_pcie_write_reg_buf(input_prikey_addr, (void *)Hex_prikey, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	if(ce_ecc_dec((u64)input_hash_addr, (u64)output_addr, blen, (u64)input_prikey_addr, pka256bForEcc,
				ECC_DEC,dev_id, DIE_0))
		return 0xff;
	StringToHex(golden_res_char,golden_res,&ByteNum);
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)outputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test read reg failed, errcode %d\n", ret);
		return -1;
	}
	if(memcmp(outputMsg,golden_res,ByteNum)){
		ecc_dec_fail = 1;
		printf("[Resport] pvp ecc_dec 256b fail !\n");
	}else{
		ecc_dec_fail = 0;
		printf("[Resport] pvp ecc_dec 256b pass !\n");
	}

	//ecc 256 sign
	blen = 32;
	inputMsg = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123";
	prikey = "2C9F3E0B9382C501CC07229F52CBB1C8F0C337B74A3D8FF843DC505AC65B940F";
	inputRand= "A756FBE87979B22AEC10BDC3D2E96792D7E0671D4CA089FA28E939EA7D3C791D";
	golden_res_char = "D9363697BFF98DBB5815BBE6363809E1157E32CEEA7D04855EC322F60E9C7C19"
		                      "B54CABF4DDF0E92A983434BADE1483736091B7C9BA32F6ADAB85C6F5EF4E2413";
	StringToHex(inputMsg,Hex_inputMsg,&ByteNum);
	ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)Hex_inputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(inputRand,Hex_inputRand,&ByteNum);
	ret = vastai_pcie_write_reg_buf(input_rand_addr, (void *)Hex_inputRand, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(prikey,Hex_prikey,&ByteNum);
	ret = vastai_pcie_write_reg_buf(input_prikey_addr, (void *)Hex_prikey, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	if(ce_ecc_sign((u64)input_addr, (u64)output_addr, blen, (u64)input_rand_addr,
				(u64)input_prikey_addr, pka256bForEcc, ECC_SIGN, dev_id, DIE_0))
		return 0xff;
	StringToHex(golden_res_char,golden_res,&ByteNum);
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)outputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test read reg failed, errcode %d\n", ret);
		return -1;
	}
#if 1
	printf("ecc_256_sign:\nsignature:\n");
	for(i = 0;i < ByteNum;i++)
	{
		printf("0x%x ",outputMsg[i]);
	}
	printf("\n");
#endif
	if(memcmp(outputMsg,golden_res,ByteNum)){
		ecc_sign_fail = 1;
		printf("[Resport] pvp ecc_sign 256b fail !\n");
	}else{
		ecc_sign_fail = 0;
		printf("[Resport] pvp ecc_sign 256b pass !\n");
	}

	//ecc 256 verify
	char  *signature  = NULL;
	unsigned char Hex_signature[96] = {0};
	u32 resultok = 0xa5a5a5a5;
	u32 resultstatus = 0;
	unsigned long long input_sign_addr;
	unsigned long long ecc_hash_verify;

	input_sign_addr = input_sign_addr0+dev_id*CSRAM_OFFSET;
	ecc_hash_verify = ecc_hash_verify0+dev_id*CSRAM_OFFSET;
	blen = 32;
	signature = "D9363697BFF98DBB5815BBE6363809E1157E32CEEA7D04855EC322"
		"F60E9C7C19B54CABF4DDF0E92A983434BADE1483736091B7C9BA32F6ADAB85C6F5EF4E2413";
	inputMsg = "FFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFF7203DF6B21C6052B53BBF40939D54123";
	pubkey = "4AEDEECB75171A00AAEED60058046F46E1B7D2A2A0DBF44D5C1BF16E62A2DCFCD167750F8"
		"5D38CB69C9A16EA40DC9047979A8A1CC4195FC8B7036034A0CF586F";
	StringToHex(inputMsg,Hex_inputMsg,&ByteNum);
	ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)Hex_inputMsg, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(pubkey,Hex_pubkey,&ByteNum);
	ret = vastai_pcie_write_reg_buf(input_pubkey_addr, (void *)Hex_pubkey, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	StringToHex(signature,Hex_signature,&ByteNum);
	ret = vastai_pcie_write_reg_buf(input_sign_addr, (void *)Hex_signature, ByteNum, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	printf("ecc_256 verify:\n");

	if(ce_ecc_verify((u64)input_addr, (u64)input_sign_addr, blen, (u64)input_pubkey_addr,pka256bForEcc, ECC_VERIFY, dev_id, DIE_0))
		return 0xff;
	ret = vastai_pcie_read_reg_buf(ecc_hash_verify, (void *)&resultstatus,sizeof(resultstatus), dev_id, DIE_0);
	printf("AAA: pvp_ecc_test ecc_hash_verify=  0x%llx\n", ecc_hash_verify);
	printf("AAA: pvp_ecc_test resultstatus=  0x%x\n", resultstatus);

	if (ret != 0) {
		printf("ERROR: pvp_ecc_test write reg failed, errcode %d\n", ret);
		return -1;
	}
	if(resultstatus == resultok){
		ecc_verify_fail = 0;
		printf("[Resport] ecc_verify 256 pass!\n");
	}else{
		ecc_verify_fail = 1;
		printf("[Resport] ecc_verify 256 fail!\n");
	}

	return ret;
}

int cedar_hash_test(void)
{
	printf("\n-----Cedar Hash Test  Start-----\n");
	int ret = 0;
	int i = 0 ,round = 0;
	unsigned char hash_res[64] = { 0 } ;
	int inputdatalen = 128;
	unsigned long long output_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;

	const unsigned char input_128bytes[128]=
	{
		0x09, 0x31, 0x82, 0xB7, 0xEE, 0xAE, 0xEE, 0xD4,
		0x16, 0xE7, 0xB5, 0x5D, 0xA1, 0x58, 0x29, 0xB6,
		0x09, 0x31, 0x82, 0xB7, 0xEE, 0xAE, 0xEE, 0xD4,
		0x16, 0xE7, 0xB5, 0x5D, 0xA1, 0x58, 0x29, 0xB6,
		0x11, 0xDE, 0xD1, 0xCF, 0x14, 0x17, 0xCC, 0xAA,
		0xAF, 0x1D, 0xC1, 0xBB, 0x78, 0x2C, 0x70, 0xBB,
		0x4A, 0x1D, 0x82, 0xAE, 0x7C, 0x2C, 0xE4, 0x4F,
		0x10, 0x8C, 0x93, 0xFC, 0xBC, 0xFE, 0xA7, 0xCA,
		0xCF, 0x14, 0x17, 0xCC, 0xAA, 0xAF, 0x1D, 0xC1,
		0xBB, 0x78, 0x2C, 0x70, 0xBB, 0x4A, 0x1D, 0x82,
		0xAE, 0x7C, 0x2C, 0xF1, 0x41, 0x7C, 0xCA, 0xAA,
		0xF1, 0xDC, 0x1B, 0xB7, 0x82, 0xC7, 0x0B, 0xB4,
		0xA1, 0xD8, 0x2A, 0xE7, 0xC2, 0xCF, 0x14, 0x17,
		0xCC, 0xAA, 0xAF, 0x1D, 0xC1, 0xBB, 0x78, 0x2C,
		0x70, 0xBB, 0x4A, 0x1D, 0x82, 0xAE, 0x7C, 0x2C,
		0xF1, 0x41, 0x7C, 0xCA, 0xAA, 0xF1, 0xDC, 0x1B
	};
	const unsigned char  sha2_224_128bytes_output_golden[28]=
	{
		0x02, 0x7E, 0xBA, 0x20, 0x09, 0x0C, 0xE9, 0xDF,
		0xE8, 0xAA, 0x94, 0x4A, 0x59, 0x59, 0x78, 0xB6,
		0xDD, 0xA5, 0x7D, 0xBD, 0x1E, 0x55, 0x60, 0x5B,
		0x2B, 0x39, 0x44, 0xC8
	};
	const unsigned char  sha2_384_128bytes_output_golden[48]=
	{
		0xA6, 0x70, 0xB7, 0xC5, 0x97, 0xDF, 0x3E, 0xE1,
		0x01, 0x86, 0x3B, 0x53, 0x4F, 0x72, 0x0A, 0x41,
		0xA0, 0x52, 0x0C, 0x9E, 0x5E, 0x89, 0x9B, 0xF9,
		0x3F, 0x58, 0x4F, 0xD5, 0x7D, 0xF1, 0xC1, 0xBD,
		0x61, 0xDD, 0xDA, 0xA9, 0xA9, 0xEC, 0xB3, 0xB9,
		0xC8, 0xC0, 0xF7, 0x73, 0xD5, 0x22, 0x27, 0x85
	};
	const unsigned char sm3_128bytes_output_golden[32]=
	{
		0xD5, 0x0A, 0x9D, 0xCF, 0x41, 0xF7, 0x04, 0x98,
		0xD8, 0x2C, 0x2D, 0x9A, 0xB6, 0xBB, 0x05, 0xE0,
		0xF8, 0xE4, 0x64, 0x49, 0xD2, 0xDA, 0xAB, 0x83,
		0x77, 0xEE, 0xDE, 0x11, 0x0B, 0xD6, 0x63, 0xF9
	};

	ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)input_128bytes,
			sizeof(input_128bytes), dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
				ret);
		return -1;
	}
	/* sha2_224*/
	printf("input 128bytes data:\n");
	for(i = 0 ;i < inputdatalen;i++)
	{
		printf("0x%x ",input_128bytes[i]);
	}
	printf("\n");

	round = 28;
	if(ce_sha2((u64)input_addr,(u64)output_addr,inputdatalen,SHA2_224,dev_id,DIE_0))
		return 0xff;
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)hash_res, round, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	printf("\nhashdata :\n");
	for(i = 0; i < round; i++)
		printf("0x%x ", hash_res[i]);

	if(memcmp(hash_res,sha2_224_128bytes_output_golden,round)){
		hash_224_fail =1;
		printf("\n[Resport] dev%d die%d sha2_224 fail!\n",dev_id,DIE_0);
	}else{
		hash_224_fail =0;
		printf("\n[Resport] dev%d die%d sha2_224 pass!\n",dev_id,DIE_0);
	}

	/* sha2_384*/
	round = 48;
	if(ce_sha2((u64)input_addr,(u64)output_addr,inputdatalen,SHA2_384,dev_id,DIE_0))
		return 0xff;
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)hash_res, round, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}

	printf("\nhashdata :\n");
	for(i = 0; i < round; i++)
		printf("0x%x ",hash_res[i]);

	if(memcmp(hash_res,sha2_384_128bytes_output_golden,round)){
		hash_384_fail =1;
		printf("\n[Resport] dev%d die%d sha2_384 fail!\n",dev_id,DIE_0);
	}else{
		hash_384_fail =0;
		printf("\n[Resport] dev%d die%d sha2_384  pass!\n",dev_id,DIE_0);
	}

	/* sm3_256*/
	round = 32;
	if(ce_sm3((u64)input_addr,(u64)output_addr, inputdatalen, SM3,dev_id,DIE_0))
		return 0xff;
	ret = vastai_pcie_read_reg_buf(output_addr, (void *)hash_res, round, dev_id, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}

	printf("\nhashdata :\n");
	for(i = 0; i < round; i++)
		printf("0x%x ",hash_res[i]);

	if(memcmp(hash_res,sm3_128bytes_output_golden,round)){
		hash_sm3_fail =1;
		printf("\n[Resport] dev%d die%d sm3_256 fail!\n",dev_id,DIE_0);
	}else{
		hash_sm3_fail =0;
		printf("\n[Resport] dev%d die%d sm3_256  pass!\n",dev_id,DIE_0);
	}

	printf("\n-----Cedar Hash Test  End-----\n");
	return ret;
}

char *sm4_ecb_64bytes_key_16bytes_res =
"244A67C61A2A1A0EA7D6E00098647B182210B30D10B38DED5ECFA32AF1C7B7EC"
"75C54D3D99666B15D9AB6E0253F02E90BD30CDD2EB99815FA34E97A7D6FF705A";
char *sm4_cbc_64bytes_key_16bytes_res =
"1C821255FDB978437E3B2662C5DA5C733413AFDD281DD229E2B0C222346374A0"
"B1C27A91880D25DCC70A0313DB3D0129F02755D6EA8B41CD3BC8568215E0B259";
char *sm4_ctr_64bytes_key_16bytes_res =
"32B26EF91467F2E16D55391317C04693E027303C775F8E6C9ACE9E4A3F9C70C4"
"A511396C4D830B05D7B78A6E72DDAA5C6E3325111E0D581C9DA7CCE5F8073685";
char *sm4_gcm_64bytes_key_16bytes_res =
"F6CB52960E8D39546767BF0EDDFA28F6146D97628276AF2878C5469C21A64BBB"
"7CB244D84107965146AE4949435F845709DDCEAEDC21A49D6B1B494DB92BF434";
char *sm4_gcm_64bytes_key_16bytes_tag = "7D6A07889AAE8C01AD770769BD4612C6";

int test_sm4_ecb(void)
{
	printf("-----SM4 ECB TEST----- \n");
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int i;
	u8 cnt = 0;
	char *mode[] = { "Normal", "Mask" };
	unsigned long long input_addr;
	unsigned long long output_addr;
	unsigned long long input_key_addr;

	input_addr = input_addr0 + dev_id*CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(s_key_16bytes_sym, sym_key, &sym_keylen);
	StringToHex(sm4_ecb_64bytes_key_16bytes_res, sym_std, &sym_data_len);

	printf("enc sym_data: \n");
	for(i = 0;i<64;i++)
	{
		printf(" 0x%x",sym_data[i]);
	}
	printf("\n");

	for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
	{
		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if (ce_sm4_ecb((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode, SM4_ECB_ENC, dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("enc sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_std, sym_data_len)) {
			cnt++;
			printf("[Resport] sm4_ecb_enc,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Resport] sm4_ecb_enc,%s  mode  succ!\n", mode[sym_mode]);
		}

		//sm4 dec
		printf("dec sym_std: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_std[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if (ce_sm4_ecb((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode, SM4_ECB_DEC,dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("dec sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_data, sym_data_len)) {
			cnt++;
			printf("[Resport] sm4_ecb_dec,%s mode fail!\n", mode[sym_mode]);

		} else {
			printf("[Resport] sm4_ecb_dec,%s  mode  succ!\n", mode[sym_mode]);
		}
	}
	if(cnt !=0)
		sm4_ecb_fail = 1;
	else
		sm4_ecb_fail = 0;

	return 0;
}

int test_sm4_cbc(void)
{
	printf("-----SM4 CBC TEST----- \n");
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int i;
	u8 cnt = 0;
	char *mode[] = { "Normal", "Mask" };
	unsigned long long output_addr;
	unsigned long long input_key_addr;
	unsigned long long input_iv_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id*CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 + dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(s_key_16bytes_sym, sym_key, &sym_keylen);
	StringToHex(sm4_cbc_64bytes_key_16bytes_res, sym_std, &sym_data_len);
	StringToHex(cbc_iv_in_sym, sym_iv, &ivlen);

	for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
	{
		printf("enc input sym_data: \n");
		for(i = 0;i<64;i++)
		{
			printf(" %x",sym_data[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if (ce_sm4_cbc((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode, (u64)input_iv_addr, SM4_CBC_ENC, dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("enc output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_std, sym_data_len)) {
			cnt++;
			printf("[Resport] sm4_cbc_enc,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Resport] sm4_cbc_enc,%s  mode  succ!\n", mode[sym_mode]);
		}

		//dec cbc
		printf("dec input sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_std[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if(ce_sm4_cbc((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode,(u64)input_iv_addr, SM4_CBC_DEC,dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("dec output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_data, sym_data_len)) {
			cnt++;
			printf("[Resport] sm4_cbc_dec,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Resport] sm4_cbc_dec,%s  mode  succ!\n", mode[sym_mode]);
		}
	}
	if(cnt !=0)
		sm4_cbc_fail = 1;
  else
		sm4_cbc_fail = 0;

	return 0;
}

int test_sm4_ctr(void)
{
	printf("-----SM4 CTR TEST----- \n");
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int i;
	u8 cnt = 0;
	char *mode[] = { "Normal", "Mask" };
	unsigned long long output_addr;
	unsigned long long input_key_addr;
	unsigned long long input_iv_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id*CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 + dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(s_key_16bytes_sym, sym_key, &sym_keylen);
	StringToHex(sm4_ctr_64bytes_key_16bytes_res, sym_std, &sym_data_len);
	StringToHex(ctr_iv_in_sym, sym_iv, &ivlen);

	for (sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
	{
		printf("enc input sym_data: \n");
		for(i = 0;i<64;i++)
		{
			printf(" %x",sym_data[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if (ce_sm4_ctr((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode, (u64)input_iv_addr, SM4_CTR_ENC, dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("enc output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_std, sym_data_len)) {
			cnt++;
			printf("[Report] sm4_ctr_enc,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Report] sm4_ctr_enc,%s  mode  succ!\n", mode[sym_mode]);
		}
		//DEC CTR
		printf("dec input sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_std[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if(ce_sm4_ctr((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode,(u64)input_iv_addr, SM4_CTR_DEC,dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("dec output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_data[i]);
		}
		printf("\n");

		if (memcmp(sym_res, sym_data, sym_data_len)) {
			cnt++;
			printf("[Resport] sm4_ctr_dec,%s mode fail!\n", mode[sym_mode]);

		} else {
			printf("[Resport] sm4_ctr_dec,%s  mode  succ!\n", mode[sym_mode]);
		}
	}
	if(cnt !=0)
		sm4_ctr_fail = 1;
	else
		sm4_ctr_fail = 0;

	return 0;
}

int test_sm4_gcm(void)
{
	printf("-----SM4 GCM TEST----- \n");
	unsigned int sym_data_len = 0;
	unsigned int sym_keylen = 0;
	unsigned int ivlen = 0;
	unsigned int msglen = 0;
	unsigned int tblen = 0;
	u8 tag[16] = { 0 };
	unsigned int i;
	u8 cnt = 0;
	char *mode[] = { "Normal", "Mask" };
	unsigned long long output_addr;
	unsigned long long input_key_addr;
	unsigned long long input_iv_addr;
	unsigned long long input_msg_addr;
	unsigned long long input_tag_addr;
	unsigned long long input_addr;

	input_addr = input_addr0 + dev_id *CSRAM_OFFSET;
	output_addr = output_addr0 + dev_id*CSRAM_OFFSET;
	input_key_addr = input_key_addr0 + dev_id*CSRAM_OFFSET;
	input_iv_addr = input_iv_addr0 + dev_id*CSRAM_OFFSET;
	input_msg_addr = input_msg_addr0 + dev_id*CSRAM_OFFSET;
	input_tag_addr = input_tag_addr0 + dev_id*CSRAM_OFFSET;

	StringToHex(s_64bytes_data_sym, sym_data, &sym_data_len);
	StringToHex(s_key_16bytes_sym, sym_key, &sym_keylen);
	StringToHex(sm4_gcm_64bytes_key_16bytes_res, sym_std, &sym_data_len);
	StringToHex(gcm_iv_in_sym, sym_iv, &ivlen);
	StringToHex(gcm_msg_in_sym, sym_msg, &msglen);
	StringToHex(sm4_gcm_64bytes_key_16bytes_tag, sym_tag_res, &tblen);

	for(sym_mode = 0; sym_mode < 2;sym_mode++) //0:AES_NORMAL_MODE ;1:AES_MASK_MODE
	{
		//enc
		printf("enc input sym_data: \n");
		for(i = 0;i<64;i++)
		{
			printf(" %x",sym_data[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_data,sym_data_len, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_msg_addr, (void *)sym_msg, msglen,dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if(ce_sm4_gcm((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode, (u64)input_iv_addr,ivlen, (u64)input_msg_addr, msglen,
					(u64)input_tag_addr, tblen,SM4_GCM_ENC, dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("enc output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if(memcmp(sym_res, sym_std, sym_data_len)) {
			cnt++;
			printf("[Report] sm4_gcm_enc,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Report] sm4_gcm_enc,%s  mode  succ!\n", mode[sym_mode]);
		}

		ret = vastai_pcie_read_reg_buf(input_tag_addr, (void *)tag,tblen, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("gcm_tag start: \n");
		for(i =0 ;i < tblen;i++)
		{
			printf( "tag[%d] =  0x%x  ",i,tag[i]);
		}
		printf("\nsym_tag finished!\n");

		if(memcmp(tag, sym_tag_res, tblen)){
			cnt++;
			printf("[Report] sm4_gcm_enc tag ,%s mode fail!\n",mode[sym_mode]);
		}else{
			printf("[Report] sm4_gcm _enc tag,%s  mode  succ!\n",mode[sym_mode]);
		}
		//gcm dec
		printf("dec input sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_std[i]);
		}
		printf("\n");

		ret = vastai_pcie_fwrite_reg_buf(input_addr, (void *)sym_std,sym_data_len, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_key_addr, (void *)sym_key,sym_keylen, dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_iv_addr, (void *)sym_iv, ivlen,dev_id,DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		ret = vastai_pcie_write_reg_buf(input_msg_addr, (void *)sym_msg, msglen,dev_id, DIE_0);
		if(ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,ret);
			return -1;
		}

		if(ce_sm4_gcm((u64)input_addr, (u64)output_addr,
					sym_data_len, (u64)input_key_addr, sym_keylen,
					sym_mode,(u64)input_iv_addr, ivlen,
					(u64)input_msg_addr, msglen,(u64)input_tag_addr, tblen,
					SM4_GCM_DEC,dev_id, DIE_0))
			return 0xff;

		ret = vastai_pcie_read_reg_buf(output_addr, (void *)sym_res,sym_data_len, dev_id, DIE_0);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n",__FUNCTION__, ret);
			return -1;
		}

		printf("dec output sym_res: \n");
		for (i = 0; i < 64; i++) {
			printf("0x%x ", sym_res[i]);
		}
		printf("\n");

		if(memcmp(sym_res, sym_data, sym_data_len)){
			cnt++;
			printf("[Report] sm4_gcm_dec,%s mode fail!\n", mode[sym_mode]);
		} else {
			printf("[Report] sm4_gcm_dec,%s  mode  succ!\n", mode[sym_mode]);
		}
	}
	if(cnt !=0)
		sm4_gcm_fail = 1;
	else
		sm4_gcm_fail = 0;

	return 0;
}

int cedar_sm4_test(void)
{
	int ret = 0;

	printf("\n-----Cedar SM4 Test  Start-----\n");

	ret = test_sm4_ecb();
	if(ret != 0)
	{
		return ret;
	}
	ret = test_sm4_cbc();
	if(ret != 0)
	{
		return ret;
	}
	ret = test_sm4_ctr();
	if(ret != 0)
	{
		return ret;
	}
	ret = test_sm4_gcm();
	if(ret != 0)
	{
		return ret;
	}
	printf("\n-----Cedar SM4 Test  End-----\n");
	return 0;
}

typedef int (*test_fn)(void);

test_fn test_case_table[] = {
	
#ifdef TEST_SHA2_256_AND_512
	General_security_alg,
#endif
#ifdef TEST_AES_ALL
	vastai_pvp_aes_test,
#endif
#ifdef TEST_ECC_ALL
	vastai_pvp_ecc_test,
#endif 
#ifdef TEST_HASH_ALL
	cedar_hash_test,
#endif
#ifdef TEST_SM4_ALL
	cedar_sm4_test,
#endif
};

int main(int argc, char *argv[])
{
	int i,ret = 0;

	if (1 == argc){
		dev_id = 0;
	}else if (argc == 2){
		dev_id = atoi(argv[1]);
	}else{
		printf("ERROR:cedar param error! \n");
		return -1;
	}

	for (i = 0; i < (int)CEDAR_ARRAY_SIZE(test_case_table); i++) {
		ret = test_case_table[i]();
		if (ret) {
			printf("test failed case_id[%d]\n", i);
		}
	}

	if (!ret) {
		printf("cedar test success!\n");
	}

	printf("-----------------------------------------------------------------------------\n\n");
	printf("*********************************TEST RESULT*********************************\n\n");

	printf("******PASS : ");
#ifdef TEST_SHA2_256_AND_512
	if(sha2_256_fail == 0)
		printf("%s  ",sha2_256_result_str);
	if(sha2_512_fail == 0)
		printf("%s  ",sha2_512_result_str);
#endif

#ifdef TEST_AES_ALL
	if(aes_ecb_fail == 0)
		printf("%s  ",aes_ecb_result_str);
	if(aes_cbc_fail == 0)
		printf("%s  ",aes_cbc_result_str);
	if(aes_ctr_fail == 0)
		printf("%s  ",aes_ctr_result_str);
	if(aes_gcm_fail == 0)
		printf("%s  ",aes_gcm_result_str);
#endif

#ifdef TEST_ECC_ALL
	if(ecc_enc_fail == 0)
		printf("%s  ",ecc_enc_result_str);
	if(ecc_dec_fail == 0)
		printf("%s  ",ecc_dec_result_str);
	if(ecc_sign_fail == 0)
		printf("%s  ",ecc_sign_result_str);
	if(ecc_verify_fail == 0)
		printf("%s  ",ecc_verify_result_str);
#endif

#ifdef TEST_SM4_ALL
	if(sm4_ecb_fail == 0)
		printf("%s  ",sm4_ecb_result_str);
	if(sm4_cbc_fail == 0)
		printf("%s  ",sm4_cbc_result_str);
	if(sm4_ctr_fail == 0)
		printf("%s  ",sm4_ctr_result_str);
	if(sm4_gcm_fail == 0)
		printf("%s  ",sm4_gcm_result_str);
#endif

#ifdef TEST_HASH_ALL
	if(hash_224_fail == 0)
		printf("%s  ",hash_224_result_str);
	if(hash_384_fail == 0)
		printf("%s  ",hash_384_result_str);
	if(hash_sm3_fail == 0)
		printf("%s  ",hash_sm3_result_str);
#endif

	printf("\n");
	printf("******FAIL : ");
#ifdef TEST_SHA2_256_AND_512
	if(sha2_256_fail == 1)
		printf("%s  ",sha2_256_result_str);
	if(sha2_512_fail == 1)
		printf("%s  ",sha2_512_result_str);
#endif

#ifdef TEST_AES_ALL
	if(aes_ecb_fail == 1)
		printf("%s  ",aes_ecb_result_str);
	if(aes_cbc_fail == 1)
		printf("%s  ",aes_cbc_result_str);
	if(aes_ctr_fail == 1)
		printf("%s  ",aes_ctr_result_str);
	if(aes_gcm_fail == 1)
		printf("%s  ",aes_gcm_result_str);
#endif

#ifdef TEST_ECC_ALL
	if(ecc_enc_fail == 1)
		printf("%s  ",ecc_enc_result_str);
	if(ecc_dec_fail == 1)
		printf("%s  ",ecc_dec_result_str);
	if(ecc_sign_fail == 1)
		printf("%s  ",ecc_sign_result_str);
	if(ecc_verify_fail == 1)
		printf("%s  ",ecc_verify_result_str);
#endif

#ifdef TEST_SM4_ALL
	if(sm4_ecb_fail == 1)
		printf("%s  ",sm4_ecb_result_str);
	if(sm4_cbc_fail == 1)
		printf("%s  ",sm4_cbc_result_str);
	if(sm4_ctr_fail == 1)
		printf("%s  ",sm4_ctr_result_str);
	if(sm4_gcm_fail == 1)
		printf("%s  ",sm4_gcm_result_str);
#endif

#ifdef TEST_HASH_ALL
	if(hash_224_fail == 1)
		printf("%s  ",hash_224_result_str);
	if(hash_384_fail == 1)
		printf("%s  ",hash_384_result_str);
	if(hash_sm3_fail == 1)
		printf("%s  ",hash_sm3_result_str);
#endif

	printf("\n");
	printf("******Running Fault: ");
 #ifdef TEST_SHA2_256_AND_512
	if(sha2_256_fail == 0xFF)
		printf("%s  ",sha2_256_result_str);
	if(sha2_512_fail == 0xFF)
		printf("%s  ",sha2_512_result_str);
#endif

#ifdef TEST_AES_ALL
	if(aes_ecb_fail == 0xFF)
		printf("%s  ",aes_ecb_result_str);
	if(aes_cbc_fail == 0xFF)
		printf("%s  ",aes_cbc_result_str);
	if(aes_ctr_fail == 0xFF)
		printf("%s  ",aes_ctr_result_str);
	if(aes_gcm_fail == 0xFF)
		printf("%s  ",aes_gcm_result_str);
#endif

#ifdef TEST_ECC_ALL
	if(ecc_enc_fail == 0xFF)
		printf("%s  ",ecc_enc_result_str);
	if(ecc_dec_fail == 0xFF)
		printf("%s  ",ecc_dec_result_str);
	if(ecc_sign_fail == 0xFF)
		printf("%s  ",ecc_sign_result_str);
	if(ecc_verify_fail == 0xFF)
		printf("%s  ",ecc_verify_result_str);
#endif

#ifdef TEST_SM4_ALL
	if(sm4_ecb_fail == 0xFF)
		printf("%s  ",sm4_ecb_result_str);
	if(sm4_cbc_fail == 0xFF)
		printf("%s  ",sm4_cbc_result_str);
	if(sm4_ctr_fail == 0xFF)
		printf("%s  ",sm4_ctr_result_str);
	if(sm4_gcm_fail == 0xFF)
		printf("%s  ",sm4_gcm_result_str);
#endif

#ifdef TEST_HASH_ALL
	if(hash_224_fail == 0xFF)
		printf("%s  ",hash_224_result_str);
	if(hash_384_fail == 0xFF)
		printf("%s  ",hash_384_result_str);
	if(hash_sm3_fail == 0xFF)
		printf("%s  ",hash_sm3_result_str);
#endif

	printf("\n\n");
	printf("-----------------------------------------------------------------------------\n\n");
	return ret;
}
