#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

typedef unsigned int u32;
typedef unsigned long long int u64;

#define VASTAI_PCI_IOCTL_DMA_TRANS 3

#define ALIGN_SIZE (sysconf(_SC_PAGESIZE))
#define ALIGN(x, a)	(((x) + ((a) - 1)) & ~((a) - 1))
#define ALIGN_ADDR(x, a) (void *)ALIGN((unsigned long)(x), a)

struct va_ioctk_kchar_cmd {
	union {
		struct {
			u32 size;
			int dma_buf_fd;
		} alloc_cmd;

		struct {
			u32 is_dev_to_host;
			u32 dma_buf_fd;
			u64 axi_addr;
			u32 length;
			u32 die_index;
		} dma_start_cmd;
		struct {
			u32 is_dev_to_host;
			void * vir_addr;
			u64 axi_addr;
			u32 length;
			u32 die_index;
			int pid;
		} dma_trans_cmd;
		struct {
			u32 is_dev_to_host;
			u32 die_index;
			u64 axi_addr;
			struct vastai_channel_buf *channel_buf;
			int channel_num;
			int pid;
		} dma_trans_sg_cmd;
		struct {
			u32 die_index;
			u32 timeout_water;
			u32 is_timeout;
			void *log_ring_ctl;
		} log_poll_cmd;
	};
};

struct vastai_channel_buf {
	void *user_buf;
	u32 width;
	u32 high;
	u32 src_width_offset;
	u32 src_high_offset;
	u32 dst_width_offset;
	u32 dst_high_offset;
};

union die_index_data{
	u32 val;
	struct {
		u32 die_id:8;
		u32 dev_id:16;
		u32 RESV:8;
	};
};

#define IMAGE_WITH 16384
#define IMAGE_HIGH 16384
#define IMAGE_SIZE (IMAGE_WITH*IMAGE_HIGH)
#define IMAGE_WITH_PAD (0)
#define IMAGE_HIGH_PAD (0)
#define IMAGE_CHN 3

static int open_vastai_ctl(int dev_id)
{
	char file_path[64] = {0};
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/va%d_ctl", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128]={0};
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!\n", file_path);
		perror(temp_buf);
	}
	return filep;
}

static int vastai_trigger_dma_by_user_addr(void *user_addr, u64 axi_addr, u32 length, u32 is_dev_to_host, u32 die_index)
{
	union die_index_data temp = { .val = die_index };
	int vastai_fd = open_vastai_ctl(temp.dev_id);
	struct va_ioctk_kchar_cmd kcmd = { .dma_trans_cmd.is_dev_to_host = is_dev_to_host,
				  .dma_trans_cmd.vir_addr = user_addr,
				  .dma_trans_cmd.axi_addr = axi_addr,
				  .dma_trans_cmd.length = length,
				  .dma_trans_cmd.die_index = die_index,};
	if (vastai_fd < 0) {
		printf("open kchar is failed %d\n", vastai_fd);
		return vastai_fd;
	}
	int ret = ioctl(vastai_fd, VASTAI_PCI_IOCTL_DMA_TRANS, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_DMA_TRANS is failed %d\n", ret);
	}
	close(vastai_fd);
	return ret;
}
#define test_size (256*1024*1024)

int dma_copy_uaddr_test(void)
{
	int ret;
	void *mem;
	void *r_mem;

	mem = malloc(test_size + ALIGN_SIZE);
	r_mem = malloc(test_size + ALIGN_SIZE);
	if (!mem) {
		printf("malloc err\n");
		return -1;
	}
	printf("mem:%p mem:%lx,%lx\n", mem, ALIGN((unsigned long)mem, ALIGN_SIZE), (unsigned long)mem);
	#if 1
	memset((void *)ALIGN((unsigned long)mem, ALIGN_SIZE), 0xaa, test_size);
	ret = vastai_trigger_dma_by_user_addr(ALIGN_ADDR(mem, ALIGN_SIZE), 0x840000000, test_size, 0, 0);
	if (ret) {
		printf("va_trigger_dma_user_addr err %d\n", ret);
	}
	ret = vastai_trigger_dma_by_user_addr(ALIGN_ADDR(r_mem, ALIGN_SIZE), 0x840000000, test_size, 1, 0);
	if (ret) {
		printf("va_trigger_dma_user_addr err %d\n", ret);
	}
	for (unsigned long i = 0; i < test_size; i++) {
		if (((char *)(ALIGN_ADDR(r_mem, ALIGN_SIZE)))[i] != (char)0xaa) {
			printf("\t\tdata is not match\n");
			printf("i:%ld,mem:%x,val:%x\n", i, ((char *)r_mem)[i], 0xaa);
			break;
		}
	}
	#endif
	free(mem);
	free(r_mem);
	return 0;
}
int main(void)
{
#if 1
	for (int i = 0; i < 100; i++)
			dma_copy_uaddr_test();
#endif
	return 0;
}


