#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <signal.h>
#include "../include/vastai_pcie_user_api.h"

void print_warnning(void)
{
	int i;
	printf("WARNNING: Will start pcie performance test.\n");
	printf("WARNNING: Please stop all data transfer for va sv100.\n");
	printf("WARNNING: You can use ctrl-c to stop this test.\n");
	for (i = 5; i > 0; i--) {
		printf("\rwait %ds...", i);
		fflush(stdout);
		sleep(1);
	}
	printf("\n");
}

#define DEVICE_TABLE_SIZE 0x1000
#define TEST_BUF_LEN 0x400000
#define VASTAI_PCI_IOCTL_ALLOC 0
struct {
	int dma_fd;
	int device_num;
	struct vastai_dev_info dev_info[0];
}* device_table;
int dma_fd[64];

struct va_ioctk_kchar_cmd {
	union {
		struct {
			u32 size;
			int dma_buf_fd;
			u64 dma_addr_t;
		} alloc_cmd;
	};
};

static int open_pcie_kchar(int dev_id)
{
	char file_path[64] = {0};
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/kchar:%d", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128]={0};
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!\n", file_path);
		perror(temp_buf);
	}
	return filep;
}

static int vastai_alloc_dma_buf_phy(int size, u32 die_index, u64 *dma_addr_t)
{
	union die_index_data temp = { .val = die_index };
	int kchar_fd = open_pcie_kchar(temp.dev_id);
	struct va_ioctk_kchar_cmd kcmd = { .alloc_cmd.size = size, .alloc_cmd.dma_buf_fd = 0 };
	if (kchar_fd < 0) {
		printf("open kchar is failed %d\n", kchar_fd);
		return kchar_fd;
	}
	
	int ret = ioctl(kchar_fd, VASTAI_PCI_IOCTL_ALLOC, &kcmd);
	if (ret < 0) {
		printf("ioctl is failed %d\n", ret);
	} else {
		ret = kcmd.alloc_cmd.dma_buf_fd;
	}
	close(kchar_fd);
	*dma_addr_t = kcmd.alloc_cmd.dma_addr_t;
	return ret;
}

void get_perf(union die_index_data die_index)
{
	unsigned long long int start_count[2];
	unsigned long long int end_count[2];
	struct timespec time_start = {0 , 0};
	struct timespec time_end = {0, 0};
	unsigned long long int ns;

SIMPLE_AGAIN:
	clock_gettime(CLOCK_REALTIME, &time_start);
	start_count[0] = vastai_get_upstream_count(die_index.val);
	start_count[1] = vastai_get_downstream_count(die_index.val);

	sleep(1);

	end_count[0] = vastai_get_upstream_count(die_index.val);
	end_count[1] = vastai_get_downstream_count(die_index.val);

	clock_gettime(CLOCK_REALTIME, &time_end);
	ns = (time_end.tv_sec - time_start.tv_sec) * 1000 * 1000 * 1000 + (time_end.tv_nsec - time_start.tv_nsec);

	if (end_count[0] < start_count[0])
		goto SIMPLE_AGAIN;
	if (end_count[1] < start_count[1])
		goto SIMPLE_AGAIN;

	printf("%d\t\t", die_index.dev_id);
	printf("%.04fMB\t\t", (end_count[0] - start_count[0]) * 4 * 1.0 / ns * 1024);
	printf("%.04fMB\n", (end_count[1] - start_count[1]) * 4 * 1.0 / ns * 1024);
}

void out(int sig)
{
	int i;
	int ret;
	/* check signal == SIGINT */
	if (sig == SIGINT) {
		printf("WARNNING: Stop pcie bandwith test\n");
		printf("WARNNING: Recover pcie fw, Please Don't Exit!\n");
		/* loop device */
		for (i = 0; i < device_table->device_num; i++) {
			printf("Recover device[%d]'s fw, Please Waiting, This step will take a few minutes.\n", device_table->dev_info[i].dev_id);
			/* set 0x8bc0748 and 0x8bc0744 */
			ret = vastai_pcie_write_reg_32bit(0x8bc0748,
							0,
							device_table->dev_info[i].dev_id,
							device_table->dev_info[i].die_info[0].die_index.die_id);
			if (ret) {
				/* ... */
				printf("ERROR: stop testcase failed. Please save dmesg for debug\n");
				printf("ERROR: set Register addr: 0x8bc0748\n");
				printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
				exit(0);
			}
			ret = vastai_pcie_write_reg_32bit(0x8bc0744,
							0,
							device_table->dev_info[i].dev_id,
							device_table->dev_info[i].die_info[0].die_index.die_id);
			if (ret) {
				/* ... */
				printf("ERROR: stop testcase failed. Please save dmesg for debug\n");
				printf("ERROR: set Register addr: 0x8bc0744\n");
				printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
				exit(0);
			}
			/* sleep */
			sleep(1);
			/* poweron video */
			ret = vastai_pcie_cmd("poweron_video", device_table->dev_info[i].dev_id);
			if (ret) {
				/* ... */
				printf("ERROR: poweron video failed. Please save dmesg for debug\n");
				printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
				exit(0);
			}
			ret = vastai_pcie_cmd("reboot", device_table->dev_info[i].dev_id);
			if (ret) {
				/* ... */
				printf("ERROR: reboot failed. Please save dmesg for debug\n");
				printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
				exit(0);
			}
			/* sleep */
			sleep(1);
			/* free dmabuf */
			close(dma_fd[i]);
		}
		/* exit */
		exit(0);
	}
}

int main(void)
{
	int ret;
	unsigned long long int dma_addr_t;
	int i;

	/* print warnning for user */
	print_warnning();

	/* get how many device in this pc */
	device_table = malloc(DEVICE_TABLE_SIZE);
	device_table->device_num = DEVICE_TABLE_SIZE / sizeof(struct vastai_dev_info);
	ret = vastai_get_dev_info(device_table->dev_info, &device_table->device_num);
	if (ret) {
		/* ... */
		printf("ERROR: get dev info failed. Please save dmesg for debug\n");
		return 0;
	}
	if (device_table->device_num == 0) {
		/* ... */
		printf("ERROR: Can not find sv100 in this pc?\n");
		return 0;
	}

	/* loop device */
	printf("WARNNING: Set pcie fw, Please Don't Exit!\n");
	for (i = 0; i < device_table->device_num; i++) {
		printf("set device[%d]'s fw, Please Waiting, This step will take a few minutes.\n", device_table->dev_info[i].dev_id);
		/* alloc a dmabuf for user */
		dma_fd[i] = vastai_alloc_dma_buf_phy(TEST_BUF_LEN , device_table->dev_info[i].die_info[0].die_index.val, &dma_addr_t);
		if (dma_fd[i] < 0) {
			/* ... */
			printf("ERROR: alloc dma buf failed. Please save dmesg for debug\n");
			printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
			return 0;
		}

		/* reboot fw into dev */
		ret = vastai_pcie_cmd("reboot_ddrbw", device_table->dev_info[i].dev_id);
		if (ret) {
			/* ... */
			printf("ERROR: reset pcie fw failed. Please save dmesg for debug\n");
			printf("ERROR: Please poweroff server\\pc for recover\n");
			printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
			return 0;
		}
		/* set 0x8bc0748 and 0x8bc0744 */
		ret = vastai_pcie_write_reg_32bit(0x8bc0748,
						  dma_addr_t,
						  device_table->dev_info[i].dev_id,
						  device_table->dev_info[0].die_info[0].die_index.die_id);
		if (ret) {
			/* ... */
			printf("ERROR: reset pcie fw failed. Please save dmesg for debug\n");
			printf("ERROR: Please poweroff server\\pc for recover\n");
			printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
			return 0;
		}
		ret = vastai_pcie_write_reg_32bit(0x8bc0744,
						  2,
						  device_table->dev_info[i].dev_id,
						  device_table->dev_info[0].die_info[0].die_index.die_id);
		if (ret) {
			/* ... */
			printf("ERROR: reset pcie fw failed. Please save dmesg for debug\n");
			printf("ERROR: Please poweroff server\\pc for recover\n");
			printf("ERROR: device id %d errn %d\n", device_table->dev_info[i].dev_id, ret);
			return 0;
		}
	}
	signal(SIGINT, out);
	printf("WARNNING: You can use ctrl-c to stop this test.\n");
	
	/* loop 1 */
	printf("start test\n");
	while (1) {
		/* print table head */
		printf("=======================================================================\n");
		printf("device id\t||dev->pc bandwith\t||pc->dev bandwith\n");
		/* loop device */
		for (i = 0; i < device_table->device_num; i++) {
			/* get and print device's perf */
			get_perf(device_table->dev_info[i].die_info[0].die_index);
		}
		/* print table tail */
		printf("=======================================================================\n");
	}
	return 0;
}
