#include "pcie_bring_up_test.h"
#include "pcie_bring_up_test_fn.h"

u8 pcie_test_log_level = BIT(0);

struct test_case case_table[] = {
	{ .test_id = "PCIE-1Device-1Die-BASIC-TC008",
	  .test_fn = die0_csram_write_read,
	  .usage = NULL },
	{ .test_id = "PCIE-1Device-1Die-BASIC-TC010",
	  .test_fn = die0_ddr_write_read,
	  .usage = NULL },
	{ .test_id = "PCIE-1Device-1Die-UDMA-TC004",
	  .test_fn = die0_ddr_dma_bulk_256b,
	  .usage = NULL },
};

int main(int argc, char *argv[])
{
	int i;
	if (argc != 2 || !strcmp(argv[1], "*")) {
		SYS_PRINT("only support * as argv now!\n");
	}

	for (i = 0; i < (int)ARRAY_SIZE(case_table); i++) {
		enum test_result result = case_table[i].test_fn();
		SYS_PRINT("%16s :%s\n", case_table[i].test_id,
			  result == FAILED ? "FAILED" : "SUCCESS");
	}
	return 0;
}

enum test_result die0_ddr_write_read(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x840000000;

	int ret = vastai_pcie_write_reg_32bit(test_addr, gold_value, 0, DIE_0);
	TEST_EQ(ret, 0);

	unsigned int value;
	ret = vastai_pcie_read_reg_32bit(test_addr, &value, 0, DIE_0);
	TEST_EQ(ret, 0);

	TEST_EQ(value, gold_value);
	return SUCCESS;
}

enum test_result die0_csram_write_read(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x8C00100;

	int ret = vastai_pcie_write_reg_32bit(test_addr, gold_value, 0, DIE_0);
	TEST_EQ(ret, 0);

	unsigned int value;
	ret = vastai_pcie_read_reg_32bit(test_addr, &value, 0, DIE_0);
	TEST_EQ(ret, 0);

	TEST_EQ(value, gold_value);
	return SUCCESS;
}

enum test_result die0_ddr_dma_bulk_256b(void)
{
	int test_len = 0x100;
	void *data_write = malloc(test_len);
	u64 test_addr = 0x840000000;

	memset(data_write, 0x55, test_len);

	struct desc_node node_write = {
		.dev_addr = test_addr,
		.data = data_write,
		.length = test_len,
	};

	long int dma_time;
	//vastai_dma_host_2_dev_bulk(0, 0, 0, &node_write, 1, &dma_time);

	void *data_read = malloc(test_len);

	memset(data_read, 0x11, test_len);

	struct desc_node node_read = {
		.dev_addr = test_addr,
		.data = data_read,
		.length = test_len,
	};

	//vastai_dma_dev_2_host_bulk(0, 0, 0, &node_read, 1, &dma_time);

	TEST_BUF(data_read, data_write, test_len);

	free(data_write);
	free(data_read);
	return SUCCESS;
}