#ifndef __PCIE_BRING_UP_TEST_H__
#define __PCIE_BRING_UP_TEST_H__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vastai_pcie_user_api.h"

enum test_result {
	FAILED,
	SUCCESS,
};

struct test_case {
	char *test_id;
	enum test_result (*test_fn)(void);
	char *usage;
};

#define ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))

typedef unsigned long long int u64;
typedef long long int s64;
typedef unsigned int u32;
typedef int s32;
typedef unsigned short u16;
typedef short s16;
typedef unsigned char u8;
typedef char s8;

#define BIT(x) (1 << x)

#define TEST_EQ(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A == (u64)B)) {                                     \
			CASE_PRINT("FAILED %lld == %lld\n", (u64)A, (u64)B);   \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_LT(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A < (u64)B)) {                                      \
			CASE_PRINT("FAILED %lld < %lld\n", (u64)A, (u64)B);    \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_LE(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A <= (u64)B)) {                                     \
			CASE_PRINT("FAILED %lld <= %lld\n", (u64)A, (u64)B);   \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_NE(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A != (u64)B)) {                                     \
			CASE_PRINT("FAILED %lld != %lld\n", (u64)A, (u64)B);   \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_GE(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A >= (u64)B)) {                                     \
			CASE_PRINT("FAILED %lld >= %lld\n", (u64)A, (u64)B);   \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_GT(A, B)                                                          \
	do {                                                                   \
		if (!((u64)A > (u64)B)) {                                      \
			CASE_PRINT("FAILED %lld > %lld\n", (u64)A, (u64)B);    \
			return FAILED;                                         \
		}                                                              \
	} while (0)

#define TEST_BUF(A, B, len)                                                    \
	do {                                                                   \
		int i;                                                         \
		for (i = 0; i < len; i++) {                                    \
			if (((char *)A)[i] != ((char *)B)[i]) {                \
				CASE_PRINT("BUF DATA mismatch\n");             \
				return FAILED;                                 \
			}                                                      \
		}                                                              \
	} while (0)

#define SYS_PRINT(fmt, args...)                                                \
	do {                                                                   \
		if (pcie_test_log_level & BIT(0))                              \
			printf("[SYSTEM] " fmt, ##args);                       \
	} while (0)

#define CASE_PRINT(fmt, args...)                                               \
	do {                                                                   \
		if (pcie_test_log_level & BIT(1))                              \
			printf("[CASE] " fmt, ##args);                         \
	} while (0)

extern u8 pcie_test_log_level;
#endif
