#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include "vastai_pcie_user_api.h"

#define READ_ADDR 0x840000000

int test_rw_kchar_performance(int read_size_once, int test_count)
{
	const unsigned long long test_addr = READ_ADDR;
	int i, ret;
	char *buf;
	clock_t t;

	buf = malloc(read_size_once);
	if (!buf) {
		printf("%s: malloc fail\n", __func__);
		return -1;
	}

	t = clock();
	for (i = 0; i < test_count; i++) {
		ret = vastai_pcie_read_reg_buf(test_addr, buf, read_size_once,
					       0, 0);
		if (ret) {
			printf("%s: read failed\n", __func__);
			free(buf);
			return -1;
		}
	}
	t = clock() - t;
	printf("%s: speed time %f seconds\n", __func__,
	       (float)t / CLOCKS_PER_SEC);
	free(buf);
	return 0;
}

int test_mmap_kchar_performance(int read_size_once, int test_count)
{
	struct vastai_dev_buf_desc *desc;
	const unsigned long long test_addr = READ_ADDR;
	int i, ret;
	char *buf;
	clock_t t;

	buf = malloc(read_size_once);
	if (!buf) {
		printf("%s: malloc fail\n", __func__);
		return -1;
	}

	t = clock();

	desc = open_vastai_dev_buf(0, "test_buf");
	if (!desc) {
		printf("%s: open dev buf fail\n", __func__);
		free(buf);
		return -1;
	}

	for (i = 0; i < test_count; i++) {
		ret = vastai_dev_buf_read(desc, test_addr, buf, read_size_once,
					  0);
		if (ret) {
			printf("%s: read failed\n", __func__);
			close_vastai_dev_buf(desc);
			free(buf);
			return -1;
		}
	}
	close_vastai_dev_buf(desc);

	t = clock() - t;
	printf("%s: speed time %f seconds\n", __func__,
	       (float)t / CLOCKS_PER_SEC);
	free(buf);
	return 0;
}

int main(int argc, char *argv[])
{
	int read_size_once = 64;
	int test_count = 1000;
	int ret;

	if (argc == 2) {
		read_size_once = atoi(argv[1]);
	} else if (argc >= 3) {
		read_size_once = atoi(argv[1]);
		test_count = atoi(argv[2]);
	}

	printf("TEST: read_size_once=%d,  test_count=%d\n", read_size_once,
	       test_count);

	ret = test_rw_kchar_performance(read_size_once, test_count);
	if (ret)
		printf("test rw kchar error\n");
	else
		printf("test rw kchar done\n");

	ret = test_mmap_kchar_performance(read_size_once, test_count);
	if (ret)
		printf("test mmap kchar error\n");
	else
		printf("test mmap kchar done\n");

	return 0;
}
