#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#include "vastai_pcie_user_api.h"

#define ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))

int write_read_32bit_base_dev0_die0(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x840000000;

	int ret = vastai_pcie_write_reg_32bit(test_addr, gold_value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned int value;
	ret = vastai_pcie_read_reg_32bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 0. \n",
		       __FUNCTION__);
		printf("gold_value=0x%x value=0x%x\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_32bit_base_dev0_die1(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x840000000;

	int ret = vastai_pcie_write_reg_32bit(test_addr, gold_value, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned int value;
	ret = vastai_pcie_read_reg_32bit(test_addr, &value, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 1. \n",
		       __FUNCTION__);
		printf("gold_value=0x%x value=0x%x\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_64bit_base_dev0_die0(void)
{
	const unsigned long long gold_value = 0x01234567ABCDEF01;
	const unsigned long long test_addr = 0x840000008;

	int ret = vastai_pcie_write_reg_64bit(test_addr, gold_value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value;
	ret = vastai_pcie_read_reg_64bit(test_addr, &value, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 0. \n",
		       __FUNCTION__);
		printf("gold_value=0x%llx value=0x%llx\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_64bit_base_dev0_die1(void)
{
	const unsigned long long gold_value = 0x01234567ABCDEF01;
	const unsigned long long test_addr = 0x840000008;

	int ret = vastai_pcie_write_reg_64bit(test_addr, gold_value, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value;
	ret = vastai_pcie_read_reg_64bit(test_addr, &value, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare. at die 1. \n",
		       __FUNCTION__);
		printf("gold_value=0x%llx value=0x%llx\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_buf_base_dev0_die0(void)
{
	int i, j;
	const unsigned long long gold_value[] = {
		0x101234567ABCDEF0,
		0x0101234567ABCDEF,
		0xF0101234567ABCDE,
		0xEF0101234567ABCD,
	};
	const unsigned long long test_addr = 0x840000010;
	int ret = vastai_pcie_write_reg_buf(test_addr, (void *)gold_value,
					    sizeof(gold_value), 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value[ARRAY_SIZE(gold_value)];
	ret = vastai_pcie_read_reg_buf(test_addr, (void *)value, sizeof(value),
				       0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	for (i = 0; i < (int)ARRAY_SIZE(value); i++) {
		if (gold_value[i] != value[i]) {
			/* dump two buf */
			printf("ERROR: %s Different Match Compare. at die 0.\n",
			       __FUNCTION__);
			for (j = 0; j < (int)ARRAY_SIZE(value); j++) {
				printf("gold_value=0x%llx value=0x%llx\n",
				       gold_value[j], value[j]);
			}
			return -2;
		}
	}

	return 0;
}

int write_read_buf_base_dev0_die1(void)
{
	int i, j;
	const unsigned long long gold_value[] = {
		0x101234567ABCDEF0,
		0x0101234567ABCDEF,
		0xF0101234567ABCDE,
		0xEF0101234567ABCD,
	};
	unsigned long long value[ARRAY_SIZE(gold_value)];
	const unsigned long long test_addr = 0x840000010;
	int ret = vastai_pcie_write_reg_buf(test_addr, (void *)gold_value,
					    sizeof(gold_value), 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_buf(test_addr, (void *)value, sizeof(value),
				       0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	for (i = 0; i < (int)ARRAY_SIZE(value); i++) {
		if (gold_value[i] != value[i]) {
			/* dump two buf */
			printf("ERROR: %s Different Match Compare. at die 1.\n",
			       __FUNCTION__);
			for (j = 0; j < (int)ARRAY_SIZE(value); j++) {
				printf("gold_value=0x%llx value=0x%llx\n",
				       gold_value[j], value[j]);
			}
			return -2;
		}
	}

	return 0;
}

int write_read_file_base_dev0_die0(void)
{
	const unsigned long long src_test_addr = 0x840000000;
	const unsigned long long dst_test_addr = 0x840000040;

	int ret = vastai_pcie_read_reg_file("./read_src.ttmp", 0x0,
					    src_test_addr, 0x30, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = vastai_pcie_write_reg_file("./read_src.ttmp", 0x0, dst_test_addr,
					 0x30, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_file("./read_dst.ttmp", 0x0, dst_test_addr,
					0x30, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = compare_file("./read_src.ttmp", "./read_dst.ttmp", 0x0, 0x30);
	if (ret < 0) {
		printf("ERROR: %s read data file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: %s Different Match Compare. at die 0.\n",
		       __FUNCTION__);
		return -2;
	}
	return 0;
}

int write_read_file_base_dev0_die1(void)
{
	const unsigned long long src_test_addr = 0x840000000;
	const unsigned long long dst_test_addr = 0x840000040;

	int ret = vastai_pcie_read_reg_file("./read_src_1.ttmp", 0x0,
					    src_test_addr, 0x30, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = vastai_pcie_write_reg_file("./read_src_1.ttmp", 0x0,
					 dst_test_addr, 0x30, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_file("./read_dst_1.ttmp", 0x0, dst_test_addr,
					0x30, 0, DIE_1);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = compare_file("./read_src_1.ttmp", "./read_dst_1.ttmp", 0x0, 0x30);
	if (ret < 0) {
		printf("ERROR: %s read data file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: %s Different Match Compare. at die 1.\n",
		       __FUNCTION__);
		return -2;
	}
	return 0;
}

int creat_test_file(char *file_path, int size, unsigned long long magic)
{
	int filep, write_size;
	void *write_buf;
	unsigned long long *tmp_p;
	unsigned long i;
	int ret = 0;

	write_buf = malloc(size);
	if (!write_buf) {
		perror("malloc memory failed!\n");
		return -1;
	}

	tmp_p = (unsigned long long *)write_buf;
	for (i = 0; i < (size / sizeof(magic)); i++) {
		tmp_p[i] = magic;
	}

	filep = open(file_path, O_RDWR | O_CREAT, 0777);
	if (filep < 2) {
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!\n",
			 file_path);
		perror(temp_buf);
		ret = -2;
		goto end;
	}

	write_size = write(filep, write_buf, size);
	if (write_size != size) {
		ret = -3;
	}

	close(filep);
end:
	free(write_buf);
	return ret;
}

int write_read_big_file_base_dev0(void)
{
	const unsigned long long test_addr = 0x840000000;
	const unsigned long long gold_value = 0x101234567ABCDEF0;
	const int size = 1024 * 1024 * 10; //10MB
	int ret = 0;

	ret = creat_test_file("./read_src_big.ttmp", size, gold_value);
	if (ret != 0) {
		printf("ERROR: %s create test file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = vastai_pcie_write_reg_file("./read_src_big.ttmp", 0x0, test_addr,
					 size, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_file("./read_dst_big.ttmp", 0x0, test_addr,
					size, 0, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = compare_file("./read_src_big.ttmp", "./read_dst_big.ttmp", 0x0,
			   size);
	if (ret < 0) {
		printf("ERROR: %s read data file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
		return -2;
	}
	return 0;
}

int write_read_32bit_base_dev1_die0(void)
{
	const unsigned int gold_value = 0x01234567;
	const unsigned long long test_addr = 0x840000000;

	int ret = vastai_pcie_write_reg_32bit(test_addr, gold_value, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned int value;
	ret = vastai_pcie_read_reg_32bit(test_addr, &value, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
		printf("gold_value=0x%x value=0x%x\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_64bit_base_dev1_die0(void)
{
	const unsigned long long gold_value = 0x01234567ABCDEF01;
	const unsigned long long test_addr = 0x840000008;

	int ret = vastai_pcie_write_reg_64bit(test_addr, gold_value, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value;
	ret = vastai_pcie_read_reg_64bit(test_addr, &value, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	if (value != gold_value) {
		printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
		printf("gold_value=0x%llx value=0x%llx\n", gold_value, value);
		return -2;
	}
	return 0;
}

int write_read_buf_base_dev1_die0(void)
{
	int i, j;
	const unsigned long long gold_value[] = {
		0x101234567ABCDEF0,
		0x0101234567ABCDEF,
		0xF0101234567ABCDE,
		0xEF0101234567ABCD,
	};
	const unsigned long long test_addr = 0x840000010;
	int ret = vastai_pcie_write_reg_buf(test_addr, (void *)gold_value,
					    sizeof(gold_value), 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	unsigned long long value[ARRAY_SIZE(gold_value)];
	ret = vastai_pcie_read_reg_buf(test_addr, (void *)value, sizeof(value),
				       1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	for (i = 0; i < (int)ARRAY_SIZE(value); i++) {
		if (gold_value[i] != value[i]) {
			/* dump two buf */
			printf("ERROR: %s Different Match Compare.\n",
			       __FUNCTION__);
			for (j = 0; j < (int)ARRAY_SIZE(value); j++) {
				printf("gold_value=0x%llx value=0x%llx\n",
				       gold_value[j], value[j]);
			}
			return -2;
		}
	}

	return 0;
}

int write_read_file_base_dev1_die0(void)
{
	const unsigned long long src_test_addr = 0x840000000;
	const unsigned long long dst_test_addr = 0x840000040;

	int ret = vastai_pcie_read_reg_file("./read_src.ttmp", 0x0,
					    src_test_addr, 0x30, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = vastai_pcie_write_reg_file("./read_src.ttmp", 0x0, dst_test_addr,
					 0x30, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_file("./read_dst.ttmp", 0x0, dst_test_addr,
					0x30, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = compare_file("./read_src.ttmp", "./read_dst.ttmp", 0x0, 0x30);
	if (ret < 0) {
		printf("ERROR: %s read data file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
		return -2;
	}
	return 0;
}

int write_read_big_file_base_dev1_die0(void)
{
	const unsigned long long test_addr = 0x840000000;
	const unsigned long long gold_value = 0x101234567ABCDEF0;
	const int size = 1024 * 1024 * 10; //10MB
	int ret = 0;

	ret = creat_test_file("./read_src_big.ttmp", size, gold_value);
	if (ret != 0) {
		printf("ERROR: %s create test file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = vastai_pcie_write_reg_file("./read_src_big.ttmp", 0x0, test_addr,
					 size, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__,
		       ret);
		return -1;
	}

	ret = vastai_pcie_read_reg_file("./read_dst_big.ttmp", 0x0, test_addr,
					size, 1, DIE_0);
	if (ret != 0) {
		printf("ERROR: %s read reg first failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}

	ret = compare_file("./read_src_big.ttmp", "./read_dst_big.ttmp", 0x0,
			   size);
	if (ret < 0) {
		printf("ERROR: %s read data file failed, errcode %d\n",
		       __FUNCTION__, ret);
		return -1;
	}
	if (ret != 1) {
		printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
		return -2;
	}
	return 0;
}

int bulk_one_desc_base(void)
{
	void *data_write = malloc(0x200);
	int ret = 0, i;

	memset(data_write, 0x55, 0x200);

	struct desc_node node_write = {
		.dev_addr = 0x840000000,
		.data = data_write,
		.length = 0x200,
	};

	long int dma_time;
	//vastai_dma_host_2_dev_bulk(0, 0, 0, &node_write, 1, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	void *data_read = malloc(0x200);

	memset(data_read, 0x11, 0x200);

	struct desc_node node_read = {
		.dev_addr = 0x840000000,
		.data = data_read,
		.length = 0x200,
	};

	//vastai_dma_dev_2_host_bulk(0, 0, 0, &node_read, 1, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	for (i = 0; i < 0x200; i++) {
		if (((char *)data_write)[i] != ((char *)data_read)[i]) {
			printf("%s %d is not same! i %d write 0x%x read 0x%x\n",
			       __FILE__, __LINE__, i, ((char *)data_write)[i],
			       ((char *)data_read)[i]);
			ret = 1;
			goto FREE;
		}
	}
FREE:
	free(data_write);
	free(data_read);
	return ret;
}

int bulk_list_base(void)
{
	void *data_write = malloc(0x200);
	int ret = 0, i;

	memset(data_write, 0x55, 0x200);

	struct desc_node node_write[2] = {
		{
			.dev_addr = 0x840000000,
			.data = data_write,
			.length = 0x100,
		},
		{
			.dev_addr = 0x840000100,
			.data = data_write + 0x100,
			.length = 0x100,
		},
	};

	long int dma_time;
	//vastai_dma_host_2_dev_bulk(0, 0, 0, node_write, 2, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	void *data_read = malloc(0x200);

	memset(data_read, 0x11, 0x200);

	struct desc_node node_read[2] = { {
						  .dev_addr = 0x840000000,
						  .data = data_read,
						  .length = 0x100,
					  },
					  {
						  .dev_addr = 0x840000100,
						  .data = data_read + 0x100,
						  .length = 0x100,
					  } };

	//vastai_dma_dev_2_host_bulk(0, 0, 0, node_read, 2, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	for (i = 0; i < 0x200; i++) {
		if (((char *)data_write)[i] != ((char *)data_read)[i]) {
			printf("%s %d is not same! i %d write 0x%x read 0x%x\n",
			       __FILE__, __LINE__, i, ((char *)data_write)[i],
			       ((char *)data_read)[i]);
			ret = 1;
			goto FREE;
		}
	}
FREE:
	free(data_write);
	free(data_read);
	return ret;
}

int scatter_list_base(void)
{
	void *data_write = malloc(0x200);
	int ret = 0, i;

	memset(data_write, 0x55, 0x200);

	struct desc_node node_write[] = {
		{
			.dev_addr = 0x0,
			.data = data_write,
			.length = 0x200,
		},
		{
			.dev_addr = 0x840000000,
			.data = 0x0,
			.length = 0x100,
		},
		{
			.dev_addr = 0x840000100,
			.data = 0x0,
			.length = 0x100,
		},
	};

	long int dma_time;
	//vastai_dma_host_2_dev_scatter(0, 0, 0, node_write, 3, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	void *data_read = malloc(0x200);

	memset(data_read, 0x11, 0x200);

	struct desc_node node_read[] = {
		{
			.dev_addr = 0x840000000,
			.data = 0x0,
			.length = 0x200,
		},
		{
			.dev_addr = 0,
			.data = data_read,
			.length = 0x100,
		},
		{
			.dev_addr = 0,
			.data = data_read + 0x100,
			.length = 0x100,
		},
	};
	vastai_dma_dev_2_host_scatter(0, 0, 0, node_read, 3, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	for (i = 0; i < 0x200; i++) {
		if (((char *)data_write)[i] != ((char *)data_read)[i]) {
			printf("%s %d is not same! i %d write 0x%x read 0x%x\n",
			       __FILE__, __LINE__, i, ((char *)data_write)[i],
			       ((char *)data_read)[i]);
			ret = 1;
			goto FREE;
		}
	}
FREE:
	free(data_write);
	free(data_read);
	return ret;
}

int gather_list_base(void)
{
	void *data_write = malloc(0x200);
	int ret = 0, i;

	memset(data_write, 0x55, 0x200);

	struct desc_node node_write[] = {
		{
			.dev_addr = 0x0,
			.data = data_write,
			.length = 0x100,
		},
		{
			.dev_addr = 0x0,
			.data = data_write + 0x100,
			.length = 0x100,
		},
		{
			.dev_addr = 0x840000000,
			.data = 0x0,
			.length = 0x200,
		},
	};

	long int dma_time;
	//vastai_dma_host_2_dev_gather(0, 0, 0, node_write, 3, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	void *data_read = malloc(0x200);

	memset(data_read, 0x11, 0x200);

	struct desc_node node_read[] = {
		{
			.dev_addr = 0x840000000,
			.data = 0x0,
			.length = 0x100,
		},
		{
			.dev_addr = 0x840000100,
			.data = 0x0,
			.length = 0x100,
		},
		{
			.dev_addr = 0,
			.data = data_read,
			.length = 0x200,
		},
	};

	vastai_dma_dev_2_host_gather(0, 0, 0, node_read, 3, &dma_time);
	printf("%s take time %ldns\n", __FUNCTION__, dma_time);

	for (i = 0; i < 0x200; i++) {
		if (((char *)data_write)[i] != ((char *)data_read)[i]) {
			printf("%s %d is not same! i %d write 0x%x read 0x%x\n",
			       __FILE__, __LINE__, i, ((char *)data_write)[i],
			       ((char *)data_read)[i]);
			ret = 1;
			goto FREE;
		}
	}
FREE:
	free(data_write);
	free(data_read);
	return ret;
}

int get_device_test(void)
{
	int device_num = vastai_get_device_num();
	printf("get num %d\n", device_num);
	return 0;
}

int get_die_test(void)
{
	int die_index[8] = { 0 };
	int die_num = 0;
	int ret = 0, i;

	ret = vastai_get_die_info(0, die_index, &die_num);
	printf("dev[%d] die_num[%d]\n", 0, die_num);
	for (i = 0; i < die_num; i++) {
		printf("die_index[%d] ", die_index[i]);
	}
	printf("\n");
	return ret;
}

int get_fw_version_test(void)
{
	int ret = 0;
	struct vastai_fw_version fw_ver;

	ret = vastai_get_fw_version(0, &fw_ver, sizeof(fw_ver));
	if (ret) {
		printf("in function %s err:%d", __FUNCTION__, ret);
		return ret;
	}
	printf("%s\r\n", fw_ver.ver);
	printf("%s\r\n", fw_ver.smcu_ver);
	printf("%s\r\n", fw_ver.bmcu_ver);
	printf("%s\r\n", fw_ver.vdmcu_ver);
	printf("%s\r\n", fw_ver.vemcu_ver);
	printf("%s\r\n", fw_ver.vdsp_ver);
	printf("%s\r\n", fw_ver.cmcu_ver);
	printf("%s\r\n", fw_ver.lmcu_ver);
	printf("%s\r\n", fw_ver.odsp_ver);
	printf("%s\r\n", fw_ver.driver_ver);

	return ret;
}

int print_dev_table(void)
{
	void *mem = malloc(sizeof(struct vastai_dev_info) * 16);
	int dev_num;

	vastai_get_dev_info(mem, &dev_num);

	for (int i = 0; i < dev_num; i++) {
		printf("dev_id[%d] unid[%lld] pcie_slot[%s] pcie_dev_name[%s] device_name[%s]\n",
		       ((struct vastai_dev_info *)mem)[i].dev_id,
		       ((struct vastai_dev_info *)mem)[i].unid,
		       ((struct vastai_dev_info *)mem)[i].pcie_slot,
		       ((struct vastai_dev_info *)mem)[i].pcie_dev_name,
		       ((struct vastai_dev_info *)mem)[i].device_name);
		printf("  pci_id[0x%x] pci_sub_id[0x%x] major[0x%x] cap.ai[%s] cap.video[%s]\n",
		       ((struct vastai_dev_info *)mem)[i].pci_id.val,
		       ((struct vastai_dev_info *)mem)[i].pci_sub_id.val,
		       ((struct vastai_dev_info *)mem)[i].major_minor.val,
		       ((struct vastai_dev_info *)mem)[i].cap.ai_cap ? "yes" :
								       "no",
		       ((struct vastai_dev_info *)mem)[i].cap.video_cap ?
			       "yes" :
			       "no");
		for (int j = 0; j < ((struct vastai_dev_info *)mem)[i].die_num;
		     j++) {
			printf("\t die_index[0x%x] vacc_name[%s] render_name[%s]\n",
			       ((struct vastai_dev_info *)mem)[i]
				       .die_info[j]
				       .die_index.val,
			       ((struct vastai_dev_info *)mem)[i]
				       .die_info[j]
				       .vacc_name,
			       ((struct vastai_dev_info *)mem)[i]
				       .die_info[j]
				       .render_name);
		}
	}
	free(mem);
	return 0;
}

int print_performance(void)
{
	u32 info_size = sizeof(struct vastai_performance_info) + 512;
	int ret;
	void *mem = malloc(info_size);

	ret = vastai_get_performance(0, mem, info_size);
	if (ret) {
		printf("get_performance get error %d\n", ret);
		free(mem);
		return 0;
	}

	for (int i = 0; i < 512 / 4; i++) {
		if (i != 0 && !(i % 4)) {
			printf("\n");
		}
		printf("0x%08x ", ((u32 *)mem)[i]);
	}
	free(mem);
	return 0;
}

void clean_ttmp(void)
{
	DIR *dir = opendir("./");
	struct dirent *entry = NULL;

	while ((entry = readdir(dir)) != NULL) {
		if (strstr(entry->d_name, ".ttmp")) {
			printf("remove %s\n", entry->d_name);
			remove(entry->d_name);
		}
	}
	closedir(dir);
}

typedef int (*test_fn)(void);

test_fn test_case_table[] = {
	get_fw_version_test,
	write_read_32bit_base_dev0_die0,
	write_read_64bit_base_dev0_die0,
	write_read_buf_base_dev0_die0,
	write_read_file_base_dev0_die0,
	write_read_32bit_base_dev0_die1,
	write_read_64bit_base_dev0_die1,
	write_read_buf_base_dev0_die1,
	write_read_file_base_dev0_die1,
	write_read_big_file_base_dev0,
	//write_read_32bit_base_dev1_die0,
	//write_read_big_file_base_dev1_die0,
	bulk_one_desc_base,
	bulk_list_base,
	scatter_list_base,
	gather_list_base,
	get_device_test,
	get_die_test,
	print_performance,
	print_dev_table,
};

int main(void)
{
	int ret = 0, i;
#if 0
	ret = vastai_pcie_cmd(BOOT_CMD, 0);

	if (ret) {
		printf("ERROR: cant boot device\n");
		return 0;
	}
#endif

	for (i = 0; i < (int)ARRAY_SIZE(test_case_table); i++) {
		ret = test_case_table[i]();
		if (ret) {
			printf("test failed case_id[%d]\n", i);
			break;
		} else {
			printf("test success case_id[%d]\n", i);
		}
	}
	if (!ret) {
		printf("test success!\n");
	}

	clean_ttmp();
	return ret;
}
