#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <dirent.h>
#include <getopt.h>
#include <malloc.h>
#include <signal.h>
#include "../include/vastai_pcie_dump_api.h"

#define u8  uint8_t
#define u32 uint32_t
#define u64 uint64_t

#define MAX_REGION_NUM		256
struct vastai_atu_region_info{
	u32 region_num;
	u8  pf_num;
	u8  vf_num;
	//0:address match mode , 1:bar match mode
	u8  mode;
	u8  bar_num;  
	u64 noc_base_addr;
	u64 host_base_addr;
	u64 host_limit_addr;
	u64 size;
};

struct inbound_atu_regs{
	u32 ctrl1_reg;
	u32 ctrl2_reg;
	u32 lo32_host_addr;
	u32 hi32_host_addr;
	u32 lo32_limit;
	u32 lo32_noc_addr;
	u32 hi32_noc_addr;
	u32 ctrl3_reg;
	u32 hi32_limit;
};

static void bubble_sort(struct vastai_atu_region_info *arry, int len) {
	int i, j;
	struct vastai_atu_region_info temp;
	for (i = 0; i < len - 1; i++)
			for (j = 0; j < len - 1 - i; j++)
					if (arry[j].noc_base_addr > arry[j + 1].noc_base_addr) {
							//1:save arry[j] element to temp 
							temp.region_num 	  = arry[j].region_num;
							temp.pf_num     	  = arry[j].pf_num;
							temp.vf_num           = arry[j].vf_num;
							temp.mode       	  = arry[j].mode;
							temp.bar_num          = arry[j].bar_num;
							temp.noc_base_addr    = arry[j].noc_base_addr;
							temp.host_base_addr   = arry[j].host_base_addr;
							temp.host_limit_addr  = arry[j].host_limit_addr;
							temp.size             = arry[j].size;

							//2:save arry[j+1] element to arry[j] 
							arry[j].region_num 	     = arry[j+1].region_num;
							arry[j].pf_num     	     = arry[j+1].pf_num;
							arry[j].vf_num           = arry[j+1].vf_num;
							arry[j].mode       	     = arry[j+1].mode;
							arry[j].bar_num          = arry[j+1].bar_num;
							arry[j].noc_base_addr    = arry[j+1].noc_base_addr;
							arry[j].host_base_addr   = arry[j+1].host_base_addr;
							arry[j].host_limit_addr  = arry[j+1].host_limit_addr;
							arry[j].size             = arry[j+1].size;

							//3:save temp element to arry[j+1] 
							arry[j+1].region_num 	   = temp.region_num;
							arry[j+1].pf_num     	   = temp.pf_num;
							arry[j+1].vf_num           = temp.vf_num;
							arry[j+1].mode       	   = temp.mode;
							arry[j+1].bar_num          = temp.bar_num;
							arry[j+1].noc_base_addr    = temp.noc_base_addr;
							arry[j+1].host_base_addr   = temp.host_base_addr;
							arry[j+1].host_limit_addr  = temp.host_limit_addr;
							arry[j+1].size             = temp.size;
					}
	return;
}

static void dump_atu(void)
{
	int region_num;
	int reg_offset;
	int valid_atu_num=0;
	int ret;
	struct vastai_atu_region_info *valid_atu_info_array;
	struct inbound_atu_regs temp_inbound_regs;
	u64 temp_noc_base_addr;
	u64 temp_host_base_addr;
	u64 temp_host_limit_addr;
	u64 temp_size;
	u8  mode;
	int i;

	valid_atu_info_array = (struct vastai_atu_region_info *)malloc(sizeof(struct vastai_atu_region_info) * MAX_REGION_NUM);
	for(region_num = 0;region_num < MAX_REGION_NUM; region_num++) {
			reg_offset=0x2800000 + 0x10000 + region_num * 2 * 0x100 + 0x100;
			//to do dump slave bar atu
			ret = vastai_dump_read_buf(0, reg_offset, sizeof(struct inbound_atu_regs), (u8 *)&temp_inbound_regs);
		if (ret < 0) {
			printf("%s region_num = 0x%x, len=%ld, error:%d\n",
					__func__, region_num, sizeof(temp_inbound_regs), ret);
			continue;
		}

		
		temp_host_base_addr  = (u64) temp_inbound_regs.hi32_host_addr << 32 | temp_inbound_regs.lo32_host_addr;
		temp_noc_base_addr   = (u64) temp_inbound_regs.hi32_noc_addr << 32 | temp_inbound_regs.lo32_noc_addr;
		temp_host_limit_addr = (u64) temp_inbound_regs.hi32_limit << 32 | temp_inbound_regs.lo32_limit;

		if((0 == ((temp_inbound_regs.ctrl2_reg>>31)&0x1))) {
			continue;
		}

		temp_size            = temp_host_limit_addr - temp_host_base_addr + 1;

		//0:address match mode  1:bar match mode
		mode = temp_inbound_regs.ctrl2_reg>>30 & 0x1;

		valid_atu_info_array[valid_atu_num].region_num      = region_num;
		valid_atu_info_array[valid_atu_num].mode            = mode;
		valid_atu_info_array[valid_atu_num].pf_num          = temp_inbound_regs.ctrl1_reg>>20 & 0xFF;
		valid_atu_info_array[valid_atu_num].noc_base_addr   = temp_noc_base_addr;
		//0:address match mode 
		if(0 == mode) {
			valid_atu_info_array[valid_atu_num].host_base_addr  = temp_host_base_addr;
			valid_atu_info_array[valid_atu_num].host_limit_addr = temp_host_limit_addr;
			valid_atu_info_array[valid_atu_num].size            = temp_size;
			valid_atu_info_array[valid_atu_num].bar_num         = 0xFF;						
		} else {
			valid_atu_info_array[valid_atu_num].bar_num         = temp_inbound_regs.ctrl2_reg>>8  & 0x7;
			
		}

		//check vf enable
		if(1 == (temp_inbound_regs.ctrl2_reg>>20 &0x1)) {
			valid_atu_info_array[valid_atu_num].vf_num = temp_inbound_regs.ctrl3_reg&0x1F;
		} else {
			valid_atu_info_array[valid_atu_num].vf_num = 0xFF; 
		}

		valid_atu_num++;
								
	}

	bubble_sort(valid_atu_info_array,valid_atu_num);
	printf("-----start to dump %d valid atu info-----\n",valid_atu_num);
	for(i = 0;i < valid_atu_num; i++) {
		if(0==valid_atu_info_array[i].mode){
			printf("rgn[%03d] pf:%d vf:%02x noc[0x%010lx ~ 0x%010lx] map to host[%016lx ~ %016lx]\n",
								valid_atu_info_array[i].region_num,
								valid_atu_info_array[i].pf_num,
								valid_atu_info_array[i].vf_num,
								valid_atu_info_array[i].noc_base_addr,
								valid_atu_info_array[i].noc_base_addr+valid_atu_info_array[i].size-1,
								valid_atu_info_array[i].host_base_addr,
								valid_atu_info_array[i].host_limit_addr);
		} else if (1==valid_atu_info_array[i].mode){
			printf("rgn[%03d] pf:%d vf:%02x noc[0x%010lx] map to host bar[%x]\n",
								valid_atu_info_array[i].region_num,
								valid_atu_info_array[i].pf_num,
								valid_atu_info_array[i].vf_num,
								valid_atu_info_array[i].noc_base_addr,
								valid_atu_info_array[i].bar_num);
		}
		
	}
	
	free(valid_atu_info_array);
	return;
}

int main(int argc, char *argv[])
{
    vastai_dump_open(0);
    dump_atu();
    vastai_dump_close(0);
}