#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <dirent.h>
#include <getopt.h>
#include <malloc.h>
#include <signal.h>
#include "../include/vastai_pcie_dump_api.h"

#define	XSPI_LOCK_EN                   (1U)

#define SMU_PHERPH_RST				0x208028
#define XSPI_CSRAM_DATA_ADDR		0x8E0000
#define XSPI_BASE_ADDR				0x20000
#define XSPI_SEC_SIZE				(4*1024)			/* 4k */
#define XSPI_BLK_SIZE				(16*XSPI_SEC_SIZE)	/* 64k */
#define XSPI_TOP_BLK_POS			(XSPI_BLK_SIZE)		/* 0x10000 top block */
#define XSPI_BTM_BLK_POS			(31*XSPI_BLK_SIZE)	/* 0x1f0000 bottom block */
#define XSPI_HALF_BLKS_POS			(16*XSPI_BLK_SIZE)	/* 100000h */
#define XSPI_ALL_BLKS_POS			(32*XSPI_BLK_SIZE)	/* 200000h, total 2M */

#define PIO_MODE					(0x1U << 30U)
#define STIG_MODE					(0x1U << 5U)
#define ACMD_MODE					(0x3U << 5U)
#define INIT_COMP					(0x1U << 16U)
#define INIT_FAIL					(0x1U << 8U)
#define CTRL_BUSY					(0x1U << 7U)
#define DMA_MASTER					(0x1U << 19U)
#define TRD_0_MSK					(0x1U << 0U)

/*****************  command status 0x0044  ******************/
#define XSPI_CMD_ERROR				(0x1U << 0U)
#define XSPI_BUS_ERROR				(0x1U << 1U)
#define XSPI_CRC_ERROR				(0x1U << 2U)
#define XSPI_DQS_ERROR				(0x1U << 3U)
#define XSPI_DEVICE_ERROR			(0x1U << 4U)
#define XSPI_ECC_CORR_ERROR			(0x1U << 5U)
#define XSPI_FAIL					(0x1U << 14U)
#define XSPI_COMPLETE				(0x1U << 15U)
#define XSPI_ECC_STAT_MSK			(0xFFU << 16U)

/*****************  register macro define  ******************/
#define XSPI_CMD_REG0				(XSPI_BASE_ADDR + (0x0000))
#define XSPI_CMD_REG1				(XSPI_BASE_ADDR + (0x0004))
#define XSPI_CMD_REG2				(XSPI_BASE_ADDR + (0x0008))
#define XSPI_CMD_REG3				(XSPI_BASE_ADDR + (0x000C))
#define XSPI_CMD_REG4				(XSPI_BASE_ADDR + (0x0010))
#define XSPI_CMD_REG5				(XSPI_BASE_ADDR + (0x0014))
#define XSPI_CMD_STATUS_PTR			(XSPI_BASE_ADDR + (0x0040))
#define XSPI_CMD_STATUS				(XSPI_BASE_ADDR + (0x0044))
#define XSPI_CTRL_STATUS			(XSPI_BASE_ADDR + (0x0100))
#define XSPI_TRD_STATUS				(XSPI_BASE_ADDR + (0x0104))
#define XSPI_INTR_STATUS			(XSPI_BASE_ADDR + (0x0110))
#define XSPI_INTR_ENABLE			(XSPI_BASE_ADDR + (0x0114))
#define XSPI_TRD_COMP_INTR_STATUS	(XSPI_BASE_ADDR + (0x0120))
#define XSPI_TRD_INTR_STATUS		(XSPI_BASE_ADDR + (0x0130))
#define XSPI_BOOT_STATUS			(XSPI_BASE_ADDR + (0x0158))
#define XSPI_CTRL_CONFIG			(XSPI_BASE_ADDR + (0x0230))
#define XSPI_SDMA_TRD_INFO			(XSPI_BASE_ADDR + (0x0244))
#define XSPI_DISCOVERY_CONTROL		(XSPI_BASE_ADDR + (0x0260))
#define XSPI_XIP_MODE_CFG			(XSPI_BASE_ADDR + (0x0388))
#define XSPI_GLOBAL_SEQ_CFG			(XSPI_BASE_ADDR + (0x0390))

#define XSPI_RST_CFG_0				(XSPI_BASE_ADDR + (0x0400))
#define XSPI_RST_CFG_1				(XSPI_BASE_ADDR + (0x0404))
#define XSPI_ERS_CFG_0				(XSPI_BASE_ADDR + (0x0410))
#define XSPI_ERS_CFG_1				(XSPI_BASE_ADDR + (0x0414))
#define XSPI_ERS_CFG_2				(XSPI_BASE_ADDR + (0x0418))
#define XSPI_PROG_CFG_0				(XSPI_BASE_ADDR + (0x0420))
#define XSPI_PROG_CFG_1				(XSPI_BASE_ADDR + (0x0424))
#define XSPI_PROG_CFG_2				(XSPI_BASE_ADDR + (0x0428))
#define XSPI_READ_CFG_0				(XSPI_BASE_ADDR + (0x0430))
#define XSPI_READ_CFG_1				(XSPI_BASE_ADDR + (0x0434))
#define XSPI_READ_CFG_2				(XSPI_BASE_ADDR + (0x0438))
#define XSPI_WE_CFG_0				(XSPI_BASE_ADDR + (0x0440))
#define XSPI_STAT_CFG_0				(XSPI_BASE_ADDR + (0x0450))
#define XSPI_STAT_CFG_1				(XSPI_BASE_ADDR + (0x0454))
#define XSPI_STAT_CFG_2				(XSPI_BASE_ADDR + (0x0458))
#define XSPI_STAT_CFG_3				(XSPI_BASE_ADDR + (0x045c))
#define XSPI_STAT_CFG_4				(XSPI_BASE_ADDR + (0x0460))
#define XSPI_STAT_CFG_5				(XSPI_BASE_ADDR + (0x0464))
#define XSPI_STAT_CFG_7				(XSPI_BASE_ADDR + (0x046c))
#define XSPI_STAT_CFG_8				(XSPI_BASE_ADDR + (0x0470))
#define XSPI_STAT_CFG_9				(XSPI_BASE_ADDR + (0x0474))
#define XSPI_STAT_CFG_10			(XSPI_BASE_ADDR + (0x0478))

#define MAX_TIMEOUT_CNT				(0x1fffffffU)
#define MDID_WINBOND				(0x14EFU)
#define MDID_WINBOND_13				(0x13EFU)
#define MDID_WINBOND_16        	    (0x16EFU)
#define MDID_WINBOND_18				(0x18EFU)
#define	MDID_WINBOND_4M         	(0x15EFU)
#define MDID_ISSI					(0x149DU)
#define MDID_ISSI_4M				(0x159DU)
#define MDID_GD_4M				(0x15C8U)

// for 2MB flash
#define WB_WPS_EN					(0x01U << 2U)	/* default is 0 */
#define WB_QE_EN					(0x01U << 1U)	/* default is 0 */
#define ISSI_QE_EN					(0x01U << 6U)	/* default is 0 */
#define GD_QE_EN					(0x01U << 1U)   /* default is 0 */

#define ISSI_LOCK_ALL				(0x09U)			/* lock all blocks */
#define ISSI_UNLOCK_ALL				(0x00U)			/* unlock all blocks */
#define ISSI_UNLOCK_0TO15			(0x05U)			/* lock 16~31 & unlock 0~15 blocks */
#define ISSI_UNLOCK_16TO31			(0x0aU)			/* lock 0~15 & unlock 16~31 blocks */

// for 4MB ISSI flash
#define		ISSI_4M_LOCK_ALL				(0x08U)  // lock all 64 blocks
#define		ISSI_4M_UNLOCK_ALL				(0x00U)  // unlock all 64 blocks
#define		ISSI_4M_UNLOCK_0TO31			(0x06U)  // lock 32~63 & unlock 0~31 blocks
#define		ISSI_4M_UNLOCK_16TO63			(0x0aU)  // lock 0~15 & unlock 16~63 blocks

// for 4MB GD flash
#define GD_4M_LOCK_ALL				(0x0fU)        /* lock all 64 blocks */
#define GD_4M_UNLOCK_ALL			(0x00U)        /* unlock all 64 blocks */
#define GD_4M_UNLOCK_0TO31			(0x06U)        /* lock 32~63 & unlock 0~31 blocks */
#define GD_4M_UNLOCK_16TO63			(0x0dU)        /* lock 0~15 & unlock 16~63 blocks */

/* command define */
#define XSPI_CMD_RDMDID				(0x90U)
#define XSPI_CMD_WREN				(0x06U)
#define XSPI_CMD_RSTEN				(0x66U)			/* Software Reset Enable */
#define XSPI_CMD_RST				(0x99U)			/* Reset */

/* for WinBond */
#define XSPI_CMD_WB_RDSR1			(0x05U)
#define XSPI_CMD_WB_WRSR1			(0x01U)
#define XSPI_CMD_WB_RDSR2			(0x35U)
#define XSPI_CMD_WB_WRSR2			(0x31U)
#define XSPI_CMD_WB_RDSR3			(0x15U)
#define XSPI_CMD_WB_WRSR3			(0x11U)
#define XSPI_CMD_WB_SECLOCK			(0x36U)
#define XSPI_CMD_WB_SECUNLOCK		(0x39U)
#define XSPI_CMD_WB_GETSECLOCK		(0x3DU)

/* for ISSI */
#define XSPI_CMD_ISSI_RDSR			(0x05U)
#define XSPI_CMD_ISSI_WRSR			(0x01U)
#define XSPI_CMD_ISSI_SECUNLOCK		(0x26U)
#define XSPI_CMD_ISSI_SECLOCK		(0x24U)

/* for GD */
#define XSPI_CMD_GD_RDSR1		(0x05U)
#define XSPI_CMD_GD_RDSR2		(0x35U)
#define XSPI_CMD_GD_WRSR		(0x01U)
/* end */

#define    XSPI_4M_QUARTER_BLKS_POS       (16*XSPI_BLK_SIZE)		// 100000h, total 4MB
#define    XSPI_4M_ALL_BLKS_POS           (64*XSPI_BLK_SIZE)		// 400000h, total 4MB
#define    XSPI_4M_BTM_BLK_POS            (63*XSPI_BLK_SIZE)		// 0x3f0000 bottom block, total 4MB */

typedef enum
{
	XSPI_SUCCESS = 0,		/* complete successfully */
	XSPI_ERROR,				/* failed to complete successfully */
	XSPI_TIMEOUT,			/* timeout to polling status */
	XSPI_ERROR_CMD = 11,	/* that an invalid descriptor sequence has been detected */
	XSPI_ERROR_BUS,			/* that the controller got an error response on the system DMA interface */
	XSPI_ERROR_CRC,			/* that the controller detected a CRC error */
	XSPI_ERROR_DQS,			/* incorrect DQS pluses number was detected during the data read operation */
	XSPI_ERROR_DEVICE,		/* device error was detected during read status operation for erase/program/read */
	XSPI_ERROR_ECC_CORR		/* ECC correctable error was detected during page read, is valid only for SPI NAND device */
} XSPI_RET;

typedef enum
{
	MODE_PIO = 1,
	MODE_STIG,
	MODE_NUM
} XSPI_MODE;

typedef enum
{
	CMD_RESET			= 0x1100,	/* device reset */
	CMD_ERASE_SECTOR	= 0x1000,	/* erase a number of sequential sectors */
	CMD_ERASE_FULL		= 0x1001,	/* erase full */
	CMD_PROGRAM_DATA	= 0x2100,	/* program a number of data bytes */
	CMD_READ_DATA		= 0x2200	/* read a number of data bytes */
} XSPI_CMD_TYPE;

typedef enum
{
	UNLOCK = 0U,
	LOCK,
	INVALID
} PROTECT_MODE;

typedef enum
{
	_RESET = 0U,
	_SET = !_RESET
} FLAG_STATUS;

typedef struct
{
	uint32_t addr;
	uint32_t size;
	PROTECT_MODE unlocked;
} XSPI_UNLOCK_NODE;

typedef struct
{
	uint8_t inited;
	uint8_t protected;
	XSPI_UNLOCK_NODE sec;
} XSPI_DEV;

static uint16_t g_flash_mdid;
static XSPI_DEV g_xspi_dev;
static uint32_t g_card_id = 0;

static uint32_t readl(uint32_t addr)
{
	uint32_t val = 0;
	vastai_dump_read(g_card_id, addr, &val);
	return val;
}

static void writel(uint32_t value, uint32_t addr)
{
	vastai_dump_write(g_card_id, addr, value);
	return;
}

static uint8_t wait_xspi_init_comp(uint32_t delay)
{
	uint8_t ret =XSPI_ERROR;
	uint32_t config = 0;
	
	/* wait reset and initiate done */
	do{
		config = readl(XSPI_CTRL_STATUS);
		if(0 == delay--)
			return ret;
	}while(!(config& INIT_COMP));

	/* check initiate status */
	if (!(config  & INIT_FAIL))
		ret = XSPI_SUCCESS;

	return ret;
}

static uint8_t check_xspi_ctrl_sts(void)
{
	uint32_t status = 0;

	/* check xspi release csr */
	status = readl(SMU_PHERPH_RST);
	if(status & (1 << 10))			/* 1-reset */
	{
		return 2;					/* xspi is reset status */
	}

	status = readl(XSPI_BASE_ADDR + 0x8000);
	if(0x7f != status)
	{
		return 3;					/* xspi phy and ctrl is reset status */
	}

	return wait_xspi_init_comp(0xfffff); /* xspi ctrl is ready,need to check flash r/w */
}

static uint8_t check_xspi_reg_flag(uint32_t reg, uint32_t flag, FLAG_STATUS flag_status, uint32_t delay)
{
	uint32_t tmp = 0;

	do {
		if ((((tmp = readl(reg)) & flag)? _SET : _RESET) == flag_status)
			break;
		delay--;
	} while (delay);

	if (delay == 0)
		return XSPI_TIMEOUT;
	else
		return XSPI_SUCCESS;
}

static uint8_t set_xspi_mode(uint8_t mode)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t config = 0;

	/* check idle status */
	if (ret = check_xspi_reg_flag(XSPI_CTRL_STATUS, CTRL_BUSY, _RESET, MAX_TIMEOUT_CNT))
		return ret;

	config = readl(XSPI_CTRL_CONFIG);
	if (mode == MODE_PIO)
		config |= ACMD_MODE;
	else if (mode == MODE_STIG)
	{
		/* STIG mode bit[6:5]=0x01 */
		config |= (1 << 5U);
		config &= ~(1 << 6U);
	}
	writel(config, XSPI_CTRL_CONFIG);

	return XSPI_SUCCESS;
}

static uint8_t get_xspi_mode()
{
	uint32_t config = 0;
	uint8_t tmp = 0;

	config = readl(XSPI_CTRL_CONFIG);
	tmp = (config >> 5) & 0x3;

	if (tmp == 0x3)
		return MODE_PIO;
	else if (tmp == 0x1)
		return MODE_STIG;
	else
		return MODE_NUM;
}

static uint8_t get_xspi_cmd_status()
{
	uint8_t ret = XSPI_ERROR;
	uint32_t status = 0;
	uint8_t mode = 0;

	/* check latest operation status */
	if (ret = check_xspi_reg_flag(XSPI_CMD_STATUS, XSPI_COMPLETE, _SET, MAX_TIMEOUT_CNT))
		return ret;
	
	status = readl(XSPI_CMD_STATUS);
	mode = get_xspi_mode();

	if (status & XSPI_CMD_ERROR)
		return XSPI_ERROR_CMD;
	else if (status & XSPI_BUS_ERROR)
		return XSPI_ERROR_BUS;
	else if (status & XSPI_CRC_ERROR)
		return XSPI_ERROR_CRC;
	else if (status & XSPI_DQS_ERROR)
		return XSPI_ERROR_DQS;
	else if ((status & XSPI_DEVICE_ERROR) && (mode == MODE_PIO))
		return XSPI_ERROR_DEVICE;
	else if ((status & XSPI_ECC_CORR_ERROR) && (mode == MODE_PIO))
		return XSPI_ERROR_ECC_CORR;
	else if (status & XSPI_FAIL)
		return XSPI_ERROR;
	else
		return XSPI_SUCCESS;
}

static uint8_t xspi_stig_read(uint8_t cmd, uint8_t anum, uint32_t addr, uint8_t dnum, uint16_t *pVal)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t tmp = 0;

	if (pVal == 0 || dnum < 1 || dnum > 2 || anum > 3)
		return ret;

	if (anum == 0)
		addr = 0;

	if (get_xspi_mode() != MODE_STIG)
		if (ret = set_xspi_mode(MODE_STIG))
			return ret;

	/* check trd_0 idle */
	if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
		return ret;

	writel((0x01 | ((addr&0xff)<<24)), XSPI_CMD_REG1);	 	/* xspi profile 1.0 layout (with gluing) */
	writel(((((addr>>8)&0xff) | ((addr>>16)&0xff)<<8)), XSPI_CMD_REG2);
	writel(((cmd<<16) | (anum<<28)), XSPI_CMD_REG3);		/* cmd + ADDR_NO_OF_BYTES */
	writel(0x10000000, XSPI_CMD_REG4);						/* INSTR_LINK = 1; */
	writel(0x0, XSPI_CMD_REG0);

	/* add a DATA_SEQ */
	writel(0x100007F, XSPI_CMD_REG1);						/* type=127 & STATUS_SOURCE = 1; */
	writel((dnum << 16), XSPI_CMD_REG2);					/* DATA_NO_OF_BYTES */
	writel(0x0, XSPI_CMD_REG3);
	writel(0x0, XSPI_CMD_REG4);
	writel(0x0, XSPI_CMD_REG0);

	/* check operation state */
	if (ret = get_xspi_cmd_status())
		return ret;

	tmp = readl(XSPI_CMD_STATUS) >> 16;
	if (dnum == 1)
		*pVal = tmp & 0xff;
	else
		*pVal = tmp & 0xffff;

	return ret;
}

static uint8_t xspi_stig_write(uint8_t cmd, uint8_t anum, uint32_t addr, uint8_t dnum, uint16_t val)
{
	uint8_t ret = XSPI_ERROR;

	if (dnum > 2 ||  anum > 3)
		return ret;

	if (dnum == 1)
		val &= ~0xff00;

	if (dnum == 0)
		val = 0;

	if (anum == 0)
		addr = 0;

	if (get_xspi_mode() != MODE_STIG)
		if (ret = set_xspi_mode(MODE_STIG))
			return ret;

	/* check trd_0 idle */
	if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
		return ret;

	/* write enable, WEL bit=1 */
	writel(0x0, XSPI_CMD_REG1);
	writel(0x0, XSPI_CMD_REG2);
	writel((XSPI_CMD_WREN<<16), XSPI_CMD_REG3); /* WEL CMD =06h */
	writel(0x0, XSPI_CMD_REG4);
	writel(0x0, XSPI_CMD_REG0);

	if (ret = get_xspi_cmd_status())
		return ret;

	writel(((val<<8) | ((addr&0xff)<<24)), XSPI_CMD_REG1);
	writel((((addr>>8)&0xff) | (((addr>>16)&0xff)<<8)), XSPI_CMD_REG2);
	writel(((cmd<<16) | (dnum<<24) | (anum<<28)), XSPI_CMD_REG3); /* cmd + DATA_NO_OF_BYTES */
	writel(0x0, XSPI_CMD_REG4);
	writel(0x0, XSPI_CMD_REG0);

	return get_xspi_cmd_status();
}

static uint8_t is_issi_flash_ready()
{
	uint8_t ret = XSPI_ERROR;
	uint32_t cyc = MAX_TIMEOUT_CNT;
	uint16_t tmp = 0;

	do {
		if (ret = xspi_stig_read(XSPI_CMD_ISSI_RDSR, 0, 0, 1, &tmp))
			return ret;

		if ((tmp & 0x1) == 0)
			break;
	} while (--cyc);

	if (cyc == 0)
		return XSPI_TIMEOUT;

	return ret;
}

static uint8_t is_winbond_flash_ready()
{
	uint8_t ret = XSPI_ERROR;
	uint32_t cyc = MAX_TIMEOUT_CNT;
	uint16_t tmp = 0;

	do {
		if (ret = xspi_stig_read(XSPI_CMD_WB_RDSR1, 0, 0, 1, &tmp))
			return ret;

		if ((tmp & 0x1) == 0)
			break;
	} while (--cyc);

	if (cyc == 0)
		return XSPI_TIMEOUT;
	return ret;
}

static uint8_t is_gd_flash_ready()
{
	uint8_t ret = XSPI_ERROR;
	uint32_t cyc = MAX_TIMEOUT_CNT;
	uint16_t tmp = 0;

	do {
		if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR1, 0, 0, 1, &tmp))
			return ret;

		if ((tmp & 0x1) == 0)
			break;
	} while (--cyc);

	if (cyc == 0)
		return XSPI_TIMEOUT;
	return ret;
}

static uint8_t set_xspi_quad_mode()
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp = 0;
	uint16_t tmp2 = 0;
	
	if ((g_flash_mdid == MDID_WINBOND) || \
			(g_flash_mdid == MDID_WINBOND_13) || \
			(g_flash_mdid == MDID_WINBOND_16) || \
			(g_flash_mdid == MDID_WINBOND_18) || \
			(g_flash_mdid == MDID_WINBOND_4M))
	{
		/* 1. read SR2 */
		if (ret = xspi_stig_read(XSPI_CMD_WB_RDSR2, 0, 0, 1, &tmp))
			return ret;

		if (tmp & WB_QE_EN)  /* is already QPI */
			return XSPI_SUCCESS;

		/* 2. set SR2 bit1=1 as QPI mode */
		if (ret = xspi_stig_write(XSPI_CMD_WB_WRSR2, 0, 0, 1, WB_QE_EN))
			return ret;

		/* 3. check process done */
		return is_winbond_flash_ready();
	}
	else if (g_flash_mdid == MDID_ISSI || g_flash_mdid == MDID_ISSI_4M)
	{
		/* 1. read SR */
		if (ret = xspi_stig_read(XSPI_CMD_ISSI_RDSR, 0, 0, 1, &tmp))
			return ret;

		if (tmp & ISSI_QE_EN) /* is already QPI */
			return XSPI_SUCCESS;

		/* 2. set SR bit6=1 as QPI mode */
		if (ret = xspi_stig_write(XSPI_CMD_ISSI_WRSR, 0, 0, 1, ISSI_QE_EN))
			return ret;

		/* 3. check process done */
		return is_issi_flash_ready();
	}
	else if (g_flash_mdid == MDID_GD_4M)
	{
		/* 1. read SR2 */
		if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR2, 0, 0, 1, &tmp))
			return ret;

		if (tmp & GD_QE_EN)
			return XSPI_SUCCESS;

		/* 2. read SR1 */
		if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR1, 0, 0, 1, &tmp))
			return ret;
		/* 3. read SR2 */
		if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR2, 0, 0, 1, &tmp2))
			return ret;
		/* 4. set Quad En bit */
		tmp2 = (tmp2 & 0xff) | (0x1 << 1);
		tmp = tmp & 0xff;
		tmp = (tmp << 8) | tmp2;
		/* 5. write SR */
		if (ret = xspi_stig_write(XSPI_CMD_GD_WRSR, 0, 0, 2, tmp))
			return ret;
		/* check process done */
		return is_gd_flash_ready();
	}
	return ret;
}

static uint8_t xspi_reset(void)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t config = 0;

	config = readl(SMU_PHERPH_RST);
	config |= (0x1 << 10);
	writel(config, SMU_PHERPH_RST);

	usleep(1000);
	config &= ~(0x1 << 10);
	writel(config, SMU_PHERPH_RST);

	writel(0x05, (XSPI_BASE_ADDR + 0x8000));
	writel(0x0, (XSPI_BASE_ADDR + 0x8004));
	writel(0x14000a, (XSPI_BASE_ADDR + 0x8018));
	writel(0x7f, (XSPI_BASE_ADDR + 0x8000));

	ret = wait_xspi_init_comp(0xffffffff);

	return ret;
}

static uint8_t xspi_get_flash_mdid(uint16_t *pVal)
{
	uint8_t ret = XSPI_ERROR;
	if (pVal == 0)
		return ret;

	return xspi_stig_read(XSPI_CMD_RDMDID, 3, 0, 2, pVal);
}

static uint8_t xspi_flash_reset(void)
{
	uint8_t ret = XSPI_ERROR;

	if (get_xspi_mode() != MODE_STIG)
		if (ret = set_xspi_mode(MODE_STIG))
			return ret;

	/* check trd_0 idle */
	if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
		return ret;

	/* write enable, WEL bit=1 */
	writel(0x0, XSPI_CMD_REG1);
	writel(0x0, XSPI_CMD_REG2);
	writel((XSPI_CMD_RSTEN << 16), XSPI_CMD_REG3);
	writel(0x0, XSPI_CMD_REG4);
	writel(0x0, XSPI_CMD_REG0);

	if (ret = get_xspi_cmd_status())
		return ret;

	writel(0x0, XSPI_CMD_REG1);
	writel(0x0, XSPI_CMD_REG2);
	writel((XSPI_CMD_RST << 16), XSPI_CMD_REG3);
	writel(0x0, XSPI_CMD_REG4);
	writel(0x0, XSPI_CMD_REG0);

	return get_xspi_cmd_status();
}

/* src:mcu addr, dest:flash addr */
static uint8_t xspi_program(uint32_t dest, uint32_t data_cnt, uint8_t *buf)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t tmp = 0;
	
	if (data_cnt < 1)
		return ret;

	vastai_dump_write_buf(g_card_id, XSPI_CSRAM_DATA_ADDR, data_cnt, buf);

	if (g_xspi_dev.inited)
	{
		/* check trd_0 idle */
		if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
			return ret;

		writel(0, XSPI_CMD_STATUS_PTR);			 /* select trd 0 for ACMD mode */
		
		if (get_xspi_mode() != MODE_PIO)
			if (ret = set_xspi_mode(MODE_PIO))
				return ret;

		writel((uint32_t)dest, XSPI_CMD_REG1);		   /* xspi addr low */
		writel(0, XSPI_CMD_REG5);				   /* xspi_addr high */
		
		writel((uint32_t)XSPI_CSRAM_DATA_ADDR, XSPI_CMD_REG2);	 /* host sys_addr_ptr low */
		writel(0, XSPI_CMD_REG3);				   /* host sys_addr_ptr high */
		
		writel((data_cnt - 1), XSPI_CMD_REG4);	  /* data counter, how many data bytes */

		tmp = CMD_PROGRAM_DATA | DMA_MASTER | PIO_MODE;
		writel(tmp, XSPI_CMD_REG0);

		ret = get_xspi_cmd_status();
	}

	return ret;
}

/* src:flash addr, dest:mcu addr */
static uint8_t xspi_read(uint32_t src, uint32_t data_cnt, uint8_t *buf)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t tmp = 0;
	
	if (data_cnt < 1)
		return ret;

	if (g_xspi_dev.inited)
	{
		/* check trd_0 idle */
		if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
			return ret;

		writel(0, XSPI_CMD_STATUS_PTR);			/* select trd 0 for ACMD mode */
		
		if (get_xspi_mode() != MODE_PIO)
			if (ret = set_xspi_mode(MODE_PIO))
				return ret;

		writel((uint32_t)src, XSPI_CMD_REG1);	/* xspi flash addr l */
		writel(0, XSPI_CMD_REG5);				/* xspi addr high */

		writel((uint32_t)XSPI_CSRAM_DATA_ADDR, XSPI_CMD_REG2);
		writel(0, XSPI_CMD_REG3);				/* host sys_addr_ptr high */
		
		writel((data_cnt - 1), XSPI_CMD_REG4);	/* data counter, how many data bytes */

		tmp = CMD_READ_DATA | DMA_MASTER | PIO_MODE;
		writel(tmp, XSPI_CMD_REG0);

		ret = get_xspi_cmd_status();
	}

	vastai_dump_read_buf(g_card_id, XSPI_CSRAM_DATA_ADDR, data_cnt, buf);

	return ret;
}

static uint8_t xspi_erase(uint32_t addr, uint16_t sec_cnt)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t tmp = 0;
	
	if (((addr & 0xfff) != 0) || (sec_cnt < 1))
		return ret;
	
	if (g_xspi_dev.inited)
	{
		/* check trd_0 idle */
		if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
			return ret;
		
		writel(0, XSPI_CMD_STATUS_PTR);			 /* select trd 0 for ACMD mode */
		
		if (get_xspi_mode() != MODE_PIO)
			if (ret = set_xspi_mode(MODE_PIO))
				return ret;
			
		writel((uint32_t)addr, XSPI_CMD_REG1);		   /* xspi addr low */
		writel(0, XSPI_CMD_REG5);				   /* xspi_addr high */
		writel((sec_cnt - 1), XSPI_CMD_REG4);	   /* sector counter */
		
		tmp = CMD_ERASE_SECTOR | PIO_MODE;
		writel(tmp, XSPI_CMD_REG0);

		ret = get_xspi_cmd_status();
	}
	return ret;	
}

static uint8_t xspi_erase_full(void)
{
	uint8_t ret = XSPI_ERROR;
	uint32_t tmp = 0;

	if (g_xspi_dev.inited)
	{
		/* check trd_0 idle */
		if (ret = check_xspi_reg_flag(XSPI_TRD_STATUS, TRD_0_MSK, _RESET, MAX_TIMEOUT_CNT))
			return ret;

		/* select trd 0 for ACMD mode */
		writel(0, XSPI_CMD_STATUS_PTR);
		
		if (get_xspi_mode() != MODE_PIO)
			if (ret = set_xspi_mode(MODE_PIO))
				return ret;
		
		tmp = CMD_ERASE_FULL | PIO_MODE;
		writel(tmp, XSPI_CMD_REG0);

		ret = get_xspi_cmd_status();
	}

	return ret;
}

static uint8_t xspi_get_protect(uint8_t *pVal)
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp = 0;

	if (pVal == 0)
		return ret;

	if ((g_flash_mdid == MDID_WINBOND) || \
			(g_flash_mdid == MDID_WINBOND_13) || \
			(g_flash_mdid == MDID_WINBOND_16) || \
			(g_flash_mdid == MDID_WINBOND_18) || \
			(g_flash_mdid == MDID_WINBOND_4M))
	{
		if (ret = xspi_stig_read(XSPI_CMD_WB_RDSR3, 0, 0, 1, &tmp))
			return ret;

		if (tmp & WB_WPS_EN)
			g_xspi_dev.protected = LOCK;
		else
			g_xspi_dev.protected = UNLOCK;

		*pVal = g_xspi_dev.protected;
	}
	else if (g_flash_mdid == MDID_ISSI)
	{
		if (ret = xspi_stig_read(XSPI_CMD_ISSI_RDSR, 0, 0, 1, &tmp))
			return ret;

		tmp = (tmp >> 2) & 0xf;				/* bit[5:2] */
		if (tmp == ISSI_LOCK_ALL)
			g_xspi_dev.protected = LOCK;	/* bit[5:2]=1001b */
		else if (tmp == 0)
			g_xspi_dev.protected = UNLOCK;	/* bit[5:2]=0000b */
		else
			g_xspi_dev.protected = INVALID;

		*pVal = g_xspi_dev.protected;
	}
	else if (g_flash_mdid == MDID_ISSI_4M)
	{
		if (ret = xspi_stig_read(XSPI_CMD_ISSI_RDSR, 0, 0, 1, &tmp))
			return ret;

		tmp = (tmp >> 2) & 0xf;				/* bit[5:2] */
		if (tmp == ISSI_4M_LOCK_ALL)
			g_xspi_dev.protected = LOCK;	/* bit[5:2]=1001b */
		else if (tmp == 0)
			g_xspi_dev.protected = UNLOCK;	/* bit[5:2]=0000b */
		else
			g_xspi_dev.protected = INVALID;

		*pVal = g_xspi_dev.protected;
	}
	else if (g_flash_mdid == MDID_GD_4M)
	{
		// add BP op for GD flash
		if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR1, 0, 0, 1, &tmp))
			return ret;
		tmp = (tmp >> 2) & 0x1f;
		if (tmp == GD_4M_LOCK_ALL)
			g_xspi_dev.protected = LOCK;    /* bit[6:2]=01111b */
		else if (tmp == 0)
			g_xspi_dev.protected = UNLOCK;   /* bit[6:2]=00000b */
		else
			g_xspi_dev.protected = INVALID;

		*pVal = g_xspi_dev.protected;
	}

	return ret;
}

static uint8_t xspi_set_bp(uint8_t val)
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp = 0;
	uint32_t cyc = MAX_TIMEOUT_CNT;

	if (ret = xspi_stig_read(XSPI_CMD_ISSI_RDSR, 0, 0, 1, &tmp))
		return ret;

	tmp &= ~(0xf<<2);		/* clear bit[5:2] */
	tmp |= (val&0xf)<<2;	/* set block protect */

	if (ret = xspi_stig_write(XSPI_CMD_ISSI_WRSR, 0, 0, 1, tmp))
		return ret;

	return is_issi_flash_ready();
}

static uint8_t xspi_gd_set_bp(uint8_t val)
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp = 0;
	uint16_t tmp2 = 0;
	uint32_t cyc = MAX_TIMEOUT_CNT;

	/* 1. read SR1 */
	if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR1, 0, 0, 1, &tmp))
		return ret;
	tmp &= 0xff;
	tmp &= ~(0x1f << 2); // SR1 BP4:0 bit[6:2]
	tmp |= (val & 0x1f) << 2;
	/* 2. read SR2 */
	if (ret = xspi_stig_read(XSPI_CMD_GD_RDSR2, 0, 0, 1, &tmp2))
		return ret;
	/* 3. set Quad En bit */
	tmp2 &= 0xff;
	tmp = (tmp << 8) | tmp2;
	/* 4. write SR 16bits */
	if (ret = xspi_stig_write(XSPI_CMD_GD_WRSR, 0, 0, 2, tmp))
		return ret;
	/* check process done */
	return is_gd_flash_ready();
}

static uint8_t xspi_set_protect(uint8_t mode)
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp = 0;

	if (mode > 1)
		return ret;

	if ((g_flash_mdid == MDID_WINBOND) || \
			(g_flash_mdid == MDID_WINBOND_13) || \
			(g_flash_mdid == MDID_WINBOND_16) || \
			(g_flash_mdid == MDID_WINBOND_18) || \
			(g_flash_mdid == MDID_WINBOND_4M))
	{
		if (ret = xspi_stig_read(XSPI_CMD_WB_RDSR3, 0, 0, 1, &tmp))
			return ret;

		if (mode == UNLOCK)
			tmp &= ~WB_WPS_EN;		/* WPS=0 */
		else if (mode == LOCK)
			tmp |= WB_WPS_EN;		/* WPS=1 */

		if (ret = xspi_stig_write(XSPI_CMD_WB_WRSR3, 0, 0, 1, tmp))
			return ret;

		if (ret = is_winbond_flash_ready())
			return ret;

		g_xspi_dev.protected = mode;
	}
	else if (g_flash_mdid == MDID_ISSI)
	{
		if (mode == UNLOCK)
		{
			if (ret = xspi_set_bp(0))
				return ret;
		}
		else if (mode == LOCK)
		{
			if (ret = xspi_set_bp(ISSI_LOCK_ALL))
				return ret;
		}
		g_xspi_dev.protected = mode;
	}
	else if (g_flash_mdid == MDID_ISSI_4M)
	{
		if (mode == UNLOCK)
		{
			if (ret = xspi_set_bp(0))
				return ret;
		}
		else if (mode == LOCK)
		{
			if (ret = xspi_set_bp(ISSI_4M_LOCK_ALL))
				return ret;
		}
		g_xspi_dev.protected = mode;
	}
	else if (g_flash_mdid == MDID_GD_4M)
	{
		// add BP op for GD flash
		if (mode == UNLOCK)
		{
			if (ret = xspi_gd_set_bp(0))
				return ret;
		}
		else if (mode == LOCK)
		{
			if (ret = xspi_gd_set_bp(GD_4M_LOCK_ALL))
				return ret;
		}
		g_xspi_dev.protected = mode;
	}
	return ret;
}

static uint8_t xspi_secblklock_winbond(uint32_t addr)
{
	uint8_t ret = XSPI_ERROR;

	if (ret = xspi_stig_write(XSPI_CMD_WB_SECLOCK, 3, addr, 0, 0))
		return ret;

	return is_winbond_flash_ready();
}

static uint8_t xspi_secblklock_issi(uint32_t addr)
{
	uint8_t ret = XSPI_ERROR;

	if (ret = xspi_stig_write(XSPI_CMD_ISSI_SECLOCK, 0, 0, 0, 0))
		return ret;

	return is_issi_flash_ready();
}

static uint8_t xspi_secblkunlock_winbond(uint32_t addr)
{
	uint8_t ret = XSPI_ERROR;

	if (ret = xspi_stig_write(XSPI_CMD_WB_SECUNLOCK, 3, addr, 0, 0))
		return ret;

	return is_winbond_flash_ready();
}

static uint8_t xspi_secblkunlock_issi(uint32_t addr)
{
	uint8_t ret = XSPI_ERROR;

	if (ret = xspi_stig_write(XSPI_CMD_ISSI_SECUNLOCK, 3, addr, 0, 0))
		return ret;

	return is_issi_flash_ready();
}

static uint8_t xspi_setlockunlock(uint32_t addr, uint32_t size /* bytes */, uint8_t op /* lock or unlock */)
{
	uint8_t ret = XSPI_ERROR;
	uint8_t tmp = 0;
	uint32_t curr_addr = addr;
	uint32_t tail_addr = addr + size;
	uint8_t (*xspi_op)(uint32_t addr);
	uint32_t top_blk_pos = XSPI_TOP_BLK_POS;
	uint32_t bottom_blk_pos = XSPI_BTM_BLK_POS;
	uint32_t all_blk_pos = XSPI_ALL_BLKS_POS;

	if (g_flash_mdid == MDID_WINBOND_4M) {
		bottom_blk_pos = XSPI_4M_BTM_BLK_POS;
		all_blk_pos = XSPI_4M_ALL_BLKS_POS;
	}

	/* WinBond flash WPS=1 Write-Protected */
	if ((g_flash_mdid == MDID_WINBOND) || \
			(g_flash_mdid == MDID_WINBOND_13) || \
			(g_flash_mdid == MDID_WINBOND_16) || \
			(g_flash_mdid == MDID_WINBOND_18) || \
			(g_flash_mdid == MDID_WINBOND_4M))
	{
		xspi_op = (op == 1) ? xspi_secblklock_winbond : xspi_secblkunlock_winbond;

		while(curr_addr < top_blk_pos)
		{
			if (ret = xspi_op(curr_addr))
				return ret;

			curr_addr += XSPI_SEC_SIZE;
			if (curr_addr >= tail_addr)
				return XSPI_SUCCESS;
		}

		while(curr_addr < bottom_blk_pos)
		{
			if (ret = xspi_op(curr_addr))
				return ret;

			curr_addr += XSPI_BLK_SIZE;
			if (curr_addr >= tail_addr)
				return XSPI_SUCCESS;
		}

		while(curr_addr < all_blk_pos)
		{
			if (ret = xspi_op(curr_addr))
				return ret;

			curr_addr += XSPI_SEC_SIZE;
			if (curr_addr >= tail_addr)
				return XSPI_SUCCESS;
		}
	}

	/* ISSI flash can only enable one Sector Write-Protected at any time*/
	else if (g_flash_mdid == MDID_ISSI)
	{
		if (size > XSPI_SEC_SIZE)
		{
			if ((addr+size) <= XSPI_HALF_BLKS_POS)	// 0~1M
				tmp = (op == 1) ? ISSI_LOCK_ALL : ISSI_UNLOCK_0TO15;
			else {
				if (addr >= XSPI_HALF_BLKS_POS)		// 1M~2M
					tmp = (op == 1) ? ISSI_LOCK_ALL : ISSI_UNLOCK_16TO31;
				else								// 0~2M
					tmp = (op == 1) ? ISSI_LOCK_ALL : ISSI_UNLOCK_ALL;
			}
			return xspi_set_bp(tmp);
		}
		else
		{
			xspi_op = (op == 1) ? xspi_secblklock_issi : xspi_secblkunlock_issi;
			return xspi_op(addr);
		}
	}

	/* ISSI flash can only enable one Sector Write-Protected at any time */
	else if (g_flash_mdid == MDID_ISSI_4M)
	{
		if (size > XSPI_SEC_SIZE)
		{
			if ((addr+size) <= XSPI_4M_QUARTER_BLKS_POS)  /* 0~1M */
				tmp = (op == 1) ? ISSI_4M_LOCK_ALL : ISSI_4M_UNLOCK_0TO31;
			else {
				if (addr >= XSPI_4M_QUARTER_BLKS_POS)    /* 1M~4M */
					tmp = (op == 1) ? ISSI_4M_LOCK_ALL : ISSI_4M_UNLOCK_16TO63;
				else                                /* 0~4M */
					tmp = (op == 1) ? ISSI_4M_LOCK_ALL : ISSI_4M_UNLOCK_ALL;
			}
			return xspi_set_bp(tmp);
		}
		else
		{
			xspi_op = (op == 1) ? xspi_secblklock_issi : xspi_secblkunlock_issi;
				return xspi_op(addr);
		}
	}
	/* GD flash */
	else if (g_flash_mdid == MDID_GD_4M)
	{
		// add BP op for GD flash
		if ((addr+size) <= XSPI_4M_QUARTER_BLKS_POS)  /* 0~1M */
			tmp = (op == 1) ? GD_4M_LOCK_ALL : GD_4M_UNLOCK_0TO31;
		else
		{
			if (addr >= XSPI_4M_QUARTER_BLKS_POS)    /* 1M~4M */
				tmp = (op == 1) ? GD_4M_LOCK_ALL : GD_4M_UNLOCK_16TO63;
			else                                /* 0~4M */
				tmp = (op == 1) ? GD_4M_LOCK_ALL : GD_4M_UNLOCK_ALL;
		}
		return xspi_gd_set_bp(tmp);
	}
	return ret;
}

static uint8_t xspi_seclock()
{
	uint8_t ret = XSPI_ERROR;

	if (g_xspi_dev.protected == UNLOCK)
		return XSPI_SUCCESS;

	if (g_xspi_dev.sec.unlocked) {
		ret = xspi_setlockunlock(g_xspi_dev.sec.addr, g_xspi_dev.sec.size, 1);
		if (ret == XSPI_SUCCESS) {
			g_xspi_dev.sec.addr = 0;
			g_xspi_dev.sec.size = 0;
			g_xspi_dev.sec.unlocked = 0;
		}
	}
	return ret;
}

static uint8_t xspi_secUnLock(uint32_t addr, uint32_t size /* bytes */)
{
	uint8_t ret = XSPI_ERROR;

	if (g_xspi_dev.protected == UNLOCK)
		return XSPI_SUCCESS;

	if (!g_xspi_dev.sec.unlocked) {
		ret = xspi_setlockunlock(addr, size, 0);
		if (ret == XSPI_SUCCESS) {
			g_xspi_dev.sec.addr = addr;
			g_xspi_dev.sec.size = size;
			g_xspi_dev.sec.unlocked = 1;
		}
	}
	return ret;
}

/* only for WinBond flash */
static uint8_t xspi_get_sec_lock(uint32_t addr, uint8_t *pVal)
{
	uint8_t ret = XSPI_ERROR;
	uint16_t tmp;
	uint32_t len;
	uint32_t start_addr;

	if (pVal == 0)
		return ret;

	if ((g_flash_mdid == MDID_WINBOND) || \
			(g_flash_mdid == MDID_WINBOND_13) || \
			(g_flash_mdid == MDID_WINBOND_16) || \
			(g_flash_mdid == MDID_WINBOND_18) || \
			(g_flash_mdid == MDID_WINBOND_4M))
	{
		if ((addr >= XSPI_TOP_BLK_POS) && (addr < XSPI_BTM_BLK_POS))
			len = XSPI_BLK_SIZE;
		else
			len = XSPI_SEC_SIZE;

		start_addr = (addr / len) * len;

		if (g_xspi_dev.protected == LOCK)
		{
			if (ret = xspi_stig_read(XSPI_CMD_WB_GETSECLOCK, 3, start_addr, 1, &tmp))
				return ret;

			*pVal = (tmp & 0x1) ? 1 : 0;
		}
	}

	return ret;
}


static int xspi_init(void)
{
	uint8_t status;
	
	status = check_xspi_ctrl_sts();
	if(status >1 )
	{
		if (xspi_reset())
			return -1;
		if( 0 != check_xspi_ctrl_sts())
			return -1;
	}
	else if(status == 1)
	{
		return -1;
	}

	/* read flash manufacturer & device ID */
	if (xspi_get_flash_mdid(&g_flash_mdid))
		return -1;

	/* reset flash device */
	if (xspi_flash_reset())
		return -1;

	usleep(1000);

	/* set QPI mode */
	if (set_xspi_quad_mode())
		return -1;

	/* get protected mode */
	if (xspi_get_protect(&status))
		return -1;

#if (XSPI_LOCK_EN == 1)
	if (status != LOCK)
	{
		if (xspi_set_protect(LOCK))
			return 0;
	}
#else
	if (status != UNLOCK)
	{
		if (xspi_set_protect(UNLOCK))
			return -1;
	}
#endif

	g_xspi_dev.inited = 1;

	printf("XSPI init done.\n");

	return 0;
}

static void exit_handle(int signo) 
{
	vastai_dump_close(g_card_id);
	exit(0);
}

int main(int argc, char *argv[])
{
	int ret;
	int file_fd;
	struct stat file_stat;
	uint32_t file_len;
	uint8_t *tmp_buf;
	uint32_t record_len;
	uint32_t write_len;
	uint32_t addr;
	uint32_t sec_cnt;

	signal(SIGINT, exit_handle);

	if (argc < 2) {
		printf("ERROR: need flie name\n");
		return -1;
	}

	if (argc > 2) {
		g_card_id = strtol(argv[2], NULL, 10);
	}

	file_fd = open(argv[1], O_RDWR);
	if (file_fd < 0) {
		printf("ERROR: %s open failed\n", argv[1]);
		return -1;
	}
	fstat(file_fd, &file_stat);
	file_len = file_stat.st_size;
	if (file_len > 0x400000) {
		printf("ERROR: %s file to long\n", argv[1]);
		close(file_fd);
		return -1;
	}
	
	ret = vastai_dump_open(g_card_id);
	if (ret) {
		printf("ERROR: dump_open failed\n");
		return -1;
	}

	ret = xspi_init();
	if (ret) {
		printf("ERROR: flash init failed\n");
		close(file_fd);
		vastai_dump_close(g_card_id);
		return -1;
	}

	tmp_buf = (uint8_t *)malloc(file_len);
	lseek(file_fd, 0, SEEK_SET);
	ret = read(file_fd, tmp_buf, file_len);
	if (-1 == ret) {
		printf("ERROR: read %s fail\n", argv[1]);
	}
	close(file_fd);
	xspi_secUnLock(0, file_len);
	sec_cnt = file_len % XSPI_SEC_SIZE ? (file_len / XSPI_SEC_SIZE + 1) : (file_len / XSPI_SEC_SIZE);
	xspi_erase(0, sec_cnt);
	record_len = 0;
	printf("Begin burn %s to flash.\n", argv[1]);
	printf("buring [%3d%%]", 0);
	while (record_len < file_len) {
		write_len = ((file_len - record_len) > XSPI_SEC_SIZE) ? XSPI_SEC_SIZE : (file_len - record_len);
		xspi_program(record_len , write_len, tmp_buf + record_len);
		record_len += write_len;
		printf("\b\b\b\b\b\b[%3d%%]", record_len * 100 / file_len);
		fflush(stdout);
	}
	xspi_seclock();
	printf("\n");
	free(tmp_buf);

	vastai_dump_close(g_card_id);

	return 0;
}

