#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <dirent.h>
#include <sys/stat.h>
#include "../include/vastai_pcie_dump_api.h"

typedef union logsys_msg_info_s
{
	struct
	{
		uint16_t msg_level	: 3;
		uint16_t msg_type	: 3;
		uint16_t msg_len	: 10;
	};
	uint16_t n_val;
}logsys_msg_info_t;

typedef struct logsys_header_s
{
	uint8_t head_flag;
	uint8_t log_sn;
	logsys_msg_info_t msg_info;
	uint32_t timestamp;
} logsys_header_t;

#define LOG_SIZE 0x100000
static char g_log_buf[LOG_SIZE*2];
static uint32_t g_card_id = 0;

static char *dump_memcpy(void * dst, const void *src, uint32_t num)
{
	uint8_t *dest = dst;
	const uint8_t *source = src;
	uint32_t *aligned_dest;
	const uint32_t *aligned_src;

	/* If the size is small, or either SRC or DST is unaligned,
	then punt into the byte copy loop.  This should be rare. */
	if (!(num < 4) && !(((uint64_t)source & 3) | ((uint64_t)dest & 3))) {
		aligned_dest = (uint32_t *)dest;
		aligned_src = (uint32_t *)source;

		/* Copy 4X long words at a time if possible. */
		while (num >= 16) {
			*aligned_dest++ = *aligned_src++;
			*aligned_dest++ = *aligned_src++;
			*aligned_dest++ = *aligned_src++;
			*aligned_dest++ = *aligned_src++;
			num -= 16;
		}

		/* Copy one long word at a time if possible. */
		while (num >= 4) {
			*aligned_dest++ = *aligned_src++;
			num -= 4;
		}

		/* Pick up any residual with a byte copier. */
		dest = (uint8_t *)aligned_dest;
		source = (uint8_t *)aligned_src;
	}

	while (num--)
		*dest++ = *source++;

	return dst;
}

int main(int argc, char *argv[])
{
	int ret;
	uint64_t log_addr = 0;
	uint32_t log_rp = 0;
	uint32_t log_cnt = 0;
	char print_log_buf[256];
	uint32_t print_log_len;
	logsys_header_t *log_header = NULL;
	uint32_t block_end_flag;
	uint32_t log_size;
	
	if (argc < 2) {
		printf("ERROR: need log address\n");
		return -1;
	}

	if ((!strncmp(argv[1], "0x", 2)) || (!strncmp(argv[1], "0X", 2))) {
		log_addr = strtol(argv[1], NULL, 16);
	}
	else {
		log_addr = strtol(argv[1], NULL, 10);
	}

	if (argc > 2) {
		g_card_id = strtol(argv[2], NULL, 10);
	}

	if(log_addr > 0x860000 && log_addr < 0x880000)
	log_size = 0x20000;
	else
	log_size = LOG_SIZE;
	ret = vastai_dump_open(g_card_id);
	if (ret) {
		printf("ERROR: va_dump_open failed\n");
		return -1;
	}

	ret = vastai_dump_read_buf(g_card_id, log_addr, log_size, g_log_buf);
	if (ret)
	{
		printf("va_dump_read_buf fail\n");
		return -1;
	}

	while(1)
	{
		log_header = (logsys_header_t *)(g_log_buf + log_rp);
		if (log_header->head_flag == '#')
		{

			print_log_len = sprintf(print_log_buf, "[%d][%d]", log_cnt, log_header->timestamp);
			log_cnt++;
			dump_memcpy((void *)(print_log_buf + print_log_len), (void *)(g_log_buf + log_rp + 8), log_header->msg_info.msg_len);
			print_log_len += log_header->msg_info.msg_len;
			print_log_buf[print_log_len] = '\0';
			printf("%s", print_log_buf);
			log_rp = log_rp + 8 + ((log_header->msg_info.msg_len + 3) / 4) * 4;
			if (log_rp >= LOG_SIZE)
				break;
		}
		else
		{
			block_end_flag = *(uint32_t *)(g_log_buf + log_rp);
			if (block_end_flag == 0x5A3B96D2)
			{
				log_rp = (log_rp / 0x1000 + 1) * 0x1000;
				if (log_rp >= LOG_SIZE)
					break;
			}
			else
				break;
		}
	}

	vastai_dump_close(g_card_id);

	return 0;
}

