#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <dirent.h>
#include "../include/vastai_pcie_jtag_api.h"

int main(int argc, char *argv[])
{
    int ret = 0;
    uint8_t tdo[16];
    uint8_t ir_code;
    uint8_t tdi[5] = {0x12,0x34,0x56,0x78,0};
    int device_id = 0;
    uint32_t data;
    
    /* demo 1: get main tap id */
    ret = vastai_jtag_open(0);
    if (ret) {
        printf("jtag_open failed\n");
        return ret;
    }

    /* device_id : 0x10F03 */
    device_id = vastai_jtag_get_device_id(0);
    printf("device id: 0x%x\n", device_id);

    vastai_jtag_close(0);



    /* demo 2: config slave tap sel and FRA */
    ret = vastai_jtag_open(0);
    if (ret)
    {
        printf("jtag_open failed\n");
        return ret;
    }

    /* 50Mhz */
    vastai_jtag_set_clock(0, 50);

    vastai_jtag_main_tap_ir_sel(0, IR_HOSTIJTAG_BYPASS_SLAVE_TAP);
    memset((void *)tdo, 0, sizeof(tdo));
    vastai_jtag_set_dr(0, tdi, tdo, 23);
    printf("IR_HOSTIJTAG_BYPASS_SLAVE_TAP : %0x %0x\n", tdo[0], tdo[1]);
    /* do again for get write data */
    memset((void *)tdo, 0, sizeof(tdo));
    vastai_jtag_set_dr(0, tdi, tdo, 23);
    printf("IR_HOSTIJTAG_BYPASS_SLAVE_TAP : %0x %0x\n", tdo[0], tdo[1]);

    ir_code = IR_HOSTIJTAG_FRA;
    vastai_jtag_set_ir(0, &ir_code, NULL, 5);
    memset((void *)tdo, 0, sizeof(tdo));
    vastai_jtag_set_dr(0, &tdi[0], tdo, 30);
    printf("IR_HOSTIJTAG_FRA : %0x %0x %0x\n", tdo[0], tdo[1], tdo[2]);
    /* do again for get write data */
    memset((void *)tdo, 0, sizeof(tdo));
    vastai_jtag_set_dr(0, &tdi[0], tdo, 30);
    printf("IR_HOSTIJTAG_FRA : %0x %0x %0x\n", tdo[0], tdo[1], tdo[2]);
    
    vastai_jtag_close(0);



    /* demo 3: vastai noc read write */
    ret = vastai_jtag_open(0);
    if (ret) {
        printf("jtag_open failed\n");
        return ret;
    }
    
    vastai_jtag_axi_read(0, 0x800000, &data);
    printf("addr:0x%lx, value:0x%x\n", (uint64_t)0x800000, data);
    ret = vastai_jtag_axi_write(0, 0x800000, 0x12345678);
    if (ret) {
        printf("jtag_axi_write failed\n");
    }
    vastai_jtag_axi_read(0, 0x800000, &data);
    printf("addr:0x%lx, value:0x%x\n", (uint64_t)0x800000, data);

    vastai_jtag_axi_read(0, 0x1038000000, &data);
    printf("addr:0x%lx, value:0x%x\n", (uint64_t)0x1038000000, data);
    ret = vastai_jtag_axi_write(0, 0x1038000000, 0x5A5AA5A5);
    if (ret) {
        printf("jtag_axi_write failed\n");
    }
    vastai_jtag_axi_read(0, 0x1038000000, &data);
    printf("addr:0x%lx, value:0x%x\n", (uint64_t)0x1038000000, data);

    vastai_jtag_close(0);



    /* demo 4: read/write efuse sram */
    ret = vastai_jtag_open(0);
    if (ret) {
        printf("jtag_open failed\n");
        return ret;
    }

    vastai_jtag_axi_read(0, 0x10000, &data);
    printf("jtag_axi_read addr:0x%lx, value:0x%x\n", (uint64_t)0x10000, data);
    data = 0;
    vastai_jtag_fuse_sram_read(0, 0x10000, &data);
    printf("jtag_fuse_sram_read addr:0x%lx, value:0x%x\n", (uint64_t)0x10000, data);

    ret = vastai_jtag_fuse_sram_write(0, 0x10000, 0x12345678);
    if (ret) {
        printf("jtag_fuse_sram_write failed\n");
    }
    printf("jtag_fuse_sram_write done\n");
    
    vastai_jtag_fuse_sram_read(0, 0x10000, &data);
    printf("jtag_fuse_sram_read addr:0x%lx, value:0x%x\n", (uint64_t)0x10000, data);
    data = 0;
    vastai_jtag_axi_read(0, 0x10000, &data);
    printf("jtag_axi_read addr:0x%lx, value:0x%x\n", (uint64_t)0x10000, data);

    vastai_jtag_close(0);

    return 0;
}

