#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include "vastai_export_enum_define.h"
#include "vastai_pcie_user_api.h"
#include "vastai_pci_test.h"
#include "diag_pcie_user_api.h"

//diag test mode
int diag_vastai_pcie_write_reg_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id)
{
	if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
	return vastai_pcie_write_reg_buf(address, (void *)&value, 4, dev_id,
					 die_id);
}

int diag_vastai_pcie_write_reg_64bit(unsigned long long address,
				unsigned long long value, int dev_id,
				int die_id)
{
	if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
	return vastai_pcie_write_reg_buf(address, (void *)&value, 8, dev_id,
					 die_id);
}

int diag_vastai_pcie_read_reg_32bit(unsigned long long address, unsigned int *value,
			       int dev_id, int die_id)
{
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
	return vastai_pcie_read_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

int diag_vastai_pcie_read_reg_64bit(unsigned long long address,
			       unsigned long long *value, int dev_id,
			       int die_id)
{
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
	return vastai_pcie_read_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

int diag_vastai_pcie_write_reg_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
    return vastai_pcie_write_reg_buf(address,buf,buf_size_byte,dev_id,die_id);
}

int diag_vastai_pcie_read_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id)
{
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
    return vastai_pcie_read_reg_buf(address, buf,buf_size_byte,dev_id,die_id);
}

int diag_vastai_pcie_write_reg_file(const char *file_path, int offset,
			       unsigned long long address, int len_byte,
			       int dev_id, int die_id)
{

    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}

    return vastai_pcie_write_reg_file(file_path,offset,address,len_byte,dev_id,die_id);

	
}

//the buf_size_byte is not align
int diag_vastai_pcie_write_buf_unalign(unsigned long long address, const void *buf,
				  int buf_size_byte, int dev_id, int die_id)
{	
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}

	return vastai_pcie_write_buf_unalign(address,buf,buf_size_byte,dev_id,die_id);

}

int diag_vastai_pcie_read_buf_unalign(unsigned long long address, void *buf,
				 int buf_size_byte, int dev_id, int die_id)
{
	
    if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
    return vastai_pcie_read_buf_unalign(address,buf,buf_size_byte,dev_id,die_id);

}

int diag_vastai_pcie_read_reg_file(const char *file_path, int offset,
			      unsigned long long address, int len_byte,
			      int dev_id, int die_id)
{
	if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}

    return vastai_pcie_read_reg_file(file_path, offset,address,len_byte,dev_id,die_id);
}

/*
mode: 0:bit clear 1: bit set 
*/
int diag_vastai_pci_modify_reg32(unsigned long long address,unsigned int mask_bit,
								unsigned int mode,int dev_id)
{
	if (address>0 && address<SPACE_SIZE_256M) {
		address += (SPACE_SIZE_64G - SPACE_SIZE_256M); //64G-256M=0xFF0000000ULL
	}
    return vastai_pci_modify_reg32(address,mask_bit,mode, dev_id);
}
