#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vastai_export_enum_define.h"
#include "vastai_pcie_user_api.h"

/*====================common api====================*/
#define ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))

/*====================read_fn====================*/
#define READ_USAGE_STRING "mem <address> [size = 4 Byte] [dev_id = 0]"
static int read_fn(int argc, char *argv[])
{
	int size_byte = 4;
	int dev_id = 0;
	unsigned long long address = 0;
	/* check arg */
	if (argc < 2 || argc > 4) {
		printf("read usage: %s\n", READ_USAGE_STRING);
		return 0;
	}

	char *endptr;
	switch (argc) {
	case 4:
		dev_id = strtol(argv[3], &endptr, 0);
		/* FALL THROUGH */
	case 3:
		size_byte = strtol(argv[2], &endptr, 0);
		/* FALL THROUGH */
	case 2:
		address = strtoull(argv[1], &endptr, 0);
		break;
	default:
		assert(0);
	}

	int ret, i;

	for (i = 0; i < size_byte; i += 4) {
		if (i % 16 == 0) {
			printf("\n0x%03llx:", address + i);
		}
		unsigned int value;
		ret = vastai_pcie_read_reg_32bit(address + i, &value, dev_id,
						 0);
		if (ret) {
			return ret;
		}
		printf(" 0x%08x", value);
	}
	printf("\n");

	return 0;
}

/*====================write_fn====================*/
#define WRITE_USAGE_STRING "w4 <address> <value> [dev_id = 0]"
static int write_fn(int argc, char *argv[])
{
	unsigned long long address = 0;
	unsigned long value = 0;
	int dev_id = 0;
	/* check arg */
	if (argc < 3 || argc > 4) {
		printf("write usage: %s\n", WRITE_USAGE_STRING);
		return 0;
	}

	char *endptr;

	switch (argc) {
	case 4:
		dev_id = strtol(argv[3], &endptr, 0);
		/* FALL THROUGH */
	case 3:
		address = strtoull(argv[1], &endptr, 0);
		value = strtoul(argv[2], &endptr, 0);
		break;
	default:
		assert(0);
	}

	address = strtoull(argv[1], &endptr, 0);
	value = strtoul(argv[2], &endptr, 0);
	return vastai_pcie_write_reg_32bit(address, value, dev_id, 0);
}

/*====================command_fn====================*/
#define COMMAND_USAGE_STRING "command <dev_id> <command_line>"
static int command_fn(int argc, char *argv[])
{
	int dev_id = 0;
	/* check arg */
	if (argc < 2) {
		printf("command usage: %s\n", COMMAND_USAGE_STRING);
		return 0;
	}

	char temp_buf[128] = { 0 };
	int string_size = 0, i;
	for (i = 1; i < argc; i++) {
		strncat(temp_buf, argv[i],
			(sizeof(temp_buf) - string_size - 1));
		string_size += strlen(argv[i]);
		strncat(temp_buf, " ", (sizeof(temp_buf) - string_size - 1));
		string_size += 1;
	}
	char *endptr;
	dev_id = strtol(argv[4], &endptr, 0);
	return vastai_pcie_cmd(temp_buf, dev_id);
}

/*====================cmd_table====================*/
struct {
	char *cmd_name;
	char *usage;
	int (*cmd_fn)(int argc, char *argv[]);
} cmd_table[] = {
	{ .cmd_name = "mem", .cmd_fn = read_fn, .usage = READ_USAGE_STRING },
	{ .cmd_name = "w4", .cmd_fn = write_fn, .usage = WRITE_USAGE_STRING },
	{ .cmd_name = "command",
	  .cmd_fn = command_fn,
	  .usage = COMMAND_USAGE_STRING },
};

int main(int argc, char *argv[])
{
	int i;
	/* check arg */
	if (argc < 2) {
		/* print usage */
		for (i = 0; i < (int)ARRAY_SIZE(cmd_table); i++) {
			printf("usage: %s\n", cmd_table[i].usage);
		}
		return 0;
	}

	/* exec cmd */
	for (i = 0; i < (int)ARRAY_SIZE(cmd_table); i++) {
		if (strcmp(cmd_table[i].cmd_name, argv[1]) == 0) {
			int ret = cmd_table[i].cmd_fn(argc - 1, &(argv[1]));
			if (ret) {
				printf("get error code %d\n", ret);
				return ret;
			}
		}
	}
	return 0;
}
