#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include "vastai_export_enum_define.h"
#include "vastai_pcie_user_api.h"
#include "vastai_pci_test.h"
#include "diag_pcie_user_api.h"

unsigned long get_time(void)
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return tv.tv_sec * 1000 + tv.tv_usec / 1000;
}

char *get_version(void)
{
	return __DATE__;
}

int open_pcie_kchar(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/kchar:%d", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output:  open /dev/kchar: dev_id failed!:XXXX(error)\n
	}
	return filep;
}

static int open_pcie_ioctl(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/va%d_ctl", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output:  open /dev/kchar: dev_id failed!:XXXX(error)\n
	}
	return filep;
}


// reset_type: 0: hot reset, 1: cond reinitial reset, 2: uncond reinitial reset, 3: flr
int vastai_do_reset(RST_TYPE_ENUM_T reset_type,	int dev_id, int die_id)
{
    int ret = 0;
	char temp_buf[128] = { 0 };
	int write_size = 0;
    
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return ERR_CODE(0);
	}
    
    if(reset_type == RST_TYPE_HOTR_RST) {
        strcpy(temp_buf, "hotset");
    }
    else if(reset_type == RST_TYPE_COND_REINIT_RST) {
        strcpy(temp_buf, "reset");
    }
    else if(reset_type == RST_TYPE_UNCOND_REINIT_RST) {
        strcpy(temp_buf, "uncond_reset");
    }
    else if(reset_type == RST_TYPE_FLR) {
        strcpy(temp_buf, "flr");
    }
    
    printf("do %s for die %d, dev_id=%d\n", temp_buf, die_id, dev_id);
	write_size = write(filep, temp_buf, strlen(temp_buf)+1);
    if(write_size < 0) {
        perror("write");
        ret = write_size;
    }
    
    return ret;
}


int vastai_pcie_write_reg_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id)
{
	return vastai_pcie_write_reg_buf(address, (void *)&value, 4, dev_id,
					 die_id);
}

static int __vastai_pcie_write_reg_buf(int filep, unsigned long long address,
				       const void *buf, int buf_size_byte,
				       int die_id)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "write_buf addr=0x%llx user_addr=0x%lx len=%d",
				 address|die_id*0,(unsigned long)buf, buf_size_byte);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return ERR_CODE(0);
	}

	return 0;
}

int vastai_pcie_write_reg_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return ERR_CODE(0);
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_pcie_write_reg_buf(filep, address + write_offset,
						  buf + write_offset,
						  write_size_once, die_id);
		if (ret) {
			ret = ERR_CODE(ret);
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pcie_write_reg_64bit(unsigned long long address,
				unsigned long long value, int dev_id,
				int die_id)
{
	return vastai_pcie_write_reg_buf(address, (void *)&value, 8, dev_id,
					 die_id);
}

static int __vastai_pcie_write_cfg_buf(int filep, unsigned long long address,
				       const void *buf, int buf_size_byte,
				       int die_id)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "wrcfg_buf addr=0x%llx user_addr=0x%lx len=%d",
				 address|die_id*0,(unsigned long)buf, buf_size_byte);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return ERR_CODE(0);
	}

	return 0;
}

int vastai_pcie_write_cfg_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return ERR_CODE(0);
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_pcie_write_cfg_buf(filep, address + write_offset,
						  buf + write_offset,
						  write_size_once, die_id);
		if (ret) {
			ret = ERR_CODE(ret);
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pcie_write_cfg_32bit(unsigned long long cfg_offset,
				unsigned long long value, int dev_id,
				int die_id)
{
	return vastai_pcie_write_cfg_buf(cfg_offset, (void *)&value, 4, dev_id,
					 die_id);
}


static int __vastai_pcie_read_cfg_buf(int filep, unsigned long long address,
				      void *buf, int buf_size_byte, int die_id)
{
	char temp_buf[128] = { 0 };
	int lseek_size = 0;
	int temp_buf_size =
		snprintf(temp_buf, sizeof(temp_buf),
			 "rdcfg_buf addr=0x%llx len=%d",
			 address|die_id*0, buf_size_byte);
	int write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("write dev file error"); //output:write dev file error:XXXX\n
		return ERR_CODE(0);
	}

	lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("read_reg_buf: lseek set error"); //output:read_reg_buf: lseek set error:XXXX\n
		return -1;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		/* error handle */
		perror("read dev file error"); //output:read dev file error:XXXX\n
		return ERR_CODE(0);
	}

	return 0;
}


int vastai_pcie_read_cfg(unsigned long long address, void *buf,
				 int buf_size_byte, int dev_id, int die_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}
	while (buf_size_byte > 0) {
		if (buf_size_byte > 4*1024)
			read_size_once = 4*1024;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pcie_read_cfg_buf(filep, address + read_offset,
						 buf + read_offset,
						 read_size_once, die_id);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}
	close(filep);
	return ret;
}

int vastai_pcie_read_cfg_32bit(unsigned long long cfg_offset, unsigned int *value,
			       int dev_id, int die_id)
{
	return vastai_pcie_read_cfg(cfg_offset, (void *)value, 4,
					dev_id, die_id);
}



static int __vastai_pci_mmio_write_buf(int filep, int bar_num, unsigned long long offset,
				       const void *buf, int buf_size_byte)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "mmio_write_buf bar=%x offset=0x%llx user_addr=0x%lx len=%d",
				 bar_num,offset, (unsigned long)buf, buf_size_byte);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return -1;
	}

	return 0;
}

int vastai_pci_mmio_write_buf(int bar_num,unsigned long long offset, const void *buf,
			      int buf_size_byte, int dev_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return ERR_CODE(0);
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_pci_mmio_write_buf(filep, bar_num , offset + write_offset,
						  buf + write_offset, write_size_once);
		if (ret) {
			ret = ERR_CODE(ret);
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pci_mmio_write_32bit(int bar_num,unsigned long long offset,
				unsigned long long value, int dev_id)
{
	return vastai_pci_mmio_write_buf(bar_num, offset, (void *)&value, 4, dev_id);
}


static int __vastai_pci_mmio_read_buf(int filep, int bar_num, unsigned long long offset,
				      void *buf, int buf_size_byte)
{
	char temp_buf[128] = { 0 };
	int lseek_size = 0;
	int temp_buf_size =
		snprintf(temp_buf, sizeof(temp_buf),
			 "mmio_read_buf bar=%x offset=0x%llx len=%d",
			 bar_num, offset, buf_size_byte);
	int write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("write dev file error"); //output:write dev file error:XXXX\n
		return ERR_CODE(0);
	}

	lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("read_reg_buf: lseek set error"); //output:read_reg_buf: lseek set error:XXXX\n
		return -1;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		/* error handle */
		perror("read dev file error"); //output:read dev file error:XXXX\n
		return ERR_CODE(0);
	}

	return 0;
}


int vastai_pci_mmio_read(int bar_num, unsigned long long offset, void *buf,
				 int buf_size_byte, int dev_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}
	while (buf_size_byte > 0) {
		if (buf_size_byte > 4*1024)
			read_size_once = 4*1024;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pci_mmio_read_buf(filep, bar_num, offset + read_offset,
						 buf + read_offset,
						 read_size_once);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}
	close(filep);
	return ret;
}

int vastai_pci_mmio_read_32bit(int bar_num, unsigned long long offset, unsigned int *value, int dev_id)
{
	return vastai_pci_mmio_read(bar_num, offset, (void *)value, 4,dev_id);
}


static int __vastai_modify_reg32(int filep, unsigned long long addr,
				       unsigned int mask_bit, unsigned int mode)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "modify_csr_bit  addr=0x%llx maskbit=0x%x mode=%d",
				 addr, mask_bit, mode);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return -1;
	}

	return 0;
}

int vastai_pci_modify_reg32(unsigned long long address,unsigned int mask_bit,
								unsigned int mode,int dev_id)
{
	int ret = 0;	
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return -1;
	}

	ret = __vastai_modify_reg32(filep,address,mask_bit,mode);

	close(filep);
	return ret;
}

static int read_buf_from_user_file(const char *file_path, int offset, void *buf,
				   int buf_size_byte)
{
	int ret = 0;
	int filep = open(file_path, O_RDWR);
	if (filep < 2) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output: open file_path failed!:XXXXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	int lseek_size = lseek(filep, offset, SEEK_SET);
	if (lseek_size != offset) {
		perror("lseek src file error, pls check ofst"); //output:lseek src file error, pls check ofst:XXX\n
		ret = ERR_CODE(0);
		goto CLOSE_SRC_FILE;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		perror("read src file error"); //output:read src file error:XXX\n
		ret = ERR_CODE(0);
	}

CLOSE_SRC_FILE:
	close(filep);
RET_ERROR_CODE:
	return ret;
}

int vastai_pcie_write_reg_file(const char *file_path, int offset,
			       unsigned long long address, int len_byte,
			       int dev_id, int die_id)
{
	int ret;
	if (len_byte % 4) {
		perror("len_byte must be align with 4Byte"); //output: len_byte must be align with 4Byte:XXX\n
		return ERR_CODE(0);
	}
	void *read_buffer = malloc(ROUND_UP(len_byte, 4096));
	if (read_buffer == NULL) {
		perror("alloc read buffer error"); //output: alloc read buffer error:XXX\n
		return ERR_CODE(0);
	}

	ret = read_buf_from_user_file(file_path, offset, read_buffer, len_byte);
	if (ret) {
		ret = ERR_CODE(ret);
		goto FREE_READ_BUF;
	}

	ret = vastai_pcie_write_reg_buf(address, read_buffer, len_byte, dev_id,
					die_id);
	if (ret) {
		ret = ERR_CODE(ret);
	}

FREE_READ_BUF:
	free(read_buffer);
	return ret;
}

int vastai_pcie_write_buf_unalign(unsigned long long address, const void *buf,
				  int buf_size_byte, int dev_id, int die_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return ERR_CODE(0);
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_pcie_write_reg_buf(filep, address + write_offset,
						  buf + write_offset,
						  write_size_once, die_id);
		if (ret) {
			ret = ERR_CODE(ret);
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}
static int __vastai_pcie_read_reg_buf(int filep, unsigned long long address,
				      void *buf, int buf_size_byte, int die_id)
{
	char temp_buf[128] = { 0 };
	int lseek_size = 0;
	int temp_buf_size =
		snprintf(temp_buf, sizeof(temp_buf),
			 "read_buf addr=0x%llx len=%d",
			 address|die_id*0, buf_size_byte);
	int write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("write dev file error"); //output:write dev file error:XXXX\n
		return ERR_CODE(0);
	}

	lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("read_reg_buf: lseek set error"); //output:read_reg_buf: lseek set error:XXXX\n
		return -1;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		/* error handle */
		perror("read dev file error"); //output:read dev file error:XXXX\n
		return ERR_CODE(0);
	}

	return 0;
}

int vastai_pcie_read_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXXX\n
		return -2;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			read_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pcie_read_reg_buf(filep, address + read_offset,
						 buf + read_offset,
						 read_size_once, die_id);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pcie_read_buf_unalign(unsigned long long address, void *buf,
				 int buf_size_byte, int dev_id, int die_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}
	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			read_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pcie_read_reg_buf(filep, address + read_offset,
						 buf + read_offset,
						 read_size_once, die_id);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}
	close(filep);
	return ret;
}

static int __vastai_pcie_fread_reg_buf(int filep, unsigned long long address,
				      void *buf, int buf_size_byte, int die_id)
{
	char temp_buf[128] = { 0 };
	int lseek_size = 0;
	int temp_buf_size =
		snprintf(temp_buf, sizeof(temp_buf),
			 "fread_buf addr=0x%llx len=%d",
			 address|die_id*0, buf_size_byte);
	int write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("write dev file error"); //output:write dev file error:XXXX\n
		return -1;
	}

	lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("read_reg_buf: lseek set error"); //output:read_reg_buf: lseek set error:XXXX\n
		return -1;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		/* error handle */
		perror("read dev file error"); //output:read dev file error:XXXX\n
		return -1;
	}

	return 0;
}

int vastai_pcie_fread_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXXX\n
		return -2;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			read_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pcie_fread_reg_buf(filep, address + read_offset,
						 buf + read_offset,
						 read_size_once, die_id);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pcie_fread_reg_32bit(unsigned long long address, unsigned int *value,
			    int dev_id, int die_id)
{
	return vastai_pcie_fread_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

int vastai_pcie_fread_reg_64bit(unsigned long long address, unsigned long long *value,
			    int dev_id, int die_id)
{
	return vastai_pcie_fread_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

static int __vastai_pcie_fwrite_reg_buf(int filep, unsigned long long address,
				       const void *buf, int buf_size_byte,
				       int die_id)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "fwrite_buf addr=0x%llx user_addr=0x%lx len=%d",
				 address|die_id*0,(unsigned long)buf, buf_size_byte);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return -1;
	}

	return 0;
}

int vastai_pcie_fwrite_reg_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return -1;
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_pcie_fwrite_reg_buf(filep, address + write_offset,
						  buf + write_offset,
						  write_size_once, die_id);
		if (ret) {
			ret = -1;
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}

int vastai_pcie_fwrite_reg_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id)
{
	return vastai_pcie_fwrite_reg_buf(address, (void *)&value, 4, dev_id,
					die_id);
}

int vastai_pcie_fwrite_reg_64bit(unsigned long long address, unsigned long long value,
				int dev_id, int die_id)
{
	return vastai_pcie_fwrite_reg_buf(address, (void *)&value, 8, dev_id,
					die_id);
}

struct vastai_dev_buf_desc *open_vastai_dev_buf(int dev_id, char *name)
{
	struct vastai_dev_buf_desc *desc;
	char file_path[64] = { 0 };
	int ret;

	desc = malloc(sizeof(struct vastai_dev_buf_desc));
	if (!desc) {
		perror("malloc vastai_buf_desc error"); //output:malloc vastai_buf_desc error:XXXX\n
		return NULL;
	}

	if (name)
		snprintf(desc->name, VASTAI_DEV_BUF_DESC_NAME_LEN, "%s", name);

	snprintf(file_path, sizeof(file_path), "/dev/kchar:%d", dev_id);
	desc->filep = open(file_path, O_RDWR);
	if (desc->filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "%s open %s failed!",
			 desc->name, file_path);
		perror(temp_buf); //output: desc->name open /dev/kchar:dev_id failed!:XXXX\n
		goto free_desc;
	}

	ret = pthread_mutex_init(&desc->mutex, PTHREAD_MUTEX_TIMED_NP);
	if (ret) {
		perror("mutex init failed"); //output: mutex init failed:XXXX\n
		goto close_file;
	}

	return desc;
close_file:
	close(desc->filep);
free_desc:
	free(desc);
	return NULL;
}

int close_vastai_dev_buf(struct vastai_dev_buf_desc *desc)
{
	if (!desc) {
		perror("close_vastai_dev_buf error"); //output:close_vastai_dev_buf error:XXXX\n
		return -1;
	}

	pthread_mutex_destroy(&desc->mutex);
	close(desc->filep);
	free(desc);
	return 0;
}

int vastai_dev_buf_read(struct vastai_dev_buf_desc *desc,
			unsigned long long address, void *buf,
			int buf_size_byte, int die_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXX\n
		return -1;
	}

	if (buf_size_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXX\n
		return -2;
	}

	pthread_mutex_lock(&desc->mutex);
	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			read_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_pcie_read_reg_buf(desc->filep,
						 address + read_offset,
						 buf + read_offset,
						 read_size_once, die_id);

		if (ret) {
			ret = -3;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}
	pthread_mutex_unlock(&desc->mutex);

	return ret;
}

int vastai_pcie_read_reg_32bit(unsigned long long address, unsigned int *value,
			       int dev_id, int die_id)
{
	return vastai_pcie_read_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

int vastai_pcie_read_reg_64bit(unsigned long long address,
			       unsigned long long *value, int dev_id,
			       int die_id)
{
	return vastai_pcie_read_reg_buf(address, (void *)value, sizeof(*value),
					dev_id, die_id);
}

int vastai_pcie_read_reg_file(const char *file_path, int offset,
			      unsigned long long address, int len_byte,
			      int dev_id, int die_id)
{
	int ret;
	if (len_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	/* malloc a buf */
	void *read_buf = malloc(len_byte);
	if (!read_buf) {
		perror("alloc memory failed"); //output：alloc memory failed：XXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	/* read pcie reg into this buf */
	ret = vastai_pcie_read_reg_buf(address, read_buf, len_byte, dev_id,
				       die_id);
	if (ret != 0) {
		perror("read reg buf failed"); //output：read reg buf failed：XXX\n
		ret = ERR_CODE(ret);
		goto FREE_READ_BUF;
	}
	/* write this buf into file */
	int filep = open(file_path, O_RDWR | O_CREAT, 0777);
	if (filep < 2) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output：open file_path failed!：XXX\n
		ret = ERR_CODE(0);
		goto FREE_READ_BUF;
	}

	int lseek_size = lseek(filep, offset, SEEK_SET);
	if (lseek_size != offset) {
		perror("lseek src file error, pls check ofst"); //output：lseek src file error, pls check ofst：XXX\n
		ret = ERR_CODE(0);
		goto CLOSE_DST_FILE;
	}

	int write_size = write(filep, read_buf, len_byte);
	if (write_size != len_byte) {
		perror("write dst file error"); //output：write dst file error：XXX\n
		ret = ERR_CODE(0);
	}

CLOSE_DST_FILE:
	close(filep);
FREE_READ_BUF:
	free(read_buf);
RET_ERROR_CODE:
	return ret;
}

int vastai_pcie_cmd(const char *cmd_line, int dev_id)
{
	int ret = 0;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output：open dev file error：XXX\n
		return ERR_CODE(0);
	}

	int string_len = strlen(cmd_line);
	int write_size = write(filep, cmd_line, string_len + 1);
	if (write_size != string_len + 1) {
		perror("write dst file error"); //output：write dst file error：XXX\n
		ret = 0;
	}

	close(filep);
	return ret;
}

int compare_file(const char *path_a, const char *path_b, int offset,
		 int compare_size_byte)
{
	int ret = 0;
	int i;
	void *buf = malloc(compare_size_byte * 2);
	if (!buf) {
		perror("alloc memory failed"); //output：alloc memory failed：XXX\n
		ret = 0;
		return ret;
	}
	void *a_file_buf = buf;
	void *b_file_buf = (void *)((char *)buf + compare_size_byte);

	ret = read_buf_from_user_file(path_a, offset, a_file_buf,
				      compare_size_byte);
	if (ret != 0) {
		ret = -1;
		goto FREE_BUF;
	}

	ret = read_buf_from_user_file(path_b, offset, b_file_buf,
				      compare_size_byte);
	if (ret != 0) {
		ret = -1;
		goto FREE_BUF;
	}

	for (i = 0; i < compare_size_byte; i++) {
		if (((unsigned char *)a_file_buf)[i] !=
		    ((unsigned char *)b_file_buf)[i]) {
			ret = 0;
			goto FREE_BUF;
		}
	}
	ret = 1;

FREE_BUF:
	free(buf);
	return ret;
}



#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))

/*
 * Func: help user compare whether ddr values are consistent with gold value and output the difference
 *       unsigned int gold_ref[1][2] ={{0x02500060,0x00000002}};
 */
int compare_ddr_gold_val(unsigned long long ddr_addr, unsigned long long (*gold_ddr_value)[2], unsigned long long byte_size, int dev_id, int die_id)
{
	unsigned long long value_t[byte_size];
	printf("ARRAY_BYTE_SIZE: %lld\n", byte_size);
	int ret = vastai_pcie_read_reg_buf(ddr_addr, (void *)value_t, sizeof(value_t), dev_id, die_id); 
	if (ret != 0) {
		printf("ERROR: %s read ddr failed, errcode %d\n", __FUNCTION__, ret);
		return -1; 
	}
    
	for (int i = 0; i < (int)ARRAY_SIZE(value_t); i++) {
		if (gold_ddr_value[i][1] != value_t[i]) {
		    printf("ERROR: %s Different Match Compare.\n", __FUNCTION__);
			printf("Diff: gold_ddr_value=0x%llx vs. read_value=0x%llx on ddr_addr=0x%llx\n", gold_ddr_value[i][1], value_t[i], gold_ddr_value[i][0]);
			return -2;
		}
	}
	return 0;
}

int vastai_get_device_num(void)
{
	DIR *dir = opendir("/dev/");
	struct dirent *entry = NULL;
	int count = 0;

	while ((entry = readdir(dir)) != NULL) {
		if (strstr(entry->d_name, "kchar")) {
			count += 1;
		}
	}
	closedir(dir);

	return count;
}

/* just a sort function, don't care about performance */
void sort(int *array, int array_size)
{
	int i, j;

	for (i = 0; i < array_size; i++) {
		for (j = i + 1; j < array_size; j++) {
			if (array[i] > array[j]) {
				int temp = array[j];
				array[j] = array[i];
				array[i] = temp;
			}
		}
	}
}

/* this function return devid from index.
   at some docker env, we will not get kchar:0 at first
   since that, we need this function for search.
*/
int vastai_get_dev_id(int index)
{
	int dev_id[64];
	DIR *dir = opendir("/dev/");
	struct dirent *entry = NULL;
	int count = 0;

	while ((entry = readdir(dir)) != NULL) {
		if (strstr(entry->d_name, "kchar")) {
			sscanf(entry->d_name, "kchar:%d", &dev_id[count]);
			count += 1;
		}
	}

	sort(dev_id, count);
	closedir(dir);
	return dev_id[index];
}

int vastai_get_die_info(int dev_id, int die_index[], int *die_num)
{
	int ret = 0;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output：open dev file error：XXX\n
		return -1;
	}

	char cmd_line[] = "get_die_info";
	int string_len = strlen(cmd_line);
	int write_size = write(filep, cmd_line, string_len + 1);
	if (write_size != string_len + 1) {
		perror("write dst file error"); //output：write dst file error：XXX\n
		ret = -2;
		goto CLOSE_FILE;
	}
	int read_size = read(filep, die_num, 4);
	if (read_size != 4) {
		perror("read dst file error"); //output：read dst file error：XXX\n
		ret = -3;
		goto CLOSE_FILE;
	}

	read_size = read(filep, die_index, (*die_num) * 4);
	if (read_size != (*die_num) * 4) {
		perror("read dst file error"); //output：read dst file error：XXX\n
		ret = -4;
	}

CLOSE_FILE:
	close(filep);
	return ret;
}

int vastai_get_fw_version(int dev_id, void *buf, int buf_size)
{
	int ret = 0;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output：open dev file error：XXX\n
		return -1;
	}

	char cmd_line[] = "get_fw_version";
	int string_len = strlen(cmd_line);
	int write_size = write(filep, cmd_line, string_len + 1);
	if (write_size != string_len + 1) {
		perror("write firmware version error"); //output：write firmware version error：XXX\n
		ret = -2;
		goto CLOSE_FILE;
	}

	int read_size = read(filep, buf, buf_size);
	if (read_size != buf_size) {
		perror("read firmware version error"); //output：read firmware version error：XXX\n

		ret = -3;
		goto CLOSE_FILE;
	}

CLOSE_FILE:
	close(filep);
	return ret;
}

int vastai_get_cedar_asy_sts(int dev_id, void *buf, int buf_size)
{
	int ret = 0;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error\n");
		return -1;
	}

	char cmd_line[] = "cedar_asy";
	int string_len = strlen(cmd_line);
	int write_size = write(filep, cmd_line, string_len + 1);
	if (write_size != string_len + 1) {
		perror("write cedar asy sts error\n");
		ret = -2;
		goto CLOSE_FILE;
	}
	int read_size = read(filep, buf, buf_size);
	if (read_size != buf_size) {
		perror("read cedar asy sts error:");
		ret = -3;
		goto CLOSE_FILE;
	}

CLOSE_FILE:
	close(filep);
	return ret;
}

int vastai_get_cedar_sym_sts(int dev_id, void *buf, int buf_size)
{
	int ret = 0;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error\n");
		return -1;
	}

	char cmd_line[] = "cedar_sym";
	int string_len = strlen(cmd_line);
	int write_size = write(filep, cmd_line, string_len + 1);
	if (write_size != string_len + 1) {
		perror("write cedar sym sts error\n");
		ret = -2;
		goto CLOSE_FILE;
	}
	int read_size = read(filep, buf, buf_size);
	if (read_size != buf_size) {
		perror("read cedar sym sts error:");
		ret = -3;
		goto CLOSE_FILE;
	}

CLOSE_FILE:
	close(filep);
	return ret;
}
static u64 fill_dma_desc(struct user_dma_desc *dma_desc, int die_id, int ch,
			 int mode, int dir, struct desc_node *node, int num)
{
	int i;
	memset(dma_desc->cmd_str, 0x0, sizeof(dma_desc->cmd_str));
	strcpy(dma_desc->cmd_str, "DMA ");

	dma_desc->is_host_to_dev = dir;
	dma_desc->mode = mode;
	dma_desc->ch = ch;
	dma_desc->die_id = die_id;
	dma_desc->num = num;

	for (i = 0; i < num; i++) {
		dma_desc->desc_tab[i].dev_addr = node[i].dev_addr;
		dma_desc->desc_tab[i].host_addr = (u64)node[i].data;
		dma_desc->desc_tab[i].length = node[i].length;
	}

	return sizeof(dma_desc->desc_tab[0]) * num;
}

int vastai_sdma_read_buf_by_sg(void *user_addr, u64 dev_addr, u32 length, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);

	struct kchar_cmd kcmd = { .dma_trans_cmd.is_dev_to_host = 1,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)user_addr,
#else
					.dma_trans_cmd.vir_addr = user_addr,
#endif
					.dma_trans_cmd.axi_addr = dev_addr,
					.dma_trans_cmd.length = length,
					.dma_trans_cmd.die_index = dev_id,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}
	int ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS_BY_SG, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_DMA_TRANS_BY_SG is failed %d\n", ret);
	}
	close(filep);
	return ret;
}

int vastai_sdma_write_buf_by_sg(void *user_addr, u64 dev_addr, u32 length, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);

	struct kchar_cmd kcmd = { .dma_trans_cmd.is_dev_to_host = 0,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)user_addr,
#else
					.dma_trans_cmd.vir_addr = user_addr,
#endif
					.dma_trans_cmd.axi_addr = dev_addr,
					.dma_trans_cmd.length = length,
					.dma_trans_cmd.die_index = dev_id,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}
	int ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS_BY_SG, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_DMA_TRANS_BY_SG is failed %d\n", ret);
	}
	close(filep);
	return ret;
}

int vastai_get_va_pid(u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);

	struct kchar_cmd kcmd = { .get_va_pid_cmd.va_pid = 0,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_GET_VA_PID, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_GET_VA_PID is failed %d\n", ret);
	}
	close(filep);

	return kcmd.get_va_pid_cmd.va_pid;
}

int vastai_free_va_pid(int va_pid, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	struct kchar_cmd kcmd = { .get_va_pid_cmd.va_pid = va_pid,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_FREE_VA_PID, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_FREE_VA_PID is failed %d\n", ret);
	}
	close(filep);

	return 0;
}

int vastai_sdma_read_buf_by_user_addr(void *user_addr, u64 dev_addr, u32 length, u32 dev_id, u32 pid)
{
	int filep = open_pcie_ioctl(dev_id);

	struct kchar_cmd kcmd = { .dma_trans_cmd.is_dev_to_host = 1,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)user_addr,
#else
					.dma_trans_cmd.vir_addr = user_addr,
#endif
					.dma_trans_cmd.axi_addr = dev_addr,
					.dma_trans_cmd.length = length,
					.dma_trans_cmd.die_index = dev_id,
					.dma_trans_cmd.pid = pid,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}
	int ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS_BY_VA, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_DMA_TRANS_BY_VA is failed %d\n", ret);
	}
	close(filep);
	return ret;
}

int vastai_sdma_write_buf_by_user_addr(void *user_addr, u64 dev_addr, u32 length, u32 dev_id, u32 pid)
{
	int filep = open_pcie_ioctl(dev_id);

	struct kchar_cmd kcmd = { .dma_trans_cmd.is_dev_to_host = 0,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)user_addr,
#else
					.dma_trans_cmd.vir_addr = user_addr,
#endif
					.dma_trans_cmd.axi_addr = dev_addr,
					.dma_trans_cmd.length = length,
					.dma_trans_cmd.die_index = dev_id,
					.dma_trans_cmd.pid = pid,};
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}
	int ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS_BY_VA, &kcmd);
	if (ret < 0) {
		printf("VA_PCI_IOCTL_DMA_TRANS_BY_VA is failed %d\n", ret);
	}
	close(filep);
	return ret;
}

int vastai_sdma_read_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_index)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;

	struct kchar_cmd dma_cmd = {.dma_trans_cmd.is_dev_to_host = 1,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)buf,
#else
					.dma_trans_cmd.vir_addr = buf,
#endif
					 .dma_trans_cmd.axi_addr = address,
					.dma_trans_cmd.length = ROUND_UP(buf_size_byte, 4),
					.dma_trans_cmd.die_index = die_index,
					.dma_trans_cmd.pid = -1	};

	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS, &dma_cmd);
	if (ret < 0) {
		printf("trigger sdma to read is failed %d\n", ret);
	}

	close(filep);
	return ret;
}


int vastai_pcie_dma_read_buf(unsigned long long address, void *buf,
			  int buf_size_byte, int dev_id, int die_index)
{
		int filep = open_pcie_ioctl(dev_id);
	int ret = 0;

	struct kchar_cmd dma_cmd = {.dma_trans_cmd.is_dev_to_host = 1,
#ifdef VIDEO_COMPAT
					.dma_trans_cmd.vir_addr = (u64)buf,
#else
					.dma_trans_cmd.vir_addr = buf,
#endif
					.dma_trans_cmd.axi_addr = address,
					.dma_trans_cmd.length = ROUND_UP(buf_size_byte, 4),
					.dma_trans_cmd.die_index = die_index,
					.dma_trans_cmd.pid = -1	};

	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_PCIE_DMA_TRANS, &dma_cmd);
	if (ret < 0) {
		printf("trigger dma is failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_sdma_write_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_index)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;

	struct kchar_cmd dma_cmd = { .dma_trans_cmd.is_dev_to_host = 0,
#ifdef VIDEO_COMPAT
				     .dma_trans_cmd.vir_addr = (u64)buf,
#else
				     .dma_trans_cmd.vir_addr = buf,
#endif
				     .dma_trans_cmd.axi_addr = address,
				     .dma_trans_cmd.length = ROUND_UP(buf_size_byte, 4),
				     .dma_trans_cmd.die_index = die_index,
				     .dma_trans_cmd.pid = -1	
				     	};

	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_TRANS, &dma_cmd);
	if (ret < 0) {
		printf("trigger sdma to write is failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_pcie_dma_write_buf(unsigned long long address, void *buf,
		  int buf_size_byte, int dev_id, int die_index)
{
		int filep = open_pcie_ioctl(dev_id);
	int ret = 0;

	struct kchar_cmd dma_cmd = { .dma_trans_cmd.is_dev_to_host = 0,
#ifdef VIDEO_COMPAT
				     .dma_trans_cmd.vir_addr = (u64)buf,
#else
				     .dma_trans_cmd.vir_addr = buf,
#endif		     
				     .dma_trans_cmd.axi_addr = address,
				     .dma_trans_cmd.length = ROUND_UP(buf_size_byte, 4),
				     .dma_trans_cmd.die_index = die_index,
				     .dma_trans_cmd.pid = -1	
				     	};

	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_PCIE_DMA_TRANS, &dma_cmd);
	if (ret < 0) {
		printf("trigger dma is failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_pcie_dma_raw_transfer(unsigned long long address, void *buf,
				int buf_size_byte, int direction, int dev_id, int die_index)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;

	struct kchar_cmd dma_cmd = { .dma_trans_cmd.is_dev_to_host = !direction,
#ifdef VIDEO_COMPAT
				     .dma_trans_cmd.vir_addr = (u64)buf,
#else
					.dma_trans_cmd.vir_addr = buf,
#endif
				     .dma_trans_cmd.axi_addr = address,
				     .dma_trans_cmd.length = ROUND_UP(buf_size_byte, 4),
				     .dma_trans_cmd.die_index = die_index,
				     .dma_trans_cmd.pid = -1	
				     	};

	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_PCIE_DMA_RAW_TRANS, &dma_cmd);
	if (ret < 0) {
		printf("trigger pcie dma is failed %d\n", ret);
	}
\
	close(filep);
	return ret;
}

int vastai_sdma_fill_data_2_ddr(unsigned long long address, unsigned long long data,
			int size_byte, int dev_id, int die_index)
{

	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd dma_cmd = {.dma_fill_cmd.data = data,
					 .dma_fill_cmd.axi_addr = address,
					 .dma_fill_cmd.length = size_byte,
					 .dma_fill_cmd.die_index = die_index };

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_FILL, &dma_cmd);
	if (ret < 0) {
		printf("trigger fill data is failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_alloc_mem(struct vastai_mem_info *mem_info, int dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd alloc_cmd = { .alloc_cmd.size = mem_info->size,
				       .alloc_cmd.dma_buf_fd = 0 };

	if (filep < 0) {
		return filep;
	}

	if (mem_info->size > 0x400000)
	{
		printf("size unvalid!\n");
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_ALLOC, &alloc_cmd);
	if (ret < 0) {
		printf("ioctl alloc is failed %d\n", ret);
		goto CLOSE_KCHAR;
	}

	mem_info->dma_buf_fd = alloc_cmd.alloc_cmd.dma_buf_fd;
	mem_info->dma_phy_addr = alloc_cmd.alloc_cmd.dma_addr_t;

CLOSE_KCHAR:
	close(filep);
	return ret;
}

void vastai_free_mem(struct vastai_mem_info *mem_info, int dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd dma_cmd = {.dma_close_cmd.dma_buf_fd = mem_info->dma_buf_fd};
	printf("dmabuf_fd = %d\n", dma_cmd.dma_close_cmd.dma_buf_fd);
	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_CLOSE, &dma_cmd);
	if (ret < 0) {
		printf("close dma failed %d\n", ret);
	}
	close(mem_info->dma_buf_fd);
	return;
}

int vastai_set_mem(struct vastai_mem_info *mem_info,int val, int dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd kcmd = { .dma_memset_cmd.fd = mem_info->dma_buf_fd,
							.dma_memset_cmd.size = mem_info->size,
							.dma_memset_cmd.val = val};


	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_MEMSET, &kcmd);
	if (ret < 0) {
		printf("ioctl memset is failed %d\n", ret);
		goto CLOSE_KCHAR;
	}

CLOSE_KCHAR:
	close(filep);
	return ret;
}

int vastai_get_file_size(const char *file_path)
{
	struct stat stFile;
	int fd = 0;
	int size = 0;

	fd = open(file_path, O_RDONLY);
	if(fd < 0)
	{
		return 0;
	}

	if( (0 == fstat(fd, &stFile)) && S_ISREG(stFile.st_mode) ) 
	{
		size = (int)stFile.st_size;
	}

	close(fd);

	return size;
}

int vastai_sdma_write_from_file(const char *file_path, int offset,
			       unsigned long long address, int dev_id, int die_id)
{
	int ret=0;
	int len_byte = 0;

	len_byte = vastai_get_file_size(file_path);

	void *read_buffer = malloc(ROUND_UP(len_byte, 128));
	if (read_buffer == NULL) {
		perror("alloc read buffer error"); //output: alloc read buffer error:XXX\n
		return ret;
	}

	ret = read_buf_from_user_file(file_path, offset, read_buffer, len_byte);
	if (ret) {
		goto FREE_READ_BUF;
	}

	ret = vastai_sdma_write_buf(address, read_buffer, ROUND_UP(len_byte, 128), dev_id,
					die_id);
	if (ret) {
		return ret;
	}

FREE_READ_BUF:
	free(read_buffer);
	return ret;
}

int vastai_sdma_read_to_file(const char *file_path, int offset,
							unsigned long long address, int len_byte,
							int dev_id, int die_id)
{
	int ret = 0;

	/* malloc a buf */
	void *read_buf = malloc(ROUND_UP(len_byte, 128));
	if (!read_buf) {
		printf("alloc memory failed");
		goto RET_ERROR_CODE;
	}

	/* read dev data into this buf */
	ret = vastai_sdma_read_buf(address, read_buf, ROUND_UP(len_byte, 128), dev_id, die_id);
	if (ret != 0) {
		printf("read reg buf failed");
		goto FREE_READ_BUF;
	}

	/* write this buf into file */
	int filep = open(file_path, O_RDWR | O_CREAT, 0777);
	if (filep < 2) {
		ret = 1;
		goto FREE_READ_BUF;
	}

	int lseek_size = lseek(filep, offset, SEEK_SET);
	if (lseek_size != offset) {
		printf("lseek src file error, pls check ofst");
		ret = 1;
		goto CLOSE_DST_FILE;
	}

	int write_size = write(filep, read_buf, len_byte);
	if (write_size != len_byte) {
		printf("write dst file error");
		ret = 1;
	}

CLOSE_DST_FILE:
	close(filep);
FREE_READ_BUF:
	free(read_buf);
RET_ERROR_CODE:
	return ret;
}

int vastai_sdma_d2d_transfer(u64 src_addr, u64 dst_addr, u64 length, u32 die_index, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd dma_tr_d2d_cmd = {.dma_d2d_cmd.src_addr = src_addr,
									.dma_d2d_cmd.dst_addr = dst_addr,
									.dma_d2d_cmd.length = length,
									.dma_d2d_cmd.die_index = die_index,
									.dma_d2d_cmd.pid = 0};

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_D2D, &dma_tr_d2d_cmd);
	if (ret < 0) {
		printf("close dma failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_sdma_s2m_read(u64 src_addr, u64 dst_addr, u64 length, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd dma_tr_s2m_cmd = {.dma_s2m_cmd.src_addr = src_addr,
									.dma_s2m_cmd.dst_addr = dst_addr,
									.dma_s2m_cmd.length = length,
									.dma_s2m_cmd.direction = 0};

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_S2M, &dma_tr_s2m_cmd);
	if (ret < 0) {
		printf("close dma failed %d\n", ret);
	}

	close(filep);
	return ret;
}

int vastai_sdma_s2m_write(u64 src_addr, u64 dst_addr, u64 length, u32 dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd dma_tr_s2m_cmd = {.dma_s2m_cmd.src_addr = src_addr,
									.dma_s2m_cmd.dst_addr = dst_addr,
									.dma_s2m_cmd.length = length,
									.dma_s2m_cmd.direction = 1};

	ret = ioctl(filep, VASTAI_PCI_IOCTL_DMA_S2M, &dma_tr_s2m_cmd);
	if (ret < 0) {
		printf("close dma failed %d\n", ret);
	}

	close(filep);
	return ret;

}

int vastai_dma_slave_2_master_bulk(int dev_id, int die_id, int ch,
				   struct desc_node *node, int num,
				   long int *time)
{
	int i;
	u64 MEM_SIZE_32M = 0x2000000;
	void *buf = malloc(MEM_SIZE_32M);
	if (buf == NULL) {
		printf("dma_slave_2_master_bulk %d no memory", __LINE__);
		return -1;
	}

	u64 transfer_size = 0;
	for (i = 0; i < num; i++) {
		transfer_size += node->length;
	}
	if (transfer_size > MEM_SIZE_32M / 2) {
		printf("dma_slave_2_master_bulk %d transfer length %lld is too long to transfer\n",
		       __LINE__, transfer_size);
		goto FREE_MEM;
	}

	int bulk = 0;
	int host_2_dev = 1;
	u64 sum_length =
		fill_dma_desc(buf, die_id, ch + 4, bulk, host_2_dev, node, num);

	((char *)(buf))[0] = 'R';
	((char *)(buf))[1] = 'C';
	((char *)(buf))[2] = ' ';

	int fp = open_pcie_kchar(dev_id);
	unsigned long start, end;
	start = get_time();
	write(fp, buf, sum_length);
	end = get_time();
	*time = (unsigned long)(end - start);

	close(fp);

FREE_MEM:
	free(buf);
	return 0;
}

int vastai_dma_master_2_slave_bulk(int dev_id, int die_id, int ch,
				   struct desc_node *node, int num,
				   long int *time)
{
	int i;
	u64 MEM_SIZE_32M = 0x2000000;
	void *buf = malloc(MEM_SIZE_32M);
	if (buf == NULL) {
		printf("dma_master_2_slave_bulk %d no memory", __LINE__);
		return -1;
	}

	u64 transfer_size = 0;
	for (i = 0; i < num; i++) {
		transfer_size += node->length;
	}
	if (transfer_size > MEM_SIZE_32M / 2) {
		printf("dma_master_2_slave_bulk %d transfer length %lld is too long to transfer\n",
		       __LINE__, transfer_size);
		goto FREE_MEM;
	}

	int bulk = 0;
	int dev_2_host = 0;
	u64 sum_length =
		fill_dma_desc(buf, die_id, ch + 4, bulk, dev_2_host, node, num);

	((char *)(buf))[0] = 'R';
	((char *)(buf))[1] = 'C';
	((char *)(buf))[2] = ' ';

	int fp = open_pcie_kchar(dev_id);
	unsigned long start, end;
	start = get_time();
	write(fp, buf, sum_length);
	end = get_time();
	*time = (unsigned long)(end - start);

	close(fp);

FREE_MEM:
	free(buf);
	return 0;
}
int vastai_pci_mmio_write_buf_perf(struct vastai_mem_info *mem_info,unsigned long long address, void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd kcmd = { .mmio_trans_cmd.dma_buf_fd = mem_info->dma_buf_fd,
			          .mmio_trans_cmd.vir_addr = buf,
				  .mmio_trans_cmd.axi_addr = address,
			          .mmio_trans_cmd.length = buf_size_byte,
                                  .mmio_trans_cmd.die_index = die_id,
                                  .mmio_trans_cmd.pid = -1};


	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_MMIO_WRITE, &kcmd);
	if (ret < 0) {
		printf("ioctl mmio write is failed %d\n", ret);
		goto CLOSE_KCHAR;
	}

CLOSE_KCHAR:
	close(filep);
	return ret;
}

int vastai_pci_mmio_read_buf_perf(struct vastai_mem_info *mem_info, unsigned long long address, void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd kcmd = { .mmio_trans_cmd.dma_buf_fd = mem_info->dma_buf_fd,
							.mmio_trans_cmd.vir_addr = buf,
							.mmio_trans_cmd.axi_addr = address,
							.mmio_trans_cmd.length = buf_size_byte,
							.mmio_trans_cmd.die_index = die_id,
							.mmio_trans_cmd.pid = -1};


	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_MMIO_READ, &kcmd);
	if (ret < 0) {
		printf("ioctl mmio read is failed %d\n", ret);
		goto CLOSE_KCHAR;
	}

CLOSE_KCHAR:
	close(filep);
	return ret;
}

int vastai_get_log_status(void *log_ctl_buf, u32 die_index)
{
	union die_index_data temp = { .val = die_index };
	int kchar_fd = open_pcie_kchar(temp.dev_id);
	struct kchar_cmd kcmd = { .log_poll_cmd.die_index = die_index,
				  .log_poll_cmd.is_timeout = 0,
				  .log_poll_cmd.timeout_water = 500,
				  /*ms*/ .log_poll_cmd.log_ring_ctl =
					  log_ctl_buf };
	if (kchar_fd < 0) {
		printf("open kchar is failed %d\n", kchar_fd);
		return kchar_fd;
	}

	int ret = ioctl(kchar_fd, VASTAI_PCI_IOCTL_POLL_LOG, &kcmd);
	if (ret < 0) {
		printf("ioctl is failed %d\n", ret);
	}
	close(kchar_fd);

	return ret;
}

int vastai_get_pcie_port(u32 dev_id, char *pcie_port)
{
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_PORT, pcie_port);
	close(filep);
	return ret;
}

int vastai_get_render_name(u32 die_index, char *render_name)
{
	struct kchar_cmd kcmd = { .get_render_name.die_index = die_index };
	int filep = open_pcie_kchar(((union die_index_data)die_index).dev_id);
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_RENDER, &kcmd);
	close(filep);
	memcpy(render_name, kcmd.get_render_name.file_name, 16);
	return ret;
}

#define VASTAI_PCI_IOCTL_PCI_ID	     0XE
#define VASTAI_PCI_IOCTL_PCI_SUB_ID  0XF
#define VASTAI_PCI_IOCTL_MAJOR_MINOR 0X10
int vastai_get_pci_id(int dev_id, u32 *pci_id)
{
	struct kchar_cmd kcmd;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_PCI_ID, &kcmd);
	close(filep);
	memcpy(pci_id, &(kcmd.get_pci_id.pci_id), 4);
	return ret;
}

int vastai_get_pci_sub_id(int dev_id, u32 *pci_sub_id)
{
	struct kchar_cmd kcmd;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_PCI_SUB_ID, &kcmd);
	close(filep);
	memcpy(pci_sub_id, &(kcmd.get_pci_sub_id.pci_sub_id), 4);
	return ret;
}

int vastai_get_pci_major(int dev_id, u32 *major_minor)
{
	struct kchar_cmd kcmd;
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_MAJOR_MINOR, &kcmd);
	close(filep);
	memcpy(major_minor, &(kcmd.get_major_minor.major_minor), 4);
	return ret;
}

int vastai_get_dev_info(struct vastai_dev_info dev_info[], int *dev_num)
{
	int dev_num_temp = vastai_get_device_num();
	int die_index_array[16];
	int ret;

	if (dev_num_temp > *dev_num)
		printf("mem is too less ?\n");

	*dev_num = dev_num_temp;
	for (int i = 0; i < dev_num_temp; i++) {
		dev_info[i].dev_id = vastai_get_dev_id(i);
		dev_info[i].unid = 0;
		ret = vastai_get_pcie_port(dev_info[i].dev_id,
					   dev_info[i].pcie_slot);
		if (ret) {
			return ret;
		}
		snprintf(dev_info[i].pcie_dev_name, 16, "kchar:%d", i);
		snprintf(dev_info[i].device_name, 16, "sv100");

		dev_info[i].cap.ai_cap = 1;
		dev_info[i].cap.video_cap = 1;
		vastai_get_pci_id(dev_info[i].dev_id,
				  &(dev_info[i].pci_id.val));
		vastai_get_pci_sub_id(dev_info[i].dev_id,
				      &(dev_info[i].pci_sub_id.val));
		vastai_get_pci_major(dev_info[i].dev_id,
				     &(dev_info[i].major_minor.val));

		ret = vastai_get_die_info(dev_info[i].dev_id, die_index_array,
					  &(dev_info[i].die_num));
		if (ret) {
			return ret;
		}
		for (int j = 0; j < dev_info[i].die_num; j++) {
			dev_info[i].die_info[j].die_index.val =
				die_index_array[j];
			snprintf(dev_info[i].die_info[j].vacc_name, 16,
				 "vacc%d",
				 dev_info[i].die_info[j].die_index.seq_num);
			ret = vastai_get_render_name(
				dev_info[i].die_info[j].die_index.val,
				dev_info[i].die_info[j].render_name);
			if (ret) {
				return ret;
			}
		}
	}
	return 0;
}

u64 vastai_get_upstream_count(u32 die_index)
{
	u64 upstream_count;
	int ret;

	ret = vastai_pcie_read_reg_64bit(
		PCIE_BANDWIDTH_UPSTREAM, &upstream_count,
		((union die_index_data)die_index).dev_id,
		((union die_index_data)die_index).die_id);
	if (ret) {
		return 0;
	}

	return upstream_count;
}

u64 vastai_get_downstream_count(u32 die_index)
{
	u64 downstream_count;
	int ret;

	ret = vastai_pcie_read_reg_64bit(
		PCIE_BANDWIDTH_DOWNSTREAM, &downstream_count,
		((union die_index_data)die_index).dev_id,
		((union die_index_data)die_index).die_id);
	if (ret) {
		return 0;
	}

	return downstream_count;
}

int vastai_get_performance(u32 die_index, struct vastai_performance_info *info,
			   u32 info_size)
{
	struct kchar_cmd kcmd = { .get_die_performance.die_index = die_index };
	int filep = open_pcie_kchar(((union die_index_data)die_index).dev_id);

	if (filep < 0) {
		printf("open kchar is failed %d\n", filep);
		return filep;
	}

	int ret = ioctl(filep, VASTAI_PCI_IOCTL_GET_PERFORMANCE, &kcmd);
	if (ret) {
		close(filep);
		return ret;
	}

	ret = __vastai_pcie_read_reg_buf(
		filep, PERFORMANCE_AI_BASE, info->csram_info,
		info_size - sizeof(*info),
		((union die_index_data)die_index).die_id);
	if (ret) {
		close(filep);
		return ret;
	}

	info->pcie_performance.pcie_bandwidth_upstream =
		vastai_get_upstream_count(die_index);
	info->pcie_performance.pcie_bandwidth_downstream =
		vastai_get_downstream_count(die_index);

	memcpy(info->pcie_performance.pcie_tx_vdsp_count,
	       kcmd.get_die_performance.pcie_tx_vdsp_count,
	       sizeof(kcmd.get_die_performance.pcie_tx_vdsp_count));

	memcpy(info->pcie_performance.pcie_rx_vdsp_count,
	       kcmd.get_die_performance.pcie_rx_vdsp_count,
	       sizeof(kcmd.get_die_performance.pcie_rx_vdsp_count));

	close(filep);
	return ret;
}

struct vastai_fifo {
	u32 rd;
	u32 wr;
	u32 elem_count;
	u32 elem_size;
};

static u32 ecc_valid_entry_count(struct vastai_fifo *fifo)
{
	return (fifo->elem_count + (((int)fifo->wr) - (int)(fifo->rd))) %
	       fifo->elem_count;
}

int vastai_get_ddr_1bit(u32 die_index, struct vastai_ecc_table *ecc_table, u32 table_size)
{
	int ret;

	ret = vastai_pcie_read_reg_buf(
		DDR_ECC_1BIT, (void *)ecc_table, table_size,
		((union die_index_data)die_index).dev_id,
		((union die_index_data)die_index).die_id);
	ecc_table->entry_count =
		ecc_valid_entry_count((struct vastai_fifo *)ecc_table);
	return ret;
}

int vastai_get_ddr_2bit(u32 die_index, struct vastai_ecc_table *ecc_table, u32 table_size)
{
	int ret;

	ret = vastai_pcie_read_reg_buf(
		DDR_ECC_2BIT, (void *)ecc_table, table_size,
		((union die_index_data)die_index).dev_id,
		((union die_index_data)die_index).die_id);
	ecc_table->entry_count =
		ecc_valid_entry_count((struct vastai_fifo *)ecc_table);
	return ret;
}

static int vastai_cedar_send_cmd(vastai_cedar_img_info_t *op, int dev_id, int die_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd cedar_cmd;

	memcpy(&cedar_cmd.cedar_ioctl.cedar_info, op, sizeof(vastai_cedar_img_info_t));

	if (filep < 0) {
		return filep;
	}
	ret = ioctl(filep, VASTAI_PCI_IOCTL_CEDAR_CMD, &cedar_cmd);
	if (ret < 0) {
		printf("trigger cedar is failed %d, die_id:%d\n", ret, die_id);
	}

	close(filep);
	return ret;
}

void big2little(u8 *data, u64 blen, u32 blockBlen)
{
	u8 temp = 0,*ptr = NULL;
	u64 round;
	//blen  is  multiple for blockblen
	if(blen%blockBlen)
		return;

	round = (u64)(blen/blockBlen);
	u64 i=0,j=0;
	for(i=0; i<round; i++)
	{
		ptr = data+i*blockBlen;

		for(j=0; j<(blockBlen>>1); j++)
		{
			temp = ptr[j];
			ptr[j] = ptr[blockBlen - 1 - j];
			ptr[blockBlen-1-j] = temp;
		}
	}
}

int ce_sha2(u64 input, u64 output, u32 blen, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type = type,
		.input_addr = input,
		.output_addr = output,
		.blen = blen,
	};

	ret = vastai_cedar_send_cmd(&alg_info, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}
	return ret;
}

int ce_sm3(u64 input, u64 output, u32 blen, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type = type,
		.input_addr = input,
		.output_addr = output,
		.blen = blen,
	};
	ret = vastai_cedar_send_cmd(&alg_info, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}
	if (((type & 0x03) == 0x2) || ((type & 0x03) == 0x3)) {
		unsigned char hash_res[64] = { 0 };
		int output_hash_len = 0;
		if (type <= 3)
			output_hash_len = 28;
		else if (type <= 7)
			output_hash_len = 32;
		else if (type <= 11)
			output_hash_len = 48;
		else if (type <= 15)
			output_hash_len = 64;
		else if (type <= 19)
			output_hash_len = 32;

		ret = vastai_pcie_read_reg_buf(CEDAR_SVC_HASH_RES,
				(void *)hash_res,
				output_hash_len, 0, die_id);
		if (ret != 0) {
			printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
		ret = vastai_pcie_write_reg_buf((unsigned long long)output,
				(void *)hash_res,
				output_hash_len, 0, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
	}
	return ret;
}

int ce_sha2_img(u64 input, u64 output, u32 width, u32 height, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type = type,
		.input_addr = input,
		.output_addr = output,
		.blen = width * height,
	};
	ret = vastai_cedar_send_cmd(&alg_info, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return ret;
}

int ce_sha3_img(u64 input, u64 output, u32 width, u32 height, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type = type,
		.input_addr = input,
		.output_addr = output,
		.blen = width * height,
	};
	ret = vastai_cedar_send_cmd(&alg_info, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	
	return ret;
}

int ce_aes_ecb(u64 input, u64 output, u32 blen, u64 key, u32 keyblen, u32 mode, svc_t type,int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen         = blen,
		.key          = key,
		.keyblen      = keyblen,
		.symmode      = mode,
	};

	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen,dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);

	ret = vastai_pcie_write_reg_buf(key, (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
}

int ce_aes_cbc(u64 input, u64 output, u32 blen, u64 key, u32 keyblen, u32 mode, u64 iv, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	//u32 key_max_len = 32;
	u32 Iv_max_len = 16;
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen          = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv           = iv,
		.symmode      = mode,
	};
	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key, (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,Iv_max_len,Iv_max_len);
	ret = vastai_pcie_write_reg_buf(iv, (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info, dev_id,die_id);
}

int ce_aes_ctr(u64 input, u64 output, u32 blen, u64 key, u32 keyblen, u32 mode, u64 iv, svc_t type,int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	//u32 key_max_len = 32;
	u32 Iv_max_len = 16;
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen         = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv           = iv,
		.symmode      = mode,
	};

	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key, (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,Iv_max_len,Iv_max_len);
	ret = vastai_pcie_write_reg_buf(iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info,dev_id, die_id);

}
int ce_aes_gcm(u64 input, u64 output, u32 blen,  u64 key, u32 keyblen,u32 mode,
		u64 iv, u32 ivblen, u64 msg, u32 msgblen,
		u64 tag, u32 tblen, svc_t type, int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	u8 msgbuf[16] = {0};
	u8 tagbuf[16]= {0};
	//u32 key_max_len = 32;
	//u32 Iv_max_len = 16;

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen         = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv            = iv,
		.ivblen       = ivblen,
		.msg          = msg,
		.msgblen      = msgblen,
		.tag          = tag,
		.tblen        = tblen,
		.symmode      = mode,
	};
	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key, (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, ivblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,ivblen,ivblen);
	ret = vastai_pcie_write_reg_buf(iv, (void *)ivbuf, ivblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)msg, (void *)msgbuf, msgblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(msgbuf,msgblen,msgblen);
	ret = vastai_pcie_write_reg_buf(msg, (void *)msgbuf, msgblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	if(type == AES_GCM_DEC)
	{
		ret = vastai_pcie_read_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
		big2little(tagbuf,tblen,tblen);
		ret = vastai_pcie_write_reg_buf(tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}

	}
	ret = vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}

	if(type == AES_GCM_ENC)
	{
		ret = vastai_pcie_read_reg_buf(tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
		big2little(tagbuf,tblen,tblen);
		ret = vastai_pcie_write_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
	}
	return ret;
}

int ce_ecc_enc(u64 input, u64 output, u32 blen, u64 random, u64 pubkey, u32 pkaMode,
		svc_t type, int dev_id,int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen		  = blen,
		.random       = random,
		.pubkey       = pubkey,
		.pkaMode	  = pkaMode,
	};

	ret = vastai_cedar_send_cmd(&alg_info,dev_id ,die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}
	return ret;
}
int ce_ecc_dec(u64 input, u64 output, u32 blen, u64 prikey, u32 pkaMode, svc_t type,int dev_id, int die_id)
{
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen		  = blen,
		.prikey       = prikey,
		.pkaMode	  = pkaMode,
	};

	return vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
}

int ce_ecc_sign(u64 input, u64 signature, u32 blen, u64 random, u64 prikey, u32 pkaMode, svc_t type,int dev_id, int die_id)
{

	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = signature,
		.blen		  = blen,
		.random       = random,
		.prikey	      = prikey,
		.pkaMode      = pkaMode,
	};

	ret = vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}
	return ret;
}

int ce_ecc_verify(u64 input, u64 signature, u32 blen, u64 pubkey, u32 pkaMode, svc_t type, int dev_id,int die_id)
{
	int ret = 0;
	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.sign		  = signature,
		.blen		  = blen,
		.pubkey	      = pubkey,
		.pkaMode      = pkaMode,
	};

	ret = vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}
	return ret;
}

int ce_sm4_ecb(u64 input, u64 output, u32 blen, u64 key, u32 keyblen, u32 mode,
		svc_t type , int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen	      = blen,
		.key          = key,
		.keyblen      = keyblen,
		.symmode      = mode,
	};
	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key , (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
}

int ce_sm4_cbc(u64 input, u64 output, u32 blen, u64 key, u32 keyblen,
		u32 mode, u64 iv, svc_t type,int dev_id, int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	u32 Iv_max_len = 16;

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen	       = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv	      = iv,
		.symmode      = mode,
	};
	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key , (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,Iv_max_len,Iv_max_len);
	ret = vastai_pcie_write_reg_buf(iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info, dev_id ,die_id);
}

int ce_sm4_ctr(u64 input, u64 output, u32 blen, u64 key, u32 keyblen,
		u32 mode, u64 iv, svc_t type, int dev_id,int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	u32 Iv_max_len = 16;

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen	      = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv	      = iv,
		.symmode      = mode,
	};

	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf(key , (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,Iv_max_len,Iv_max_len);
	ret = vastai_pcie_write_reg_buf(iv , (void *)ivbuf, Iv_max_len, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	return vastai_cedar_send_cmd(&alg_info,dev_id, die_id);
}

int ce_sm4_gcm(u64 input, u64 output, u32 blen,  u64 key, u32 keyblen,u32 mode,
		u64 iv, u32 ivblen, u64 msg, u32 msgblen,
		u64 tag, u32 tblen, svc_t type, int dev_id,int die_id)
{
	int ret = 0;
	u8 keybuf[32]={0};
	u8 ivbuf[16] = {0};
	u8 msgbuf[16] = {0};
	u8 tagbuf[16]= {0};

	vastai_cedar_img_info_t alg_info = {
		.alg_type     = type,
		.input_addr   = input,
		.output_addr  = output,
		.blen	      = blen,
		.key          = key,
		.keyblen      = keyblen,
		.iv	       = iv,
		.ivblen       = ivblen,
		.msg 	      = msg,
		.msgblen      = msgblen,
		.tag          = tag,
		.tblen        = tblen,
		.symmode      = mode,
	};

	ret = vastai_pcie_read_reg_buf((u64)key,  (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s read reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(keybuf,keyblen,keyblen);
	ret = vastai_pcie_write_reg_buf((u64)key , (void *)keybuf, keyblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)iv , (void *)ivbuf, ivblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(ivbuf,ivblen,ivblen);
	ret = vastai_pcie_write_reg_buf((u64)iv , (void *)ivbuf, ivblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	ret = vastai_pcie_read_reg_buf((u64)msg, (void *)msgbuf, msgblen, dev_id, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	big2little(msgbuf,msgblen,msgblen);
	ret = vastai_pcie_write_reg_buf((u64)msg , (void *)msgbuf, msgblen, 0, die_id);
	if (ret != 0) {
		printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
		return -1;
	}
	if(type == SM4_GCM_DEC)
	{
		ret = vastai_pcie_read_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
		big2little(tagbuf,tblen,tblen);
		ret = vastai_pcie_write_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}

	}
	ret = vastai_cedar_send_cmd(&alg_info, dev_id,die_id);
	if (ret != 0) {
		printf("ERROR: %s cedar send cmd failed, errcode %d\n", __FUNCTION__, ret);
		return -2;
	}

	if(type == SM4_GCM_ENC)
	{
		ret = vastai_pcie_read_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
		big2little(tagbuf,tblen,tblen);
		ret = vastai_pcie_write_reg_buf((u64)tag, (void *)tagbuf, tblen, dev_id, die_id);
		if (ret != 0) {
			printf("ERROR: %s write reg failed, errcode %d\n", __FUNCTION__, ret);
			return -1;
		}
	}
	return ret;

}
