#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include "vastai_export_enum_define.h"
#include "vastai_pcie_user_api.h"
#include "vastai_pci_test.h"
#include "diag_pcie_user_api.h"

#define UNUSED(x) (void)x

static int __vastai_write_csr_by_smcu_buf(int filep, unsigned long long addr,
				       const void *buf, int buf_size_byte)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "smcu_write_buf  addr=0x%llx user_addr=0x%lx len=%d",
				 addr, (unsigned long)buf, buf_size_byte);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("smcu_write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return -1;
	}

	return 0;
}

static int open_pcie_ioctl(int dev_id)
{
	char file_path[64] = { 0 };
	int filep;

	snprintf(file_path, sizeof(file_path), "/dev/va%d_ctl", dev_id);
	filep = open(file_path, O_RDWR);
	if (filep < 0) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output:  open /dev/kchar: dev_id failed!:XXXX(error)\n
	}
	return filep;
}

static int __vastai_write_32bit_by_smcu(unsigned long long addr,void *val,int dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd kcmd = {.smcu_trans_cmd.axi_addr = addr,
				  .smcu_trans_cmd.val = *(u32 *)val};


	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_SMCU_WRITE, &kcmd);
	if (ret < 0) {
		printf("ioctl smcu read is failed %d\n", ret);
		goto CLOSE_IOCTL;
	}

CLOSE_IOCTL:
	close(filep);
	return ret;

}

int vastai_write_csr_by_smcu(unsigned long long addr, const void *buf,
			      int buf_size_byte, int dev_id)
{
	unsigned long write_offset = 0;
	int write_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return -1;
	}

	while (buf_size_byte > 0) {
		if (buf_size_byte > VASTAI_PCI_RW_BUF_LEN_ONCE)
			write_size_once = VASTAI_PCI_RW_BUF_LEN_ONCE;
		else
			write_size_once = buf_size_byte;

		ret = __vastai_write_csr_by_smcu_buf(filep, addr + write_offset,
						  buf + write_offset, write_size_once);
		if (ret) {
			ret = -1;
			break;
		}
		write_offset += write_size_once;
		buf_size_byte -= write_size_once;
	}

	close(filep);
	return ret;
}

int diag_vastai_smcu_write_32bit(unsigned long long address, unsigned int value,
				int dev_id, int die_id)
{
	UNUSED(die_id);
	return __vastai_write_32bit_by_smcu(address,(void *)&value,dev_id);
}

int diag_vastai_smcu_write_64bit(unsigned long long address,
				unsigned long long value, int dev_id,
				int die_id)
{
   UNUSED(die_id);
   unsigned int Lo32Value;
   unsigned int Hi32Value;
   int ret = 0;
   Lo32Value = value & 0xFFFFFFFF;
   Hi32Value = value >> 32;
   ret = __vastai_write_32bit_by_smcu(address,(void *)&Lo32Value, dev_id);
   ret = __vastai_write_32bit_by_smcu(address+4, (void *)&Hi32Value,dev_id);
   return ret;
}

int diag_vastai_smcu_write_buf(unsigned long long address, const void *buf,
			      int buf_size_byte, int dev_id, int die_id)
{
	UNUSED(die_id);
	unsigned long write_offset = 0;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output: buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % 4) {
		perror("write_buf must be align with 4Byte"); //output: write_buf must be align with 4Byte:XXXX\n
		return -1;
	}

	while (buf_size_byte > 0) {	
		ret = __vastai_write_32bit_by_smcu(address + write_offset, (void *)(buf + write_offset), dev_id);
		if (ret) {
			ret = ERR_CODE(ret);
			break;
		}
		write_offset  += 4;
		buf_size_byte -= 4;
	}
	return ret;
}

int vastai_smcu_write_buf_unalign(unsigned long long address, const void *buf,
				  int buf_size_byte, int dev_id, int die_id)
{
	return diag_vastai_smcu_write_buf(address,buf,buf_size_byte,dev_id,die_id);
}

int vastai_write_32_bit_csr_by_smcu(unsigned long long address,
				unsigned long long value, int dev_id)
{
	return __vastai_write_32bit_by_smcu(address,(void *)&value,dev_id);
}

static int __vastai_read_csr_by_smcu_buf(int filep, unsigned long long addr,
				      void *buf, int buf_size_byte)
{
	char temp_buf[128] = { 0 };
	int lseek_size = 0;
	int temp_buf_size =
		snprintf(temp_buf, sizeof(temp_buf),
			 "smcu_read_buf addr=0x%llx len=%d",
			  addr, buf_size_byte);
	int write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("write dev file error"); //output:write dev file error:XXXX\n
		return -1;
	}

	lseek_size = lseek(filep, 0, SEEK_SET);
	if (lseek_size != 0) {
		perror("smcu_read_buf: lseek set error"); //output:read_reg_buf: lseek set error:XXXX\n
		return -1;
	}

	int read_size = read(filep, buf, buf_size_byte);
	if (read_size != buf_size_byte) {
		/* error handle */
		perror("read dev file error"); //output:read dev file error:XXXX\n
		return -1;
	}

	return 0;
}

static int __vastai_read_32bit_by_smcu(unsigned long long addr,void *val,int dev_id)
{
	int filep = open_pcie_ioctl(dev_id);
	int ret = 0;
	struct kchar_cmd kcmd = {.smcu_trans_cmd.axi_addr = addr,
				  .smcu_trans_cmd.val = 0};


	if (filep < 0) {
		return filep;
	}

	ret = ioctl(filep, VASTAI_PCI_IOCTL_SMCU_READ, &kcmd);
	if (ret < 0) {
		printf("ioctl smcu read is failed %d\n", ret);
		goto CLOSE_IOCTL;
	}

	*(u32 *)val = kcmd.smcu_trans_cmd.val;

CLOSE_IOCTL:
	close(filep);
	return ret;

}


int vastai_read_csr_by_smcu(unsigned long long addr, void *buf,
				 int buf_size_byte, int dev_id)
{
	unsigned long read_offset = 0;
	int read_size_once;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output:open dev file error:XXXX\n
		return -3;
	}
	while (buf_size_byte > 0) {
		if (buf_size_byte > 4*1024)
			read_size_once = 4*1024;
		else
			read_size_once = buf_size_byte;

		ret = __vastai_read_csr_by_smcu_buf(filep, addr + read_offset,
						 buf + read_offset,read_size_once);

		if (ret) {
			ret = -4;
			break;
		}
		read_offset += read_size_once;
		buf_size_byte -= read_size_once;
	}
	close(filep);
	return ret;
}

int diag_vastai_smcu_read_64bit(unsigned long long address,
			       unsigned long long *value, int dev_id,
			       int die_id)
{
   UNUSED(die_id);
   unsigned int Lo32Value;
   unsigned int Hi32Value;
   int ret = 0;
   ret = __vastai_read_32bit_by_smcu(address,    (void *)&Lo32Value, dev_id);
   ret = __vastai_read_32bit_by_smcu(address + 4,(void *)&Hi32Value, dev_id);
   *value = (unsigned long long)Hi32Value<<32|Lo32Value;

   return ret;
}


int diag_vastai_smcu_read_32bit(unsigned long long address, unsigned int *value,
			       int dev_id, int die_id)
{
	UNUSED(die_id);	
	return __vastai_read_32bit_by_smcu(address,(void *)value, dev_id);
}

int vastai_read_32_bit_csr_by_smcu(unsigned long long addr, unsigned int *value, int dev_id)
{
	return __vastai_read_32bit_by_smcu(addr,(void *)value, dev_id);
}

int vastai_smcu_read_reg_buf(unsigned long long address, void *buf,
			     int buf_size_byte, int dev_id, int die_id)
{
	UNUSED(die_id);
	unsigned long read_offset = 0;
	int ret = 0;

	if (!buf) {
		perror("buf addr error, is NULL"); //output:buf addr error, is NULL:XXXX\n
		return -1;
	}
	if (buf_size_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXXX\n
		return -2;
	}

	while (buf_size_byte > 0) {
		ret = __vastai_read_32bit_by_smcu(address + read_offset,(void *)(buf + read_offset), dev_id);
		if (ret) {
			ret = -1;
			break;
		}
		read_offset   += 4;
		buf_size_byte -= 4;
	}
	return ret;
}

int diag_vastai_smcu_read_reg_file(const char *file_path, int offset,
			      unsigned long long address, int len_byte,
			      int dev_id, int die_id)
{
	
    int ret;
	if (len_byte % sizeof(int)) {
		perror("len_byte must be align with 4Byte"); //output:len_byte must be align with 4Byte:XXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	/* malloc a buf */
	void *read_buf = malloc(len_byte);
	if (!read_buf) {
		perror("alloc memory failed"); //output：alloc memory failed：XXX\n
		ret = ERR_CODE(0);
		goto RET_ERROR_CODE;
	}

	/* read pcie reg into this buf */
	ret = vastai_smcu_read_reg_buf(address, read_buf, len_byte, dev_id,die_id);
	if (ret != 0) {
		perror("read reg buf failed"); //output：read reg buf failed：XXX\n
		ret = ERR_CODE(ret);
		goto FREE_READ_BUF;
	}
	/* write this buf into file */
	int filep = open(file_path, O_RDWR | O_CREAT, 0777);
	if (filep < 2) {
		/*error handle*/
		char temp_buf[128] = { 0 };
		snprintf(temp_buf, sizeof(temp_buf), "open %s failed!",
			 file_path);
		perror(temp_buf); //output：open file_path failed!：XXX\n
		ret = ERR_CODE(0);
		goto FREE_READ_BUF;
	}

	int lseek_size = lseek(filep, offset, SEEK_SET);
	if (lseek_size != offset) {
		perror("lseek src file error, pls check ofst"); //output：lseek src file error, pls check ofst：XXX\n
		ret = ERR_CODE(0);
		goto CLOSE_DST_FILE;
	}

	int write_size = write(filep, read_buf, len_byte);
	if (write_size != len_byte) {
		perror("write dst file error"); //output：write dst file error：XXX\n
		ret = ERR_CODE(0);
	}

CLOSE_DST_FILE:
	close(filep);
FREE_READ_BUF:
	free(read_buf);
RET_ERROR_CODE:
	return ret;
}

static int __vastai_smcu_modify_reg32(int filep, unsigned long long addr,
				       unsigned int mask_bit, unsigned int mode)
{
	char temp_buf[128] = { 0 };
	int temp_buf_size, write_size;

	temp_buf_size = snprintf(temp_buf, sizeof(temp_buf),
				 "smcu_modify_csr_bit  addr=0x%llx maskbit=0x%x mode=%d",
				 addr, mask_bit, mode);
	write_size = write(filep, temp_buf, temp_buf_size + 1);

	if (write_size != (temp_buf_size + 1)) {
		/* error handle */
		perror("__write_buf dev file error"); //output: __write_buf dev file error:XXXX\n
		return -1;
	}

	return 0;
}

int diag_vastai_smcu_modify_reg32(unsigned long long address,unsigned int mask_bit,
								unsigned int mode,int dev_id)
{
	int ret = 0;	
	int filep = open_pcie_kchar(dev_id);
	if (filep < 0) {
		/* error handle */
		perror("open dev file error"); //output: open dev file error:XXXX\n
		return -1;
	}

	ret = __vastai_smcu_modify_reg32(filep,address,mask_bit,mode);

	close(filep);
	return ret;
}
