﻿#ifndef __VATOOLS_DEBUGGERF_H__
#define __VATOOLS_DEBUGGERF_H__

#pragma pack(push)
#pragma pack(1)
#ifdef __cplusplus
extern "C" {
#endif

// 20210926 新接口

//*************************************************************************
//*************************** 下面公共代码拷贝  ****************************
//*************************************************************************
#define PRINT_MACRO_HELPER(x) #x
#define PRINT_MACRO(x)	      #x "=" PRINT_MACRO_HELPER(x)

/*************************************************************************
 * ioctl cmd
 */
#define VADBG_IOCTL_CMD (VATOOLS_DEBUG_SET) //( 0xA182 )

// 轮询命令的执行结果最大次数
#define VADBG_POLL_FLAG_TOTAL_COUNT (20)
// 每次轮询命令的时间间隔
#define VADBG_POLL_FLAG_INTERVAL_START	(1) // 单位: ms. 轮询开始第一次时间间隔
#define VADBG_POLL_FLAG_INTERVAL_ADDING (10) // 单位: ms. 每次增加的时间间隔
#define VADBG_POLL_FLAG_ADDING_COUNT                                           \
	(5) // 增加时间间隔的次数. 超过后以最后一次时间轮询剩余的次数

/*************************************************************************
 * fw 结构引用
 */
struct s_config_info; // 保存debug 配置信息
struct s_break_req; // host 下发用户设置的断点信息
struct s_break_rsp; // fw 返回断点状态, host 轮询flag
struct s_break_cmd; // host 下发命令, host 轮询flag
struct s_break_next; // 保存由next命令生成的断点信息

/*************************************************************************
 *host 下发的debug命令
 */

enum e_vadbg_cmd_code {
	VADBG_CMD_START = 1,
	VADBG_CMD_CONTINUE,
	VADBG_CMD_NEXT,
	VADBG_CMD_STEP,
	VADBG_CMD_DUMP,
	VADBG_CMD_WRITE,
	VADBG_CMD_STOP,
};

enum e_vadbg_dump_type {
	VADBG_DUMP_DDR = 1, // soc ddr, 物理绝对地址
	VADBG_DUMP_MCUDDR, // mcu ddr, 经过 mcu entry 映射的ddr地址
	VADBG_DUMP_CSRAM,
	VADBG_DUMP_DRAM,
	VADBG_DUMP_SSRAM,
	VADBG_DUMP_CSR,
	VADBG_DUMP_VAR, // TODO:  mcu 提供变量列表index, host选择dump的变量
	VADBG_DUMP_MODEL_INFO,
	VADBG_DUMP_MCU_STATUS,
	VADBG_DUMP_MCU_REGISTER,
	VADBG_DUMP_MCU_SYSDUMP, // LogSys_DumpData_Debug
	VADBG_DUMP_MAX_TYPE = 9999,
};

/*
应用层下发数据, flag = host set, cmcu fw 收到中断后, 修改为 receive, cmcu fw 下发中断到各个core后, 修改为 broadcast,
进入断点后, 判断flag, 如果flag ==  broadcast, 则修改flag为 break, 进入断点状态. 如果flag != broadcast, 表示app
重新写入了断点数据, core fw 丢弃本次断点. 继续运行程序.
进入断点后, 执行子操作结束后, 由app发送特殊命令  next, fw 接收后, 退出断点, 退出断点时, 设定flag = done, 由host轮询.
*/

//断点状态flag
enum e_vadbg_break_flag {
	VADBG_BP_FLAG_NONE = 0x00, // 无断点
	VADBG_BP_FLAG_HOST_SET = 0x01, // host 设置断点
	VADBG_BP_FLAG_FW_RECEIVE = 0x02, // fw 收到host发送的设置断点命令
	VADBG_BP_FLAG_FW_BROADCAST = 0x03, // fw 发送中断将断点信息到各个core,
	VADBG_BP_FLAG_FW_BREAK = 0x04, // fw 进入断点状态, while=1, 循环
	VADBG_BP_FLAG_FW_DONE = 0x05, // fw 完成工作. 设置变量值结束等.
	VADBG_BP_FLAG_FW_DUMP_DONE = 0x06, // fw dump 数据结束
	VADBG_BP_FLAG_FW_DUMP_NEXT =
		0x07, // fw dump 数据为完成, 还有数据需要继续返回给host, host继续发指令.
	VADBG_BP_FLAG_FW_ERROR = 0x81, // fw 错误
	VADBG_BP_FLAG_FW_DUMP_ERROR = 0x82, // fw dunmp 错误
};

#define VADBG_ENABLE  (1)
#define VADBG_DISABLE (0)

#define VADBG_ENTER_DEBUG_WHILE (1)
#define VADBG_EXIT_DEBUG_WHILE	(0)

// fw csram 保存基本信息 32 bytes
typedef struct s_config_info {
	u32 version; // 保留
	u32 core_num; // host启动debug app时设置, 目前29个.
	u64 soc_ddr_address; // 分配给debug的ddr的base地址
	u32 soc_ddr_size; // 分配给debug的ddr的数据总长度 1M bytes
	// u_break_flag vadbg_break_flag[ VADBG_CORE_NUMBER ];  //断点的状态flag,  00-无断点, 01-host设置了断点
	// 02-fw收到中断, 03-fw 配置了debug_ctrl到各个core, 04-fw进入断点状态 8x-错误
	u32 rev1;
	u32 rev2;
	u32 rev3;
} t_config_info;

// host可以设置的断点总数量. 代码类型断点, 所有的core共同使用.
#define VADBG_CORE_CANDIDATE_BREAKPOINT_NUMBER (50)
// 每个core分配一个断点结构, 保存已经进入断点状态的core的断点信息, 从host设置的候选的break info里面拷贝
#define VADBG_CORE_BREAKED_NUMBER (CORE_NUMBER)
// 保存 next / step 命令时, 根据当前场景生成的断点, 每个core有一个. 检查时和用户设置的断点操作相同.
#define VADBG_CORE_NEXT_STEP_BREAKPOINT_NUMBER (CORE_NUMBER)
// 所有的断点总数量
#define VADBG_CORE_CANDIDATE_BREAKPOINT_TOTAL                                  \
	(VADBG_CORE_CANDIDATE_BREAKPOINT_NUMBER +                              \
	 VADBG_CORE_NEXT_STEP_BREAKPOINT_NUMBER)

/*************************************************************************
 * fw 地址定义
 */

/*
内存布局:

base address:
----
  config: s_config_info
----
  breaked info: s_break_rsp, 每个core 一个, 共 VADBG_CORE_BREAKED_NUMBER 个. 按 e_vadbg_core_index 排序
enum core_bit_index {
	CORE_LMCU0,
	CORE_LMCU1,
	CORE_LMCU2,
	CORE_LMCU3,
	CORE_LMCU4,
	CORE_LMCU5,
	CORE_ODSP2,
	CORE_ODSP3,
	CORE_ODSP4,
	CORE_ODSP5,
	CORE_ODSP6,
	CORE_ODSP7,
	CORE_VEMCU0,
	CORE_VEMCU1,
	CORE_VEMCU2,
	CORE_VEMCU3,
	CORE_VDMCU0,
	CORE_VDMCU1,
	CORE_VDMCU2,
	CORE_SMCU,
	CORE_VDSP0,
	CORE_VDSP1,
	CORE_VDSP2,
	CORE_VDSP3,
	CORE_CMCU,
	CORE_NUMBER,
};

----
  host set breakpoint: s_break_req, 总共 VADBG_CORE_CANDIDATE_BREAKPOINT_NUMBER 个. 所有core公用
----
  host 下发命令: s_break_cmd, host 轮询 flag
----
  dump data: 1K bytes
----
*/

// CSRAM中保存debugger 信息的地址
#define VADBG_DUMP_BODY_LEN (1024) // dump 数据使用的内存
#define VADBG_CSRAM_ADDR_END                                                   \
	(VADBG_CSRAM_ADDR + VADBG_CSRAM_LEN) // debug 使用的 csram 结束地址

// debug 配置信息
#define VADBG_CSRAM_CONFIG_ADDR                                                \
	(VADBG_CSRAM_ADDR) // debug 配置信息, 32 bytes t_config_info
#define VADBG_CSRAM_CONFIG_LEN                                                 \
	(sizeof(struct s_config_info)) // 控制信息长度 32 bytes t_config_info

// break point 每个core的已经进入的断点状态的信息
#define VADBG_CSRAM_BREAKED_ADDR                                               \
	(VADBG_CSRAM_CONFIG_ADDR + VADBG_CSRAM_CONFIG_LEN)
#define VADBG_CSRAM_BREAKED_LEN (sizeof(struct s_break_rsp))

// 下面两组断点数据, 内存一定要连续在一起
// break point host 设置的候选断点信息
#define VADBG_CSRAM_BREAKPOINT_ADDR                                            \
	(VADBG_CSRAM_BREAKED_ADDR + VADBG_CSRAM_BREAKED_LEN)
#define VADBG_CSRAM_BREAKPOINT_LEN (sizeof(struct s_break_req))

// break point next命令生成的断点信息
#define VADBG_CSRAM_BREAKPOINT_NEXT_ADDR                                       \
	(VADBG_CSRAM_BREAKPOINT_ADDR + VADBG_CSRAM_BREAKPOINT_LEN)
#define VADBG_CSRAM_BREAKPOINT_NEXT_LEN (sizeof(struct s_break_next))

// host 发送debug 命令
#define VADBG_CSRAM_BREAK_CMD_ADDR                                             \
	(VADBG_CSRAM_BREAKPOINT_NEXT_ADDR + VADBG_CSRAM_BREAKPOINT_NEXT_LEN)
#define VADBG_CSRAM_BREAK_CMD_LEN (sizeof(struct s_break_cmd))

// debug dump 信息
// 使用csram 1k 空间dump数据, 大于1k的数据, 分块 dump
// dump 返回值内存排列: s_break_cmd  +  dump body(1k bytes), s_break_cmd 就是上面的 VADBG_CSRAM_BREAK_CMD_ADDR
#define VADBG_DUMP_DATA_ADDR (VADBG_CSRAM_BREAK_CMD_ADDR)
#define VADBG_DUMP_BODY_ADDR                                                   \
	(VADBG_CSRAM_BREAK_CMD_ADDR + VADBG_CSRAM_BREAK_CMD_LEN)
#define VADBG_DUMP_DATA_LEN (VADBG_DUMP_BODY_LEN + VADBG_CSRAM_BREAK_CMD_LEN)

// check 地址空间是否越界
#define VADBG_CSRAM_USED_OVERLAP                                               \
	((VADBG_DUMP_DATA_ADDR + VADBG_DUMP_DATA_LEN) > VADBG_CSRAM_ADDR_END)

// TODO: logsys dump 考虑内存存放问题,暂时测试使用csram
#define VADBG_LOGSYS_ENABLE (0)
// logsys dump 0x08CF4F00数据. 存放 LogDumpData 数据
#define VADBG_LOGSYS_DUMP_ADDR (0x08CF4F00)

/*************************************************************************
 * fw 结构定义
 */

typedef union union_break_ctrl {
	u32 val;
	struct {
		u32 resv : 16;
		u32 model_en : 1; // 1 - 标识有 model 断点, 各个core的model循环判断此值
		u32 layer_en : 1; // 1 - 标识有 layer 断点, 各个core的 layer 循环判断此值
		u32 opcode_en : 1; // 1 - 标识有 opcode 断点, 各个core的 opcode 循环判断此值
		u32 condition_en : 1; // 1 - 标识有 条件/事件 断点, 各个core的 opcode  循环 及相关代码 判断此值
		u32 variable_en : 1; // 1 - 标识有 变量类型 断点, 各个core的 opcode 循环 及相关代码 判断此值
		u32 resv2 : 6;
		u32 write_en : 1; // 1 - host 发起 write 命令
		u32 dump_en : 1; // 1 - host 发起 dump 命令
		u32 main_en : 1; // 1 - 进入 fw 运行模型前的断点
		u32 run_next : 1; // host设置断点时, 设置为1  , fw进入debug模式时, 判断此值, 如果为1, 进入while(1) 循环,
			// host发送 next/run 继续运行时, 设置此值为0, fw循环中判断如果为0 , break while.
		u32 debug_en : 1; // 1 - 进入debug模式. 0 - 退出debug模式
	};
} u_break_ctrl;

typedef union union_break_flag {
	u32 val;
	struct {
		u32 oper_index : 24; //返回索引
		u32 oper_flag : 8; //状态
	};
} u_break_flag;

//断点结构
typedef struct s_breakpoint {
	u32 break_index; //断点序号 0 : 不使用, 其他:使用. host 设置断点时, 由host负责修改; fw rsp 设置进入断点状态时,由fw
		//负责增加1. 用来区分上面的一个断点.
	u32 mcu_core_id; // core id: core type | core index  //TODO: 改为U32
	u16 model_index; // 如果 break_ctrl 设置了 model_en=1, 则检查此项是否匹配
	u16 layer_index; // 如果 break_ctrl 设置了 layer_en=1, 则检查此项是否匹配
	u16 opcode_index; // 如果 break_ctrl 设置了 opcode_en=1, 则检查此项是否匹配
	u16 rsvd;
	u_break_ctrl break_ctrl; // 中断方式或中断位置
	u_break_flag break_flag; // 放到最后. 保证前面数据完整写入
} t_breakpoint;

typedef struct s_sub_dump {
	u32 dump_type; // dump 类型: ddr, csr, dram ,ssram, csram, var etc.
	u64 dump_addr; // 要dump的地址. ddr, csram, csr 提供soc address. 其他的类型发指令到core, 由 core 读取
	u32 dump_len; // 要dump的数据长度. host下发时, ddr 等ram 类的需要指定长度; 其他host不清楚的,设为0,
		// 标识不指定长度.由fw判断并填入.
	u64 buffer_addr; // 返回给用户层的数据buffer. 保留host下发的数据值
	u32 buffer_len; // 返回给用户层的数据buffer 长度. 保留host下发时的数据值.
} t_sub_dump;

typedef struct s_sub_set {
	u32 type; // 地址类型: ddr, csr, dram ,ssram, csram, var etc.
	u64 addr;
	u32 val;
} t_sub_set;

// host 先发送 s_break_req 数据到csram, 再发送 s_break_cmd 命令, 此时才发送中断到cmcu, 开始执行.
// host 发送到 fw 的 所有断点 信息
typedef struct s_break_req {
	T_DEVICE device;
	t_breakpoint breakpoints[VADBG_CORE_CANDIDATE_BREAKPOINT_NUMBER];
} t_break_req;

// next / step 命令生成的断点, 每个core一个结构
typedef struct s_break_next {
	t_breakpoint breakpoints[VADBG_CORE_NEXT_STEP_BREAKPOINT_NUMBER];
} t_break_next;

// host 轮询 driver 的 所有 core 的当前断点状态
typedef struct s_break_rsp {
	t_breakpoint breakpoints[VADBG_CORE_BREAKED_NUMBER];
} t_break_rsp;

// host 发送命令开始执行debug功能, 断点命令的返回值需要host 轮询s_break_rsp. 其他命令的返回值 轮询 s_break_cmd,
// fw 收到host 命令, 清除本core t_break_cmd 数据. 标识进入新的命令状态
typedef struct s_break_cmd {
	T_DEVICE device;
	u32 mcu_core_id; // core id: core type | core index, 标识哪个core 处理此命令, TODO: 改为U32
	u32 cmd; // start, continue, next, step, dump, write //TODO: 改为U32
	union {
		t_sub_dump dump; // read
		t_sub_set write; // write
	} sub;
	u_break_flag break_flag;
} t_break_cmd;

/*
代码处理流程:
1. 每次 fw 收到中断后, 如果flag==01 && 是控制命令(start, next, step, continue等)
   则检查更新所有的断点数据, 更新flag = 02, 标识fw 已经开始处理
   同时开始处理命令, 如果控制命令的前提不满足,则忽略此控制命令.
   例如: next 命令, 如果当前没有断点存在则忽略此命令.

   如果是数据命令, (dump, write, watch 等) 则不检查断点数据. 只执行数据命令.

2. checkpoint 检查断点匹配时, 只检查flag == 02 断点.
   如果匹配, 则修改flag = 04 VADBG_BP_FLAG_FW_BREAK


*/

/*************************************************************************
 * host 通过ioctl发送数据到 driver 接口.
 */
enum s_host_ioctl_type {
	HOST_CMD = 0,
	HOST_PREPARE_BREAKPOINTS = 1,
	HOST_POLL_BREAK_STATUS = 2,
};

/*************************************************************************
 * coreid 和 coreindex 转换
 */
/* #define INTERNAL_GET_ENTRYNUM_BY_COREID( coreid )  \
      ( ( ( ( coreid & 0xFC0 ) >> 6 ) > 0x8 ) ? 8 : ( ( ( coreid & 0xFC0 ) >> 6 ) == 0 ) ? 8 : 4 )
*/

#define COREID_DEFINE_3(t) (t << 6) | 0, (t << 6) | 1, (t << 6) | 2
#define COREID_DEFINE_4(t)                                                     \
	(t << 6) | 0, (t << 6) | 1, (t << 6) | 2, (t << 6) | 3
#define COREID_DEFINE_8(t)                                                     \
	(t << 6) | 0, (t << 6) | 1, (t << 6) | 2, (t << 6) | 3, (t << 6) | 4,  \
		(t << 6) | 5, (t << 6) | 6, (t << 6) | 7

#define CORE_TYPE_CMCU	0x0 // id[0-0]	1
#define CORE_TYPE_LMCU	0x1 // id[0-7]	8
#define CORE_TYPE_ODSP	0x2 // id[0-7]	8
#define CORE_TYPE_VDSP	0x8 // id[0-3]	4
#define CORE_TYPE_VDMCU 0xA // id[0-2]	3
#define CORE_TYPE_VEMCU 0xB // id[0-3]	4
#define CORE_TYPE_SMCU	0xF // id[0-0]	1
#pragma pack()
//*************************************************************************
//*************************** 上面面公共代码拷贝  **************************
//*************************************************************************

/*************************************************************************
 * function
 */

loff_t debugger_llseek(struct file *filp, loff_t offset, int orig);
ssize_t debugger_read(struct file *filp, char __user *buf, size_t size,
		      loff_t *pos);
ssize_t debugger_write(struct file *filp, const char __user *buf, size_t size,
		       loff_t *pos);
ssize_t debugger_write_iter(struct kiocb *iocb, struct iov_iter *from);
unsigned int debugger_poll(struct file *filp, struct poll_table_struct *wait);
long debugger_ioctl(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);
int debugger_mmap(struct file *filp, struct vm_area_struct *pvm);
int debugger_open(struct inode *inode, struct file *filp);
int debugger_release(struct inode *ignored, struct file *filp);
int debugger_fasync(int fd, struct file *filp, int mode);
int debugger_set_app_category(struct file *filp, unsigned int cmd,
			      IOCTL_ARG_T arg);
int debugger_wait_flag(void *priv, u32 die_index, u64 addr, u8 flag[],
		       u32 flag_count, t_break_cmd *p_break_cmd);
int debugger_ioctl_dump(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg,
			t_break_cmd *p_host_break_cmd,
			struct S_HOST_ARG_VAL *p_host_cmd);
int debugger_ioctl_breakpoint_cmd(struct file *filp, unsigned int cmd,
				  IOCTL_ARG_T arg,
				  struct S_HOST_ARG_VAL *p_host_cmd);
int debugger_ioctl_prepare_breakpoints(struct file *filp, unsigned int cmd,
				       IOCTL_ARG_T arg,
				       struct S_HOST_ARG_VAL *p_host_cmd);
int debugger_ioctl_poll_break_status(struct file *filp, unsigned int cmd,
				     IOCTL_ARG_T arg,
				     struct S_HOST_ARG_VAL *p_host_cmd);
int debugger_ioctl_set(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);
int debugger_ioctl_get(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);
int debugger_ioctl_config(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);
int debugger_ioctl_fetch(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);

#ifdef __cplusplus
}
#endif
#pragma pack(pop)
#endif //__VATOOLS_DEBUGGERF_H__
