﻿#include "vatools.h"
#include "vatools_log.h"

/******************************************************************************
 * vatools all file ops
 *
 *
 */

//根据app_category排列
static const struct file_operations_tool app_fops[] = {
	{
		.owner = THIS_MODULE,
		.llseek = dummy_llseek,
		.read = dummy_read,
		.write = dummy_write,
		.write_iter = dummy_write_iter,
		.poll = dummy_poll,
		.unlocked_ioctl = dummy_ioctl,
		.compat_ioctl = dummy_ioctl,
		.mmap = dummy_mmap,
		.open = dummy_open,
		.release = dummy_release,
		.fasync = dummy_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = smi_llseek,
		.read = smi_read,
		.write = smi_write,
		.write_iter = smi_write_iter,
		.poll = smi_poll,
		.unlocked_ioctl = smi_ioctl,
		.compat_ioctl = smi_ioctl,
		.mmap = smi_mmap,
		.open = smi_open,
		.release = smi_release,
		.fasync = smi_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = profiler_llseek,
		.read = profiler_read,
		.write = profiler_write,
		.write_iter = profiler_write_iter,
		.poll = profiler_poll,
		.unlocked_ioctl = profiler_ioctl,
		.compat_ioctl = profiler_ioctl,
		.mmap = profiler_mmap,
		.open = profiler_open,
		.release = profiler_release,
		.fasync = profiler_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = debugger_llseek,
		.read = debugger_read,
		.write = debugger_write,
		.write_iter = debugger_write_iter,
		.poll = debugger_poll,
		.unlocked_ioctl = debugger_ioctl,
		.compat_ioctl = debugger_ioctl,
		.mmap = debugger_mmap,
		.open = debugger_open,
		.release = debugger_release,
		.fasync = debugger_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = logger_llseek,
		.read = logger_read,
		.write = logger_write,
		.write_iter = logger_write_iter,
		.poll = logger_poll,
		.unlocked_ioctl = logger_ioctl,
		.compat_ioctl = logger_ioctl,
		.mmap = logger_mmap,
		.open = logger_open,
		.release = logger_release,
		.fasync = logger_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = dummy_llseek,
		.read = dummy_read,
		.write = dummy_write,
		.write_iter = dummy_write_iter,
		.poll = dummy_poll,
		.unlocked_ioctl = dummy_ioctl,
		.compat_ioctl = dummy_ioctl,
		.mmap = dummy_mmap,
		.open = dummy_open,
		.release = dummy_release,
		.fasync = dummy_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = sharedmem_llseek,
		.read = sharedmem_read,
		.write = sharedmem_write,
		.write_iter = sharedmem_write_iter,
		.poll = sharedmem_poll,
		.unlocked_ioctl = sharedmem_ioctl,
		.compat_ioctl = sharedmem_ioctl,
		.mmap = sharedmem_mmap,
		.open = sharedmem_open,
		.release = sharedmem_release,
		.fasync = sharedmem_fasync,
	},
	{
		.owner = THIS_MODULE,
		.llseek = dummy_llseek,
		.read = dummy_read,
		.write = dummy_write,
		.write_iter = dummy_write_iter,
		.poll = dummy_poll,
		.unlocked_ioctl = dummy_ioctl,
		.compat_ioctl = dummy_ioctl,
		.mmap = dummy_mmap,
		.open = dummy_open,
		.release = dummy_release,
		.fasync = dummy_fasync,
	}
};

/******************************************************************************
 * add global vatools node open list, used for hot plug
 *
 *
 */
LIST_HEAD(vatools_open_list);
#if (TOOLS_WIN == 1)
struct mutex g_mutex_open_list;
#else
DEFINE_MUTEX(g_mutex_open_list);
#endif

static int vatools_add_open_list(enum va_f_type type);
static void vatools_del_open_list(enum va_f_type type);
atomic_t _vatools_in_kill_routine = ATOMIC_INIT(0);

/******************************************************************************
 * mutex for tools_init and tools_exit
 *
 *
 */
#if (TOOLS_WIN == 1)
struct mutex g_mutex_tools_init;
#else
DEFINE_MUTEX(g_mutex_tools_init);
#endif

static struct vatools_node *vatools_get_node_from_minor(int minor)
{
	struct vatools_node *node = NULL;

	list_for_each_entry (node, vatools_get_vastai_head(), list_nodes) {
		if (node->misc.minor == minor)
			return node;
	}
	return NULL;
}

struct vatools_node *vatools_get_node(void)
{
	struct vatools_node *node = NULL;

	list_for_each_entry (node, vatools_get_vastai_head(), list_nodes) {
		if (node != NULL)
			return node;
	}
	return NULL;
}

static long vatools_get_device_num(struct file *filp, unsigned int cmd,
				   IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	unsigned int device_num = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	device_num = vatools_get_vastai_pci_device_number();
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "device_num=%d\n", device_num);

	if (copy_to_user_ex((void __user *)argp, &device_num,
			    sizeof(unsigned int))) {
		ret = -EFAULT;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		goto out;
	}

	ret = 0;

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_dev_info(struct file *filp, unsigned int cmd,
				 IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vatools_node *node = vatools_file_get_node(filp);
	vatools_ioctl_arg_t *ioctl_arg;
	vatools_dev_info_t *dev_info_p = NULL;
	struct vastai_cdev *vacc_dev_p = NULL;
	struct vastai_cdev *vacc_dev_temp = NULL;
	struct vastai_render_file *render_file_p = NULL;
	struct vastai_render_file *render_file_temp = NULL;

	struct vastai_pci_info *pci_info = NULL;
	u32 device_num = 0;
	int die = 0;
	void *buffer = NULL;
	int buffer_size = 0;
	u32 die_index = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	V_UNREFERENCE(die);
	V_UNREFERENCE(render_file_p);
	V_UNREFERENCE(render_file_temp);
	V_UNREFERENCE(vacc_dev_p);
	V_UNREFERENCE(vacc_dev_temp);
	VATOOLS_FUNC_ENTERY;

	ioctl_arg = (vatools_ioctl_arg_t *)kvmalloc(sizeof(vatools_ioctl_arg_t),
						    GFP_KERNEL);
	if (ioctl_arg == NULL) {
		VATOOLS_ERR(pci_info, die_index, "kvmalloc error. size=%ld\n",
			    sizeof(vatools_ioctl_arg_t));
		ret = -ENOMEM;
		goto out;
	}

	ret = copy_from_user_ex(ioctl_arg, (void __user *)argp,
				sizeof(vatools_ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(pci_info, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	device_num = vatools_get_vastai_pci_device_number();
	if (device_num == 0) {
		ret = -ENODEV;
		ioctl_arg->header.length = 0;
		if (copy_to_user_ex((void __user *)argp, ioctl_arg,
				    sizeof(vatools_ioctl_arg_t))) {
			ret = -EFAULT;
		}
		goto out_free_ioctl_arg;
	}

	buffer_size = device_num * sizeof(vatools_dev_info_t);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "device_num=%d buffer_size=%d\n",
		    device_num, buffer_size);
	buffer = kvmalloc(buffer_size, GFP_KERNEL);
	if (buffer == NULL) {
		VATOOLS_ERR(pci_info, die_index, "kvmalloc error. size=%d\n",
			    buffer_size);
		ret = -ENOMEM;
		goto out_free_ioctl_arg;
	}

	dev_info_p = (vatools_dev_info_t *)buffer;
	memset(buffer, 0, buffer_size);

	// TODO: 支持热插拔, 板卡插拔后, device 序号发生变化, 不再从0开始连续
	list_for_each_entry (pci_info, &vastai_pci_info_list, dev_list) {
		if (pci_info == NULL) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID, "pci_info is NULL\n");
			continue;
		}
		if (vatools_check_pci_state(pci_info, VASTAI_NORMAL_STATE,
					    -1)) {
			VATOOLS_DBG(pci_info, die_index,
				    "pci_info->pci_state=0x%x\n",
				    atomic_read(&pci_info->pci_state));
			continue;
		}

		dev_info_p->dev_id = pci_info->dev_id;
		dev_info_p->die_num = pci_info->die_num;
		VATOOLS_DBG(pci_info, die_index,
			    "dev_id=%d die_num=%d buffer_size=%d\n",
			    dev_info_p->dev_id, dev_info_p->die_num,
			    buffer_size);

		// bar size
		memcpy(dev_info_p->bar_address, vatools_die_check,
		       sizeof(vatools_die_check));

		// char unid[ VASTAI_BUF_SIZE_32 ];       ///< SN号; get from common (host); not here
		// char card_type[ VASTAI_BUF_SIZE_32 ];  ///< 卡类型; get from common (host); not here

		dev_info_p->pciid.vender_id = pci_info->dev->vendor;
		dev_info_p->pciid.device_id = pci_info->dev->device;
		dev_info_p->pcisubid.sub_vender_id =
			pci_info->dev->subsystem_vendor;
		dev_info_p->pcisubid.sub_device_id =
			pci_info->dev->subsystem_device;

		memcpy(dev_info_p->pcie_slot, pci_name(pci_info->dev),
		       VASTAI_BUF_SIZE_32);

		dev_info_p->kchar.majormin.major =
			MAJOR(pci_info->p_char->major);
		dev_info_p->kchar.majormin.minor =
			MINOR(pci_info->p_char->major);

		// memcpy(dev_info_p->kchar.name,
		//        pci_info->p_char->file_info.file_name,
		//        VASTAI_BUF_SIZE_32);

		if (pci_info->version_file != NULL) {
			dev_info_p->vastai_version.majormin.major = 0;
			dev_info_p->vastai_version.majormin.major = MAJOR(
				pci_info->version_file->file_info.dev_num);
			dev_info_p->vastai_version.majormin.minor = MINOR(
				pci_info->version_file->file_info.dev_num);
			memcpy(dev_info_p->vastai_version.name,
			       pci_info->version_file->file_info.file_name,
			       VASTAI_BUF_SIZE_32);
		}
		if (pci_info->ctl_file != NULL) {
			dev_info_p->vastai_ctrl.majormin.major =
				MAJOR(pci_info->ctl_file->file_info.dev_num);
			dev_info_p->vastai_ctrl.majormin.minor =
				MINOR(pci_info->ctl_file->file_info.dev_num);
			memcpy(dev_info_p->vastai_ctrl.name,
			       pci_info->ctl_file->file_info.file_name,
			       VASTAI_BUF_SIZE_32);
		}

		dev_info_p->vatools.majormin.minor = (u16)node->misc.minor;
		dev_info_p->vatools.majormin.major = 10;
		memcpy(dev_info_p->vatools.name, node->misc.name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->cap.ai_cap =
			dev_info_p->cap.ai_cap; ///< AI VIDEO 能力集
		dev_info_p->cap.video_cap =
			dev_info_p->cap.video_cap; ///< AI VIDEO 能力集

		dev_info_p++;
	}

	ioctl_arg->header.err = 0;
	ioctl_arg->header.length = buffer_size;

	if (copy_to_user_ex((void __user *)argp, ioctl_arg,
			    sizeof(vatools_ioctl_arg_t))) {
		ret = -EFAULT;
		goto out_free_buffer;
	}

	if (copy_to_user_ex((void __user *)(u8 *)argp +
				    sizeof(vatools_ioctl_arg_header_t),
			    buffer, buffer_size)) {
		ret = -EFAULT;
		goto out_free_buffer;
	}

	ret = 0;

out_free_buffer:
	kvfree(buffer);

out_free_ioctl_arg:
	kvfree(ioctl_arg);

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

static long vatools_get_die_info(struct file *filp, unsigned int cmd,
				 IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;

	int die_info[MAX_DIE_NUM_PER_DEVICE + 2] = { 0 };
	int die_index_array[MAX_DIE_NUM_PER_DEVICE] = { 0 };
	int die_num = 0;
	int i = 0;
	u32 die_index = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	ret = copy_from_user_ex(die_info, (void __user *)argp,
				sizeof(die_info));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}
	priv = vatools_get_vastai_pci_device_info((char)die_info[0]);
	if (priv == NULL) {
		VATOOLS_INFO(priv, die_index, "priv is NULL, dev_id=0x%x\n",
			     die_info[0]);
		return -ENODEV;
	}
	if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE, -1)) {
		VATOOLS_DBG(priv, die_index, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}
	VATOOLS_DBG(priv, die_index, "device_id: %d\n", die_info[0]);

	ret = vatools_get_vastai_die_info_via_device_id(
		(char)die_info[0], die_index_array, &die_num);
	if (ret) {
		VATOOLS_INFO(priv, die_index, "device_id=%d, error %ld\n",
			     die_info[0], ret);
		goto out;
	}
	die_info[1] = die_num;
	for (i = 0; i < die_num; i++) {
		die_info[2 + i] = die_index_array[i];
	}

	if (copy_to_user_ex((void __user *)argp, die_info, sizeof(die_info))) {
		ret = -EFAULT;
		VATOOLS_ERR(priv, die_index, "ret=%ld\n", ret);
		goto out;
	}

	ret = 0;
out:
	VATOOLS_DBG(priv, die_index, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_dev_pcie_bus_id(struct file *filp, unsigned int cmd,
					IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;

	struct vastai_pci_info *priv = NULL;
	TS_SMI_DEV_PCIE_BUS_ID dev_pcie_bus_id = { 0 };
	u32 die_index = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&dev_pcie_bus_id, 0x00, sizeof(TS_SMI_DEV_PCIE_BUS_ID));
	ret = copy_from_user_ex(&dev_pcie_bus_id, (void __user *)argp,
				sizeof(TS_SMI_DEV_PCIE_BUS_ID));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy from user ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}

	priv = vatools_get_vastai_pci_device_info((char)dev_pcie_bus_id.dev_id);
	if (priv == NULL) {
		VATOOLS_INFO(priv, die_index, "priv is NULL, dev_id=0x%x\n",
			     dev_pcie_bus_id.dev_id);
		return -ENODEV;
	}
	if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE, -1)) {
		VATOOLS_DBG(priv, die_index, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		return -ENODEV;
	}
#if VATOOLS_INDEPENDANCE_DRIVER
	strcpy(dev_pcie_bus_id.pcie_port, "pcie-simu\n");
#else
	strcpy(dev_pcie_bus_id.pcie_port, pci_name(priv->dev));
#endif
	ret = copy_to_user_ex((void __user *)argp, &dev_pcie_bus_id,
			      sizeof(TS_SMI_DEV_PCIE_BUS_ID));
	if (ret) {
		ret = -EFAULT;
		VATOOLS_ERR(priv, die_index, "ret=%ld\n", ret);
		goto out;
	}

	ret = 0;

out:
	VATOOLS_DBG(priv, die_index, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_dev_pcie_id(struct file *filp, unsigned int cmd,
				    IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	TS_SMI_DEV_PCIE_ID dev_pcie_id = { 0 };

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&dev_pcie_id, 0x00, sizeof(TS_SMI_DEV_PCIE_ID));
	ret = copy_from_user_ex(&dev_pcie_id, (void __user *)argp,
				sizeof(TS_SMI_DEV_PCIE_ID));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy from user ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	dev_pcie_id.pcie_id = 0x1ec6 | 0x0100 << 16;

	ret = copy_to_user_ex((void __user *)argp, &dev_pcie_id,
			      sizeof(TS_SMI_DEV_PCIE_ID));
	if (ret) {
		ret = -EFAULT;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "ret=%ld\n", ret);
		goto out;
	}
	ret = 0;

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_dev_pcie_sub_id(struct file *filp, unsigned int cmd,
					IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	TS_SMI_DEV_PCIE_SUB_ID dev_pcie_sub_id = { 0 };

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&dev_pcie_sub_id, 0x00, sizeof(TS_SMI_DEV_PCIE_SUB_ID));
	ret = copy_from_user_ex(&dev_pcie_sub_id, (void __user *)argp,
				sizeof(TS_SMI_DEV_PCIE_SUB_ID));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy from user ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	dev_pcie_sub_id.pcie_sub_id = 0xFFFF;

	ret = copy_to_user_ex((void __user *)argp, &dev_pcie_sub_id,
			      sizeof(TS_SMI_DEV_PCIE_SUB_ID));
	if (ret) {
		ret = -EFAULT;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "ret=%ld\n", ret);
		goto out;
	}
	ret = 0;

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_dev_major_minor_id(struct file *filp, unsigned int cmd,
					   IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	TS_SMI_DEV_MAJOR_MINOR dev_major_minor = { 0 };

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&dev_major_minor, 0x00, sizeof(TS_SMI_DEV_MAJOR_MINOR));
	ret = copy_from_user_ex(&dev_major_minor, (void __user *)argp,
				sizeof(TS_SMI_DEV_MAJOR_MINOR));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy from user ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	dev_major_minor.major_minor = 0x0;

	ret = copy_to_user_ex((void __user *)argp, &dev_major_minor,
			      sizeof(TS_SMI_DEV_MAJOR_MINOR));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy to user ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}
	ret = 0;

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_die_render(struct file *filp, unsigned int cmd,
				   IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	TS_SMI_DIE_RENDER die_render = { 0 };
	u32 die_index = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&die_render, 0x00, sizeof(TS_SMI_DIE_RENDER));
	ret = copy_from_user_ex(&die_render, (void __user *)argp,
				sizeof(die_render));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy from user ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}

	priv = vatools_get_vastai_pci_device_info((char)die_render.dev_id);
	if (priv == NULL) {
		VATOOLS_INFO(priv, die_index, "priv is NULL, dev_id=0x%x\n",
			     die_render.dev_id);
		ret = -ENODEV;
		goto out;
	}
	if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE, -1)) {
		VATOOLS_DBG(priv, die_index, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}
	snprintf(die_render.render, 16, "renderD%d", priv->dev_id);
	ret = copy_to_user_ex((void __user *)argp, &die_render,
			      sizeof(TS_SMI_DIE_RENDER));
	if (ret) {
		ret = -EFAULT;
		VATOOLS_ERR(priv, die_index, "ret=%ld\n", ret);
		goto out;
	}

	ret = 0;

out:
	VATOOLS_DBG(priv, die_index, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_get_device_capability(struct file *filp, unsigned int cmd,
					  IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vastai_pci_info *priv = NULL;
	SMI_DEVICE_CAPABILITY device_cap = { 0 };
	u32 die_index = 0;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	memset(&device_cap, 0x00, sizeof(SMI_DEVICE_CAPABILITY));
	ret = copy_from_user_ex(&device_cap, (void __user *)argp,
				sizeof(SMI_DEVICE_CAPABILITY));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy from user ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}

	priv = vatools_get_vastai_pci_device_info((char)device_cap.dev_id);
	if (priv == NULL) {
		VATOOLS_INFO(priv, die_index, "priv is NULL, dev_id=0x%x\n",
			     device_cap.dev_id);
		ret = -ENODEV;
		goto out;
	}
	if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE, -1)) {
		VATOOLS_DBG(priv, die_index, "priv->pci_state=0x%x\n",
			    atomic_read(&priv->pci_state));
		ret = -ENODEV;
		goto out;
	}

	// memcpy(&device_cap.capability, &priv->vadev,
	//        sizeof(struct vastai_dev_info));

	VATOOLS_DBG(priv, die_index, "out: ret=%ld\n", ret);

	ret = copy_to_user_ex((void __user *)argp, &device_cap,
			      sizeof(SMI_DEVICE_CAPABILITY));
	if (ret) {
		ret = -EFAULT;
		VATOOLS_ERR(priv, die_index, "ret=%ld\n", ret);
		goto out;
	}

	ret = 0;

out:
	VATOOLS_DBG(priv, die_index, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static long vatools_set_app_category(struct file *filp, unsigned int cmd,
				     IOCTL_ARG_T arg)
{
	struct vatools_reader *reader = vatools_file_get_reader(filp);
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	int die_index = 0;

	VATOOLS_FUNC_ENTERY;
	ret = copy_from_user_ex(&(reader->trans_category), (void __user *)argp,
				sizeof(reader->trans_category));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	//下面category需要同时绑定 device 信息. 在release可以释放对应device 的资源
	if ((reader->trans_category.app_category == VATOOLS_APP_SMI) ||
	    (reader->trans_category.app_category == VATOOLS_APP_PROFILER) ||
	    (reader->trans_category.app_category == VATOOLS_APP_DEBUGGER) ||
	    (reader->trans_category.app_category == VATOOLS_APP_MONITOR)) {
		die_index = vatools_get_vastai_pci_die_index(
			(char)reader->trans_category.device.dev_id,
			reader->trans_category.device.die_id);
		if (die_index < 0) {
			ret = -ENODEV;
			goto out;
		}
		reader->trans_category.device.die_index = die_index;
	} else {
		// 在共享内存状态下的数据由其他用处
		if (reader->trans_category.app_category !=
		    VATOOLS_APP_SHAREDMEM) {
			reader->trans_category.block_id = SMI_CMD_BLOCK_ID_0;
			reader->trans_category.app_category = VATOOLS_APP_SMI;
			reader->trans_category.device.dev_id = 0;
			reader->trans_category.device.die_id = 0;
			reader->trans_category.device.die_index = 0;
		} else {
			// 用户态的pid 换成内核态的pid 保证在docker中能正确识别
			reader->trans_category.device.dev_id = current->tgid;
			reader->trans_category.device.die_id = 0;
			reader->trans_category.device.die_index = current->tgid;
		}
	}
	// VATOOLS_DBG
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"reader=%p,app_category=%d  block_id=%d  device_id=%d  die_id=%d"
		"die_index=0x%x\n",
		reader, reader->trans_category.app_category,
		reader->trans_category.block_id,
		reader->trans_category.device.dev_id,
		reader->trans_category.device.die_id,
		reader->trans_category.device.die_index);

	switch (reader->trans_category.app_category) {
	case VATOOLS_APP_SHAREDMEM:
		sharedmem_set_app_category(filp, cmd, arg);
		break;
	case VATOOLS_APP_DEBUGGER:
		debugger_set_app_category(filp, cmd, arg);
		break;
	}

out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;

	return ret;
}

static unsigned int vatools_get_app_category(struct file *filp)
{
	struct vatools_reader *reader = vatools_file_get_reader(filp);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "app_category=%d\n",
		    reader->trans_category.app_category);

	return (unsigned int)reader->trans_category.app_category;
}

/******************************************************************************
 * smi
 *
 *
 */

static long vatools_driver_read(struct file *file, unsigned int cmd,
				IOCTL_ARG_T arg)
{
	return vatools_driver_smi_fetch(file, cmd, arg);
}

static long vatools_driver_write(struct file *file, unsigned int cmd,
				 IOCTL_ARG_T arg)
{
	return vatools_driver_smi_fetch(file, cmd, arg);
}

/******************************************************************************
 * pcie fops
 *
 *
 */

static long vatools_pcie_ioctl_bandwidth(T_SMI_IOCTL_TRANS_DATA *pkdata,
					 struct vastai_pci_info *priv,
					 int die_index)
{
	long ret = -EINVAL;

	u64 upstream_count = 0;
	u64 downstream_count = 0;
	u64 upstream_count_mb = 0;
	u64 downstream_count_mb = 0;
	u32 val = 10000; // interval is 10ms

	/* UPSTREAM */
	ret = vastai_pcie_monitor(priv, val, 1, &upstream_count);
	if (ret < 0) {
		VATOOLS_INFO(priv, die_index, "pcie_monitor : upstream fail\n");
		goto out;
	}

	upstream_count_mb = upstream_count * 1000000 / val / 1024 / 1024;
	VATOOLS_DBG(
		priv, die_index,
		"PCIE upstream bandwidth:%lld MB/s upstream_count=%lld bytes\n",
		upstream_count_mb, upstream_count);

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->output_buf.buf_addr +
				      sizeof(TS_SMI_CMD_REQ),
			      &upstream_count_mb, sizeof(u64));
	if (ret < 0) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}

	/* DOWNSTREAM */
	ret = vastai_pcie_monitor(priv, val, 0, &downstream_count);
	if (ret < 0) {
		VATOOLS_INFO(priv, die_index,
			     "pcie_monitor : downstream fail\n");
	}

	downstream_count_mb = downstream_count * 1000000 / val / 1024 / 1024;

	VATOOLS_DBG(
		priv, die_index,
		"PCIE downstream bandwidth:%lld MB/s downstream_count=%lld bytes\n",
		downstream_count_mb, downstream_count);

	VATOOLS_DBG(priv, die_index, "PCIE total bandwidth:%lld MB/s\n",
		    (upstream_count_mb + downstream_count_mb));

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->output_buf.buf_addr +
				      sizeof(TS_SMI_CMD_REQ) + sizeof(u64),
			      &downstream_count_mb, sizeof(u64));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}

out:
	return ret;
}

static long vatools_pcie_ioctl_reset(T_SMI_IOCTL_TRANS_DATA *pkdata,
				     struct vastai_pci_info *priv,
				     int die_index)
{
	long ret = -EINVAL;
	// domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	VATOOLS_DBG(
		priv, die_index,
		"smi trigger die_index=0x%x : reset. current ddr_bw_test=%d\n",
		die_index, priv->tools_info.ddr_bw_test);
	priv->tools_info.ddr_bw_test = 0;
	ret = vastai_device_reset(priv, VASTAI_RESET_MSG_TRIGER);
	// domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	return ret;
}

static long vatools_pcie_ioctl_boot(T_SMI_IOCTL_TRANS_DATA *pkdata,
				    struct vastai_pci_info *priv, int die_index)
{
	long ret = -EINVAL;

	// domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	VATOOLS_DBG(
		priv, die_index,
		"smi trigger die_index=0x%x : boot. current ddr_bw_test=%d\n",
		die_index, priv->tools_info.ddr_bw_test);
	priv->tools_info.ddr_bw_test = 0;
	ret = vastai_device_reset(priv, VASTAI_RESET_BOOT_DEV);
	// domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	return ret;
}

static long vatools_pcie_ioctl_reboot(T_SMI_IOCTL_TRANS_DATA *pkdata,
				      struct vastai_pci_info *priv,
				      int die_index)
{
	long ret = -EINVAL;

	// domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	VATOOLS_DBG(
		priv, die_index,
		"smi trigger die_index=0x%x : reboot. current ddr_bw_test=%d\n",
		die_index, priv->tools_info.ddr_bw_test);
	priv->tools_info.ddr_bw_test = 0;
	// ret = vastai_device_reset(priv, VASTAI_RESET_MSG_TRIGER |
	// 					VASTAI_RESET_BOOT_DEV);

	ret = vastai_pci_reboot(priv);
	// domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	return ret;
}

static long vatools_pcie_ioctl_fw_version(T_SMI_IOCTL_TRANS_DATA *pkdata,
					  struct vastai_pci_info *priv,
					  int die_index)
{
	long ret = -EINVAL;

	struct vastai_fw_version fw_ver;
	memset(&fw_ver, 0, sizeof(struct vastai_fw_version));
	ret = vatools_pci_mem_read(priv, die_index, FW_VERSION_ADDR, &fw_ver,
				   sizeof(struct vastai_fw_version));
	if (ret < 0) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}
	VATOOLS_DUMP_BRIEF("fw_ver: ", &fw_ver,
			   sizeof(struct vastai_fw_version));
	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      &fw_ver, sizeof(struct vastai_fw_version));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}
out:
	return ret;
}

static long vatools_pcie_ioctl_ecc_1bit(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;

	struct vastai_fifo *ecc_fifo;
	u32 table_size = pkdata->output_buf.buf_size;
	u8 *buffer;

	if (table_size > DDR_ECC_1BIT_SIZE) {
		table_size = DDR_ECC_1BIT_SIZE;
	}

	ret = 0;
	die_index = pkdata->device.die_index;
	buffer = (u8 *)kvmalloc(table_size, GFP_KERNEL);
	if (buffer == NULL) {
		VATOOLS_ERR(
			priv, die_index,
			"PCIE_CMD_FLAG_DDR_ECC_1BIT kvmalloc error. size=%d\n",
			table_size);
		ret = -ENOMEM;
		goto out;
	}

	ret = vatools_pci_mem_read(priv, die_index, DDR_ECC_1BIT, buffer,
				   table_size);
	if (ret < 0) {
		ret = -EFAULT;
		goto out_free_write_buf_1bit;
	}
	ecc_fifo = (struct vastai_fifo *)buffer;
	VATOOLS_DUMP_BRIEF("ecc 1bit: ", buffer, table_size);
	// ecc_fifo->rd 返回到应用层, 对应的变量是entry_count
	if (ecc_fifo->elem_count == 0) {
		VATOOLS_DBG(priv, die_index,
			    "ecc1bit: ecc_fifo->elem_count = 0\n");
		ecc_fifo->rd = 0;
	} else {
		ecc_fifo->rd = (ecc_fifo->elem_count +
				(((int)ecc_fifo->wr) - (int)(ecc_fifo->rd))) %
			       ecc_fifo->elem_count;
	}

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      buffer, table_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free_write_buf_1bit;
	}

out_free_write_buf_1bit:
	kvfree(buffer);
out:
	return ret;
}

static long vatools_pcie_ioctl_ecc_2bit(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	struct vastai_fifo *ecc_fifo;
	u32 table_size = pkdata->output_buf.buf_size;
	u8 *buffer;

	if (table_size > DDR_ECC_2BIT_SIZE) {
		table_size = DDR_ECC_2BIT_SIZE;
	}

	ret = 0;
	die_index = pkdata->device.die_index;
	buffer = (u8 *)kvmalloc(table_size, GFP_KERNEL);
	if (buffer == NULL) {
		VATOOLS_ERR(
			priv, die_index,
			"PCIE_CMD_FLAG_DDR_ECC_1BIT kvmalloc error. size=%d\n",
			table_size);
		ret = -ENOMEM;
		goto out;
	}

	ret = vatools_pci_mem_read(priv, die_index, DDR_ECC_2BIT, buffer,
				   table_size);
	if (ret < 0) {
		ret = -EFAULT;
		goto out_free_write_buf_2bit;
	}
	ecc_fifo = (struct vastai_fifo *)buffer;
	VATOOLS_DUMP_BRIEF("ecc 2bit: ", buffer, table_size);
	// ecc_fifo->rd 返回到应用层, 对应的变量是entry_count
	if (ecc_fifo->elem_count == 0) {
		VATOOLS_DBG(priv, die_index,
			    "ecc2bit: ecc_fifo->elem_count = 0\n");
		ecc_fifo->rd = 0;
	} else {
		ecc_fifo->rd = (ecc_fifo->elem_count +
				(((int)ecc_fifo->wr) - (int)(ecc_fifo->rd))) %
			       ecc_fifo->elem_count;
	}

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      buffer, table_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free_write_buf_2bit;
	}

out_free_write_buf_2bit:
	kvfree(buffer);
out:
	return ret;
}

static long vatools_pcie_ioctl_vdsp_count(T_SMI_IOCTL_TRANS_DATA *pkdata,
					  struct vastai_pci_info *priv,
					  int die_index)
{
	long ret = -EINVAL;

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      priv->tools_info.pcie_tx_vdsp_count,
			      sizeof(priv->tools_info.pcie_tx_vdsp_count));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	ret = copy_to_user_ex(
		(void __user *)((u8 *)pkdata->output_buf.buf_addr) +
			sizeof(u32) * 4,
		priv->tools_info.pcie_rx_vdsp_count,
		sizeof(priv->tools_info.pcie_rx_vdsp_count));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}
out:
	return ret;
}

static long vatools_pcie_ioctl_ai_count(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	u8 *ai_count_buffer;
	u32 ai_count_buffer_size = PERFORMANCE_AI_LEN;

	if (pkdata->output_buf.buf_size < ai_count_buffer_size) {
		ret = -EFAULT;
		VATOOLS_INFO(priv, die_index,
			     "output buffer size < %d ret=%ld\n",
			     ai_count_buffer_size, ret);
		goto out;
	}

	ai_count_buffer = (u8 *)kvmalloc(ai_count_buffer_size, GFP_KERNEL);
	if (ai_count_buffer == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "PCIE_CMD_FLAG_AI_COUNT kvmalloc. size=%d\n",
			    ai_count_buffer_size);
		ret = -ENOMEM;
		goto out;
	}

	die_index = pkdata->device.die_index;
	ret = vatools_pci_mem_read(priv, die_index, PERFORMANCE_AI_BASE,
				   ai_count_buffer, ai_count_buffer_size);
	if (ret < 0) {
		ret = -EFAULT;
		goto out_free_write_buf_ai_count;
	}

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      ai_count_buffer, ai_count_buffer_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free_write_buf_ai_count;
	}
out_free_write_buf_ai_count:
	kvfree(ai_count_buffer);
out:
	return ret;
}
static long
vatools_pcie_ioctl_ring_buf_statistic(T_SMI_IOCTL_TRANS_DATA *pkdata,
				      struct vastai_pci_info *priv,
				      int die_index)
{
	long ret = -EINVAL;

	u32 i = 0;
	u32 static_buf_cnt = 0;
	u8 *static_buf = NULL;
	u32 static_buf_len = 0;

	switch (priv->priv_hw_cfg->sys_cfg.fn_mode) {
	case ONLY_1_PF:
		static_buf_cnt = ONLY_1PF_RING_BUF_MAX;
		break;
	case ONLY_2_PF:
		static_buf_cnt = ONLY_2PF_RING_BUF_MAX;
		break;
	case ONLY_4_PF:
	case PF_4_WITH_SRIOV:
		static_buf_cnt = COMMON_RING_BUF_MAX;
		break;
	default:
		VATOOLS_ERR(priv, die_index, "FATAL: unknown fn_mode=%d.\n",
			    priv->priv_hw_cfg->sys_cfg.fn_mode);
		ret = -EFAULT;
		goto out;
	}

	// gpu ring buf
	static_buf_len = static_buf_cnt * sizeof(T_VASTAI_FIFO_HEADER);
	if (pkdata->output_buf.buf_size < static_buf_len) {
		ret = -EFAULT;
		VATOOLS_INFO(priv, pkdata->device.die_index,
			     "output buffer size < %d ret=%ld\n",
			     static_buf_len, ret);
		goto out;
	}
	static_buf = (u8 *)kvmalloc(static_buf_len, GFP_KERNEL);
	if (NULL == static_buf) {
		VATOOLS_ERR(priv, pkdata->device.die_index,
			    "PCIE_CMD_FLAG_HOST_COUNT kvmalloc. size=%d\n",
			    static_buf_len);
		ret = -ENOMEM;
		goto out;
	}
	memset(static_buf, 0x0, static_buf_len);

	for (i = 0; i < static_buf_cnt; i++) {
		if (NULL == priv->ring_buf[i]) {
			VATOOLS_DBG(
				priv, pkdata->device.die_index,
				"Ring buf read memory addr error,ring_buf index:%d\n",
				i);
			continue;
		}
		ret = vatools_pci_mem_read(
			priv, pkdata->device.die_index,
			priv->ring_buf[i]->ring_buf_base,
			(void *)(static_buf + i * sizeof(T_VASTAI_FIFO_HEADER)),
			sizeof(T_VASTAI_FIFO_HEADER));
		if (ret < 0) {
			VATOOLS_INFO(
				priv, pkdata->device.die_index,
				"Ring buf read memory base addr 0x%x error\n",
				priv->ring_buf[i]->ring_buf_base);
			ret = -EFAULT;
			goto out_free_static_buf;
		}
	}

	pkdata->address = static_buf_cnt;

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->output_buf.buf_addr +
				      sizeof(TS_SMI_CMD_REQ),
			      static_buf, static_buf_len);
	if (ret) {
		VATOOLS_ERR(priv, pkdata->device.die_index,
			    "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free_static_buf;
	}

out_free_static_buf:
	kvfree(static_buf);
out:
	return ret;
}

static long
vatools_pcie_ioctl_ring_buf_from_hw_cfg(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;

	u32 i = 0;
	u32 static_buf_cnt = 0;
	u8 *static_buf = NULL;
	u32 static_buf_len = 0;
	static_buf_cnt = priv->priv_hw_cfg->header.total_cfg_entry;
	// gpu ring buf
	static_buf_len = static_buf_cnt * sizeof(T_VASTAI_FIFO_HEADER_WITHNAME);
	if (pkdata->output_buf.buf_size < static_buf_len) {
		ret = -EFAULT;
		VATOOLS_DBG(priv, pkdata->device.die_index,
			    "output buffer size < %d ret=%ld\n", static_buf_len,
			    ret);
		goto out;
	}
	static_buf = (u8 *)kvmalloc(static_buf_len, GFP_KERNEL);
	if (NULL == static_buf) {
		VATOOLS_DBG(priv, pkdata->device.die_index,
			    "PCIE_CMD_FLAG_HOST_COUNT kvmalloc. size=%d\n",
			    static_buf_len);
		ret = -ENOMEM;
		goto out;
	}
	memset(static_buf, 0x0, static_buf_len);

	for (i = 0; i < static_buf_cnt; i++) {
		// if (NULL == priv->ring_buf[i]) {
		// 	VATOOLS_DBG(
		// 		priv, pkdata->device.die_index,
		// 		"Ring buf read memory addr error,ring_buf index:%d\n",
		// 		i);
		// 	continue;
		// }
		if (0 == priv->priv_hw_cfg->ring_buf_entry[i].ring_buf_base) {
			continue;
		}
		if (NOT_VALID_INFO ==
		    priv->priv_hw_cfg->ring_buf_entry[i].msgq_ctrl_reg_base) {
			ret = vatools_pci_mem_read(
				priv, pkdata->device.die_index,
				priv->priv_hw_cfg->ring_buf_entry[i]
					.ring_buf_base,
				(void *)(static_buf +
					 i * sizeof(T_VASTAI_FIFO_HEADER_WITHNAME)),
				sizeof(T_VASTAI_FIFO_HEADER));
			if (ret < 0) {
				VATOOLS_ERR(
					priv, pkdata->device.die_index,
					"Ring buf read memory base addr 0x%x error\n",
					priv->priv_hw_cfg->ring_buf_entry[i]
						.ring_buf_base);
				ret = -EFAULT;
				goto out_free_static_buf;
			}
			memcpy(static_buf +
				       i * sizeof(T_VASTAI_FIFO_HEADER_WITHNAME) +
				       sizeof(T_VASTAI_FIFO_HEADER),
			       priv->priv_hw_cfg->ring_buf_entry[i].name,
			       CMD_BUF_NAME_LONG);
			VATOOLS_INFO(
				priv, pkdata->device.die_index,
				"Name: %s, total buf count %d, now loop %d times,\n",
				priv->priv_hw_cfg->ring_buf_entry[i].name,
				static_buf_cnt, i);
		} else {
			VATOOLS_INFO(
				priv, pkdata->device.die_index,
				"Msgq name: %s, total buf count %d, now loop %d times,\n",
				priv->priv_hw_cfg->ring_buf_entry[i].name,
				static_buf_cnt, i);
		}
	}

	pkdata->address = static_buf_cnt;

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->output_buf.buf_addr,
			      static_buf, static_buf_len);
	if (ret) {
		VATOOLS_ERR(priv, pkdata->device.die_index,
			    "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free_static_buf;
	}

out_free_static_buf:
	kvfree(static_buf);
out:
	return ret;
}
#ifdef CONFIG_VASTAI_VIDEO

static long
vatools_pcie_ioctl_set_video_multi_core(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	u32 work_mode = (pkdata->address & 0xFFFFFFFF);
	// call video callback function here
	VATOOLS_DBG(priv, die_index,
		    "die inex=0x%x set video multicore work_mode to %d\n",
		    pkdata->device.die_index, work_mode);
	ret = vastai_enc_set_work_mode(pkdata->device.die_index,
				       (encoder_workmode_t)work_mode, 0);

	return ret;
}

static long
vatools_pcie_ioctl_get_video_multi_core(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	u32 work_mode = 0;
	int job_count = 0;
	// call video callback function here
	ret = vastai_enc_get_work_mode(pkdata->device.die_index,
				       (encoder_workmode_t *)&work_mode,
				       &job_count);
	pkdata->address = work_mode;
	pkdata->address = pkdata->address << 32 | job_count;

	VATOOLS_DBG(
		priv, die_index,
		"die inex=0x%x get video multicore work_mode=%d, job_count=%d\n",
		pkdata->device.die_index, work_mode, job_count);
	return ret;
}
#endif

static long vatools_pcie_ioctl_heart_beat(T_SMI_IOCTL_TRANS_DATA *pkdata,
					  struct vastai_pci_info *priv,
					  int die_index)
{
	long ret = -EINVAL;
	core_heartbeat_t core_heartbeat;
	memset(&core_heartbeat, 0, sizeof(core_heartbeat_t));

	ret = vatools_pci_mem_read(priv, die_index, CORE_HEARTBEAT_ADDR,
				   &core_heartbeat, CORE_HEARTBEAT_LEN);
	if (ret < 0) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			     ret);
		ret = -EFAULT;
		goto out;
	}

	VATOOLS_DBG(priv, die_index, "die inex=0x%x read heartbeat ret=%ld\n",
		    pkdata->device.die_index, ret);

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      &core_heartbeat, CORE_HEARTBEAT_LEN);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
	}
	goto out;
out:
	return ret;
}

static long vatools_pcie_ioctl_barreset(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	u32 bar_readdata = 0;
	ret = 0;
	ret = vatools_pci_mem_read(priv, die_index, DDR_PCIE_BARRESET,
				   &bar_readdata, sizeof(u32));
	if (ret < 0) {
		ret = -EFAULT;
		goto out;
	}
	VATOOLS_DBG(priv, die_index, "bar_readdata: ret=0x%x\n", bar_readdata);

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->output_buf.buf_addr +
				      sizeof(TS_SMI_CMD_REQ),
			      &bar_readdata, sizeof(u32));
	if (ret < 0) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}
out:
	return ret;
}

static long vatools_pcie_ioctl_barstatus(T_SMI_IOCTL_TRANS_DATA *pkdata,
					 struct vastai_pci_info *priv,
					 int die_index)
{
	long ret = -EINVAL;

	int i = 0;
	struct bar_info *p_at_addr_array = NULL;
	p_at_addr_array = (struct bar_info *)vmalloc(sizeof(struct bar_info) *
						     VASTAI_PCI_BAR_NUM);
	if (p_at_addr_array == NULL) {
		VATOOLS_ERR(priv, die_index, "malloc p_at_addr_array failed\n");
		goto out;
	}
	memset(p_at_addr_array, 0x0,
	       sizeof(struct bar_info) * VASTAI_PCI_BAR_NUM);
	for (; i < VASTAI_PCI_BAR_NUM; i++) {
		VATOOLS_DBG(priv, die_index, "noc_addr:%llx\n",
			    priv->bar[i].noc_addr);
	}
	memcpy(p_at_addr_array, &priv->bar[0],
	       sizeof(struct bar_info) * VASTAI_PCI_BAR_NUM);
	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      p_at_addr_array,
			      sizeof(struct bar_info) * VASTAI_PCI_BAR_NUM);

	if (ret < 0) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
		goto out_free;
	}
out_free:
	vfree(p_at_addr_array);
out:
	return ret;
}

static long vatools_pcie_ioctl_reboot_ddrbw(T_SMI_IOCTL_TRANS_DATA *pkdata,
					    struct vastai_pci_info *priv,
					    int die_index)
{
	long ret = -EINVAL;

	// domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	VATOOLS_DBG(
		priv, die_index,
		"smi trigger die_index=0x%x: ddr bw test. current ddr_bw_test=%d\n",
		die_index, priv->tools_info.ddr_bw_test);
	priv->tools_info.ddr_bw_test = 1;
	atomic_set(&(priv->pci_state), VASTAI_DEBUG_STATE);
	ret = vastai_ddr_bw_test(priv);
	priv->tools_info.ddr_bw_test = 0;
	// domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	return ret;
}

static long vatools_pcie_ioctl_reboot_pciedma(T_SMI_IOCTL_TRANS_DATA *pkdata,
					      struct vastai_pci_info *priv,
					      int die_index)
{
	long ret = -EINVAL;

	// domain_mutex_lock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	VATOOLS_DBG(
		priv, die_index,
		"smi trigger die_index=0x%x: pcie bw test. current ddr_bw_test=%d\n",
		die_index, priv->tools_info.ddr_bw_test);

	atomic_set(&(priv->pci_state), VASTAI_DEBUG_STATE);
	ret = vastai_pcie_bw_test(priv);
	// domain_mutex_unlock(priv, die_index, VATOOLS_IOCTL_DOMAIN_DEVICE);
	return ret;
}

static long vatools_pcie_ioctl_runtime_info(T_SMI_IOCTL_TRANS_DATA *pkdata,
					    struct vastai_pci_info *priv,
					    u32 die_index)
{
	long ret = -EINVAL;

#ifdef CONFIG_VASTAI_AI
	u32 k = 0;
	u32 j = 0;
	vatools_runtime_info_head_t *info_head_p = NULL;
	vatools_runtime_info_head_t *info_head_one_p = NULL;
	vacc_info_head_t info_head;

	vatools_runtime_zone_usage_t *zone_usage_p = NULL;
	vatools_runtime_zone_usage_t *zone_usage_one_p = NULL;
	vacc_zone_usage_t zone_usage[zone_count];
	vatools_runtime_process_info_t *process_info_p = NULL;
	vatools_runtime_process_info_t *process_info_one_p = NULL;
	vacc_process_info_t *vacc_process_info_p = NULL;
	u32 vacc_process_info_count = 0;

	vastai_runtime_statistic_t runtime_statistic;
	struct vastai_cdev *vacc_dev_p = NULL;
	struct vastai_cdev *vacc_dev_temp = NULL;

	union die_index_data did;
	union die_index_data die_index_return;
	struct vastai_pci_info *pci_info = NULL;
	u32 device_num = 0;
#endif

	VATOOLS_FUNC_ENTERY;

#ifdef CONFIG_VASTAI_AI

	VATOOLS_DBG(priv, die_index, "die_index=0x%x\n", die_index);

	memset(&runtime_statistic, 0, sizeof(vastai_runtime_statistic_t));

	ret = copy_from_user_ex(&runtime_statistic,
				(const void __user *)pkdata->input_buf.buf_addr,
				sizeof(vastai_runtime_statistic_t));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

	// 获取当前进程的信息
	ret = vatools_get_current_process_info(
		&runtime_statistic.current_process_info);
	if (ret < 0) {
		VATOOLS_DBG(priv, die_index,
			    "get current process failed. ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}
	runtime_statistic.current_process_info.die_index = die_index;

	// 获取 device 数量
	device_num = vatools_get_vastai_pci_device_number();
	VATOOLS_DBG(
		priv, die_index,
		"die_index=0x%x device_num=%d runtime_statistic.info_head_max_num=%d zone_usage_max_num=%d process_info_max_num=%d\n",
		die_index, device_num, runtime_statistic.info_head_max_num,
		runtime_statistic.zone_usage_max_num,
		runtime_statistic.process_info_max_num);

	if (device_num == 0) {
		ret = -ENODEV;
		VATOOLS_DBG(priv, die_index, "device_num is 0. ret=%ld\n", ret);
		goto out;
	}

	// TODO: 获取 info_head 遍历所有卡和die， 每个 die 一个结构
	k = 0;
	if (runtime_statistic.info_head_max_num) {
		VATOOLS_DBG(priv, die_index, "info_head_max_num=%d\n",
			    runtime_statistic.info_head_max_num);
		// 每个 die 一个结构
		info_head_p = vmalloc(runtime_statistic.info_head_max_num *
				      sizeof(vatools_runtime_info_head_t));
		if (info_head_p == NULL) {
			VATOOLS_ERR(
				priv, die_index, "vmalloc error. size=%ld\n",
				runtime_statistic.info_head_max_num *
					sizeof(vatools_runtime_info_head_t));
			ret = -ENOMEM;
			goto out;
		}
		memset(info_head_p, 0,
		       runtime_statistic.info_head_max_num *
			       sizeof(vatools_runtime_info_head_t));

		// 遍历所有 device
		list_for_each_entry (pci_info, &vastai_pci_info_list,
				     dev_list) {
			if (pci_info == NULL) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "pci_info is NULL\n");
				continue;
			}
			if (vatools_check_pci_state(pci_info,
						    VASTAI_NORMAL_STATE, -1)) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "priv->pci_state=0x%x\n",
					    atomic_read(&pci_info->pci_state));
				continue;
			}

			// vacc
			list_for_each_entry_safe (vacc_dev_p, vacc_dev_temp,
						  &pci_info->vacc_dev_head,
						  vacc_dev_node) {
				VATOOLS_DBG(pci_info, die_index,
					    "vacc_dev_p=%p\n", vacc_dev_p);

				if (((u32)die_index == 0xffffffff) ||
				    (vacc_dev_p->die_index == die_index)) {
					VATOOLS_DBG(
						pci_info, die_index,
						"get info_head from vacc_dev_p=%p\n",
						vacc_dev_p);
					info_head_one_p =
						(vatools_runtime_info_head_t
							 *)(info_head_p + k);
					memset(&info_head, 0,
					       sizeof(vacc_info_head_t));
					ret = vaccrt_die_info_get_head(
						vacc_dev_p, &info_head);
					if (ret < 0) {
						VATOOLS_DBG(
							pci_info, die_index,
							"vaccrt_die_info_get_head failed. ret=%ld\n",
							ret);
					} else {
						info_head_one_p->die_index =
							vacc_dev_p->die_index;
						did.val = info_head_one_p
								  ->die_index;
						info_head_one_p->die_id =
							0; //did.seq_num;
						info_head_one_p->ai_version =
							info_head.ai_version;
						info_head_one_p->zone_count =
							info_head.zone_count;
						info_head_one_p->process_count =
							info_head.process_count;

						vaccrt_version_number2string(
							info_head_one_p
								->ai_version,
							info_head_one_p
								->ai_version_str);
						k++;
						runtime_statistic.info_head_num =
							k;
						VATOOLS_DBG(
							pci_info, die_index,
							"runtime_statistic.info_head_num=%d ai_version=0x%x zone_count=%d process_count=%d info_head_num=%d\n",
							runtime_statistic
								.info_head_num,
							info_head_one_p
								->ai_version,
							info_head_one_p
								->zone_count,
							info_head_one_p
								->process_count,
							runtime_statistic
								.info_head_num);

						if (runtime_statistic
							    .info_head_num ==
						    runtime_statistic
							    .info_head_max_num) {
							VATOOLS_DBG(
								pci_info,
								die_index,
								"runtime_statistic.info_head_num=%d reach max. exit loop.\n",
								runtime_statistic
									.info_head_num);
							goto out_info_head_max;
						}
					}
				} // if die_index
			} // list loop
		} // for (dev
	out_info_head_max:
		ret = copy_to_user_ex(
			(void __user *)runtime_statistic.info_head_p,
			info_head_p,
			runtime_statistic.info_head_max_num *
				sizeof(vatools_runtime_info_head_t));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_to_user_ex info_head ret=%ld\n", ret);
			ret = -EFAULT;
			goto out_info_head;
		}

		ret = copy_to_user_ex((void __user *)pkdata->input_buf.buf_addr,
				      &runtime_statistic,
				      sizeof(vastai_runtime_statistic_t));
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_to_user_ex runtime_statistic ret=%ld\n",
				ret);
			ret = -EFAULT;
			goto out_info_head;
		}
	out_info_head:
		vfree(info_head_p);
	} // end of info_head

	// TODO: 获取 zone_usage 遍历所有卡和die， 每个 die 一个结构
	k = 0;
	if (runtime_statistic.zone_usage_max_num) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "zone_usage_max_num=%d\n",
			    runtime_statistic.zone_usage_max_num);
		// 每个 die 一个结构
		zone_usage_p = vmalloc(runtime_statistic.zone_usage_max_num *
				       sizeof(vatools_runtime_zone_usage_t));
		if (zone_usage_p == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID, "vmalloc error. size=%ld\n",
				runtime_statistic.zone_usage_max_num *
					sizeof(vatools_runtime_zone_usage_t));
			ret = -ENOMEM;
			goto out;
		}
		memset(zone_usage_p, 0,
		       runtime_statistic.zone_usage_max_num *
			       sizeof(vatools_runtime_zone_usage_t));

		// 遍历所有 device
		list_for_each_entry (pci_info, &vastai_pci_info_list,
				     dev_list) {
			if (pci_info == NULL) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "pci_info is NULL\n");
				continue;
			}
			if (vatools_check_pci_state(pci_info,
						    VASTAI_NORMAL_STATE, -1)) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "priv->pci_state=0x%x\n",
					    atomic_read(&pci_info->pci_state));
				continue;
			}

			// vacc
			list_for_each_entry_safe (vacc_dev_p, vacc_dev_temp,
						  &pci_info->vacc_dev_head,
						  vacc_dev_node) {
				VATOOLS_DBG(pci_info, die_index,
					    "vacc_dev_p=%p\n", vacc_dev_p);

				if (((u32)die_index == 0xffffffff) ||
				    (vacc_dev_p->die_index == die_index)) {
					VATOOLS_DBG(
						pci_info, die_index,
						"get zone_usage from vacc_dev_p=%p\n",
						vacc_dev_p);
					zone_usage_one_p =
						(vatools_runtime_zone_usage_t
							 *)(zone_usage_p + k);
					memset(zone_usage, 0,
					       sizeof(vacc_zone_usage_t) *
						       zone_count);
					ret = vaccrt_die_info_get_zone_ddr(
						vacc_dev_p, zone_usage);
					if (ret < 0) {
						VATOOLS_DBG(
							pci_info, die_index,
							"vaccrt_die_info_get_zone_ddr failed. ret=%ld\n",
							ret);
					} else {
						zone_usage_one_p->die_index =
							vacc_dev_p->die_index;
						die_index_return.val = 0;
						die_index_return.dev_id =
							vacc_dev_p->pci_info
								->dev_id;
						die_index_return.RESV =
							vacc_dev_p->pci_info
								->dev_id;
						zone_usage_one_p->die_index =
							die_index_return.val;
						VATOOLS_DBG(
							pci_info, die_index,
							"Dev_id: %d,Die_index； 0x%x\n",
							vacc_dev_p->pci_info
								->dev_id,
							zone_usage_one_p
								->die_index);
						memcpy(zone_usage_one_p
							       ->zone_usage,
						       zone_usage,
						       sizeof(vacc_zone_usage_t) *
							       zone_count);

						zone_usage_one_p
							->ddr_total_size =
							vacc_dev_p->soc
								.ddr_total_size;
						k++;
						runtime_statistic
							.zone_usage_num = k;
						VATOOLS_DBG(
							pci_info, die_index,
							"runtime_statistic.zone_usage_num=%d total_size=%lld free_size=%lld zone_usage_num=%d\n",
							runtime_statistic
								.zone_usage_num,
							zone_usage_one_p
								->zone_usage[0]
								.total_size,
							zone_usage_one_p
								->zone_usage[0]
								.free_size,
							runtime_statistic
								.zone_usage_num);

						if (runtime_statistic
							    .zone_usage_num ==
						    runtime_statistic
							    .zone_usage_max_num) {
							VATOOLS_DBG(
								pci_info,
								die_index,
								"runtime_statistic.zone_usage_num=%d reach max. exit loop.\n",
								runtime_statistic
									.zone_usage_num);
							goto out_zone_usage_max;
						}
					}
				} // if die_index
			} // list loop
		} // for (dev
	out_zone_usage_max:
		ret = copy_to_user_ex(
			(void __user *)runtime_statistic.zone_usage_p,
			zone_usage_p,
			runtime_statistic.zone_usage_max_num *
				sizeof(vatools_runtime_zone_usage_t));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_to_user_ex zone_usage ret=%ld\n",
				    ret);
			ret = -EFAULT;
			goto out_zone_usage;
		}

		ret = copy_to_user_ex((void __user *)pkdata->input_buf.buf_addr,
				      &runtime_statistic,
				      sizeof(vastai_runtime_statistic_t));
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_to_user_ex runtime_statistic ret=%ld\n",
				ret);
			ret = -EFAULT;
			goto out_zone_usage;
		}
	out_zone_usage:
		vfree(zone_usage_p);
	} // end of zone_usage

	// TODO: 获取 process_info 遍历所有卡和die， 每个 die 一个结构
	k = 0;
	if (runtime_statistic.process_info_max_num) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process_info_max_num=%d\n",
			    runtime_statistic.process_info_max_num);
		// 每个 进程 一个结构
		// 返回给app的数据buf
		process_info_p =
			vmalloc(runtime_statistic.process_info_max_num *
				sizeof(vatools_runtime_process_info_t));
		if (process_info_p == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"vmalloc error process_info_p. size=%ld\n",
				runtime_statistic.process_info_max_num *
					sizeof(vatools_runtime_process_info_t));
			ret = -ENOMEM;
			goto out;
		}
		// 提供给vaccrt 接口返回数据的buf
		vacc_process_info_p =
			vmalloc(runtime_statistic.process_info_max_num *
				sizeof(vacc_process_info_t));
		if (vacc_process_info_p == NULL) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"vmalloc error vacc_process_info_p. size=%ld\n",
				runtime_statistic.process_info_max_num *
					sizeof(vacc_process_info_t));
			ret = -ENOMEM;
			goto out_process_info;
		}

		memset(process_info_p, 0,
		       runtime_statistic.process_info_max_num *
			       sizeof(vatools_runtime_process_info_t));

		memset(vacc_process_info_p, 0,
		       runtime_statistic.process_info_max_num *
			       sizeof(vacc_process_info_t));

		// 遍历所有 device
		list_for_each_entry (pci_info, &vastai_pci_info_list,
				     dev_list) {
			if (pci_info == NULL) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "pci_info is NULL\n");
				continue;
			}
			if (vatools_check_pci_state(pci_info,
						    VASTAI_NORMAL_STATE, -1)) {
				VATOOLS_DBG(NULL, DUMMY_DIE_ID,
					    "priv->pci_state=0x%x\n",
					    atomic_read(&pci_info->pci_state));
				continue;
			}

			// vacc
			list_for_each_entry_safe (vacc_dev_p, vacc_dev_temp,
						  &pci_info->vacc_dev_head,
						  vacc_dev_node) {
				VATOOLS_DBG(pci_info, die_index,
					    "vacc_dev_p=%p\n", vacc_dev_p);

				if (((u32)die_index == 0xffffffff) ||
				    (vacc_dev_p->die_index == die_index)) {
					VATOOLS_DBG(
						pci_info, die_index,
						"get process_info from vacc_dev_p=%p\n",
						vacc_dev_p);

					ret = vaccrt_die_info_get_process(
						vacc_dev_p, vacc_process_info_p,
						runtime_statistic
							.process_info_max_num,
						&vacc_process_info_count);

					VATOOLS_DBG(
						pci_info, die_index,
						"get process_info vacc_process_info_count=%d ret=%ld\n",
						vacc_process_info_count, ret);

					if (ret < 0) {
						VATOOLS_DBG(
							pci_info, die_index,
							"vaccrt_die_info_get_process failed. ret=%ld\n",
							ret);
					} else {
						for (j = 0;
						     j <
						     vacc_process_info_count;
						     j++) {
							process_info_one_p =
								(vatools_runtime_process_info_t
									 *)(process_info_p +
									    k);

							process_info_one_p
								->die_index =
								vacc_dev_p
									->die_index;
							die_index_return.val =
								0;
							die_index_return.dev_id =
								vacc_dev_p
									->pci_info
									->dev_id;
							die_index_return.RESV =
								vacc_dev_p
									->pci_info
									->dev_id;
							process_info_one_p
								->die_index =
								die_index_return
									.val;
							VATOOLS_DBG(
								pci_info,
								die_index,
								"process_info=%u pid=%u vpid=%u level=%u ns=%llu zone=%llu %llu %llu %llu %llu\n",
								j,
								(vacc_process_info_p +
								 j)
									->pid,
								(vacc_process_info_p +
								 j)
									->vpid,
								(vacc_process_info_p +
								 j)
									->level,
								(vacc_process_info_p +
								 j)
									->ns,
								(vacc_process_info_p +
								 j)
									->zone_used
										[0],
								(vacc_process_info_p +
								 j)
									->zone_used
										[1],
								(vacc_process_info_p +
								 j)
									->zone_used
										[2],
								(vacc_process_info_p +
								 j)
									->zone_used
										[3],
								(vacc_process_info_p +
								 j)
									->zone_used
										[4]);

							memcpy((void *)((u8 *)process_info_one_p +
									sizeof(u32)),
							       (void *)(vacc_process_info_p +
									j),
							       sizeof(vacc_process_info_t));

							k++;
							runtime_statistic
								.process_info_num =
								k;
							VATOOLS_DBG(
								pci_info,
								die_index,
								"runtime_statistic.process_info_num=%d die_index=0x%x pid=%u vpid=%u level=%u ns=%llu zone=%llu %llu %llu %llu %llu\n",
								runtime_statistic
									.process_info_num,
								process_info_one_p
									->die_index,
								process_info_one_p
									->pid,
								process_info_one_p
									->vpid,
								process_info_one_p
									->level,
								process_info_one_p
									->ns,
								process_info_one_p
									->zone_used
										[0],
								process_info_one_p
									->zone_used
										[1],
								process_info_one_p
									->zone_used
										[2],
								process_info_one_p
									->zone_used
										[3],
								process_info_one_p
									->zone_used
										[4]);

							if (runtime_statistic
								    .process_info_num ==
							    runtime_statistic
								    .process_info_max_num) {
								VATOOLS_DBG(
									pci_info,
									die_index,
									"runtime_statistic.process_info_num=%d reach max. exit loop.\n",
									runtime_statistic
										.process_info_num);
								goto out_process_info_max;
							}
						} // for process
					} // vaccrt_die_info_get_process success
				} // if die_index
			} // list loop
		} // for (dev
	out_process_info_max:
		ret = copy_to_user_ex(
			(void __user *)runtime_statistic.process_info_p,
			process_info_p,
			runtime_statistic.process_info_max_num *
				sizeof(vatools_runtime_process_info_t));
		if (ret) {
			VATOOLS_ERR(NULL, DUMMY_DIE_ID,
				    "copy_to_user_ex process_info ret=%ld\n",
				    ret);
			ret = -EFAULT;
			goto out_process_info;
		}

		ret = copy_to_user_ex((void __user *)pkdata->input_buf.buf_addr,
				      &runtime_statistic,
				      sizeof(vastai_runtime_statistic_t));
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_to_user_ex runtime_statistic ret=%ld\n",
				ret);
			ret = -EFAULT;
			goto out_process_info;
		}
	out_process_info:
		if (process_info_p) {
			vfree(process_info_p);
			process_info_p = NULL;
		}
		if (vacc_process_info_p) {
			vfree(vacc_process_info_p);
			vacc_process_info_p = NULL;
		}
	} // end of process_info

#endif
	VATOOLS_FUNC_EXIT;
out:
	return ret;
}

static long
vatools_pcie_ioctl_current_process_info(T_SMI_IOCTL_TRANS_DATA *pkdata,
					struct vastai_pci_info *priv,
					int die_index)
{
	long ret = -EINVAL;
	vatools_current_process_info_t current_process_info;

	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(priv, die_index, "die_index=0x%x\n", die_index);

	memset(&current_process_info, 0,
	       sizeof(vatools_current_process_info_t));
	// 获取当前进程的信息
	ret = vatools_get_current_process_info(&current_process_info);
	if (ret < 0) {
		VATOOLS_INFO(priv, die_index,
			     "get current process failed. ret=%ld\n", ret);
		ret = -EFAULT;
		goto out;
	}
	current_process_info.die_index = die_index;
	ret = copy_to_user_ex((void __user *)pkdata->input_buf.buf_addr,
			      &current_process_info,
			      sizeof(vatools_current_process_info_t));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

out:
	VATOOLS_FUNC_EXIT;
	return ret;
}

static long vatools_pcie_ioctl_heartbeat_count(T_SMI_IOCTL_TRANS_DATA *pkdata,
					       struct vastai_pci_info *priv,
					       int die_index)
{
	long ret = -EINVAL;
#if 0
//TODO: tobedone
	u32 die_id = 0;
	union die_index_data did;
	vatools_heartbeat_count_t *heartbeat_count_p = NULL;
	int i = 0;
	int interval_seconds = 3;

	VATOOLS_FUNC_ENTERY;
	did.val = die_index;
	die_id = did.die_id;
	VATOOLS_DBG(priv, die_index, "die_index=0x%x die_id=%d\n", die_index,
		    die_id);

	heartbeat_count_p = vmalloc(sizeof(vatools_heartbeat_count_t));
	if (heartbeat_count_p == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "heartbeat_count_p vmalloc error. size=%ld\n",

			    sizeof(vatools_heartbeat_count_t));
		ret = -ENOMEM;
		goto out;
	}

	memset(heartbeat_count_p, 0, sizeof(vatools_heartbeat_count_t));

	for (i = 0; i < HEARTBEAT_MAX_CNT; i++) {
		ret = vastai_read_heartbeat(
			priv, die_id, i * interval_seconds,
			&(heartbeat_count_p->heartbeat_counts[i][0]));
		if (ret < 0) {
			VATOOLS_INFO(priv, die_index,
				     "get heartbeat failed. ret=%ld\n", ret);
			ret = -EFAULT;
			goto out_free;
		}
	}

	heartbeat_count_p->die_index = die_index;
	ret = copy_to_user_ex((void __user *)pkdata->input_buf.buf_addr,
			      heartbeat_count_p,
			      sizeof(vatools_heartbeat_count_t));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free;
	}
out_free:
	vfree(heartbeat_count_p);
out:
	VATOOLS_FUNC_EXIT;
#endif
	return ret;
}

static long vatools_pcie_ioctl_ddr_bandwidth(T_SMI_IOCTL_TRANS_DATA *pkdata,
					     struct vastai_pci_info *priv,
					     int die_index)
{
	long ret = -EINVAL;

	u64 read_cont = 0;
	u64 write_cont = 0;
	u64 upstream_count = 0;
	u64 downstream_count = 0;
	u32 ch =
		0; // ddr channel, when 4 ddr use channel 2 6 3 7, 8 ddr use 0~7
	u32 port = 0; //ddr axi port
	u32 val = 10000; // interval is 10ms
	vastai_ddr_bandwidth_t ddr_bandwidth;

	memset(&ddr_bandwidth, 0, sizeof(vastai_ddr_bandwidth_t));
	for (ch = 0; ch < SG100_DDR_MC_PHY_MAX; ch++) {
		upstream_count = 0;
		downstream_count = 0;
		/* UPSTREAM read from ddr*/
		for (port = 0; port < SG100_DDR_MC_PHY_PORT_MAX; port++) {
			ret = vastai_ddr_monitor(priv, val, 1, ch, port,
						 &read_cont);
			if (ret < 0) {
				VATOOLS_ERR(
					priv, DUMMY_DIE_ID,
					"ddr_monitor : mc %d upstream fail\n",
					ch);
				goto out;
			}
			upstream_count += read_cont;
		}
		ddr_bandwidth.ddr_bandwidth[ch].upstream_count_mb =
			upstream_count * 1000000 / val / 1024 / 1024;

		/* DOWNSTREAM write to ddr*/
		for (port = 0; port < SG100_DDR_MC_PHY_PORT_MAX; port++) {
			ret = vastai_ddr_monitor(priv, val, 0, ch, port,
						 &write_cont);
			if (ret < 0) {
				VATOOLS_ERR(
					priv, DUMMY_DIE_ID,
					"ddr_monitor : mc %d downstream fail\n",
					ch);
				goto out;
			}
			downstream_count += write_cont;
		}
		ddr_bandwidth.ddr_bandwidth[ch].downstream_count_mb =
			downstream_count * 1000000 / val / 1024 / 1024;

		VATOOLS_DBG(
			priv, DUMMY_DIE_ID,
			"DDR upstream (read from ddr) bandwidth:%lld MB/s\n",
			upstream_count * 1000000 / val / 1024 / 1024);

		VATOOLS_DBG(
			priv, DUMMY_DIE_ID,
			"DDR downstream (write to ddr) bandwidth:%lld MB/s\n",
			downstream_count * 1000000 / val / 1024 / 1024);

		VATOOLS_DBG(priv, DUMMY_DIE_ID,
			    "DDR total bandwidth:%lld MB/s\n",
			    (upstream_count + downstream_count) * 1000000 /
				    val / 1024 / 1024);
	}

	ret = copy_to_user_ex((void __user *)(u8 *)pkdata->input_buf.buf_addr,
			      &ddr_bandwidth, sizeof(vastai_ddr_bandwidth_t));
	if (ret) {
		VATOOLS_ERR(priv, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out;
	}

out:
	return ret;
}

static long vatools_pcie_ioctl_card_cpu_topo(T_SMI_IOCTL_TRANS_DATA *pkdata,
					     struct vastai_pci_info *priv,
					     int die_index)
{
	long ret = -EINVAL;
	char *cpu_card_topo_str =
		(char *)kvmalloc(CPU_AND_CARD_TOPO_STR_LEN, GFP_KERNEL);
	if (cpu_card_topo_str == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "malloc cpu_card_topo_str buffer failed\n");
		return -EINVAL;
	}
	memset(cpu_card_topo_str, 0x0, CPU_AND_CARD_TOPO_STR_LEN);
	//ret = vastai_pci_numa_cpu_info(priv, cpu_card_topo_str,
	//			       CPU_AND_CARD_TOPO_STR_LEN);
	ret = vastai_pci_numa_cpu_info_all(cpu_card_topo_str,
					   CPU_AND_CARD_TOPO_STR_LEN);
	if (ret) {
		VATOOLS_INFO(priv, die_index,
			     "get topological relation failed, ret=%ld\n", ret);
		ret = -EFAULT;
		goto free;
	}
	VATOOLS_DBG(priv, die_index, "get topological relation: %s\n",
		    cpu_card_topo_str);
	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      cpu_card_topo_str, CPU_AND_CARD_TOPO_STR_LEN);
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto free;
	}
free:
	kvfree(cpu_card_topo_str);
	return ret;
}

static long vatools_pcie_ioctl_gfx_utilization(T_SMI_IOCTL_TRANS_DATA *pkdata,
					       struct vastai_pci_info *priv,
					       int die_index)
{
	long ret = -EINVAL;
	u32 gfx_cmd = TOOL_CMD_GFX_NONE;
	gfx_cmd_input_args_t *gfx_cmd_input_args_p = NULL;
	gfx_utilization_info_t *gfx_utilization_info_p = NULL;
	int i = 0;

	gfx_cmd = pkdata->address;

	// input args
	gfx_cmd_input_args_p = (gfx_cmd_input_args_t *)kvmalloc(
		sizeof(gfx_cmd_input_args_t), GFP_KERNEL);
	if (gfx_cmd_input_args_p == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "malloc gfx_cmd_input_args_p buffer failed\n");
		goto out;
	}
	memset(gfx_cmd_input_args_p, 0x0, sizeof(gfx_cmd_input_args_t));

	// output data
	gfx_utilization_info_p = (gfx_utilization_info_t *)kvmalloc(
		sizeof(gfx_utilization_info_t), GFP_KERNEL);
	if (gfx_utilization_info_p == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "malloc gfx_utilization_info_p buffer failed\n");
		goto out_free_input_args;
	}
	memset(gfx_utilization_info_p, 0x0, sizeof(gfx_utilization_info_t));

	ret = copy_from_user_ex(gfx_cmd_input_args_p,
				(void __user *)pkdata->input_buf.buf_addr,
				pkdata->input_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index,
			    "copy_from_user_ex ret=%ld gfx_cmd=0x%x\n", ret,
			    gfx_cmd);
		ret = -EFAULT;
		goto out_free_info;
	}
	VATOOLS_DBG(priv, die_index, "gfx_cmd=%d pf_num=%d pf_index[0]=%d\n",
		    gfx_cmd, gfx_cmd_input_args_p->pf_num,
		    gfx_cmd_input_args_p->pf_index[0]);

	ret = vatools_gfx_interface(priv, gfx_cmd, (u64)gfx_cmd_input_args_p,
				    sizeof(gfx_cmd_input_args_t),
				    (u64)gfx_utilization_info_p,
				    sizeof(gfx_utilization_info_t));
	if (ret) {
		VATOOLS_DBG(priv, die_index, "vatools_gfx_interface ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_info;
	}

	VATOOLS_DBG(priv, die_index, "gfx_cmd=%d pf_num=%d ret=%ld\n", gfx_cmd,
		    gfx_utilization_info_p->pf_num, ret);
	// DEBUG: for dump data
	for (i = 0; i < gfx_utilization_info_p->pf_num; i++) {
		VATOOLS_DBG(priv, die_index,
			    "[%02d] gfx pf_index=%d utilization=%d\n", i,
			    gfx_utilization_info_p->utilization[i].pf_index,
			    gfx_utilization_info_p->utilization[i].utilization);
	}

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      gfx_utilization_info_p,
			      sizeof(gfx_utilization_info_t));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
	}
	goto out_free_info;

out_free_info:
	kvfree(gfx_utilization_info_p);

out_free_input_args:
	kvfree(gfx_cmd_input_args_p);
out:
	return ret;
}

static long vatools_pcie_ioctl_gfx_memory_usage(T_SMI_IOCTL_TRANS_DATA *pkdata,
						struct vastai_pci_info *priv,
						int die_index)
{
	long ret = -EINVAL;
	u32 gfx_cmd = TOOL_CMD_GFX_NONE;
	gfx_cmd_input_args_t *gfx_cmd_input_args_p = NULL;
	gfx_memory_usage_info_t *memory_usage_info_p = NULL;
	int i = 0;

	gfx_cmd = pkdata->address;

	// input args
	gfx_cmd_input_args_p = (gfx_cmd_input_args_t *)kvmalloc(
		sizeof(gfx_cmd_input_args_t), GFP_KERNEL);
	if (gfx_cmd_input_args_p == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "malloc gfx_cmd_input_args_p buffer failed\n");
		goto out;
	}
	memset(gfx_cmd_input_args_p, 0x0, sizeof(gfx_cmd_input_args_t));

	// output data
	memory_usage_info_p = (gfx_memory_usage_info_t *)kvmalloc(
		sizeof(gfx_memory_usage_info_t), GFP_KERNEL);
	if (memory_usage_info_p == NULL) {
		VATOOLS_ERR(priv, die_index,
			    "malloc memory_usage_info_p buffer failed\n");
		goto out_free_input_args;
	}
	memset(memory_usage_info_p, 0x0, sizeof(gfx_memory_usage_info_t));

	ret = copy_from_user_ex(gfx_cmd_input_args_p,
				(void __user *)pkdata->input_buf.buf_addr,
				pkdata->input_buf.buf_size);
	if (ret) {
		VATOOLS_ERR(priv, die_index,
			    "copy_from_user_ex ret=%ld gfx_cmd=0x%x\n", ret,
			    gfx_cmd);
		ret = -EFAULT;
		goto out_free_info;
	}
	VATOOLS_DBG(priv, die_index, "gfx_cmd=%d pf_num=%d pf_index[0]=%d\n",
		    gfx_cmd, gfx_cmd_input_args_p->pf_num,
		    gfx_cmd_input_args_p->pf_index[0]);

	ret = vatools_gfx_interface(priv, gfx_cmd, (u64)gfx_cmd_input_args_p,
				    sizeof(gfx_cmd_input_args_t),
				    (u64)memory_usage_info_p,
				    sizeof(gfx_memory_usage_info_t));
	if (ret) {
		VATOOLS_DBG(priv, die_index, "vatools_gfx_interface ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_info;
	}

	VATOOLS_DBG(priv, die_index, "gfx_cmd=%d pf_num=%d ret=%ld\n", gfx_cmd,
		    memory_usage_info_p->pf_num, ret);
	// DEBUG: for dump data
	for (i = 0; i < memory_usage_info_p->pf_num; i++) {
		VATOOLS_DBG(priv, die_index, "[%02d] gfx pf_index=%d\n", i,
			    memory_usage_info_p->gfx_memory_usage[i].pf_index);
		VATOOLS_DBG(
			priv, die_index, "        MemoryTotal : %d\n",
			memory_usage_info_p->gfx_memory_usage[i].MemoryTotal);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageKMalloc : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageKMalloc);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageKMallocMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageKMallocMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageVMalloc : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageVMalloc);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageVMallocMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageVMallocMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocPTu32_MemoryUMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocPTu32_MemoryUMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocPTu32_MemoryUMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocPTu32_MemoryUMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageVMapPTUMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageVMapPTUMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageVMapPTUMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageVMapPTUMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocPTu32_MemoryLMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocPTu32_MemoryLMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocPTu32_MemoryLMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocPTu32_MemoryLMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageIORemapPTLMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageIORemapPTLMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageIORemapPTLMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageIORemapPTLMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemLMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemLMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemLMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemLMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemUMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemUMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemUMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemUMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemUMAPool : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemUMAPool);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageAllocGPUMemUMAPoolMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageAllocGPUMemUMAPoolMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageMappedGPUMemUMA_LMA : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageMappedGPUMemUMA_LMA);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageMappedGPUMemUMA_LMAMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageMappedGPUMemUMA_LMAMax);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageDmaBufImport : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageDmaBufImport);
		VATOOLS_DBG(priv, die_index,
			    "        MemoryUsageDmaBufImportMax : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageDmaBufImportMax);
		VATOOLS_DBG(priv, die_index,
			    "      MemoryUsageDisplay_Heap_Size : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageDisplay_Heap_Size);
		VATOOLS_DBG(priv, die_index,
			    "       MemoryUsageDisplay_Heap_Use : %d\n",
			    memory_usage_info_p->gfx_memory_usage[i]
				    .MemoryUsageDisplay_Heap_Use);
	}

	ret = copy_to_user_ex((void __user *)pkdata->output_buf.buf_addr,
			      memory_usage_info_p,
			      sizeof(gfx_memory_usage_info_t));
	if (ret) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret = -EFAULT;
	}
	goto out_free_info;

out_free_info:
	kvfree(memory_usage_info_p);

out_free_input_args:
	kvfree(gfx_cmd_input_args_p);
out:
	return ret;
}

static long vatools_pcie_ioctl_gfx_cmd(T_SMI_IOCTL_TRANS_DATA *pkdata,
				       struct vastai_pci_info *priv,
				       int die_index)
{
	long ret = -EINVAL;
	u32 gfx_cmd = TOOL_CMD_GFX_NONE;

	gfx_cmd = pkdata->address;

	switch (gfx_cmd) {
	case TOOL_CMD_GFX_GET_DRIVER_STATUS: {
		break;
	}
	case TOOL_CMD_GFX_GET_GPU_UTILIZATION: {
		ret = vatools_pcie_ioctl_gfx_utilization(pkdata, priv,
							 die_index);
		break;
	}
	case TOOL_CMD_GFX_GET_MEMORY_USAGE: {
		ret = vatools_pcie_ioctl_gfx_memory_usage(pkdata, priv,
							  die_index);
		break;
	}
	default:
		ret = -EINVAL;
	}

	return ret;
}

// pcie 命令接口, 直接调用pcie驱动函数. 不发送到smcu
static long vatools_pcie_ioctl(struct file *filp, unsigned int cmd,
			       IOCTL_ARG_T arg)
{
	// struct vatools_node*   node   = vatools_file_get_node( filp );
	// struct vatools_reader* reader = vatools_file_get_reader( filp );

	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	int write_len = 0;
	int read_len = 0;
	T_SMI_IOCTL_TRANS_DATA kdata = { 0 };
	struct vastai_pci_info *priv = NULL;
	u32 die_index = 0;
	int ret2 = 0;
	int pci_info_status = VASTAI_NO_RUN_STATE;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(priv, die_index, "ioctl_cmd=0x%x\n", cmd);

	memset(&kdata, 0x00, sizeof(T_SMI_IOCTL_TRANS_DATA));
	ret = copy_from_user_ex(&kdata, (void __user *)argp, sizeof(kdata));
	if (ret) {
		VATOOLS_ERR(priv, die_index,
			    "copy_from_user_ex ret=%ld ioctl_cmd=0x%x\n", ret,
			    cmd);
		ret = -EFAULT;
		goto out;
	}

	VATOOLS_DBG(priv, die_index,
		    "block_id=%d  device_id=%d  die_id=%d  flag=0x%x  addr= "
		    "0x%llx\n",
		    kdata.block_id, kdata.device.dev_id, kdata.device.die_id,
		    kdata.flag, kdata.address);
	VATOOLS_DBG(
		priv, die_index,
		"from host: read_len(output)=%d  read_buf(output)=0x%llx  write_len(input)=%d"
		"write_buf(input)=0x%llx\n",
		kdata.output_buf.buf_size, kdata.output_buf.buf_addr,
		kdata.input_buf.buf_size, kdata.input_buf.buf_addr);

	write_len = kdata.input_buf.buf_size;
	read_len = kdata.output_buf.buf_size;
	// commands do not care about priv or die_index, emulate all devices/dies
	if (kdata.flag == PCIE_CMD_FLAG_RUNTIME_INFO ||
	    kdata.flag == PCIE_CMD_FLAG_CURRENT_PROCESS_INFO) {
		priv = NULL;
		die_index = kdata.device.die_index;
	} else {
		priv = vatools_get_vastai_pci_device_info(
			(char)kdata.device.dev_id);

		// device reboot reset boot do not check status
		switch (kdata.flag) {
		case PCIE_CMD_FLAG_RESET:
		case PCIE_CMD_FLAG_REBOOT:
		case PCIE_CMD_FLAG_BOOT: {
			if (priv == NULL) {
				VATOOLS_INFO(
					priv, die_index,
					"priv is NULL, dev_id= 0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			pci_info_status = atomic_read(&priv->pci_state);
			if ((pci_info_status == VASTAI_HOTP_STATE)) {
				VATOOLS_INFO(
					priv, die_index,
					"priv->pci_state= 0x%x ioctl_cmd=0x%x\n",
					pci_info_status, cmd);
				ret = -EACCES;
				goto out;
			}
			die_index = kdata.device.die_index;
		} break;
		// trigger dma test only run at VASTAI_NORMAL_STATE
		case PCIE_CMD_FLAG_REBOOT_DDRBW:
		case PCIE_CMD_FLAG_REBOOT_PCIEDMA: {
			if (priv == NULL) {
				VATOOLS_INFO(
					priv, die_index,
					"priv is NULL, dev_id= 0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			pci_info_status = atomic_read(&priv->pci_state);
			if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE,
						    -1)) {
				VATOOLS_DBG(
					priv, die_index,
					"priv->pci_state= 0x%x ioctl_cmd=0x%x\n",
					pci_info_status, cmd);
				ret = -EACCES;
				goto out;
			}
			die_index = kdata.device.die_index;
		} break;
		// read dma test result can run at VASTAI_DEBUG_STATE
		case PCIE_CMD_FLAG_BANDWIDTH:
		case PCIE_CMD_FLAG_DDR_BANDWIDTH: {
			if (priv == NULL) {
				VATOOLS_INFO(
					priv, die_index,
					"priv is NULL, dev_id= 0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			pci_info_status = atomic_read(&priv->pci_state);
			if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE,
						    VASTAI_DEBUG_STATE, -1)) {
				VATOOLS_DBG(
					priv, die_index,
					"priv->pci_state= 0x%x ioctl_cmd=0x%x\n",
					pci_info_status, cmd);
				ret = -EACCES;
				goto out;
			}
			die_index = kdata.device.die_index;
		} break;
		// other smi or pcie cmd(not DMA ) can run on VASTAI_DEBUG_STATE
		default: {
			if (priv == NULL) {
				VATOOLS_INFO(
					NULL, DUMMY_DIE_ID,
					"priv is NULL, dev_id= 0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			pci_info_status = atomic_read(&priv->pci_state);
			if (vatools_check_pci_state(priv, VASTAI_NORMAL_STATE,
						    VASTAI_DEBUG_STATE, -1)) {
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID,
					"priv->pci_state=0x%x ioctl_cmd=0x%x\n",
					pci_info_status, cmd);
				ret = -EACCES;
				goto out;
			}
			die_index = vatools_get_vastai_pci_die_index(
				(char)kdata.device.dev_id, kdata.device.die_id);
			if (die_index < 0) {
				ret = -ENODEV;
				goto out;
			}
		} break;
		}

#if 0
		priv = vatools_get_vastai_pci_device_info(
			(char)kdata.device.dev_id);

		// device reboot reset boot do not check status
		if (kdata.flag != PCIE_CMD_FLAG_RESET &&
		    kdata.flag != PCIE_CMD_FLAG_REBOOT &&
		    kdata.flag != PCIE_CMD_FLAG_BOOT &&
		    kdata.flag != PCIE_CMD_FLAG_REBOOT_DDRBW &&
		    kdata.flag != PCIE_CMD_FLAG_REBOOT_PCIEDMA) {
			if (priv == NULL) {
				VATOOLS_DBG(
					priv, die_index,
					"priv is NULL, dev_id=0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			if ((atomic_read(&priv->pci_state) !=
			     VASTAI_NORMAL_STATE)) {
				VATOOLS_DBG(
					priv, die_index,
					"priv->pci_state=0x%x ioctl_cmd=0x%x\n",
					atomic_read(&priv->pci_state), cmd);
				ret = -ENODEV;
				goto out;
			}
			die_index = vatools_get_vastai_pci_die_index(
				(char)kdata.device.dev_id, kdata.device.die_id);
			if (die_index < 0) {
				ret = -ENODEV;
				goto out;
			}
		} else {
			if (priv == NULL) {
				VATOOLS_DBG(
					priv, die_index,
					"priv is NULL, dev_id=0x%x ioctl_cmd=0x%x\n",
					kdata.device.dev_id, cmd);
				ret = -ENODEV;
				goto out;
			}
			if ((atomic_read(&priv->pci_state) ==
			     VASTAI_HOTP_STATE)) {
				VATOOLS_DBG(
					priv, die_index,
					"priv->pci_state=0x%x ioctl_cmd=0x%x\n",
					atomic_read(&priv->pci_state), cmd);
				ret = -ENODEV;
				goto out;
			}
			die_index = kdata.device.die_index;
		}
#endif
	}

	kdata.errcode = 0;

	switch (kdata.flag) {
	case PCIE_CMD_FLAG_BANDWIDTH: {
		ret = vatools_pcie_ioctl_bandwidth(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_RESET: {
		ret = vatools_pcie_ioctl_reset(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_BOOT: {
		ret = vatools_pcie_ioctl_boot(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_REBOOT: {
		ret = vatools_pcie_ioctl_reboot(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_FW_VERSION: {
		ret = vatools_pcie_ioctl_fw_version(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_DDR_ECC_1BIT: {
		ret = vatools_pcie_ioctl_ecc_1bit(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_DDR_ECC_2BIT: {
		ret = vatools_pcie_ioctl_ecc_2bit(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_VDSP_COUNT: {
		ret = vatools_pcie_ioctl_vdsp_count(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_AI_COUNT: {
		ret = vatools_pcie_ioctl_ai_count(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_RING_BUF_STATISTIC: {
		ret = vatools_pcie_ioctl_ring_buf_statistic(&kdata, priv,
							    die_index);
	} break;
	case PCIE_CMD_FLAG_RING_BUF_FROM_HW_CFG: {
		ret = vatools_pcie_ioctl_ring_buf_from_hw_cfg(&kdata, priv,
							      die_index);
	} break;
#ifdef CONFIG_VASTAI_VIDEO
	// 设置video使用multicore; 0: 不使用; 1: 使用
	case PCIE_CMD_FLAG_SET_VIDEO_MULTI_CORE: {
		ret = vatools_pcie_ioctl_set_video_multi_core(&kdata, priv,
							      die_index);
	} break;
	case PCIE_CMD_FLAG_GET_VIDEO_MULTI_CORE: {
		ret = vatools_pcie_ioctl_get_video_multi_core(&kdata, priv,
							      die_index);
	} break;
#endif
	// core heartbeat
	case PCIE_CMD_FLAG_HEART_BEAT: {
		ret = vatools_pcie_ioctl_heart_beat(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_BARRESET: {
		ret = vatools_pcie_ioctl_barreset(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_BARSTATUS: {
		ret = vatools_pcie_ioctl_barstatus(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_REBOOT_DDRBW: {
		ret = vatools_pcie_ioctl_reboot_ddrbw(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_REBOOT_PCIEDMA: {
		ret = vatools_pcie_ioctl_reboot_pciedma(&kdata, priv,
							die_index);
	} break;

	case PCIE_CMD_FLAG_RUNTIME_INFO: {
		ret = vatools_pcie_ioctl_runtime_info(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_CURRENT_PROCESS_INFO: {
		ret = vatools_pcie_ioctl_current_process_info(&kdata, priv,
							      die_index);
	} break;

	case PCIE_CMD_FLAG_HEART_BEAT_COUNT: {
		ret = vatools_pcie_ioctl_heartbeat_count(&kdata, priv,
							 die_index);
	} break;

	case PCIE_CMD_FLAG_DDR_BANDWIDTH: {
		ret = vatools_pcie_ioctl_ddr_bandwidth(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_CARD_CPU_TOPO: {
		ret = vatools_pcie_ioctl_card_cpu_topo(&kdata, priv, die_index);
	} break;

	case PCIE_CMD_FLAG_GFX_CMD: {
		ret = vatools_pcie_ioctl_gfx_cmd(&kdata, priv, die_index);
	} break;

	default:
		ret = E_UNKNOWN_SUB_CMD;
		VATOOLS_DBG(priv, die_index,
			    "Pcie cmd id %d don't support in current system\n",
			    kdata.flag);
		ret = -EINVAL;
		break;
	}

out:
	kdata.errcode = ret;
	if (priv) {
		pci_info_status = atomic_read(&priv->pci_state);
		kdata.block_id = pci_info_status;
	}
	ret2 = copy_to_user_ex((void __user *)argp, &kdata, sizeof(kdata));
	if (ret2) {
		VATOOLS_ERR(priv, die_index, "copy_to_user_ex ret=%ld\n", ret);
		ret2 = -EFAULT;
	}
	VATOOLS_DBG(priv, die_index, "out: ret=%ld   ret2=%d\n", ret, ret2);
	VATOOLS_FUNC_EXIT;

	return ret2;
}

/// 内部测试使用
static int vatools_ioctl_test(struct file *filp, unsigned int cmd,
			      IOCTL_ARG_T arg)
{
	long ret = -EIO;
	// struct vatools_node*   node   = NULL;
	// struct vatools_reader* reader = NULL;
	// void __user*           argp   = ( void __user* )arg;

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	V_UNREFERENCE(arg);
	VATOOLS_FUNC_ENTERY;

	// node   = vatools_file_get_node( filp );
	// reader = vatools_file_get_reader( filp );

	return ret;
}

/******************************************************************************
 * vatools
 *
 *
 */

static loff_t vatools_llseek(struct file *filp, loff_t offset, int orig)
{
	loff_t ret = 0;

	ret = app_fops[vatools_get_app_category(filp)].llseek(filp, offset,
							      orig);
	return ret;
}

static ssize_t vatools_read(struct file *filp, char __user *buf, size_t size,
			    loff_t *pos)
{
	ssize_t ret = 0;

	ret = app_fops[vatools_get_app_category(filp)].read(filp, buf, size,
							    pos);
	return ret;
}

static ssize_t vatools_write(struct file *filp, const char __user *buf,
			     size_t size, loff_t *pos)
{
	ssize_t ret = 0;

	ret = app_fops[vatools_get_app_category(filp)].write(filp, buf, size,
							     pos);
	return ret;
}

static ssize_t vatools_write_iter(struct kiocb *iocb, struct iov_iter *from)
{
	ssize_t ret = 0;

	ret = app_fops[vatools_get_app_category(iocb->ki_filp)].write_iter(
		iocb, from);
	return ret;
}

static unsigned int vatools_poll(struct file *filp,
				 struct poll_table_struct *wait)
{
	int ret = 0;

	ret = app_fops[vatools_get_app_category(filp)].poll(filp, wait);
	return ret;
}
static int vatools_mmap(struct file *filp, struct vm_area_struct *pvm)
{
	int ret = 0;
	ret = app_fops[vatools_get_app_category(filp)].mmap(filp, pvm);
	return ret;
}
static int vatools_fasync(int fd, struct file *filp, int mode)
{
	int ret = 0;
	ret = app_fops[vatools_get_app_category(filp)].fasync(fd, filp, mode);
	return ret;
}

static long vatools_ioctl(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg)
{
	long ret = -EIO;
	struct vatools_node *node = NULL;
	struct vatools_reader *reader = NULL;

	reader = vatools_file_get_reader(filp);
	if (!reader) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"ioctl cmd=0x%x node=0x%p filp=0x%p reader=0x%p is NULL\n",
			cmd, node, filp, reader);
		return ret;
	}
	node = vatools_file_get_node(filp);
	if (!node) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"ioctl cmd=0x%x node=0x%p is NULL filp=0x%p reader=0x%p\n",
			cmd, node, filp, reader);
		return ret;
	}

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"[vatools_ioctl] [tgid:%d pid:%d] ioctl cmd=0x%x node=0x%p filp=%p reader=0x%p app_category=%d\n",
		current->tgid, current->pid, cmd, node, filp, reader,
		reader->trans_category.app_category);

	// 根据命令指定 app_category, 无论用户是否设置了 app_category
	switch (cmd) {
	case VATOOLS_GENERAL_IOCTL_LOG_STATUS:
		reader->trans_category.app_category = VATOOLS_APP_LOGGER;
		break;
#ifdef CONFIG_TOOLS_V2
	case VATOOLS_IOCTL_V2_SMI_CMD:
		reader->trans_category.app_category = VATOOLS_APP_SMI;
		break;
	case VATOOLS_IOCTL_V2_GET_DEVICE_INFO:
		reader->trans_category.app_category = VATOOLS_APP_SMI;
		break;
	case VATOOLS_IOCTL_V2_PROFILER:
		reader->trans_category.app_category = VATOOLS_APP_PROFILER;
		break;
	case VATOOLS_IOCTL_V2_FLASH:
		reader->trans_category.app_category = VATOOLS_APP_SMI;
		break;
	case VATOOLS_IOCTL_V2_GUARD:
		reader->trans_category.app_category = VATOOLS_APP_SMI;
		break;
	case VATOOLS_IOCTL_V2_SHARE_MEM:
		reader->trans_category.app_category = VATOOLS_APP_SHAREDMEM;
		break;
	case VATOOLS_IOCTL_V2_VIDEO_DEBUGGER:
		reader->trans_category.app_category = VATOOLS_APP_DEBUGGER;
		break;
	case VATOOLS_IOCTL_V2_AI_DEBUGGER:
		reader->trans_category.app_category = VATOOLS_APP_DEBUGGER;
		break;
	case VATOOLS_IOCTL_V2_DEVICE_EVENT:
		reader->trans_category.app_category = VATOOLS_APP_EVENT;
		break;
	case VATOOLS_IOCTL_V2_CORE_TIME_SYNC:
		reader->trans_category.app_category = VATOOLS_APP_PROFILER;
		break;
#endif
	default:
		// reader->trans_category.app_category = VATOOLS_APP_SMI;
		break;
	}

#ifdef CONFIG_TOOLS_V2
	//==========================================================
	/// V2 版本 ioctl
	ret = -0x0effffff; // 不可能的返回值
	switch (cmd) {
		/// 测试命令
	case VATOOLS_IOCTL_V2_TEST: {
		// ret = vatools_ioctl_test(filp, cmd, arg);
	} break;
		/// 进程锁机制, 优先处理
	case VATOOLS_IOCTL_V2_GUARD: {
		// ret = vatools_ioctl_process_mutex_cmd(filp, cmd, arg);
	} break;
		/// 设置打开的文件fd的类别
	case VATOOLS_IOCTL_V2_SET_APP_CATEGORY: {
		// ret = vatools_ioctl_set_app_category(filp, cmd, arg);
	} break;
		/// 获取device info
	case VATOOLS_IOCTL_V2_GET_DEVICE_INFO: {
		/// 读取后缓存, 在设置命令后, 清空缓存, 重新读取
		ret = vatools_ioctl_v2_get_device_info(filp, cmd, arg);
	} break;
		/// 通过 smi 读写数据, 先写,触发中断到smcu, 后读, 读写通过一次调用完成
	case VATOOLS_IOCTL_V2_SMI_CMD: {
		// ret = vatools_ioctl_smi_cmd(filp, cmd, arg);
	} break;
		/// 普通方式读取数据; 只使用vastai_pcie 读取; 地址转换判断在host/common里面完成
	case VATOOLS_IOCTL_V2_READ_BUF: {
		// ret = vatools_ioctl_read_buf(filp, cmd, arg);
	} break;
		/// 普通方式写入数据; 只使用vastai_pcie 写入; 地址转换判断在host/common里面完成
	case VATOOLS_IOCTL_V2_WRITE_BUF: {
		// ret = vatools_ioctl_write_buf(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_PCIE_CMD: {
		// ret = vatools_ioctl_pcie_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_PROFILER: {
		// ret = vatools_ioctl_profiler_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_FLASH: {
		ret = vatools_ioctl_flash_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_SHARE_MEM: {
		// ret = vatools_ioctl_sharedmem_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_AI_DEBUGGER: {
		// ret = vatools_ioctl_ai_debugger_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_CORE_TIME_SYNC: {
		// ret = vatools_ioctl_core_time_sync_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_LOG_STATUS: {
		// ret = vatools_ioctl_get_log_status_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_DMA: {
		// ret = vatools_ioctl_dma_cmd(filp, cmd, arg);
	} break;
	case VATOOLS_IOCTL_V2_BASE_INFO: {
		ret = vatools_ioctl_get_base_info(filp, cmd, arg);
	} break;
	default: {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "[vatools_ioctl] unknown v2 ioctl cmd 0x%x\n", cmd);
		break;
	}
	}
	if (ret != -0x0effffff) {
		ret = E_UNKNOWN_IOCTL;
		reader->ioctl_version = VATOOLS_IOCTL_V2_MAGIC;
		// exit ioctl v2 here
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "[vatools_ioctl] v2 ioctl cmd 0x%x return %ld\n",
			    cmd, ret);
		return ret;
	}
	/// V2 版本 ioctl 结束V
	//==========================================================
#endif

	switch (cmd) {
	case VATOOLS_GENERAL_IOCTL_TEST:
		ret = vatools_ioctl_test(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_SET_APP_CATEGORY:
		ret = vatools_set_app_category(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_DEVICE_NUM:
		ret = vatools_get_device_num(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_DEV_INFO:
		ret = vatools_get_dev_info(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_DIE_INFO:
		ret = vatools_get_die_info(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_READ:
		ret = vatools_driver_read(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_WRITE:
		ret = vatools_driver_write(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_FETCH:
		ret = vatools_driver_smi_fetch(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_DEV_PCIE_BUS_ID:
		ret = vatools_get_dev_pcie_bus_id(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_PCIE_ID:
		ret = vatools_get_dev_pcie_id(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_PCIE_SUB_ID:
		ret = vatools_get_dev_pcie_sub_id(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_MAJOR_MINOR_ID:
		ret = vatools_get_dev_major_minor_id(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_GET_DIE_RENDER:
		ret = vatools_get_die_render(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_PCIE:
		ret = vatools_pcie_ioctl(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_DEVICE_CAPABILITY:
		ret = vatools_get_device_capability(filp, cmd, arg);
		break;

		//分配 DMA 内存
	case VATOOLS_GENERAL_IOCTL_DMA_ALLOC:
		ret = vatools_ioctl_dma_alloc(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_DMA_FREE:
		ret = vatools_ioctl_dma_free(filp, cmd, arg);
		break;
		//开始 DMA 传输数据
	case VATOOLS_GENERAL_IOCTL_DMA_START:
		ret = vatools_ioctl_dma_start(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_DMA_TRANS:
		ret = vatools_ioctl_dma_trans(filp, cmd, arg);
		break;

		//普通方式读取数据
	case VATOOLS_GENERAL_IOCTL_READ_BUF:
		ret = vatools_ioctl_read_reg_buf(filp, cmd, arg);
		break;

		//普通方式写入数据
	case VATOOLS_GENERAL_IOCTL_WRITE_BUF:
		ret = vatools_ioctl_write_reg_buf(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_GET_BMCU_XSPI_VERSION:
		ret = vatools_ioctl_get_bmcu_xspi_ver(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_SRIOV_NUMBER:
		ret = vatools_ioctl_get_sriov_number(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_FLASH_BMCU_FW:
		ret = vatools_ioctl_flash_bmcu_fw(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_FLASH_XSPI_FW:
		ret = vatools_ioctl_flash_xspi_fw(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_GET_FLASH_STA:
		ret = vatools_ioctl_flash_sta(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_SWITCH_BMCU:
		ret = vatools_ioctl_bmcu_switch(filp, cmd, arg);
		break;

		// core timestamp sync
	case VATOOLS_GENERAL_IOCTL_CORE_TIME_SYNC:
		ret = vatools_ioctl_core_time_sync(filp, cmd, arg);
		break;

	case VATOOLS_GENERAL_IOCTL_DEVICE_EVENT:
		ret = vatools_ioctl_device_event(filp, cmd, arg);
		break;
	case VATOOLS_GENERAL_IOCTL_DEVICE_EVENT_SELECT:
		ret = vatools_ioctl_device_event_select(filp, cmd, arg);
		break;

	default:
		// 只有在shared_memory 时 使用内部锁控制，其他的保持原流程暂不考虑并发
		// logger 使用自己的 ioctl和 mutex
		// if (reader->trans_category.app_category ==
		// 	    VATOOLS_APP_SHAREDMEM ||
		//     reader->trans_category.app_category == VATOOLS_APP_LOGGER ||
		//     VATOOLS_GENERAL_IOCTL_LOG_STATUS == cmd) {
		// }

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"[vatools_ioctl] ioctl_cmd=0x%X run app_fops.ioctl\n",
			cmd);

		ret = app_fops[vatools_get_app_category(filp)].unlocked_ioctl(
			filp, cmd, arg);
		// if (reader->trans_category.app_category ==
		// 	    VATOOLS_APP_SHAREDMEM ||
		//     reader->trans_category.app_category == VATOOLS_APP_LOGGER ||
		//     VATOOLS_GENERAL_IOCTL_LOG_STATUS == cmd) {
		// 	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "ioctl return %ld\n",
		// 		    ret);
		// 	return ret;
		// } else {
		break;
	}
	// out:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "[vatools_ioctl] ioctl return %ld\n",
		    ret);

	return ret;
}

#ifdef VATOOLS_HOT_PLUG_ENABLE
static int vatools_add_open_list(enum va_f_type type)
{
	struct pid *current_pid = NULL;
	struct vatools_pid_entry *pid_entry = NULL;
	struct vatools_pid_entry *pid_loop = NULL;

	current_pid = get_pid(task_tgid(current));
	pid_entry = kzalloc(sizeof(struct vatools_pid_entry), GFP_KERNEL);
	if (!pid_entry) {
		VATOOLS_INFO(
			NULL, DUMMY_DIE_ID,
			"[vatools_add_open_list] open list kzalloc error\n");
		return -ENOMEM;
	}
	pid_entry->pid = get_pid(task_tgid(current));
	pid_entry->refcount = 1;
	pid_entry->type = type;

	mutex_lock(&g_mutex_open_list);
	list_for_each_entry (pid_loop, &vatools_open_list, node) {
		if (pid_loop->pid == current_pid) {
			pid_loop->refcount++;
			kfree(pid_entry);
			mutex_unlock(&g_mutex_open_list);
			VATOOLS_INFO(
				NULL, DUMMY_DIE_ID,
				"[vatools_add_open_list] pid %d already there\n",
				pid_nr(pid_entry->pid));
			return 0;
		}
	}

	VATOOLS_INFO(NULL, DUMMY_DIE_ID,
		     "[vatools_add_open_list] pid %d new added\n",
		     pid_nr(pid_entry->pid));
	list_add(&pid_entry->node, &vatools_open_list);
	mutex_unlock(&g_mutex_open_list);

	return 0;
}
#else
static int vatools_add_open_list(enum va_f_type type)
{
	V_UNREFERENCE(type);
	return 0;
}
#endif

#ifdef VATOOLS_HOT_PLUG_ENABLE
static void vatools_del_open_list(enum va_f_type type)
{
	struct pid *current_pid = NULL;
	struct vatools_pid_entry *pid_loop = NULL;

	current_pid = get_pid(task_tgid(current));

	mutex_lock(&g_mutex_open_list);
	list_for_each_entry (pid_loop, &vatools_open_list, node) {
		if (pid_loop->pid == current_pid) {
			pid_loop->refcount--;
			if (pid_loop->refcount == 0) {
				list_del(&pid_loop->node);
			}
			VATOOLS_INFO(
				NULL, DUMMY_DIE_ID,
				"[vatools_del_open_list] pid %d refcount %d\n",
				pid_nr(pid_loop->pid), pid_loop->refcount);
			mutex_unlock(&g_mutex_open_list);
			if (pid_loop->refcount == 0) {
				VATOOLS_INFO(
					NULL, DUMMY_DIE_ID,
					"[vatools_del_open_list] pid %d del from list\n",
					pid_nr(pid_loop->pid));
				put_pid(pid_loop->pid);
				kfree(pid_loop);
			}
			return;
		}
	}

	mutex_unlock(&g_mutex_open_list);
}
#else
static void vatools_del_open_list(enum va_f_type type)
{
	V_UNREFERENCE(type);
}
#endif

static int vatools_open(struct inode *inode, struct file *filp)
{
	struct vatools_node *node = NULL;
	int ret = 0;
	struct vatools_reader *reader = NULL;

	V_UNREFERENCE(inode);
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"[vatools_open] [tgid:%d pid:%d] inode=%p filp=%p f_mode=0x%x\n",
		current->tgid, current->pid, inode, filp, filp->f_mode);

	ret = nonseekable_open(inode, filp);
	if (ret) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "[vatools_open] nonseekable_open ret=%d\n", ret);
		return ret;
	}

	node = vatools_get_node_from_minor(MINOR(inode->i_rdev));
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "[vatools_open] vatools_node *node=0x%p\n", node);

	if (!node) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "[vatools_open] node is NULL\n");
		return -ENODEV;
	}

	reader = (struct vatools_reader *)kvmalloc(
		sizeof(struct vatools_reader), GFP_KERNEL);
	if (!reader) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[vatools_open] kvmalloc error. size=%ld\n",
			    sizeof(struct vatools_reader));
		return -ENOMEM;
	}

	memset(reader, 0, sizeof(struct vatools_reader));

	reader->node = node;
	reader->r_ver = 0;
	reader->r_all = in_egroup_p(inode->i_gid) || capable(CAP_SYSLOG);
	reader->profiler_checkpoint_corebitmap = 0;
	reader->event_timestamp_ns = 0;

	// 设置默认值
	reader->trans_category.app_category = VATOOLS_APP_DUMMY;
	reader->trans_category.block_id = SMI_CMD_BLOCK_ID_0;
	reader->trans_category.device.dev_id = 0; // 0x7FFFFFFF;
	reader->trans_category.device.die_id = 0; // 0x7FFFFFFF;
	reader->trans_category.device.die_index = 0; // 0x7FFFFFFF;
	reader->event_timestamp_ns = 0;
#ifdef CONFIG_TOOLS_V2
	mutex_lock(&node->node_reader_mutex);
#else
	mutex_lock(&node->node_mutex);
#endif
	INIT_LIST_HEAD(&reader->list_readers);
	list_add_tail(&reader->list_readers, &node->list_node_readers);
	filp->private_data = reader;
#ifdef CONFIG_TOOLS_V2
	mutex_unlock(&node->node_reader_mutex);
#else
	mutex_unlock(&node->node_mutex);
#endif

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "[vatools_open] node=0x%p  reader=0x%p\n", node, reader);

	VATOOLS_FUNC_EXIT;
	return 0;
}

static int vatools_release(struct inode *ignored, struct file *filp)
{
	struct vatools_node *node = vatools_file_get_node(filp);
	struct vatools_reader *reader = vatools_file_get_reader(filp);

	V_UNREFERENCE(ignored);
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"[vatools_release] [tgid:%d pid:%d] node=0x%p filp=0x%p f_mode=0x%x\n",
		current->tgid, current->pid, node, filp, filp->f_mode);

#ifdef CONFIG_TOOLS_V2
	// if (reader->ioctl_version == VATOOLS_IOCTL_V2_MAGIC)
	if (node) {
		// TODO:  优先释放进程间锁  mutex
		// vatools_process_release_mutex_unlock(filp, 0x0, 0x0, NULL,
		// 				     NULL);

		vatools_add_open_list(VATOOLS_CLOSE);

		mutex_lock(&node->node_mutex);
		mutex_lock(&node->node_sharedmem_mutex);
		mutex_lock(&node->node_reader_mutex);
		app_fops[vatools_get_app_category(filp)].release(ignored, filp);
		reader = (struct vatools_reader *)filp->private_data;
		if (reader) {
			// pr_info( "vatools_release reader=%p\n", reader );
			list_del(&reader->list_readers);
			kvfree(reader);
			filp->private_data = NULL;
		}
		mutex_unlock(&node->node_reader_mutex);
		mutex_unlock(&node->node_sharedmem_mutex);
		mutex_unlock(&node->node_mutex);

		vatools_del_open_list(VATOOLS_CLOSE);
	}
#else
	if (node) {
		mutex_lock(&node->node_mutex);
		mutex_lock(&node->node_sharedmem_mutex);
		app_fops[vatools_get_app_category(filp)].release(ignored, filp);
		reader = (struct vatools_reader *)filp->private_data;
		if (reader) {
			// pr_info( "vatools_release reader=%p\n", reader );
			list_del(&reader->list_readers);
			kvfree(reader);
			filp->private_data = NULL;
		}
		mutex_unlock(&node->node_sharedmem_mutex);
		mutex_unlock(&node->node_mutex);
	}
#endif
	VATOOLS_FUNC_EXIT;
	return 0;

	// pr_info( "vatools_fasync reader " );
}

static const struct file_operations vatools_fops = {
	.owner = THIS_MODULE,
	.llseek = vatools_llseek,
	.read = vatools_read,
	.write = vatools_write,
	.write_iter = vatools_write_iter,
	.poll = vatools_poll,
	.unlocked_ioctl = vatools_ioctl,
	.compat_ioctl = vatools_ioctl,
	.mmap = vatools_mmap,
	.open = vatools_open,
	.release = vatools_release,
	.fasync = vatools_fasync,
};

/******************************************************************************
 *
 *
 *
 */
static int create_node(void *pci_dev, const char *node_name, int size)
{
	int ret = 0;
	struct vatools_node *node = NULL;
	unsigned char *buffer = NULL;
	struct vastai_fifo *fifo = NULL;
	char vatools_dev_name[VATOOLS_DEVICE_NAME_LEN] = { 0 };
	struct vastai_pci_info *priv = (struct vastai_pci_info *)pci_dev;
	VATOOLS_FUNC_ENTERY;

	buffer = (unsigned char *)kvmalloc(size, GFP_KERNEL);
	if (buffer == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[create_node] kvmalloc error. size=%d\n", size);
		return -ENOMEM;
	}

	node = (struct vatools_node *)kzalloc(sizeof(struct vatools_node),
					      GFP_KERNEL);
	if (node == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "[create_node] node is NULL\n");
		ret = -ENOMEM;
		goto out_free_buffer;
	}

	memset(vatools_dev_name, 0, sizeof(char) * VATOOLS_DEVICE_NAME_LEN);
	sprintf(vatools_dev_name, "%s", node_name);

	node->node_buffer = buffer;
	node->size = size;
	node->pci_dev = pci_dev;

	node->misc.minor = MISC_DYNAMIC_MINOR;
	node->misc.name = kstrdup(vatools_dev_name, GFP_KERNEL);
	if (node->misc.name == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[create_node] node->misc.name == NULL\n");
		ret = -ENOMEM;
		goto out_free_node;
	}

	node->misc.fops = &vatools_fops;
	node->misc.parent = NULL;

	node->misc.mode = 0666;
	node->dev_info = NULL;

	init_waitqueue_head(&node->wq);
	INIT_LIST_HEAD(&node->list_node_readers);
	mutex_init(&node->node_mutex);
	mutex_init(&node->node_sharedmem_mutex);
	mutex_init(&node->node_common_mutex);
	mutex_init(&node->node_profiler_mutex);
	mutex_init(&node->node_debugger_mutex);
	mutex_init(&node->node_logger_mutex);
	mutex_init(&node->node_smi_mutex);
#ifdef CONFIG_TOOLS_V2
	mutex_init(&node->node_dummy_mutex);
	mutex_init(&node->node_driver_mutex);
	mutex_init(&node->node_reader_mutex);
#endif
	mutex_init(&pcie_timeline_dma_mutex);
	mutex_init(&pcie_timeline_msg_mutex);

	// init timeline of pci dma
	node->profiler_dma_ringbuf =
		kvmalloc(PROFILER_PCI_DMA_FIFO_LENGTH, GFP_KERNEL);
	if (node->profiler_dma_ringbuf == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[create_node] profile dma ringbuf is NULL\n");
		ret = -ENOMEM;
		goto out_free_misc_name;
	}
	gp_profiler_dma_fifo = node->profiler_dma_ringbuf;
	fifo = (struct vastai_fifo *)node->profiler_dma_ringbuf;
	fifo->elem_count =
		(PROFILER_PCI_DMA_FIFO_LENGTH - sizeof(struct vastai_fifo)) /
		sizeof(profiler_dma_checkpoint_t);
	fifo->elem_size = sizeof(profiler_dma_checkpoint_t);
	//spin_lock_init(&node->profiler_dma_ringbuf_lock);

	// init timeline of pci msg
	node->profiler_msg_ringbuf =
		kvmalloc(PROFILER_PCI_MSG_FIFO_LENGTH, GFP_KERNEL);
	if (node->profiler_msg_ringbuf == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[create_node] profile msg ringbuf is NULL\n");
		ret = -ENOMEM;
		goto out_free_dma_fifo;
	}
	gp_profiler_msg_fifo = node->profiler_msg_ringbuf;
	fifo = (struct vastai_fifo *)node->profiler_msg_ringbuf;
	fifo->elem_count =
		(PROFILER_PCI_MSG_FIFO_LENGTH - sizeof(struct vastai_fifo)) /
		sizeof(profiler_msg_checkpoint_t);
	fifo->elem_size = sizeof(profiler_msg_checkpoint_t);
	//spin_lock_init(&node->profile_msg_ringbuf_lock);

	// init event ringbuf
	node->event_ringbuf = kvmalloc(EVENT_FIFO_LENGTH, GFP_KERNEL);
	if (node->event_ringbuf == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "[create_node] event_ringbuf is NULL\n");
		ret = -ENOMEM;
		goto out_free_msg_fifo;
	}
	memset(node->event_ringbuf, 0, EVENT_FIFO_LENGTH);
	fifo = (struct vastai_fifo *)node->event_ringbuf;
	fifo->elem_count = (EVENT_FIFO_LENGTH - sizeof(struct vastai_fifo)) /
			   sizeof(device_event_t);
	fifo->elem_size = sizeof(device_event_t);
	mutex_init(&node->event_mutex);
	init_waitqueue_head(&node->event_wait_queue);

	INIT_LIST_HEAD(&node->list_nodes);

	INIT_LIST_HEAD(
		&node->list_node_process_mutex); // TODO:  使用内核mutex实现进程互斥保护

	// INIT_LIST_HEAD( &node->t_sharedmem_mem_info_head );
	// INIT_LIST_HEAD( &node->t_sharedmem_signal_info_head );

	list_add_tail(&node->list_nodes, vatools_get_vastai_head());

	/* finally, initialize the misc device for this node */
	ret = misc_register(&node->misc);
	if (unlikely(ret)) {
		VATOOLS_ERR(
			NULL, DUMMY_DIE_ID,
			"[create_node] failed to register misc device for node '%s'!\n",
			node->misc.name);
		goto out_free_event_fifo;
	}

	VATOOLS_INFO(
		priv, DUMMY_DIE_ID,
		"[create_node] [SUCCESS] pci_dev=%p created %lu bytes on node '%s'\n",
		pci_dev, (unsigned long)node->size, node->misc.name);

	return 0;
out_free_event_fifo:
	kfree(node->event_ringbuf);

out_free_msg_fifo:
	kfree(node->profiler_msg_ringbuf);

out_free_dma_fifo:
	kfree(node->profiler_dma_ringbuf);

out_free_misc_name:
	kvfree((void *)node->misc.name);

out_free_node:
	kvfree(node);

out_free_buffer:
	kvfree(buffer);
	VATOOLS_FUNC_EXIT;
	VATOOLS_INFO(
		NULL, DUMMY_DIE_ID,
		"[create_node] [FAILED] pci_dev=%p created %lu bytes on node '%s' failed! ret=%d\n",
		pci_dev, (unsigned long)size, node_name, ret);
	return ret;
}

// FCT 测试使用, 表示设备probe 成功
int vatools_fct_probe_done(struct vastai_pci_info *priv, unsigned char devfn,
			   unsigned int card_id)
{
	// Important: add this below line only for FCT test. DO NOT edit below line.
	// If you want to change this, notice tools team. Thanks.
	VASTAI_PCI_INFO(
		priv, DUMMY_DIE_ID,
		"[FCT]************probe devfn=%d of card=%u done*********\n",
		devfn, card_id);
	return 0;
}

/******************************************************************************
 *
 *
 *
 */

static int vatools_driver_init(void *priv)
{
	int ret = 0;
	VATOOLS_FUNC_ENTERY;
	ret = create_node(priv, VATOOLS_DEV_NAME, VATOOLS_BUFFER_SIZE);
	if (unlikely(ret)) {
		VATOOLS_INFO(priv, DUMMY_DIE_ID,
			     "[driver_init] init %s failed. ret=%d\n",
			     VATOOLS_DEV_NAME, ret);
		goto out;
	}
	VATOOLS_INFO(priv, DUMMY_DIE_ID, "[driver_init] init %s ok\n",
		     VATOOLS_DEV_NAME);
out:
	VATOOLS_FUNC_EXIT;
	return ret;
}

static void vatools_driver_exit(void *pci_dev)
{
	struct vatools_node *current_node = NULL, *next_node = NULL;
	V_UNREFERENCE(pci_dev);
	VATOOLS_FUNC_ENTERY;

	list_for_each_entry_safe (current_node, next_node,
				  vatools_get_vastai_head(), list_nodes) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "[driver_exit] deinit %s\n",
			     current_node->misc.name);
		list_del(&current_node->list_nodes);
		/* we have to delete all the entry inside headlist */
		misc_deregister(&current_node->misc);
		kvfree(current_node->node_buffer);
		kvfree((void *)current_node->misc.name);

		// free profiler timeline buffer
		kvfree(current_node->profiler_dma_ringbuf);
		kvfree(current_node->profiler_msg_ringbuf);
		kvfree(current_node->event_ringbuf);
		gp_profiler_dma_fifo = NULL;
		gp_profiler_msg_fifo = NULL;
		current_node->event_ringbuf = NULL;

		// free node
		kvfree(current_node);
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "[driver_exit] deinit ok\n");
	}

	VATOOLS_FUNC_EXIT;
}

//*************************************************************************************
// 加载/卸载驱动时, 更新数据
static int vatools_update_data_probe(void *priv)
{
	int ret = 0;
	ret = vatools_v2_clear_device_info(priv);
	return ret;
}

static int vatools_update_data_remove(void *priv)
{
	int ret = 0;
	ret = vatools_v2_clear_device_info(priv);
	return ret;
}

// 开启编译时检查结构体长度， 和应用层对齐
#if (VATOOLS_CHECK_BYTE_ALIGH == 1)
static int vatools_check_align_compile(void)
{
	// 编译期检查 dev_info 结构size是否符合要求
	BUILD_BUG_ON_MSG(
		sizeof(vatools_v2_device_info_t) !=
			SIZEOF_V2_DEVICE_INFO_STRUCT,
		"*** VATOOLS complile error: vatools_v2_device_info_t size does NOT match");
	BUILD_BUG_ON(sizeof(vatools_v2_base_info_u) != VATOOLS_BASE_INFO_SIZE);
	BUILD_BUG_ON(sizeof(vatools_v2_device_info_u) !=
		     VATOOLS_DEVUCE_INFO_SIZE);
	BUILD_BUG_ON(sizeof(vatools_v2_die_info_u) != VATOOLS_DIE_INFO_SIZE);
	// 编译检查 mutex 是否 8 字节对齐
	BUILD_BUG_ON(offsetof(struct vatools_node, node_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_sharedmem_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_common_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_profiler_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_debugger_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_logger_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_smi_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_dummy_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_driver_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, node_reader_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_node, event_mutex) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_reader, debug_comp) % 8);
	BUILD_BUG_ON(offsetof(struct vatools_reader, sharedmem_mutex) % 8);

	return 0;
}
#endif

//*************************************************************************************
// 对于所有的device,vatools只加载一次. 使用此变量标识device计数 init 时
// 如果等于0 加载驱动; exit 时,如果等于0, 卸载驱动.
static atomic_t _vatools_init_device_count = ATOMIC_INIT(0);
// 每次卡变化, 计数加 1
atomic_t _vatools_changed_device_count = ATOMIC_INIT(0);

// pcie 驱动调用函数, 初始化vatools驱动. 只有一个节点,和device, die不相关.
int tools_init(void *priv)
{
	int ret = 0;
	mutex_lock(&g_mutex_tools_init);

	VATOOLS_INFO(priv, DUMMY_DIE_ID,
		     "[tools_init] priv=0x%p before init, device_count=%d\n",
		     priv, atomic_read(&_vatools_init_device_count));

	if (0 == atomic_read(&_vatools_init_device_count)) {
		ret = vatools_driver_init(priv);
	}
	atomic_inc(&_vatools_init_device_count);
	atomic_inc(&_vatools_changed_device_count);
	if (ret < 0) {
		VATOOLS_INFO(
			priv, DUMMY_DIE_ID,
			"[tools_init] init failed, total device_count=%d\n",
			atomic_read(&_vatools_init_device_count));
	} else {
		VATOOLS_INFO(priv, DUMMY_DIE_ID,
			     "[tools_init] init ok, total device_count=%d\n",
			     atomic_read(&_vatools_init_device_count));
	}

	ret = vatools_update_data_probe(priv);
	mutex_unlock(&g_mutex_tools_init);
	return ret;
}

void tools_exit(void *priv)
{
	mutex_lock(&g_mutex_tools_init);

	atomic_dec(&_vatools_init_device_count);
	atomic_inc(&_vatools_changed_device_count);

	VATOOLS_INFO(priv, DUMMY_DIE_ID,
		     "[tools_exit] priv=0x%p, device_count=%d\n", priv,
		     atomic_read(&_vatools_init_device_count));

	vatools_update_data_remove(priv);

	if (0 == atomic_read(&_vatools_init_device_count)) {
		vatools_driver_exit(priv);
	}

// 开启编译时检查结构体长度， 和应用层对齐
#if (VATOOLS_CHECK_BYTE_ALIGH == 1)
	vatools_check_align_compile();
#endif
	VATOOLS_INFO(priv, DUMMY_DIE_ID,
		     "[tools_exit] done. priv=0x%p, device_count=%d\n", priv,
		     atomic_read(&_vatools_init_device_count));
	mutex_unlock(&g_mutex_tools_init);
	return;
}

#if VATOOLS_INDEPENDANCE_DRIVER

static int __init vatools_init(void)
{
	return tools_init(NULL);
}
static void __exit vatools_exit(void)
{
	tools_exit(NULL);
}

module_init(vatools_init);
module_exit(vatools_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Frank Bai<frank.bai@vastaitech.com>");
MODULE_DESCRIPTION("vatools driver");
MODULE_VERSION("0.1");

#endif
