﻿#include "vatools.h"
#include "vatools_log.h"

/******************************************************************************
 * dummy fops
 *
 *
 */
loff_t dummy_llseek(struct file *filp, loff_t offset, int orig)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(offset);
	V_UNREFERENCE(orig);

	return 0;
}

ssize_t dummy_read(struct file *filp, char __user *buf, size_t size,
		   loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t dummy_write(struct file *filp, const char __user *buf, size_t size,
		    loff_t *pos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buf);
	V_UNREFERENCE(size);
	V_UNREFERENCE(pos);

	return 0;
}

ssize_t dummy_write_iter(struct kiocb *iocb, struct iov_iter *from)
{
	V_UNREFERENCE(iocb);
	V_UNREFERENCE(from);

	return 0;
}

unsigned int dummy_poll(struct file *filp, struct poll_table_struct *wait)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(wait);

	return 0;
}

long dummy_ioctl(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg)
{
	long ret = -EIO;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "filp=0x%p\n", filp);

	V_UNREFERENCE(filp);
	V_UNREFERENCE(cmd);
	V_UNREFERENCE(arg);
	return ret;
}

int dummy_mmap(struct file *filp, struct vm_area_struct *pvm)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(pvm);

	return 0;
}
int dummy_open(struct inode *inode, struct file *filp)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "filp=0x%p\n", filp);

	V_UNREFERENCE(inode);
	V_UNREFERENCE(filp);

	return 0;
}

int dummy_release(struct inode *ignored, struct file *filp)
{
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "filp=0x%p\n", filp);

	V_UNREFERENCE(ignored);
	V_UNREFERENCE(filp);

	return 0;
}

int dummy_fasync(int fd, struct file *filp, int mode)
{
	V_UNREFERENCE(fd);
	V_UNREFERENCE(filp);
	V_UNREFERENCE(mode);

	return 0;
}

#ifdef CONFIG_TOOLS_V2
int vatools_v2_clear_device_info(void *priv)
{
#if (TOOLS_DEBUG_NO_DEV_INFO_CACHE == 0)
	struct vastai_pci_info *pci_info = (struct vastai_pci_info *)priv;
	struct vatools_node *node_p = vatools_get_node();
	u32 pci_id = 0;

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "clear device: vendor: 0x%x  device: 0x%x  info cache\n",
		    pci_info->dev->vendor, pci_info->dev->device);
	pci_id = pci_info->dev->vendor | ((pci_info->dev->device) << 16);
	switch (pci_id) {
	case DEVICE_CLASS_CURRENT:
	case DEVICE_CLASS_SV100:
	case DEVICE_CLASS_SV100_VF:
	case DEVICE_CLASS_SG100_SERVER_PF:
	case DEVICE_CLASS_SG100_SERVER_PF_HARVEST1:
	case DEVICE_CLASS_SG100_SERVER_PF_HARVEST2:
	case DEVICE_CLASS_SG100_SERVER_VF:
	case DEVICE_CLASS_SG100_PF0_GFX:
	case DEVICE_CLASS_SG100_PF1_AUDIO:
		domain_mutex_lock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);
		if (node_p->dev_info != NULL) {
			vfree(node_p->dev_info);
			node_p->dev_info = NULL;
			node_p->device_num = 0;
		}
		domain_mutex_unlock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);
		break;
	default:
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "unknown device: 0x%x\n",
			    pci_id);
		break;
	}
#endif
	return 0;
}

long vatools_ioctl_v2_get_device_info(struct file *filp, unsigned int cmd,
				      IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	struct vatools_node *node = vatools_get_node();
	ioctl_arg_t *ioctl_arg_p;
	vatools_v2_device_info_u *dev_info_p = NULL;
#ifdef CONFIG_VASTAI_AI
	struct vastai_cdev *vacc_dev_p = NULL;
	struct vastai_cdev *vacc_dev_temp = NULL;
#endif
#ifdef CONFIG_VASTAI_VIDEO
	struct vastai_render_file *render_file_p = NULL;
	struct vastai_render_file *render_file_temp = NULL;
#endif
#ifdef CONFIG_DRM_DRIVER
	struct vastai_device_handle *device_handle;
#endif
	struct vastai_pci_info *pci_info = NULL;
	u32 device_num = 0;
	u32 die = 0;
	void *buffer = NULL;
	int buffer_size = 0;
	char *spi_buf = NULL; ///< 卡类型
	vatools_devices_info_t devices_info;
	u32 die_index_seq_num = 0;
	struct va_gfx_render_info gfx_render_info_origin;
	struct va_gfx_display_info gfx_display_info_origin;
	u32 real_device_num = 0; // 实际获取到卡info的卡数量

	VATOOLS_FUNC_ENTERY;

	V_UNREFERENCE(cmd);

	domain_mutex_lock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);
	if (node == NULL) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "vatools node is null\n");
		ret = -EINVAL;
		goto out;
	}

	ioctl_arg_p = (ioctl_arg_t *)kvmalloc(sizeof(ioctl_arg_t), GFP_KERNEL);
	if (ioctl_arg_p == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%ld\n",
			    sizeof(ioctl_arg_t));
		ret = -ENOMEM;
		goto out;
	}

	ret = copy_from_user_ex(ioctl_arg_p, (void __user *)argp,
				sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	memset(&devices_info, 0, sizeof(vatools_devices_info_t));
	ret = copy_from_user_ex(&devices_info,
				(void __user *)ioctl_arg_p->output_buf.buf_addr,
				sizeof(vatools_devices_info_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	// 获取 device 数量
	device_num = vatools_get_vastai_pci_device_number();
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "device_num=%d\n", device_num);

	// 拷贝版本号
	devices_info.base.info.magic_number = VATOOLS_MAGIC_CODE;
	devices_info.base.info.tools_ver_major = VATOOLS_DRV_VERSION_MAJOR_CODE;
	devices_info.base.info.tools_ver_middle =
		VATOOLS_DRV_VERSION_MIDDLE_CODE;
	devices_info.base.info.tools_ver_minor = VATOOLS_DRV_VERSION_MINOR_CODE;
	devices_info.base.info.device_number = device_num;
	devices_info.base.info.device_info_size =
		sizeof(vatools_v2_device_info_t);

	ret = copy_to_user_ex((void __user *)ioctl_arg_p->output_buf.buf_addr,
			      &devices_info, sizeof(vatools_devices_info_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	if ((VATOOLS_IOCTL_DEVICE_INFO_REFRESH == ioctl_arg_p->sub_cmd) ||
	    TOOLS_DEBUG_NO_DEV_INFO_CACHE) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "node->dev_info=%p\n",
			    node->dev_info);
		// 清除缓存, 重新读取数据
		// mutex_lock(&node->node_common_mutex);
		if (node->dev_info) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"sub_cmd=%d debug=%d: remove cache node=0x%p node->dev_info=%p\n",
				ioctl_arg_p->sub_cmd,
				TOOLS_DEBUG_NO_DEV_INFO_CACHE, node,
				node->dev_info);
			vfree(node->dev_info);
			node->dev_info = NULL;
			node->device_num = 0;
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "sub_cmd=%d debug=%d: remove cache done.\n",
				    ioctl_arg_p->sub_cmd,
				    TOOLS_DEBUG_NO_DEV_INFO_CACHE);
		}
		// mutex_unlock(&node->node_common_mutex);
	}

	if (node->dev_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "copy from device info cache. node->dev_info=%p\n",
			    node->dev_info);

		// mutex_lock(&node->node_common_mutex);

		buffer_size =
			node->device_num * sizeof(vatools_v2_device_info_u);
		ioctl_arg_p->errcode = 0;
		ioctl_arg_p->output_buf.buf_size = buffer_size;
		ioctl_arg_p->count = node->device_num;

		if (copy_to_user_ex((void __user *)argp, ioctl_arg_p,
				    sizeof(ioctl_arg_t))) {
			ret = -EFAULT;
		}
		if (copy_to_user_ex((void __user *)devices_info.device_info_p,
				    node->dev_info, buffer_size)) {
			ret = -EFAULT;
		}
		// mutex_unlock(&node->node_common_mutex);
		ret = 0;
		goto out_free_ioctl_arg;
	}

	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "get device info from SoC. node->dev_info=%p\n",
		    node->dev_info);

	if (device_num == 0 || device_num > MAX_DEVICE_NUM_PER_MANAGER) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "get device_num failed. device_num=%d\n",
			    device_num);
		ret = -ENODEV;
		ioctl_arg_p->count = device_num;
		ioctl_arg_p->errcode = ret;
		if (copy_to_user_ex((void __user *)argp, ioctl_arg_p,
				    sizeof(ioctl_arg_t))) {
			ret = -EFAULT;
		}
		goto out_free_ioctl_arg;
	}

	buffer_size = device_num * sizeof(vatools_v2_device_info_u);
	buffer = vmalloc(buffer_size);
	if (buffer == NULL) {
		ret = -ENOMEM;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "vmalloc error. size=%d ret=%ld\n", buffer_size,
			    ret);
		goto out_free_ioctl_arg;
	}

	spi_buf = vmalloc(SPI_BUF_LEN_MAX);
	if (spi_buf == NULL) {
		// ret = ENOMEM;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "vmalloc spi_buf error. size=%d ret=%ld\n",
			    SPI_BUF_LEN_MAX, ret);
		// do not exit, read other device info but spi
		// goto out_free_ioctl_arg;
	}

	dev_info_p = (vatools_v2_device_info_u *)buffer;
	memset(buffer, 0, buffer_size);

	// TODO: 支持热插拔, 板卡插拔后, device 序号发生变化, 不再从0开始连续
	list_for_each_entry (pci_info, &vastai_pci_info_list, dev_list) {
		if (pci_info == NULL) {
			VATOOLS_DBG(pci_info, DUMMY_DIE_ID, "priv is NULL\n");
			continue;
		}
		if (vatools_check_pci_state(pci_info, VASTAI_NORMAL_STATE,
					    VASTAI_NO_RUN_STATE,
					    VASTAI_DEBUG_STATE, -1)) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"pci_info->pci_state=0x%x  skip emulate device\n",
				atomic_read(&pci_info->pci_state));
#if 0
			//TODO: 卡状态不是可以正常使用的状态， 只获取状态， 不获取其他信息
			dev_info_p->info.dev_id = pci_info->dev_id;
			dev_info_p->info.pci_state = pci_info->pci_state;

			// 循环下一个device
			dev_info_p++;
			real_device_num++;
#endif
			continue;
		}

		dev_info_p->info.board_type = vastai_get_board_type(pci_info);
		dev_info_p->info.dev_id = pci_info->dev_id;
		dev_info_p->info.die_num = pci_info->die_num;
		VATOOLS_DBG(pci_info, DUMMY_DIE_ID,
			    "dev_id=%d die_num=%d buffer_size=%d\n",
			    dev_info_p->info.dev_id, dev_info_p->info.die_num,
			    buffer_size);

		// bar size
		memcpy(dev_info_p->info.bar_address, vatools_die_check,
		       sizeof(vatools_die_check));

		dev_info_p->info.pciid.vender_id = pci_info->dev->vendor;
		dev_info_p->info.pciid.device_id = pci_info->dev->device;
		dev_info_p->info.pcisubid.sub_vender_id =
			pci_info->dev->subsystem_vendor;
		dev_info_p->info.pcisubid.sub_device_id =
			pci_info->dev->subsystem_device;

		memcpy(dev_info_p->info.pcie_slot, pci_name(pci_info->dev),
		       VASTAI_BUF_SIZE_32);

		dev_info_p->info.kchar.majormin.major =
			MAJOR(pci_info->p_char->major);
		dev_info_p->info.kchar.majormin.minor =
			MINOR(pci_info->p_char->major);

		snprintf(dev_info_p->info.kchar.name, VASTAI_BUF_SIZE_32,
			 "%s%d", VA_NODE_NAME_KCHAR, pci_info->dev_id);

		// memcpy(dev_info_p->info.kchar.name,
		//        pci_info->p_char->file_info.file_name,
		//        VASTAI_BUF_SIZE_32);

		if (pci_info->version_file != NULL) {
			dev_info_p->info.vastai_version.majormin.major = MAJOR(
				pci_info->version_file->file_info.dev_num);
			dev_info_p->info.vastai_version.majormin.minor = MINOR(
				pci_info->version_file->file_info.dev_num);
			memcpy(dev_info_p->info.vastai_version.name,
			       pci_info->version_file->file_info.file_name,
			       VASTAI_BUF_SIZE_32);
		}
		if (pci_info->ctl_file != NULL) {
			dev_info_p->info.vastai_ctrl.majormin.major =
				MAJOR(pci_info->ctl_file->file_info.dev_num);
			dev_info_p->info.vastai_ctrl.majormin.minor = MINOR_L(
				MINOR(pci_info->ctl_file->file_info.dev_num));
			dev_info_p->info.vastai_ctrl.majormin.minor_h = MINOR_H(
				MINOR(pci_info->ctl_file->file_info.dev_num));

			memcpy(dev_info_p->info.vastai_ctrl.name,
			       pci_info->ctl_file->file_info.file_name,
			       VASTAI_BUF_SIZE_32);
		}

		dev_info_p->info.vatools.majormin.major = 10;
		dev_info_p->info.vatools.majormin.minor =
			MINOR_L((u16)node->misc.minor);
		dev_info_p->info.vatools.majormin.minor_h =
			MINOR_H((u16)node->misc.minor);

		memcpy(dev_info_p->info.vatools.name, node->misc.name,
		       VASTAI_BUF_SIZE_32);

		dev_info_p->info.cap.ai_cap =
			dev_info_p->info.cap.ai_cap; ///< AI VIDEO 能力集
		dev_info_p->info.cap.video_cap =
			dev_info_p->info.cap.video_cap; ///< AI VIDEO 能力集

		// 虚拟化
		dev_info_p->info.dev_vfpf_flag.is_physfn = pci_info->is_pf;
		dev_info_p->info.dev_vfpf_flag.is_virtfn = pci_info->is_vf;

		// 设置板卡或驱动能力集
		// 支持flash 无感升级
		if (vatools_flash_support_update_background(
			    pci_info, VATOOLS_FLASH_UPDATE_BMCU) == 1) {
			dev_info_p->info.capability0 |= 0x1;
		}
		if (vatools_flash_support_update_background(
			    pci_info, VATOOLS_FLASH_UPDATE_BL0) == 1) {
			dev_info_p->info.capability0 |= 0x2;
		}
		if (vatools_flash_support_update_background(
			    pci_info, VATOOLS_FLASH_UPDATE_PMCU) == 1) {
			dev_info_p->info.capability0 |= 0x4;
		}

		// get fw version of die
		memcpy(&(dev_info_p->info.fw_ver.sg100_fw_version),
		       &(pci_info->fw_ver), sizeof(pci_info->fw_ver));

		// get sys_cfg
		// BUILD_BUG_ON(sizeof(vatools_sys_cfg_u) <
		// 	     sizeof(struct system_cfg));

		// memcpy(&(dev_info_p->info.sys_cfg.sys_cfg),
		//        &(pci_info->priv_hw_cfg->sys_cfg),
		//        sizeof(struct system_cfg));

		dev_info_p->info.sys_cfg.sys_cfg.devfn =
			pci_info->priv_hw_cfg->sys_cfg.devfn;
		dev_info_p->info.sys_cfg.sys_cfg.pfn =
			pci_info->priv_hw_cfg->sys_cfg.pfn;
		dev_info_p->info.sys_cfg.sys_cfg.vfn =
			pci_info->priv_hw_cfg->sys_cfg.vfn;
		dev_info_p->info.sys_cfg.sys_cfg.vf_active =
			pci_info->priv_hw_cfg->sys_cfg.vf_active;
		dev_info_p->info.sys_cfg.sys_cfg.stream_id =
			pci_info->priv_hw_cfg->sys_cfg.stream_id;
		dev_info_p->info.sys_cfg.sys_cfg.gfx_mpu_en =
			pci_info->priv_hw_cfg->sys_cfg.gfx_mpu_en;
		dev_info_p->info.sys_cfg.sys_cfg.dp_en =
			pci_info->priv_hw_cfg->sys_cfg.dp_en;
		dev_info_p->info.sys_cfg.sys_cfg.gfx_mode =
			pci_info->priv_hw_cfg->sys_cfg.gfx_mode;
		dev_info_p->info.sys_cfg.sys_cfg.pkg_id =
			pci_info->priv_hw_cfg->sys_cfg.pkg_id;
		dev_info_p->info.sys_cfg.sys_cfg.atu_or_gart =
			pci_info->priv_hw_cfg->sys_cfg.atu_or_gart;
		dev_info_p->info.sys_cfg.sys_cfg.fn_mode =
			pci_info->priv_hw_cfg->sys_cfg.fn_mode;
		dev_info_p->info.sys_cfg.sys_cfg.is_super_pf =
			pci_info->priv_hw_cfg->sys_cfg.is_super_pf;
		dev_info_p->info.sys_cfg.sys_cfg.is_admin_pf =
			pci_info->priv_hw_cfg->sys_cfg.is_admin_pf;
		dev_info_p->info.sys_cfg.sys_cfg.bar4_5_cfg_size =
			pci_info->priv_hw_cfg->sys_cfg.bar4_5_cfg_size;
		dev_info_p->info.sys_cfg.sys_cfg.card_type =
			pci_info->priv_hw_cfg->sys_cfg.card_type;
		dev_info_p->info.sys_cfg.sys_cfg.sum_ddr_sz =
			pci_info->priv_hw_cfg->sys_cfg.sum_ddr_sz;
		dev_info_p->info.sys_cfg.sys_cfg.smmu_sync_01 =
			pci_info->priv_hw_cfg->sys_cfg.smmu_sync_01;
		dev_info_p->info.sys_cfg.sys_cfg.card_id =
			pci_info->priv_hw_cfg->sys_cfg.card_id;
		dev_info_p->info.sys_cfg.sys_cfg.total_pf_per_card =
			pci_info->priv_hw_cfg->sys_cfg.total_pf_per_card;
		dev_info_p->info.sys_cfg.sys_cfg.total_vf_per_pf =
			pci_info->priv_hw_cfg->sys_cfg.total_vf_per_pf;
		dev_info_p->info.sys_cfg.sys_cfg.decoder_sriov_en =
			pci_info->priv_hw_cfg->sys_cfg.decoder_sriov_en;
		dev_info_p->info.sys_cfg.sys_cfg.cur_pf_vf_num =
			pci_info->priv_hw_cfg->sys_cfg.cur_pf_vf_num;
		dev_info_p->info.sys_cfg.sys_cfg.gart_size =
			pci_info->priv_hw_cfg->sys_cfg.gart_size;
		dev_info_p->info.sys_cfg.sys_cfg.reserved32_03 =
			pci_info->priv_hw_cfg->sys_cfg.reserved32_03;

		// pci_bus
		dev_info_p->info.pci_bus.pci_bus.domain_id =
			pci_domain_nr(pci_info->dev->bus);
		// dev_info_p->info.pci_bus.pci_bus.card_id =
		// 	(dev_info_p->info.pci_bus.pci_bus.domain_id << 8) |
		// 	pci_info->dev->bus->number;
		dev_info_p->info.pci_bus.pci_bus.card_id = pci_info->card_id;
		dev_info_p->info.pci_bus.pci_bus.number =
			pci_info->dev->bus->number;
		dev_info_p->info.pci_bus.pci_bus.primary =
			pci_info->dev->bus->primary;
		dev_info_p->info.pci_bus.pci_bus.max_bus_speed =
			pci_info->dev->bus->max_bus_speed;
		dev_info_p->info.pci_bus.pci_bus.cur_bus_speed =
			pci_info->dev->bus->cur_bus_speed;
		memcpy(dev_info_p->info.pci_bus.pci_bus.name,
		       pci_info->dev->bus->name, 48);
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"dev_info_p->info.pci_bus.pci_bus.number=%d pci_info->dev->bus->number=%d domain_id=0x%x card_id=0x%x\n",
			dev_info_p->info.pci_bus.pci_bus.number,
			pci_info->dev->bus->number,
			dev_info_p->info.pci_bus.pci_bus.domain_id,
			dev_info_p->info.pci_bus.pci_bus.card_id);

		// vvv gfx node
		memset(&gfx_render_info_origin, 0x0,
		       sizeof(struct va_gfx_render_info));

		if (0 ==
		    vastai_get_render_info(pci_info, &gfx_render_info_origin)) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"gfx_render_info_origin.drm_major=%d gfx_primary_id=%d gfx_render_id=%d\n",
				gfx_render_info_origin.drm_major,
				gfx_render_info_origin.gfx_primary_id,
				gfx_render_info_origin.gfx_render_id);

			snprintf(dev_info_p->info.gfx_primary_node.name,
				 VASTAI_BUF_SIZE_32, "%s%d", VA_NODE_NAME_CARD,
				 gfx_render_info_origin.gfx_primary_id);
			dev_info_p->info.gfx_primary_node.majormin.major =
				(u16)(gfx_render_info_origin.drm_major &
				      0xffff);
			dev_info_p->info.gfx_primary_node.majormin.minor =
				(u16)(gfx_render_info_origin.gfx_primary_id &
				      0xffff);

			snprintf(dev_info_p->info.gfx_render_node.name,
				 VASTAI_BUF_SIZE_32, "%s%d",
				 VA_NODE_NAME_RENDER,
				 gfx_render_info_origin.gfx_render_id);
			dev_info_p->info.gfx_render_node.majormin.major = (u16)(
				gfx_render_info_origin.drm_major & 0xffff);
			dev_info_p->info.gfx_render_node.majormin.minor = (u16)(
				gfx_render_info_origin.gfx_render_id & 0xffff);
		}

		memset(&gfx_display_info_origin, 0x0,
		       sizeof(struct va_gfx_display_info));

		if (0 == vastai_get_display_info(pci_info,
						 &gfx_display_info_origin)) {
			snprintf(
				dev_info_p->info.gfx_nulldisp_primary_node.name,
				VASTAI_BUF_SIZE_32, "%s%d", VA_NODE_NAME_CARD,
				gfx_display_info_origin.nulldisp_primary_id);
			dev_info_p->info.gfx_nulldisp_primary_node.majormin
				.major = (u16)(
				gfx_display_info_origin.drm_major & 0xffff);
			dev_info_p->info.gfx_nulldisp_primary_node.majormin
				.minor = (u16)(
				gfx_display_info_origin.nulldisp_primary_id &
				0xffff);
		}

		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"gfx_primary_node=%s major=%d minor=%d gfx_render_node=%s major=%d minor=%d "
			"gfx_nulldisp_primary_node=%s major=%d minor=%d\n",
			dev_info_p->info.gfx_primary_node.name,
			dev_info_p->info.gfx_primary_node.majormin.major,
			dev_info_p->info.gfx_primary_node.majormin.minor,
			dev_info_p->info.gfx_render_node.name,
			dev_info_p->info.gfx_render_node.majormin.major,
			dev_info_p->info.gfx_render_node.majormin.minor,
			dev_info_p->info.gfx_nulldisp_primary_node.name,
			dev_info_p->info.gfx_nulldisp_primary_node.majormin
				.major,
			dev_info_p->info.gfx_nulldisp_primary_node.majormin
				.minor);
		//^^^ gfx node

		// ddr info
		memset(&(dev_info_p->info.ddr_info), 0,
		       sizeof(vatools_ddr_info_t));
		dev_info_p->info.ddr_info.card_ddr_size =
			pci_info->ddr_info.card_ddr_size;
		dev_info_p->info.ddr_info.fn_ddr_size =
			pci_info->ddr_info.fn_ddr_size;
		// 因为排列顺序变化, 为了兼容 app, 单独拷贝
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_VF_MANAGE].noc_addr =
			pci_info->ddr_info.ddr_entry[VF_MANAGE].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_VF_MANAGE].fn_addr =
			pci_info->ddr_info.ddr_entry[VF_MANAGE].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_VF_MANAGE].cpu_pa =
			pci_info->ddr_info.ddr_entry[VF_MANAGE].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_VF_MANAGE]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[VF_MANAGE].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_VF_MANAGE].size =
			pci_info->ddr_info.ddr_entry[VF_MANAGE].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_META_FW].noc_addr =
			pci_info->ddr_info.ddr_entry[META_FW].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_META_FW].fn_addr =
			pci_info->ddr_info.ddr_entry[META_FW].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_META_FW].cpu_pa =
			pci_info->ddr_info.ddr_entry[META_FW].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_META_FW].bar_offset =
			pci_info->ddr_info.ddr_entry[META_FW].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_META_FW].size =
			pci_info->ddr_info.ddr_entry[META_FW].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_ENC_DEC_CMD_BUF]
			.noc_addr =
			pci_info->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_ENC_DEC_CMD_BUF]
			.fn_addr =
			pci_info->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_ENC_DEC_CMD_BUF]
			.cpu_pa =
			pci_info->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_ENC_DEC_CMD_BUF]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_ENC_DEC_CMD_BUF]
			.size =
			pci_info->ddr_info.ddr_entry[ENC_DEC_CMD_BUF].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_AI_MODULE_OPCODE]
			.noc_addr =
			pci_info->ddr_info.ddr_entry[AI_MODULE_OPCODE].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_AI_MODULE_OPCODE]
			.fn_addr =
			pci_info->ddr_info.ddr_entry[AI_MODULE_OPCODE].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_AI_MODULE_OPCODE]
			.cpu_pa =
			pci_info->ddr_info.ddr_entry[AI_MODULE_OPCODE].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_AI_MODULE_OPCODE]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[AI_MODULE_OPCODE]
				.bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_AI_MODULE_OPCODE]
			.size =
			pci_info->ddr_info.ddr_entry[AI_MODULE_OPCODE].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_SMMU_CD_TABLE]
			.noc_addr =
			pci_info->ddr_info.ddr_entry[SMMU_CD_TABLE].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_SMMU_CD_TABLE]
			.fn_addr =
			pci_info->ddr_info.ddr_entry[SMMU_CD_TABLE].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_SMMU_CD_TABLE]
			.cpu_pa =
			pci_info->ddr_info.ddr_entry[SMMU_CD_TABLE].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_SMMU_CD_TABLE]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[SMMU_CD_TABLE].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_SMMU_CD_TABLE].size =
			pci_info->ddr_info.ddr_entry[SMMU_CD_TABLE].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RESERVED_DDR]
			.noc_addr =
			pci_info->ddr_info.ddr_entry[RESERVED_DDR].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RESERVED_DDR]
			.fn_addr =
			pci_info->ddr_info.ddr_entry[RESERVED_DDR].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RESERVED_DDR]
			.cpu_pa =
			pci_info->ddr_info.ddr_entry[RESERVED_DDR].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RESERVED_DDR]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[RESERVED_DDR].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RESERVED_DDR].size =
			pci_info->ddr_info.ddr_entry[RESERVED_DDR].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RUNTIME].noc_addr =
			pci_info->ddr_info.ddr_entry[RUNTIME].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RUNTIME].fn_addr =
			pci_info->ddr_info.ddr_entry[RUNTIME].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RUNTIME].cpu_pa =
			pci_info->ddr_info.ddr_entry[RUNTIME].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RUNTIME].bar_offset =
			pci_info->ddr_info.ddr_entry[RUNTIME].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_RUNTIME].size =
			pci_info->ddr_info.ddr_entry[RUNTIME].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_HEAP].noc_addr =
			pci_info->ddr_info.ddr_entry[GPU_HEAP].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_HEAP].fn_addr =
			pci_info->ddr_info.ddr_entry[GPU_HEAP].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_HEAP].cpu_pa =
			pci_info->ddr_info.ddr_entry[GPU_HEAP].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_HEAP]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[GPU_HEAP].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_HEAP].size =
			pci_info->ddr_info.ddr_entry[GPU_HEAP].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_DISPLAY]
			.noc_addr =
			pci_info->ddr_info.ddr_entry[GPU_DISPLAY].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_DISPLAY]
			.fn_addr =
			pci_info->ddr_info.ddr_entry[GPU_DISPLAY].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_DISPLAY].cpu_pa =
			pci_info->ddr_info.ddr_entry[GPU_DISPLAY].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_DISPLAY]
			.bar_offset =
			pci_info->ddr_info.ddr_entry[GPU_DISPLAY].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GPU_DISPLAY].size =
			pci_info->ddr_info.ddr_entry[GPU_DISPLAY].size;

		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GART].noc_addr =
			pci_info->ddr_info.ddr_entry[GART].noc_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GART].fn_addr =
			pci_info->ddr_info.ddr_entry[GART].fn_addr;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GART].cpu_pa =
			pci_info->ddr_info.ddr_entry[GART].cpu_pa;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GART].bar_offset =
			pci_info->ddr_info.ddr_entry[GART].bar_offset;
		dev_info_p->info.ddr_info.ddr_entry[VATOOLS_GART].size =
			pci_info->ddr_info.ddr_entry[GART].size;

		// no die in gpu, use die_num=1 to save die_index, convinient to use in app
		dev_info_p->info.die_num = 1;
		for (die = 0; die < dev_info_p->info.die_num; die++) {
			dev_info_p->info.die_info[die].info.die_index.dev_id =
				pci_info->dev_id;
			dev_info_p->info.die_info[die].info.die_index.die_id =
				0;

			dev_info_p->info.die_info[die].info.die_index.seq_num =
				pci_info->dev_id;
			die_index_seq_num++;

			VATOOLS_DBG(
				pci_info, die,
				"dies[%d] dev_info_p->info.die_index=0x%x\n",
				die,
				dev_info_p->info.die_info[die]
					.info.die_index.val);

#ifdef CONFIG_VASTAI_VIDEO
			// vastai_video
			list_for_each_entry_safe (render_file_p,
						  render_file_temp,
						  &pci_info->render_head,
						  node) {
				if (render_file_p) {
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"video render_file_p->die_index=%d render_file_p->pci_info->dev_id=%d pci_info->dev_id=%d\n",
						render_file_p->die_index,
						render_file_p->pci_info->dev_id,
						pci_info->dev_id);

					if (render_file_p->pci_info->dev_id ==
					    pci_info->dev_id) {
						dev_info_p->info.die_info[die]
							.info.vastai_video
							.majormin.major = MAJOR(
							render_file_p->file_info
								.dev_num);
						dev_info_p->info.die_info[die]
							.info.vastai_video
							.majormin.minor = MINOR(
							render_file_p->file_info
								.dev_num);

						snprintf(
							dev_info_p->info
								.die_info[die]
								.info
								.vastai_video
								.name,
							VASTAI_BUF_SIZE_32,
							"%s",
							render_file_p->file_info
								.file_name);

						VATOOLS_DBG(
							NULL, DUMMY_DIE_ID,
							"video die_index=%d pci_info->dev_id=%d major=%d minor=%d name=%s $\n",
							render_file_p->die_index,
							pci_info->dev_id,
							dev_info_p->info
								.die_info[die]
								.info
								.vastai_video
								.majormin.major,
							dev_info_p->info
								.die_info[die]
								.info
								.vastai_video
								.majormin.minor,
							dev_info_p->info
								.die_info[die]
								.info
								.vastai_video
								.name);
						break;
					}
				}
			}
#endif
#ifdef CONFIG_VASTAI_AI
			// vacc
			list_for_each_entry_safe (vacc_dev_p, vacc_dev_temp,
						  &pci_info->vacc_dev_head,
						  vacc_dev_node) {
				if (vacc_dev_p) {
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"vacc vacc_dev_p->die_index=%d vacc_dev_p->pci_info->dev_id=%d pci_info->dev_id=%d\n",
						vacc_dev_p->die_index,
						vacc_dev_p->pci_info->dev_id,
						pci_info->dev_id);

					if (vacc_dev_p->pci_info->dev_id ==
					    pci_info->dev_id) {
						dev_info_p->info.die_info[die]
							.info.vacc.majormin
							.major = MAJOR(
							vacc_dev_p->file_info
								.dev_num);
						dev_info_p->info.die_info[die]
							.info.vacc.majormin
							.minor = MINOR(
							vacc_dev_p->file_info
								.dev_num);

						snprintf(
							dev_info_p->info
								.die_info[die]
								.info.vacc.name,
							VASTAI_BUF_SIZE_32,
							"%s",
							vacc_dev_p->file_info
								.file_name);

						VATOOLS_DBG(
							NULL, DUMMY_DIE_ID,
							"vacc die_index=%d pci_info->dev_id=%d major=%d minor=%d name=%s $\n",
							vacc_dev_p->die_index,
							pci_info->dev_id,
							dev_info_p->info
								.die_info[die]
								.info.vacc
								.majormin.major,
							dev_info_p->info
								.die_info[die]
								.info.vacc
								.majormin.minor,
							dev_info_p->info
								.die_info[die]
								.info.vacc.name);
						break;
					}
				}
			}
#endif

			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "spi die_index=0x%x pci_info->dev_id=%d\n",
				    dev_info_p->info.die_info[die]
					    .info.die_index.val,
				    pci_info->dev_id);
			// get spi config of die
			if (spi_buf) {
				memset(spi_buf, 0, SPI_BUF_LEN_MAX);
				vatools_get_spi_buf(dev_info_p->info
							    .die_info[die]
							    .info.die_index.val,
						    SPI_BLOCK_EEPROM, 0,
						    SPI_BUF_LEN_MAX, spi_buf);

				memcpy(dev_info_p->info.die_info[die]
					       .info.spi_buf,
				       spi_buf, SPI_BUF_LEN_MAX);
			}

			// get fw version of die
			memcpy(&dev_info_p->info.die_info[die].info.fw_ver,
			       &(pci_info->fw_ver),
			       sizeof(struct vastai_fw_version));

			// get pcie_nls, pcie_nlw
			dev_info_p->info.die_info[die].info.pcie_nls =
				pci_info->pcie_nls;
			dev_info_p->info.die_info[die].info.pcie_nlw =
				pci_info->pcie_nlw;

#if 0
#ifdef CONFIG_DRM_DRIVER
			// renderD
			list_for_each_entry_safe (render_file_p,
						  render_file_temp,
						  &pci_info->render_head,
						  node) {
				if (render_file_p->die_index ==
				    pci_info->dies[die].die_index) {
					device_handle =
						(struct vastai_device_handle
							 *)(render_file_p
								    ->dec_private_data);

					dev_info_p->info.die_info[die]
						.info.render.majormin.major =
						226; // drm 设备有系统指定主设备号为 226, 不变.

					dev_info_p->info.die_info[die]
						.info.render.majormin.minor =
						device_handle->drm_dev->render
							->index;

					snprintf(dev_info_p->info.die_info[die]
							 .info.render.name,
						 VASTAI_BUF_SIZE_32,
						 "%s%d",VA_NODE_NAME_RENDER,
						 pci_info->dies[die].render_id);
					break;
				}
			}
#endif

#endif
		} // end of die
		// 获取虚拟化数据
		memset(&dev_info_p->info.sriov_number, 0,
		       sizeof(struct sriov_number_s));
		dev_info_p->info.sriov_number.dev_id = pci_info->dev_id;
		dev_info_p->info.sriov_number.sriov_num =
			vatools_pci_get_sriov_numvfs(
				dev_info_p->info.sriov_number.dev_id);

		// 循环下一个device
		dev_info_p++;
		real_device_num++;
	}

	node->dev_info = (vatools_v2_device_info_u *)buffer;
	node->device_num = real_device_num;

	ioctl_arg_p->errcode = 0;
	ioctl_arg_p->count = real_device_num;
	ioctl_arg_p->output_buf.buf_size = buffer_size;

	if (copy_to_user_ex((void __user *)argp, ioctl_arg_p,
			    sizeof(ioctl_arg_t))) {
		ret = -EFAULT;
		VATOOLS_ERR(
			NULL, DUMMY_DIE_ID,
			"copy_to_user_ex failed: ret=%ld, sizeof(ioctl_arg_t)=%ld\n",
			ret, sizeof(ioctl_arg_t));
		goto out_free_buffer;
	}

	if (copy_to_user_ex((void __user *)devices_info.device_info_p, buffer,
			    buffer_size)) {
		ret = -EFAULT;
		VATOOLS_ERR(NULL, DUMMY_DIE_ID,
			    "copy_to_user_ex failed: ret=%ld, buffer_size=%d\n",
			    ret, buffer_size);
		goto out_free_buffer;
	}

	ret = 0;

out_free_buffer:
	// vfree(buffer);
	if (spi_buf) {
		vfree(spi_buf);
	}

out_free_ioctl_arg:
	kvfree(ioctl_arg_p);

out:
	domain_mutex_unlock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "real_device_num=%d out: ret=%ld\n",
		    real_device_num, ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

extern atomic_t _vatools_changed_device_count;

long vatools_ioctl_get_base_info(struct file *filp, unsigned int cmd,
				 IOCTL_ARG_T arg)
{
	long ret = -EINVAL;
	void __user *argp = (void __user *)arg;
	ioctl_arg_t *ioctl_arg_p;
	u32 device_num = 0;
	vatools_v2_base_info_u *base_info_p = NULL;

	VATOOLS_FUNC_ENTERY;
	V_UNREFERENCE(cmd);

	domain_mutex_lock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);

	ioctl_arg_p = (ioctl_arg_t *)vmalloc(sizeof(ioctl_arg_t));
	if (ioctl_arg_p == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%ld\n",
			    sizeof(ioctl_arg_t));
		ret = -ENOMEM;
		goto out;
	}

	ret = copy_from_user_ex(ioctl_arg_p, (void __user *)argp,
				sizeof(ioctl_arg_t));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_ioctl_arg;
	}

	base_info_p = (vatools_v2_base_info_u *)vmalloc(
		sizeof(vatools_v2_base_info_u));
	if (base_info_p == NULL) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "kvmalloc error. size=%ld\n",
			    sizeof(vatools_v2_base_info_u));
		ret = -ENOMEM;
		goto out_free_base_info;
	}

	memset(base_info_p, 0, sizeof(vatools_v2_base_info_u));

	// 获取 device 数量
	device_num = vatools_get_vastai_pci_device_number();

	// 拷贝版本号
	base_info_p->info.magic_number = VATOOLS_MAGIC_CODE;
	base_info_p->info.tools_ver_major = VATOOLS_DRV_VERSION_MAJOR_CODE;
	base_info_p->info.tools_ver_middle = VATOOLS_DRV_VERSION_MIDDLE_CODE;
	base_info_p->info.tools_ver_minor = VATOOLS_DRV_VERSION_MINOR_CODE;
	base_info_p->info.device_number = device_num;
	base_info_p->info.device_info_size = sizeof(vatools_v2_device_info_t);
	base_info_p->info.card_change_count =
		atomic_read(&_vatools_changed_device_count);

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"magic_number=0x%x tools_ver_major=%d tools_ver_middle=%d tools_ver_minor=%d device_number=%d "
		"device_info_size=%d card_change_count=%d\n",
		base_info_p->info.magic_number,
		base_info_p->info.tools_ver_major,
		base_info_p->info.tools_ver_middle,
		base_info_p->info.tools_ver_minor,
		base_info_p->info.device_number,
		base_info_p->info.device_info_size,
		base_info_p->info.card_change_count);

	ret = copy_to_user_ex((void __user *)ioctl_arg_p->output_buf.buf_addr,
			      base_info_p, sizeof(vatools_v2_base_info_u));
	if (ret) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex ret=%ld\n",
			    ret);
		ret = -EFAULT;
		goto out_free_base_info;
	}

	ioctl_arg_p->errcode = 0;
	ioctl_arg_p->count = device_num;
	ioctl_arg_p->output_buf.buf_size = sizeof(vatools_v2_base_info_u);

	if (copy_to_user_ex((void __user *)argp, ioctl_arg_p,
			    sizeof(ioctl_arg_t))) {
		ret = -EFAULT;
		goto out_free_base_info;
	}

	ret = 0;
out_free_base_info:
	vfree(base_info_p);

out_free_ioctl_arg:
	vfree(ioctl_arg_p);

out:
	domain_mutex_unlock(NULL, 0, VATOOLS_IOCTL_DOMAIN_DRIVER);

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "out: ret=%ld\n", ret);
	VATOOLS_FUNC_EXIT;
	return ret;
}

#endif

/**
 * 0: 64bit
 * 1: 32bit
 */
int is_32_bit_thread(void)
{
#if (TOOLS_WIN == 1)
	return 0;
#else
	int is_32_bit = 0;
#if (defined(__aarch64__) || defined(__arm__))
	is_32_bit = test_thread_flag(TIF_32BIT);
#else
	is_32_bit = test_thread_flag(TIF_ADDR32);
#endif

	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "thread is %d bit\n",
		    is_32_bit ? 32 : 64);
	return is_32_bit;
#endif
}

unsigned long copy_to_user_ex(void __user *to, const void *from,
			      unsigned long n)
{
	unsigned long ret = 0;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "copy_to_user_ex: to=%p from=%p len=%ld\n", to, from, n);
#ifdef CONFIG_COMPAT
	if (is_32_bit_thread()) {
		ret = copy_to_user((void __user *)compat_ptr((compat_uptr_t)(
					   ((u64)to) & 0xffffffff)),
				   from, n);
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_to_user_ex: fail: 32bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	} else
#endif
	{
		ret = copy_to_user(to, from, n);
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_to_user_ex: fail: 64bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	}
	return 0;
}

unsigned long copy_from_user_ex(void *to, const void __user *from,
				unsigned long n)
{
	unsigned long ret = 0;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "copy_from_user_ex: to=%p from=%p len=%ld\n", to, from, n);
#ifdef CONFIG_COMPAT
	if (is_32_bit_thread()) {
		ret = copy_from_user(to,
				     (void __user *)compat_ptr((compat_uptr_t)(
					     ((u64)from) & 0xffffffff)),
				     n);
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_from_user_ex: fail: 32bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	} else
#endif
	{
		ret = copy_from_user(to, from, n);
		if (ret) {
			VATOOLS_ERR(
				NULL, DUMMY_DIE_ID,
				"copy_from_user_ex: fail: 64bit: to=%p from=%p len=%ld ret=%ld\n",
				to, from, n, ret);
		}
		return ret;
	}
	return 0;
}

char *vaccrt_version_number2string(uint32_t num, char *str)
{
	uint8_t val1 = (uint8_t)(num % 256);
	uint8_t val2 = (uint8_t)((num >> 8) % 256);
	uint8_t val3 = (uint8_t)((num >> 16) % 256);

	if (str == NULL) {
		return NULL;
	}
	memset(str, 0, strlen(str));
	sprintf(str, "%hhu.%hhu.%hhu", val3, val2, val1);
	return str;
}
