﻿#ifndef __VATOOLS_LOG_H__
#define __VATOOLS_LOG_H__

#pragma pack(1)

#define VATOOLS_PREFIX	      ""
#define VATOOLS_DRIVER_PREFIX VATOOLS_PREFIX "tools "
#define VATOOLS_DEBUG_PREFIX  VATOOLS_DRIVER_PREFIX
#define DUMMY_DIE_ID	      0xff

#ifdef pr_fmt
#undef pr_fmt
#endif
#define pr_fmt(fmt) VATOOLS_DEBUG_PREFIX "(%d) " fmt,  __LINE__

#define VASTAI_PCI_NORMAL_LEVEL (0x01)
#define VASTAI_PCI_DEBUG_LEVEL	(0x02)
#define VASTAI_PCI_DATA_LEVEL	(0x04)
#define VASTAI_PCI_PERF_LEVEL	(0x08)

#ifndef CONFIG_VASTAI_KUNIT
#ifndef VASTAI_GET_PCI_NAME
#define VASTAI_GET_PCI_NAME(pcie)                                              \
	({                                                                     \
		struct vastai_pci_info *__priv = pcie;                         \
		(__priv ? pci_name((__priv)->dev) : "NULL");                   \
	})
#endif
#else
#ifndef VASTAI_GET_PCI_NAME
#define VASTAI_GET_PCI_NAME(pcie) ({ ("fake-pci-name"); })
#endif
#endif
u32 vastai_pci_get_die_index(void *pci_info, u32 die_id);
#ifndef VASTAI_GET_DIE_INDEX
#define VASTAI_GET_DIE_INDEX(pcie, die_id)                                     \
	({ vastai_pci_get_die_index(pcie, die_id); })
#endif

extern unsigned char va_pci_log_level;

/* va_pci_log_level = 0, printing pr_err information */
#define VATOOLS_ERR(pcie, die_id, fmt, args...)                                \
	do {                                                                   \
		pr_err("ERROR %s:%02x:%07x: " fmt, VASTAI_GET_PCI_NAME(pcie),  \
		       die_id, VASTAI_GET_DIE_INDEX(pcie, die_id), ##args);    \
	} while (0)

#define VATOOLS_ERR_RATELIMIT(pcie, die_id, fmt, args...)                      \
	do {                                                                   \
		pr_err_ratelimited("ERROR %s:%02x:%07x: " fmt,                 \
				   VASTAI_GET_PCI_NAME(pcie), die_id,          \
				   VASTAI_GET_DIE_INDEX(pcie, die_id),         \
				   ##args);                                    \
	} while (0)

/* VASTAI_PCI_NORMAL_LEVEL, printing pr_info information */
#define VATOOLS_INFO(pcie, die_id, fmt, args...)                               \
	do {                                                                   \
		struct vastai_pci_info *__priv = pcie;                         \
		if (va_pci_log_level & VASTAI_PCI_NORMAL_LEVEL) {          \
			if (pcie == NULL)                                      \
				pr_info("INFO " fmt, ##args);                  \
			else if (die_id >= __priv->die_num)                    \
				pr_info("INFO %s: " fmt,                       \
					VASTAI_GET_PCI_NAME(pcie), ##args);    \
			else                                                   \
				pr_info("INFO %s:%02x:%07x: " fmt,             \
					VASTAI_GET_PCI_NAME(pcie), die_id,     \
					VASTAI_GET_DIE_INDEX(pcie, die_id),    \
					##args);                               \
		}                                                              \
	} while (0)

#define VATOOLS_INFO_RATELIMIT(pcie, die_id, fmt, args...)                     \
	do {                                                                   \
		struct vastai_pci_info *__priv = pcie;                         \
		if (va_pci_log_level & VASTAI_PCI_NORMAL_LEVEL) {          \
			if (pcie == NULL)                                      \
				pr_info_ratelimited("INFO " fmt, ##args);      \
			else if (die_id >= __priv->die_num)                    \
				pr_info_ratelimited("INFO %s: " fmt,           \
						    VASTAI_GET_PCI_NAME(pcie), \
						    ##args);                   \
			else                                                   \
				pr_info_ratelimited(                           \
					"INFO %s:%02x:%07x: " fmt,             \
					VASTAI_GET_PCI_NAME(pcie), die_id,     \
					VASTAI_GET_DIE_INDEX(pcie, die_id),    \
					##args);                               \
		}                                                              \
	} while (0)

/* VASTAI_PCI_DEBUG_LEVEL, printing debug information */
#define VATOOLS_DBG_RATELIMIT(pcie, die_id, fmt, args...)                      \
	do {                                                                   \
		if (va_pci_log_level & VASTAI_PCI_DEBUG_LEVEL) {           \
			if (pcie == NULL)                                      \
				pr_info_ratelimited("DEBUG " fmt, ##args);     \
			else if (die_id == 0xff)                               \
				pr_info_ratelimited("DEBUG %s: " fmt,          \
						    VASTAI_GET_PCI_NAME(pcie), \
						    ##args);                   \
			else                                                   \
				pr_info_ratelimited(                           \
					"DEBUG %s:%02x:%07x: " fmt,            \
					VASTAI_GET_PCI_NAME(pcie), die_id,     \
					VASTAI_GET_DIE_INDEX(pcie, die_id),    \
					##args);                               \
		}                                                              \
	} while (0)

#define VATOOLS_DBG(pcie, die_id, fmt, args...)                                \
	do {                                                                   \
		if (va_pci_log_level & VASTAI_PCI_DEBUG_LEVEL) {           \
			if (pcie == NULL)                                      \
				pr_info_ratelimited("DEBUG " fmt, ##args);     \
			else if (die_id == 0xff)                               \
				pr_info_ratelimited("DEBUG %s: " fmt,          \
						    VASTAI_GET_PCI_NAME(pcie), \
						    ##args);                   \
			else                                                   \
				pr_info_ratelimited(                           \
					"DEBUG %s:%02x:%07x: " fmt,            \
					VASTAI_GET_PCI_NAME(pcie), die_id,     \
					VASTAI_GET_DIE_INDEX(pcie, die_id),    \
					##args);                               \
		}                                                              \
	} while (0)

/* VASTAI_PCI_DATA_LEVEL, printing data or dump information */
#define VATOOLS_HEX_DUMP(level, prefix_str, prefix_type, rowsize, groupsize,   \
			 buf, len, ascii)                                      \
	do {                                                                   \
		if (va_pci_log_level & VASTAI_PCI_DEBUG_LEVEL)             \
			print_hex_dump(level, prefix_str, prefix_type,         \
				       rowsize, groupsize, buf, len, ascii);   \
	} while (0)

/* VASTAI_PCI_PERF_LEVEL, printing perf information */
#define VATOOLS_PERF(fmt, args...)                                             \
	do {                                                                   \
		if (va_pci_log_level & VASTAI_PCI_PERF_LEVEL)              \
			trace_printk(" DEBUG: " fmt, ##args);                  \
	} while (0)

#define VATOOLS_INFO_PID(fmt, ...)                                             \
	do {                                                                   \
		pr_info_ratelimited("[tgid:%d pid:%d] (%d) " fmt,           \
				    current->tgid, current->pid, __LINE__,     \
				    ##__VA_ARGS__);              \
	} while (0)

#define VATOOLS_DUMP_BRIEF(title, buf, buf_size_byte)                          \
	VATOOLS_HEX_DUMP(KERN_DEBUG, VATOOLS_DEBUG_PREFIX title,               \
			 DUMP_PREFIX_NONE, 32, 4, buf, buf_size_byte, false)

#define VATOOLS_FUNC_ENTERY                                                    \
	VATOOLS_DBG_RATELIMIT(NULL, DUMMY_DIE_ID, "pid:%d %d entry !\n",       \
			      current->pid, __LINE__)
#define VATOOLS_FUNC_EXIT                                                      \
	VATOOLS_DBG_RATELIMIT(NULL, DUMMY_DIE_ID, "pid:%d %d exit !\n",        \
			      current->pid, __LINE__)

#pragma pack()

#endif //__VATOOLS_LOG_H__
