﻿#include "vatools.h"
#include "vatools_log.h"

/******************************************************************************
 * shared memory
 *
 *
 */
//创建内存，把内存挂载到reader上
static long sharedmem_create_buf(struct vatools_reader *reader,
				 T_MEMORY_ENTRY *p_memory, int is_debug)
{
	T_MEM_INFO *p_mem_info = NULL;
	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;

	VATOOLS_FUNC_ENTERY;

	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		p_memory->n_shard_addr, p_memory->tdata.n_type);

	if (p_memory->tdata.n_len < sizeof(union vatools_filter)) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "unsupported tdata.n_len\n");
		goto ERR_MEM_INFO;
	}

	p_mem_info = (T_MEM_INFO *)kvmalloc(sizeof(T_MEM_INFO), GFP_KERNEL);
	if (!p_mem_info) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, " kvmalloc error. size=%ld\n",
			    sizeof(T_MEM_INFO));
		goto ERR_MEM_INFO;
	}

	p_mem_info->p_buf =
		kvmalloc((p_memory->tdata.n_len + sizeof(u64)), GFP_KERNEL);

	if (!p_mem_info->p_buf) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, " kvmalloc error. size=%ld\n",
			    p_memory->tdata.n_len + sizeof(u64));
		goto ERR_MEM_INFO_BUF;
	}
	//保存数据，查找数据 lupa 2021-12-15
	//创建时用户回调的数据
	p_mem_info->user_data = p_memory->n_user_addr;
	p_mem_info->callback = p_memory->callback;
	memcpy(&p_mem_info->tchannel, &p_memory->tchannel,
	       sizeof(p_memory->tchannel));
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"sharedmem_create_buf p_mem_info->tchannel.id=%d,p_mem_info->tchannel.type = "
		"%d,p_mem_info->tchannel.writetype=%d\n",
		p_mem_info->tchannel.id, p_mem_info->tchannel.type,
		p_mem_info->tchannel.writetype);
	//保存数据英语数据查找
	p_mem_info->n_effic_len = p_memory->tdata.n_len;
	INIT_LIST_HEAD(&p_mem_info->t_sharedmem_node);
	//用于video读取数据的节点 lupa add 2021-12-15 add begin
	p_mem_info->tvideo_debugger.debugtype = 0;
	p_mem_info->tvideo_debugger.debuginfo = NULL;
	p_mem_info->tvideo_debugger.needsendmsg = 0;
	p_mem_info->tvideo_debugger.startflag = 0;
	p_mem_info->tvideo_debugger.data_cnt = 0;
	p_mem_info->tvideo_debugger.is_debugbuf = is_debug;
	p_mem_info->tvideo_debugger.pmeminfo = p_mem_info;
	mutex_init(&p_mem_info->tvideo_debugger.data_info_mutex);
	INIT_LIST_HEAD(&p_mem_info->tvideo_debugger.t_node_head);
	//用于video读取数据的节点 lupa add 2021-12-15 add end
	mutex_init(&p_mem_info->mem_info_mutex);

	reader->filter.filter_info.element_len = p_memory->tdata.n_len;
	//对应的父节点带入进去
	p_mem_info->reader = reader;
	if (is_debug) {
		p_mem_info->tvideo_debugger.debugtype = p_memory->tdata.n_type;
	}
	mutex_lock(&reader->sharedmem_mutex);
	list_add_tail(&p_mem_info->t_sharedmem_node,
		      &reader->t_sharedmem_mem_info_head);
	mutex_unlock(&reader->sharedmem_mutex);

	p_memory->n_shard_addr = (u64)p_mem_info;
	p_buf_header = (T_SHAREDMEM_BUF_HEADER *)p_mem_info->p_buf;
	p_buf_header->n_addr = p_memory->n_shard_addr;
	// VATOOLS_DBG( " end." );
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "end . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		    p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		    p_memory->n_shard_addr, p_memory->tdata.n_type);

	VATOOLS_FUNC_EXIT;
	return 0;

ERR_MEM_INFO_BUF:
	p_mem_info->p_buf = NULL;
	kvfree(p_mem_info);
ERR_MEM_INFO:
	p_mem_info = NULL;
	VATOOLS_FUNC_EXIT;

	return -EFAULT;
}
//根据通道获取对应的文件句柄指针 lupa 2015-12-15 add
static T_MEM_INFO *
sharedmem_get_memptr_by_channel(struct vatools_reader *reader,
				T_MEMORY_ENTRY *p_memory)
{
	//内存信息
	T_MEM_INFO *p_node1 = NULL, *p_tmp1 = NULL;
	T_MEM_INFO *p_mem_info = NULL;
	// struct list_head* ptr        = NULL;
	//文件节点信息
	struct vatools_node *p_current = NULL, *p_next = NULL;
	struct vatools_reader *p_current_reader = NULL, *p_next_reader = NULL;
	if (NULL == p_memory) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return NULL;
	}
	if (VATOOLS_MEM_CMD_CHANNEL != p_memory->cmd_type) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"the cmd type is error: no equal VATOOLS_MEM_CMD_CHANNEL\n");
		return NULL;
	}
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"0.p_memory->tchannel.id=%d p_memory->tchannel.type=%d p_memory->tchannel.writetype=%d\n",
		p_memory->tchannel.id, p_memory->tchannel.type,
		p_memory->tchannel.writetype);
	if (NULL == reader) {
		//遍历所有的文件节点。目前只有一个
		list_for_each_entry_safe (p_current, p_next,
					  vatools_get_vastai_head(),
					  list_nodes) {
			/*
            // mutex_lock( &p_current->node_mutex );
            list_for_each( ptr, &p_current->list_node_readers )
            {
                if ( NULL == ptr || ptr->next == NULL || NULL == ptr->prev )
                {
                    VATOOLS_DBG( "sharedmem_get_memptr_by_channel " );
                    dump_stack();
                    return NULL;
                }
            }
            */
			//获取所有的文件打开句柄
			list_for_each_entry_safe (p_current_reader,
						  p_next_reader,
						  &p_current->list_node_readers,
						  list_readers) {
				//只比较共享内存使用的文件句柄
				if (p_current_reader->trans_category
					    .app_category ==
				    VATOOLS_APP_SHAREDMEM) {
					//查找对应的channel信息--目前只支持对数据区域的排序
					if (p_current_reader
						    ->n_sharedmem_init_done !=
					    VATOOLS_INIT_DONE) {
						VATOOLS_DBG(
							NULL, DUMMY_DIE_ID,
							"2.sharedmem_get_other_reader.p_current_reader->n_sharedmem_init_done != "
							"VATOOLS_INIT_DONE\n");
						continue;
					}
					mutex_lock(&p_current_reader
							    ->sharedmem_mutex);
					list_for_each_entry_safe (
						p_node1, p_tmp1,
						&p_current_reader
							 ->t_sharedmem_mem_info_head,
						t_sharedmem_node) {
						VATOOLS_DBG(
							NULL, DUMMY_DIE_ID,
							"1.p_node->tchannel.id=%d p_node->tchannel.type=%d p_node->tchannel.writetype=%d\n",
							p_node1->tchannel.id,
							p_node1->tchannel.type,
							p_node1->tchannel
								.writetype);
						if (p_node1->tchannel.id ==
							    p_memory->tchannel
								    .id &&
						    p_node1->tchannel.type ==
							    p_memory->tchannel
								    .type &&
						    p_node1->tchannel.writetype ==
							    p_memory->tchannel
								    .writetype) {
							p_mem_info = p_node1;
							break;
						}
					}
					mutex_unlock(
						&p_current_reader
							 ->sharedmem_mutex);
				}
				//查找到退出循环
				if (NULL != p_mem_info) {
					break;
				}
			}
			// mutex_unlock( &p_current->node_mutex );
			//查找到退出循环
			if (NULL != p_mem_info) {
				break;
			}
		}
	} else {
		if (reader->trans_category.app_category ==
		    VATOOLS_APP_SHAREDMEM) {
			// if ( p_memory->tchannel.writetype == VATOOLS_SHAREDMEM_WRITE_DATA
			//     || p_memory->tchannel.writetype == VATOOLS_SHAREDMEM_WRITE_DEBUG )
			// {
			if (reader->n_sharedmem_init_done !=
			    VATOOLS_INIT_DONE) {
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID,
					"2.sharedmem_get_other_reader.p_current_reader->n_sharedmem_init_done != "
					"VATOOLS_INIT_DONE\n");
				goto GET_SHARED_END;
			}
			mutex_lock(&reader->sharedmem_mutex);
			list_for_each_entry_safe (
				p_node1, p_tmp1,
				&reader->t_sharedmem_mem_info_head,
				t_sharedmem_node) {
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID,
					"2.p_node->tchannel.id=%d p_node->tchannel.type=%d p_node->tchannel.writetype=%d\n",
					p_node1->tchannel.id,
					p_node1->tchannel.type,
					p_node1->tchannel.writetype);
				if (p_node1->tchannel.id ==
					    p_memory->tchannel.id &&
				    p_node1->tchannel.type ==
					    p_memory->tchannel.type &&
				    p_node1->tchannel.writetype ==
					    p_memory->tchannel.writetype) {
					p_mem_info = p_node1;
					break;
				}
			}
			mutex_unlock(&reader->sharedmem_mutex);
			//  }
		}
	}
GET_SHARED_END:
	if (NULL == p_mem_info) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"no find the channel ptr, id=%d ,type=%d writetype=%d\n",
			p_memory->tchannel.id, p_memory->tchannel.type,
			p_memory->tchannel.writetype);
	}
	VATOOLS_FUNC_EXIT;
	return p_mem_info;
}
//获取对应的文件句柄指针 lupa 2015-12-15 add
static T_MEM_INFO *sharedmem_get_memptr(struct vatools_reader *reader,
					T_MEMORY_ENTRY *p_memory)
{
	T_MEM_INFO *p_mem_info = NULL;
	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;
	if (NULL == p_memory) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return NULL;
	}
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "cmd_type=%d\n", p_memory->cmd_type);
	switch (p_memory->cmd_type) {
	case VATOOLS_MEM_CMD_HANDLE:
		p_mem_info = (T_MEM_INFO *)p_memory->n_shard_addr;
		break;
	case VATOOLS_MEM_CMD_CHANNEL:
		p_mem_info = sharedmem_get_memptr_by_channel(reader, p_memory);
		break;
	default:
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    " no find the command ,the command type=%d .\n",
			    p_memory->cmd_type);
		p_mem_info = NULL;
		break;
	}
	if (p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info=%p\n", p_mem_info);
		mutex_lock(&p_mem_info->mem_info_mutex);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info  %p=lock\n",
			    p_mem_info);
		p_buf_header = (T_SHAREDMEM_BUF_HEADER *)p_mem_info->p_buf;
		//简单校验。保障数据的一致性
		if (p_buf_header->n_addr != (u64)p_mem_info) {
			// BUGID :8816 lupa mod 2022-05-10
			//p_mem_info = NULL;
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "addr is not mash ptr\n");
			mutex_unlock(&p_mem_info->mem_info_mutex);
			VATOOLS_FUNC_EXIT;
			return NULL;
		}

		mutex_unlock(&p_mem_info->mem_info_mutex);
	}
	VATOOLS_FUNC_EXIT;
	return p_mem_info;
}

static struct vatools_reader *
sharedmem_get_other_reader(struct vatools_reader *reader)
{
	struct vatools_node *p_current = NULL, *p_next = NULL;
	struct vatools_reader *p_current_reader = NULL, *p_next_reader = NULL;
	struct vatools_reader *p_reader = NULL;
	// struct list_head*      ptr      = NULL;
	if (NULL == reader) {
		// pr_err( "error  NULL == reader\n" );
		return NULL;
	}
	VATOOLS_FUNC_ENTERY;

	list_for_each_entry_safe (p_current, p_next, vatools_get_vastai_head(),
				  list_nodes) {
		/*
        // mutex_lock( &p_current->node_mutex );
        list_for_each( ptr, &p_current->list_node_readers )
        {
            // pr_info( "sharedmem_get_other_reader ptr=%p\n", ptr );
            if ( NULL == ptr || ptr->next == NULL || NULL == ptr->prev )
            {
                VATOOLS_DBG( "sharedmem_get_memptr_by_channel " );
                dump_stack();
                return NULL;
            }
        }
        */
		list_for_each_entry_safe (p_current_reader, p_next_reader,
					  &p_current->list_node_readers,
					  list_readers) {
			if (p_current_reader->trans_category.app_category ==
			    VATOOLS_APP_SHAREDMEM) {
				if (p_current_reader->n_sharedmem_init_done !=
				    VATOOLS_INIT_DONE) {
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"sharedmem_get_other_reader.p_current_reader->n_sharedmem_init_done != VATOOLS_INIT_DONE\n");
					continue;
				}
				//在共享内存中die_index 被用于表示同一类的文件句柄类型，即 一个业务节点对应一个等待节点
				// pr_info( "p_current_reader=%p\n", p_current_reader );
				// pr_info( "reader=%p\n", reader );
				// pr_info( " die_index=0x%x\n", p_current_reader->trans_category.device.die_index );
				// pr_info( " dev_id=%d\n", p_current_reader->trans_category.device.dev_id );
				// pr_info( " die_id=%d\n", p_current_reader->trans_category.device.die_id );
				if ((p_current_reader != reader) &&
				    (p_current_reader->trans_category.device
					     .dev_id ==
				     reader->trans_category.device.dev_id) &&
				    (p_current_reader->trans_category.device
					     .die_id ==
				     reader->trans_category.device.die_id) &&
				    (p_current_reader->trans_category.device
					     .die_index ==
				     reader->trans_category.device.die_index)) {
					p_reader = p_current_reader;
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"p_reader=%p,dev_id=%d,die_id=%d,die_index=0x%x\n",
						p_reader,
						reader->trans_category.device
							.dev_id,
						reader->trans_category.device
							.die_id,
						reader->trans_category.device
							.die_index);
					break;
				}
			}
		}
		// mutex_unlock( &p_current->node_mutex );
	}
	if (!p_reader) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "no find the filetype, filetype=%d\n",
			    reader->trans_category.device.die_index);
	}

	VATOOLS_FUNC_EXIT;
	return p_reader;
}
int sharemen_set_comp_complete(struct vatools_reader *reader)
{
	if (reader) {
		if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"sharemen_set_comp_complete reader->n_sharedmem_init_done not int\n");
			return 0;
		}
		mutex_lock(&reader->sharedmem_mutex);
		complete(&reader->debug_comp);
		mutex_unlock(&reader->sharedmem_mutex);
	}
	return 0;
}
int sharedmen_free_video_debugbuf(t_video_debug_head *video_debug)
{
	t_video_debug_info *p_current = NULL, *p_tmp = NULL;
	T_MEM_INFO *p_mem_info = NULL;
	T_MEMORY_ENTRY t_memory;
	if (NULL == video_debug) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "mem_info is null .\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;

	//对于debugger相关的需要释放资源
	if (1 == video_debug->is_debugbuf) {
		//通知业务资源释放
		memset(&t_memory, 0x00, sizeof(t_memory));
		t_memory.cmd_type = VATOOLS_MEM_CMD_CHANNEL;
		memcpy(&t_memory.tchannel, &video_debug->pmeminfo->tchannel,
		       sizeof(t_memory.tchannel));
		//由于节点挂载业务节点上所有要 查找业务节点
		t_memory.tchannel.writetype = VATOOLS_SHAREDMEM_WRITE_DATA;
		//获取该通道对应的数据查找所有的节点
		p_mem_info = sharedmem_get_memptr_by_channel(NULL, &t_memory);
		if (p_mem_info) {
			if (1 == p_mem_info->tvideo_debugger.startflag) {
				//需要找到对应的wait句柄进行处理
				struct vatools_reader *wait_reader =
					sharedmem_get_other_reader(
						p_mem_info->reader);
				if (wait_reader) {
					//设置对应的debugger数据
					p_mem_info->tvideo_debugger.debugtype =
						video_debug->debugtype;
					p_mem_info->tvideo_debugger.needsendmsg =
						1;
					p_mem_info->tvideo_debugger.startflag =
						0;
					//设置等待超时
					sharemen_set_comp_complete(wait_reader);
				}
			}
			if (NULL != p_mem_info->tvideo_debugger.debuginfo) {
				kvfree(p_mem_info->tvideo_debugger.debuginfo);
				p_mem_info->tvideo_debugger.debuginfo = NULL;
			}
		}
	}
	list_for_each_entry_safe (p_current, p_tmp, &video_debug->t_node_head,
				  t_video_node) {
		//先重队列中取下来
		list_del(&p_current->t_video_node);
		//再释放空间
		kvfree(p_current->p_buf);
		kvfree(p_current);
	}
	video_debug->data_cnt = 0;
	if (0 != video_debug->debuginfo) {
		kvfree(video_debug->debuginfo);
		video_debug->debuginfo = 0;
	}
	VATOOLS_FUNC_EXIT;
	return 0;
}
static long sharedmem_delete_buf(struct vatools_reader *reader,
				 T_MEMORY_ENTRY *p_memory, int is_debug)
{
	T_MEM_INFO *p_mem_info = NULL;
	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;
	if (!p_memory) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "Input parameter is null error .\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		p_memory->n_shard_addr, p_memory->tdata.n_type);
	p_mem_info = sharedmem_get_memptr(reader, p_memory);
	// pr_info( "sharedmem_delete_buf is_debug=%d\n", is_debug );
	if (!p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info is null .\n");
		goto ERR_MEM_INFO;
	}

	p_buf_header = (T_SHAREDMEM_BUF_HEADER *)p_mem_info->p_buf;
	if (!is_debug && p_buf_header->n_addr != p_memory->n_shard_addr) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, " delete addr is error .\n");
		goto ERR_MEM_INFO;
	}
	// delete 已经整体加锁了，内部可以不用加了
	//删除对应的debugger节点信息 lupa 2021-12-15 add
	mutex_lock(&reader->sharedmem_mutex);
	list_del(&p_mem_info->t_sharedmem_node);
	mutex_unlock(&reader->sharedmem_mutex);
	sharedmen_free_video_debugbuf(&p_mem_info->tvideo_debugger);
	kvfree(p_mem_info->p_buf);
	p_mem_info->p_buf = NULL;

	kvfree(p_mem_info);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, " end\n");
	VATOOLS_FUNC_EXIT;
	return 0;

ERR_MEM_INFO:

	VATOOLS_FUNC_EXIT;

	return -EFAULT;
}

static long sharemem_dlen_per_reader(struct vatools_reader *reader,
				     void __user *p_userbuf, int max_len,
				     int n_type)
{
	int n_len_cnt = 0;
	T_MEM_INFO *p_node = NULL, *p_tmp = NULL;

	void __user *p_buf = (void __user *)p_userbuf;

	if (!reader) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			" list_for_each_entry_safe reader = NULL or p_buf == NULL\n");
		return -EFAULT;
	}
	if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"reader->n_sharedmem_init_done ! = VATOOLS_INIT_DONE\n");
		return 0;
	}
	if (!p_buf) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "Input parameter is null error .\n");
		return -EFAULT;
	}

	list_for_each_entry_safe (p_node, p_tmp,
				  &reader->t_sharedmem_mem_info_head,
				  t_sharedmem_node) {
		if (!p_node) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				" list_for_each_entry_safe p_node = NULL\n");
			continue;
		}
		if (n_type != p_node->tchannel.writetype ||
		    p_node->n_effic_len == 0) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"n_type(%d) != p_node->tchannel.writetype(%d)  ,p_node->n_effic_len=%d  .\n",
				n_type, p_node->tchannel.writetype,
				p_node->n_effic_len);
			continue;
		}

		if (n_len_cnt + p_node->n_effic_len > max_len) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "User buf is too small .\n");
			break;
		}

		n_len_cnt += p_node->n_effic_len;
	}

	return n_len_cnt;
}

static long sharedmem_read_buf(struct vatools_reader *reader,
			       void __user *p_userbuf, int max_len, int n_type)
{
	int n_len_cnt = 0;
	int n_ret = 0;
	T_MEM_INFO *p_node = NULL, *p_tmp = NULL;

	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;
	// struct list_head* t_lsit_head = NULL;
	void __user *p_buf = (void __user *)p_userbuf;

	if (!reader) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			" list_for_each_entry_safe reader = NULL or p_buf == NULL\n");
		return -EFAULT;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "begin .\n");
	//在多进程运行过程中可能出现只open就被其他进程处理的场景
	//没有初始化的数据不处理
	if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"reader->n_sharedmem_init_done ! = VATOOLS_INIT_DONE\n");
		return 0;
	}
	if (!p_buf) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "Input parameter is null error .\n");
		return -EFAULT;
	}
	mutex_lock(&reader->sharedmem_mutex);
	list_for_each_entry_safe (p_node, p_tmp,
				  &reader->t_sharedmem_mem_info_head,
				  t_sharedmem_node) {
		if (!p_node) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				" list_for_each_entry_safe p_node = NULL\n");
			continue;
		}
		if (n_type != p_node->tchannel.writetype ||
		    p_node->n_effic_len == 0) {
			VATOOLS_DBG(
				NULL, DUMMY_DIE_ID,
				"n_type(%d) != p_node->tchannel.writetype(%d)  ,p_node->n_effic_len=%d  .\n",
				n_type, p_node->tchannel.writetype,
				p_node->n_effic_len);
			continue;
		}

		if (n_len_cnt + p_node->n_effic_len > max_len) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "User buf is too small .\n");
			break;
		}
		mutex_lock(&p_node->mem_info_mutex);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info  %p=lock\n",
			    p_node);
		p_buf_header = (T_SHAREDMEM_BUF_HEADER *)p_node->p_buf;
		// pr_info( "#%d %scopy_to_user=%p %p\n", __LINE__, __FUNCTION__, p_buf_header->c_bodybuf, p_buf + n_len_cnt );
		if (NULL != p_buf_header && p_node->n_effic_len > 0) {
			if (copy_to_user_ex((u8 *)p_buf + n_len_cnt,
					    p_buf_header->c_bodybuf,
					    p_node->n_effic_len)) {
				VATOOLS_ERR(
					NULL, DUMMY_DIE_ID,
					"body info copy_to_user_ex error .\n");
				n_ret = -EFAULT;
			}
		}

		mutex_unlock(&p_node->mem_info_mutex);
		if (n_ret < 0) {
			VATOOLS_INFO(NULL, DUMMY_DIE_ID, " n_ret=%d error .\n",
				     n_ret);
			break;
		}

		n_len_cnt += p_node->n_effic_len;
	}
	mutex_unlock(&reader->sharedmem_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end . n_len_cnt=%d\n", n_len_cnt);
	return n_len_cnt;
}
static long sharedmem_read_all(T_MEMORY_ENTRY *p_memory)
{
	struct vatools_node *p_current = NULL, *p_next = NULL;
	struct vatools_reader *p_current_reader = NULL, *p_next_reader = NULL;
	u32 n_len_cnt = 0;
	int n_ret = 0;

	// struct list_head*     ptr   = NULL;
	T_SHAREDMEM_BUF_INFO *p_buf =
		(T_SHAREDMEM_BUF_INFO *)p_memory->tdata.n_buf_addr;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		p_memory->n_shard_addr, p_memory->tdata.n_type);
	if (!p_buf) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     " Input parameter is null error .\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	list_for_each_entry_safe (p_current, p_next, vatools_get_vastai_head(),
				  list_nodes) {
#if 1
		/*
                // mutex_lock( &p_current->node_mutex );
                list_for_each( ptr, &p_current->list_node_readers )
                {
                    if ( NULL == ptr || ptr->next == NULL || NULL == ptr->prev )
                    {
                        VATOOLS_DBG( "sharedmem_get_memptr_by_channel " );
                        dump_stack();
                        return -EFAULT;
                    }
                }
                */
		list_for_each_entry_safe (p_current_reader, p_next_reader,
					  &p_current->list_node_readers,
					  list_readers) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    " p_current_reader=0x%p  p_current=0x%p\n",
				    p_current_reader, p_current);

			if (p_current_reader->trans_category.app_category ==
			    VATOOLS_APP_SHAREDMEM) {
				//按照节点拷贝数据
				if (n_len_cnt >= p_memory->tdata.n_len) {
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"n_len_cnt=%d greater then p_memory->n_len=%d .\n",
						n_len_cnt,
						p_memory->tdata.n_len);
					break;
				}

				n_ret = sharemem_dlen_per_reader(
					p_current_reader,
					(u8 *)p_buf->c_buf + n_len_cnt,
					(p_memory->tdata.n_len - n_len_cnt),
					p_memory->tdata.n_type);
				if (n_ret < 0) {
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"dlen check return=%d error .\n",
						n_ret);
					break;
				}

				if (n_ret != 0) {
					p_current_reader->filter.filter_info
						.data_len = n_ret;
					if (copy_to_user_ex(
						    (u8 *)p_buf->c_buf +
							    n_len_cnt,
						    &p_current_reader->filter,
						    sizeof(union vatools_filter))) {
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"filter info copy_to_user_ex error .\n");
						n_ret = -EFAULT;
						break;
					}
					VATOOLS_DBG(
						NULL, DUMMY_DIE_ID,
						"vpid %d, ns %lld, level %d, element %d\n",
						p_current_reader->filter
							.filter_info.vpid,
						p_current_reader->filter
							.filter_info.ns,
						p_current_reader->filter
							.filter_info.level,
						p_current_reader->filter
							.filter_info
							.element_len);
					n_len_cnt +=
						p_current_reader->filter
							.filter_info.element_len;
				}

				n_ret = sharedmem_read_buf(
					p_current_reader,
					(u8 *)p_buf->c_buf + n_len_cnt,
					(p_memory->tdata.n_len - n_len_cnt),
					p_memory->tdata.n_type);
				if (n_ret < 0) {
					VATOOLS_DBG(NULL, DUMMY_DIE_ID,
						    " return=%d error .\n",
						    n_ret);
					break;
				}
				n_len_cnt += n_ret;
			}
		}
		// mutex_unlock( &p_current->node_mutex );
		if (n_ret < 0) {
			return n_ret;
		}
#endif
	}
	if (copy_to_user_ex((void __user *)&p_buf->n_len, &n_len_cnt,
			    sizeof(p_buf->n_len))) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex error .\n");
		return -EFAULT;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end .\n");
	VATOOLS_FUNC_EXIT;
	return 0;
}
static long sharedmem_write_buf(struct vatools_reader *reader,
				T_MEMORY_ENTRY *p_memory)
{
	long n_ret = 0;
	T_MEM_INFO *p_mem_info = NULL;
	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;

	V_UNREFERENCE(reader);
	if (NULL == p_memory) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     " Input parameter is null error .\n");
		return -EFAULT;
	}
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		p_memory->n_shard_addr, p_memory->tdata.n_type);
	p_mem_info = sharedmem_get_memptr(NULL, p_memory);
	if (!p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    " p_mem_info is null error .\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	p_buf_header = (T_SHAREDMEM_BUF_HEADER *)p_mem_info->p_buf;
	if (p_buf_header->n_addr != p_memory->n_shard_addr) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, " write addr is error .\n");
		return -EFAULT;
	}
	if (p_mem_info->n_effic_len < (int)p_memory->tdata.n_len) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     " write date len(%d) malloc len(%d)  is error .\n",
			     p_memory->tdata.n_len, p_mem_info->n_effic_len);
		return -EFAULT;
	}
	mutex_lock(&p_mem_info->mem_info_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info  %p=lock\n", p_mem_info);
	// pr_info( "#%d %copy_from_user_ex=%p %p\n", __LINE__, __FUNCTION__, p_buf_header->c_bodybuf,
	//         ( const void __user* )p_memory->tdata.n_buf_addr );
	if (copy_from_user_ex(p_buf_header->c_bodybuf,
			      (const void __user *)p_memory->tdata.n_buf_addr,
			      p_memory->tdata.n_len)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex error .\n");
		n_ret = -EFAULT;
	}
	mutex_unlock(&p_mem_info->mem_info_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end .\n");
	VATOOLS_FUNC_EXIT;
	return n_ret;
}
static long sharedmem_check_reader(struct vatools_reader *reader)
{
	long ret = 0;
	struct vatools_node *p_current = NULL, *p_next = NULL;
	struct vatools_reader *p_current_reader = NULL, *p_next_reader = NULL;

	// struct list_head* ptr = NULL;

	//查询节点是否存在
	list_for_each_entry_safe (p_current, p_next, vatools_get_vastai_head(),
				  list_nodes) {
		/*
                list_for_each( ptr, &p_current->list_node_readers )
                {
                    if ( NULL == ptr || NULL == ptr->next || NULL == ptr->prev )
                    {
                        VATOOLS_DBG( "sharedmem_get_memptr_by_channel " );
                        dump_stack();
                        return -EFAULT;
                    }
                }
        */
		// mutex_lock( &p_current->node_mutex );
		list_for_each_entry_safe (p_current_reader, p_next_reader,
					  &p_current->list_node_readers,
					  list_readers) {
			if (reader == p_current_reader) {
				ret = 1;
				break;
			}
		}
		// mutex_unlock( &p_current->node_mutex );
	}
	return ret;
}
static long sharedmem_poll_buf(struct vatools_reader *reader,
			       T_MEMORY_ENTRY *p_memory)
{
	T_MEM_INFO *p_node = NULL, *p_tmp = NULL;
	struct vatools_node *p_dev = NULL;
	u32 timeout_water = 3000; //循环等待3s后执行
	long ret = 0;
	T_SHAREDMEM_BUF_INFO *p_bufIn = NULL;
	void __user *p_user_buf = NULL;
	//获取业务等待的句柄
	struct vatools_reader *reader_data = NULL;

	T_SHAREDMEM_BUF_INFO *p_buf =
		(T_SHAREDMEM_BUF_INFO *)p_memory->tdata.n_buf_addr;
	if (NULL == p_memory || NULL == reader) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin . addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		p_memory->tdata.n_buf_addr, p_memory->tdata.n_len,
		p_memory->n_shard_addr, p_memory->tdata.n_type);
	//取出根节点
	p_dev = reader->node;

	//设置超时机制
	p_memory->cmd_type = 0;

	ret = wait_for_completion_interruptible_timeout(
		&(reader->debug_comp), msecs_to_jiffies(timeout_water));
	reinit_completion(&(reader->debug_comp));

	if (ret == 0) {
		p_memory->cmd_type = VATOOLS_MEM_CMD_TIMEOUT;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "timeout! (%d ms) ret=%ld\n",
			    timeout_water, ret);
		ret = -EIO;
		goto END;
	} else if (ret == -ERESTARTSYS || ret == -EINTR) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "process be killed! ret=%ld\n",
			    ret);
		ret = -EINTR;
		goto END;
	} else if (ret < 0) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "Unknow err! ret=%ld\n", ret);
		ret = -EIO;
		goto END;
	}

	mutex_lock(&p_dev->node_mutex);
	mutex_lock(&p_dev->node_sharedmem_mutex);
	ret = 0;
	// goto POLL_END;

	if (0 == sharedmem_check_reader(reader)) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "reader is  no find\n");
		goto POLL_END;
	}

	if (1 == reader->exit) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "recv other_reader is  exit\n");
		goto POLL_END;
	}
	// goto POLL_END;
	//查找对应的业务节点
	//  pr_info( "sharedmem_poll_buf ptr=%p\n", reader );
	reader_data = sharedmem_get_other_reader(reader);
	if (NULL == reader_data) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "on find the file node %d\n",
			    reader->trans_category.device.die_index);
		goto POLL_END;
	}
	if (reader_data->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"reader->n_sharedmem_init_done ! = VATOOLS_INIT_DONE\n");
		goto POLL_END;
	}
	p_user_buf = (void __user *)p_buf->c_buf;
	mutex_lock(&reader_data->sharedmem_mutex);
	list_for_each_entry_safe (p_node, p_tmp,
				  &reader_data->t_sharedmem_mem_info_head,
				  t_sharedmem_node) {
		if (1 == p_node->tvideo_debugger.needsendmsg) {
			//如果有数据就表示本次操作是有效的。
			p_memory->cmd_type = 0;
			p_memory->callback = p_node->callback; //回调函数
			p_memory->tchannel.id =
				p_node->tchannel.id; //对应的通道id
			p_memory->tchannel.type =
				p_node->tchannel.type; //对应的通道类型
			p_memory->tchannel.writetype =
				p_node->tvideo_debugger.startflag; //开始结束标志
			p_memory->n_user_addr = p_node->user_data; //用户数据
			p_memory->tdata.n_type =
				p_node->tvideo_debugger
					.debugtype; //本次debugger数据的
			p_memory->n_shard_addr =
				(u64)p_node; //共享内存所对应的地址
			// user 数据拷贝
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "p_node->tvideo_debugger.debuginfo=%p\n",
				    p_node->tvideo_debugger.debuginfo);
			p_bufIn = (T_SHAREDMEM_BUF_INFO *)
					  p_node->tvideo_debugger.debuginfo;
			if (1 == p_node->tvideo_debugger.startflag &&
			    0 != p_bufIn) {
				VATOOLS_DUMP_BRIEF(
					"tvideo_debugger.debuginfo\n", p_bufIn,
					sizeof(T_SHAREDMEM_BUF_INFO) +
						p_bufIn->n_len);
				VATOOLS_DBG(
					NULL, DUMMY_DIE_ID,
					"p_bufIn->c_buf %p p_bufIn->n_len=%d\n",
					p_bufIn->c_buf, p_bufIn->n_len);
				if (0 != p_bufIn->n_len) {
					// pr_info( "#%d %scopy_to_user=%p %p\n", __LINE__, __FUNCTION__, p_user_buf, p_bufIn->c_buf );
					if (copy_to_user_ex(
						    (void __user *)p_user_buf,
						    p_bufIn->c_buf,
						    p_bufIn->n_len)) {
						VATOOLS_ERR(
							NULL, DUMMY_DIE_ID,
							"copy_to_user_ex error .\n");
						ret = -EFAULT;
						goto READ_END;
					}
				}
				if (copy_to_user_ex((void __user *)&p_buf->n_len,
						    &p_bufIn->n_len,
						    sizeof(p_bufIn->n_len))) {
					VATOOLS_ERR(
						NULL, DUMMY_DIE_ID,
						"copy_to_user_ex error .\n");
					ret = -EFAULT;
					goto READ_END;
				}
			}

			//表示已经发送数据给上层
			p_node->tvideo_debugger.needsendmsg = 0;
			break;
		}
	}
READ_END:
	mutex_unlock(&reader_data->sharedmem_mutex);
POLL_END:
	mutex_unlock(&p_dev->node_sharedmem_mutex);
	mutex_unlock(&p_dev->node_mutex);
END:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end\n");
	VATOOLS_FUNC_EXIT;
	return ret;
}
static long sharedmem_start_video_debugger(struct vatools_reader *reader,
					   T_MEMORY_ENTRY *p_memory)
{
	T_MEM_INFO *p_mem_info = NULL;
	T_MEMORY_ENTRY tMemEntry;
	int n_ret = 0;
	struct vatools_reader *reader_wait = NULL;
	T_SHAREDMEM_BUF_INFO *p_buf = NULL;
	if (NULL == p_memory || NULL == reader) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return -EFAULT;
	}
	// return 0;
	VATOOLS_FUNC_ENTERY;
	memcpy(&tMemEntry, p_memory, sizeof(T_MEMORY_ENTRY));
	//取业务节点
	tMemEntry.tchannel.writetype = VATOOLS_SHAREDMEM_WRITE_DATA;
	p_mem_info = sharedmem_get_memptr(NULL, &tMemEntry);
	if (NULL == p_mem_info) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		n_ret = -EFAULT;
		goto SHARED_END;
	}
	//找到业务节点 找到 fdtype
	if (p_mem_info->tvideo_debugger.pmeminfo) {
		// pr_info( "sharedmem_start_video_debugger ptr=%p\n", p_mem_info->reader );
		reader_wait = sharedmem_get_other_reader(p_mem_info->reader);
		if (NULL == reader_wait) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "no find wait for thread\n");
			n_ret = -EFAULT;
			goto SHARED_END;
		}
		if (1 == p_mem_info->tvideo_debugger.startflag) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "the channel is starting debugger\n");
			n_ret = -EEXIST;
			goto SHARED_END;
		}
		//创建debugger相关数据
		p_mem_info->tvideo_debugger.debuginfo = kvmalloc(
			sizeof(T_SHAREDMEM_BUF_INFO) + p_memory->tdata.n_len,
			GFP_KERNEL);
		if (NULL == p_mem_info) {
			VATOOLS_INFO(NULL, DUMMY_DIE_ID,
				     "kvmalloc is error.\n");
			n_ret = -EFAULT;
			goto SHARED_END;
		}
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_mem_info->tvideo_debugger.debuginfo=%p\n",
			    p_mem_info->tvideo_debugger.debuginfo);
		p_mem_info->tvideo_debugger.debugtype = p_memory->tdata.n_type;
		//对数据的内容进行赋值
		p_buf = (T_SHAREDMEM_BUF_INFO *)
				p_mem_info->tvideo_debugger.debuginfo;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_memory->tdata.n_len=%d\n",
			    p_memory->tdata.n_len);
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "p_memory->tdata.n_buf_addr=0x%llx\n",
			    p_memory->tdata.n_buf_addr);

		if (p_memory->tdata.n_len > 0) {
			// pr_info( "#%d %s copy_from_user_ex=%p %p\n", __LINE__, __FUNCTION__, p_buf->c_buf,
			//         ( const void __user* )p_memory->tdata.n_buf_addr );
			if (copy_from_user_ex(p_buf->c_buf,
					      (const void __user *)p_memory
						      ->tdata.n_buf_addr,
					      p_memory->tdata.n_len)) {
				VATOOLS_ERR(NULL, DUMMY_DIE_ID,
					    "copy_from_user_ex error .\n");
				n_ret = -EFAULT;
				kvfree(p_mem_info->tvideo_debugger.debuginfo);
				p_mem_info->tvideo_debugger.debuginfo = NULL;
				goto SHARED_END;
			}
		}

		p_buf->n_len = p_memory->tdata.n_len;
		VATOOLS_DUMP_BRIEF(">tvideo_debugger.debuginfo", p_buf,
				   sizeof(T_SHAREDMEM_BUF_INFO) +
					   p_memory->tdata.n_len);
		//设置开始标志位
		p_mem_info->tvideo_debugger.needsendmsg = 1;
		p_mem_info->tvideo_debugger.startflag = 1;
		//指针数据带回给业务
		p_memory->n_shard_addr = (u64)p_mem_info;
		// complete( &reader_wait->debug_comp );
		sharemen_set_comp_complete(reader_wait);
	} else {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the pmeminfo  is null\n");
	}
	//为了stop时能够释放对应的资源
	n_ret = sharedmem_create_buf(reader, p_memory, 1);

SHARED_END:
	VATOOLS_FUNC_EXIT;

	//创建自己的共享内存数据。用于释放时通知业务
	return n_ret;
}
static long sharedmem_stop_video_debugger(struct vatools_reader *reader,
					  T_MEMORY_ENTRY *p_memory)
{
	T_MEM_INFO *p_mem_info = NULL;
	T_MEMORY_ENTRY tMemEntry;
	int n_ret = 0;
	if (NULL == p_memory || NULL == reader) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	memcpy(&tMemEntry, p_memory, sizeof(T_MEMORY_ENTRY));
	//取业务节点
	tMemEntry.tchannel.writetype = VATOOLS_SHAREDMEM_WRITE_DATA;

	p_mem_info = sharedmem_get_memptr(NULL, &tMemEntry);
	if (NULL == p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "no find  memory info\n");
		n_ret = -EFAULT;
		goto SHARED_END;
	}
	// pr_info( "sharedmem_stop_video_debugger" );
	sharedmen_free_video_debugbuf(&p_mem_info->tvideo_debugger);
	n_ret = sharedmem_delete_buf(reader, p_memory, 1);
SHARED_END:
	VATOOLS_FUNC_EXIT;
	return n_ret;
}
static long sharedmem_read_video_debugger(struct vatools_reader *reader,
					  T_MEMORY_ENTRY *p_memory)
{
	//需要添加一个全局的锁
	t_video_debug_info *p_current = NULL, *p_next = NULL;
	T_MEM_INFO *p_mem_info = NULL;
	void __user *p_user_buf = NULL;
	T_MEMORY_ENTRY tMemEntry;
	int n_ret = 0;
	int n_len_cnt = 0;
	T_SHAREDMEM_BUF_INFO *p_buf =
		(T_SHAREDMEM_BUF_INFO *)p_memory->tdata.n_buf_addr;
	int n_max_len = p_memory->tdata.n_len;
	if (NULL == p_memory || NULL == reader || NULL == p_buf) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	memcpy(&tMemEntry, p_memory, sizeof(T_MEMORY_ENTRY));
	//取业务节点
	tMemEntry.tchannel.writetype = VATOOLS_SHAREDMEM_WRITE_DATA;
	p_mem_info = sharedmem_get_memptr(NULL, &tMemEntry);
	if (NULL == p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info is null\n");
		n_ret = -EFAULT;
		goto SHARED_END;
	}
	p_user_buf = p_buf->c_buf;
	//获取video相关数据
	list_for_each_entry_safe (p_current, p_next,
				  &p_mem_info->tvideo_debugger.t_node_head,
				  t_video_node) {
		T_SHAREDMEM_BUF_HEADER *p_buf_header =
			(T_SHAREDMEM_BUF_HEADER *)p_current->p_buf;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "n_len_cnt=%d\n", n_len_cnt);
		if (n_max_len <= n_len_cnt + p_current->n_effic_len) {
			VATOOLS_DBG(NULL, DUMMY_DIE_ID,
				    "the memory too large please get again");
			n_ret = n_len_cnt;
			goto SHARED_END;
		}
		// pr_info( "#%d %s copy_to_user_ex=%p %p\n", __LINE__, __FUNCTION__, p_user_buf + n_len_cnt,
		//          p_buf_header->c_bodybuf );
		if (p_buf_header->c_bodybuf) {
			if (copy_to_user_ex((u8 *)p_user_buf + n_len_cnt,
					    p_buf_header->c_bodybuf,
					    p_current->n_effic_len)) {
				VATOOLS_ERR(NULL, DUMMY_DIE_ID,
					    "copy_to_user_ex error .\n");
				n_ret = n_len_cnt;
				goto SHARED_END;
			}
		}

		n_len_cnt += p_current->n_effic_len;
		//先重队列中取下来
		list_del(&p_current->t_video_node);
		//再释放空间
		kvfree(p_current->p_buf);
		kvfree(p_current);
	}
	if (copy_to_user_ex((void __user *)&p_buf->n_len, &n_len_cnt,
			    sizeof(p_buf->n_len))) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex error .\n");
		return -EFAULT;
	}
SHARED_END:
	VATOOLS_FUNC_EXIT;
	return n_ret;
}
static long sharedmem_write_video_debugger(struct vatools_reader *reader,
					   T_MEMORY_ENTRY *p_memory)
{
	t_video_debug_info *pdebuginfo = NULL;
	T_SHAREDMEM_BUF_HEADER *p_buf_header = NULL;
	T_MEM_INFO *p_mem_info = NULL;
	int nstartflag = 0;
	T_MEMORY_ENTRY tMemEntry;
	if (NULL == p_memory || NULL == reader) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		return -EFAULT;
	}
	VATOOLS_FUNC_ENTERY;
	memcpy(&tMemEntry, p_memory, sizeof(T_MEMORY_ENTRY));

	//取业务节点 内存挂载在业务节点下面
	tMemEntry.tchannel.writetype = VATOOLS_SHAREDMEM_WRITE_DATA;
	p_mem_info = sharedmem_get_memptr(NULL, &tMemEntry);
	if (NULL == p_mem_info) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "the memory info is null\n");
		goto ERR_MEM_INFO;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "p_mem_info->tchannel.id=%d,type=%d,writetype=%d\n",
		    p_mem_info->tchannel.id, p_mem_info->tchannel.type,
		    p_mem_info->tchannel.writetype);

	//当出现startflag为0 时 不需要写入数据了
	mutex_lock(&p_mem_info->mem_info_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "p_mem_info  %p=lock\n", p_mem_info);
	nstartflag = p_mem_info->tvideo_debugger.startflag;
	mutex_unlock(&p_mem_info->mem_info_mutex);
	if (0 == nstartflag) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "the memory entry is null\n");
		goto ERR_MEM_INFO;
	}
	pdebuginfo = (t_video_debug_info *)kvmalloc(sizeof(t_video_debug_info),
						    GFP_KERNEL);
	if (NULL == p_mem_info) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "kvmalloc is error.\n");
		goto ERR_MEM_INFO;
	}
	pdebuginfo->p_buf =
		kvmalloc(p_memory->tdata.n_len + sizeof(u64), GFP_KERNEL);
	if (!pdebuginfo->p_buf) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, " kvmalloc error. size=%ld\n",
			     p_memory->tdata.n_len + sizeof(u64));
		goto ERR_MEM_INFO_BUF;
	}
	INIT_LIST_HEAD(&pdebuginfo->t_video_node);
	pdebuginfo->n_effic_len = p_memory->tdata.n_len;
	p_buf_header = (T_SHAREDMEM_BUF_HEADER *)pdebuginfo->p_buf;
	p_buf_header->n_addr = (u64)p_memory;
	if (copy_from_user_ex(p_buf_header->c_bodybuf,
			      (const void __user *)p_memory->tdata.n_buf_addr,
			      p_memory->tdata.n_len)) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex error .\n");
		goto ERR_MEM_INFO_TYPE;
	}
	mutex_lock(&p_mem_info->tvideo_debugger.data_info_mutex);
	p_mem_info->tvideo_debugger.data_cnt++;
	list_add_tail(&p_mem_info->tvideo_debugger.t_node_head,
		      &pdebuginfo->t_video_node);
	while (p_mem_info->tvideo_debugger.data_cnt > 100) {
		//删除第一个元素
#if (TOOLS_WIN == 1)
		list_del(p_mem_info->tvideo_debugger.t_node_head.Flink);
#else
		list_del(p_mem_info->tvideo_debugger.t_node_head.next);
#endif
		p_mem_info->tvideo_debugger.data_cnt--;
	}
	mutex_unlock(&p_mem_info->tvideo_debugger.data_info_mutex);
	VATOOLS_FUNC_EXIT;
	return 0;

ERR_MEM_INFO_TYPE:
	kvfree(pdebuginfo->p_buf);
ERR_MEM_INFO_BUF:
	pdebuginfo->p_buf = NULL;
	kvfree(pdebuginfo);
ERR_MEM_INFO:
	pdebuginfo = NULL;
	VATOOLS_FUNC_EXIT;

	return -EFAULT;
}

loff_t sharedmem_llseek(struct file *filp, loff_t offset, int orig)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(offset);
	V_UNREFERENCE(orig);

	return 0;
}
ssize_t sharedmem_read(struct file *filp, char __user *buffer, size_t size,
		       loff_t *ppos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buffer);
	V_UNREFERENCE(size);
	V_UNREFERENCE(ppos);

	return 0;
}
ssize_t sharedmem_write(struct file *filp, const char __user *buffer,
			size_t size, loff_t *ppos)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(buffer);
	V_UNREFERENCE(size);
	V_UNREFERENCE(ppos);

	return 0;
}
unsigned int sharedmem_poll(struct file *filp, struct poll_table_struct *wait)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(wait);

	return 0;
}
/*
static long sharedmem_create_video_debugger_func( struct vatools_reader* reader, T_MEMORY_ENTRY* p_memory )
{
    long        n_ret      = 0;
    T_MEM_INFO* p_mem_info = NULL;
    if ( NULL == p_memory || NULL == reader )
    {
        VATOOLS_INFO( "the memory entry is null" );
        return -EFAULT;
    }
    p_mem_info = sharedmem_get_memptr( reader, p_memory );

    if ( !p_mem_info )
    {
        VATOOLS_INFO( "p_mem_info is null ." );
        return -EFAULT;
    }
    mutex_lock( &p_mem_info->mem_info_mutex );
    p_mem_info->tchannel.id   = p_memory->tchannel.id;
    p_mem_info->tchannel.type = p_memory->tchannel.type;
    p_mem_info->callback      = p_memory->callback;
    mutex_unlock( &p_mem_info->mem_info_mutex );
    return 0;
}

static long sharedmem_delete_video_debugger_func( struct vatools_reader* reader, T_MEMORY_ENTRY* p_memory )
{
    long        n_ret      = 0;
    T_MEM_INFO* p_mem_info = NULL;
    if ( NULL == p_memory || NULL == reader )
    {
        VATOOLS_INFO( "the memory entry is null" );
        return -EFAULT;
    }
    p_mem_info = sharedmem_get_memptr( reader, p_memory );

    if ( !p_mem_info )
    {
        VATOOLS_INFO( "p_mem_info is null ." );
        return -EFAULT;
    }
    mutex_lock( &p_mem_info->mem_info_mutex );
    p_mem_info->tchannel.id   = -1;
    p_mem_info->tchannel.type = -1;
    p_mem_info->callback      = NULL;
    mutex_unlock( &p_mem_info->mem_info_mutex );
    return 0;
}
*/
long sharedmem_ioctl(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg)
{
	long n_ret = -EIO;
	T_MEMORY_ENTRY t_memory = { 0 };
	struct vatools_node *p_dev = NULL;
	struct vatools_reader *reader = NULL;
	void __user *p_argp = (void __user *)arg;
	//int                    pull     = 0;
	VATOOLS_FUNC_ENTERY;

	reader = vatools_file_get_reader(filp);
	if (NULL == reader) {
		// pr_info( " NULL == reader  ." );
		n_ret = -EFAULT;
		goto SHARED_END;
	}
	p_dev = vatools_file_get_node(filp);
	mutex_lock(&p_dev->node_mutex);
	mutex_lock(&p_dev->node_sharedmem_mutex);
	VATOOLS_DBG(
		NULL, DUMMY_DIE_ID,
		"begin cmd=0x%x  p_dev=0x%p  reader=0x%p  &reader->sharedmem_mutex=0x%p\n",
		cmd, p_dev, reader, &reader->sharedmem_mutex);

	if (copy_from_user_ex(&t_memory, p_argp, sizeof(T_MEMORY_ENTRY))) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_from_user_ex error .\n");
		n_ret = -EFAULT;
		goto SHARED_MUTEX_END;
	}

	//如果没有初始化好久不让业务处理
	if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID,
			     "n_sharedmem_init_done is not inited .\n");
		n_ret = -EFAULT;
		goto SHARED_MUTEX_END;
	}
	mutex_unlock(&p_dev->node_sharedmem_mutex);
	mutex_unlock(&p_dev->node_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID,
		    "addr=0x%llx  len=%d  shared_addr=0x%llx  type=%d\n",
		    t_memory.tdata.n_buf_addr, t_memory.tdata.n_len,
		    t_memory.n_shard_addr, t_memory.tdata.n_type);

	switch (cmd) {
	//创建共享内存
	case VATOOLS_CREATE_SHARE_MEM:
	case OLD_VATOOLS_CREATE_SHARE_MEM:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_create_buf(reader, &t_memory, 0);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//删除共享内存
	case VATOOLS_DELETE_SHARE_MEM:
	case OLD_VATOOLS_DELETE_SHARE_MEM:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_delete_buf(reader, &t_memory, 0);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//读取所有共享内存数据
	case VATOOLS_READ_SHARE_MEM:
	case OLD_VATOOLS_READ_SHARE_MEM:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_read_all(&t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//写入共享内存
	case VATOOLS_WRITE_SHARE_MEM:
	case OLD_VATOOLS_WRITE_SHARE_MEM:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_write_buf(reader, &t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//监听共享内存 lupa 2021-12-15 add
	case VATOOLS_POLL_SHARE_MEM:
	case OLD_VATOOLS_POLL_SHARE_MEM:
		//pull  = 1;
		n_ret = sharedmem_poll_buf(reader, &t_memory);
		break;
	//开始监听video debugger数据 lupa 2021-12-15 add
	case VATOOLS_SHARE_MEM_VIDEO_START_DEBUGGER:
	case OLD_VATOOLS_SHARE_MEM_VIDEO_START_DEBUGGER:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_start_video_debugger(reader, &t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//停止debugger lupa 2021-12-15 add
	case VATOOLS_SHARE_MEM_VIDEO_STOP_DEBUGGER:
	case OLD_VATOOLS_SHARE_MEM_VIDEO_STOP_DEBUGGER:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_stop_video_debugger(reader, &t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//读取debugger信息 lupa 2021-12-15 add
	case VATOOLS_SHARE_MEM_VIDEO_READ_DEBUGGER:
	case OLD_VATOOLS_SHARE_MEM_VIDEO_READ_DEBUGGER:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_read_video_debugger(reader, &t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	//写入denbugger信息 lupa 2021-12-15 add
	case VATOOLS_SHARE_MEM_VIDEO_WRITE_DEBUGGER:
	case OLD_VATOOLS_SHARE_MEM_VIDEO_WRITE_DEBUGGER:
		mutex_lock(&p_dev->node_mutex);
		mutex_lock(&p_dev->node_sharedmem_mutex);
		n_ret = sharedmem_write_video_debugger(reader, &t_memory);
		mutex_unlock(&p_dev->node_sharedmem_mutex);
		mutex_unlock(&p_dev->node_mutex);
		break;
	// case VATOOLS_CREATE_VIDEO_MEM_FUNCTION:
	//     n_ret = sharedmem_create_video_debugger_func( reader, &t_memory );
	//     break;
	// case VATOOLS_DELETE_VIDEO_MEM_FUNCTION:
	//     n_ret = sharedmem_delete_video_debugger_func( reader, &t_memory );
	//     break;
	default:
		n_ret = E_UNKNOWN_SUB_CMD;
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "unknown oictl:0x%x.\n", cmd);
		break;
	}
	if (copy_to_user_ex((void __user *)p_argp, &t_memory,
			    sizeof(T_MEMORY_ENTRY))) {
		VATOOLS_ERR(NULL, DUMMY_DIE_ID, "copy_to_user_ex error .\n");
		n_ret = -EFAULT;
		goto SHARED_END;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end cmd=0x%x. ret=%ld\n", cmd, n_ret);
	VATOOLS_FUNC_EXIT;
	//正常退出
	return n_ret;
SHARED_MUTEX_END:
	mutex_unlock(&p_dev->node_sharedmem_mutex);
	mutex_unlock(&p_dev->node_mutex);
SHARED_END:
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end cmd=0x%x. ret=%ld\n", cmd, n_ret);
	VATOOLS_FUNC_EXIT;
	return n_ret;
}

int sharedmem_open(struct inode *inode, struct file *filp)
{
	V_UNREFERENCE(inode);
	V_UNREFERENCE(filp);
#if 0
    struct vatools_node*   p_dev = vatools_file_get_node( filp );
    struct vatools_node*   anonymous_dev;
    int                    ret;
    struct vatools_reader* reader;
    VATOOLS_DBG( "begin " );

    ret = nonseekable_open( inode, filp );
    if ( ret )
    {
        return ret;
    }

    anonymous_dev = kvmalloc( sizeof( *p_dev ), GFP_KERNEL );
    if ( !anonymous_dev )
    {
        VATOOLS_INFO( "kvmalloc error . size=%d\n", sizeof( *p_dev ) );
        return -ENOMEM;
    }
    memcpy( anonymous_dev, p_dev, sizeof( *p_dev ) );
    INIT_LIST_HEAD( &anonymous_dev->list_nodes );
    INIT_LIST_HEAD( &anonymous_dev->t_sharedmem_mem_info_head );
    INIT_LIST_HEAD( &anonymous_dev->t_sharedmem_signal_info_head );
    mutex_init( &anonymous_dev->sharedmem_mutex );
    mutex_lock( &p_dev->sharedmem_mutex );
    list_add_tail( &anonymous_dev->list_nodes, vatools_get_vastai_head() );
    mutex_unlock( &p_dev->sharedmem_mutex );

    // filp->private_data = anonymous_dev;
    reader       = vatools_file_get_reader( filp );
    reader->node = anonymous_dev;
#endif
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end\n");
	return 0;
}
int sharedmem_fasync(int fd, struct file *filp, int mode)
{
	struct vatools_reader *reader = NULL;
	reader = vatools_file_get_reader(filp);

	return fasync_helper(fd, filp, mode, &reader->sharedmem_pt_fasync);
}

int sharedmen_free_buf(struct vatools_reader *reader)
{
	T_MEM_INFO *p_current = NULL, *p_tmp = NULL;
	struct vatools_reader *reader_other = NULL;
	// struct list_head*      ptr          = NULL;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "begin.\n");
	if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID,
			    "n_sharedmem_init_done is not int_done.\n");
		return 0;
	}

	/*
    list_for_each( ptr, &reader->t_sharedmem_mem_info_head )
    {
        if ( NULL == ptr || ptr->next == NULL || NULL == ptr->prev )
        {
            VATOOLS_DBG( "t_sharedmem_mem_info_head " );
            dump_stack();
            return NULL;
        }
    }
    */
	mutex_lock(&reader->sharedmem_mutex);
	list_for_each_entry_safe (p_current, p_tmp,
				  &reader->t_sharedmem_mem_info_head,
				  t_sharedmem_node) {
		VATOOLS_DBG(
			NULL, DUMMY_DIE_ID,
			"list_for_each_entry_safe.t_sharedmem_mem_info_head\n");
		list_del(&p_current->t_sharedmem_node);
		//先释放debugger数据
		sharedmen_free_video_debugbuf(&p_current->tvideo_debugger);
		p_current->tvideo_debugger.pmeminfo = NULL;
		p_current->tvideo_debugger.debuginfo = NULL;
		kvfree(p_current->p_buf);
		kvfree(p_current);
	}
	mutex_unlock(&reader->sharedmem_mutex);
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "complete( &reader->debug_comp )\n");
	//释放资源 --该资源对应的数据
	//资源都释放了，标志位回执
	reader->n_sharedmem_init_done = VATOOLS_UNINIT;
	//只有是0 的进程退出时才需要通知相对进行
	if (0 == reader->exit) {
		// pr_info( "sharedmen_free_buf ptr=%p\n", reader );
		reader_other = sharedmem_get_other_reader(reader);
		if (NULL != reader_other) {
			// pr_info( "sharedmen_free_buf reader_other=%p\n", reader_other );
			reader_other->exit = 1;
			sharemen_set_comp_complete(reader_other);
			// reinit_completion( &( reader_other->debug_comp ) );
		}
	}

	// list_for_each_entry_safe( p_current, p_tmp, &reader->t_sharedmem_signal_info_head, t_sharedmem_node )
	//{
	//     VATOOLS_DBG( "list_for_each_entry_safe.t_sharedmem_signal_info_head " );
	//     kvfree( p_current->p_buf );
	//     list_del( &p_current->t_sharedmem_node );
	//     kvfree( p_current );
	// }
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end .\n");
	return 0;
}
int sharedmem_release(struct inode *inode, struct file *filp)
{
	struct vatools_node *p_dev = NULL;
	struct vatools_reader *reader = NULL;

	V_UNREFERENCE(inode);
	p_dev = vatools_file_get_node(filp);
	if (!p_dev) {
		VATOOLS_INFO(NULL, DUMMY_DIE_ID, "private_data is null.\n");
		return -ENOMEM;
	}
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_lock begin.\n");
	// mutex_lock( &p_dev->node_sharedmem_mutex );
	reader = vatools_file_get_reader(filp);
	if (reader) {
		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "begin.\n");
		// pr_info( "sharedmem_release reader=%p\n", reader );
		list_del(&reader->list_readers);
		//   p_dev = container_of(inode->i_cdev, struct vatools_node, t_cdev);
		sharedmen_free_buf(reader);
		kvfree(reader);
		filp->private_data = NULL;
	} else {
		// pr_info( "sharedmem_release  is null " );
	}

	// mutex_unlock( &p_dev->node_sharedmem_mutex );
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "mutex_lock end.\n");
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end.\n");
	// pr_info( "sharedmem_release end  " );
	return 0;
}

ssize_t sharedmem_write_iter(struct kiocb *iocb, struct iov_iter *from)
{
	V_UNREFERENCE(iocb);
	V_UNREFERENCE(from);

	return 0;
}

int sharedmem_mmap(struct file *filp, struct vm_area_struct *pvm)
{
	V_UNREFERENCE(filp);
	V_UNREFERENCE(pvm);

	return 0;
}

int sharedmem_set_app_category(struct file *filp, unsigned int cmd,
			       IOCTL_ARG_T arg)
{
	struct vatools_node *p_dev = NULL;
	struct vatools_reader *reader = NULL;
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "VATOOLS_APP_SHAREDMEM  cmd=0x%x\n",
		    cmd);

	V_UNREFERENCE(cmd);
	V_UNREFERENCE(arg);
	p_dev = vatools_file_get_node(filp);
	reader = vatools_file_get_reader(filp);
	// mutex_lock( &p_dev->node_sharedmem_mutex );
	//  TODO: 检测不能重复初始化.
	if (reader->n_sharedmem_init_done != VATOOLS_INIT_DONE) {
		// TODO: reader 已经启用新的了， 没有必要在这里启动anonymous_dev了。
		INIT_LIST_HEAD(&reader->t_sharedmem_mem_info_head);
		// INIT_LIST_HEAD( &reader->t_sharedmem_signal_info_head );
		mutex_init(&reader->sharedmem_mutex);
		//初始化信号量 lupa 2021-12-15 add
		init_completion(&reader->debug_comp);
		// mutex_lock( &p_dev->sharedmem_mutex );
		// list_add_tail( &anonymous_dev->list_nodes, vatools_get_vastai_head() );
		// mutex_unlock( &p_dev->sharedmem_mutex );
		// filp->private_data = anonymous_dev;
		// reader->node = anonymous_dev;
		reader->n_sharedmem_init_done = VATOOLS_INIT_DONE;
		reader->exit = 0;

		reader->filter.filter_info.version =
			(SHAREMEM_VERSION_SG100 << SHAREMEM_VERSION_OFFSET) |
			SHAREMEM_VERSION_INITIAL;
		reader->filter.filter_info.head.nDieID = 0x55AA55AB;
		reader->filter.filter_info.head.nDeviceID = 0x55AA55AB;
		reader->filter.filter_info.vpid = task_tgid_vnr(current);
		reader->filter.filter_info.ns =
			(u64)task_active_pid_ns(current);
		reader->filter.filter_info.level =
			(u32)task_active_pid_ns(current)->level;

		VATOOLS_DBG(NULL, DUMMY_DIE_ID, "vpid %d, ns %lld, level %d\n",
			    reader->filter.filter_info.vpid,
			    reader->filter.filter_info.ns,
			    reader->filter.filter_info.level);
	}
	// mutex_unlock( &p_dev->node_sharedmem_mutex );
	VATOOLS_DBG(NULL, DUMMY_DIE_ID, "end\n");
	return 0;
}
