﻿
#ifndef __VATOOLS_SHAREDMEM_H__
#define __VATOOLS_SHAREDMEM_H__

#pragma pack(push)
#pragma pack(1)
#ifdef __cplusplus
extern "C" {
#endif

#define SHAREDMEM_TOOLS_MAX_DEVICES 1

enum enum_sharedmem_write_type {
	VATOOLS_SHAREDMEM_WRITE_DATA = 0, //正常的数据
	VATOOLS_SHAREDMEM_WRITE_SINGAL = 1, //信号数据
	VATOOLS_SHAREDMEM_WRITE_DEBUG = 2, // debug数据
};

typedef struct tar_data_info {
	u32 n_type; //数据类型
	u32 n_len; //数据长度
	u64 n_buf_addr; //数据地址
} t_data_info;
typedef struct tar_channel_info {
	int id; // channel id
	int type; // channel type : decode encode
	int writetype; //对该通道的写操作类型 详见 enum_sharedmem_write_type
} t_channle_info;
typedef struct memory_entry {
	int cmd_type; //操作类型
	t_channle_info tchannel; //通道信息。
	u64 n_shard_addr; //开辟空间位置
	u64 n_user_addr; //用户回传数据
	t_data_info tdata; //数据保存区域
	// void *callback; //回调函数
	u64 callback; //回调函数

	/// 应用调用输入参数
	u32 version;                // 版本号, 现在写0
	u8  flag_encode_or_decode;  // 0:编码  1:解码
	u8  flag_specify_die;       /* 0: 不指定 1: specify die */
	u8  flag_allocate_policy;   // 分配策略方式: 0: 取最小资源占用die
	u8  flag_rsvd;
	u32 die_seq;
	u32 die_weight; /* allocated die weight */
} T_MEMORY_ENTRY;

typedef struct tar_video_debug_info {
	struct list_head t_video_node;
	int n_effic_len; //数据区有效数据长度
	void *p_buf; //真实共享数据
} t_video_debug_info;

typedef struct t_video_debug_header {
	int is_debugbuf; //该区域是否是debug区域
	int needsendmsg; //是否需要发送消息
	int startflag; // debugger启动标志
	int debugtype; // debug的数据
	int data_cnt; //一个有多少块数据
	int rest; //预留字段
	struct mutex data_info_mutex; //数据区内部锁
	struct list_head
		t_node_head; // 用于video debugger时的信息记录 lupa add 2021-12-15
	struct t_mem_info *pmeminfo; //该数据所对应的头部指针
	void *debuginfo; //用于存放debugger相关数据
} t_video_debug_head;

#if (TOOLS_WIN == 1)
// TODO: tobedone: need check this
// remove compile warning: Warning	C4366	The result of the unary '&' operator may be unaligned
#pragma pack(8)
#endif
//共享内存的基本单元。每个通道一个
typedef struct t_mem_info {
	t_channle_info tchannel; //通道信息。
	int rest; //预留字段
	u64 user_data; //用于存放用户对应的数据指针
	struct list_head t_sharedmem_node;
	struct mutex mem_info_mutex; //数据区内部锁
	int n_effic_len; //数据区有效数据长度
	t_video_debug_head tvideo_debugger; //用于记录videodebugger的数据
	void *p_buf; //真实共享数据
	// void *callback; //回调函数
	u64 callback; //回调函数
	struct vatools_reader *reader; //把对应的父节点
	struct vatools_reader *waitreader; //把对应的父节点
} T_MEM_INFO;

#pragma pack(1)

typedef struct tar_sharedmen_buf_info {
	int n_len;
	char c_buf[0];
} T_SHAREDMEM_BUF_INFO;

typedef struct tar_sharedmen_buf_header {
	u64 n_addr; //存放的是sharedmem的地址用于快速找到这个消息头
	char c_bodybuf[0];
} T_SHAREDMEM_BUF_HEADER;

//共享内存相关操作  begin  lupa add 2021-12-15
//暂未使用
loff_t sharedmem_llseek(struct file *filp, loff_t offset, int orig);
//暂未使用
ssize_t sharedmem_read(struct file *filp, char __user *buf, size_t size,
		       loff_t *pos);
//暂未使用
ssize_t sharedmem_write(struct file *filp, const char __user *buf, size_t size,
			loff_t *pos);
//暂未使用
ssize_t sharedmem_write_iter(struct kiocb *iocb, struct iov_iter *from);
//暂未使用
unsigned int sharedmem_poll(struct file *filp, struct poll_table_struct *wait);
//暂未使用
int sharedmem_mmap(struct file *filp, struct vm_area_struct *pvm);
//控制类数据 lupa modify 2021-12-15
long sharedmem_ioctl(struct file *filp, unsigned int cmd, IOCTL_ARG_T arg);
//暂未使用
int sharedmem_open(struct inode *inode, struct file *filp);
//资源释放
int sharedmem_release(struct inode *ignored, struct file *filp);
int sharedmem_fasync(int fd, struct file *filp, int mode);
//设置使用类型为sharedmemory
int sharedmem_set_app_category(struct file *filp, unsigned int cmd,
			       IOCTL_ARG_T arg);
int sharemen_set_comp_complete(struct vatools_reader *reader);
int sharedmen_free_video_debugbuf(t_video_debug_head *video_debug);
int sharedmen_free_buf(struct vatools_reader *reader);
//共享内存相关操作  end  lupa add 2021-12-15

#ifdef __cplusplus
}
#endif
#pragma pack(pop)
#endif //__VATOOLS_SHAREDMEM_H__
