﻿#ifndef __VATOOLS_SMI_CMD_H__
#define __VATOOLS_SMI_CMD_H__

#include "vatools_types.h"

#pragma pack(push)
#pragma pack(1)
#ifdef __cplusplus
extern "C" {
#endif

/* Write and bitset command test addreas is 0x08cf6cf0 */

// SMI 功能使用的csram首地址
#define SMI_DIE_ADDRESS_OFFSET_BASE ((u32)(0x08cf3600))

// block 数量
#define SMI_BLOCK_ID_MAX (8)

//初始化写入csram smi header区域
typedef struct SMI_BLOCK_HEADER_s {
	u32 smi_version;
	u32 block_header_num; //高8bit默认设置0xA1, 标识smi区域已经设置过.
	u32 block_address[SMI_BLOCK_ID_MAX];
} T_SMI_BLOCK_HEADER;

//每个block的地址和使用标记
typedef struct SMI_BLOCK_s {
	u32 block_id;
	u32 block_address;
	u32 block_len;
	u32 block_used_flag;
	u32 query_count;
	u32 query_interval;
} T_SMI_BLOCK;

#define SMI_CMD_BLOCK_LEN (1024) ///< block 总长度
#define SMI_CMD_HEADER_LEN                                                     \
	(sizeof(TS_SMI_CMD_REQ)) ///< smi 命令数据最大长度， 需要和 fw 配合修改
#define SMI_CMD_BODY_LEN	(1000) ///< smi 命令数据最大长度， 需要和 fw 配合修改
#define SMI_CMD_BLOCK0_LEN	(SMI_CMD_BLOCK_LEN) ///< SMI block0 默认总长度
#define SMI_CMD_BLOCK0_DATA_LEN (SMI_CMD_BODY_LEN) ///< SMI block0 默认数据长度
#define SMI_CMD_BLOCK0_RSVD	(8) ///< SMI block0 保留值
#define SMI_CMD_BLOCK0_ADDR                                                    \
	((u32)(SMI_DIE_ADDRESS_OFFSET_BASE +                                   \
	       sizeof(T_SMI_BLOCK_HEADER))) ///< SMI block0 默认地址

#define SMI_CMD_BLOCK1_LEN	(SMI_CMD_BLOCK_LEN) ///< SMI block1 默认总长度
#define SMI_CMD_BLOCK1_DATA_LEN (SMI_CMD_BODY_LEN) ///< SMI block1 默认数据长度
#define SMI_CMD_BLOCK1_RSVD	(8) ///< SMI block1 保留值
#define SMI_CMD_BLOCK1_ADDR                                                    \
	((u32)(SMI_CMD_BLOCK0_ADDR +                                           \
	       SMI_CMD_BLOCK0_LEN)) ///< SMI block1 默认地址

#define SMI_CMD_BLOCK2_ADDR ((u32)0) ///< SMI block2 默认总长度
#define SMI_CMD_BLOCK2_LEN  (0) ///< SMI block2 默认总长度
#define SMI_CMD_BLOCK3_ADDR ((u32)0) ///< SMI block3 默认总长度
#define SMI_CMD_BLOCK3_LEN  (0) ///< SMI block3 默认总长度
#define SMI_CMD_BLOCK4_ADDR ((u32)0) ///< SMI block4 默认总长度
#define SMI_CMD_BLOCK4_LEN  (0) ///< SMI block4 默认总长度
#define SMI_CMD_BLOCK5_ADDR ((u32)0) ///< SMI block5 默认总长度
#define SMI_CMD_BLOCK5_LEN  (0) ///< SMI block5 默认总长度
#define SMI_CMD_BLOCK6_ADDR ((u32)0) ///< SMI block6 默认总长度
#define SMI_CMD_BLOCK6_LEN  (0) ///< SMI block6 默认总长度
#define SMI_CMD_BLOCK7_ADDR ((u32)0) ///< SMI block7 默认总长度
#define SMI_CMD_BLOCK7_LEN  (0) ///< SMI block7 默认总长度

// pcie读取数据单次最多 4MB
#define SMI_VASTAI_PCI_RW_BUF_LEN_ONCE 0x400000 // 4MB

//定义可以管理的最多的device数目
#ifndef MAX_DEVICE_NUM_PER_MANAGER
#define MAX_DEVICE_NUM_PER_MANAGER 2048
#endif
//定义每个device里面die的最大数目
#ifndef MAX_DIE_NUM_PER_DEVICE
#define MAX_DIE_NUM_PER_DEVICE VASTAI_DIE_MAX_NUM
#endif
//定义每个die里面unit的最大数目, 如果超过, 需要各个命令自己扩充,
//无法使用这个通用的定义
#define MAX_UNIT_NUM_PER_DIE 32 //这个值和命令定义有关, 不能修改为更大的数目

/*flag 说明:
app和fw交流状态:
    最高bit:0 表示状态信息
    0x01U: 应用发送数据,等等fw处理
    0x02U: fw收到中断后, 写为0x02,表示已经在处理中,以后收到中断不再重复处理
    0x03U: fw数据准备好，写为0x03, 表示应用层可以读取数据了.
    0x04U: fw错误
    0x81U: fw收到未知命令
    0x82U: fw出现错误
	0x83U: fw系统忙
*/
//指示fw是否准备好数据 flag
#define FLAG_APP_PREPARE	  0x00 // 无命令
#define FLAG_APP_REQ		  0x01 // 应用发起命令
#define FLAG_FW_WORKING		  0x02 // smcu 开始处理命令
#define FLAG_FW_DATA_READY	  0x03 // smcu 命令执行成功
#define FLAG_FW_ERROR_UNKNOWN_CMD 0x81 // smcu 检测未知命令
#define FLAG_FW_ERROR_ABORT	  0x82 // smcu 命令执行错误
#define FLAG_FW_ERROR_SYSTEM_BUSY 0x83 // smcu 繁忙
#define FLAG_FW_ERROR_PARAM	  0x84 // 命令参数错误

//定义发送命令结构, 共 64 bit，
/*
命令格式: 共 64 bit
u8 cmd;          //命令组: 3bit  具体命令:5 bit
u8 flag;         //指示fw是否已经准备好数据， 0x03U，数据准备好， 0x01U, 0x02U,
0x04U数据未准备好, 应用层发送命令时填写0x01U u16 data_len;
//指示发生给fw的数据或者fw返回的数据字节长度，以byte为单位,
或者按照命令和执行代码解析自行解析 u32 unit;        //命令操作的指定单元,
每个bit代表一个unit，右侧第一个bit代表序号0,
如果命令不需要解析，填0，如果是全部，填0xFFFFFFFF

flag 说明, 表示app和fw交流状态:
0x01U: 应用发送数据,等等fw处理
0x02U: fw收到中断后, 写为0x02,表示已经在处理中,以后收到中断不再重复处理
0x03U: fw数据准备好，写为0x03, 表示应用层可以读取数据了. 此时数据已经读回,
表示数据有效,应用层可以分析 0x04U: fw出现错误, 应用层放弃此次请求
*/

typedef struct DIE_CMD_REQ_s {
	u32 unit; //命令操作的指定单元，从0开始，如果命令不需要解析，填0，如果是全部，填0xFFFF,
		//此字段也可以做命令扩展
	u16 data_len; //指示发送给fw的数据或者fw返回的数据字节长度，以byte为单位
	u8 flag; //指示fw是否已经准备好数据， 0x01U，数据准备好，
		// 0x00U数据未准备好, 应用层发送命令时填写0x00U
	u8 cmd; //具体命令
	u64 rw_addr;
} TS_SMI_CMD_REQ;

//发送命令联合体

//包含address参数的命令头部结构, 由u64 cmd和u64 param组成,
//为了统一和fw的数据处理, 增加了addr 和 value, 不用时可以忽略
typedef struct SMI_MEM32_s {
	u32 addr;
	u32 value;
} TS_SMI_MEM32;

typedef struct SMI_MEM64_s {
	u64 addr;
	u64 value;
} TS_SMI_MEM64;

typedef struct VAL_RANGE {
	u32 min;
	u32 max;
} TS_VAL_RANGE;
#define SMI_CMD_VAL_RANGE_CHECK(_val, _min, _max)                              \
	((_val) < _min && (_val) > _max)

//定义空unit
#define SMI_CMD_UNIT_NULL (0x00000000U)
//定义全部unit
#define SMI_CMD_UNIT_ALL (0xFFFFFFFFU)
//定义组成命令的默认后56bit
#define SMI_CMD_DEFAULT_56BIT 0x0001000000000000UL

//定义命令解析宏
#define SMI_CMD_EXTRACT_CMD(c)	   ((u8)(((u64)c >> 56) & 0xFF))
#define SMI_CMD_EXTRACT_FLAG(c)	   ((u8)(((u64)c >> 48) & 0xFF))
#define SMI_CMD_EXTRACT_DATALEN(c) ((u16)(((u64)c >> 32) & 0xFFFF))
#define SMI_CMD_EXTRACT_UNIT(c)	   ((u32)(((u64)c) & 0xFFFFFFFF))

//定义命令组合宏
#define SMI_CMD_PACK_CMD(c) ((u64)((((u64)(c)) << 56) | SMI_CMD_DEFAULT_56BIT))
#define SMI_CMD_PACK_CMD_UNIT(c, u)                                            \
	((u64)((((u64)(c)) << 56) | SMI_CMD_DEFAULT_56BIT) | u)
#define SMI_CMD_PACK_FLAG(x, c)	   ((u64)((u64)c | (u64)x << 48))
#define SMI_CMD_PACK_DATALEN(x, c) ((u64)((u64)c | (u64)x << 32))
#define SMI_CMD_PACK_UNIT(x, c)	   ((u64)((u64)c | (u64)x))

// smi 命令码, 填充 u8 cmd 字段

/***********************************************************************************
******   SMI 命令列表, 按照命令值排序, 新增加命令需要在此加入, 方便后续分配命令 ********
************************************************************************************

#define SMI_CMD_CODE_CTL_RESET ( 0x01 )
#define SMI_CMD_CODE_CTL_READ ( 0x02 )
#define SMI_CMD_CODE_PVT_PROCESS_GET ( 0x05 )
#define SMI_CMD_CODE_PVT_VOLTAGE_GET ( 0x06 )
#define SMI_CMD_CODE_TEMPERATURE_GET ( 0x07 )
#define SMI_CMD_CODE_PLL_CLOCK_GET ( 0x08 )
#define SMI_CMD_CODE_TEMPERATURE_GET_DEVICE ( 0x09 )
#define SMI_CMD_CODE_INF_GET_SUMMERY ( 0x0A )
#define SMI_CMD_CODE_INF_GET_MEMORY ( 0x0B )
#define SMI_CMD_CODE_INF_GET_SERIALNO ( 0x0C )
#define SMI_CMD_CODE_INF_GET_ALARM ( 0x0E )
#define SMI_CMD_CODE_CTL_MODULE_ENABLE ( 0x0F )
#define SMI_CMD_CODE_CTL_WRITE_TEST ( 0x10 )
#define SMI_CMD_CODE_AIC_PIN_VOLTAGE_GET ( 0x11 )
#define SMI_CMD_CODE_PIN_VOLTAGE_GET ( 0x12 )
#define SMI_CMD_CODE_POWER_VOLTAGE_GET ( 0x13 )
#define SMI_CMD_CODE_POWER_CURRENT_GET ( 0x14 )
#define SMI_CMD_CODE_POWER_GET ( 0x15 )
#define SMI_CMD_CODE_CTL_BATCH_READ ( 0x16 )
#define SMI_CMD_CODE_CTL_BATCH_WRITE ( 0x17 )
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_SET ( 0x18 )
#define SMI_CMD_CODE_BMCU_STATUS_GET ( 0x19 )
#define SMI_CMD_CODE_GET_FAN_SPEED_MODE ( 0x1A )
#define SMI_CMD_CODE_SET_FAN_SPEED_MODE ( 0x1B )
#define SMI_CMD_CODE_GET_FAN_SPEED ( 0x1C )
#define SMI_CMD_CODE_SET_FAN_SPEED ( 0x1D )
#define SMI_CMD_CODE_CTL_MODULE_STATUS ( 0x1F )
#define SMI_CMD_CODE_SPI_PROTECT_CONTROL ( 0x20 )
#define SMI_CMD_CODE_SPI_READ ( 0x21 )
#define SMI_CMD_CODE_PVT_PSM_MODEL_SET ( 0x23 )
#define SMI_CMD_CODE_PVT_PSM_DISABLE ( 0x24 )
#define SMI_CMD_CODE_PVT_PSM_GET ( 0x25 )
#define SMI_CMD_CODE_POWER_CURRENT_THRESHOLD_GET (0x26)
#define SMI_CMD_CODE_TOP_TEST_SET (0x27)
#define SMI_CMD_CODE_FCT (0x28)
#define SMI_CMD_CODE_VCC_CTRL (0x29)
#define SMI_CMD_CODE_CONFIG_GET (0x30)
#define SMI_CMD_CODE_CONFIG_SET (0x31)
#define SMI_CMD_CODE_CTL_WRITE ( 0x82 )
#define SMI_CMD_CODE_CTL_BITSET ( 0x83 )
#define SMI_CMD_CODE_PLL_CLOCK_SET ( 0x88 )
#define SMI_CMD_CODE_CTL_MODULE_DISABLE ( 0x8F )
#define SMI_CMD_CODE_AIC_PIN_VOLTAGE_SET ( 0x91 )
#define SMI_CMD_CODE_PIN_VOLTAGE_SET ( 0x92 )
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_GET ( 0x98 )
#define SMI_CMD_CODE_SPI_WRITE ( 0xA1 )
#define SMI_CMD_CODE_POWER_CURRENT_THRESHOLD_SET (0xA6)
#define PCIE_CMD_CODE_CONTROL ( 0xF1 )

************************************************************************************
******   SMI 命令列表, 按照命令值排序, 新增加命令需要在此加入, 方便后续分配命令 ********
************************************************************************************
*/

// reset 未使用
#define SMI_CMD_CODE_CTL_RESET (0x01)

// 读写内存数据
#define SMI_CMD_CODE_CTL_READ  (0x02)
#define SMI_CMD_CODE_CTL_WRITE (0x82)

// 按位读写
// bitset, 设置指定bit 为1, 其他保留原值. -B address mask.
// mask里需要操作的bit置1, 其他为0, 与地址原数据做 或 操作后写回.
#define SMI_CMD_CODE_CTL_BITSET (0x83)

// 控制命令, 设置enable/disable 模块/unit, 使用unit bit标识具体模块
// bit值0：disable； bit值1：enable
// unit bit 0: Ai 模块使能/禁用/状态
// unit bit 1: Video 模块使能/禁用/状态
// unit bit 2：autofreq 自动频率调整功能使能/禁用/状态
// unit bit 3: ecc 使能/禁用/状态
#define SMI_CMD_CODE_CTL_MODULE_ENABLE	(0x0F)
#define SMI_CMD_CODE_CTL_MODULE_DISABLE (0x8F)
#define SMI_CMD_CODE_CTL_MODULE_STATUS	(0x1F)

//内部测试使用
#define SMI_CMD_CODE_CTL_WRITE_TEST (0x10)

//批量读写数据
#define SMI_CMD_CODE_CTL_BATCH_READ  (0x16)
#define SMI_CMD_CODE_CTL_BATCH_WRITE (0x17)

// PVT x: 具体单元号码
// process , 暂未使用
#define SMI_CMD_CODE_PVT_PROCESS_GET (0x05)

// pvt电压监测点, 单位: 毫伏mV, 7组: dapb0 dapt0 dapm ddr_phy0 ddr_phy1 ddr_phy2 ddr_phy3
#define SMI_CMD_CODE_PVT_VOLTAGE_GET (0x06)

// Vpin 电压读写, 单位: 毫伏mV, 3组: VID, DLC, SoC,  VID 范围700mV~1100mV,  DLC范围:650mV~1200mV  SoC范围: 700mV~1100mV
#define SMI_CMD_CODE_PIN_VOLTAGE_GET (0x12)
#define SMI_CMD_CODE_PIN_VOLTAGE_SET (0x92)

// add in card类型卡Vpin电压读写, 单位: 毫伏mV, 3组: VID, DLC, SoC,  VID 范围700mV~1100mV,  DLC范围:650mV~1200mV
// SoC范围: 700mV~1100mV
#define SMI_CMD_CODE_AIC_PIN_VOLTAGE_GET (0x11)
#define SMI_CMD_CODE_AIC_PIN_VOLTAGE_SET (0x91)

//功耗电压读取, 单位mV, 3组, DDR, VID, DLC+SoC
#define SMI_CMD_CODE_POWER_VOLTAGE_GET (0x13)
//功耗电流读取, 单位mA, 3组, DDR, VID, DLC+SoC
#define SMI_CMD_CODE_POWER_CURRENT_GET (0x14)
//读取功耗, 单位 mW, 4组, DDR, VID, DLC+SoC, Total
#define SMI_CMD_CODE_POWER_GET (0x15)
//读取bmcu 状态
#define SMI_CMD_CODE_BMCU_STATUS_GET (0x19)

// SPI flash
#define SMI_CMD_CODE_SPI_PROTECT_CONTROL (0x20)
#define SMI_CMD_CODE_SPI_READ		 (0x21)
#define SMI_CMD_CODE_SPI_WRITE		 (0xA1)

// PVT PSM
#define SMI_CMD_CODE_PVT_PSM_MODEL_SET (0x23)
#define SMI_CMD_CODE_PVT_PSM_DISABLE   (0x24)
#define SMI_CMD_CODE_PVT_PSM_GET       (0x25)

// Power current threshold
#define SMI_CMD_CODE_POWER_CURRENT_THRESHOLD_GET (0x26)
#define SMI_CMD_CODE_POWER_CURRENT_THRESHOLD_SET (0xA6)

// Fan mode: 读取风扇模式: 自动: 0; 手动: 1
#define SMI_CMD_CODE_GET_FAN_SPEED_MODE (0x1A)
// Fan mode: 设置风扇模式: 自动: 0; 手动: 1
#define SMI_CMD_CODE_SET_FAN_SPEED_MODE (0x1B)
// Fan speed: 读取风扇速度, 返回两个值:u32 0~100; 速度的百分比 %; u32频率, 单位Hz
#define SMI_CMD_CODE_GET_FAN_SPEED (0x1C)
// Fan speed: 设置风扇速度: 0~100; 速度的百分比 %
#define SMI_CMD_CODE_SET_FAN_SPEED (0x1D)
enum SMI_FAN_MODE_e {
	FAN_MODE_AUTO = 0, ///< 自动模式
	FAN_MODE_MANUAL = 1, ///< 手动模式
};

// Top test
#define SMI_CMD_CODE_TOP_TEST_SET (0x27)

// 读取SoC温度 收到的mcu数据要除100, 转变为摄氏度
#define SMI_CMD_CODE_TEMPERATURE_GET (0x07)
// 读写温度阈值
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_SET (0x18)
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_GET (0x98)
// 读取板卡温度 收到的mcu数据要除100, 转变为摄氏度
// 温度排列: bmcu; 进风口; 出风口;
#define SMI_CMD_CODE_TEMPERATURE_GET_DEVICE (0x09)

// PLL: 频率读写, u64标识一个频率值
#define SMI_CMD_CODE_PLL_CLOCK_GET (0x08)
#define SMI_CMD_CODE_PLL_CLOCK_SET (0x88)

// 读取小结信息
#define SMI_CMD_CODE_INF_GET_SUMMERY (0x0A)
// 内存使用
#define SMI_CMD_CODE_INF_GET_MEMORY (0x0B)
// 读取序列号
#define SMI_CMD_CODE_INF_GET_SERIALNO (0x0C)
// 读取告警异常数据, 使用unit bit标识具体模块
// bit值0：disable； bit值1：enable
#define SMI_CMD_CODE_INF_GET_ALARM (0x0E)

// PCIe 命令, 不使用smcu. 直接访问驱动
#define PCIE_CMD_CODE_CONTROL (0xF1)

// FCT 生产测试使用, unit 作为子命令
#define SMI_CMD_CODE_FCT (0x28)
// FCT 命令unit子命令定义
enum SMI_FCT_SUBCMD_e {
	SMI_CMD_CODE_FCT_PMU31_PAGE0 = 1, // PMU31 相关寄存器访问
	SMI_CMD_CODE_FCT_PMU31_PAGE1, // PMU31 相关寄存器访问
	SMI_CMD_CODE_FCT_PMU31_PAGE2, // PMU31 相关寄存器访问
	SMI_CMD_CODE_FCT_PMU32_PAGE0, // PMU32 相关寄存器访问
	SMI_CMD_CODE_FCT_PMU32_PAGE1, // PMU32 相关寄存器访问
	SMI_CMD_CODE_FCT_PMU32_PAGE2, // PMU32 相关寄存器访问
	SMI_CMD_CODE_FCT_CLOCK, // 时钟芯片配置,
	SMI_CMD_CODE_FCT_CLOCK_SEL, // SV100时钟输入选择: 0x00100280.bit0 == 1 --> 100M ;0-->25M
	SMI_CMD_CODE_FCT_BMCU_TEMPERATURE, // BMCU 温度
	SMI_CMD_CODE_FCT_BMCU_SMBUS, // BMCU SMBUS access
	SMI_CMD_CODE_FCT_AVS_GET_TEMPERATURE, // AVS get temperature
	SMI_CMD_CODE_FCT_AVS_GET_VLOTAGE, // AVS get voltage
	SMI_CMD_CODE_FCT_AVS_GET_CURRENT, // AVS get current
	SMI_CMD_CODE_FCT_BMCU_HW_ID, // BMCU hw id: 0: VG1200 1: VG1000
};
#define SMI_CMD_CODE_FCT_CLOCK_SEL_REG (0x00100280)

// vcc volt 控制命令
#define SMI_CMD_CODE_VCC_CTRL (0x29)
enum SMI_VCC_CTRL_e {
	VCC_CTRL_VOLT_1V8,
	VCC_CTRL_VOLT_0V75,
	VCC_CTRL_GETSTEP_1V8,
	VCC_CTRL_GETSTEP_0V75,
	VCC_CTRL_SETSTEP_1V8,
	VCC_CTRL_SETSTEP_0V75,
	VCC_CTRL_MAX,
};

// 配置命令, 使用 unit 标识每个需要配置的功能项. 使用 data[0] 标识每个功能项的参数值或者参数序号
#define SMI_CMD_CODE_CONFIG_GET (0x30)
#define SMI_CMD_CODE_CONFIG_SET (0x31)

//参数范围

#define FREQUENCY_DISPLAY_MAX                                                  \
	4000000000 // modify for remove compare with U32 warning

//定义命令构成宏
//定义命令类型, 最高bit:0 读 最高bit:1 写
//判断命令是读函数写
#define IS_WRITE_CMD(x)                                                        \
	((u8)(((u64)x & 0x8000000000000000UL) == 0x8000000000000000UL))

// Control command
#define SMI_CMD_CTL_RESET SMI_CMD_PACK_CMD(SMI_CMD_CODE_CTL_RESET)
// read, 从地址读, 命令格式: -r address data_len
#define SMI_CMD_CTL_READ SMI_CMD_PACK_CMD(SMI_CMD_CODE_CTL_READ)
// write, 写参数到地址 -w address data_len data
#define SMI_CMD_CTL_WRITE SMI_CMD_PACK_CMD(SMI_CMD_CODE_CTL_WRITE)
// bitset, 设置指定bit 为1, 其他保留原值. -B address mask.
// mask里需要操作的bit置1, 其他为0, 与地址原数据做 或 操作后写回.
#define SMI_CMD_CTL_BITSET SMI_CMD_PACK_CMD(SMI_CMD_CODE_CTL_BITSET)
// bitclear, 设置指定bit 为0, 其他保留原值. -C address mask.

//内部测试使用,模拟write操作
#define SMI_CMD_CTL_WRITE_TEST SMI_CMD_PACK_CMD(SMI_CMD_CODE_CTL_WRITE_TEST)

//批量设置读取
#define SMI_CMD_BATCH_FETCH SMI_CMD_PACK_CMD(SMI_CMD_CODE_BATCH_FETCH)
#define SMI_CMD_BATCH_GET   SMI_CMD_PACK_CMD(SMI_CMD_CODE_BATCH_GET)

// PVT x: 具体单元号码
// process , 暂未使用
#define SMI_CMD_PVT_PROCESS_GET(x)                                             \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_PVT_PROCESS_GET, x)

// pvt电压监测点, 单位: 毫伏mV
#define SMI_CMD_PVT_VOLTAGE_GET(x)                                             \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_PVT_VOLTAGE_GET, x)
#define PVT_VOLTAGE_DISPLAY_MIN 0
#define PVT_VOLTAGE_DISPLAY_MAX 100000
#define PVT_VOLT_VID_MIN	700
#define PVT_VOLT_VID_MAX	1100
#define PVT_VOLT_DLC_MIN	650
#define PVT_VOLT_DLC_MAX	1200
#define PVT_VOLT_SOC_MIN	700
#define PVT_VOLT_SOC_MAX	1100

// Vpin 电压读写, 3组 VID, DLC, SOC mV, VID有效值: 0.7~1.1v,  DLC有效值:0.65 ~ 1.2v, SOC 有效值:0.7~1.1v
#define SMI_CMD_PIN_VOLTAGE_GET(x)                                             \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_PIN_VOLTAGE_GET, x)
#define SMI_CMD_PIN_VOLTAGE_SET(x)                                             \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_PIN_VOLTAGE_SET, x)
#define SMI_CMD_PIN_VOLTAGE_VID_MIN 474
#define SMI_CMD_PIN_VOLTAGE_VID_MAX 1185
#define SMI_CMD_PIN_VOLTAGE_DLC_MIN 474
#define SMI_CMD_PIN_VOLTAGE_DLC_MAX 1185
#define SMI_CMD_PIN_VOLTAGE_SOC_MIN 470
#define SMI_CMD_PIN_VOLTAGE_SOC_MAX 1185
#define PIN_VOLTAGE_DISPLAY_MIN	    0
#define PIN_VOLTAGE_DISPLAY_MAX	    2000000

//功耗电压电流读取, 3组, DDR, VID, DLC+SOC , mV, mA
#define SMI_CMD_POWER_VOLTAGE_GET(x)                                           \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_POWER_VOLTAGE_GET, x)
#define SMI_CMD_POWER_CURRENT_GET(x)                                           \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_POWER_CURRENT_GET, x)
#define POWER_VOLTAGE_DISPLAY_MIN 0
#define POWER_VOLTAGE_DISPLAY_MAX 20000000
#define POWER_CURRENT_DISPLAY_MIN 0
#define POWER_CURRENT_DISPLAY_MAX 10000000

//读取功耗, 4组, DDR, VID, DLC+SOC, total mW
#define SMI_CMD_POWER_GET(x) SMI_CMD_PACK_CMD_UNIT(SIM_CMD_CODE_POWER_GET, x)
#define POWER_DISPLAY_MIN    0
#define POWER_DISPLAY_MAX    1000000000

// 收到的mcu数据要除100, 转变为摄氏度
#define SMI_CMD_TEMPERATURE_GET(x)                                             \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_TEMPERATURE_GET, x)
#define TEMPERATURE_DISPLAY_MIN 0
#define TEMPERATURE_DISPLAY_MAX 10000

#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_SD_MIN 20
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_SD_MAX 120
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_DF_MIN 50
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_DF_MAX 140
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_IF_MIN 70
#define SMI_CMD_CODE_TEMPERATURE_THRESHOLD_IF_MAX 150

// INFO
#define SMI_CMD_INF_GET_SUMMERY	 SMI_CMD_PACK_CMD(SMI_CMD_CODE_INF_GET_SUMMERY)
#define SMI_CMD_INF_GET_MEMORY	 SMI_CMD_PACK_CMD(SMI_CMD_CODE_INF_GET_MEMORY)
#define SMI_CMD_INF_GET_SERIALNO SMI_CMD_PACK_CMD(SMI_CMD_CODE_INF_GET_SERIALNO)
//告警
#define SMI_CMD_INF_GET_ALARM(x)                                               \
	SMI_CMD_PACK_CMD_UNIT(SMI_CMD_CODE_INF_GET_ALARM, x)

#ifdef __cplusplus
}
#endif
#pragma pack(pop)

#endif //__VATOOLS_SMI_CMD_H__
