#include "vastai_pci.h"
#include "hw_queue.h"
#include "sg100_gmcu.h"
#include "vastai_cmd.h"
#include "hw_config.h"
#include "vastai_fifo.h"
#include "vastai_ras.h"
#include "vastai_state.h"

u8 only_1pf_coreid[ONLY_1PF_RING_BUF_MAX] = {0};
u8 only_2pf_coreid[ONLY_2PF_RING_BUF_MAX] = {0};
u8 common_coreid[COMMON_RING_BUF_MAX] = {0};

static void init_ring_buf_coreid(struct vastai_pci_info *priv)
{
	switch (priv->fn_mode) {
	case ONLY_1_PF:
		only_1pf_coreid[ONLY_1PF_HOST_TO_SMCU_CMD_BUF]   = CORE_POINT_SMCU;
		only_1pf_coreid[ONLY_1PF_HOST_TO_CMCU_CMD_BUF]   = CORE_POINT_CMCU;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VEMCU0_CMD_BUF] = CORE_POINT_VEMCU0;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VEMCU1_CMD_BUF] = CORE_POINT_VEMCU1;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VEMCU2_CMD_BUF] = CORE_POINT_VEMCU2;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VEMCU3_CMD_BUF] = CORE_POINT_VEMCU3;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VDSP0_CMD_BUF]  = CORE_POINT_VDSP0;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VDSP1_CMD_BUF]  = CORE_POINT_VDSP1;
		only_1pf_coreid[ONLY_1PF_HOST_TO_ODSP0_CMD_BUF]  = CORE_POINT_ODSP0;
		only_1pf_coreid[ONLY_1PF_HOST_TO_ODSP1_CMD_BUF]  = CORE_POINT_ODSP1;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VDMCU0_CMD_BUF] = CORE_POINT_VDMCU0;
		only_1pf_coreid[ONLY_1PF_HOST_TO_VDMCU1_CMD_BUF] = CORE_POINT_VDMCU1;
		only_1pf_coreid[ONLY_1PF_HOST_TO_GMCU1_CMD_BUF]  = CORE_POINT_GMCU1;
		only_1pf_coreid[ONLY_1PF_HOST_TO_GMCU3_CMD_BUF]  = CORE_POINT_GMCU3;
		only_1pf_coreid[ONLY_1PF_SMCU_TO_HOST_INFO_BUF]  = CORE_POINT_SMCU;
		only_1pf_coreid[ONLY_1PF_CMCU_TO_HOST_INFO_BUF]  = CORE_POINT_CMCU;
		only_1pf_coreid[ONLY_1PF_VEMCU0_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU0;
		only_1pf_coreid[ONLY_1PF_VEMCU1_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU1;
		only_1pf_coreid[ONLY_1PF_VEMCU2_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU2;
		only_1pf_coreid[ONLY_1PF_VEMCU3_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU3;
		only_1pf_coreid[ONLY_1PF_VDSP0_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0;
		only_1pf_coreid[ONLY_1PF_VDSP1_TO_HOST_INFO_BUF] = CORE_POINT_VDSP1;
		only_1pf_coreid[ONLY_1PF_ODSP0_TO_HOST_INFO_BUF] = CORE_POINT_ODSP0;
		only_1pf_coreid[ONLY_1PF_ODSP1_TO_HOST_INFO_BUF] = CORE_POINT_ODSP1;
		only_1pf_coreid[ONLY_1PF_VDMCU0_TO_HOST_INFO_BUF]= CORE_POINT_VDMCU0;
		only_1pf_coreid[ONLY_1PF_VDMCU1_TO_HOST_INFO_BUF]= CORE_POINT_VDMCU1;
		only_1pf_coreid[ONLY_1PF_GMCU1_TO_HOST_INFO_BUF] = CORE_POINT_GMCU1;
		only_1pf_coreid[ONLY_1PF_GMCU3_TO_HOST_INFO_BUF] = CORE_POINT_GMCU3;
		only_1pf_coreid[ONLY_1PF_DISPLAY_H2D_BUF]        = CORE_POINT_GMCU1;
		only_1pf_coreid[ONLY_1PF_DISPLAY_H2D_BUF]        = CORE_POINT_GMCU1;
		break;

	case ONLY_2_PF:
		only_2pf_coreid[ONLY_2PF_HOST_TO_SMCU_CMD_BUF]   	= CORE_POINT_SMCU;
		only_2pf_coreid[ONLY_2PF_HOST_TO_CMCU_CMD_BUF]   	= CORE_POINT_CMCU;
		only_2pf_coreid[ONLY_2PF_SMCU_TO_HOST_INFO_BUF]     = CORE_POINT_SMCU;
		only_2pf_coreid[ONLY_2PF_CMCU_TO_HOST_INFO_BUF]     = CORE_POINT_CMCU;
		only_2pf_coreid[ONLY_2PF_DISPLAY_H2D_BUF]           = CORE_POINT_GMCU1;
		only_2pf_coreid[ONLY_2PF_DISPLAY_D2H_BUF]           = CORE_POINT_GMCU1;
		if(0 == priv->priv_hw_cfg->sys_cfg.pfn){
			only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VEMCU_CMD_BUF] = CORE_POINT_VEMCU0;
			only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VEMCU_CMD_BUF] = CORE_POINT_VEMCU1;
			only_2pf_coreid[ONLY_2PF_HOST_TO_VDMCU_CMD_BUF]     = CORE_POINT_VDMCU0;
			only_2pf_coreid[ONLY_2PF_HOST_TO_GMCU_CMD_BUF]      = CORE_POINT_GMCU1;
			only_2pf_coreid[ONLY_2PF_1ST_VEMCU_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU0;
			only_2pf_coreid[ONLY_2PF_2ND_VEMCU_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU1;
			only_2pf_coreid[ONLY_2PF_VDMCU_TO_HOST_INFO_BUF]    = CORE_POINT_VDMCU0;
			only_2pf_coreid[ONLY_2PF_GMCU_TO_HOST_INFO_BUF]     = CORE_POINT_GMCU1;
			if (1 == priv->priv_hw_cfg->dsp_cfg.valid_vdsp_entry_n) {
				only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF]  = CORE_POINT_VDSP0;
				//this ringbuf is reserved
				only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF]  = CORE_POINT_VDSP0;
				only_2pf_coreid[ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0;
				//this ringbuf is reserved
				only_2pf_coreid[ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0;
			} else if (2 == priv->priv_hw_cfg->dsp_cfg.valid_vdsp_entry_n) {
				only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF]  = CORE_POINT_ODSP0;
				only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF]  = CORE_POINT_VDSP0;
				only_2pf_coreid[ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_ODSP0;
				only_2pf_coreid[ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0;
			}								
		} else if (1 == priv->priv_hw_cfg->sys_cfg.pfn){
			only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VEMCU_CMD_BUF] = CORE_POINT_VEMCU2;
			only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VEMCU_CMD_BUF] = CORE_POINT_VEMCU3;
			only_2pf_coreid[ONLY_2PF_HOST_TO_VDMCU_CMD_BUF]     = CORE_POINT_VDMCU1;
			only_2pf_coreid[ONLY_2PF_HOST_TO_GMCU_CMD_BUF]      = CORE_POINT_GMCU3;
			only_2pf_coreid[ONLY_2PF_1ST_VEMCU_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU2;
			only_2pf_coreid[ONLY_2PF_2ND_VEMCU_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU3;
			only_2pf_coreid[ONLY_2PF_VDMCU_TO_HOST_INFO_BUF]    = CORE_POINT_VDMCU1;
			only_2pf_coreid[ONLY_2PF_GMCU_TO_HOST_INFO_BUF]     = CORE_POINT_GMCU3;
			if (1 == priv->priv_hw_cfg->dsp_cfg.valid_vdsp_entry_n) {
				only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF]  = CORE_POINT_VDSP1;
				//this ringbuf is reserved
				only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF]  = CORE_POINT_VDSP1;
				only_2pf_coreid[ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP1;
				//this ringbuf is reserved
				only_2pf_coreid[ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP1;
			} else if (2 == priv->priv_hw_cfg->dsp_cfg.valid_vdsp_entry_n) {
				only_2pf_coreid[ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF]  = CORE_POINT_ODSP1;
				only_2pf_coreid[ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF]  = CORE_POINT_VDSP1;
				only_2pf_coreid[ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_ODSP1;
				only_2pf_coreid[ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP1;
			}
		}
		
		break;
	
	case ONLY_4_PF:
	case PF_4_WITH_SRIOV:
		common_coreid[COMMON_HOST_TO_SMCU_CMD_BUF]  = CORE_POINT_SMCU;
		common_coreid[COMMON_HOST_TO_CMCU_CMD_BUF]  = CORE_POINT_CMCU;
		common_coreid[COMMON_HOST_TO_VEMCU_CMD_BUF] = CORE_POINT_VEMCU0 + priv->priv_hw_cfg->sys_cfg.pfn;
		common_coreid[COMMON_HOST_TO_VDMCU_CMD_BUF] = CORE_POINT_VDMCU0 + priv->priv_hw_cfg->sys_cfg.pfn/2;
		common_coreid[COMMON_HOST_TO_GMCU_CMD_BUF]  = CORE_POINT_GMCU0  + priv->priv_hw_cfg->sys_cfg.pfn;
		common_coreid[COMMON_SMCU_TO_HOST_INFO_BUF] = CORE_POINT_SMCU;
		common_coreid[COMMON_CMCU_TO_HOST_INFO_BUF] = CORE_POINT_CMCU;
		common_coreid[COMMON_VEMCU_TO_HOST_INFO_BUF]= CORE_POINT_VEMCU0 + priv->priv_hw_cfg->sys_cfg.pfn;
		common_coreid[COMMON_VDMCU_TO_HOST_INFO_BUF]= CORE_POINT_VDMCU0 + priv->priv_hw_cfg->sys_cfg.pfn/2;
		common_coreid[COMMON_GMCU_TO_HOST_INFO_BUF] = CORE_POINT_GMCU0  + priv->priv_hw_cfg->sys_cfg.pfn;
		if(2 == priv->priv_hw_cfg->dsp_cfg.vdsp_num) {
			common_coreid[COMMON_HOST_TO_VDSP_CMD_BUF]  = CORE_POINT_VDSP0 + priv->priv_hw_cfg->sys_cfg.pfn/2;
			common_coreid[COMMON_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0 + priv->priv_hw_cfg->sys_cfg.pfn/2;
		} else if (4 == priv->priv_hw_cfg->dsp_cfg.vdsp_num){
			if(0 == priv->priv_hw_cfg->sys_cfg.pfn){
				common_coreid[COMMON_HOST_TO_VDSP_CMD_BUF]  = CORE_POINT_ODSP0;
				common_coreid[COMMON_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_ODSP0;
			} else if (1 == priv->priv_hw_cfg->sys_cfg.pfn) {
				common_coreid[COMMON_HOST_TO_VDSP_CMD_BUF]  = CORE_POINT_VDSP0;
				common_coreid[COMMON_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP0;
			} else if (2 == priv->priv_hw_cfg->sys_cfg.pfn) {
				common_coreid[COMMON_HOST_TO_VDSP_CMD_BUF]  = CORE_POINT_ODSP1;
				common_coreid[COMMON_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_ODSP1;
			} else if (3 == priv->priv_hw_cfg->sys_cfg.pfn) {
				common_coreid[COMMON_HOST_TO_VDSP_CMD_BUF]  = CORE_POINT_VDSP1;
				common_coreid[COMMON_VDSP_TO_HOST_INFO_BUF] = CORE_POINT_VDSP1;
			}	
		}		
		break;
	
	default:
		break;
	}
}

static void vastai_ring_buf_init(struct vastai_pci_info *priv,
				int index,char *ring_buf_name)
{
	struct  ring_buf_entry *hw_entry;
	hw_entry = find_hw_cfg_acord_name(priv,ring_buf_name);
	if( NULL != hw_entry ) {
		priv->ring_buf[index] = kzalloc(sizeof(struct ring_buf_info), GFP_KERNEL);
		spin_lock_init(&priv->ring_buf[index]->wr_ring_buf_lock);
		spin_lock_init(&priv->ring_buf[index]->rd_ring_buf_lock);
		priv->ring_buf[index]->ring_buf_base   = hw_entry->ring_buf_base;
		priv->ring_buf[index]->ring_buf_size   = hw_entry->ring_buf_size;
		priv->ring_buf[index]->vf_m2o_reg_base = hw_entry->m2o_reg_base;
		priv->ring_buf[index]->ring_buf_fn_base= 0x800000 +(hw_entry->ring_buf_base - priv->priv_hw_cfg->header.csram_base);
		switch (priv->fn_mode){
			case ONLY_1_PF:
				priv->ring_buf[index]->core_id = only_1pf_coreid[index];
				break;
			
			case ONLY_2_PF:
				priv->ring_buf[index]->core_id = only_2pf_coreid[index];				
				break;
			
			case ONLY_4_PF:
			case PF_4_WITH_SRIOV:
				priv->ring_buf[index]->core_id = common_coreid[index];				
				break;
			
			default:
				break;
		}
	} else {
		VASTAI_PCI_INFO(NULL, DUMMY_DIE_ID, "%s buf init fail\n",ring_buf_name);
	}
	return;
}

/**
 * @brief init all cmd buffer 
 * 
 */
int init_all_ring_buf(struct vastai_pci_info *priv)
{
	init_ring_buf_coreid(priv);
	if( ONLY_1_PF == priv->fn_mode)	{
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_SMCU_CMD_BUF,   		ONLY_1_PF_HOST_WR_M2O_CMD_SMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_CMCU_CMD_BUF,   		ONLY_1_PF_HOST_WR_M2O_CMD_CMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VEMCU0_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU0_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VEMCU1_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU1_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VEMCU2_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU2_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VEMCU3_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU3_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VDSP0_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_VDSP0_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VDSP1_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_VDSP1_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_ODSP0_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_ODSP0_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_ODSP1_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_ODSP1_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VDMCU0_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VDMCU0_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_VDMCU1_CMD_BUF, 		ONLY_1_PF_HOST_WR_M2O_CMD_VDMCU1_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_GMCU1_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_GMCU1_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_HOST_TO_GMCU3_CMD_BUF,  		ONLY_1_PF_HOST_WR_M2O_CMD_GMCU3_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_SMCU_TO_HOST_INFO_BUF,  		ONLY_1_PF_SMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_CMCU_TO_HOST_INFO_BUF,  		ONLY_1_PF_CMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VEMCU0_TO_HOST_INFO_BUF,		ONLY_1_PF_VEMCU0_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VEMCU1_TO_HOST_INFO_BUF,		ONLY_1_PF_VEMCU1_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VEMCU2_TO_HOST_INFO_BUF,		ONLY_1_PF_VEMCU2_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VEMCU3_TO_HOST_INFO_BUF,		ONLY_1_PF_VEMCU3_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VDSP0_TO_HOST_INFO_BUF, 		ONLY_1_PF_VDSP0_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VDSP1_TO_HOST_INFO_BUF, 		ONLY_1_PF_VDSP1_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_ODSP0_TO_HOST_INFO_BUF, 		ONLY_1_PF_ODSP0_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_ODSP1_TO_HOST_INFO_BUF, 		ONLY_1_PF_ODSP1_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VDMCU0_TO_HOST_INFO_BUF,		ONLY_1_PF_VDMCU0_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_VDMCU1_TO_HOST_INFO_BUF,		ONLY_1_PF_VDMCU1_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_GMCU1_TO_HOST_INFO_BUF, 		ONLY_1_PF_GMCU1_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_GMCU3_TO_HOST_INFO_BUF, 		ONLY_1_PF_GMCU3_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_DISPLAY_H2D_BUF, 				ONLY_1_PF_DISPLAY_H2D_NAME);
		vastai_ring_buf_init(priv, ONLY_1PF_DISPLAY_D2H_BUF, 				ONLY_1_PF_DISPLAY_D2H_NAME);
	} else if (ONLY_2_PF == priv->fn_mode){
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_SMCU_CMD_BUF,        	ONLY_2_PF_HOST_WR_M2O_CMD_SMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_CMCU_CMD_BUF,        	ONLY_2_PF_HOST_WR_M2O_CMD_CMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_1ST_VEMCU_CMD_BUF,   	ONLY_2_PF_HOST_WR_M2O_CMD_1ST_VEMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_2ND_VEMCU_CMD_BUF,   	ONLY_2_PF_HOST_WR_M2O_CMD_2ND_VEMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_1ST_VDSP_CMD_BUF,    	ONLY_2_PF_HOST_WR_M2O_CMD_1ST_VDSP_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_2ND_VDSP_CMD_BUF,    	ONLY_2_PF_HOST_WR_M2O_CMD_2ND_VDSP_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_VDMCU_CMD_BUF,       	ONLY_2_PF_HOST_WR_M2O_CMD_VDMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_HOST_TO_GMCU_CMD_BUF,        	ONLY_2_PF_HOST_WR_M2O_CMD_GMCU_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_SMCU_TO_HOST_INFO_BUF,       	ONLY_2_PF_SMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_CMCU_TO_HOST_INFO_BUF,       	ONLY_2_PF_CMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_1ST_VEMCU_TO_HOST_INFO_BUF,  	ONLY_2_PF_1ST_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_2ND_VEMCU_TO_HOST_INFO_BUF,  	ONLY_2_PF_2ND_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_1ST_VDSP_TO_HOST_INFO_BUF,   	ONLY_2_PF_1ST_VDSP_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_2ND_VDSP_TO_HOST_INFO_BUF,   	ONLY_2_PF_2ND_VDSP_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_VDMCU_TO_HOST_INFO_BUF,      	ONLY_2_PF_VDMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_GMCU_TO_HOST_INFO_BUF,       	ONLY_2_PF_GMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_DISPLAY_H2D_BUF,       	        ONLY_2_PF_DISPLAY_H2D_NAME);
		vastai_ring_buf_init(priv, ONLY_2PF_DISPLAY_D2H_BUF,       	        ONLY_2_PF_DISPLAY_D2H_NAME);
	} else {
		vastai_ring_buf_init(priv, COMMON_HOST_TO_SMCU_CMD_BUF,   			COMMON_HOST_WR_M2O_CMD_SMCU_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_HOST_TO_CMCU_CMD_BUF,   			COMMON_HOST_WR_M2O_CMD_CMCU_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_HOST_TO_VEMCU_CMD_BUF,  			COMMON_HOST_WR_M2O_CMD_VEMCU_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_HOST_TO_VDSP_CMD_BUF,   			COMMON_HOST_WR_M2O_CMD_VDSP_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_HOST_TO_VDMCU_CMD_BUF,  			COMMON_HOST_WR_M2O_CMD_VDMCU_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_HOST_TO_GMCU_CMD_BUF,   			COMMON_HOST_WR_M2O_CMD_GMCU_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_SMCU_TO_HOST_INFO_BUF,  			COMMON_SMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_CMCU_TO_HOST_INFO_BUF,  			COMMON_CMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_VEMCU_TO_HOST_INFO_BUF,			COMMON_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_VDSP_TO_HOST_INFO_BUF,  			COMMON_VDSP_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_VDMCU_TO_HOST_INFO_BUF, 			COMMON_VDMCU_WR_MSIX_INFO_HOST_RD_NAME);
		vastai_ring_buf_init(priv, COMMON_GMCU_TO_HOST_INFO_BUF,  			COMMON_GMCU_WR_MSIX_INFO_HOST_RD_NAME);
		
	}	
			
	return 0;
}

void deinit_all_ring_buf(struct vastai_pci_info *priv)
{
	int ring_buf_index = 0;
	for(ring_buf_index = 0 ;ring_buf_index < ONLY_1PF_RING_BUF_MAX;ring_buf_index++){
		if(NULL != priv->ring_buf[ring_buf_index]){
			kfree(priv->ring_buf[ring_buf_index]);
		}
	}
	
	return;
}



/*==================== DMA transfer && MSG api ====================*/
/* Don't call this function at irq, This only use spin_lock */
int vastai_pci_send_msg_pid(void *pcie_dev, int die_index,
			int index, void *msg_buf, u32 msg_len, pid_t pid)
{
	int ret = 0;
	struct vastai_pci_info *priv=pcie_dev;
	u64   vf_m2o_int_addr  = 0;
	u64   msg_addr         = 0;
	u32   triger_m2o_int   = 0;
	int state = 0;
	unsigned long flags;

	if(NULL == priv->ring_buf[index])
		return -EFAULT;

	state = vastai_get_pci_state(priv);

	switch (state) {
		case VASTAI_NORMAL_STATE:
		case VASTAI_DEBUG_STATE:
		case VASTAI_BOOT_STATE:
			break;

		default:
			VASTAI_PCI_ERR(priv, priv->pkg_id, "pci_send_msg_pid:pcie state error[%s],Please Retry Again!\n",
				state_name(state));
			return -ENOMEM;
	}

	if(priv->core_status_bitmap & (1<<priv->ring_buf[index]->core_id)) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "%s Hang!,Please Retry Again!\n",
				core_name(priv,priv->ring_buf[index]->core_id));
		return -ENOMEM;
	}

	vf_m2o_int_addr  = priv->ring_buf[index]->vf_m2o_reg_base;
	msg_addr         = priv->ring_buf[index]->ring_buf_base;
	triger_m2o_int   = 0x80000000 | (1<<0);

	spin_lock_irqsave(&priv->ring_buf[index]->wr_ring_buf_lock, flags);

	ret = vastai_fifo_push_elem(priv, die_index, msg_addr, msg_buf, NULL);
	vastai_pci_mem_write(priv, 0, vf_m2o_int_addr, &triger_m2o_int, 4);

	spin_unlock_irqrestore(&priv->ring_buf[index]->wr_ring_buf_lock, flags);

	return ret;
}

int vastai_pci_send_msg(void *pcie_dev, int die_index,
			int index, void *msg_buf, u32 msg_len)
{
	return vastai_pci_send_msg_pid(pcie_dev, die_index,
				index, msg_buf, msg_len, -1);

}

int vastai_pci_receive_msg(void *pcie_dev, int die_index,
			int index, void *elem)
{
	struct vastai_pci_info *priv=pcie_dev;
	u64 msg_addr   = priv->ring_buf[index]->ring_buf_base;
	unsigned long flags;
	int ret;

	if(priv->core_status_bitmap & (1<<priv->ring_buf[index]->core_id)) {
		VASTAI_PCI_ERR(priv, priv->pkg_id, "%s Hang!,Please Retry Again!\n",
				core_name(priv,priv->ring_buf[index]->core_id));
		return -ENOMEM;
	}

	spin_lock_irqsave(&priv->ring_buf[index]->rd_ring_buf_lock, flags);

	ret = vastai_fifo_pop_elem(pcie_dev, die_index, msg_addr,elem, NULL);

	spin_unlock_irqrestore(&priv->ring_buf[index]->rd_ring_buf_lock, flags);

	return	ret;
}

