#ifndef __VASTAI_PCIE_EXPORT_H__
#define __VASTAI_PCIE_EXPORT_H__

#include <linux/dmaengine.h>

struct va_dma_descriptor {
	struct list_head elem_head;
	enum dma_transfer_direction direction;
	void (*callback)(void *param);
	void *callback_param;
	/* private data for compatible with different chip series */
	void *priv;
	u32 elem_num;
	u8 pid;
	/* task priority, 0:high level,use sdma0; 1:low level,use sdma1 */
	u8 priority; 
	u8 dest_mmu;
	u8 src_mmu;
	/* The sdma descriptor storage location. 0:csram, 1:ddr*/
	u8 desc_loc;
	/* just used for slave die. 0:slave to host 1:slave to master */
	u8 d2d;
	/* when sync_tail=1, gmcu will not trigger msix to host after sdma done */
	u32 sync_tail;
	/* dma performance monitor */
	u32 total_len;
	u64 desc_addr;  /* just used when desc_loc = 1 */
	ktime_t trigger_time;
	ktime_t done_time;
};

typedef enum cedar_svc_type {
	SHA2_224 = 0u,
	SHA2_224_UPDATE,
	SHA2_224_IMG_1080,
	SHA2_224_IMG_1088,
	SHA2_256,
	SHA2_256_UPDATE,
	SHA2_256_IMG_1080,
	SHA2_256_IMG_1088,
	SHA2_384,
	SHA2_384_UPDATE,
	SHA2_384_IMG_1080,
	SHA2_384_IMG_1088,
	SHA2_512,
	SHA2_512_UPDATE,
	SHA2_512_IMG_1080,
	SHA2_512_IMG_1088,
	SM3,
	SM3_UPDATE,
	SM3_IMG_1080,
	SM3_IMG_1088,
	SHA3_224,
	SHA3_224_UPDATE,
	SHA3_224_IMG_1080,
	SHA3_224_IMG_1088,
	AES_ECB_ENC,
	AES_ECB_DEC,
	AES_CBC_ENC,
	AES_CBC_DEC,
	AES_CTR_ENC,
	AES_CTR_DEC,
	AES_GCM_ENC,
	AES_GCM_DEC,
	SM4_ECB_ENC,
	SM4_ECB_DEC,
	SM4_CBC_ENC,
	SM4_CBC_DEC,
	SM4_CTR_ENC,
	SM4_CTR_DEC,
	SM4_GCM_ENC,
	SM4_GCM_DEC,
	DES_ECB_ENC,
	DES_ECB_DEC,
	DES_CBC_ENC,
	DES_CBC_DEC,
	ECC_ENC,
	ECC_DEC,
	ECC_SIGN,
	ECC_VERIFY,
	SM2_ENC,
	SM2_DEC,
	SM2_SIGN,
	SM2_VERIFY,
	RSA_ENC,
	RSA_DEC,
	RSA_SIGN,
	RSA_VERIFY,
	RNG,
} svc_t;

typedef struct __attribute__((packed)) vastai_cedar_img_info {
	u64 input_addr;
	u64 output_addr;
	svc_t alg_type;
	u32 blen;
	u64 key;
	u64 iv;
	u64 msg;
	u64 tag;
	u32 keyblen;
	u32 ivblen;
	u32 msgblen;
	u32 tblen;
	u64 random;
	u64 pubkey;
	u64 prikey;
	u64 sign;
	u32 pkaMode;
	u32 config;
	u32 width;
	u32 height;
	u32 rngmode;
	u32 symmode;// mask or normal mode
}vastai_cedar_img_info_t;

/**************** cmd type *****************/
typedef enum {
	TOOL_CMD_GFX_NONE = 0,
	TOOL_CMD_GFX_GET_DRIVER_STATUS = 1,
	TOOL_CMD_GFX_GET_GPU_UTILIZATION = 2,
	TOOL_CMD_GFX_GET_MEMORY_USAGE = 3,
	TOOL_CMD_GFX_NUM_TYPE = 4,
} PVRSRV_CMD_TYPE;

#define GFX_PF_MAX (128) //0xFFFF  // vf: pf_index >> 16
/******************* cmd_args *****************/
typedef struct
{
	u32 pf_num;
	u32 pf_index[GFX_PF_MAX];
	u64 input_data_addr;
	u32 input_data_size;
} gfx_cmd_input_args_s;


/******************** info ********************/
typedef struct 
{
	u32 pf_index;
	u32 utilization;
} GFX_UTILIZATION;

typedef struct
{
	u32 pf_num;
	GFX_UTILIZATION info[GFX_PF_MAX];
} GFX_UTILIZATION_INFO;

typedef struct
{
	u32 pf_index;
	u32 MemoryTotal;
	u32 MemoryUsageKMalloc;
	u32 MemoryUsageKMallocMax;
	u32 MemoryUsageVMalloc;
	u32 MemoryUsageVMallocMax;
	u32 MemoryUsageAllocPTu32_MemoryUMA;
	u32 MemoryUsageAllocPTu32_MemoryUMAMax;
	u32 MemoryUsageVMapPTUMA;
	u32 MemoryUsageVMapPTUMAMax;
	u32 MemoryUsageAllocPTu32_MemoryLMA;
	u32 MemoryUsageAllocPTu32_MemoryLMAMax;
	u32 MemoryUsageIORemapPTLMA;
	u32 MemoryUsageIORemapPTLMAMax;
	u32 MemoryUsageAllocGPUMemLMA;
	u32 MemoryUsageAllocGPUMemLMAMax;
	u32 MemoryUsageAllocGPUMemUMA;
	u32 MemoryUsageAllocGPUMemUMAMax;
	u32 MemoryUsageAllocGPUMemUMAPool;
	u32 MemoryUsageAllocGPUMemUMAPoolMax;
	u32 MemoryUsageMappedGPUMemUMA_LMA;
	u32 MemoryUsageMappedGPUMemUMA_LMAMax;
	u32 MemoryUsageDmaBufImport;
	u32 MemoryUsageDmaBufImportMax;
	u32 MemoryUsageDisplay_Heap_Size;
	u32 MemoryUsageDisplay_Heap_Use;
} GFX_MEMORY;

typedef struct
{
	u32 pf_num;
	GFX_MEMORY info[GFX_PF_MAX];
} MEMORY_USAGE_INFO;

struct va_dma_descriptor *va_dma_add_desc_by_pa(void *dma,
				u64 cpu_addr,
				u64 dev_addr,
				u32 size,
				enum dma_transfer_direction direction);
struct va_dma_descriptor *va_dma_prep_desc_array(void *dma,
                 u64 *mem_addr, u64 *dev_addr, u32 *size,
                 u32 elem_nums,
                enum dma_transfer_direction direction);
struct va_dma_descriptor *va_dma_prep_slave_sg(void *dma,
				struct scatterlist *sgl,
				unsigned int sg_len,
				u64 dev_addr,
				enum dma_transfer_direction direction);
void va_dma_descriptor_free(struct va_dma_descriptor *desc);
void *va_dma_get_byname(struct device *dev, char *name);
int add_elem_to_dma_desc(struct va_dma_descriptor *desc, u32 len, u64 mem, u64 dev);
int va_dma_desc_submit(void *dma, struct va_dma_descriptor *desc);
int va_dma_free_gfx(struct device *dev, void *dma);

int va_register_display_interrupt(struct device *dev,
				void (*handler_function)(void *), void *arg);

int va_unregister_display_interrupt(struct device *dev);
int va_get_display_irq_data(struct device *dev, void *irq_data);
int va_register_gpu_interrupt(struct device *dev, int interrupt_id,
				void (*handler_function)(void *), void *data);
int va_unregister_gpu_interrupt(struct device *dev, int interrupt_id);
int va_enable_gpu_interrupt(struct device *dev, int interrupt_id);
int va_disable_gpu_interrupt(struct device *dev, int interrupt_id);

void va_register_physheap_alloc_func(void *priv, void *alloc);
void va_register_physheap_free_func(void *priv, void *free);
void va_register_pvr_devnode(void *priv, void *dev_node);
void va_register_get_gfx_status(void *priv, void *get_gfx_status);
void va_unregister_get_gfx_status(void *priv);
void va_install_pvr_devnode(void *priv, void *dev_node);
void va_print_pci_devid(void *priv);

int vastai_dev_cedar_send_cmd(void *pci_info, void *cedar_info, int dev_id, int die_id, void *hash_output);
void vastai_die_lock(void *pci_info);
void vastai_die_unlock(void *pci_info);
int vastai_gfx_reset(void *pci_info);
int vastai_pci_flr(void *pci_info);
unsigned char* va_get_model_name(void *pci_info);
unsigned char* va_get_product_sn(void *pci_info);
#endif
