#ifndef __VASTAI_VIDEO_FENCE__
#define __VASTAI_VIDEO_FENCE__

#include <linux/version.h>

#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8, 3)
    #include <linux/reservation.h>
  #else
    #include <linux/dma-resv.h>
  #endif
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0))
  #include <linux/dma-resv.h>
#else
  #include <linux/reservation.h>
#endif

#if defined(RHEL_RELEASE_VERSION)

  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(8, 3)
    #define dma_resv                      reservation_object
    #define dma_resv_list                 reservation_object_list

    #define dma_resv_lock_interruptible   reservation_object_lock_interruptible
    #define dma_resv_unlock               reservation_object_unlock
    #define dma_resv_reserve_shared       reservation_object_reserve_shared
    #define dma_resv_excl_fence           reservation_object_get_excl
    #define dma_resv_add_excl_fence       reservation_object_add_excl_fence
    #define dma_resv_add_shared_fence     reservation_object_add_shared_fence
    #define dma_resv_get_fences           reservation_object_get_fences_rcu
    #define dma_resv_init                 reservation_object_init
    #define dma_resv_fini                 reservation_object_fini
    #define dma_resv_shared_list          reservation_object_get_list
    #define dma_resv_held                 reservation_object_held
    #define dma_resv_test_signaled        reservation_object_test_signaled_rcu
    #define dma_resv_wait_timeout         reservation_object_wait_timeout_rcu
  #elif RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
    #define dma_resv_get_fences           dma_resv_get_fences_rcu
    #define dma_resv_shared_list          dma_resv_get_list
    #define dma_resv_excl_fence           dma_resv_get_excl
    #define dma_resv_wait_timeout         dma_resv_wait_timeout_rcu
    #define dma_resv_test_signaled        dma_resv_test_signaled_rcu
  #else
    #define dma_resv_reserve_shared       dma_resv_reserve_fences
  #endif

#elif (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0))
    /* Reservation object types */
    #define dma_resv                      reservation_object
    #define dma_resv_list                 reservation_object_list

    /* Reservation object functions */
  #if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 16, 0))
    #define dma_resv_lock_interruptible   reservation_object_lock
  #else
    #define dma_resv_lock_interruptible   reservation_object_lock_interruptible
  #endif

    #define dma_resv_unlock               reservation_object_unlock
    #define dma_resv_reserve_shared       reservation_object_reserve_shared
    #define dma_resv_excl_fence           reservation_object_get_excl
    #define dma_resv_add_excl_fence       reservation_object_add_excl_fence
    #define dma_resv_add_shared_fence     reservation_object_add_shared_fence
    #define dma_resv_get_fences           reservation_object_get_fences_rcu
    #define dma_resv_init                 reservation_object_init
    #define dma_resv_fini                 reservation_object_fini
    #define dma_resv_shared_list          reservation_object_get_list
    #define dma_resv_held                 reservation_object_held
    #define dma_resv_test_signaled        reservation_object_test_signaled_rcu
    #define dma_resv_wait_timeout         reservation_object_wait_timeout_rcu


#elif (LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0))

    #define dma_resv_get_fences           dma_resv_get_fences_rcu
    #define dma_resv_shared_list          dma_resv_get_list
    #define dma_resv_excl_fence           dma_resv_get_excl
    #define dma_resv_wait_timeout         dma_resv_wait_timeout_rcu
    #define dma_resv_test_signaled        dma_resv_test_signaled_rcu

#elif (LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0))

#else

  #define dma_resv_reserve_shared         dma_resv_reserve_fences

#endif

#endif /*__VASTAI_VIDEO_FENCE__*/
