/*
 * Vastai Decoder device driver (kernel module)
*
* Copyright (C) 2020 VASTAI Microelectronics Co., Ltd.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
------------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/module.h>
/* needed for __init,__exit directives */
#include <linux/init.h>
/* needed for remap_page_range
    SetPageReserved
    ClearPageReserved
*/
#include <linux/mm.h>
/* obviously, for kmalloc */
#include <linux/slab.h>
/* for struct file_operations, register_chrdev() */
#include <linux/fs.h>
/* standard error codes */
#include <linux/errno.h>

#include <linux/moduleparam.h>
/* request_irq(), free_irq() */
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <linux/semaphore.h>
#include <linux/spinlock.h>
/* needed for virt_to_phys() */
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/uaccess.h>
#include <linux/ioport.h>

#include <asm/irq.h>

#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/sched/signal.h>
#include <linux/signal.h>
#include <linux/string.h>


/* our own stuff */
#include <linux/platform_device.h>


/* #include "vastaidec.h" */
#include "subsys.h"

#include"vastai_pci.h"
#include"vastai_pci_api.h"


#include "vastai_dec.h"
#include "vastai_sv100_reg.h"
#include "vastai_render.h"
#include "vastai_video_utils.h"

#include "hw_config.h"
#include "vastai_udma_engine.h"
#include "vastai_pcie_public.h"

/*
 * Macros to help debugging
 */
#define VASTAI           "va"

// #define VASTAI_VCMD_DRIVER_DEBUG
#define VASTAI_DISABLE_VCMD

#undef PDEBUG   /* undef it, just in case */
#ifdef VASTAI_VCMD_DRIVER_DEBUG
#  ifdef __KERNEL__
    /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_INFO "pid[%d] va_decoder: " fmt, current->pid,  ## args)

#  else
    /* This one for user space */
#    define PDEBUG(fmt, args...) printf(":%d: " fmt, __LINE__ , ## args)
#  endif
#else
#  define PDEBUG(fmt, args...)  /* not debugging: nothing */
#endif

/*------------------------------------------------------------------------
*****************************VCMD CONFIGURATION BY CUSTOMER********************************
-------------------------------------------------------------------------*/

//video decoder vcmd configuration
#define VCMD_DEC_IO_ADDR_0                      0x10000   /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_0                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_DEC_INT_PIN_0                      -1
#define VCMD_DEC_MODULE_TYPE_0                  2
#define VCMD_DEC_MODULE_MAIN_ADDR_0             0x800    /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_0           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_0          0X2000
#define VCMD_DEC_MODULE_MMU_ADDR_0              0XFFFF


#define VCMD_DEC_IO_ADDR_1                      0x700000   /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_1                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_DEC_INT_PIN_1                      -1
#define VCMD_DEC_MODULE_TYPE_1                  2
#define VCMD_DEC_MODULE_MAIN_ADDR_1             0x800    /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_1           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_1          0X2000
#define VCMD_DEC_MODULE_MMU_ADDR_1              0XFFFF

#define VCMD_DEC_IO_ADDR_2                      0xb2000   /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_2                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_DEC_INT_PIN_2                      -1
#define VCMD_DEC_MODULE_TYPE_2                  2
#define VCMD_DEC_MODULE_MAIN_ADDR_2             0x0000    /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_2           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_2          0XFFFF
#define VCMD_DEC_MODULE_MMU_ADDR_2              0XFFFF

#define VCMD_DEC_IO_ADDR_3                      0xb3000   /*customer specify according to own platform*/
#define VCMD_DEC_IO_SIZE_3                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_DEC_INT_PIN_3                      -1
#define VCMD_DEC_MODULE_TYPE_3                  2
#define VCMD_DEC_MODULE_MAIN_ADDR_3             0x0000    /*customer specify according to own platform*/
#define VCMD_DEC_MODULE_DEC400_ADDR_3           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_DEC_MODULE_L2CACHE_ADDR_3          0XFFFF
#define VCMD_DEC_MODULE_MMU_ADDR_3              0XFFFF


//JPEG decoder vcmd configuration

#define VCMD_JPEGD_IO_ADDR_0                      0x600000   /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_0                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_JPEGD_INT_PIN_0                      -1
#define VCMD_JPEGD_MODULE_TYPE_0                  4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_0             0x800    /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_0           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_0          0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_0              0XFFFF

#define VCMD_JPEGD_IO_ADDR_1                      0xD1000   /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_1                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_JPEGD_INT_PIN_1                      -1
#define VCMD_JPEGD_MODULE_TYPE_1                  4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_1             0x0000    /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_1           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_1          0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_1              0XFFFF

#define VCMD_JPEGD_IO_ADDR_2                      0xD2000   /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_2                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_JPEGD_INT_PIN_2                      -1
#define VCMD_JPEGD_MODULE_TYPE_2                  4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_2             0x0000    /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_2           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_2          0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_2              0XFFFF

#define VCMD_JPEGD_IO_ADDR_3                      0xD3000   /*customer specify according to own platform*/
#define VCMD_JPEGD_IO_SIZE_3                      (ASIC_VCMD_SWREG_AMOUNT * 4)    /* bytes */
#define VCMD_JPEGD_INT_PIN_3                      -1
#define VCMD_JPEGD_MODULE_TYPE_3                  4
#define VCMD_JPEGD_MODULE_MAIN_ADDR_3             0x0000    /*customer specify according to own platform*/
#define VCMD_JPEGD_MODULE_DEC400_ADDR_3           0XFFFF    /*0xffff means no such kind of submodule*/
#define VCMD_JPEGD_MODULE_L2CACHE_ADDR_3          0XFFFF
#define VCMD_JPEGD_MODULE_MMU_ADDR_3              0XFFFF

/*these size need to be modified according to hw config.*/
#define VCMD_ENCODER_REGISTER_SIZE              (479 * 4)
#define VCMD_DECODER_REGISTER_SIZE              (512 * 4)
#define VCMD_IM_REGISTER_SIZE                   (479 * 4)
#define VCMD_JPEG_ENCODER_REGISTER_SIZE         (479 * 4)
#define VCMD_JPEG_DECODER_REGISTER_SIZE         (512 * 4)

#define HW_WORK_STATE_PEND            3

#define MAX_CMDBUF_INT_NUMBER         1
#define INT_MIN_SUM_OF_IMAGE_SIZE    (4096*2160*MAX_SAME_MODULE_TYPE_CORE_NUMBER*MAX_CMDBUF_INT_NUMBER)
#define MAX_PROCESS_CORE_NUMBER         4*8
#define PROCESS_MAX_VIDEO_SIZE          (4096*2160*MAX_SAME_MODULE_TYPE_CORE_NUMBER*MAX_PROCESS_CORE_NUMBER)
#define PROCESS_MAX_JPEG_SIZE           (2147483648) //32768*32768*2
#define PROCESS_MAX_SUM_OF_IMAGE_SIZE   (PROCESS_MAX_VIDEO_SIZE>PROCESS_MAX_JPEG_SIZE?PROCESS_MAX_VIDEO_SIZE:PROCESS_MAX_JPEG_SIZE)


/********variables declaration related with race condition**********/

#define REG_PACKAGE_SIZE     (CMDBUF_MAX_SIZE / 2)

//#define DDR_BASE_ADDR				(0x800000000)

#define VCMD_DEC_DDR_SIZE                   (0x900000)
#define VCMD_DEC_DDR_BASE_ADDR_0            (0x1035300000)
#define VCMD_DEC_DDR_BASE_ADDR_1            (VCMD_DEC_DDR_BASE_ADDR_0 + VCMD_DEC_DDR_SIZE)
#define VCMD_DEC_DDR_BASE_ADDR_0_SRIOV      (0x102e000000)
#define VCMD_DEC_DDR_BASE_ADDR_1_SRIOV      (VCMD_DEC_DDR_BASE_ADDR_0_SRIOV + VCMD_DEC_DDR_SIZE)
//#define VCMD_DEC_DDR_BASE_ADDR_2            (VCMD_DEC_DDR_BASE_ADDR_1 + VCMD_DEC_DDR_SIZE)
//#define VCMD_DEC_OFFSET_ADDR              (VCMD_DDR_BASE_ADDR - DDR_BASE_ADDR)

#define align(x, y) (x + (y - 1)) / y

#define HW_VCMD_IRQ_INTCMD_MASK  (0xFFFF << 16)
#define HW_VCMD_IRQ_RESET_MASK   (1 << 5)
#define HW_VCMD_IRQ_ABORT_MASK   (1 << 4)
#define HW_VCMD_IRQ_CMDERR_MASK  (1 << 3)
#define HW_VCMD_IRQ_TIMEOUT_MASK (1 << 2)
#define HW_VCMD_IRQ_BUSERR_MASK  (1 << 1)
#define HW_VCMD_IRQ_ENDCMD_MASK  (1 << 0)
#define HW_VCMD_IRQ_DEC_MASK  (0x40)

#define MAX_DATATYPE_NUM  2

#define VASTAI_DEC_CORE0  VDMCU0_2_HOST_INT
#define VASTAI_DEC_CORE1  VDMCU1_2_HOST_INT
#define VASTAI_DEC_CORE2  VDMCU2_2_HOST_INT
#define CORE0_IRQ_BASE    VASTAI_DEC_CORE0

#define TO_MCU_DATA_TYPE  0
#define TO_MCU_CMDBUF_LEN 1

#define MAX_DECODE_CAPACITY		(60 * 1000)
#define MAX_SUPPORTED_RESOLUTION	(7680 * 4320)
#define BASE_DECODE_RESOLUTION		(1920 * 1080)

extern struct dma_trans *dma_trans_tx;

/* u8 msg_buf[128]; */

/*for all vcmds, the core info should be listed here for subsequent use*/
struct vcmd_config vcmd_core_array[MAX_SUBSYS_NUM]= {
    //decoder configuration
    {VCMD_DEC_IO_ADDR_0,
    VCMD_DEC_IO_SIZE_0,
    VCMD_DEC_INT_PIN_0,
    VCMD_DEC_MODULE_TYPE_0,
    VCMD_DEC_MODULE_MAIN_ADDR_0,
    VCMD_DEC_MODULE_DEC400_ADDR_0,
    VCMD_DEC_MODULE_L2CACHE_ADDR_0,
    VCMD_DEC_MODULE_MMU_ADDR_0},

    {VCMD_DEC_IO_ADDR_1,
    VCMD_DEC_IO_SIZE_1,
    VCMD_DEC_INT_PIN_1,
    VCMD_DEC_MODULE_TYPE_1,
    VCMD_DEC_MODULE_MAIN_ADDR_1,
    VCMD_DEC_MODULE_DEC400_ADDR_1,
    VCMD_DEC_MODULE_L2CACHE_ADDR_1,
    VCMD_DEC_MODULE_MMU_ADDR_1},
};


//#define CSRAM_BASE_ADDR          				(0x8C00000)
/* vdmcu write hw_id build_id to host addr, ASIC_ID:4B HW_BUILDID:4B L2_CACHE_VERSION:4B reserved:4B*/
#define VDEC0_REPORT_ID_ADDR        			(CSRAM_BASE_ADDR + 0x000F0800) // 16 bytes
#define VDEC1_REPORT_ID_ADDR        			(CSRAM_BASE_ADDR + 0x000F0810) // 16 bytes
#define VDEC2_REPORT_ID_ADDR        			(CSRAM_BASE_ADDR + 0x000F0820) // 16 bytes
#define VDEC3_REPORT_ID_ADDR        			(CSRAM_BASE_ADDR + 0x000F0830) // 16 bytes
#define VDEC4_REPORT_ID_ADDR        			(CSRAM_BASE_ADDR + 0x000F0840) // 16 bytes

static u64 VDEC_REPORT_ID_ADDR[5] = {VDEC0_REPORT_ID_ADDR, VDEC1_REPORT_ID_ADDR, VDEC2_REPORT_ID_ADDR, VDEC3_REPORT_ID_ADDR, VDEC4_REPORT_ID_ADDR};

extern unsigned long gBaseDDRHw;        /* PCI base register address (memalloc) */
extern unsigned int mmu_enable;
struct list_head dec_device_handle_head;

/***************************TYPE AND FUNCTION DECLARATION****************/

/* here's all the must remember stuff */

static int allocate_cmdbuf(struct die_dec_info *die, struct noncache_mem* new_cmdbuf_addr, struct noncache_mem* new_status_cmdbuf_addr, u16 core_id);


/*********************local variable declaration*****************/
/* and this is our MAJOR; use 0 for dynamic allocation (recommended)*/
int total_vcmd_core_num = 0;
static unsigned int vcmd;
static struct subsys_config vpu_subsys[MAX_SUBSYS_NUM];

static void vastai_pci_done_callback(void* arg, u32 core_id)
{
    u32 irq_status = 0;
    u32 cmdbuf_id = 0;
    int ret = 0;
    int work_ret = 0;
    vdmcu_to_host_msg_t vdmcu_msg;
    struct die_dec_info *die = NULL;
    struct vastaivcmd_dev *dev = NULL;
    struct vastai_pci_info *pcie_dev = NULL;

    if (!arg) {
        printk(KERN_ERR "va_pci_done_callback err, input arg is NULL!\n");
        return;
    }

    die = (struct die_dec_info *)arg;
    pcie_dev = (struct vastai_pci_info *)die->vcmd_pri;
    if (!pcie_dev){
        printk(KERN_ERR "va_pci_done_callback err, pcie_dev is NULL!\n");
        return;
    }

    if (!die->vastaivcmd_data){
        printk(KERN_ERR "va_pci_done_callback err, vcmd_data is NULL!\n");
        return; 
    }
    
    while (ret != -ENOMEM) {
#if VASTAI_DEC_USING_PCIE_FUNCTION
        ret = vastai_pci_receive_msg(pcie_dev, 0, die->vastaivcmd_data->vdec_buf_index[core_id], &vdmcu_msg);
        if (ret) {
            PDEBUG("error receive msg ret = %d !!\n", ret);
            continue;
        }
#else
        ret = vastai_video_fifo_pop_elem(pcie_dev, 0, die->vastaivcmd_data->vdmcu2host_ringbuffer[core_id]->ring_buf_base, &vdmcu_msg, NULL);
        if (ret) {
            PDEBUG("error receive msg ret = %d !!\n", ret);
            continue;
        }
#endif
        irq_status = vdmcu_msg.payload.status;
        cmdbuf_id = vdmcu_msg.payload.cmd_id;

        dev = &(die->vastaivcmd_data[core_id]);
        dev->mcu_vcmd_status = irq_status;

        PDEBUG(" in_fifo: core[%d] ---> cmdbuf_id=%d -- mcu_vcmd_status = %d \n", core_id,
               cmdbuf_id, irq_status);

        ret = vastai_cmdid_enqueue(&dev->cmdbufId_queue, cmdbuf_id, irq_status);
        if (ret) {
            printk(KERN_ERR "cmdbuf id queue is full, return!\n");
            continue;
        }

        if (cmdbuf_id >= dev->total_vcmd_num + dev->cmdbuf_offset) {
            printk(KERN_ERR "callback error cmdbuf_id greater than the ceiling !!\n");
            break;
        }

        work_ret = queue_work(die->workqueue, &(dev->work));
    }

    PDEBUG(" pcie-callback-done: core[%d] ---> cmdbuf_id=%d -- irq_status=0x%x work_ret=%d\n",
           core_id, cmdbuf_id, irq_status, work_ret);
}

static void vastai_pci_done_core0_callback(void* arg)
{
    u32 core_id = 0;
    vastai_pci_done_callback(arg, core_id);
}

static void vastai_pci_done_core1_callback(void* arg)
{
    u32 core_id = 1;
    vastai_pci_done_callback(arg, core_id);
}

static void vcmd_workqueue_schedue(struct work_struct *work)
{
	struct vastaivcmd_dev *dev = container_of(work, struct vastaivcmd_dev, work);

	unsigned int handled = 0;
	struct cmdbuf_obj* cmdbuf_obj = NULL;
	u32 cmdbuf_id = 0;
	u32 irq_status = 0, ret = 0;
	u32 cmdbuf_processed_num=0;

	PDEBUG("\x1b[31m +++-> workqueue debug  \x1b[0m \n");

	while (ret == 0) {
		ret = vastai_cmdid_dequeue(&dev->cmdbufId_queue, &cmdbuf_id, &irq_status);
		if (ret) {
			break;
		}

		PDEBUG(" out_fifo:die[%d] core[%d] ---> cmdbuf_id=%d -- mcu_vcmd_status=%d \n", dev->die->die_id, dev->core_id, cmdbuf_id, irq_status);
		//mutex_lock(&dev->mutex);
		if (cmdbuf_id) {
			if (cmdbuf_id >= dev->total_vcmd_num + dev->cmdbuf_offset) {
				printk(KERN_ERR "vcmd_workqueue_schedue error cmdbuf_id greater than the ceiling !!\n");
				//mutex_unlock(&dev->mutex);
				continue;

			}
			cmdbuf_obj = &dev->cmdbuf_obj_sets[cmdbuf_id];
			if (cmdbuf_obj == NULL) {
				printk(KERN_ERR "vcmd_workqueue_schedue error cmdbuf_id !!\n");
				//mutex_unlock(&dev->mutex);
				continue;

			}
			cmdbuf_obj->executing_status = irq_status;
			if (dev->cmdbuf_used[cmdbuf_id] == 1) {
				cmdbuf_obj->cmdbuf_run_done = 1;
				cmdbuf_processed_num++;
			}
			//complete(&cmdbuf_obj->vcmd_process_comp);
			handled++;
		}
		//mutex_unlock(&dev->mutex);
	}

	if (!handled) {
		PDEBUG("IRQ received, but not process!\n");
	}
	if (cmdbuf_processed_num)
		wake_up_interruptible_all(&dev->wait_queue);

	PDEBUG(" work_queue-done: die[%d] core[%d] ---> cmdbuf_id=%d -- irq_status=0x%x \n", dev->die->die_id, dev->core_id, cmdbuf_id, irq_status);
}

#ifdef VCMD_DEBUG_INTERNAL
static void PrintInstr(u32 i, u32 instr, u32 *size) {
  if ((instr & 0xF8000000) == OPCODE_WREG) {
    int length = ((instr >> 16) & 0x3FF);
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s %s %d 0x%x]\n", i, instr, "WREG", ((instr >> 26) & 0x1) ? "FIX" : "",
           length, (instr & 0xFFFF));
    *size = ((length+2)>>1)<<1;
  } else if ((instr & 0xF8000000) == OPCODE_END) {
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s]\n", i, instr, "END");
    *size = 2;
  } else if ((instr & 0xF8000000) == OPCODE_NOP) {
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s]\n", i, instr, "NOP");
    *size = 2;
  } else if ((instr & 0xF8000000) == OPCODE_RREG) {
    int length = ((instr >> 16) & 0x3FF);
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s %s %d 0x%x]\n", i, instr, "RREG", ((instr >> 26) & 0x1) ? "FIX" : "",
           length, (instr & 0xFFFF));
    *size = 4;
  } else if ((instr & 0xF8000000) == OPCODE_JMP) {
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s %s %s]\n", i, instr, "JMP", ((instr >> 26) & 0x1) ? "RDY" : "",
           ((instr >> 25) & 0x1) ? "IE" : "");
    *size = 4;
  } else if ((instr & 0xF8000000) == OPCODE_STALL) {
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s %s 0x%x]\n", i, instr, "STALL", ((instr >> 26) & 0x1) ? "IM" : "",
           (instr & 0xFFFF));
    *size = 2;
  } else if ((instr & 0xF8000000) == OPCODE_CLRINT) {
    printk(KERN_INFO "current cmdbuf data %d = 0x%08x => [%s %d 0x%x]\n", i, instr, "CLRINT", (instr >> 25) & 0x3,
           (instr & 0xFFFF));
    *size = 2;
  } else
    *size = 1;
}
#endif

static void free_cmdbuf_mem(struct die_dec_info *die, u32 cmdbuf_id, u32 core_id)
{
    //spin_lock_irqsave(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_alloc_lock, flags);
    mutex_lock(&die->vastaivcmd_data[core_id].mutex);
    die->vastaivcmd_data[core_id].cmdbuf_last_rel_pos = cmdbuf_id;
    die->vastaivcmd_data[core_id].cmdbuf_used[cmdbuf_id] = 0;
    die->vastaivcmd_data[core_id].cmdbuf_used_residual += 1;
    mutex_unlock(&die->vastaivcmd_data[core_id].mutex);
    //spin_unlock_irqrestore(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_alloc_lock, flags);
    wake_up_interruptible_sync(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_memory_wait);
}

static void clean_cmdbuf_obj(struct cmdbuf_obj* cmdbuf_obj, u32 cmdbuf_id, u32 core_id, struct die_dec_info* die)
{
	if (cmdbuf_obj == NULL) {
		PDEBUG ("%s\n","remove_cmdbuf_node NULL");
		return;
	}

	cmdbuf_obj->cmdbuf_need_remove = 1;
	cmdbuf_obj->cmdbuf_busAddress = 0;
	cmdbuf_obj->cmdbuf_size = 0;
	cmdbuf_obj->cmdbuf_id = -1;
	cmdbuf_obj->status_busAddress = 0;
	cmdbuf_obj->status_size = 0;
	cmdbuf_obj->module_type = -1;
	cmdbuf_obj->core_id = -1;
	cmdbuf_obj->filp = NULL;
	cmdbuf_obj->cmdbuf_run_done = 0;
//	cmdbuf_obj->cmdbuf_done_flag = 0;

	free_cmdbuf_mem(die, cmdbuf_id, core_id);
	return;
}

static struct cmdbuf_obj* create_cmdbuf_obj(struct die_dec_info *die, u16 core_id)
{
	/* bi_list_node* current_node=NULL; */
	struct cmdbuf_obj* cmdbuf_obj = NULL;
	struct noncache_mem  new_cmdbuf_addr = {0};
	struct noncache_mem  new_status_cmdbuf_addr = {0};
	u16 get_cmdbuf_id = -1;

	if(wait_event_interruptible_exclusive(die->vastaivcmd_data[core_id].vcmd_cmdbuf_memory_wait, allocate_cmdbuf(die, &new_cmdbuf_addr, &new_status_cmdbuf_addr, core_id)))
			return NULL;

    get_cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;

	cmdbuf_obj = &(die->vastaivcmd_data[core_id].cmdbuf_obj_sets[get_cmdbuf_id]);
	cmdbuf_obj->cmdbuf_busAddress = new_cmdbuf_addr.busAddress;
	cmdbuf_obj->cmdbuf_size = new_cmdbuf_addr.size;
	cmdbuf_obj->cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;
	cmdbuf_obj->status_busAddress = new_status_cmdbuf_addr.busAddress;
	cmdbuf_obj->status_size = new_status_cmdbuf_addr.size;

	return cmdbuf_obj;
}

/**********************************************************************************************************\
*cmdbuf pool management
\***********************************************************************************************************/
static int allocate_cmdbuf(struct die_dec_info *die, struct noncache_mem* new_cmdbuf_addr, struct noncache_mem* new_status_cmdbuf_addr, u16 core_id)
{
	u16 i = 0;

	mutex_lock(&die->vastaivcmd_data[core_id].mutex);
	//spin_lock_irqsave(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_alloc_lock, flags);
	if (die->vastaivcmd_data[core_id].cmdbuf_used_residual == 0) {
		mutex_unlock(&die->vastaivcmd_data[core_id].mutex);
		//spin_unlock_irqrestore(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_alloc_lock, flags);
		//no empty cmdbuf
		return 0;
	}
	//there is one cmdbuf at least
	while (1) {
		i = die->vastaivcmd_data[core_id].cmdbuf_used_pos;
		if ((die->vastaivcmd_data[core_id].cmdbuf_used[i] == 0)&&(die->vastaivcmd_data[core_id].cmdbuf_last_rel_pos != i)){
			die->vastaivcmd_data[core_id].cmdbuf_used[i] = 1;
			die->vastaivcmd_data[core_id].cmdbuf_used_residual -=1;

			new_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->cmdbuf_id = i;

			new_cmdbuf_addr->busAddress = die->vastaivcmd_data[core_id].vcmd_buf_mem_pool.busAddress + i * CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->mmu_bus_address = die->vastaivcmd_data[core_id].vcmd_buf_mem_pool.mmu_bus_address + i * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->busAddress = die->vastaivcmd_data[core_id].vcmd_status_buf_mem_pool.busAddress + i * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->mmu_bus_address = die->vastaivcmd_data[core_id].vcmd_status_buf_mem_pool.mmu_bus_address + i * CMDBUF_MAX_SIZE;

			new_status_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->cmdbuf_id = i;
			die->vastaivcmd_data[core_id].cmdbuf_used_pos++;

			if (die->vastaivcmd_data[core_id].cmdbuf_used_pos >= die->vastaivcmd_data[core_id].total_vcmd_num + die->vastaivcmd_data[core_id].cmdbuf_offset)
				die->vastaivcmd_data[core_id].cmdbuf_used_pos  = die->vastaivcmd_data[core_id].cmdbuf_offset;

			mutex_unlock(&die->vastaivcmd_data[core_id].mutex);
			//spin_unlock(&die->vastaivcmd_data[core_id].vcmd_cmdbuf_alloc_lock);
			return 1;
		} else {
			die->vastaivcmd_data[core_id].cmdbuf_used_pos++;
			if (die->vastaivcmd_data[core_id].cmdbuf_used_pos >= die->vastaivcmd_data[core_id].total_vcmd_num + die->vastaivcmd_data[core_id].cmdbuf_offset)
				die->vastaivcmd_data[core_id].cmdbuf_used_pos  = die->vastaivcmd_data[core_id].cmdbuf_offset;
		}
	}
	return 0;
}

#if 0
static void dump_cmdbuf(struct die_dec_info *die, u16 core_id)
{
    u16 i = 0;
    u32 used =0;
    struct cmdbuf_obj* cmdbuf_obj=NULL;

    //there is one cmdbuf at least

    for(i=0;i<TOTAL_DISCRETE_CMDBUF_NUM;i++)
    {
        used = die->vastaivcmd_data[core_id].cmdbuf_used[i];
        cmdbuf_obj = &(die->vastaivcmd_data[core_id].cmdbuf_obj_sets[i]);
        printk("id,%d, used %d, run %d, file 0x%x\n",i, used, cmdbuf_obj->cmdbuf_run_done, cmdbuf_obj->filp);

    }
    printk("core %d, pos %d, residual %d\n",core_id,die->vastaivcmd_data[core_id].cmdbuf_used_pos, die->vastaivcmd_data[core_id].cmdbuf_used_residual);
}
#endif

static long reserve_cmdbuf(struct file *filp, struct die_dec_info *die, struct exchange_parameter* input_para)
{
  struct cmdbuf_obj* cmdbuf_obj=NULL;

  input_para->cmdbuf_id = 0;
  if (input_para->cmdbuf_size > CMDBUF_MAX_SIZE) {
    return -1;
  }
  PDEBUG("core[%d]:reserve cmdbuf filp %p\n", input_para->core_id, (void *)filp);

  cmdbuf_obj = create_cmdbuf_obj(die, input_para->core_id);
  if (cmdbuf_obj == NULL) {
	  PDEBUG("core[%d] create_cmdbuf_obj failed \n", input_para->core_id);
	  return -1;
  }

  cmdbuf_obj->module_type = input_para->module_type;
  cmdbuf_obj->priority = input_para->priority;
  cmdbuf_obj->executing_time = input_para->executing_time;
  cmdbuf_obj->cmdbuf_size = CMDBUF_MAX_SIZE;
  cmdbuf_obj->core_id = input_para->core_id;
  cmdbuf_obj->cmdbuf_run_done = 0;
  cmdbuf_obj->filp = filp;
  input_para->cmdbuf_size = CMDBUF_MAX_SIZE;
  input_para->cmdbuf_id = cmdbuf_obj->cmdbuf_id;

  //reinit_completion(&cmdbuf_obj->vcmd_process_comp);

  return 0;
}

static long release_cmdbuf(struct file *filp, u16 cmdbuf_id, u16 core_id, struct die_dec_info* die)
{
	struct cmdbuf_obj* cmdbuf_obj=NULL;

	cmdbuf_obj = &(die->vastaivcmd_data[core_id].cmdbuf_obj_sets[cmdbuf_id]);
	if (cmdbuf_obj->filp!=filp ||  core_id != cmdbuf_obj->core_id) {
		//should not happen
		printk(KERN_ERR "%d-->:core[%d] ERROR cmdbuf_id=%d !!\n", __LINE__, core_id, cmdbuf_obj->cmdbuf_id);
		return -1;
	}

	clean_cmdbuf_obj(cmdbuf_obj, cmdbuf_id, core_id, die);

	return 0;
}

static long link_and_run_cmdbuf(struct file *filp, struct exchange_parameter *input_para,
                                struct die_dec_info *die_info)
{
    struct cmdbuf_obj *cmdbuf_obj = NULL;
    struct vastai_pci_info *pcie_dev = (struct vastai_pci_info *)die_info->vcmd_pri;
    // struct vastaivcmd_dev *dev = NULL;
    int total = 0, count = 0, ret = -1, msg_len = 2;
    struct vastai_dmadesc *dma_buf = NULL;
    union core_bitmap triger_core = {{0}};
    u32 *user_vcmd_end = NULL;
    host_to_vdmcu_msg_t *vcmd_msg_package = NULL;
    u16 cmdbuf_id = input_para->cmdbuf_id;
    u16 core_id = input_para->core_id;
    int retry;

    cmdbuf_obj = &(die_info->vastaivcmd_data[core_id].cmdbuf_obj_sets[cmdbuf_id]);
    if (cmdbuf_obj->core_id != input_para->core_id) {
        // should not happen
        printk(KERN_ERR "link_and_run_cmdbuf: ERROR cmdbuf_id=%d ----> core[%d] !!\n", cmdbuf_id, core_id);
        BUG_ON(1);
    }

    if (cmdbuf_obj->filp != filp) {
        // should not happen
        printk(KERN_ERR "%d link_and_run_cmdbuf: ERROR cmdbuf_id !!\n", __LINE__);
        return -1;
    }

    cmdbuf_obj->cmdbuf_data_loaded = 1;
    cmdbuf_obj->cmdbuf_size = input_para->cmdbuf_size; // (16 + offset_inc)*4
    cmdbuf_obj->waited = 0;

#ifdef VCMD_DEBUG_INTERNAL
    if (filp != NULL) {
        {
            u32 i, inst = 0, size = 0;
            printk(KERN_INFO "vcmd link, current cmdbuf content\n");
            for (i = 0; i < cmdbuf_obj->cmdbuf_size / 4; i++) {
                __get_user(inst, (u32 *)input_para->vcmdbuf_desc.vcmdbuf_src_addr + i);
                if (i == inst) {
                    PrintInstr(i, inst, &size);
                    inst += size;
                } else {
                    printk(KERN_INFO "current cmdbuf data %d = 0x%x\n", i, inst);
                }
            }
        }
    }
#endif

    cmdbuf_obj->has_end_cmdbuf = 0;       // 0: has jmp opcode,1 has end code
    cmdbuf_obj->no_normal_int_cmdbuf = 1; // 0: interrupt when JMP,1 not interrupt when JMP
    cmdbuf_obj->cmdbuf_User_virtualAddress = (u32 *)(input_para->vcmdbuf_desc.vcmdbuf_src_addr);

    // dev = &(die_info->vastaivcmd_data[core_id]);

    PDEBUG("Allocate cmd buffer [%d] to core [%d]\n", cmdbuf_id, input_para->core_id);

    user_vcmd_end =
        (u32 *)(input_para->vcmdbuf_desc.vcmdbuf_src_addr) + (cmdbuf_obj->cmdbuf_size / 4);

    vcmd_msg_package = (host_to_vdmcu_msg_t *)kmalloc(sizeof(host_to_vdmcu_msg_t) + msg_len * 4, GFP_KERNEL);
    if (vcmd_msg_package == NULL) {
        PDEBUG("%s\n", "kvzalloc for vcmd_dec_package fail!");
        return -ENOMEM;
    }

    vcmd_msg_package->header.magic = MAGIC_NUMBER;
    vcmd_msg_package->header.len = sizeof(header_t) + msg_len * 4;
    vcmd_msg_package->header.channel = input_para->vcmdbuf_desc.channel;
    vcmd_msg_package->header.type = 0;
    vcmd_msg_package->header.cmd = DECODE;
    vcmd_msg_package->payload.len = input_para->vcmdbuf_desc.vcmd_size;
    vcmd_msg_package->payload.cmd_id = cmdbuf_id;

    /************************ dma-api ******************************/

    total = input_para->vcmdbuf_desc.stream_size / 0x400000;
    if (input_para->vcmdbuf_desc.stream_size % 0x400000 != 0) {
        total++;
    }

    dma_buf = kmalloc((total + 1) * sizeof(struct vastai_dmadesc), GFP_KERNEL);
    if (dma_buf == NULL) {
        kfree(vcmd_msg_package);
        PDEBUG("%s\n", "kvzalloc for vcmd_dec_package fail!");
        return -ENOMEM;
    }

    triger_core.decode = BIT(input_para->core_id);
    dma_buf[VCMD].is_src_not_user_mem = 0;
    dma_buf[VCMD].is_src_dma_addr = 0;
    dma_buf[VCMD].is_host_to_dev = 1;
    dma_buf[VCMD].host_addr.vir_addr = (void *)(input_para->vcmdbuf_desc.vcmdbuf_src_addr);
    dma_buf[VCMD].dev_addr = cmdbuf_obj->cmdbuf_busAddress;
    dma_buf[VCMD].dma_lenth = (cmdbuf_obj->cmdbuf_size + 127)&(~127);
    dma_buf[VCMD].user_prive[TO_MCU_DATA_TYPE] = VCMD;
    dma_buf[VCMD].user_prive[TO_MCU_CMDBUF_LEN] = cmdbuf_obj->cmdbuf_size;

    while (count < total) {
        dma_buf[STREAM + count].is_src_not_user_mem = 0;
        dma_buf[STREAM + count].is_src_dma_addr = 0;
        dma_buf[STREAM + count].is_host_to_dev = 1;
        dma_buf[STREAM + count].host_addr.vir_addr =
            (void *)(input_para->vcmdbuf_desc.stream_src_addr + count * 0x400000);
        dma_buf[STREAM + count].dma_lenth =
            (count == total - 1) ? ((((input_para->vcmdbuf_desc.stream_size - 1) % 0x400000 + 1)+127)&(~127))
                                 : 0x400000;
        dma_buf[STREAM + count].dev_addr =
            input_para->vcmdbuf_desc.stream_dst_addr + count * 0x400000;
        dma_buf[STREAM + count].user_prive[TO_MCU_DATA_TYPE] = STREAM;
        dma_buf[STREAM + count].user_prive[TO_MCU_CMDBUF_LEN] = 0;
        count++;
    }

    /* transfer vcmd buffer and stream data to device through DMA. */
    retry = 5;
    while (ret && retry) {
        ret = vastai_pci_dma_transfer_sync(pcie_dev, die_info->die_id, triger_core, dma_buf,
                                           total + 1);
        if (ret != 0) {
            if (ret == -EBUSY) {
                printk(KERN_WARNING "warning va_pci_dma_transfer_sync: ret = %d !!\n", ret);
                mdelay(200);
                retry--;
            } else {
                printk(KERN_ERR "error va_pci_dma_transfer_sync: ret = %d !!\n", ret);
                break;
            }
        }
    }

    kfree(dma_buf);
    if (ret) {
        kfree(vcmd_msg_package);
        printk(KERN_ERR "error transfer vcmd buf and stream buf, ret = %d !!\n", ret);
        return ret;
    }

    /* send msg to vdmcu through CSRAM. */
#if VASTAI_DEC_USING_PCIE_FUNCTION
    if (ONLY_1_PF == pcie_dev->fn_mode) {
        ret = vastai_pci_send_msg(pcie_dev, 0,
                                  core_id ? ONLY_1PF_HOST_TO_VDMCU1_CMD_BUF
                                          : ONLY_1PF_HOST_TO_VDMCU0_CMD_BUF,
                                  vcmd_msg_package, sizeof(host_to_vdmcu_msg_t));
    } else if (ONLY_2_PF == pcie_dev->fn_mode) {
        ret = vastai_pci_send_msg(pcie_dev, 0, ONLY_2PF_HOST_TO_VDMCU_CMD_BUF, vcmd_msg_package,
                                  sizeof(host_to_vdmcu_msg_t));
    } else if (ONLY_4_PF == pcie_dev->fn_mode) {
        ret = vastai_pci_send_msg(pcie_dev, 0, COMMON_HOST_TO_VDMCU_CMD_BUF, vcmd_msg_package,
                                  sizeof(host_to_vdmcu_msg_t));
    }
#else
    if (ONLY_1_PF == pcie_dev->fn_mode) {
        mutex_lock(&die_info->host2vdmcu_ringbuffer_mutex[core_id]);
        ret = vastai_video_fifo_push_elem(pcie_dev, 0,
					die_info->host2vdmcu_ringbuffer[core_id]->ring_buf_base, (void *)vcmd_msg_package,NULL);
        mutex_unlock(&die_info->host2vdmcu_ringbuffer_mutex[core_id]);
    } else {
        mutex_lock(&die_info->host2vdmcu_ringbuffer_mutex[0]);
        ret = vastai_video_fifo_push_elem(pcie_dev, 0,
					die_info->host2vdmcu_ringbuffer[0]->ring_buf_base, (void *)vcmd_msg_package,NULL);
        mutex_unlock(&die_info->host2vdmcu_ringbuffer_mutex[0]);
    }
#endif
    if (ret != 0)
        printk(KERN_ERR "error send msg ret = %d !!\n", ret);
    
    kfree(vcmd_msg_package);
    return ret;
}

static int check_cmdbuf_irq(struct vastaivcmd_dev *dev, struct cmdbuf_obj *cmdbuf_obj,
                            u32 *irq_status_ret)
{
    int rdy = 0;
    if (cmdbuf_obj->cmdbuf_run_done != 0) {
        rdy = 1;
        cmdbuf_obj->cmdbuf_run_done = 0;
        *irq_status_ret = cmdbuf_obj->executing_status; // need to decide how to assign this
                                                        // variable
    }
    return rdy;
}

static unsigned int wait_cmdbuf_ready(struct file *filp, u32 cmdbuf_id, u32 core_id,
                                      u32 *status_ret, struct die_dec_info *die)
{
    struct cmdbuf_obj *cmdbuf_obj = NULL;
    int ret = 0;
    struct vastaivcmd_dev *dev = NULL;

    PDEBUG("cmdbuf_id=%d, HZ=%d\n", cmdbuf_id, HZ);
    dev = &die->vastaivcmd_data[core_id];
    if (cmdbuf_id != ANY_CMDBUF_ID) {
        cmdbuf_obj = &(die->vastaivcmd_data[core_id].cmdbuf_obj_sets[cmdbuf_id]);
        if (cmdbuf_obj->filp != filp) {
            // should not happen
            printk(KERN_ERR "wait_cmdbuf_ready:%d ERROR cmdbuf_id=%d !!\n",
                   __LINE__, cmdbuf_id);
            return -1;
        }
#if 0
	  ret = wait_for_completion_interruptible_timeout(&cmdbuf_obj->vcmd_process_comp,10 * HZ / 100);
	  if (ret == 0) {
		  PDEBUG("wait vcmd completion timeout core_id=%d cmdbuf_id=%d\n", core_id, cmdbuf_id );
		  //BUG_ON(1);
		  int i;
		 // for (i = 0; i < 3; i++ )
		  //	spin_lock(&die->vastaivcmd_data[i].vcmd_cmdbuf_alloc_lock);
		  //panic("oops");
	  }
#endif
        ret = wait_event_interruptible_timeout(
            dev->wait_queue, check_cmdbuf_irq(dev, cmdbuf_obj, status_ret), 10 * HZ);
        if (ret < 0) {
            pr_err("vcmd_wait_queue_0 interrupted\n");
            printk("\n\nvcmd_wait_queue_0 error\n\n\n");
            return -1;
        } else if (ret == 0) {
            // mutex_lock(&dev->mutex);
            pr_err("wait event timeout, die_id[0x%x], core[%d], cmdbuf_id=%d\n", die->die_id,
                   core_id, cmdbuf_id);
            cmdbuf_obj->executing_status = CMDBUF_EXE_STATUS_OK;
            return -1;
        }
        *status_ret = cmdbuf_obj->executing_status; // need to decide how to assign this variable
        PDEBUG("return,ret=%d, cmdbufid=%d, status=%d\n", ret, cmdbuf_id,
               *status_ret);
        return 0;
    } else {
        return 0;
    }
}

static int vastai_release_core_channel(struct die_dec_info* die, struct file *filp)
{
	struct list_head *p = NULL, *n = NULL;
	channel_desc_t *channel_desc = NULL;
	int i = 0;

	//spin_lock(&die->cfg_core_lock);
	mutex_lock(&die->allocate_core_mutex);
	list_for_each_safe(p, n, &die->die_channel_head) {
		channel_desc = list_entry(p, channel_desc_t, list);
		if (channel_desc->filp == filp) {
			list_del(&channel_desc->list);
			die->die_channels--;
			i = channel_desc->channel_info.core_id;
			die->dec_res.capacity[i] -= channel_desc->dec_ratio;
			PDEBUG("channel release: channel[%d] ---> core[%d], cap=%d\n",channel_desc->channel_info.channel,
					channel_desc->channel_info.core_id, die->dec_res.capacity[i]);
			kfree(channel_desc);
		}
	}
	//spin_unlock(&die->cfg_core_lock);
	mutex_unlock(&die->allocate_core_mutex);
	return 0;
}


static int list_available_channel(u32 channel, struct die_dec_info* die)
{
	struct list_head *p = NULL;
	channel_desc_t *channel_desc = NULL;

	list_for_each(p, &die->die_channel_head) {
		channel_desc = list_entry(p, channel_desc_t, list);
		if (channel_desc->channel_info.channel == channel)
			return -1;
	}

	return 0;
}

static int vastai_find_core(int dec_ratio, struct die_dec_info* die)
{
	int capacity = MAX_DECODE_CAPACITY;
	int i, index = 0, ret = 0;
	dec_res_t *dec_res = NULL;
	dec_res = &die->dec_res;

	for (i = 0; i < die->core_num; i++) {
		if (capacity > dec_res->capacity[i]) {
			index = i;
			capacity = dec_res->capacity[i];
		}
	}

	if (MAX_DECODE_CAPACITY < (capacity + dec_ratio)) {
		pr_err("already beyond the max capacity.\n");
		ret = -1;
		goto exit;
	}

	capacity += dec_ratio;
	PDEBUG("get core_id = %d, cur_capacity: %d\n", index, capacity);

	ret = index;

exit:
	return ret;
}

static int vastai_get_core_channel(channel_info_t *channel_info, struct die_dec_info *die, struct file *filp)
{
	int ret = 0, dec_ratio = 0;
	channel_desc_t *channel_desc = NULL;
	u32 channel_id = 0;

	dec_ratio = ((channel_info->width * channel_info->height) * 1000 / BASE_DECODE_RESOLUTION);
	if(channel_info->width * channel_info->height >= MAX_SUPPORTED_RESOLUTION) {
		dec_ratio =  MAX_DECODE_CAPACITY;
	}

	channel_desc = kzalloc(sizeof(channel_desc_t), GFP_KERNEL);
	channel_desc->dec_ratio = dec_ratio;

	//spin_lock(&die->cfg_core_lock);
	mutex_lock(&die->allocate_core_mutex);
	/* find the core that has the minimum loading */
	ret = vastai_find_core(dec_ratio, die);
	if (ret < 0) {
		pr_err("cannot find suitable res to allocate.\n");
		ret = -1;
		goto exit;
	}

	die->dec_res.capacity[ret] += dec_ratio;
	channel_id = die->die_channels;
	while (list_available_channel(channel_id, die) < 0)
		channel_id++;
	channel_info->channel = channel_id;
	channel_info->core_id = ret;

	PDEBUG("Done: channel=%d ---> core=%d\n", channel_info->channel, channel_info->core_id);

	channel_desc->channel_info = *channel_info;
	channel_desc->filp = filp;
	list_add(&channel_desc->list, &die->die_channel_head);
	die->die_channels++;
	//spin_unlock(&die->cfg_core_lock);
	mutex_unlock(&die->allocate_core_mutex);

	return 0;

exit:
	//spin_unlock(&die->cfg_core_lock);
	mutex_unlock(&die->allocate_core_mutex);
	kfree(channel_desc);
	return ret;
}

static void vastai_get_available_channels(channels_t *channels, struct die_dec_info *die, struct file *filp)
{
	int i = 0, dec_ratio = 0;

	PDEBUG("Get channels, width=%d, height=%d, die id=%d, core_num=%d.\n", channels->width, channels->height,
																		   die->die_id, die->core_num);
	dec_ratio = ((channels->width * channels->height) * 1000 / BASE_DECODE_RESOLUTION);
	if(channels->width * channels->height >= MAX_SUPPORTED_RESOLUTION) {
		dec_ratio =  MAX_DECODE_CAPACITY;
	}

	for (i = 0; i < die->core_num; i++) {
		if (MAX_DECODE_CAPACITY < die->dec_res.capacity[i]) {
			continue;
		}
		channels->channels[i] = (MAX_DECODE_CAPACITY - die->dec_res.capacity[i])/dec_ratio;
	};
}

long vastai_dec_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
    int ret = 0, count = 0, total = 0;
    static int last_polling_cmd = 0;
//	struct drm_file *file_priv = filp->private_data;
	dec_device_handle_t *device_handle = NULL;
	struct vastai_video_info *video_info = NULL;
	struct die_dec_info *die = NULL;
	struct vastaivcmd_dev *vastaivcmd_data = NULL;
	struct vastai_pci_info *pcie_dev = NULL;
	struct vastai_dmadesc *dma_buf = NULL;
	union core_bitmap triger_core = {{0}};

    if (cmd != VASTAI_VCMD_IOCH_POLLING_CMDBUF) {
      last_polling_cmd = 0;
      PDEBUG("ioctl cmd 0x%08x\n", cmd);
    } else {
      //if (!last_polling_cmd)
      //  PDEBUG("ioctl cmd 0x%08x\n", cmd);
      last_polling_cmd = 1;
    }
    /*
     * extract the type and number bitfields, and don't encode
     * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
     */
    if(_IOC_TYPE(cmd) != VASTAI_VCMD_IOC_MAGIC )
        return -ENOTTY;
    if((_IOC_TYPE(cmd) == VASTAI_VCMD_IOC_MAGIC &&
        _IOC_NR(cmd) > VASTAI_VCMD_IOC_MAXNR)  )
        return -ENOTTY;
    /*
     * the direction is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. `Type' is user-oriented, while
     * access_ok is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */

    //device_handle = container_of(file_priv->minor->dev, dec_device_handle_t, drm_dev);
    if (strncmp(filp->f_path.dentry->d_iname, VASTAI, sizeof(VASTAI) - 1)) {
		printk("not define drm\n");
		return -EFAULT;
    } else {
		video_info = (struct vastai_video_info *)filp->private_data;
		device_handle = video_info->dec_device_handle;
    }



    die = &device_handle->die;
    pcie_dev = (struct vastai_pci_info *)die->vcmd_pri;
    vastaivcmd_data = die->vastaivcmd_data;
	/* core_id = vastaivcmd_data->core_id; */

    switch (cmd) {
    case VASTAI_VCMD_IOCH_GET_CMDBUF_PARAMETER:
      {
       struct cmdbuf_mem_parameter local_cmdbuf_mem_data = {0};
	   u16 core_id = -1;

	   if (video_copy_from_user(&local_cmdbuf_mem_data, (struct cmdbuf_mem_parameter*)arg, sizeof(struct cmdbuf_mem_parameter))) {
		   return -EFAULT;
	   }
	   core_id = local_cmdbuf_mem_data.vcmd_core_id;
	   PDEBUG("core[%d]: get vcmdbuf param!!\n", core_id);

       local_cmdbuf_mem_data.cmdbuf_unit_size = CMDBUF_MAX_SIZE;
       local_cmdbuf_mem_data.status_cmdbuf_unit_size = CMDBUF_MAX_SIZE;
       local_cmdbuf_mem_data.cmdbuf_total_size = CMDBUF_POOL_TOTAL_SIZE;
       local_cmdbuf_mem_data.status_cmdbuf_total_size = CMDBUF_POOL_TOTAL_SIZE;
       local_cmdbuf_mem_data.phy_status_cmdbuf_addr = die->vastaivcmd_data[core_id].vcmd_status_buf_mem_pool.busAddress;
       local_cmdbuf_mem_data.phy_cmdbuf_addr = die->vastaivcmd_data[core_id].vcmd_buf_mem_pool.busAddress;

       local_cmdbuf_mem_data.mmu_phy_status_cmdbuf_addr = 0;
       local_cmdbuf_mem_data.mmu_phy_cmdbuf_addr = 0;
       local_cmdbuf_mem_data.vcmd_reg_mem_busAddress = 0;
       local_cmdbuf_mem_data.base_ddr_addr = 0;

	   local_cmdbuf_mem_data.vcmd_reg_mem_busAddress = vastaivcmd_data[core_id].vcmd_reg_mem_busAddress;

	   if (video_copy_to_user((struct cmdbuf_mem_parameter*)arg, &local_cmdbuf_mem_data, sizeof(struct cmdbuf_mem_parameter))) {
		   return -EFAULT;
	   }
       break;
      }
    case VASTAI_VCMD_IOCH_GET_VCMD_PARAMETER:
      {
       struct config_parameter input_para = {0};
       PDEBUG(" VCMD get vcmd config parameter \n");
	   if (video_copy_from_user(&input_para,(struct config_parameter*)arg,sizeof(struct config_parameter))) {
		   return -EFAULT;
	   }
       if(die->vcmd_type_core_num[input_para.module_type])
       {
         input_para.submodule_main_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_main_addr;
         input_para.submodule_dec400_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_dec400_addr;
         input_para.submodule_L2Cache_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_L2Cache_addr;
         input_para.submodule_MMU_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMU_addr;
         input_para.submodule_MMUWrite_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMUWrite_addr;
         input_para.submodule_axife_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_axife_addr;
         input_para.config_status_cmdbuf_id = die->vcmd_manager[input_para.module_type][0]->status_cmdbuf_id;
         input_para.vcmd_hw_version_id = die->vcmd_manager[input_para.module_type][0]->hw_version_id;
         input_para.vcmd_core_num = die->vcmd_type_core_num[input_para.module_type];
       }
       else
       {
         input_para.submodule_main_addr = 0xffff;
         input_para.submodule_dec400_addr = 0xffff;
         input_para.submodule_L2Cache_addr = 0xffff;
         input_para.submodule_MMU_addr = 0xffff;
         input_para.submodule_MMUWrite_addr = 0xffff;
         input_para.submodule_axife_addr = 0xffff;
         input_para.config_status_cmdbuf_id = 0;
         input_para.vcmd_core_num = 0;
         input_para.vcmd_hw_version_id =HW_ID_1_0_C;
       }
	   if (video_copy_to_user((struct config_parameter*)arg, &input_para, sizeof(struct config_parameter))) {
		   return -EFAULT;
	   }
       break;
      }
    case VASTAI_VCMD_IOCH_RESERVE_CMDBUF:
      {
       int ret = 0;
       struct exchange_parameter input_para = {0};
	   if (video_copy_from_user(&input_para, (struct exchange_parameter*)arg, sizeof(struct exchange_parameter))) {
		   return -EFAULT;
	   }
       PDEBUG("----core[%d]: VCMD Reserve CMDBUF\n", input_para.core_id);

       ret = reserve_cmdbuf(filp, die, &input_para);
	   if (ret == 0) {
		   if (video_copy_to_user((struct exchange_parameter*)arg, &input_para, sizeof(struct exchange_parameter))) {
			   return -EFAULT;
		   }
	   }
       return ret;
      }

    case VASTAI_VCMD_IOCH_LINK_RUN_CMDBUF:
      {
        struct exchange_parameter input_para = {0};
        long retVal = 0;
        PDEBUG(" VCMD link and run cmdbuf\n");

		if (video_copy_from_user(&input_para,(struct exchange_parameter*)arg, sizeof(struct exchange_parameter))) {
			return -EFAULT;
		}

        retVal = link_and_run_cmdbuf(filp, &input_para, die);
		if (video_copy_to_user((struct exchange_parameter*)arg,&input_para,sizeof(struct exchange_parameter))) {
			return -EFAULT;
		}
        return retVal;
        break;
      }

    case VASTAI_VCMD_IOCH_WAIT_CMDBUF:
      {
		struct waitcmd_parameter wait_param = {0};
        u32 status_ret = -1, tmp = -1;
        u32 cmdbuf_id = -1;
        u32 core_id = -1;

		if (video_copy_from_user(&wait_param, (struct waitcmd_parameter*)arg, sizeof(struct waitcmd_parameter))) {
			return -EFAULT;
		}
		cmdbuf_id = wait_param.cmdbuf_id;
		core_id = wait_param.core_id;

        PDEBUG("core[%d] wait for CMDBUF. cmdbuf_id=%d\n", core_id, cmdbuf_id);
        //TODO
        tmp = wait_cmdbuf_ready(filp, cmdbuf_id, core_id, &status_ret, die);

		wait_param.status = status_ret;
		if (tmp==0) {
			if (video_copy_to_user((struct waitcmd_parameter*)arg, &wait_param, sizeof(struct waitcmd_parameter))) {
				return -EFAULT;
			}
			return tmp;
		} else {
			wait_param.status = -1;
			if (video_copy_to_user((struct waitcmd_parameter*)arg, &wait_param, sizeof(struct waitcmd_parameter))) {
				return -EFAULT;
			}
			return -1;
		}
		break;
	  }

    case VASTAI_VCMD_IOCH_RELEASE_CMDBUF:
      {
		struct waitcmd_parameter wait_param = {0};
        u16 cmdbuf_id = -1, core_id = -1;

		if (video_copy_from_user(&wait_param, (struct waitcmd_parameter*)arg, sizeof(struct waitcmd_parameter))) {
			return -EFAULT;
		}
        cmdbuf_id = wait_param.cmdbuf_id;
		core_id = wait_param.core_id;

        PDEBUG("release CMDBUF: core[%d]: cmdbuf_id=%d\n", core_id, cmdbuf_id);
        return release_cmdbuf(filp, cmdbuf_id, core_id, die);

      }
#if 0
    case VASTAI_VCMD_IOCH_DUMP_CMDBUF:
      {
        u32 core_id = -1;

		if (video_copy_from_user(&core_id, (void *)arg, sizeof(u32))) {
			return -EFAULT;
		}
        dump_cmdbuf(die, core_id);
        return 0;
      }
#endif
    case VASTAI_VCMD_IOCH_POLLING_CMDBUF:
      {
        //u16 core_id;
        //__get_user(core_id, (u16*)arg);
        /*16 bits are cmdbuf_id*/
        //if(core_id>=total_vcmd_core_num)
          //return -1;
        //vastaivcmd_isr(core_id,&vastaivcmd_data[core_id]);
        return 0;
        break;
      }
    case VASTAIDEC_IOCX_GET_VCMDCFG: {
		__put_user(vcmd, (unsigned int *)arg);
		break;
	}
    case VASTAI_VCMD_IOCH_GET_FRAMEBUF: {
        framebuf_desc_t frame_desc = {0};
        PDEBUG(" VCMD get_framebuf  \n");
		if (video_copy_from_user(&frame_desc, (framebuf_desc_t*)arg, sizeof(framebuf_desc_t))) {
		    return -EFAULT;
		}

		/****************  dma-get-stream *************/
        if (frame_desc.stream_size == 0) {
            return -EFAULT;
        }

        total = frame_desc.stream_size / 0x400000;
        if (frame_desc.stream_size % 0x400000 != 0) {
            total++;
        }

        dma_buf = vmalloc((total + 1) * sizeof(struct vastai_dmadesc));
		if (NULL == dma_buf) {
            PDEBUG("Alloc mem for dma failed!\n");
            return -ENOMEM;
        }
        /* union core_bitmap triger_core = {0}; */
        triger_core.decode = 0;

        while (count < total) {
            dma_buf[count].is_src_not_user_mem = 0;
            dma_buf[count].is_src_dma_addr = 0;
            dma_buf[count].is_host_to_dev = 0;
            dma_buf[count].host_addr.vir_addr =
                (void *)(frame_desc.stream_dst_addr + count * 0x400000);
            dma_buf[count].dev_addr = frame_desc.stream_src_addr + count * 0x400000;
            dma_buf[count].dma_lenth =
                (count == total - 1) ? ((frame_desc.stream_size - 1) % 0x400000 + 1) : 0x400000;
            PDEBUG("%d: src=%lx, dst=%lx, size=%d\n", __LINE__,
                   (long unsigned int)dma_buf[count].host_addr.vir_addr,
                   (long unsigned int)dma_buf[count].dev_addr, dma_buf[count].dma_lenth);
            count++;
        }

        ret = vastai_pci_dma_transfer_sync(pcie_dev, die->die_id, triger_core, dma_buf, total);
        if (ret != 0) {
            printk(KERN_ERR "va_pci_dma_transfer_sync: ERROR ret = %d !!\n", ret);
        }
        vfree(dma_buf);
        return ret;
	}
    case VASTAI_VCMD_IOCH_GET_ID_PARAMETER:{
		u16 core_id = -1;
		vcmd_id_parameter id_params = {0};
		u32 reg[4] = {0};
		if (video_copy_from_user(&id_params, (vcmd_id_parameter*)arg, sizeof(vcmd_id_parameter))) {
			return -EFAULT;
		}

		core_id = id_params.vcmd_core_id;
		if (core_id < 0 || core_id > 4) {
			pr_err("invalid core_id\n");
			return -EFAULT;;
		}

		PDEBUG("core[%d]:vcmd_get_id_parameter.\n", core_id);

		vastai_video_read_pcie(die->vcmd_pri, die->die_id, (void*)&reg, (void*)VDEC_REPORT_ID_ADDR[core_id], sizeof(reg));
		id_params.decoderAsicId  = reg[0];
		id_params.hwBuildId      = reg[1];
		id_params.l2CacheVersion = reg[2];

		PDEBUG("asicid=0x%x, buildid=0x%x, cachever=0x%x", id_params.decoderAsicId, id_params.hwBuildId, id_params.l2CacheVersion);
		/* id_params.vcmd_die_id = die->die_id; */
		if (video_copy_to_user((vcmd_id_parameter*)arg, &id_params, sizeof(vcmd_id_parameter))) {
			return -EFAULT;
		}
        break;
    }

	case VASTAI_CONFIG_CORE_CHANNEL:
	{
		channel_info_t channel_info = {0};
		int tmp = -1;
        union die_index_data  die_data = {0};

		PDEBUG("config core channel id.\n");

		tmp = video_copy_from_user(&channel_info, (void*)arg, sizeof(channel_info_t));
		if (tmp) {
			pr_err("config core channel err: video_copy_from_user failed, returned %d\n", tmp);
			return -EFAULT;
		}

		tmp = vastai_get_core_channel(&channel_info, die, filp);
		if (tmp) {
			pr_err("config core channel failed, returned %d\n", tmp);
			return -EFAULT;
		}
		// channel_info.die_id = die->die_id;        
        die_data.dev_id = ((struct vastai_pci_info *)die->vcmd_pri)->dev_id;
        die_data.RESV = ((struct vastai_pci_info *)die->vcmd_pri)->dev_id;
        channel_info.die_id = die_data.val;
        
		if (video_copy_to_user((u32 *) arg, &channel_info, sizeof(channel_info_t))) {
			return -EFAULT;
		}
		break;
	}

	case VASTAI_RELEASE_CORE_CHANNEL:
	{
		u32 tmp = -1;
		tmp = vastai_release_core_channel(die, filp);
		if (tmp) {
			pr_err("release core channel failed, returned %d\n", tmp);
			return -EFAULT;
		}
		break;
	}

	case VASTAI_GET_AVAILABLE_CHANNELS:
	{
		channels_t channels = {0};
		int tmp = -1;

		PDEBUG("get available channels.\n");

		tmp = video_copy_from_user(&channels, (void*)arg, sizeof(channels_t));
		if (tmp) {
			pr_err("get available channels err: video_copy_from_user failed, returned %d\n", tmp);
			return -EFAULT;
		}

		vastai_get_available_channels(&channels, die, filp);

		if (video_copy_to_user((u32 *)arg, &channels, sizeof(channels_t))) {
			return -EFAULT;
		}
		break;
	}
	case VASTAI_DEC_GET_DIE_ID:
	{
		int tmp = -1;

		union die_index_data  die_data;
		die_data.dev_id = ((struct vastai_pci_info *)die->vcmd_pri)->dev_id;
		die_data.RESV = ((struct vastai_pci_info *)die->vcmd_pri)->dev_id;
		die->die_id = die_data.val;

		tmp = video_copy_to_user((u32 *)arg, &(die->die_id), sizeof(int));
		if (tmp) {
		    pr_err("video_copy_to_user failed, returned %d\n", tmp);
		    return -EFAULT;
		}
		break;
	}

    case VASTAI_VCMD_IOCH_READWRITE_DEVICE_MEM:{
        transfer_mem_param mem_params = {0};
        void* temp_buffer = NULL;
        long tmp = 0;
        int ret = 0;
        tmp = video_copy_from_user(&mem_params, (void*)arg, sizeof(transfer_mem_param));
        if(tmp){
            PDEBUG("video_copy_from_user failed, ioctl: %lx, returned %li\n", VASTAI_VCMD_IOCH_READWRITE_DEVICE_MEM, tmp);
            return -EFAULT;
        }

        if(!mem_params.bus_addr || !mem_params.local_addr) {
            PDEBUG("param is invalid, return!\n");
            return -EINVAL;
        }

        temp_buffer = vmalloc(mem_params.len);
        if (NULL == temp_buffer) {
            PDEBUG("Alloc mem failed!\n");
            return -ENOMEM;
        }
        if (mem_params.dir == VASTAI_MEM_READ) {
            //read from device to host;
            vastai_video_read_pcie(die->vcmd_pri, die->die_id, temp_buffer, (void*)mem_params.bus_addr, mem_params.len);
            tmp = video_copy_to_user((void __user *)mem_params.local_addr, temp_buffer, mem_params.len);
            if(tmp){
                PDEBUG("video_copy_to_user failed, ioctl: %lx, returned %li\n", VASTAI_VCMD_IOCH_READWRITE_DEVICE_MEM, tmp);
                ret = -EFAULT;
            }
        } else if(mem_params.dir == VASTAI_MEM_WRITE){
            //write from host to device
            tmp = video_copy_from_user(temp_buffer, (void __user *)mem_params.local_addr, mem_params.len);
            if(tmp){
                PDEBUG("video_copy_from_user failed, ioctl: %lx, returned %li\n", VASTAI_VCMD_IOCH_READWRITE_DEVICE_MEM, tmp);
                ret = -EFAULT;
                goto end;
            }
            vastai_video_write_pcie(die->vcmd_pri, die->die_id, temp_buffer, (void *)mem_params.bus_addr, mem_params.len);
        } else {
            PDEBUG("invalid param! dir=%d\n", mem_params.dir);
            ret = -EINVAL;
        }
    end:
        vfree(temp_buffer);
        return ret;
    }

    default:
        break;
    }
    return 0;
}

int vastai_dec_open(struct inode *inode, struct file *filp)
{
    return 0;
}

int vastai_dec_release(struct file *filp, struct die_dec_info* die)
{
	struct vastaivcmd_dev *dev = die->vastaivcmd_data;
	u32 core_id = 0, cmdbuf_id = 0;
	u32 release_cmdbuf_num=0;
	struct cmdbuf_obj* cmdbuf_obj_temp = NULL;

	PDEBUG("va_dec_release %p\n", (void *)filp);

	for (core_id = 0; core_id < die->core_num; core_id++) {
		if ((&dev[core_id]) == NULL)
			continue;
		for (cmdbuf_id = dev->cmdbuf_offset; cmdbuf_id < dev->total_vcmd_num + dev->cmdbuf_offset; cmdbuf_id++) {
			cmdbuf_obj_temp = &(dev[core_id].cmdbuf_obj_sets[cmdbuf_id]);
			if (cmdbuf_obj_temp->filp == filp) {
				clean_cmdbuf_obj(cmdbuf_obj_temp, cmdbuf_id, core_id, die);
				release_cmdbuf_num++;
			}
		}
	}
	vastai_release_core_channel(die, filp);

	return 0;
}


/*------------------------------------------------------------------------------
 Function name   : vcmd_pcie_init
 Description     : Initialize PCI Hw access

 Return type     : int
 ------------------------------------------------------------------------------*/
static int vcmd_init(struct die_dec_info *die_info, int vcmd_core_offset)
{
	int i = 0;
	u64 core_busaddress[VASTAI_DEC_MAX_CORES] = {VCMD_DEC_DDR_BASE_ADDR_0, VCMD_DEC_DDR_BASE_ADDR_1};

	if (die_info->fn_mode == PF_4_WITH_SRIOV && die_info->decoder_sriov_en) {
		core_busaddress[0] = VCMD_DEC_DDR_BASE_ADDR_0_SRIOV;
		core_busaddress[1] = VCMD_DEC_DDR_BASE_ADDR_1_SRIOV;
	}

	for (i = 0; i < die_info->core_num; i++) {
		die_info->vastaivcmd_data[i].vcmd_buf_mem_pool.busAddress = core_busaddress[i + vcmd_core_offset];
		die_info->vastaivcmd_data[i].vcmd_buf_mem_pool.size = CMDBUF_POOL_TOTAL_SIZE;

		printk(KERN_INFO"VCMD[%d] buf mem addr=0x%lx, size=0x%08x\n", i, (unsigned long)die_info->vastaivcmd_data[i].vcmd_buf_mem_pool.busAddress,
				die_info->vastaivcmd_data[i].vcmd_buf_mem_pool.size);

		die_info->vastaivcmd_data[i].vcmd_status_buf_mem_pool.busAddress = core_busaddress[i + vcmd_core_offset] + CMDBUF_POOL_TOTAL_SIZE;
		die_info->vastaivcmd_data[i].vcmd_status_buf_mem_pool.size = CMDBUF_POOL_TOTAL_SIZE;

		printk(KERN_INFO "VCMD[%d] status buf mem addr=0x%lx, size=0x%08x\n", i, (unsigned long)die_info->vastaivcmd_data[i].vcmd_status_buf_mem_pool.busAddress,
				die_info->vastaivcmd_data[i].vcmd_status_buf_mem_pool.size);

		die_info->vastaivcmd_data[i].vcmd_registers_mem_pool.busAddress = core_busaddress[i + vcmd_core_offset]  + CMDBUF_POOL_TOTAL_SIZE * 2;
		die_info->vastaivcmd_data[i].vcmd_registers_mem_pool.size = CMDBUF_VCMD_REGISTER_TOTAL_SIZE;

		printk(KERN_INFO "VCMD[%d] registers mem addr=0x%lx, size=0x%08x\n", i, (unsigned long)die_info->vastaivcmd_data[i].vcmd_registers_mem_pool.busAddress,
				die_info->vastaivcmd_data[i].vcmd_registers_mem_pool.size);

	}
    return 0;
}

int vastai_dec_init(struct die_dec_info *die_info)
{
    int i = 0, k = 0, vcmd_core_offset = 0;
    int result = 0;
    struct kernel_addr_desc;
    struct vastaivcmd_dev *vastaivcmd_data = NULL;
    unsigned int hwid = 0x43421101; // for dbg
    struct vastai_pci_info *pcie_dev = (struct vastai_pci_info *)die_info->vcmd_pri;
    u16 total_vcmd_num = TOTAL_DISCRETE_CMDBUF_NUM;
    u16 vcmd_offset = 0;

    if (die_info->fn_mode != ONLY_1_PF && die_info->fn_mode != ONLY_2_PF &&
        die_info->fn_mode != ONLY_4_PF && die_info->fn_mode != PF_4_WITH_SRIOV) {
        printk(KERN_ERR "Video decoder do not support current fn_mode : %d\n", die_info->fn_mode);
        return -1;
    }

    CheckSubsysCoreArray(vpu_subsys, &vcmd);

    vastaivcmd_data = die_info->vastaivcmd_data =
        (struct vastaivcmd_dev *)vmalloc(sizeof(struct vastaivcmd_dev) * total_vcmd_core_num);
    if (vastaivcmd_data == NULL) {
        printk(KERN_ERR "[dec] Fail to malloc vcmd data for pf [%d] in fn_mod [%d]\n", die_info->pfn,
               die_info->fn_mode);
        goto err;
    }

    memset(vastaivcmd_data, 0, sizeof(struct vastaivcmd_dev) * total_vcmd_core_num);

    if (ONLY_1_PF == die_info->fn_mode) {
        vcmd_core_offset = 0;
#if VASTAI_DEC_USING_PCIE_FUNCTION
        vastaivcmd_data->vdec_buf_index[0] = ONLY_1PF_VDMCU0_TO_HOST_INFO_BUF;
        vastaivcmd_data->vdec_buf_index[1] = ONLY_1PF_VDMCU1_TO_HOST_INFO_BUF;
#else
        vastaivcmd_data->vdmcu2host_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, ONLY_1_PF_VDMCU0_WR_MSIX_INFO_HOST_RD_NAME);
        vastaivcmd_data->vdmcu2host_ringbuffer[1] = find_hw_cfg_acord_name(pcie_dev, ONLY_1_PF_VDMCU1_WR_MSIX_INFO_HOST_RD_NAME);
        die_info->host2vdmcu_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VDMCU0_RD_NAME);
        die_info->host2vdmcu_ringbuffer[1] = find_hw_cfg_acord_name(pcie_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VDMCU1_RD_NAME);
#endif
        result = va_register_pcie_interrupt(pcie_dev, ONLY_1_PF_MSIX_VDMCU_0_VECTOR,
                                            vastai_pci_done_core0_callback, die_info);
        if (result) {
            printk(KERN_ERR "[dec] Fail to register pcie interrupt for core0 in 1 PF mode\n");
            goto err;
        }

        result = va_register_pcie_interrupt(pcie_dev, ONLY_1_PF_MSIX_VDMCU_1_VECTOR,
                                            vastai_pci_done_core1_callback, die_info);
        if (result) {
            printk(KERN_ERR "[dec] Fail to register pcie interrupt for core1 in 1 PF mode\n");
            goto err;
        }
    } else if (ONLY_2_PF == die_info->fn_mode) {
        vcmd_core_offset = die_info->pfn % 2;
#if VASTAI_DEC_USING_PCIE_FUNCTION
        vastaivcmd_data->vdec_buf_index[0] = ONLY_2PF_VDMCU_TO_HOST_INFO_BUF;
#else
        vastaivcmd_data->vdmcu2host_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, ONLY_2_PF_VDMCU_WR_MSIX_INFO_HOST_RD_NAME);
        die_info->host2vdmcu_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, ONLY_2_PF_HOST_WR_M2O_CMD_VDMCU_RD_NAME);
#endif
        result = va_register_pcie_interrupt(pcie_dev, ONLY_2_PF_MSIX_VDMCU_VECTOR,
                                            vastai_pci_done_core0_callback, die_info);
        if (result) {
            printk(KERN_ERR "[dec] Fail to register pcie interrupt in 2 PF mode\n");
            goto err;
        }
        total_vcmd_core_num = 1;
    } else if (ONLY_4_PF == die_info->fn_mode) {
        vcmd_core_offset = (die_info->pfn / 2) % 2; // TODO : check multi-die cases
#if VASTAI_DEC_USING_PCIE_FUNCTION
        vastaivcmd_data->vdec_buf_index[0] = COMMON_VDMCU_TO_HOST_INFO_BUF;
#else
        vastaivcmd_data->vdmcu2host_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, COMMON_VDMCU_WR_MSIX_INFO_HOST_RD_NAME);
        die_info->host2vdmcu_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, COMMON_HOST_WR_M2O_CMD_VDMCU_RD_NAME);
#endif
        result = va_register_pcie_interrupt(pcie_dev, COMMON_MSIX_VDMCU_VECTOR,
                                            vastai_pci_done_core0_callback, die_info);
        if (result) {
            printk(KERN_ERR "[dec] Fail to register pcie interrupt in 4 PF mode\n");
            goto err;
        }
        total_vcmd_num = TOTAL_DISCRETE_CMDBUF_NUM / 2;
        if (die_info->pfn % 2)
            vcmd_offset = total_vcmd_num;
        total_vcmd_core_num = 1;
    } else if (PF_4_WITH_SRIOV == die_info->fn_mode && die_info->decoder_sriov_en) {
        vcmd_core_offset = (die_info->pfn / 2) % 2; // TODO : check multi-die cases
#if VASTAI_DEC_USING_PCIE_FUNCTION
        vastaivcmd_data->vdec_buf_index[0] = COMMON_VDMCU_TO_HOST_INFO_BUF;
#else
        vastaivcmd_data->vdmcu2host_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, COMMON_VDMCU_WR_MSIX_INFO_HOST_RD_NAME);
        die_info->host2vdmcu_ringbuffer[0] = find_hw_cfg_acord_name(pcie_dev, COMMON_HOST_WR_M2O_CMD_VDMCU_RD_NAME);
#endif
        result = va_register_pcie_interrupt(pcie_dev, COMMON_MSIX_VDMCU_VECTOR,
                                            vastai_pci_done_core0_callback, die_info);
        if (result) {
            printk(KERN_ERR "[dec] Fail to register pcie interrupt in 4 PF SRIOV mode\n");
            goto err;
        }

        total_vcmd_num = TOTAL_DISCRETE_CMDBUF_NUM / (die_info->total_vf_per_pf * 2);
        vcmd_offset = ((die_info->pfn % 2) * die_info->total_vf_per_pf + die_info->vfn) * total_vcmd_num;

        total_vcmd_core_num = 1;
    }

    die_info->core_num = total_vcmd_core_num;
    printk("vcmd_core_offset=%d, vcmd_offset=%d total_vcmd_num=%d\n", vcmd_core_offset, vcmd_offset, total_vcmd_num);

    result = vcmd_init(die_info, vcmd_core_offset);
    if (result)
        goto err;

    for (k = 0; k < MAX_VCMD_TYPE; k++) {
        die_info->vcmd_type_core_num[k] = 0;
        /* die_info->vcmd_position[k]=0; */
        for (i = 0; i < MAX_VCMD_NUMBER; i++) {
            die_info->vcmd_manager[k][i] = NULL;
        }
    }

    die_info->workqueue = create_workqueue("video_decoder_workqueue");
    for (i = 0; i < total_vcmd_core_num; i++) {
        vastaivcmd_data[i].die = die_info;
        vastaivcmd_data[i].vcmd_core_cfg = vcmd_core_array[i + vcmd_core_offset];
        vastaivcmd_data[i].hwregs = NULL;
        vastaivcmd_data[i].core_id = i;
        vastaivcmd_data[i].working_state = WORKING_STATE_IDLE;
        vastaivcmd_data[i].sw_cmdbuf_rdy_num = 0;
        mutex_init(&vastaivcmd_data[i].mutex);

        INIT_WORK(&(vastaivcmd_data[i].work), vcmd_workqueue_schedue);
        vastaivcmd_data[i].duration_without_int = 0;

        init_waitqueue_head(&die_info->vastaivcmd_data[i].vcmd_cmdbuf_memory_wait);
        init_waitqueue_head(&die_info->vastaivcmd_data[i].wait_queue);

        vastai_cmdid_init(&vastaivcmd_data[i].cmdbufId_queue, DECODER);

        die_info->vcmd_manager
            [vcmd_core_array[i + vcmd_core_offset].sub_module_type]
            [die_info->vcmd_type_core_num[vcmd_core_array[i + vcmd_core_offset].sub_module_type]] =
            &vastaivcmd_data[i];
        die_info->vcmd_type_core_num[vcmd_core_array[i + vcmd_core_offset].sub_module_type]++;
        vastaivcmd_data[i]
            .vcmd_type_core_num[vcmd_core_array[i + vcmd_core_offset].sub_module_type]++;
        vastaivcmd_data[i].vcmd_reg_mem_busAddress =
            die_info->vastaivcmd_data[i].vcmd_registers_mem_pool.busAddress;
        vastaivcmd_data[i].vcmd_reg_mem_size = VCMD_REGISTER_SIZE;
        vastaivcmd_data[i].hw_version_id = hwid;

        die_info->vastaivcmd_data[i].total_vcmd_num = total_vcmd_num;
        die_info->vastaivcmd_data[i].cmdbuf_offset = vcmd_offset;
        die_info->vastaivcmd_data[i].cmdbuf_used_pos = 2 + vcmd_offset;
        die_info->vastaivcmd_data[i].cmdbuf_used_residual = total_vcmd_num - 2;
        die_info->vastaivcmd_data[i].cmdbuf_used[0] = 1;
        die_info->vastaivcmd_data[i].cmdbuf_used[1] = 1;
        die_info->vastaivcmd_data[i].cmdbuf_last_rel_pos = 0; 

#if 0
		for (k = 2; k < TOTAL_DISCRETE_CMDBUF_NUM; k++) {
			init_completion(&die_info->vastaivcmd_data[i].cmdbuf_obj_sets[k].vcmd_process_comp);
		}
#endif
        spin_lock_init(&die_info->vastaivcmd_data[i].vcmd_cmdbuf_alloc_lock);
#if !VASTAI_DEC_USING_PCIE_FUNCTION
        mutex_init(&die_info->host2vdmcu_ringbuffer_mutex[i]);
#endif
    }

    // spin_lock_init(&die_info->cfg_core_lock);
    mutex_init(&die_info->allocate_core_mutex);
    die_info->die_channels = 0;
    INIT_LIST_HEAD(&die_info->die_channel_head);

    (die_info->vcmd_manager[VCMD_TYPE_DECODER][0])->status_cmdbuf_id = 1;

    return 0;
err:
    if (vastaivcmd_data != NULL)
        vfree(vastaivcmd_data);

    printk(KERN_INFO "va_decoder_driver: module not inserted\n");
    return result;
}

void vastai_dec_deinit(void *pcie_dev)
{
	struct list_head *p = NULL, *n = NULL;
	dec_device_handle_t *device_handle_node = NULL;
	struct die_dec_info *die;
    int i;

	pr_info("va_dec_cleanup\n");

	list_for_each_safe(p, n, &dec_device_handle_head) {
		/* release drm_dev */
		device_handle_node = list_entry(p, dec_device_handle_t, node);
		if (pcie_dev == device_handle_node->die.vcmd_pri) {
			pr_info("va_dec_cleanup find die to remove\n");
			list_del(&device_handle_node->node);
			die = &device_handle_node->die;
			if (NULL != die) {
				flush_workqueue(die->workqueue);
				destroy_workqueue(die->workqueue);
#if !VASTAI_DEC_USING_PCIE_FUNCTION
				for (i = 0; i < die->core_num; i++) {
					mutex_destroy(&die->host2vdmcu_ringbuffer_mutex[i]);
				}
#endif
				vfree(die->vastaivcmd_data);
			}

			kfree(device_handle_node);
			device_handle_node = NULL;
		}

	}
	pr_info("va decoder driver removed\n");
}

