#ifndef __VASTAI_DEV_H__
#define __VASTAI_DEV_H__

#include "vastai_vcmd.h"
#include "subsys.h"
#include "vastai_video_utils.h"
#include "hw_config.h"

#define MAGIC_NUMBER		(0x5AA5)

typedef enum cmd {
    INIT,
    DECODE,
    /* START, */
    RESET,
    HANDLE_EXCEPTION,
    // RELEASE, // abort-->release
    MAX_CMD,
} cmd_t;

typedef struct header {
    uint32_t magic;
    uint32_t len;
    uint32_t channel;
    uint32_t type;
    cmd_t cmd;
} header_t;

typedef struct payload {
    union {
        uint32_t status;
        uint32_t len;
    };
    uint32_t cmd_id;
} payload_t;

typedef struct host_to_vdmcu_msg {
    header_t header;
    payload_t payload;
} host_to_vdmcu_msg_t;

typedef struct vdmcu_to_host_msg {
    uint32_t core_id;
    payload_t payload;
} vdmcu_to_host_msg_t;

struct die_dec_info
{
	/* pointer to pcie dev */
	void *vcmd_pri;
	/* die id, start from 0 */
	int die_id;
	/* pointer to vcmd for one die */
	struct vastaivcmd_dev* vastaivcmd_data;
	struct vastaivcmd_dev* vcmd_manager[MAX_VCMD_TYPE][MAX_VCMD_NUMBER];

	int vcmd_type_core_num[MAX_VCMD_TYPE];

	wait_queue_head_t wait_queue_vcmd[MAX_VCMD_NUMBER];
	wait_queue_head_t abort_queue_vcmd[MAX_VCMD_NUMBER];


	/* bi_list_node* global_cmdbuf_node[TOTAL_DISCRETE_CMDBUF_NUM]; */


	wait_queue_head_t mc_wait_queue;
	struct workqueue_struct *workqueue;
	//spinlock_t cfg_core_lock;
	struct mutex allocate_core_mutex;
	struct list_head die_channel_head;
	dec_res_t dec_res;
	u32 die_channels;

	int core_num;

	u8  is_pf;
	u8  is_vf;
	u8  devfn;
	u8  fn_mode;
	u8  pkg_id;
	u8  pfn;
	u8  vfn;
	u8  decoder_sriov_en;
	u8  total_vf_per_pf;


	vastai_video_workmode_t dec_work_mode;
#if !VASTAI_DEC_USING_PCIE_FUNCTION
	struct ring_buf_entry *host2vdmcu_ringbuffer[VASTAI_DEC_MAX_CORES];
	struct mutex host2vdmcu_ringbuffer_mutex[VASTAI_DEC_MAX_CORES];
#endif
};


typedef struct dec_device_handle {
	struct die_dec_info die;
	struct list_head node;
} dec_device_handle_t;


#if 0
typedef struct pci_die_info {
	void *pci_dev;
	void *die_info;
};
#endif

int vastai_dec_open(struct inode *inode, struct file *filp);
int vastai_dec_release(struct file *filp, struct die_dec_info *die);
long vastai_dec_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);

int vastai_dec_init(struct die_dec_info *die);
void vastai_dec_deinit(void *pcie_dev);

#endif
