/*------------------------------------------------------------------------------
--
--           This software is confidential and proprietary and may be used
--            only as expressly authorized by a licensing agreement from
--
--                                Verisilicon.
--
--                       (C) COPYRIGHT 2014 VERISILICON
--                              ALL RIGHTS RESERVED
--
--                   The entire notice above must be reproduced
--                   on all copies and should not be removed.
--
--------------------------------------------------------------------------------
--
--  Abstract : H2 Encoder device driver (kernel module)
--
------------------------------------------------------------------------------*/

#include <linux/version.h>
#include <linux/uaccess.h>
#include <linux/delay.h>
#include <linux/time.h>
#include <linux/poll.h>
#include <linux/timer.h>

#include "vastai_pci.h"
#include "vastai_pcie_public.h"

#include "vastai_enc.h"
#include "vastai_render.h"
#include "vastai_vcmd.h"
#include "vastai_video_utils.h"

//#define VENC_MULTICORE_SUPPORT
#define VENC_RECV_QUEUE_SUPPORT
#define VENC_EMULATOR
//#define VENC_FENCE_TEST

#define VASTAI_ENC_CORE0  VA_VEMCU_INTERRUPT_NUM
#define VASTAI_ENC_CORE1  VASTAI_ENC_CORE0
#define VASTAI_ENC_CORE2  VASTAI_ENC_CORE0
#define VASTAI_ENC_CORE3  VASTAI_ENC_CORE0
#define ENCCORE0_IRQ_BASE VASTAI_ENC_CORE0

#ifdef VENC_EMULATOR
#define VENC_CHANNEL_LOCK_TIMEOUT   1000000
#else
#define VENC_CHANNEL_LOCK_TIMEOUT   1000
#endif

#define FENCE_WAIT_TIMEOUT (3*1000)

#ifdef VENC_MULTICORE_SUPPORT
/*------------------------------------------------------------------------
*****************************VCMD CONFIGURATION BY CUSTOMER********************************
-------------------------------------------------------------------------*/
//video encoder vcmd configuration
#define VASTAI_DDR_BASE_ADDRESS             0x800000000//0x30000000

#define VCMD_ENC_IO_ADDR_0                  0x980000//0x20000   /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_0                  (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_ENC_INT_PIN_0                  -1
#define VCMD_ENC_MODULE_TYPE_0              0
#define VCMD_ENC_MODULE_MAIN_ADDR_0         0x1000	/*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_0       0XFFFF	/*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_0      0XFFFF
#define VCMD_ENC_MODULE_MMU0_ADDR_0         0XFFFF //0X2000
#define VCMD_ENC_MODULE_MMU1_ADDR_0         0XFFFF //0X4000
#define VCMD_ENC_MODULE_AXIFE0_ADDR_0       0XFFFF //0X3000
#define VCMD_ENC_MODULE_AXIFE1_ADDR_0       0XFFFF //0X5000

#define VCMD_ENC_IO_ADDR_1                  0x9a0000   /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_1                  (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_ENC_INT_PIN_1                  -1
#define VCMD_ENC_MODULE_TYPE_1              0
#define VCMD_ENC_MODULE_MAIN_ADDR_1         0x1000	/*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_1       0XFFFF	/*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_1      0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_1          0XFFFF

#define VCMD_ENC_IO_ADDR_2                  0x9c0000   /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_2                  (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_ENC_INT_PIN_2                  -1
#define VCMD_ENC_MODULE_TYPE_2              0
#define VCMD_ENC_MODULE_MAIN_ADDR_2         0x1000	/*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_2       0XFFFF	/*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_2      0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_2          0XFFFF

#define VCMD_ENC_IO_ADDR_3                  0x9e0000   /*customer specify according to own platform*/
#define VCMD_ENC_IO_SIZE_3                  (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_ENC_INT_PIN_3                  -1
#define VCMD_ENC_MODULE_TYPE_3              0
#define VCMD_ENC_MODULE_MAIN_ADDR_3         0x1000	/*customer specify according to own platform*/
#define VCMD_ENC_MODULE_DEC400_ADDR_3       0XFFFF	/*0xffff means no such kind of submodule*/
#define VCMD_ENC_MODULE_L2CACHE_ADDR_3      0XFFFF
#define VCMD_ENC_MODULE_MMU_ADDR_3          0XFFFF

//video encoder cutree/IM  vcmd configuration


#define VCMD_IM_IO_ADDR_0                   0x982000//0x22000	/*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_0                   (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_IM_INT_PIN_0                   -1
#define VCMD_IM_MODULE_TYPE_0               1
#define VCMD_IM_MODULE_MAIN_ADDR_0          0x1000	 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_0        0XFFFF	 /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_0       0XFFFF
#define VCMD_IM_MODULE_MMU0_ADDR_0          0XFFFF //0X2000
#define VCMD_IM_MODULE_MMU1_ADDR_0          0XFFFF
#define VCMD_IM_MODULE_AXIFE0_ADDR_0        0XFFFF //0X3000
#define VCMD_IM_MODULE_AXIFE1_ADDR_0        0XFFFF // 0XFFFF

#define VCMD_IM_IO_ADDR_1                   0x9a2000	/*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_1                   (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_IM_INT_PIN_1                   -1
#define VCMD_IM_MODULE_TYPE_1               1
#define VCMD_IM_MODULE_MAIN_ADDR_1          0x1000	 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_1        0XFFFF	 /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_1       0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_1           0XFFFF

#define VCMD_IM_IO_ADDR_2                   0x9c2000	/*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_2                   (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_IM_INT_PIN_2                   -1
#define VCMD_IM_MODULE_TYPE_2               1
#define VCMD_IM_MODULE_MAIN_ADDR_2          0x1000	 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_2        0XFFFF	 /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_2       0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_2           0XFFFF

#define VCMD_IM_IO_ADDR_3                   0x9e2000	/*customer specify according to own platform*/
#define VCMD_IM_IO_SIZE_3                   (ASIC_VCMD_SWREG_AMOUNT * 4)	/* bytes */
#define VCMD_IM_INT_PIN_3                   -1
#define VCMD_IM_MODULE_TYPE_3               1
#define VCMD_IM_MODULE_MAIN_ADDR_3          0x1000	 /*customer specify according to own platform*/
#define VCMD_IM_MODULE_DEC400_ADDR_3        0XFFFF	 /*0xffff means no such kind of submodule*/
#define VCMD_IM_MODULE_L2CACHE_ADDR_3       0XFFFF
#define VCMD_IM_MODULE_MMU_ADDR_3           0XFFFF

#define VASTAI_ENC_MAX_CORES                (4)
#define VCMD_ENC_DDR_SIZE                   (1024 * 1024 * 9)
#define VCMD_ENC_DDR_BASE_ADDR_0            (0x1028000000)
//#define VCMD_ENC_DDR_BASE_ADDR_1			(VCMD_ENC_DDR_BASE_ADDR_0 + VCMD_ENC_DDR_SIZE)
//#define VCMD_ENC_DDR_BASE_ADDR_2			(VCMD_ENC_DDR_BASE_ADDR_1 + VCMD_ENC_DDR_SIZE)
//#define VCMD_ENC_DDR_BASE_ADDR_3			(VCMD_ENC_DDR_BASE_ADDR_3 + VCMD_ENC_DDR_SIZE)

static struct vcmd_config vcmd_core_array[]= {
	//encoder configuration
	{VCMD_ENC_IO_ADDR_0,
	VCMD_ENC_IO_SIZE_0,
	VCMD_ENC_INT_PIN_0,
	VCMD_ENC_MODULE_TYPE_0,
	VCMD_ENC_MODULE_MAIN_ADDR_0,
	VCMD_ENC_MODULE_DEC400_ADDR_0,
	VCMD_ENC_MODULE_L2CACHE_ADDR_0,
	VCMD_ENC_MODULE_MMU0_ADDR_0,
	VCMD_ENC_MODULE_MMU1_ADDR_0,
	VCMD_ENC_MODULE_AXIFE0_ADDR_0,
	VCMD_ENC_MODULE_AXIFE1_ADDR_0},

	{VCMD_ENC_IO_ADDR_1,
	VCMD_ENC_IO_SIZE_1,
	VCMD_ENC_INT_PIN_1,
	VCMD_ENC_MODULE_TYPE_1,
	VCMD_ENC_MODULE_MAIN_ADDR_1,
	VCMD_ENC_MODULE_DEC400_ADDR_1,
	VCMD_ENC_MODULE_L2CACHE_ADDR_1,
	VCMD_ENC_MODULE_MMU_ADDR_1},

	{VCMD_ENC_IO_ADDR_2,
	VCMD_ENC_IO_SIZE_2,
	VCMD_ENC_INT_PIN_2,
	VCMD_ENC_MODULE_TYPE_2,
	VCMD_ENC_MODULE_MAIN_ADDR_2,
	VCMD_ENC_MODULE_DEC400_ADDR_2,
	VCMD_ENC_MODULE_L2CACHE_ADDR_2,
	VCMD_ENC_MODULE_MMU_ADDR_2},

	{VCMD_ENC_IO_ADDR_3,
	VCMD_ENC_IO_SIZE_3,
	VCMD_ENC_INT_PIN_3,
	VCMD_ENC_MODULE_TYPE_3,
	VCMD_ENC_MODULE_MAIN_ADDR_3,
	VCMD_ENC_MODULE_DEC400_ADDR_3,
	VCMD_ENC_MODULE_L2CACHE_ADDR_3,
	VCMD_ENC_MODULE_MMU_ADDR_3},

	//cutree/IM configuration
	{VCMD_IM_IO_ADDR_0,
	VCMD_IM_IO_SIZE_0,
	VCMD_IM_INT_PIN_0,
	VCMD_IM_MODULE_TYPE_0,
	VCMD_IM_MODULE_MAIN_ADDR_0,
	VCMD_IM_MODULE_DEC400_ADDR_0,
	VCMD_IM_MODULE_L2CACHE_ADDR_0,
	VCMD_IM_MODULE_MMU0_ADDR_0,
	VCMD_IM_MODULE_MMU1_ADDR_0,
	VCMD_IM_MODULE_AXIFE0_ADDR_0,
	VCMD_IM_MODULE_AXIFE1_ADDR_0},

	{VCMD_IM_IO_ADDR_1,
	VCMD_IM_IO_SIZE_1,
	VCMD_IM_INT_PIN_1,
	VCMD_IM_MODULE_TYPE_1,
	VCMD_IM_MODULE_MAIN_ADDR_1,
	VCMD_IM_MODULE_DEC400_ADDR_1,
	VCMD_IM_MODULE_L2CACHE_ADDR_1,
	VCMD_IM_MODULE_MMU_ADDR_1},

	{VCMD_IM_IO_ADDR_2,
	VCMD_IM_IO_SIZE_2,
	VCMD_IM_INT_PIN_2,
	VCMD_IM_MODULE_TYPE_2,
	VCMD_IM_MODULE_MAIN_ADDR_2,
	VCMD_IM_MODULE_DEC400_ADDR_2,
	VCMD_IM_MODULE_L2CACHE_ADDR_2,
	VCMD_IM_MODULE_MMU_ADDR_2},

	{VCMD_IM_IO_ADDR_3,
	VCMD_IM_IO_SIZE_3,
	VCMD_IM_INT_PIN_3,
	VCMD_IM_MODULE_TYPE_3,
	VCMD_IM_MODULE_MAIN_ADDR_3,
	VCMD_IM_MODULE_DEC400_ADDR_3,
	VCMD_IM_MODULE_L2CACHE_ADDR_3,
	VCMD_IM_MODULE_MMU_ADDR_3},
};

static int total_vcmd_core_num = 0;
#endif
/*
 * Macros to help debugging
 */

static int venc_log_level = VIDEO_LOGLEVEL_ERROR;
#define VASTAI_ENC_DEBUG
#undef PDEBUG
#ifdef VASTAI_ENC_DEBUG
#   define PDEBUG(level, fmt, args...) {if (level <= venc_log_level) printk( KERN_INFO "pid[%d] va_enc: " fmt, current->pid, ## args);}
#else
#   define PDEBUG(fmt, args...)
#endif

struct list_head enc_device_handle_head;

static int wait_alloc_channel_unlock(void *die_info);
static int wait_alloc_channel_lock(void *die_info);
static void vcmd_workqueue_schedue(struct die_enc_info *die, struct work_struct *work, int core_id);

#define VCMD_WORKQUEUE_SCHEDULE(core_id) \
	static void vcmd_workqueue_schedue##core_id(struct work_struct *work) \
	{ \
		struct die_enc_info* die = container_of(work, struct die_enc_info, work##core_id); \
		vcmd_workqueue_schedue(die, work, core_id); \
	} \

VCMD_WORKQUEUE_SCHEDULE(0);
VCMD_WORKQUEUE_SCHEDULE(1);
VCMD_WORKQUEUE_SCHEDULE(2);
VCMD_WORKQUEUE_SCHEDULE(3);


#define VASTAI_PCI_ENC_CALLBACK(core_id) \
	static void vastai_pci_enc_callback##core_id (void *die_info) { \
		int work_ret = -1;\
		struct die_enc_info* die = (struct die_enc_info*)die_info;\
		work_ret = queue_work(die->workqueue[core_id], &(die->work##core_id));\
		if (!work_ret) {\
			PDEBUG(VIDEO_LOGLEVEL_INFO, "queue_work error!!! devfn=%d work_ret=%d\n",  die->devfn, work_ret); \
		} else { \
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "queue_work done: devfn=%d work_ret=%d\n",  die->devfn, work_ret); \
		} \
	} \

VASTAI_PCI_ENC_CALLBACK(0)
VASTAI_PCI_ENC_CALLBACK(1)
VASTAI_PCI_ENC_CALLBACK(2)
VASTAI_PCI_ENC_CALLBACK(3)

static inline int check_dmabuf_fd_valid (int dmabuf_fd) {
	return dmabuf_fd < 3 ? 0 : 1;
}


static int wait_alloc_channel_lock(void *die_info) {	
	int locked = 0;
	int counter = 0;
	int ret = 0;
	struct die_enc_info* die = (struct die_enc_info*)die_info;	
	if (!die)
		return -1;	

	if (die->timestamp != 0) {
		ktime_t current_time = ktime_get_seconds();
		if (current_time - die->timestamp > 5) {
			printk("unlock the zomb thread lock\n");
			wait_alloc_channel_unlock(die);
		}
	}

	do {
		locked = mutex_trylock(&die->chan_alloc_mutex);
		if (!locked) {
			//msleep(10);
			ret = msleep_interruptible(10);
			if (ret > 0) {
				printk(KERN_ERR "pid[%d] msleep_interruptible interrupted\n",  current->pid);
				return -1;
			}
			counter++;
		} else {
			die->timestamp = ktime_get_seconds();
		}

		if (counter == VENC_CHANNEL_LOCK_TIMEOUT) {
			printk(KERN_ERR "pid[%d] venc failed to lock, timeout\n", current->pid);
			return -1;
		}
	} while (!locked);

	//printk(KERN_INFO "pid[%d] venc: die_index:%x locked\n", current->pid, die_index);	
	return 0;
}

static int wait_alloc_channel_unlock(void *die_info) {	
	struct die_enc_info* die = (struct die_enc_info*)die_info;	
	if (!die)
		return -1;	
	
	mutex_unlock(&die->chan_alloc_mutex);
	die->timestamp = 0;
	return 0;	
}

static struct enc_chn_info *get_enc_chn_info(struct file *filp)
{
	struct enc_chn_info *info;
	struct vastai_video_info *video_info;

	video_info = (struct vastai_video_info *)filp->private_data;
	info = video_info->enc_info;

	return info;
}

static void set_enc_chn_info(struct file *filp, struct enc_chn_info *info)
{
	struct vastai_video_info *video_info;

	video_info = (struct vastai_video_info *)filp->private_data;
	video_info->enc_info = info;

}

//fence start
static const char *vastai_dma_fence_get_name(struct dma_fence *fence)
{
	return "video-enc-fence";
}

static void vastai_dma_fence_release(struct dma_fence *fence)
{
	if (!fence) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error,  invalid fence input!\n", __LINE__);
		return;
	}

	if (fence->lock) {
		kfree(fence->lock);
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d free fence %p\n", __LINE__, fence);

	kfree_rcu(fence, rcu);
}

static const struct dma_fence_ops fence_ops = {
	.get_driver_name = vastai_dma_fence_get_name,
	.get_timeline_name = vastai_dma_fence_get_name,
	.release = vastai_dma_fence_release,
};

static inline u64
venc_fence_context_seqno_next(struct venc_fence_context *fctx)
{
	return atomic_inc_return(&fctx->fence_seqno) - 1;
}
#if 0
static struct dma_fence *venc_create_fence(struct venc_fence_context* fctx)
{
	struct dma_fence *fence;
	u64 seqno;

	if (!fctx) {
		printk("%d error,  invalid fctx input!\n", __LINE__);
		return NULL;
	}

	fence = kzalloc(sizeof(*fence), GFP_KERNEL);
	if (!fence) {
		printk("%d alloc fence mem failed!\n", __LINE__);
		return NULL;
	}

	seqno = venc_fence_context_seqno_next(fctx);

	dma_fence_init(fence, &fence_ops, &fctx->lock, fctx->fence_context, seqno);

	return fence;
}
#endif

static struct dma_fence *venc_create_fence_ex(void)
{
	struct dma_fence *fence;
	spinlock_t* fence_lock;

	fence = kzalloc(sizeof(*fence), GFP_KERNEL);
	if (!fence) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d alloc fence mem failed!\n", __LINE__);
		return NULL;
	}

	fence_lock = kzalloc(sizeof(spinlock_t), GFP_KERNEL);
	spin_lock_init(fence_lock);
	dma_fence_init(fence, &fence_ops, fence_lock, 0, 0);
	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d create fence %p\n", __LINE__, fence);
	return fence;
}

static int venc_get_devaddr_from_fd(venc_exchange_data_t *data, struct die_enc_info* die)
{
	struct dma_buf *dmabuf = NULL;
	struct dma_buf_attachment *attachment = NULL;
	struct sg_table *table = NULL;
	unsigned long long dev_addr, buffer_size;

	if (!data || !die) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, data or die is null! invalid param, data=%p, die=%p\n", __LINE__, data, die);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d get dev addr from fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d devfn=%d dev_id=%d\n", __LINE__, data->dmabuf_fd, die->devfn, die->priv_dev->dev_id);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d dmabuf is null! invalid param, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	attachment = dma_buf_attach(dmabuf, &(die->priv_dev->dev->dev));
	if (!attachment) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, attachment is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

	table = dma_buf_map_attachment(attachment, DMA_BIDIRECTIONAL);
	if (!table) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, sg_table is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

	dev_addr = sg_dma_address(table->sgl);
	buffer_size = sg_dma_len(table->sgl);

#ifdef VENC_FENCE_TEST
	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "dev_addr = 0x%llx, buffer_size = 0x%llx soc_addr=0x%llx\n", dev_addr, buffer_size, *((uint64_t*)dev_addr));
	data->dev_addr = *((uint64_t*)dev_addr);
#else
	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "dev_addr = 0x%llx, buffer_size = 0x%llx\n", dev_addr, buffer_size);
	data->dev_addr = dev_addr;
#endif

	dma_buf_unmap_attachment(attachment, table, DMA_BIDIRECTIONAL);
	dma_buf_detach(dmabuf, attachment);

	dma_buf_put(dmabuf);

	return 0;
}

static int venc_acquire_surface(venc_exchange_data_t *data, struct die_enc_info* die)
{
	struct dma_buf *dmabuf = NULL;
	struct dma_resv *resv = NULL;
	struct dma_fence *shared_fence = NULL;
	int ret = -1;

	if (!data || !die) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, data or die is null! invalid param, data=%p, die=%p\n", __LINE__, data, die);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d insert share fence fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d devfn=%d dev_id=%d\n", __LINE__, data->dmabuf_fd, die->devfn, die->priv_dev->dev_id);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	resv = dmabuf->resv;
	if (!resv) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, resv is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

	shared_fence = venc_create_fence_ex();

	if (!shared_fence) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, create_fence failed\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

	dma_resv_lock_interruptible(resv, NULL);

#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 1)
	ret = dma_resv_reserve_shared(resv, 1);
  #else
	ret = dma_resv_reserve_shared(resv);
  #endif
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0))
	ret = dma_resv_reserve_shared(resv, 1);  // tmp code
#else
	ret = dma_resv_reserve_shared(resv);
#endif
	if (ret) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d dma_resv_reserve_shared failed ret=%d !\n", __LINE__, ret);
		dma_fence_put(shared_fence);
		dma_buf_put(dmabuf);
		return -1;
	}

#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
    dma_resv_add_shared_fence(resv, shared_fence);
  #else
    dma_resv_add_fence(resv, shared_fence, DMA_RESV_USAGE_READ);
  #endif
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0))
	dma_resv_add_fence(resv, shared_fence, DMA_RESV_USAGE_READ);
#else
 	dma_resv_add_shared_fence(resv, shared_fence);
#endif

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d insert share fence success! fd=%d\n", __LINE__, data->dmabuf_fd);

	dma_resv_unlock(resv);

	dma_buf_put(dmabuf);

	return 0;

}

static int venc_release_surface(venc_exchange_data_t *data)
{
	struct dma_buf *dmabuf = NULL;
	struct dma_resv *resv = NULL;
#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
	struct dma_fence *excl_fence = NULL, **shared_fence = NULL;
	int pshared_count = 0;
  #else
	struct dma_fence **excl_fence = NULL, **shared_fence = NULL;
	int pshared_count = 0, pexcl_count = 0;
  #endif
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0)
	struct dma_fence **excl_fence = NULL, **shared_fence = NULL;
	int pshared_count = 0, pexcl_count = 0;
#else
	struct dma_fence *excl_fence = NULL, **shared_fence = NULL;
	int pshared_count = 0;
#endif
	int ret = -1;

	if (!data) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid param data=%p \n", __LINE__, data);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d release share fence fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	resv = dmabuf->resv;
	if (!resv) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, resv is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
	ret = dma_resv_get_fences(resv, &excl_fence, &pshared_count, &shared_fence);
	if (!pshared_count) {
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
  #else
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_READ, &pshared_count, &shared_fence);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, shared fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_WRITE, &pexcl_count, &excl_fence);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, excl fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
  #endif
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0)
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_READ, &pshared_count, &shared_fence);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, shared fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_WRITE, &pexcl_count, &excl_fence);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, excl fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
#else
	ret = dma_resv_get_fences(resv, &excl_fence, &pshared_count, &shared_fence);
	if (!pshared_count) {
		PDEBUG(VIDEO_LOGLEVEL_WARN, "%d warn, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
#endif

	if (shared_fence[0]) {
		dma_fence_signal(shared_fence[0]);
		dma_fence_put(shared_fence[0]);
		PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d release share fence success! fd=%d\n", __LINE__, data->dmabuf_fd);
#ifdef VENC_FENCE_TEST
		dma_fence_put(shared_fence[0]); //release this fence
#endif
	}

	dma_buf_put(dmabuf);

	return 0;
}

static int venc_user_release_surface(venc_exchange_data_t *data)
{
	struct dma_buf *dmabuf = NULL;
	venc_surface_node_t *surface = NULL;
	int ret = -1;

	if (!data) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid param, data=%p\n", __LINE__, data);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d  fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	if (!dmabuf->priv) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf->priv is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	surface = (venc_surface_node_t*)dmabuf->priv;
	if (surface->flag_encode_complete) {
		ret = venc_release_surface(data);
		if (ret) {
			PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d release surface failed, dmabuf_id=%d\n", __LINE__, data->dmabuf_fd);
			return -1;
		}
	} else {
		surface->flag_need_release = 1;
	}

	return 0;
}

static int venc_wait_surface_ready(venc_exchange_data_t *data, struct enc_chn_info *info, struct die_enc_info* die)
{
	struct dma_buf *dmabuf = NULL;
	struct dma_resv *resv = NULL;
#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
	struct dma_fence *excl_fence = NULL;
  #else
	struct dma_fence **excl_fence_list = NULL, *excl_fence = NULL;
	int pexcl_count = 0;
  #endif
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0)
	struct dma_fence **excl_fence_list = NULL, *excl_fence = NULL;
	int pexcl_count = 0;
#else	
	struct dma_fence *excl_fence = NULL;
#endif

	int ret = -1;

	if (!data || !info) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid param data=%p info=%p\n", __LINE__, data, info);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d wati excl fence fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d devfn=%d dev_id=%d\n", __LINE__, data->dmabuf_fd, die->devfn, die->priv_dev->dev_id);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	resv = dmabuf->resv;
	if (!resv) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, resv is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}

#if defined(RHEL_RELEASE_VERSION)
  #if RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0)
	excl_fence = dma_resv_excl_fence(resv);
	if (!excl_fence) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}
  #else
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_WRITE, &pexcl_count, &excl_fence_list);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
	excl_fence = excl_fence_list[0];
  #endif
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0)
	ret = dma_resv_get_fences(resv, DMA_RESV_USAGE_WRITE, &pexcl_count, &excl_fence_list);
	if (ret)
	{
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return 0;
	}
	excl_fence = excl_fence_list[0];
#else
	excl_fence = dma_resv_excl_fence(resv);
	if (!excl_fence) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, fence is null\n", __LINE__);
		dma_buf_put(dmabuf);
		return -1;
	}
#endif

	ret = dma_fence_get_status(excl_fence);
	if (ret == 1) {
		dma_buf_put(dmabuf);
		return 0;
	} else if (ret < 0) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d dma_fence_get_status return error, ret=%d!\n", __LINE__, ret);
		dma_buf_put(dmabuf);
		return -1;
	}

	ret = dma_fence_wait_timeout(excl_fence, 1, msecs_to_jiffies(FENCE_WAIT_TIMEOUT));
	if (ret == 0) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d dma_fence_wait_timeout timeout, ret=%d!\n", __LINE__, ret);
		dma_buf_put(dmabuf);
		return -1;
	} else if (ret < 0) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d dma_fence_wait_timeout interrupted, ret=%d!\n", __LINE__, ret);
		dma_buf_put(dmabuf);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d wati excl fence success! fd=%d\n", __LINE__, data->dmabuf_fd);

	dma_buf_put(dmabuf);

	return 0;
}

static int venc_save_surface_dmabuf(venc_exchange_data_t *data)
{
	struct dma_buf *dmabuf = NULL;
	venc_surface_node_t *new_surface = NULL, *surface = NULL;

	if (!data) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid param data=%p\n", __LINE__, data);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d save dmabuf to surface fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %dn", __LINE__, data->dmabuf_fd);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	if (dmabuf->priv) {

		surface = (venc_surface_node_t*)dmabuf->priv;

		surface->flag_encode_complete = 0;
		surface->flag_need_release = 0;

		return 0;
	}

	new_surface = (venc_surface_node_t*)kzalloc(sizeof(venc_surface_node_t), GFP_KERNEL);
	if (!new_surface)
	{
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d  alloc surface node failed\n", __LINE__);
		return -1;
	}

	new_surface->dmabuf_fd = data->dmabuf_fd;
	new_surface->dmabuf = dmabuf;
	spin_lock_init(&surface->lock);

	dmabuf->priv = (void*)new_surface;

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d create surface for dmabuf fd=%d\n", __LINE__, data->dmabuf_fd);

	return 0;
}

static int venc_check_release_surface_dmabuf(venc_exchange_data_t *data)
{
	struct dma_buf *dmabuf = NULL;
	venc_surface_node_t *surface = NULL;
	int ret = -1;

	if (!data) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid param data=%p\n", __LINE__, data);
		return -1;
	}

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "%d check release fd=%d\n", __LINE__, data->dmabuf_fd);

	if (!check_dmabuf_fd_valid(data->dmabuf_fd)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, invalid dmabuf_fd %d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	dmabuf = dma_buf_get(data->dmabuf_fd);
	if (IS_ERR_OR_NULL(dmabuf)) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	if (!dmabuf->priv) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error, dmabuf->priv is null, dmabuf_fd=%d\n", __LINE__, data->dmabuf_fd);
		return -1;
	}

	surface = (venc_surface_node_t*)dmabuf->priv;

	surface->flag_encode_complete = 1;

	if (surface->flag_need_release) {
		ret = venc_release_surface(data);
		if (ret) {
			PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d release surface failed, dmabuf_id=%d\n", __LINE__, data->dmabuf_fd);
			return -1;
		}
	}

	return 0;
}

#ifdef VENC_FENCE_TEST
static int venc_init_fence_ctx(struct venc_fence_context* fctx, char* name)
{
	if (!fctx) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d err, invalid input fctx\n", __LINE__);
		return -1;
	}

	spin_lock_init(&fctx->lock);
	atomic_set(&fctx->fence_seqno, 0);

	spin_lock_init(&fctx->list_lock);
	INIT_LIST_HEAD(&fctx->surface_list);

	fctx->fence_context = dma_fence_context_alloc(1);
	if (!fctx->fence_context) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d err, alloc fenct context failed!\n", __LINE__);
		return -1;
	}

	if (name) {
		strlcpy(fctx->name, name, sizeof(fctx->name));
	}

	return 0;
}

static int venc_deinit_fence_ctx(struct enc_chn_info *chn_info, struct die_enc_info* die)
{
	return 0;
}
#endif
//fence end


#ifdef VENC_MULTICORE_SUPPORT
static int allocate_cmdbuf(struct die_enc_info *die, struct noncache_mem* new_cmdbuf_addr, struct noncache_mem* new_status_cmdbuf_addr, u16 core_id)
{
	u16 i = 0;

	mutex_lock(&die->mutex);
	if (die->cmdbuf_used_residual == 0) {
		mutex_unlock(&die->mutex);
		//no empty cmdbuf
		return 0;
	}
	//there is one cmdbuf at least
	while (1) {
		i = die->cmdbuf_used_pos;
		if (die->cmdbuf_used[i] == 0) {

			die->cmdbuf_used[i] = 1;
			die->cmdbuf_used_residual -=1;

			new_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->cmdbuf_id = i;

			new_cmdbuf_addr->busAddress = die->vcmd_buf_mem_pool.busAddress + i * CMDBUF_MAX_SIZE;
			new_cmdbuf_addr->mmu_bus_address = die->vcmd_buf_mem_pool.mmu_bus_address + i * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->busAddress = die->vcmd_status_buf_mem_pool.busAddress + i * CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->mmu_bus_address = die->vcmd_status_buf_mem_pool.mmu_bus_address + i * CMDBUF_MAX_SIZE;

			new_status_cmdbuf_addr->size = CMDBUF_MAX_SIZE;
			new_status_cmdbuf_addr->cmdbuf_id = i;
			die->cmdbuf_used_pos++;

			if (die->cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
				die->cmdbuf_used_pos=0;

			mutex_unlock(&die->mutex);

			return 1;
		} else {
			die->cmdbuf_used_pos++;
			if(die->cmdbuf_used_pos >= TOTAL_DISCRETE_CMDBUF_NUM)
				die->cmdbuf_used_pos=0;
		}
	}
	return 0;
}

static struct cmdbuf_obj* create_cmdbuf_obj(struct die_enc_info *die, u16 core_id)
{
	/* bi_list_node* current_node=NULL; */
	struct cmdbuf_obj* cmdbuf_obj = NULL;
	struct noncache_mem  new_cmdbuf_addr = {0};
	struct noncache_mem  new_status_cmdbuf_addr = {0};
	u16 get_cmdbuf_id = -1;

	if(wait_event_interruptible_exclusive(die->vcmd_cmdbuf_memory_wait, allocate_cmdbuf(die, &new_cmdbuf_addr, &new_status_cmdbuf_addr, core_id)))
		return NULL;

	get_cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;

	cmdbuf_obj = &(die->cmdbuf_obj_sets[get_cmdbuf_id]);
	cmdbuf_obj->cmdbuf_busAddress = new_cmdbuf_addr.busAddress;
	cmdbuf_obj->cmdbuf_size = new_cmdbuf_addr.size;
	cmdbuf_obj->cmdbuf_id = new_cmdbuf_addr.cmdbuf_id;
	cmdbuf_obj->status_busAddress = new_status_cmdbuf_addr.busAddress;
	cmdbuf_obj->status_size = new_status_cmdbuf_addr.size;

	return cmdbuf_obj;
}

static long reserve_cmdbuf(struct file *filp, struct die_enc_info *die, struct exchange_parameter* input_para)
{
	struct cmdbuf_obj* cmdbuf_obj=NULL;
	input_para->cmdbuf_id = 0;
	input_para->core_id = 0; //for multicore mode, not need core id when reserve cmdbuf

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "venc reserve cmdbuf filp %p, module_type=%d\n", (void *)filp, input_para->module_type);

	cmdbuf_obj = create_cmdbuf_obj(die, input_para->core_id);
	if (cmdbuf_obj == NULL) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "core[%d] create_cmdbuf_obj failed \n", input_para->core_id);
		return -1;
	}

	cmdbuf_obj->module_type = input_para->module_type;
	cmdbuf_obj->priority = input_para->priority;
	cmdbuf_obj->executing_time = input_para->executing_time;
	cmdbuf_obj->cmdbuf_size = CMDBUF_MAX_SIZE;
	cmdbuf_obj->core_id = 0;
	cmdbuf_obj->cmdbuf_run_done = 0;
	cmdbuf_obj->filp = filp;

	input_para->cmdbuf_id = cmdbuf_obj->cmdbuf_id;

	return 0;
}

static long link_and_run_cmdbuf(struct file *filp, struct exchange_parameter* input_para, struct die_enc_info *die_info)
{
	struct cmdbuf_obj* cmdbuf_obj=NULL;
	struct vastai_pci_info *pcie_dev = (struct vastai_pci_info *)die_info->priv_dev;
	int ret = -1;
	struct vastai_dmadesc *dma_buf = NULL;
	union core_bitmap triger_core = {{0}};
	u16 cmdbuf_id = input_para->cmdbuf_id;
	//u16 core_id = input_para->core_id;
	host_to_vemcu_msg_t msg;
	int retry;

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "cmdbuf_id=%d, module_type=%d, vcmdbuf_src_addr=%lx, vcmdbuf_dst_addr=0x%lx, cmdbuf_size=%d\n", cmdbuf_id, input_para->module_type,
				(unsigned long int)input_para->vcmdbuf_desc.vcmdbuf_src_addr,
				input_para->vcmdbuf_desc.vcmdbuf_dst_addr, input_para->vcmdbuf_desc.vcmd_size);

	cmdbuf_obj = &(die_info->cmdbuf_obj_sets[cmdbuf_id]);

	if (cmdbuf_obj->filp != filp) {
		printk(KERN_ERR "%d venc vcmd: ERROR cmdbuf_id !!\n", __LINE__);
		return -1;
	}

	cmdbuf_obj->cmdbuf_data_loaded = 1;
	cmdbuf_obj->cmdbuf_size = input_para->cmdbuf_size;
	cmdbuf_obj->waited = 0;

#ifdef VCMD_DEBUG_INTERNAL
	if (filp != NULL) {
		{
			u32 i, inst = 0, size = 0;
			printk(KERN_INFO"venc vcmd link, current cmdbuf content\n");
			for(i=0;i<cmdbuf_obj->cmdbuf_size/4;i++)
			{
				__get_user(inst, (u32*)input_para->vcmdbuf_desc.vcmdbuf_src_addr + i);
				if (i == inst) {
					PrintInstr(i, inst, &size);
					inst += size;
				} else {
					printk(KERN_INFO"current cmdbuf data %d = 0x%x\n",i, inst);
				}
			}
		}
	}
#endif

	cmdbuf_obj->has_end_cmdbuf = 0; //0: has jmp opcode,1 has end code
	cmdbuf_obj->no_normal_int_cmdbuf = 1; //0: interrupt when JMP,1 not interrupt when JMP
	cmdbuf_obj->cmdbuf_User_virtualAddress = (u32*)(input_para->vcmdbuf_desc.vcmdbuf_src_addr);

	dma_buf = kmalloc(sizeof(struct vastai_dmadesc), GFP_KERNEL);
	//triger_core.decode = BIT(input_para->core_id);
	dma_buf[VCMD].is_src_not_user_mem = 0;
	dma_buf[VCMD].is_src_dma_addr = 0;
	dma_buf[VCMD].is_host_to_dev = 1;
	dma_buf[VCMD].host_addr.vir_addr = (void*)(input_para->vcmdbuf_desc.vcmdbuf_src_addr);
	dma_buf[VCMD].dev_addr = cmdbuf_obj->cmdbuf_busAddress;
	dma_buf[VCMD].dma_lenth = cmdbuf_obj->cmdbuf_size;

	retry = 5;
	while(ret && retry) {
		ret = vastai_pci_dma_transfer_sync(pcie_dev,  die_info->die_index, triger_core, dma_buf, 1);
		if (ret != 0) {
			if (ret == -EBUSY) {
				printk(KERN_WARNING "warning va_pci_dma_transfer_sync: ret = %d !!\n", ret);
				mdelay(200);
				retry--;
			} else {
				printk(KERN_ERR "error va_pci_dma_transfer_sync: ret = %d !!\n", ret);
				break;
			}
		}
	}

	kfree(dma_buf);

	msg.magic = VENC_MAGIC_NUMBER;
	msg.cmdbuf_id = input_para->cmdbuf_id;
	msg.cmdbuf_len = input_para->cmdbuf_size;
	msg.module_type = input_para->module_type;
	msg.executing_time = cmdbuf_obj->executing_time;
#if 1
	mutex_lock(&die_info->vastai_fifo_mutex[0]);
	ret = vastai_video_fifo_push_elem(pcie_dev, die_info->die_index, HOST_VEMCU0_MSG_ADDR, &msg, NULL);
	if (ret) {
		pr_err("venc write ring buffer failed!!! ret=%d, die_index=0x%x, cmdbuf_id=%d, module_type=%d\n",
			ret, die_info->die_index, input_para->cmdbuf_id, input_para->module_type);
	}
	mutex_unlock(&die_info->vastai_fifo_mutex[0]);
#endif
	return ret;
}

static void free_cmdbuf_mem(struct die_enc_info *die, u32 cmdbuf_id)
{
	mutex_lock(&die->mutex);
	die->cmdbuf_used[cmdbuf_id] = 0;
	die->cmdbuf_used_residual += 1;
	mutex_unlock(&die->mutex);
	wake_up_interruptible_sync(&die->vcmd_cmdbuf_memory_wait);
}

static void clean_cmdbuf_obj(struct cmdbuf_obj* cmdbuf_obj, u32 cmdbuf_id, struct die_enc_info* die)
{
	if (cmdbuf_obj == NULL) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "%s\n","remove_cmdbuf_node NULL");
		return;
	}

	cmdbuf_obj->cmdbuf_need_remove = 1;
	cmdbuf_obj->cmdbuf_busAddress = 0;
	cmdbuf_obj->cmdbuf_size = 0;
	cmdbuf_obj->cmdbuf_id = -1;
	cmdbuf_obj->status_busAddress = 0;
	cmdbuf_obj->status_size = 0;
	cmdbuf_obj->module_type = -1;
	cmdbuf_obj->core_id = -1;
	cmdbuf_obj->filp = NULL;
	cmdbuf_obj->cmdbuf_run_done = 0;
//	cmdbuf_obj->cmdbuf_done_flag = 0;

	free_cmdbuf_mem(die, cmdbuf_id);
	return;
}

static long release_cmdbuf(struct file *filp, u16 cmdbuf_id, struct die_enc_info* die)
{
	struct cmdbuf_obj* cmdbuf_obj=NULL;
	PDEBUG(VIDEO_LOGLEVEL_TRACE, "die_index=0x%x, cmdbuf_id=%d\n", die->die_index, cmdbuf_id);

	cmdbuf_obj = &(die->cmdbuf_obj_sets[cmdbuf_id]);
	if (cmdbuf_obj->filp!=filp) {
		pr_err("venc invalid param, die_index=0x%x, cmdbuf_id=%d!!\n", die->die_index, cmdbuf_obj->cmdbuf_id);
		return -1;
	}

	clean_cmdbuf_obj(cmdbuf_obj, cmdbuf_id, die);

	return 0;
}

static int check_cmdbuf_irq(struct cmdbuf_obj* cmdbuf_obj)
{
  int rdy = 0;
  if(cmdbuf_obj->cmdbuf_run_done != 0)
  {
	rdy = 1;
	cmdbuf_obj->cmdbuf_run_done = 0;
  }
  return rdy;
}

static int wait_cmdbuf_ready(struct file *filp,  struct die_enc_info* die, u32 cmdbuf_id)
{
	struct cmdbuf_obj* cmdbuf_obj = NULL;
	int ret = -1;

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "die_index=0x%x, cmdbuf_id=%d, HZ=%d\n", die->die_index, cmdbuf_id, HZ);

	if (cmdbuf_id != ANY_CMDBUF_ID) {

		if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM) {
			pr_err("venc vcmd:%d invalid cmdbuf_id=%d !!\n", __LINE__, cmdbuf_id);
			return -1;
		}

		cmdbuf_obj = &(die->cmdbuf_obj_sets[cmdbuf_id]);
		if(cmdbuf_obj->filp != filp) {
			pr_err("venc vcmd:%d ERROR cmdbuf_id=%d !!\n", __LINE__, cmdbuf_id);
			return -1;
		}

		ret = wait_event_interruptible_timeout(die->wait_queue, check_cmdbuf_irq(cmdbuf_obj), 10*HZ);
		if (ret < 0) {
			pr_err("venc vcmd_wait_queue interrupted, die_index=0x%x, cmdbuf_id=%d\n", die->die_index, cmdbuf_id);
			//mutex_lock(&die->mutex);
			//mutex_lock(&die->mutex);
			return -1;
		} else if (ret == 0) {
			pr_err("venc wait event timeout, die_index=0x%x, cmdbuf_id=%d\n", die->die_index, cmdbuf_id);
			cmdbuf_obj->cmdbuf_run_done = 1;
			cmdbuf_obj->executing_status = CMDBUF_EXE_STATUS_OK;
		}

		PDEBUG(VIDEO_LOGLEVEL_INFO, "venc return,ret=%d, die_index=0x%x, cmdbufid=%d, core_id=%d, status=%d\n",
				ret, die->die_index, cmdbuf_id, die->mcu_msg[cmdbuf_id].core_id, die->mcu_msg[cmdbuf_id].encstate);

		return 0;
	}

	return -1;

}
#endif


long vastai_enc_ioctl(struct file *filp,
						  unsigned int cmd, unsigned long arg)
{
	enc_device_handle_t *enc_device_handle;
	struct die_enc_info *die;
	struct vastai_video_info *video_info;
	int ret;

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "filp=%p, ioctl cmd 0x%x\n", filp, cmd);

	/*
	 * extract the type and number bitfields, and don't encode
	 * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
	 */
	if(_IOC_TYPE(cmd) != VASTAI_ENC_IOC_MAGIC)
		return -ENOTTY;
	if((_IOC_TYPE(cmd) == VASTAI_ENC_IOC_MAGIC &&
		_IOC_NR(cmd) > VASTAI_ENC_IOC_MAXNR)
		)
		return -ENOTTY;

	/*
	 * the direction is a bitmask, and VERIFY_WRITE catches R/W
	 * transfers. `Type' is user-oriented, while
	 * access_ok is kernel-oriented, so the concept of "read" and
	 * "write" is reversed
	 */

	video_info = (struct vastai_video_info *)filp->private_data;
	enc_device_handle = video_info->enc_device_handle;
	die = &enc_device_handle->die;

	switch (cmd)
	{
		case VASTAI_ENC_IOCH_GET_DIE_ID:
		{
			int tmp = -1;
			union die_index_data die_index;

			die_index.val = die->die_index;
			die_index.dev_id = die->priv_dev->dev_id;
			die_index.RESV = die->priv_dev->dev_id;

			tmp = video_copy_to_user((u32 *)arg, &die_index, sizeof(die_index));
			if (tmp) {
				pr_err("video_copy_to_user failed, returned %d\n", tmp);
				return -EFAULT;
			}

			PDEBUG(VIDEO_LOGLEVEL_INFO,"current vfn = %d dev_id:%d\n", die->devfn, die_index.dev_id);
			break;
		}
		
		case VASTAI_ENC_IOCH_INT_VEMCU:
		{
			TIntVEMCU tVemcuIntParam = {0};
			u64 qwAddr = 0;
			int ret;
			ret = video_copy_from_user(&tVemcuIntParam, (void*)arg, sizeof(TIntVEMCU));
			//printk("triger dieid %d core %d , addr %x val %x \n",tVemcuIntParam.dwDieId, tVemcuIntParam.dwCoreId, tVemcuIntParam.dwCsrAddr, tVemcuIntParam.dwVal);
			if(tVemcuIntParam.dwCoreId >= 0 && tVemcuIntParam.dwCoreId <= 3 && (tVemcuIntParam.dwCsrAddr & 0xffff0000) == 0x8bc0000)
			{
				struct vastai_pci_info *pcie_dev = get_vastai_pci_device_info(tVemcuIntParam.dwDeviceId);
				qwAddr = (u64)(tVemcuIntParam.dwCsrAddr);
				return vastai_pci_mem_write(pcie_dev, tVemcuIntParam.dwDieId, qwAddr, &tVemcuIntParam.dwVal, sizeof(int));
			}
			else
			{
				printk(KERN_ERR  "VA_ENC_IOCH_INT_VEMCU error \n");
				return -1;
			}

			break;
		}

		case VASTAI_ENC_IOCH_ALLOCCHANNEL_LOCK:
		{
			u32 die_index;
			//__get_user(die_index, (u32*)arg);
            int ret;
     		ret = video_copy_from_user(&die_index, (u32*)arg, sizeof(die_index));

			if (wait_alloc_channel_lock(die) != 0) {
				return -1;
			}

			break;
		}

		case VASTAI_ENC_IOCH_ALLOCCHANNEL_UNLOCK:
		{
			u32 die_index;
			//__get_user(die_index, (u32*)arg);
            int ret;
     		ret = video_copy_from_user(&die_index, (u32*)arg, sizeof(die_index));

			if (wait_alloc_channel_unlock(die) != 0) {
				return -1;
			}

			break;
		}

		case VASTAI_ENC_IOCH_ATTACH_VENCCHNL:
		{
			venc_msg_chninfo_t chninfo = {0};
			u32 core_id;
			u32 die_index;
			u32 jobidx;
			struct enc_chn_info *info;

			info = get_enc_chn_info(filp);
			if (info != 0)
			{
				PDEBUG(VIDEO_LOGLEVEL_INFO, "chn is exist,to destry die idx:%d core:%d\n",info->die_index,info->core_id);
				info->core_id = 0;
				info->die_index = 0;
				info->instance_addr = 0;
				info->msg_addr = 0;
			}
			if (video_copy_from_user(&chninfo, (void*)arg, sizeof(venc_msg_chninfo_t)))
				return -2;

			core_id = chninfo.core_id>>16;
			jobidx = chninfo.core_id&0xFF;
			die_index = chninfo.die_index;

			if (jobidx >= VASTAI_MAX_CORE_CHANNEL)
				return -3;

			info = &(die->venc_core_info[core_id].enc_chn_set[jobidx]);

			set_enc_chn_info(filp, info);
			init_waitqueue_head(&info->wait_queue);
			info->core_id	= core_id;
			info->die_index = die_index;
			info->instance_addr = chninfo.instance_addr;
			info->msg_addr = (u64)(0x1000000000|chninfo.msg_addr);
			info->event = 0;
#ifdef VENC_RECV_QUEUE_SUPPORT
			memset(info->rcv, 0, LOCAL_MAX_FIFO_LEN);
			info->rcv->rd = info->rcv->wr = 0;
			info->rcv->elem_count = (LOCAL_MAX_FIFO_LEN - sizeof(struct vastai_video_fifo)) / sizeof(vemcu_to_host_msg_t);
			info->rcv->elem_size = sizeof(vemcu_to_host_msg_t);
#endif
#ifdef VENC_FENCE_TEST
			venc_init_fence_ctx(&info->fctx, "venc-fence");
#endif
			break;
		}

		case VASTAI_ENC_IOCH_DETACH_VENCCHNL:
		{
			venc_msg_chninfo_t chninfo;
			u32 core_id;
			u32 die_index;
			u32 jobidx;
			struct enc_chn_info *info;
#ifdef VENC_FENCE_TEST
			int ret = -1;
#endif
			info = get_enc_chn_info(filp);
			if (info == NULL)
				return -EFAULT;

			if (video_copy_from_user(&chninfo, (void*)arg, sizeof(venc_msg_chninfo_t)))
				return -EFAULT;

			core_id = chninfo.core_id>>16;
			jobidx = chninfo.core_id&0xFF;
			die_index = chninfo.die_index;
			if (die_index != info->die_index)
				return -EFAULT;

			if (jobidx >= VASTAI_MAX_CORE_CHANNEL)
				return -3;

			if (info != &(die->venc_core_info[core_id].enc_chn_set[jobidx]))
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "chn error die idx:%d core:%d jobidx:%d\n",info->die_index,info->core_id,jobidx);

			info = &(die->venc_core_info[core_id].enc_chn_set[jobidx]);

#ifdef VENC_FENCE_TEST
			ret = venc_deinit_fence_ctx(info, die);
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d devfn=%d venc_deinit_fence_ctx failed ret=%d\n", __LINE__, die->devfn, ret);
				return -EFAULT;
			}
#endif
			info->core_id = 0;
			info->die_index = 0;
			info->instance_addr = 0;
			info->msg_addr = 0;

			set_enc_chn_info(filp, NULL);
			break;
		}

		case VASTAI_ENC_IOCH_SET_VENCCHNLMSG:
		{

			TVidEncCfgParam tvidencparam = {0};
			struct enc_chn_info *info;

			info = get_enc_chn_info(filp);
			if (info == NULL)
				return -EFAULT;

			ret = video_copy_from_user(&tvidencparam, (void*)arg, sizeof(TVidEncCfgParam));
			if (ret)
			{
				pr_err("devfn=%d, copy from user failed for ser venc channel msg, ret = %d\n", die->devfn, ret);
				return -1;
			}

			mutex_lock(&die->vastai_fifo_mutex[info->core_id]);
			ret = vastai_video_fifo_push_elem(die->priv_dev, die->die_index,
					die->host2vemcu_ringbuffer[info->core_id]->ring_buf_base, &tvidencparam,NULL);
			mutex_unlock(&die->vastai_fifo_mutex[info->core_id]);
			if (ret)
			{
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, push elem to host->vemcu ringbuffer failed, addr=0x%x, ret = %d\n", die->devfn,
					die->host2vemcu_ringbuffer[info->core_id]->ring_buf_base, ret);
				return -1;
			}
			break;
		}

		case VASTAI_ENC_IOCH_GET_VENCCHNLMSG:
		{
			struct enc_chn_info *info;

			info = get_enc_chn_info(filp);
			if (info == NULL) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, get channel info failed, info=null\n", die->devfn);
				return -EFAULT;
			}
#ifdef VENC_RECV_QUEUE_SUPPORT
			vastai_video_dequeue_data(info->rcv, (uint8_t*)&info->mcu_msg);
#endif
			if (video_copy_to_user((vemcu_to_host_msg_t*)arg, &info->mcu_msg, sizeof(vemcu_to_host_msg_t))) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, get channel msg failed, instance=0x%x\n", die->devfn, info->instance_addr);
				return -EFAULT;
			}

			info->event =0;

			break;
		}

		case VASTAI_ENC_IOCH_GET_VCMD_ENABLE:
		{
			//__put_user(1, (u32 *) arg);
            u32 enable  = 1;
            if (video_copy_to_user((u32*)arg, &enable, sizeof(enable))) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "VA_ENC_IOCH_GET_VCMD_ENABLE failed!\n");
				return -EFAULT;
			}
			break;
		}


		case VASTAI_ENC_IOCH_GET_ENCMODE:
		{
			//__put_user(die->encoder_mode, (u16 *) arg);
            if (video_copy_to_user((u32*)arg, &die->encoder_mode, sizeof(u32))) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "VA_ENC_IOCH_GET_VCMD_ENABLE failed!\n");
				return -EFAULT;
			}
			break;
		}

		case VASTAI_ENC_IOCH_WAIT_SURFACE_READY:
		{
			venc_exchange_data_t data;
			struct enc_chn_info *info;

			info = get_enc_chn_info(filp);

			ret = video_copy_from_user(&data, (void*)arg, sizeof(venc_exchange_data_t));
			if (ret)
			{
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, copy from user failed for ser venc channel msg, ret = %d\n", die->devfn, ret);
				return -EFAULT;
			}

			if (data.dmabuf_fd != -1) {
				ret = venc_wait_surface_ready(&data, info, die);
				if (ret)
				{
					PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d devfn=%d venc_wait_surface_ready failed ret=%d\n", __LINE__, die->devfn, ret);
					return -EFAULT;
				}
			}

			break;
		}

		case VASTAI_ENC_IOCH_SAVE_SURFACE:
		{
			venc_exchange_data_t data;
			struct enc_chn_info *info;

			return 0;

			info = get_enc_chn_info(filp);

			ret = video_copy_from_user(&data, (void*)arg, sizeof(venc_exchange_data_t));
			if (ret)
			{
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, copy from user failed for ser venc channel msg, ret = %d\n", die->devfn, ret);
				return -EFAULT;
			}

			if (data.dmabuf_fd != -1) {
				ret = venc_save_surface_dmabuf(&data);
				if (ret)
				{
					PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d devfn=%d venc_save_surface_dmabuf failed ret=%d\n", __LINE__, die->devfn, ret);
					return -EFAULT;
				}

			}

			break;
		}

		case VASTAI_ENC_IOCH_REMOVE_SURFACE:
		{
			venc_exchange_data_t data;
			struct enc_chn_info *info;

			return 0;

			info = get_enc_chn_info(filp);

			ret = video_copy_from_user(&data, (void*)arg, sizeof(venc_exchange_data_t));
			if (ret)
			{
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error copy from user ret = %d\n", __LINE__, ret);
				return -EFAULT;
			}

			if (data.dmabuf_fd != -1) {
				ret = venc_check_release_surface_dmabuf(&data);
				if (ret)
				{
					PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d devfn=%d venc_check_release_surface_dmabuf failed ret=%d\n", __LINE__, die->devfn, ret);
					return -EFAULT;
				}

			}

			break;
		}

		case VASTAI_ENC_IOCH_ACQUIRE_SURFACE:
		{
			int ret = -1;
			venc_exchange_data_t data;

			ret = video_copy_from_user(&data, (void*)arg, sizeof(data));
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "error VA_ENC_IOCH_ACQUIRE_SURFACE failed!\n");
				return -EFAULT;
			}

			ret = venc_acquire_surface(&data, die);
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "error venc_acquire_surface failed! dmabuf_fd=%d dev_addr=0x%llx\n", data.dmabuf_fd, data.dev_addr);
				return -EFAULT;
			}

			break;

		}

		case VASTAI_ENC_IOCH_RELEASE_SURFACE:
		{
			int ret = -1;
			venc_exchange_data_t data;

			ret = video_copy_from_user(&data, (void*)arg, sizeof(data));
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error copy from user!\n", __LINE__);
				return -EFAULT;
			}

			ret = venc_user_release_surface(&data);
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d venc_user_release_surface error dmabuf_fd=%d\n", __LINE__, data.dmabuf_fd);
				return -EFAULT;
			}

			break;
		}

		case VASTAI_ENC_IOCH_GET_DEVADDR_FROM_FD:
		{
			int ret = -1;
			venc_exchange_data_t data;

			ret = video_copy_from_user(&data, (void*)arg, sizeof(data));
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error ret=%d\n", __LINE__, ret);
				return -EFAULT;
			}

			ret = venc_get_devaddr_from_fd(&data, die);
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error ret=%d\n", __LINE__, ret);
				return -EFAULT;
			}

			ret = video_copy_to_user((void *)arg, &data, sizeof(venc_exchange_data_t));
			if (ret) {
				PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d error copy to user ret=%d\n", __LINE__, ret);
				return -EFAULT;
			}

			break;

		}

#ifdef VENC_MULTICORE_SUPPORT
		case VASTAI_ENC_IOCH_GET_CMDBUF_PARAMETER:
		{
			struct cmdbuf_mem_parameter local_cmdbuf_mem_data = {0};
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd get cmdbuf parameter die_index=0x%x\n", die->die_index);

			if (video_copy_from_user(&local_cmdbuf_mem_data, (struct cmdbuf_mem_parameter*)arg, sizeof(struct cmdbuf_mem_parameter))) {
				return -EFAULT;
			}

			local_cmdbuf_mem_data.cmdbuf_unit_size = CMDBUF_MAX_SIZE;
			local_cmdbuf_mem_data.status_cmdbuf_unit_size = CMDBUF_MAX_SIZE;
			local_cmdbuf_mem_data.cmdbuf_total_size = CMDBUF_POOL_TOTAL_SIZE;
			local_cmdbuf_mem_data.status_cmdbuf_total_size = CMDBUF_POOL_TOTAL_SIZE;
			local_cmdbuf_mem_data.phy_status_cmdbuf_addr = die->vcmd_status_buf_mem_pool.busAddress;
			local_cmdbuf_mem_data.phy_cmdbuf_addr = die->vcmd_buf_mem_pool.busAddress;

			local_cmdbuf_mem_data.mmu_phy_status_cmdbuf_addr = 0;
			local_cmdbuf_mem_data.mmu_phy_cmdbuf_addr = 0;
			local_cmdbuf_mem_data.base_ddr_addr = 0;

			//local_cmdbuf_mem_data.vcmd_reg_mem_busAddress = die_info->vcmd_reg_mem_busAddress;

			if (video_copy_to_user((struct cmdbuf_mem_parameter*)arg, &local_cmdbuf_mem_data, sizeof(struct cmdbuf_mem_parameter))) {
				return -EFAULT;
			}

			break;
		}

		case VASTAI_ENC_IOCH_GET_VCMD_PARAMETER:
		{
			struct config_parameter input_para = {0};
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd get vcmd config parameter die_index=0x%x\n", die->die_index);

			if (video_copy_from_user(&input_para,(struct config_parameter*)arg,sizeof(struct config_parameter))) {
				return -EFAULT;
			}

			if (die->vcmd_type_core_num[input_para.module_type]) {
				input_para.submodule_main_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_main_addr;
				input_para.submodule_dec400_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_dec400_addr;
				input_para.submodule_L2Cache_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_L2Cache_addr;
				input_para.submodule_MMU_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMU_addr;
				input_para.submodule_MMUWrite_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_MMUWrite_addr;
				input_para.submodule_axife_addr = die->vcmd_manager[input_para.module_type][0]->vcmd_core_cfg.submodule_axife_addr;
				input_para.config_status_cmdbuf_id = die->vcmd_manager[input_para.module_type][0]->status_cmdbuf_id;
				input_para.vcmd_hw_version_id = die->vcmd_manager[input_para.module_type][0]->hw_version_id;
				input_para.vcmd_core_num = die->vcmd_type_core_num[input_para.module_type];

			} else {
				input_para.submodule_main_addr = 0xffff;
				input_para.submodule_dec400_addr = 0xffff;
				input_para.submodule_L2Cache_addr = 0xffff;
				input_para.submodule_MMU_addr = 0xffff;
				input_para.submodule_MMUWrite_addr = 0xffff;
				input_para.submodule_axife_addr = 0xffff;
				input_para.config_status_cmdbuf_id = 0;
				input_para.vcmd_core_num = 0;
				input_para.vcmd_hw_version_id =HW_ID_1_0_C;
			}

			if (copy_to_user((struct config_parameter*)arg, &input_para, sizeof(struct config_parameter))) {
				return -EFAULT;
			}

			break;
		}

		case VASTAI_ENC_IOCH_RESERVE_CMDBUF:
		{
			int ret = -1;
			struct exchange_parameter input_para = {0};
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd reserve cmdbuf die_index=0x%x\n", die->die_index);

			if (video_copy_from_user(&input_para, (struct exchange_parameter*)arg, sizeof(struct exchange_parameter))) {
				return -EFAULT;
			}

			ret = reserve_cmdbuf(filp, die, &input_para);
			if (ret == 0) {
				if (video_copy_to_user((struct exchange_parameter*)arg,&input_para,sizeof(struct exchange_parameter))) {
					return -EFAULT;
				}
			} else {
				pr_err("reserve_cmdbuf failed ret=%d, die_index=0x%x!!!\n", ret, die->die_index);
				return -EFAULT;
			}

			PDEBUG(VIDEO_LOGLEVEL_INFO, "reserve_cmdbuf success, die_index=0x%x cmdbuf_id=%d\n", die->die_index, input_para.cmdbuf_id);

			break;
		}

		case VASTAI_ENC_IOCH_LINK_RUN_CMDBUF:
		{
			struct exchange_parameter input_para = {0};
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd link and run cmdbuf die_index=0x%x, filp=%p\n", die->die_index, filp);

			if (video_copy_from_user(&input_para,(struct exchange_parameter*)arg, sizeof(struct exchange_parameter))) {
				return -EFAULT;
			}

			ret = link_and_run_cmdbuf(filp, &input_para, die);
			if (ret == 0) {
				if (vdieo_copy_to_user((struct exchange_parameter*)arg, &input_para, sizeof(struct exchange_parameter))) {
					return -EFAULT;
				}
			} else {
				pr_err("link_and_run_cmdbuf failed ret=%d, die_index=0x%x, cmdbuf_id=%d, core_id=%d!!!\n", 
					ret, die->die_index,  input_para.cmdbuf_id, input_para.core_id);
				return -EFAULT;
			}

			PDEBUG(VIDEO_LOGLEVEL_INFO, "link_and_run_cmdbuf success, die_index=0x%x, cmdbuf_id=%d, core_id=%d\n", die->die_index,  input_para.cmdbuf_id, input_para.core_id);

			break;
		}

		case VASTAI_ENC_IOCH_WAIT_CMDBUF:
		{
			enc_waitcmd_parameter wait_param = {0};
			u32 cmdbuf_id = -1;
			u32 core_id = -1;
			int ret = -1;

			if (video_copy_from_user(&wait_param, (enc_waitcmd_parameter*)arg, sizeof(enc_waitcmd_parameter))) {
				return -EFAULT;
			}

			cmdbuf_id = wait_param.cmdbuf_id;

			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd wait for cmdbuf. die_index=0x%x, cmdbuf_id=%d\n", die->die_index, cmdbuf_id);
			ret = wait_cmdbuf_ready(filp, die, cmdbuf_id);

			if (ret == 0) {
				if (vdieo_copy_to_user((enc_waitcmd_parameter*)arg, &die->mcu_msg[cmdbuf_id], sizeof(enc_waitcmd_parameter))) {
					return -EFAULT;
				}
			} else {
				pr_err("wait_cmdbuf_ready failed ret=%d, die_index=0x%x, cmdbuf_id=%d, core_id=%d!!!\n",
					ret, die->die_index, cmdbuf_id, core_id);
				return -EFAULT;
			}

			PDEBUG(VIDEO_LOGLEVEL_INFO, "wait_cmdbuf_ready success, die_index=0x%x, cmdbuf_id=%d, core_id=%d!!!\n", die->die_index, cmdbuf_id, core_id);

			break;
		}

		case VASTAI_ENC_IOCH_RELEASE_CMDBUF:
		{
			int ret = -1;
			u16 cmdbuf_id;
			//__get_user(cmdbuf_id, (u16*)arg);
            if( video_copy_from_user(&cmdbuf_id, (u16*)arg, sizeof(cmdbuf_id)) ) {
                return -EFAULT;
            }


			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc vcmd release cmdbuf: die_index=0x%x, cmdbuf_id=%d\n", die->die_index, cmdbuf_id);
			ret = release_cmdbuf(filp, cmdbuf_id, die);
			if (ret != 0) {
				pr_err("release_cmdbuf failed ret=%d, die_index=0x%x, cmdbuf_id=%d!!!\n", 
					die->die_index, ret, cmdbuf_id);
				return -EFAULT;
			}

			PDEBUG(VIDEO_LOGLEVEL_INFO, "release_cmdbuf success, die_index=0x%x, cmdbuf_id=%d!!!\n", die->die_index, cmdbuf_id);

			break;
		}
#endif
		default:
		{
			pr_err("invalid ioctl cmd=0x%x!!!\n", cmd);
			return -1;
		}
	}

	return 0;
}

int vastai_enc_open(struct inode *inode, struct file *filp)
{
	int result = 0;
	struct enc_chn_info *info = NULL;
	set_enc_chn_info(filp, info);

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "va_enc_open %p\n", (void *)filp);
	return result;
}
int vastai_enc_release(struct inode *inode, struct file *filp)
{
	struct vastai_video_info *video_info = NULL;
	struct die_enc_info *die = NULL;
#ifdef VENC_FENCE_TEST
	int ret = -1;
#endif

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "va_enc_release %p\n", (void *)filp);

	video_info = (struct vastai_video_info *)filp->private_data;
	if (video_info == NULL || video_info->enc_device_handle == NULL) {
		pr_err("video_info or enc_device_handle is null, return\n");
		return -1;
	}

	die = (struct die_enc_info*)&video_info->enc_device_handle->die;

	if (die->encoder_mode == SINGLE_CORE_MODE) {

		struct enc_chn_info *info;
		info = get_enc_chn_info(filp);
		if (info == NULL)
			return 0;
#ifdef VENC_FENCE_TEST
		ret = venc_deinit_fence_ctx(info, die);
		if (ret) {
			PDEBUG(VIDEO_LOGLEVEL_ERROR, "%d devfn=%d dev_id=%d failed ret=%d\n", __LINE__, die->devfn, die->priv_dev->dev_id, ret);
			return -EFAULT;
		}
#endif
		info->core_id = 0;
		info->die_index = 0;
		info->instance_addr = 0;
		info->msg_addr = 0;

		set_enc_chn_info(filp, NULL);

#ifdef VENC_MULTICORE_SUPPORT
	} else if (die->encoder_mode == MULTI_CORE_MODE) {

		u32 cmdbuf_id = 0, release_cmdbuf_num = 0;
		struct cmdbuf_obj* cmdbuf_obj_temp = NULL;

		for (cmdbuf_id = 0; cmdbuf_id < TOTAL_DISCRETE_CMDBUF_NUM; cmdbuf_id++) {
			cmdbuf_obj_temp = &(die->cmdbuf_obj_sets[cmdbuf_id]);
			if (cmdbuf_obj_temp->filp == filp) {
				clean_cmdbuf_obj(cmdbuf_obj_temp, cmdbuf_id, die);
				release_cmdbuf_num++;
			}
		}
		PDEBUG(VIDEO_LOGLEVEL_INFO, "va_enc_release %p, released %d cmdbuf\n", (void *)filp, release_cmdbuf_num);
#endif
	} else {
		pr_err("error encode mode, return\n");
		return -1;
	}

	return 0;
}

unsigned int vastai_enc_poll(struct file *filp, poll_table *wait)
{
	unsigned int mask = 0;
	struct enc_chn_info *info;

	info = get_enc_chn_info(filp);
	if (info == NULL)
		return mask;

	poll_wait(filp, &info->wait_queue, wait);
	if (info->event) {
		mask |= POLLIN;
	}
	//printk("mask:%x\n", mask);
	return mask;
}

static int vastai_singlecore_callback(void *die_info, void *mcu_msg)
{
	u32 index = 0;
	struct enc_chn_info* chn_info = NULL;
	vemcu_to_host_msg_t* msg = (vemcu_to_host_msg_t*)mcu_msg;
	struct die_enc_info* die = (struct die_enc_info*)die_info;

	PDEBUG(VIDEO_LOGLEVEL_INFO, "in_fifo: devfn=%d---> status=%d--insaddr= %x\n", die->devfn, msg->enc_status, msg->instance_addr);

	for (index = 0; index < VASTAI_MAX_CORE_CHANNEL; index++)
	{

		chn_info = &die->venc_core_info[msg->core_id].enc_chn_set[index];

		if (chn_info->rcv && chn_info->instance_addr == msg->instance_addr)
		{
			chn_info->event =1;
#ifdef VENC_RECV_QUEUE_SUPPORT
			vastai_video_enqueue_data(chn_info->rcv, mcu_msg, sizeof(vemcu_to_host_msg_t));
#else
			memcpy(&chn_info->mcu_msg, mcu_msg, sizeof(vemcu_to_host_msg_t));
#endif
			wake_up_interruptible(&chn_info->wait_queue);
			PDEBUG(VIDEO_LOGLEVEL_DEBUG, "wakeup devfn=%d---> status=%d--insaddr= %x, index=%d\n", die->devfn, msg->enc_status, msg->instance_addr, index);
			break;
		}
	}

	if (index == VASTAI_MAX_CORE_CHANNEL) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "err msg devfn=%d, instance_addr:%u\n", die->devfn, msg->instance_addr);
		return -EINVAL;
	}

	return 0;
}

#ifdef VENC_MULTICORE_SUPPORT

static int vastai_multicore_callback(void *pci_dev, u32 die_id, int irq, void *die_info, void *mcu_msg)
{
	multivemcu_to_host_msg_t* vemcu_msg = (multivemcu_to_host_msg_t *)mcu_msg;
	u32 cmdbuf_id = vemcu_msg->cmdbuf_id;
	u32 status = vemcu_msg->encstate;
	struct die_enc_info* die = (struct die_enc_info*)die_info;
	int ret = -1;
	struct cmdbuf_obj* cmdbuf_obj = NULL;
	u32 cmdbuf_processed_num = 0;

	PDEBUG(VIDEO_LOGLEVEL_TRACE, "venc Enter, die_index=0x%x\n", die->die_index);

	if (cmdbuf_id) {
		if (cmdbuf_id >= TOTAL_DISCRETE_CMDBUF_NUM) {
			pr_err("venc error, die_inidex=%d, cmdbuf_id %d greater than the ceiling !!\n", die->die_index, cmdbuf_id);
			return -EINVAL;
		}

		cmdbuf_obj = &die->cmdbuf_obj_sets[cmdbuf_id];

		if (cmdbuf_obj == NULL) {
			pr_err("venc error, die_inidex=%d, cmdbuf_id %d cmdbuf_obj is NULL !!\n", die->die_index, cmdbuf_id);
			return -EINVAL;
		}

		cmdbuf_obj->core_id = vemcu_msg->core_id;
		cmdbuf_obj->executing_status = vemcu_msg->encstate;

		if (die->cmdbuf_used[cmdbuf_id] == 1) {
			cmdbuf_obj->cmdbuf_run_done = 1;
			cmdbuf_processed_num++;
		}

		PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc one cmdbuf_id done: die_index=0x%x core_id=%d cmdbuf_id=%d status=0x%x \n", die->die_index, vemcu_msg->core_id, cmdbuf_id,status);

	}


	if (cmdbuf_processed_num)
		wake_up_interruptible_all(&die->wait_queue);

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "venc work_queue-done:  die_index=0x%x, cmdbuf_processed_num = %d\n", die->die_index, cmdbuf_processed_num);

	return ret;
}
#endif
static void vcmd_workqueue_schedue(struct die_enc_info *die, struct work_struct *work, int core_id)
{
	int ret = 0;
	vemcu_to_host_msg_t mcu_msg;

	PDEBUG(VIDEO_LOGLEVEL_DEBUG, "devfn=%d, encoder_mode=%d,core_id=%d, die_info=%p\n", die->devfn, die->encoder_mode, core_id, die);

	mutex_lock(&die->vastai_fifo_mutex[core_id]);
	while (!ret) {
		ret = vastai_video_fifo_pop_elem(die->priv_dev, die->die_index, die->vemcu2host_ringbuffer[core_id]->ring_buf_base, &mcu_msg, NULL);
		if (ret != 0) {
			if (ret == -ENOMEM) {
				PDEBUG(VIDEO_LOGLEVEL_INFO, "devfn=%d, encoder_mode=%d fifo is empty ret=%d\n", die->devfn, die->encoder_mode, ret);
				break;
			}

			PDEBUG(VIDEO_LOGLEVEL_ERROR, "devfn=%d, encoder_mode=%d pop elem failed ret=%d\n", die->devfn, die->encoder_mode, ret);
			mutex_unlock(&die->vastai_fifo_mutex[core_id]);
			return;
		}

		if (die->encoder_mode == SINGLE_CORE_MODE) {
			mcu_msg.core_id = core_id;
			vastai_singlecore_callback(die, (void*)&mcu_msg);
	#ifdef VENC_MULTICORE_SUPPORT
		} else if (die->encoder_mode == MULTI_CORE_MODE) {

			vastai_multicore_callback(die->priv_dev, die->die_index, 0, die, (void*)&mcu_msg);
	#endif
		} else {
			pr_err("unsupported encoder mode!!!");
		}
	}
	mutex_unlock(&die->vastai_fifo_mutex[core_id]);
}

static int vastai_enc_init_ringbuffer(struct die_enc_info* die) {

	int index = 0;
	int ret = 0;
	int host2vemcu_elem_size = 0, host2vemcu_elem_count = 0;

	switch (die->fn_mode) {

		case ONLY_1_PF:
		{
			die->vemcu2host_ringbuffer[index] 	= find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_VEMCU0_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU0_RD_NAME);

			die->vemcu2host_ringbuffer[index]   = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_VEMCU1_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU1_RD_NAME);

			die->vemcu2host_ringbuffer[index]   = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_VEMCU2_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU2_RD_NAME);

			die->vemcu2host_ringbuffer[index]   = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_VEMCU3_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_1_PF_HOST_WR_M2O_CMD_VEMCU3_RD_NAME);

			break;
		}

		case ONLY_2_PF:
		{
			die->vemcu2host_ringbuffer[index] 	= find_hw_cfg_acord_name(die->priv_dev, ONLY_2_PF_1ST_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_2_PF_HOST_WR_M2O_CMD_1ST_VEMCU_RD_NAME);

			die->vemcu2host_ringbuffer[index]   = find_hw_cfg_acord_name(die->priv_dev, ONLY_2_PF_2ND_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, ONLY_2_PF_HOST_WR_M2O_CMD_2ND_VEMCU_RD_NAME);

			break;
		}

		case ONLY_4_PF:
		case PF_4_WITH_SRIOV:
		default:
		{
			die->vemcu2host_ringbuffer[index] 	= find_hw_cfg_acord_name(die->priv_dev, COMMON_VEMCU_WR_MSIX_INFO_HOST_RD_NAME);
			die->host2vemcu_ringbuffer[index++] = find_hw_cfg_acord_name(die->priv_dev, COMMON_HOST_WR_M2O_CMD_VEMCU_RD_NAME);

			break;
		}

	}

	for (index = 0; index < die->core_num_per_pf; index++) {

		if (!die->vemcu2host_ringbuffer[index]->ring_buf_base|| !die->host2vemcu_ringbuffer[index]->ring_buf_base) {
			printk("error: devfn=%d index=%d, invalid ringbuffer addr! vemcu2host_ringbuffer=0x%x, host2vemcu_ringbuffer=0x%x\n",
					 die->devfn, index, die->vemcu2host_ringbuffer[index]->ring_buf_base, die->host2vemcu_ringbuffer[index]->ring_buf_base);
			return -1;
		} else {
			printk("devfn=%d index=%d, get ringbuffer success, vemcu2host_ringbuffer=0x%x, len=%d, host2vemcu_ringbuffer=0x%x, len=%d\n",
					die->devfn, index,
					die->vemcu2host_ringbuffer[index]->ring_buf_base, die->vemcu2host_ringbuffer[index]->ring_buf_size,
					die->host2vemcu_ringbuffer[index]->ring_buf_base, die->host2vemcu_ringbuffer[index]->ring_buf_size);
		}

		host2vemcu_elem_size = sizeof(TVidEncCfgParam);
		host2vemcu_elem_count =	(die->host2vemcu_ringbuffer[index]->ring_buf_size - sizeof(struct vastai_video_fifo)) / host2vemcu_elem_size;

		ret = vastai_video_fifo_init(die->priv_dev, die->die_index, die->host2vemcu_ringbuffer[index]->ring_buf_base,
				host2vemcu_elem_count, host2vemcu_elem_size);
		if(ret) {
			pr_err("devfn=%d init host2vemcu ringbuffer failed! ret=%d\n", die->devfn, ret);
			return -1;
		} else {
			printk("devfn=%d init host2vemcu ringbuffer success, addr=0x%x, len=%d\n",  die->devfn,
					die->host2vemcu_ringbuffer[index]->ring_buf_base,
					die->host2vemcu_ringbuffer[index]->ring_buf_size);
		}

	}

	return 0;
}

static int vastai_enc_regsiter_interrupt(struct die_enc_info* die) {
	int ret;

	switch (die->fn_mode) {

		case ONLY_1_PF:
		{
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_1_PF_MSIX_VEMCU_0_VECTOR, 
				vastai_pci_enc_callback0, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_1_PF_MSIX_VEMCU_1_VECTOR, 
				vastai_pci_enc_callback1, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_1_PF_MSIX_VEMCU_2_VECTOR, 
				vastai_pci_enc_callback2, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_1_PF_MSIX_VEMCU_3_VECTOR, 
				vastai_pci_enc_callback3, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}

			break;
		}

		case ONLY_2_PF:
		{
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_2_PF_MSIX_VEMCU_0_VECTOR, 
				vastai_pci_enc_callback0, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}
			ret = va_register_pcie_interrupt(die->priv_dev, ONLY_2_PF_MSIX_VEMCU_1_VECTOR, 
				vastai_pci_enc_callback1, (void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}

			break;
		}

		case ONLY_4_PF:
		case PF_4_WITH_SRIOV:
		default:
		{
			ret = va_register_pcie_interrupt(die->priv_dev,
				COMMON_MSIX_VEMCU_VECTOR,
				vastai_pci_enc_callback0,
				(void *)die);
			if (ret) {
				printk("devfn=%d, error register vemcu interrupt, ret=%d!\n", die->devfn, ret);
				return -1;
			}
			break;
		}

	}

	return 0;
}

/*------------------------------------------------------------------------------
 Function name   : vcmd_init
 Description	 : Initialize vcmd bus addr

 Return type	 : int
 ------------------------------------------------------------------------------*/
#ifdef VENC_MULTICORE_SUPPORT
static int vcmd_init(struct die_enc_info *die_info)
{
	int i, k;
	struct vastaivcmd_dev* vastaivcmd_data = NULL;
	unsigned int hwid = 0x43421200;
	struct vastai_video_fifo fifo = {.rd = 0, .wr = 0, .elem_count=HOST_VEMCU_MAX_MSG_NUM, .elem_size=HOST_VEMCU_ELEM_SIZE};

	//set vcmd cmdbuf addr start
	die_info->vcmd_buf_mem_pool.busAddress = VCMD_ENC_DDR_BASE_ADDR_0;
	die_info->vcmd_buf_mem_pool.size = CMDBUF_POOL_TOTAL_SIZE;

	printk(KERN_INFO"VENC VCMD buf mem addr=0x%lx, size=0x%08x\n", (unsigned long)die_info->vcmd_buf_mem_pool.busAddress,
			die_info->vcmd_buf_mem_pool.size);

	die_info->vcmd_status_buf_mem_pool.busAddress = VCMD_ENC_DDR_BASE_ADDR_0 + CMDBUF_POOL_TOTAL_SIZE;
	die_info->vcmd_status_buf_mem_pool.size = CMDBUF_POOL_TOTAL_SIZE;

	printk(KERN_INFO "VENC VCMD status buf mem addr=0x%lx, size=0x%08x\n", (unsigned long)die_info->vcmd_status_buf_mem_pool.busAddress,
			die_info->vcmd_status_buf_mem_pool.size);

	die_info->vcmd_registers_mem_pool.busAddress = VCMD_ENC_DDR_BASE_ADDR_0  + CMDBUF_POOL_TOTAL_SIZE * 2;
	die_info->vcmd_registers_mem_pool.size = CMDBUF_VCMD_REGISTER_TOTAL_SIZE;

	printk(KERN_INFO "VENC VCMD registers mem addr=0x%lx, size=0x%08x\n", (unsigned long)die_info->vcmd_registers_mem_pool.busAddress ,
			die_info->vcmd_registers_mem_pool.size);
	//set vcmd cmdbuf addr end

	vastaivcmd_data = (struct vastaivcmd_dev *)vmalloc(sizeof(struct vastaivcmd_dev) * total_vcmd_core_num);
	if (vastaivcmd_data == NULL) {
		pr_err("alloc mem failed!\n");
		return -1;
	}

	for (k=0; k < MAX_VCMD_TYPE; k++) {
		die_info->vcmd_type_core_num[k] = 0;
		for (i=0; i<MAX_VCMD_NUMBER; i++) {
			die_info->vcmd_manager[k][i] = NULL;
		}
	}

	vastai_cmdid_init(&die_info->cmdbufId_queue, ENCODER);
	vastai_pci_mem_write(die_info->priv_dev, die_info->die_index, HOST_VEMCU0_MSG_ADDR, &fifo, sizeof(fifo));

	memset(vastaivcmd_data, 0, sizeof(struct vastaivcmd_dev) * total_vcmd_core_num);
	printk("total_vcmd_core_num=%d\n", total_vcmd_core_num);

	for (i = 0; i < total_vcmd_core_num; i++) {
		vastaivcmd_data[i].vcmd_core_cfg = vcmd_core_array[i];
		vastaivcmd_data[i].hwregs = NULL;
		vastaivcmd_data[i].core_id = i;
		vastaivcmd_data[i].working_state = WORKING_STATE_IDLE;
		vastaivcmd_data[i].sw_cmdbuf_rdy_num = 0;
		mutex_init(&vastaivcmd_data[i].mutex);

		vastaivcmd_data[i].duration_without_int = 0;

		die_info->vcmd_manager[vcmd_core_array[i].sub_module_type][die_info->vcmd_type_core_num[vcmd_core_array[i].sub_module_type]] = &vastaivcmd_data[i];
		die_info->vcmd_type_core_num[vcmd_core_array[i].sub_module_type]++;
		vastaivcmd_data[i].vcmd_type_core_num[vcmd_core_array[i].sub_module_type]++;

		vastaivcmd_data[i].hw_version_id = hwid;
		spin_lock_init(&vastaivcmd_data[i].vcmd_cmdbuf_alloc_lock);

		die_info->cmdbuf_used[i+1] = 1;
	}

	die_info->vastaivcmd_data = vastaivcmd_data;


	init_waitqueue_head(&die_info->vcmd_cmdbuf_memory_wait);
	init_waitqueue_head(&die_info->wait_queue);

	die_info->cmdbuf_used[0] = 1;
	die_info->cmdbuf_used_pos = total_vcmd_core_num + 1;
	die_info->cmdbuf_used_residual = TOTAL_DISCRETE_CMDBUF_NUM - total_vcmd_core_num - 1;
	(die_info->vcmd_manager[VCMD_TYPE_ENCODER][0])->status_cmdbuf_id = 1;
	(die_info->vcmd_manager[VCMD_TYPE_ENCODER][0])->status_cmdbuf_id = 1;

	return 0;
}

static int do_work_mode_change(struct die_enc_info *die, encoder_workmode_t work_mode, u32 init) {
	int core_id;
	int magic_number = (work_mode == SINGLE_CORE_MODE ? ENCODER_SINGLE_MODE_MAGIC : ENCODER_MULTI_MODE_MAGIC);

	if (die->encoder_mode == work_mode && !init) {
		printk("current work mode(%s) is same with dest work mode, no need switch\n", (work_mode == SINGLE_CORE_MODE ? "single-core" : "multi-core"));
		return 0;
	}

	vastai_pci_mem_write(die->priv_dev, die->die_index, ENCODER_WORKMODE_ADDR, &magic_number, sizeof(magic_number));

	if (!init) {
		//because multi-core mode we use vemcu0 to control venc 0~3, so reset vemcu0 finally
		/*for (core_id = CORE_POINT_VEMCU3; core_id >= CORE_POINT_VEMCU0; core_id--) {
			vastai_send_pcie_cmd(die->priv_dev,die->die_index,
						VASTAI_PCIE_SUB_CORE_RESET, core_id);
			msleep(100);
		}*/

		if (work_mode == MULTI_CORE_MODE) {
			int i;
			struct vastai_video_fifo fifo = {.rd = 0, .wr = 0, .elem_count=HOST_VEMCU_MAX_MSG_NUM, .elem_size=HOST_VEMCU_ELEM_SIZE};
			// 1. reset cmdbuf id
			mutex_lock(&die->mutex);
			die->cmdbuf_used_pos = total_vcmd_core_num + 1;
			die->cmdbuf_used_residual = TOTAL_DISCRETE_CMDBUF_NUM - total_vcmd_core_num - 1;
			memset(die->cmdbuf_used, 0, sizeof(die->cmdbuf_used));
			for (i = 0; i < total_vcmd_core_num + 1; i++) {
				die->cmdbuf_used[i] = 1;
			}
			mutex_unlock(&die->mutex);

			// 2. reinit host->vemcu ring buffer
			//vastai_pci_mem_write(die->priv_dev, die->die_index, HOST_VEMCU0_MSG_ADDR, &fifo, sizeof(fifo));
			vastai_pci_mem_write(die->priv_dev, die->die_index, die->host2vemcu_ringbuffer[0]->ring_buf_base, &fifo, sizeof(fifo));	
		}

		printk("vasmi set encoder work mode to %s success!\n", (work_mode == SINGLE_CORE_MODE ? "single-core" : "multi-core"));
	} else {
		printk("init set encoder work mode to %s success!\n", (work_mode == SINGLE_CORE_MODE ? "single-core" : "multi-core"));
	}

	die->encoder_mode = work_mode;

	return 0;
}

int vastai_enc_get_work_mode(int die_index, encoder_workmode_t* work_mode, int* job_count) {
	enc_device_handle_t *device_handle_node = NULL;
	int ret = -1;

	ret = vastai_enc_get_device_from_dieindex(die_index, &device_handle_node);
	if (ret) {
		pr_err("failed to get die device, return\n");
		return -1;
	}

	*work_mode = device_handle_node->die.encoder_mode;
	*job_count = device_handle_node->die.enc_job_count;

	return 0;
}

int vastai_enc_set_work_mode(int die_index, encoder_workmode_t work_mode, u32 init) {
	enc_device_handle_t *device_handle_node = NULL;
	u32 job_count = 0;
	int ret = -1;

	ret = vastai_enc_get_device_from_dieindex(die_index, &device_handle_node);
	if (ret) {
		pr_err("failed to get die device, return\n");
		return -1;
	}

	mutex_lock(&device_handle_node->die.job_mutex);
	job_count = device_handle_node->die.enc_job_count;
	printk("current job count = %d\n", job_count);
	mutex_unlock(&device_handle_node->die.job_mutex);

	if (job_count) {
		printk("still run %d task on die: 0x%x, switch encoder work mode failed, return\n", job_count, die_index);
		return -1;
	}

	do_work_mode_change((struct die_enc_info*)&(device_handle_node->die), work_mode, init);

	return 0;
}
#else
int vastai_enc_get_work_mode(int die_index, encoder_workmode_t* work_mode, int* job_count) {
	return 0;
}
int vastai_enc_set_work_mode(int die_index, encoder_workmode_t work_mode, u32 init) {
	return 0;
}
#endif

int vastai_enc_release_surface(int dev_id, int dmabuf_fd) {
	venc_exchange_data_t data;
	int ret = -1;

	data.dmabuf_fd = dmabuf_fd;
	ret = venc_user_release_surface(&data);
	if (ret) {
		PDEBUG(VIDEO_LOGLEVEL_ERROR, "user release surface failed! dev_id=%d, dmabuf_fd=%d, ret=%d\n", dev_id, dmabuf_fd, ret);
		return -1;
	}

	return 0;
}
EXPORT_SYMBOL(vastai_enc_release_surface);


void vastai_enc_set_log_level(int level) {
	venc_log_level = level;
	printk("ser enc log level %d success\n", venc_log_level);
}

int vastai_enc_get_device_from_dieindex(u32 die_index, enc_device_handle_t** enc_device_handle) {

	struct list_head *p = NULL, *n = NULL;
	enc_device_handle_t *device_handle_node = NULL;

	if(enc_device_handle == NULL || list_empty(&enc_device_handle_head)) {
		pr_err("invalid paramater! or enc_device_handle_head is empty!!!\n");
		return -1;
	}

	list_for_each_safe(p, n, &enc_device_handle_head) {
		device_handle_node = list_entry(p, enc_device_handle_t, node);
		if (die_index == device_handle_node->die.die_index) {
			*enc_device_handle = device_handle_node;
			return 0;
		}
	}

	pr_err("can not find encode device!!!\n");

	return -1;
}

int vastai_enc_init(struct die_enc_info *die) {
	int ret = 0;
	int index = 0, channel_index = 0;

#ifdef VENC_MULTICORE_SUPPORT
	total_vcmd_core_num = sizeof(vcmd_core_array)/sizeof(struct vcmd_config);
	die->core_num = total_vcmd_core_num;

	ret = vcmd_init(die);
	if(ret)
		goto err;
#endif

	switch (die->fn_mode) {

		case ONLY_1_PF:
			die->core_num_per_pf  = VASTAI_MAX_ENC_CORES;
			break;

		case ONLY_2_PF:
			die->core_num_per_pf = VASTAI_MAX_ENC_CORES / 2;
			break;

		case ONLY_4_PF:
		case PF_4_WITH_SRIOV:
		default:
			die->core_num_per_pf = VASTAI_MAX_ENC_CORES / 4;
			break;
	}

	ret = vastai_enc_init_ringbuffer(die);
	if(ret)
		goto err;

	ret = vastai_enc_regsiter_interrupt(die);
	if(ret)
		goto err;

	for(index = 0; index < die->core_num_per_pf; index++)
	{
		for(channel_index = 0; channel_index < VASTAI_MAX_CORE_CHANNEL; channel_index++)
		{
			die->venc_core_info[index].enc_chn_set[channel_index].core_id = 0;
			die->venc_core_info[index].enc_chn_set[channel_index].die_index = 0;
			die->venc_core_info[index].enc_chn_set[channel_index].instance_addr = 0;
			die->venc_core_info[index].enc_chn_set[channel_index].msg_addr = 0;
#ifdef VENC_RECV_QUEUE_SUPPORT
			die->venc_core_info[index].enc_chn_set[channel_index].rcv = (struct vastai_video_fifo*)kzalloc(LOCAL_MAX_FIFO_LEN, GFP_KERNEL);;
			die->venc_core_info[index].enc_chn_set[channel_index].rcv->elem_count = (LOCAL_MAX_FIFO_LEN - sizeof(struct vastai_video_fifo)) / sizeof(vemcu_to_host_msg_t);;
			die->venc_core_info[index].enc_chn_set[channel_index].rcv->elem_size = sizeof(vemcu_to_host_msg_t);
#endif
		}

		die->workqueue[index] = create_workqueue("video_encoder_workqueue");
		mutex_init(&die->vastai_fifo_mutex[index]);
	}

	INIT_WORK(&(die->work0), vcmd_workqueue_schedue0);
	INIT_WORK(&(die->work1), vcmd_workqueue_schedue1);
	INIT_WORK(&(die->work2), vcmd_workqueue_schedue2);
	INIT_WORK(&(die->work3), vcmd_workqueue_schedue3);

	mutex_init(&die->chan_alloc_mutex);
	mutex_init(&die->job_mutex);
	mutex_init(&die->mutex);


	//do_work_mode_change(die, SINGLE_CORE_MODE, true);
	die->encoder_mode = SINGLE_CORE_MODE;
	printk("va_enc_init exit %d\n",die->die_index);

	return 0;

  err:
	if (die->vastaivcmd_data != NULL)
		vfree(die->vastaivcmd_data);

	pr_err("devfn=%d va_enc_init: module not inserted\n", die->devfn);
	return ret;
}
void vastai_enc_deinit(void* pcie_dev)
{
	struct list_head *p = NULL, *n = NULL;
	enc_device_handle_t *device_handle_node = NULL;
	u32 die_index;
	int index, channel_index;

	list_for_each_safe(p, n, &enc_device_handle_head) {
		device_handle_node = list_entry(p, enc_device_handle_t, node);
		if (pcie_dev == device_handle_node->die.priv_dev) {
			list_del(&device_handle_node->node);
			die_index = device_handle_node->die.die_index;

#ifdef VENC_RECV_QUEUE_SUPPORT
			for(index = 0; index < VASTAI_MAX_ENC_CORES; index++)
			{
				for(channel_index = 0; channel_index < VASTAI_MAX_CORE_CHANNEL; channel_index++)
				{
					if (device_handle_node->die.venc_core_info[index].enc_chn_set[channel_index].rcv) {
						kfree(device_handle_node->die.venc_core_info[index].enc_chn_set[channel_index].rcv);
					}
				}
			}
#endif

			mutex_destroy(&device_handle_node->die.chan_alloc_mutex);
			mutex_destroy(&device_handle_node->die.job_mutex);
			mutex_destroy(&device_handle_node->die.mutex);

			for(index = 0; index < device_handle_node->die.core_num_per_pf; index++)
			{
				mutex_destroy(&device_handle_node->die.vastai_fifo_mutex[index]);
				flush_workqueue(device_handle_node->die.workqueue[index]);
				destroy_workqueue(device_handle_node->die.workqueue[index]);
			}

#ifdef VENC_MULTICORE_SUPPORT
	        int value = 0;
			if (device_handle_node->die.vastaivcmd_data != NULL)
				vfree(device_handle_node->die.vastaivcmd_data);

			//clear encoder work mode flag, because vemcu will check it when re-insmode driver.
			vastai_pci_mem_write(device_handle_node->die.priv_dev, device_handle_node->die.die_index, ENCODER_WORKMODE_ADDR, &value, sizeof(value));

#endif

			kfree(device_handle_node);
			device_handle_node = NULL;
			printk(KERN_INFO "va_enc_deinit: die:%x removed\n", die_index);
		}
	}

	return;
}
