/*------------------------------------------------------------------------------
--                                                                                                                               --
--       This software is confidential and proprietary and may be used                                   --
--        only as expressly authorized by a licensing agreement from                                     --
--                                                                                                                               --
--                            Verisilicon.                                                                                    --
--                                                                                                                               --
--                   (C) COPYRIGHT 2019 VERISILICON                                                            --
--                            ALL RIGHTS RESERVED                                                                    --
--                                                                                                                               --
--                 The entire notice above must be reproduced                                                 --
--                  on all copies and should not be removed.                                                    --
--                                                                                                                               --
--------------------------------------------------------------------------------
--
--  Abstract : VC8000 Vcmd device driver (kernel module)
--
------------------------------------------------------------------------------*/

#ifndef _VC8000_VCMD_DRIVER_H_
#define _VC8000_VCMD_DRIVER_H_
#include <linux/poll.h>
#include <linux/dma-buf.h>
#include <linux/dma-fence.h>
//#include <linux/dma-resv.h>
#include "vastai_video_fence.h"
#include "vastai_vcmd.h"
#include "hw_config.h"

#ifdef __FREERTOS__
/* needed for the _IOW etc stuff used later */
#include "base_type.h"
#include "osal.h"
#include "dev_common_freertos.h"
#elif defined(__linux__)
#include <linux/ioctl.h>    /* needed for the _IOW etc stuff used later */
#else //For other os
//TODO...
#endif
#ifdef HANTROMMU_SUPPORT
#include "hantrommu.h"
#endif

#ifdef HANTROAXIFE_SUPPORT
#include "vc8000_axife.h"
#endif

#ifdef __FREERTOS__
//ptr_t has been defined in base_type.h //Now the FreeRTOS mem need to support 64bit env
#elif defined(__linux__)
#undef  ptr_t
#define ptr_t  PTR_T_KERNEL
typedef size_t ptr_t;
#endif


#ifdef VIDEO_COMPAT_ENC
#define _IOW_ENC(magic, nr, type)                           _IO(magic, nr)
#define _IOR_ENC(magic, nr, type)                           _IO(magic, nr)
#define _IOWR_ENC(magic, nr, type)                          _IO(magic, nr)
#else
#define _IOW_ENC                                            _IOW
#define _IOR_ENC                                            _IOR
#define _IOWR_ENC                                           _IOWR
#endif


#define VASTAI_ENC_IOC_MAGIC  'e'

/*
 * S means "Set" through a ptr,
 * T means "Tell" directly with the argument value
 * G means "Get": reply by setting through a pointer
 * Q means "Query": response is on the return value
 * X means "eXchange": G and S atomically
 * H means "sHift": T and Q atomically
 */

#define VASTAI_ENC_IOCH_SET_ENCODER                             _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 24, u16)
#define VASTAI_ENC_IOCH_GET_CMDBUF_PARAMETER                    _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 25,struct cmdbuf_mem_parameter *)
#define VASTAI_ENC_IOCH_GET_CMDBUF_POOL_SIZE                    _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 26,unsigned long)
#define VASTAI_ENC_IOCH_SET_CMDBUF_POOL_BASE                    _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 27,unsigned long)
#define VASTAI_ENC_IOCH_GET_VCMD_PARAMETER                      _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 28, struct config_parameter *)
#define VASTAI_ENC_IOCH_RESERVE_CMDBUF                          _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 29, struct exchange_parameter *)
#define VASTAI_ENC_IOCH_LINK_RUN_CMDBUF                         _IOR_ENC(VASTAI_ENC_IOC_MAGIC, 30, struct exchange_parameter *)
#define VASTAI_ENC_IOCH_WAIT_CMDBUF                             _IOR_ENC(VASTAI_ENC_IOC_MAGIC, 31, struct enc_waitcmd_parameter*)
#define VASTAI_ENC_IOCH_RELEASE_CMDBUF                          _IOR_ENC(VASTAI_ENC_IOC_MAGIC, 32, struct enc_waitcmd_parameter*)
#define VASTAI_ENC_IOCH_POLLING_CMDBUF                          _IOR_ENC(VASTAI_ENC_IOC_MAGIC, 33,u16 *)
#define VASTAI_ENC_IOCH_GET_ENCMODE                             _IOR_ENC(VASTAI_ENC_IOC_MAGIC, 34,u16 *)
#define VASTAI_ENC_IOCH_INT_VEMCU                               _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 41,unsigned long *)
#define VASTAI_ENC_IOCH_ALLOCCHANNEL_LOCK          	            _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 42,unsigned int*)
#define VASTAI_ENC_IOCH_ALLOCCHANNEL_UNLOCK           	        _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 43,unsigned int*)
#define VASTAI_ENC_IOCH_ATTACH_VENCCHNL                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 44, unsigned long*)
#define VASTAI_ENC_IOCH_DETACH_VENCCHNL                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 45, unsigned long*)
#define VASTAI_ENC_IOCH_SET_VENCCHNLMSG                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 46, TVidEncCfgParam*)
#define VASTAI_ENC_IOCH_GET_VENCCHNLMSG                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 47, unsigned long*)
#define VASTAI_ENC_IOCH_ACQUIRE_SURFACE                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 48, venc_exchange_data_t*)
#define VASTAI_ENC_IOCH_RELEASE_SURFACE                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 49, venc_exchange_data_t*)
#define VASTAI_ENC_IOCH_GET_VCMD_ENABLE                         _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 50, unsigned long)
#define VASTAI_ENC_IOCH_GET_DEVADDR_FROM_FD                     _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 51, venc_exchange_data_t*)
#define VASTAI_ENC_IOCH_WAIT_SURFACE_READY                      _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 52, venc_exchange_data_t*)
#define VASTAI_ENC_IOCH_SAVE_SURFACE                            _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 53, venc_exchange_data_t*)
#define VASTAI_ENC_IOCH_REMOVE_SURFACE                          _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 54, venc_exchange_data_t*)

#define VASTAI_ENC_IOCH_GET_DIE_ID                              _IOWR_ENC(VASTAI_ENC_IOC_MAGIC, 60, int *)

#define VASTAI_ENC_IOC_MAXNR 60
#define VASTAI_MAX_ENC_CORES  (4)
#define VASTAI_MAX_CORE_CHANNEL  (64)

#define VENC_MAGIC_NUMBER		(0x6aa6)
//#define CORE_MAX  (CORE_MMU)

#define HOST_VEMCU0_MSG_ADDR    (0x81b400)//(0x8c56000)
#define HOST_VEMCU1_MSG_ADDR    (0x8c56400)
#define HOST_VEMCU2_MSG_ADDR    (0x8c56800)
#define HOST_VEMCU3_MSG_ADDR    (0x8c56c00)

#define ENCODER_WORKMODE_ADDR       (0x1046000000)//(0x814000000)
#define ENCODER_SINGLE_MODE_MAGIC   (0xaaaaaaaa)
#define ENCODER_MULTI_MODE_MAGIC    (0xbbbbbbbb)

#define HOST_VEMCU_MAX_FIFO_LEN  (0x1000) //4k for vemcu 0~3
#define HOST_VEMCU_ELEM_SIZE     (sizeof(host_to_vemcu_msg_t))
#define HOST_VEMCU_MAX_MSG_NUM	((HOST_VEMCU_MAX_FIFO_LEN - sizeof(struct vastai_video_fifo)) / HOST_VEMCU_ELEM_SIZE)

#define LOCAL_MAX_FIFO_LEN  (0x400) //1k local ringbuffer

#define ASIC_STATUS_ALL       (ASIC_STATUS_SEGMENT_READY |\
                               ASIC_STATUS_FUSE_ERROR |\
                               ASIC_STATUS_SLICE_READY |\
                               ASIC_STATUS_LINE_BUFFER_DONE |\
                               ASIC_STATUS_HW_TIMEOUT |\
                               ASIC_STATUS_BUFF_FULL |\
                               ASIC_STATUS_HW_RESET |\
                               ASIC_STATUS_ERROR |\
                               ASIC_STATUS_FRAME_READY)

enum
{
  CORE_VC8000E = 0,
  CORE_VC8000EJ = 1,
  CORE_CUTREE = 2,
  CORE_DEC400 = 3,
  CORE_MMU = 4,
  CORE_L2CACHE = 5,
  CORE_AXIFE = 6,
  CORE_APBFT = 7,
  CORE_MMU_1 = 8,
  CORE_AXIFE_1 = 9,
  CORE_MAX
};

typedef enum {
    SINGLE_CORE_MODE,    //one stream encode on one encoder core
    MULTI_CORE_MODE,     //one stream encode on multi encoder core
    WORK_MODE_MAX,
}encoder_workmode_t;

typedef struct CoreWaitOut
{
  u32 job_id[4];
  u32 irq_status[4];
  u32 irq_num;
} CORE_WAIT_OUT;

typedef struct
{
  u32 subsys_idx;
  u32 core_type;
  unsigned long offset;
  u32 reg_size;
  int irq;
}CORE_CONFIG;

typedef struct
{
  unsigned long base_addr;
  u32 iosize;
  u32 resouce_shared; //indicate the core share resources with other cores or not.If 1, means cores can not work at the same time.
}SUBSYS_CONFIG;

typedef struct
{
  u32 type_info; //indicate which IP is contained in this subsystem and each uses one bit of this variable
  unsigned long offset[CORE_MAX];
  unsigned long regSize[CORE_MAX];
  int irq[CORE_MAX];
}SUBSYS_CORE_INFO;

typedef struct
{
  SUBSYS_CONFIG cfg;
  SUBSYS_CORE_INFO core_info;
}SUBSYS_DATA;

// add by taoz dma  dma
typedef struct tagDmaDataTrans
{
  unsigned int    dwStructVer;
  unsigned int    dwSize;
  unsigned int    dwDataSize;
  unsigned int    dwOutputBufLen;
  unsigned int    dwDeviceId;
  unsigned int    dwReserve;
  void* pInSrcAddr;
  void* pOutAddr;
}TDmaDataTrans;


#ifdef VIDEO_COMPAT_ENC
#pragma pack(1)
#endif

typedef struct tagIntVEMCU
{
  unsigned int    dwStructVer;
  unsigned int    dwSize;
  unsigned int    dwDeviceId;
  unsigned int    dwDieId;
  unsigned int    dwCoreId;
  unsigned int    dwCsrAddr;
  unsigned int    dwVal;

} TIntVEMCU;

typedef struct vemcu_to_host_msg
{
  unsigned int instance_addr;
  unsigned int enc_status;
  unsigned int core_id;
  unsigned int reserved;
} vemcu_to_host_msg_t;

typedef struct multivemcu_to_host_msg
{
  unsigned int cmdbuf_id;
  unsigned int encstate;
  unsigned int core_id;
  unsigned int reserved;
} multivemcu_to_host_msg_t;

typedef struct multivemcu_to_host_msg  enc_waitcmd_parameter;

typedef struct host_to_vemcu_msg
{
  unsigned int magic;
  unsigned int cmdbuf_id;
  unsigned int cmdbuf_len;
  unsigned int module_type;
  u64 executing_time;
} host_to_vemcu_msg_t;

typedef struct host2vemcu_msg_sg100
{
  unsigned int instance_addr;
  unsigned int enc_status;
  unsigned int core_id;
  unsigned int reserved;
} host2vemcu_msg_sg100_t;

typedef struct tagVidEncDest
{
  u32 dwInstAddr;
	u32 dwReserved;
}TVidEncDest;

typedef struct tagVidEncInit
{
	u32 dwReserved;
}TVidEncInit;
typedef struct tagVidEncPutData
{
	u32 dwInstAddr;
	u32 dwPsAddr;
	u32 dwReserved;
}TVidEncPutData;

typedef struct tagVidEncCfgParam
{
  u32 dwStructVer;
  u32 dwSize;
  u32 dwCmdId;
  u32 dwEncType;
  union
  {
    TVidEncDest tParamDest;         // dwCmdId = VEMCU_CMDID_DEST_CODEC
    TVidEncInit tParamInit;         // dwCmdId = VEMCU_CMDID_INIT_CODEC
    TVidEncPutData tParamPutData;   // dwCmdId = VEMCU_CMDID_PUT_DATA
    /* data */
  }uParam;
} TVidEncCfgParam;
typedef struct venc_msg_chninfo
{
  unsigned int instance_addr;
  unsigned int msg_addr;    // to init vastai_fifo
  unsigned int die_index;
  unsigned int core_id;
} venc_msg_chninfo_t;


#ifdef VIDEO_COMPAT_ENC
#pragma pack(0)
#endif

typedef struct venc_exchange_data
{
    int dmabuf_fd;
    uint64_t dev_addr;
    TVidEncCfgParam tvidencparam;
}__attribute__((packed)) venc_exchange_data_t;

typedef struct venc_surface_node
{
    struct dma_buf* dmabuf;
    int dmabuf_fd;
    int flag_need_release;
    int flag_encode_complete;
    spinlock_t lock;
    struct list_head node;
} venc_surface_node_t;

/**
 * venc_fence_context - venc fence context used to create and manage video encoder fences
 * @lock: protects the context and fences created on the context
 * @name: fence context name (used for debugging)
 * @fence_context: fence context with which to associate fences
 * @fence_seqno: sequence number to use for the next fence

 * @list_lock: protects the surface list
 * @surface_list: list of surface
 */
struct venc_fence_context
{
	spinlock_t lock;
	char name[32];
	u64 fence_context;
	atomic_t fence_seqno;

	spinlock_t list_lock;
	struct list_head surface_list;
};


struct enc_chn_info
{
  u32 core_id;
  u32 die_index;
  u32 instance_addr;
  u64 msg_addr;
  wait_queue_head_t wait_queue;
  struct list_head node;
  struct vastai_video_fifo* rcv;
  u32 event;
  vemcu_to_host_msg_t mcu_msg;
  struct venc_fence_context fctx;
};

struct core_info
{
  struct list_head enc_chn_head;
  struct enc_chn_info enc_chn_set[VASTAI_MAX_CORE_CHANNEL];
};

struct die_enc_info
{
	/* pointer to pcie dev */
  u32 die_index;
	struct vastai_pci_info *priv_dev;
  struct core_info venc_core_info[VASTAI_MAX_ENC_CORES];
  struct vastaivcmd_dev* vastaivcmd_data;
  int core_num;
  ktime_t timestamp;

  int vcmd_type_core_num[MAX_VCMD_TYPE];
	struct vastaivcmd_dev* vcmd_manager[MAX_VCMD_TYPE][MAX_VCMD_NUMBER];

	struct noncache_mem vcmd_buf_mem_pool;
	struct noncache_mem vcmd_status_buf_mem_pool;
	struct noncache_mem vcmd_registers_mem_pool;
	u16 cmdbuf_used[TOTAL_DISCRETE_CMDBUF_NUM];
	u16 cmdbuf_used_pos;
	u16 cmdbuf_used_residual;
	u16 vcmd_position[MAX_VCMD_TYPE];
  struct cmdbuf_obj cmdbuf_obj_sets[TOTAL_DISCRETE_CMDBUF_NUM];

  struct wait_queue_head vcmd_cmdbuf_memory_wait;
  struct mutex mutex;
  struct mutex vastai_fifo_mutex[VASTAI_MAX_ENC_CORES];
  wait_queue_head_t wait_queue;
  struct work_struct work0;
  struct work_struct work1;
  struct work_struct work2;
  struct work_struct work3;
  struct workqueue_struct *workqueue[VASTAI_MAX_ENC_CORES];
  VastCmdbufQueue cmdbufId_queue;
  multivemcu_to_host_msg_t mcu_msg[TOTAL_DISCRETE_CMDBUF_NUM];   //encode information send from vemcu when one frame is ready

  u32 enc_job_count;
  encoder_workmode_t encoder_mode;
  struct mutex chan_alloc_mutex;
  struct mutex job_mutex;

  u8  is_pf;
  u8  is_vf;
  u8  devfn;
  u8  fn_mode;
  u8  pkg_id;
  u8  pfn;
  u8  vfn;

  u8  core_num_per_pf;

  struct ring_buf_entry *host2vemcu_ringbuffer[VASTAI_MAX_ENC_CORES];
  struct ring_buf_entry *vemcu2host_ringbuffer[VASTAI_MAX_ENC_CORES];

  vastai_video_workmode_t enc_work_mode;
};

typedef struct enc_device_handle
{
  struct die_enc_info die;
  struct list_head node;
} enc_device_handle_t;

#ifndef MIN
#define MIN(a, b)  ((a) < (b) ? (a):(b))
#endif

#ifndef MAX
#define MAX(a, b)  ((a) > (b) ? (a):(b))
#endif

int vastai_enc_open(struct inode *inode, struct file *filp);
int vastai_enc_release(struct inode *inode, struct file *filp);
long vastai_enc_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
unsigned int vastai_enc_poll(struct file *filp, poll_table *wait);

int vastai_enc_get_device_from_dieindex(u32 die_index, enc_device_handle_t** enc_device_handle);
int vastai_enc_set_work_mode(int die_index, encoder_workmode_t work_mode, u32 init);
int vastai_enc_get_work_mode(int die_index, encoder_workmode_t* work_mode, int* job_count);
void vastai_enc_set_log_level(int level);
int vastai_enc_release_surface(int dev_id, int dmabuf_fd);

int  vastai_enc_init(struct die_enc_info* die);
void vastai_enc_deinit(void *pcie_dev);

#endif /* !_VC8000_VCMD_DRIVER_H_ */


